//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//



#include <zebra.h>
#include <sstream>

#include <vector>

#include "g2pcerad.hh"
#include "g2pcera_trace.hh"
#include "g2pcera_config.hh"
#include "g2pcera_algovis.hh"
#include "g2pcera_dm.hh"


PCERA::algoStubVisitor::algoStubVisitor()
{
	METIN();

	METOUT();
}

PCERA::algoStubVisitor::~algoStubVisitor()
{
	METIN();

	METOUT();
}

void
PCERA::algoStubVisitor::visitTopology(Topology *	ptr)
{
	METIN();

	assert(ptr);
	ptr->dump("[algoStubVisitor]:   ", false);

	METOUT();
}


void
PCERA::algoStubVisitor::visitNode(Node *	ptr)
{
	METIN();

	assert(ptr);
	ptr->dump("[algoStubVisitor]:   ");

	METOUT();
}


void
PCERA::algoStubVisitor::visitNetNode(NetNode *	ptr)
{
	METIN();

	assert(ptr);
	ptr->dump("[algoStubVisitor]:   ", false);

	METOUT();
}


void
PCERA::algoStubVisitor::visitTna(Tna *	ptr)
{
	METIN();

	assert(ptr);
	ptr->dump("[algoStubVisitor]:   ");

	METOUT();
}


void
PCERA::algoStubVisitor::visitTeLink(TeLink *	ptr)
{
	METIN();

	assert(ptr);
	ptr->dump("[algoStubVisitor]:   ");

	METOUT();
}


void
PCERA::algoStubVisitor::visitGridNode(GridNode *	ptr)
{
	METIN();

	assert(ptr);
	ptr->dump("[algoStubVisitor]:   ", false);

	METOUT();
}


void
PCERA::algoStubVisitor::visitGridService(GridService *	ptr)
{
	METIN();

	assert(ptr);
	ptr->dump("[algoStubVisitor]:   ");

	METOUT();
}


void
PCERA::algoStubVisitor::visitGridCompElem(GridCompElem *	ptr)
{
	METIN();

	assert(ptr);
	ptr->dump("[algoStubVisitor]:   ");

	METOUT();
}


void
PCERA::algoStubVisitor::visitGridSubCluster(GridSubCluster *	ptr)
{
	METIN();

	assert(ptr);
	ptr->dump("[algoStubVisitor]:   ");

	METOUT();
}


void
PCERA::algoStubVisitor::visitGridStorageElem(GridStorageElem *	ptr)
{
	METIN();

	assert(ptr);
	ptr->dump("[algoStubVisitor]:   ");

	METOUT();
}


std::list<ero_hop_t>
stubEroComp(uint32_t		rootLsr,
	    g2mpls_addr_t &	eTna,
	    uint32_t		dstNode)
{
	METIN();

	std::list<ero_hop_t>	tmp;
	g2mpls_addr_t		tna1, tna2;
	uint32_t		lsr1,  lsr9, lsr8, lsr11, lsr10;
	ero_hop_t		hop12, hop13;
	ero_hop_t		hop23;
	ero_hop_t		hop39;
	ero_hop_t		hop811, hop118;
	ero_hop_t		hop98, hop910, hop911;
	ero_hop_t		hop108, hop109, hop1011;
	ipv4_t			ridAddr, telAddr;
	label_id_t		label;

	tmp.clear();

	memset(&hop12,   0, sizeof(hop12));
	memset(&hop13,   0, sizeof(hop13));

	memset(&hop23,   0, sizeof(hop23));

	memset(&hop39,   0, sizeof(hop39));

	memset(&hop811,  0, sizeof(hop811));
	memset(&hop118,  0, sizeof(hop118));

	memset(&hop98,   0, sizeof(hop98));
	memset(&hop910,  0, sizeof(hop910));
	memset(&hop911,  0, sizeof(hop911));

	memset(&hop108,  0, sizeof(hop108));
	memset(&hop109,  0, sizeof(hop109));
	memset(&hop1011, 0, sizeof(hop1011));

	memset(&label, 0, sizeof(label));

	label.type = LABEL_32BIT;
	label.value.label32.id = 0x28000019;

	inet_aton("192.168.100.138", &ridAddr);
	inet_aton("1.2.1.2"        , &telAddr);
	hop12.node               = ntohl(ridAddr.s_addr);
	hop12.te_link.type       = IPv4;
	hop12.te_link.preflen    = 30;
	hop12.te_link.value.ipv4 = telAddr;
	hop12.upstream_label     = label;
	hop12.downstream_label   = label;

	inet_aton("192.168.100.139", &ridAddr);
	inet_aton("1.3.1.2"        , &telAddr);
	hop13.node               = ntohl(ridAddr.s_addr);
	hop13.te_link.type       = IPv4;
	hop13.te_link.preflen    = 30;
	hop13.te_link.value.ipv4 = telAddr;
	hop13.upstream_label     = label;
	hop13.downstream_label   = label;

	inet_aton("192.168.100.139", &ridAddr);
	inet_aton("2.3.1.2"        , &telAddr);
	hop23.node               = ntohl(ridAddr.s_addr);
	hop23.te_link.type       = IPv4;
	hop23.te_link.preflen    = 30;
	hop23.te_link.value.ipv4 = telAddr;
	hop23.upstream_label     = label;
	hop23.downstream_label   = label;

	inet_aton("192.168.100.9", &ridAddr);
	inet_aton("6.9.1.2"      , &telAddr);
	hop39.node               = ntohl(ridAddr.s_addr);
	hop39.te_link.type       = IPv4;
	hop39.te_link.preflen    = 30;
	hop39.te_link.value.ipv4 = telAddr;
	hop39.upstream_label     = label;
	hop39.downstream_label   = label;

	inet_aton("192.168.100.11", &ridAddr);
	inet_aton("8.11.1.2"      , &telAddr);
	hop811.te_link.type       = IPv4;
	hop811.te_link.preflen    = 30;
	hop811.te_link.value.ipv4 = telAddr;

	inet_aton("192.168.100.8", &ridAddr);
	inet_aton("8.11.1.1"      , &telAddr);
	hop118.node               = ntohl(ridAddr.s_addr);
	hop118.te_link.type       = IPv4;
	hop118.te_link.preflen    = 30;
	hop118.te_link.value.ipv4 = telAddr;

	inet_aton("192.168.100.8", &ridAddr);
	inet_aton("8.9.1.1"      , &telAddr);
	hop98.node               = ntohl(ridAddr.s_addr);
	hop98.te_link.type       = IPv4;
	hop98.te_link.preflen    = 30;
	hop98.te_link.value.ipv4 = telAddr;

	inet_aton("192.168.100.10", &ridAddr);
	inet_aton("9.10.1.2"      , &telAddr);
	hop910.node               = ntohl(ridAddr.s_addr);
	hop910.te_link.type       = IPv4;
	hop910.te_link.preflen    = 30;
	hop910.te_link.value.ipv4 = telAddr;

	inet_aton("192.168.100.11", &ridAddr);
	inet_aton("9.11.1.2"      , &telAddr);
	hop911.node               = ntohl(ridAddr.s_addr);
	hop911.te_link.type       = IPv4;
	hop911.te_link.preflen    = 30;
	hop911.te_link.value.ipv4 = telAddr;

	inet_aton("192.168.100.8", &ridAddr);
	inet_aton("8.10.1.1"      , &telAddr);
	hop108.node               = ntohl(ridAddr.s_addr);
	hop108.te_link.type       = IPv4;
	hop108.te_link.preflen    = 30;
	hop108.te_link.value.ipv4 = telAddr;

	inet_aton("192.168.100.9", &ridAddr);
	inet_aton("9.10.1.1"      , &telAddr);
	hop109.node               = ntohl(ridAddr.s_addr);
	hop109.te_link.type       = IPv4;
	hop109.te_link.preflen    = 30;
	hop109.te_link.value.ipv4 = telAddr;

	inet_aton("192.168.100.11", &ridAddr);
	inet_aton("10.11.1.2"      , &telAddr);
	hop1011.node               = ntohl(ridAddr.s_addr);
	hop1011.te_link.type       = IPv4;
	hop1011.te_link.preflen    = 30;
	hop1011.te_link.value.ipv4 = telAddr;

	inet_aton("192.168.100.137", &ridAddr);
	lsr1 =  ntohl(ridAddr.s_addr);

	inet_aton("192.168.100.9", &ridAddr);
	lsr9 =  ntohl(ridAddr.s_addr);

	inet_aton("192.168.100.8", &ridAddr);
	lsr8 =  ntohl(ridAddr.s_addr);

	inet_aton("192.168.100.10", &ridAddr);
	lsr10 =  ntohl(ridAddr.s_addr);

	inet_aton("192.168.100.11", &ridAddr);
	lsr11 =  ntohl(ridAddr.s_addr);

	inet_aton("20.20.20.1"      , &telAddr);
	tna1.type       = IPv4;
	tna1.preflen    = 32;
	tna1.value.ipv4 = telAddr;

	inet_aton("30.30.30.1"      , &telAddr);
	tna2.type       = IPv4;
	tna2.preflen    = 32;
	tna2.value.ipv4 = telAddr;

	std::vector< std::list<ero_hop_t> > erop(2);
	std::vector< std::list<ero_hop_t> > erou20(2);
	std::vector< std::list<ero_hop_t> > erou30(2);
	std::vector< std::list<ero_hop_t> > erouu20(2);
	std::vector< std::list<ero_hop_t> > erouu30(2);

	erop[0].push_front(hop39);
	erop[0].push_front(hop13);
	erop[1].push_front(hop39);
	erop[1].push_front(hop23);
	erop[1].push_front(hop12);

	erou20[0].push_front(hop98);
	erou20[1].push_front(hop108);
	erou20[1].push_front(hop910);

	erou30[0].push_front(hop911);
	erou30[1].push_front(hop1011);
	erou30[1].push_front(hop910);

	erouu20[0].push_front(hop98);
	erouu20[0].push_front(hop109);
	erouu20[1].push_front(hop118);
	erouu20[1].push_front(hop1011);

	erouu30[0].push_front(hop911);
	erouu30[0].push_front(hop109);
	erouu30[1].push_front(hop811);
	erouu30[1].push_front(hop98);
	erouu30[1].push_front(hop109);

	size_t pos;
	pos = myrand(0, 1);

	if (rootLsr == lsr1) {
		tmp = erop[pos];
	} else if (rootLsr == lsr9) {

		if (addr_equal(eTna, tna1) == 1) {
			tmp = erou20[pos];
		} else if (addr_equal(eTna, tna2) == 1) {
			tmp = erou30[pos];
		} else if (dstNode == lsr8) {
			tmp = erou20[pos];
		} else if (dstNode == lsr11) {
			tmp = erou30[pos];
		} else {
			G2PCERA_ERROR("Unsupported TNA case (%s)",
				      addr_ntoa(eTna));
		}
	} else if (rootLsr == lsr10) {

		if (addr_equal(eTna, tna1) == 1) {
			tmp = erouu20[pos];
		} else if (addr_equal(eTna, tna2) == 1) {
			tmp = erouu30[pos];
		} else if (dstNode == lsr8) {
			tmp = erouu20[pos];
		} else if (dstNode == lsr11) {
			tmp = erouu30[pos];
		} else {
			G2PCERA_ERROR("Unsupported TNA case (%s)",
				      addr_ntoa(eTna));
		}
	} else {
		G2PCERA_ERROR("Unsupported rootLsr (0x%08x/0x%08x/0x%08x)",
			      rootLsr, lsr1, lsr9);
	}

	METOUT();
	return tmp;
}


std::list<ero_hop_t>
stubGnsEroComp(uint32_t			rootLsr,
	       g2mpls_addr_t &		eTna,
	       std::list<ero_hop_t> &	ero)
{
	METIN();

	std::list<ero_hop_t>	tmp;
	uint32_t		lsr1, lsr9, lsr10;
	g2mpls_addr_t		tna1, tna2;
	ipv4_t			ridAddr, telAddr;

	tmp.clear();

	inet_aton("192.168.100.137", &ridAddr);
	lsr1 =  ntohl(ridAddr.s_addr);

	inet_aton("192.168.100.9", &ridAddr);
	lsr9 =  ntohl(ridAddr.s_addr);

	inet_aton("192.168.100.10", &ridAddr);
	lsr10 =  ntohl(ridAddr.s_addr);

	inet_aton("20.20.20.1"      , &telAddr);
	tna1.type       = IPv4;
	tna1.preflen    = 32;
	tna1.value.ipv4 = telAddr;

	inet_aton("30.30.30.1"      , &telAddr);
	tna2.type       = IPv4;
	tna2.preflen    = 32;
	tna2.value.ipv4 = telAddr;

	if (rootLsr == lsr1) {
		ero_hop_t hopdmn;

		memset(&hopdmn, 0, sizeof(hopdmn));
		tmp = ero;

		if (addr_equal(eTna, tna1) == 1) {
			inet_aton("192.168.100.8", &ridAddr);
		} else if (addr_equal(eTna, tna2) == 1) {
			inet_aton("192.168.100.11", &ridAddr);
		}
		inet_aton("0.0.0.0", &telAddr);
		hopdmn.node               = ntohl(ridAddr.s_addr);
		hopdmn.te_link.type       = IPv4;
		hopdmn.te_link.preflen    = 30;
		hopdmn.te_link.value.ipv4 = telAddr;

		tmp.push_back(hopdmn);
	} else if (G2PCERA_MASTER.routerId() == lsr9) {
		tmp.push_front(ero.back());
	} else if (G2PCERA_MASTER.routerId() == lsr10) {
		tmp.push_front(ero.back());
	}

	METOUT();
	return tmp;
}

g2mpls_addr_t
stubTnaFromGridSE(PCERA::Topology & topo)
{
	METIN();

	g2mpls_addr_t atmp;
	std::list<PCERA::Node *> nn;
	std::list<PCERA::Node *>::const_iterator iter;
	PCERA::GridStorageElem * max_storage;
	uint32_t free_size;

	memset(&atmp, 0, sizeof(atmp));

	nn = topo.getNodes();
	max_storage = 0;
	free_size   = 0;

	for (iter = nn.begin(); iter != nn.end(); iter++) {
		if ((*iter)->type() != TOPO_NODE_GRID) {
			continue;
		}
		PCERA::GridNode * tmp1;
		PCERA::GridStorageElem * tmp2;
		uint32_t tmpfree_size;

		tmp1 = static_cast<PCERA::GridNode *>(*iter);
		assert(tmp1);

		tmp2 = tmp1->getGridStorageElem();

		if (!tmp2) {
			continue;
		}

		grid_se_data_t data;
		if (!tmp2->getData(data)) {
			continue;
		}

		tmpfree_size = (data.online_size.total -
				data.online_size.used);

		if (tmpfree_size < free_size) {
			continue;
		}

		max_storage = tmp2;
		free_size   = tmpfree_size;

		G2PCERA_DEBUG(">>>> max_storage = %s, "
			      "free_size = %d",
			      subnodeIdent2string(tmp2->ident()).
			      c_str(),
			      free_size);
	}

	if (max_storage) {
		node_ident_t nid;
		tna_ident_t tna;
		std::list<tna_ident_t> tnas;
		PCERA::Node *		tmp;
		PCERA::NetNode *	ptr;

		nid.id = max_storage->parent()->getPeRouterId();
		nid.type = TOPO_NODE_NETWORK;

		tmp = topo.getNode(nid);
		ptr = static_cast<PCERA::NetNode *>(tmp);

		if (ptr) {
			tnas = ptr->getTnasIdents(0);
		} else {
			// peRouter could be under a domain
			std::list<node_ident_t>		   nodeList;
			std::list<node_ident_t>::iterator  iter;

			nodeList = topo.getNodesIdents();

			for (iter  = nodeList.begin();
			     iter != nodeList.end();
			     iter++) {
				node_ident_t	id;

				id = (*iter);

				if (id.type != TOPO_NODE_NETWORK) {
					continue;
				}

				tmp = topo.getNode(id);
				ptr = static_cast<PCERA::NetNode *>(tmp);
				if (!ptr->isDomain()) {
					continue;
				}

				tnas = ptr->getTnasIdents(nid.id);

				if (tnas.size()) {
					break;
				}
			}
		}

		if (!ptr) {
			G2PCERA_ERROR("Cannot get peering netNode %s",
				      nodeIdent2string(nid).c_str());
			METOUT();
			return atmp;
		}

		if (tnas.empty()) {
			G2PCERA_ERROR("Cannot get a TNA on "
				      "peering netNode %s",
				      nodeIdent2string(nid).c_str());
			METOUT();
			return atmp;
		}

		tna = tnas.front();

		atmp = tna.id;
	}

	METOUT();
	return atmp;
}

g2mpls_addr_t
stubTnaFromGridCE(PCERA::Topology & topo,
		  uint32_t          reqCPUCount)
{
	METIN();

	g2mpls_addr_t atmp;
	std::list<PCERA::Node *> nn;
	std::list<PCERA::Node *>::const_iterator iter;
	PCERA::GridCompElem * max_ce;
	uint32_t free_slots;

	memset(&atmp, 0, sizeof(atmp));

	nn = topo.getNodes();
	max_ce     = 0;
	free_slots = 0;

	for (iter = nn.begin(); iter != nn.end(); iter++) {
		if ((*iter)->type() != TOPO_NODE_GRID) {
			continue;
		}
		PCERA::GridNode * tmp1;
		PCERA::GridCompElem * tmp2;
		uint32_t tmpfree_slots;

		tmp1 = static_cast<PCERA::GridNode *>(*iter);
		assert(tmp1);

		tmp2 = tmp1->getGridCompElem();

		if (!tmp2) {
			continue;
		}

		grid_ce_data_t data;
		if (!tmp2->getData(data)) {
			continue;
		}

		tmpfree_slots = data.jobs_state.free_job_slots;

		if (tmpfree_slots < reqCPUCount) {
			continue;
		}

		if (tmpfree_slots < free_slots) {
			continue;
		}

		max_ce = tmp2;
		free_slots   = tmpfree_slots;

		G2PCERA_DEBUG(">>>> max_ce = %s, "
			      "free_slots = %d",
			      subnodeIdent2string(tmp2->ident()).
			      c_str(),
			      free_slots);
	}

	if (max_ce) {
		node_ident_t nid;
		tna_ident_t tna;
		std::list<tna_ident_t> tnas;
		PCERA::Node *		tmp;
		PCERA::NetNode *	ptr;

		nid.id = max_ce->parent()->getPeRouterId();
		nid.type = TOPO_NODE_NETWORK;

		tmp = topo.getNode(nid);
		ptr = static_cast<PCERA::NetNode *>(tmp);

		if (ptr) {
			tnas = ptr->getTnasIdents(0);
		} else {
			// peRouter could be under a domain
			std::list<node_ident_t>		   nodeList;
			std::list<node_ident_t>::iterator  iter;

			nodeList = topo.getNodesIdents();

			for (iter  = nodeList.begin();
			     iter != nodeList.end();
			     iter++) {
				node_ident_t	id;

				id = (*iter);

				if (id.type != TOPO_NODE_NETWORK) {
					continue;
				}

				tmp = topo.getNode(id);
				ptr = static_cast<PCERA::NetNode *>(tmp);
				if (!ptr->isDomain()) {
					continue;
				}

				tnas = ptr->getTnasIdents(nid.id);

				if (tnas.size()) {
					break;
				}
			}
		}

		if (!ptr) {
			G2PCERA_ERROR("Cannot get peering netNode %s",
				      nodeIdent2string(nid).c_str());
			METOUT();
			return atmp;
		}

		if (tnas.empty()) {
			G2PCERA_ERROR("Cannot get a TNA on "
				      "peering netNode %s",
				      nodeIdent2string(nid).c_str());
			METOUT();
			return atmp;
		}

		tna = tnas.front();

		atmp = tna.id;
	}

	METOUT();
	return atmp;
}
