//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//



#ifndef __G2PCERA_APIS_HH__
#define __G2PCERA_APIS_HH__

#include "g2pcera_dm.hh"
#include "g2pcera_common.hh"
#include <string>
#include <list>
#include <map>

//
//   Topology API
//

namespace PCERA_TOPO_API {

	pceraErrorCode_t
	topologyGet(uint32_t					g2pceraId,
		    topology_summary_data_t &			data,
		    std::string &				resp);

	pceraErrorCode_t
	nodeAdd(uint32_t					g2pceraId,
		const node_ident_t &				ident,
		std::string &					resp);

	pceraErrorCode_t
	nodeDel(uint32_t					g2pceraId,
		const node_ident_t &				ident,
		std::string &					resp);

	std::list<node_ident_t>
	nodeGetAll(uint32_t					g2pceraId,
		   std::string &				resp);

	pceraErrorCode_t
	netNodeUpdate(uint32_t					g2pceraId,
		      uint32_t					rId,
		      const net_node_data_t &			data,
		      std::string &				resp);

	pceraErrorCode_t
	netNodeGet(uint32_t					g2pceraId,
		   uint32_t					rId,
		   net_node_data_t &				data,
		   std::string &				resp);

#ifdef G2MPLS
	pceraErrorCode_t
	gridSiteUpdate(uint32_t					g2pceraId,
		       uint32_t					siteId,
		       const grid_site_data_t&			data,
		       std::string &				resp);

	pceraErrorCode_t
	gridSiteGet(uint32_t					g2pceraId,
		    uint32_t					siteId,
		    grid_site_data_t &				data,
		    grid_subnodes_t &				snodes,
		    std::string &				resp);

	pceraErrorCode_t
	gridSubNodeDel(uint32_t					g2pceraId,
		       uint32_t					siteId,
		       grid_subnode_ident_t &			snIdent,
		       std::string &				resp);

	pceraErrorCode_t
	gridServiceUpdate(uint32_t				g2pceraId,
			  uint32_t				siteId,
			  uint32_t				id,
			  const grid_service_data_t &		data,
			  std::string &				resp);

	pceraErrorCode_t
	gridServiceGet(uint32_t					g2pceraId,
		       uint32_t					siteId,
		       uint32_t					id,
		       grid_service_data_t &			data,
		       std::string &				resp);

	pceraErrorCode_t
	gridCompElemUpdate(uint32_t				g2pceraId,
			   uint32_t				siteId,
			   uint32_t				id,
			   const grid_ce_data_t &		data,
			   std::string &			resp);

	pceraErrorCode_t
	gridCompElemGet(uint32_t				g2pceraId,
			uint32_t				siteId,
			uint32_t				id,
			grid_ce_data_t &			data,
			std::string &				resp);

	pceraErrorCode_t
	gridSubClusterUpdate(uint32_t				g2pceraId,
			     uint32_t				siteId,
			     uint32_t				id,
			     const grid_subcluster_data_t &	data,
			     std::string &			resp);

	pceraErrorCode_t
	gridSubClusterGet(uint32_t				g2pceraId,
			  uint32_t				siteId,
			  uint32_t				id,
			  grid_subcluster_data_t &		data,
			  std::string &				resp);

	pceraErrorCode_t
	gridStorageElemUpdate(uint32_t				g2pceraId,
			      uint32_t				siteId,
			      uint32_t				id,
			      const grid_se_data_t &		data,
			      std::string &			resp);

	pceraErrorCode_t
	gridStorageElemGet(uint32_t				g2pceraId,
			   uint32_t				siteId,
			   uint32_t				id,
			   grid_se_data_t &			data,
			   std::string &			resp);
#endif // G2MPLS

	pceraErrorCode_t
	tnaAdd(uint32_t						g2pceraId,
	       const tna_ident_t &				ident,
	       std::string &					resp);

	pceraErrorCode_t
	tnaDel(uint32_t						g2pceraId,
	       const tna_ident_t &				ident,
	       std::string &					resp);

	std::list<tna_ident_t>
	tnaGetAllFromNode(uint32_t				g2pceraId,
			  uint32_t				rId,
			  bool					isDomain,
			  std::string &				resp);

	pceraErrorCode_t
	linkAdd(uint32_t					g2pceraId,
		const telink_ident_t &				ident,
		std::string &					resp);

	pceraErrorCode_t
	linkDel(uint32_t					g2pceraId,
		const telink_ident_t &				ident,
		std::string &					resp);

	std::list<telink_ident_t>
	teLinkGetAllFromNode(uint32_t				g2pceraId,
			     uint32_t				rId,
			     std::string &			resp);

	pceraErrorCode_t
	teLinkUpdateCom(uint32_t				g2pceraId,
			const telink_ident_t &			ident,
			const telink_com_data_t &		data,
			std::string &				resp);

	pceraErrorCode_t
	teLinkGetCom(uint32_t					g2pceraId,
		     const telink_ident_t &			ident,
		     telink_com_data_t &			data,
		     std::string &				resp);

	pceraErrorCode_t
	teLinkUpdateTdm(uint32_t				g2pceraId,
			const telink_ident_t &			ident,
			const telink_tdm_data_t &		data,
			std::string &				resp);

	pceraErrorCode_t
	teLinkGetTdm(uint32_t					g2pceraId,
		     const telink_ident_t &			ident,
		     telink_tdm_data_t &			data,
		     std::string &				resp);

	pceraErrorCode_t
	teLinkUpdateLscG709(uint32_t				g2pceraId,
			    const telink_ident_t &		ident,
			    const telink_lscg709_data_t &	data,
			    std::string &			resp);

	pceraErrorCode_t
	teLinkGetLscG709(uint32_t				g2pceraId,
			 const telink_ident_t &			ident,
			 telink_lscg709_data_t &		data,
			 std::string &				resp);

	pceraErrorCode_t
	teLinkUpdateLscWdm(uint32_t				g2pceraId,
			   const telink_ident_t &		ident,
			   const telink_lscwdm_data_t &		data,
			   std::string &			resp);

	pceraErrorCode_t
	teLinkGetLscWdm(uint32_t				g2pceraId,
			const telink_ident_t &			ident,
			telink_lscwdm_data_t &			data,
			std::string &				resp);

	pceraErrorCode_t
	teLinkUpdateStates(uint32_t				g2pceraId,
			   const telink_ident_t &		ident,
			   opstate_t				opState,
			   admstate_t				admState,
			   std::string &			resp);

	pceraErrorCode_t
	teLinkGetStates(uint32_t				g2pceraId,
			const telink_ident_t &			ident,
			opstate_t &				opState,
			admstate_t &				admState,
			std::string &				resp);

	pceraErrorCode_t
	teLinkUpdateGenBw(uint32_t				g2pceraId,
			  const telink_ident_t &		ident,
			  const bw_per_prio_t &			avail_bw,
			  std::string &				resp);

	pceraErrorCode_t
	teLinkGetGenBw(uint32_t					g2pceraId,
		       const telink_ident_t &			ident,
		       bw_per_prio_t &				avail_bw,
		       std::string &				resp);

	pceraErrorCode_t
	teLinkUpdateTdmBw(uint32_t				g2pceraId,
			  const telink_ident_t &		ident,
			  const std::list<uint32_t> &		freeTS,
			  std::string &				resp);

	pceraErrorCode_t
	teLinkGetTdmBw(uint32_t					g2pceraId,
		       const telink_ident_t &			ident,
		       std::list<uint32_t> &			freeTS,
		       std::string &				resp);

	pceraErrorCode_t
	teLinkUpdateLscG709Bw(uint32_t				g2pceraId,
			      const telink_ident_t &		ident,
			      const std::list<uint32_t> &	freeODUk,
			      const std::list<uint32_t> &	freeOCh,
			      std::string &			resp);

	pceraErrorCode_t
	teLinkGetLscG709Bw(uint32_t				g2pceraId,
			   const telink_ident_t &		ident,
			   std::list<uint32_t> &		freeODUk,
			   std::list<uint32_t> &		freeOCh,
			   std::string &			resp);

	pceraErrorCode_t
	teLinkUpdateLscWdmBw(uint32_t				g2pceraId,
			     const telink_ident_t &		ident,
			     wdm_link_lambdas_bitmap_t &	bm,
			     std::string &			resp);

	pceraErrorCode_t
	teLinkGetLscWdmBw(uint32_t				g2pceraId,
			  const telink_ident_t &		ident,
			  wdm_link_lambdas_bitmap_t &		bm,
			  std::string &				resp);

	pceraErrorCode_t
	teLinkAppendSrlgs(uint32_t				g2pceraId,
			  const telink_ident_t &		ident,
			  const std::list<uint32_t> &		srlgs,
			  std::string &				resp);

	pceraErrorCode_t
	teLinkGetSrlgs(uint32_t					g2pceraId,
		       const telink_ident_t &			ident,
		       std::list<uint32_t> &			srlgs,
		       std::string &				resp);

	pceraErrorCode_t
	teLinkAppendCalendar(uint32_t				g2pceraId,
			     const telink_ident_t &		ident,
			     const std::map<uint32_t,bw_per_prio_t> & cal,
			     std::string &			resp);

	pceraErrorCode_t
	teLinkGetCalendar(uint32_t				g2pceraId,
			  const telink_ident_t &		ident,
			  std::map<uint32_t,bw_per_prio_t> &	cal,
			  std::string &				resp);


	pceraErrorCode_t
	teLinkAppendIsc(uint32_t				g2pceraId,
			const telink_ident_t &			ident,
			const std::list<isc_t> &		iscs,
			std::string &				resp);

	pceraErrorCode_t
	teLinkGetIsc(uint32_t					g2pceraId,
		     const telink_ident_t &			ident,
		     std::list<isc_t> &				iscs,
		     std::string &				resp);
};


//
//   ComputAPI
//
namespace PCERA_CALC_API {

	pceraErrorCode_t
	nodeGetFromTnaRes(uint32_t				g2pceraId,
			  const  net_res_spec_t &		tnaRes,
			  uint32_t	&			rId,
			  std::string &				resp);

	pceraErrorCode_t
	nodeGetFromTna(uint32_t				g2pceraId,
		       const  g2mpls_addr_t &		tna,
		       uint32_t &			rId,
		       std::string &			resp);
#ifdef G2MPLS
	pceraErrorCode_t
	nodeGetFromGnsTna(uint32_t			g2pceraId,
			  const grid_res_spec_t &	tnaGnsRes,
			  const std::list<uint32_t> &	excludeSet,
			  uint32_t &			netNodeId,
			  uint32_t &			gridSiteId,
			  std::string &			resp);
#endif // G2MPLS

	pceraErrorCode_t
	nodeGetFromTeLink(uint32_t			g2pceraId,
			  const g2mpls_addr_t &		teLink,
			  uint32_t &			rId,
			  uint32_t &			adjRid,
			  std::string &			resp);

	pceraErrorCode_t
	callRoute(uint32_t				g2pceraId,
		  const net_res_spec_t &		src,
		  const net_res_spec_t &		dst,
		  const call_ident_t &			callIdent,
		  const call_info_t &			callInfo,
		  const recovery_info_t &		recInfo,
		  const lsp_info_t &			lspInfo,
		  std::list<ero_hop_t> &		wEro,
		  std::string &				resp);

	pceraErrorCode_t
	callRouteAny(uint32_t				g2pceraId,
		     const net_res_spec_t &		src,
		     const grid_res_spec_t &		dstr,
		     const call_ident_t &		callIdent,
		     const call_info_t &		callInfo,
		     const recovery_info_t &		recInfo,
		     const lsp_info_t &			lspInfo,
		     std::list<ero_hop_t> &		wEro,
		     net_res_spec_t &			eTna,
		     std::string &			resp);

	pceraErrorCode_t
	callFlush(uint32_t				g2pceraId,
		  const call_ident_t &			callIdent,
		  std::string &				resp);

	pceraErrorCode_t
	callConfirm(uint32_t				g2pceraId,
		    const call_ident_t &		callIdent,
		    std::string &			resp);

	pceraErrorCode_t
	connectionRoute(uint32_t			g2pceraId,
			const ero_hop_t &		srcHop,
			const ero_hop_t &		dstHop,
			const call_ident_t &		callIdent,
			const lsp_info_t &		lspInfo,
			std::list<ero_hop_t> &		wEro,
			std::list<ero_hop_t> &		pEro,
			std::string &			resp);

	//	typedef std::list<ero_hop_t> ero_t;

	pceraErrorCode_t
	connectionRouteDisjoint(uint32_t			g2pceraId,
				const ero_hop_t &		srcHop,
				const ero_hop_t &		dstHop,
				const call_ident_t &		callIdent,
				const lsp_info_t &		lspInfo,
				const std::list< std::list<ero_hop_t> > & mXro,
				std::list<ero_hop_t> &		wEro,
				std::list<ero_hop_t> &		pEro,
				std::string &			resp);
};

//
//   PCERA API
//
namespace PCERA_API {
        pceraErrorCode_t	g2pceraStart(std::string &		resp);
        pceraErrorCode_t	g2pceraStop(std::string &		resp);

        pceraErrorCode_t	g2pceraStart(uint32_t			rootId,
					     std::string &		resp);
        pceraErrorCode_t	g2pceraStop(uint32_t			rootId,
					    std::string &		resp);

        pceraErrorCode_t	g2pceraLoadFromFile(std::string &	filenm,
						    std::string &	resp);

        std::list<uint32_t>	g2pceraGetAll(void);
};

#endif // __G2PCERA_APIS_HH__
