//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//



#include "g2pcera_common.hh"
#include "g2pcera_trace.hh"

#include <string>
#include <sstream>
#include <iomanip>
#include <iostream>
#include <stdlib.h>

size_t
myrand(size_t min, size_t max)
{
	float fmax;
	float fmin;

	fmax = (float) max;
	fmin = (float) min;

	return (min + (size_t) (fmax * (rand() / (RAND_MAX + 1.0))));
}

bool
ero_hop::is_null(void) const
{
	if (loose) {
		return false;
	}
	if (node) {
		return false;
	}
	if (is_addr_null(te_link) == 0) {
		return false;
	}
	if (is_addr_null(upstream_datalink) == 0) {
		return false;
	}
	if (is_addr_null(downstream_datalink) == 0) {
		return false;
	}

	label_id_t nullLbl;

	memset(&nullLbl, 0, sizeof(nullLbl));

	if (upstream_label != nullLbl) {
		return false;
	}
	if (downstream_label != nullLbl) {
		return false;
	}

	if (idmn) {
		return false;
	}

	return true;
}

bool
ero_hop::operator==(const struct ero_hop & other) const
{
	if (loose != other.loose) {
		return false;
	}
	if (node != other.node) {
		return false;
	}
	if ((is_addr_null(te_link) == 0) &&
	    (is_addr_null(other.te_link) == 0) &&
	    (addr_equal(te_link, other.te_link) == 0)) {
		return false;
	}
	if ((is_addr_null(upstream_datalink) == 0) &&
	    (is_addr_null(other.upstream_datalink) == 0) &&
	    (addr_equal(upstream_datalink, other.upstream_datalink) == 0)) {
		return false;
	}
	if ((is_addr_null(downstream_datalink) == 0) &&
	    (is_addr_null(other.downstream_datalink) == 0) &&
	    (addr_equal(downstream_datalink, other.downstream_datalink) == 0)){
		return false;
	}

	label_id_t nullLbl;

	memset(&nullLbl, 0, sizeof(nullLbl));

	if ((upstream_label != nullLbl) &&
	    (other.upstream_label != nullLbl) &&
	    (upstream_label != other.upstream_label)) {
		return false;
	}
	if ((downstream_label != nullLbl) &&
	    (other.downstream_label != nullLbl) &&
	    (downstream_label != other.downstream_label)) {
		return false;
	}

	if (idmn != other.idmn) {
		return false;
	}

	return true;
}

bool
ero_hop::operator!=(const struct ero_hop & other) const
{
	return !(*this == other);
}

bool
node_ident::operator==(const struct node_ident & other) const
{
	if (id != other.id) {
		return false;
	}
	if (type != other.type) {
		return false;
	}

	return true;
}

bool
node_ident::operator!=(const struct node_ident & other) const
{
	return !(*this == other);
}

bool
grid_subnode_ident::operator==(const struct grid_subnode_ident & other) const
{
	if (id != other.id) {
		return false;
	}
	if (type != other.type) {
		return false;
	}

	return true;
}

bool
grid_subnode_ident::operator!=(const struct grid_subnode_ident & other) const
{
	return !(*this == other);
}


bool
telink_ident::operator==(const struct telink_ident & other) const
{
	if (lcl_rid != other.lcl_rid) {
		return false;
	}
	if (addr_equal(lcl_id, other.lcl_id) == 0) {
		return false;
	}
	if (rem_rid != other.rem_rid) {
		return false;
	}
	if (addr_equal(rem_id, other.rem_id) == 0) {
		return false;
	}
	if ((mode != LINK_MODE_ENNI_INTERDOMAIN) &&
	    (mode != LINK_MODE_ENNI_INTRADOMAIN) &&
	    !lcl_rc_id                           &&
	    !rem_rc_id) {
		// partial match is acceptable
		return true;
	}
	if ((other.mode != LINK_MODE_ENNI_INTERDOMAIN) &&
	    (other.mode != LINK_MODE_ENNI_INTRADOMAIN) &&
	    !other.lcl_rc_id                           &&
	    !other.rem_rc_id) {
		// partial match is acceptable
		return true;
	}
	if (mode != other.mode) {
		return false;
	}
	if (lcl_rc_id != other.lcl_rc_id) {
		return false;
	}

	if (rem_rc_id != other.rem_rc_id) {
		return false;
	}

	return true;
}

bool
telink_ident::operator!=(const struct telink_ident & other) const
{
	return !(*this == other);
}

bool
tna_ident::operator==(const struct tna_ident & other) const
{
	if (rc_id != other.rc_id) {
		return false;
	}
	if (rid != other.rid) {
		return false;
	}
	if (addr_equal(id, other.id) == 0) {
		return false;
	}

	return true;
}

bool
tna_ident::operator!=(const struct tna_ident & other) const
{
	return !(*this == other);
}

bool
tna_ident::operator>=(const struct tna_ident & other) const
{
	if (rc_id != other.rc_id) {
		return false;
	}
	if (rid != other.rid) {
		return false;
	}

	if (addr_in_net(id, other.id) == 0) {
		return false;
	}

	return true;
}

bool
isc::operator==(const struct isc & other) const
{
	size_t i;

	if (switching != other.switching) {
		return false;
	}
	if (encoding != other.encoding) {
		return false;
	}
	for (i = 0; i < GMPLS_BW_PRIORITIES; i++) {
		if (max_lsp_bw.bw[i] != other.max_lsp_bw.bw[i]) {
			return false;
		}
	}
	if (spec.raw != other.spec.raw) {
		return false;
	}
	return true;
}

bool
isc::operator!=(const struct isc & other) const
{
	return !(*this == other);
}

std::string
bool2string(bool key)
{
	std::string        tmp;
	std::ostringstream localId;

	tmp = std::string("");

	localId << key ? "True" : "False";

	tmp += localId.str();

	return tmp;
}

std::string
uint8string(const uint8_t & key)
{
	std::string		tmp;
	std::ostringstream	localId;
	size_t			id;

	id = key;

	tmp = std::string("");

	localId << id << "(0x" << std::hex
		<< std::setw(2) << std::setfill('0')  << id << ")";

	tmp += localId.str();

	return tmp;
}

std::string
uint16string(const uint16_t & key)
{
	std::string        tmp;
	std::ostringstream localId;

	tmp = std::string("");

	localId << key << "(0x" << std::hex
		<< std::setw(4) << std::setfill('0')  << key << ")";

	tmp += localId.str();

	return tmp;
}


std::string
uint32string(const uint32_t & key)
{
	std::string        tmp;
	std::ostringstream localId;

	tmp = std::string("");

	localId << key << "(0x" << std::hex
		<< std::setw(8) << std::setfill('0')  << key << ")";

	tmp += localId.str();

	return tmp;
}

std::string
key2string(const uint64_t & key)
{
	std::string        tmp;
	std::ostringstream localId;

	tmp = std::string("");

	localId << "0x" << std::hex
		<< std::setw(16) << std::setfill('0')  << key;

	tmp += localId.str();

	return tmp;
}

std::string
version2string(const version_t & ver)
{
	std::string        tmp;
	std::ostringstream verStr;

	tmp = std::string("");

	verStr << "v" << ver.major_revision
	       << "." << ver.minor_revision
	       << "." << ver.build_fix;

	tmp += verStr.str();

	return tmp;
}

std::string
range2string(const range_set_t & range)
{
	std::string        tmp;
	std::ostringstream rangeStr;

	tmp = std::string("");

	rangeStr << ((range.lb_included == 1) ? "[" : "(")
		 << std::setfill('0') << range.lower_bound
		 << ","
		 << std::setfill('0') << range.upper_bound
		 << ((range.ub_included == 1) ? "]" : ")");

	tmp += rangeStr.str();

	return tmp;
}

std::string
nodeIdent2string(const node_ident_t & ident)
{
	std::string        tmp;
	std::ostringstream nodeStr;

	tmp = std::string("");

	nodeStr << "<" << NODE_TYPE_TO_STRING(ident.type) << "/ "
		<< "ID=" << numb_htoa(ident.id)
		<< ">";

	tmp += nodeStr.str();

	return tmp;
}

std::string
subnodeIdent2string(const grid_subnode_ident_t & ident)
{
	std::string        tmp;
	std::ostringstream nodeStr;

	tmp = std::string("");

	nodeStr << "<" << GRID_SUBNODE_TO_STRING(ident.type) << "/ "
		<< "ID=" << numb_htoa(ident.id)
		<< ">";

	tmp += nodeStr.str();

	return tmp;
}

std::string
teLinkIdent2string(const telink_ident_t & ident)
{
	std::string        tmp;
	std::ostringstream linkStr;

	tmp = std::string("");

	linkStr << "<LCL= " << addr_ntoa(ident.lcl_id);
	linkStr << " (RID= " << numb_htoa(ident.lcl_rid);
	linkStr << " , RC-ID= " << numb_htoa(ident.lcl_rc_id) << ")";
	linkStr << " / REM= " << addr_ntoa(ident.rem_id);
	linkStr << " (RID= " << numb_htoa(ident.rem_rid);
	linkStr << " , RC-ID= " << numb_htoa(ident.rem_rc_id) << ")";
	linkStr << " / MODE= " << LINK_MODE_TO_STRING(ident.mode) << ">";

	tmp += linkStr.str();

	return tmp;
}

std::string
tnaIdent2string(const tna_ident_t & ident)
{
	std::string        tmp;
	std::ostringstream tnaStr;

	tmp = std::string("");

	tnaStr << addr_ntoa(ident.id);
	tnaStr << " (RID= " << numb_htoa(ident.rid) << ",";
	tnaStr << " RC-ID= "
	       << (ident.rc_id ? numb_htoa(ident.rc_id) : "<mine>") << ")";

	tmp += tnaStr.str();

	return tmp;
}

std::string
rid2string(const uint32_t & rid)
{
	std::string        tmp;
	std::ostringstream ridStr;

	tmp = std::string("");

	ridStr << numb_htoa(rid);

	tmp += ridStr.str();

	return tmp;
}

std::string
geoCoord2string(const geo_coords_t & geo)
{
	std::string        tmp;
	std::ostringstream geoStr;

	double	val1;
	double	val2;
	size_t	val1res;
	size_t	val2res;

	val1    = (*((double *)(&(geo.latitude))));
	val2    = (*((double *)(&(geo.longitude))));
	val1res = geo.lat_resolution;
	val2res = geo.lon_resolution;

	tmp = std::string("");

	geoStr << "LAT_r"   << val1res << "="
	       << std::fixed << std::setprecision(3) << val1
	       << "/LONG_r" << val2res << "="
	       << std::fixed << std::setprecision(3) << val2;

	tmp += geoStr.str();

	return tmp;
}

std::string
unixTime2string(const uint32_t & t)
{
	std::string        tmp;

	tm *               curTime;
	const char *       timeStringFormat = "%a, %d %b %Y %H:%M:%S %zGMT";
	const int          timeStringLength = 40;
	char               timeString[timeStringLength];


	curTime = localtime((time_t *) &t);
	strftime(timeString, timeStringLength, timeStringFormat, curTime);

	tmp += timeString;

	return tmp;
}

std::string
wdmLabel2string(const uint32_t & t)
{
	std::string        tmp;
	std::ostringstream lblStr;

	if (!t) {
		tmp = "0";
		return tmp;
	}

	switch (GMPLS_LABEL_TO_WDM_GRID(t)) {
		case WDM_GRID_ITUT_DWDM: {
			float lambda;

			GMPLS_LABEL_TO_DWDM_LAMBDA(t, lambda);
			lblStr << "[DWDM] "
			       << std::fixed << std::setprecision(3)
			       << lambda << " GHz";
		}
			break;
		case WDM_GRID_ITUT_CWDM: {
			uint32_t wave;

			GMPLS_LABEL_TO_CWDM_WAVELENGTH(t, wave);
			lblStr << "[CWDM] "  << wave << " nm";
		}
			break;
		default:
			BUG();
			break;
	}

	tmp += lblStr.str();

	return tmp;
}

std::string
nodeFlags2string(const nodeFlags_t & flags)
{
	std::string        tmp;

	tmp = std::string("");

	if (flags.inCandidateList)     { tmp +=  "In Candidate List, ";    }
	if (flags.inTree)              { tmp +=  "In Tree, ";              }
	if (flags.inTopoTransform)     { tmp +=  "In Topol. Transform, ";  }
	if (flags.inBhandariTransform) { tmp +=  "In Bhandari Transform, ";}

	if (tmp.empty())               { tmp = "=== EMPTY ==="; }

	return tmp;
}

bool
isValid(const net_res_spec_t & res, std::string & err)
{
	err = "";

	if (!BITMASK_BITTEST(res.mask_, tna)) {
		err = "tna unspecified";
		return false;
	}
	if (BITMASK_BITTEST(res.mask_, tna) &&
	    (is_addr_null(res.tna) != 0)) {
		err = "null tna";
		return false;
	}
	if (BITMASK_BITTEST(res.mask_, data_link) &&
	    (is_addr_null(res.data_link) != 0)) {
		err = "null data_link";
		return false;
	}

	label_id_t nullLbl;

	memset(&nullLbl, 0, sizeof(nullLbl));
	if (BITMASK_BITTEST(res.mask_, label) &&
	    (res.label == nullLbl)) {
		err = "null label";
		return false;
	}

	return true;
}

bool
isValid(const grid_res_spec_t & res, std::string & err)
{
	if (!BITMASK_BITTEST(res.mask_, application)    &&
	    !BITMASK_BITTEST(res.mask_, cand_host)      &&
	    !BITMASK_BITTEST(res.mask_, fs_resources)   &&
	    !BITMASK_BITTEST(res.mask_, sys_caps)       &&
	    !BITMASK_BITTEST(res.mask_, ind_cpu_speed)  &&
	    !BITMASK_BITTEST(res.mask_, ind_cpu_time)   &&
	    !BITMASK_BITTEST(res.mask_, ind_cpu_count)  &&
	    !BITMASK_BITTEST(res.mask_, ind_net_bw)     &&
	    !BITMASK_BITTEST(res.mask_, ind_phy_mem)    &&
	    !BITMASK_BITTEST(res.mask_, ind_vir_mem)    &&
	    !BITMASK_BITTEST(res.mask_, ind_disk_space) &&
	    !BITMASK_BITTEST(res.mask_, tot_cpu_time)   &&
	    !BITMASK_BITTEST(res.mask_, tot_cpu_count)  &&
	    !BITMASK_BITTEST(res.mask_, tot_phy_mem)    &&
	    !BITMASK_BITTEST(res.mask_, tot_vir_mem)    &&
	    !BITMASK_BITTEST(res.mask_, tot_disk_space) &&
	    !BITMASK_BITTEST(res.mask_, tot_res_count)  &&
	    !BITMASK_BITTEST(res.mask_, data_staging)   &&
	    !BITMASK_BITTEST(res.mask_, grid_site_id)) {
		err = "grid_res_spec unspecified";
		return false;
	}

	// XXX more to add
	return true;
}

bool
isValid(const call_ident_t &  id, std::string & err)
{
	err = "";

	if (is_addr_null(id.src_addr) != 0) {
		err = "src_addr unspecified";
		return false;
	}

	if (!id.local_id) {
		err = "local_id unspecified";
		return false;
	}

	if ((id.type == CALLID_GLOBUNIQ) && !id.itu_country_code) {
		err = "itu_country_code unspecified";
		return false;
	}

	if ((id.type == CALLID_GLOBUNIQ) && !id.itu_carrier_code) {
		err = "itu_carrier_code unspecified";
		return false;
	}

	if ((id.type == CALLID_GLOBUNIQ) && !id.unique_ap) {
		err = "unique_ap unspecified";
		return false;
	}

	return true;
}

bool
isValid(const times_t times, std::string & err)
{
	err = "";

	if (!times.start_time && !times.end_time) {
		return true;
	}

	time_t      unixTime;

	time(&unixTime);

	if (times.start_time < unixTime) {
		err  = "time specification mismatch";
		err += " -- start-time in the past";
		return false;
	}

	if (times.end_time < unixTime) {
			err  = "time specification mismatch";
			err += "  -- end-time in the past";
			return false;
	}

	if (times.start_time >= times.end_time) {
		err  = "time specification mismatch";
		err += " -- start and end-times collapse";
		return false;
	}

	return true;
}

bool
isValid(const call_info_t & callInfo, std::string & err)
{
	err = "";

	if (!BITMASK_BITTEST(callInfo.mask_, call_type)) {
		err = "call_type not specified";
		return false;
	}
	if (callInfo.call_type > CALL_TYPE_aEGWzEGW) {
		err = "call_type out-of-range";
		return false;
	}
	if (BITMASK_BITTEST(callInfo.mask_, times) &&
	    !isValid(callInfo.times, err)) {
		return false;
	}
	if (BITMASK_BITTEST(callInfo.mask_, iTNA_res)) {
		if (BITMASK_BITTEST(callInfo.iTNA_res.mask_, net) &&
		    !isValid(callInfo.iTNA_res.net, err)) {
			err += " in iTNA_res";
			return false;
		}
		if (BITMASK_BITTEST(callInfo.iTNA_res.mask_, grid) &&
		    !isValid(callInfo.iTNA_res.grid, err)) {
			err += " in iTNA_res";
			return false;
		}
	}
	if (BITMASK_BITTEST(callInfo.mask_, eTNA_res)) {
		if (BITMASK_BITTEST(callInfo.eTNA_res.mask_, net) &&
		    !isValid(callInfo.eTNA_res.net, err)) {
			err += " in eTNA_res";
			return false;
		}
		if (BITMASK_BITTEST(callInfo.eTNA_res.mask_, grid) &&
		    !isValid(callInfo.eTNA_res.grid, err)) {
			err += " in eTNA_res";
			return false;
		}
	}

	return true;
}

bool
isValid(const recovery_info_t & recInfo, std::string & err)
{
	err = "";

	if ((!BITMASK_BITTEST(recInfo.mask_, rec_type) &&
	     BITMASK_BITTEST(recInfo.mask_, disj_type)) ||
	    (BITMASK_BITTEST(recInfo.mask_, rec_type) &&
	     !BITMASK_BITTEST(recInfo.mask_, disj_type))) {
		err = "recovery_info not fully specified";
		return false;
	}
	if (BITMASK_BITTEST(recInfo.mask_, rec_type) &&
	    recInfo.rec_type > RECOVERY_REVERTIVEONTHEFLY) {
		err = "recovery_type out-of-range";
		return false;
	}
	if (BITMASK_BITTEST(recInfo.mask_, disj_type) &&
	    recInfo.disj_type > DISJOINTNESS_SRLG) {
		err = "disjointness_type out-of-range";
		return false;
	}

	return true;
}

bool
isValid(const lsp_info_t & lspInfo, std::string & err)
{
	err = "";

	if (!BITMASK_BITTEST(lspInfo.mask_, sw_cap)) {
		err = "sw_cap not specified";
		return false;
	}
	if (!BITMASK_BITTEST(lspInfo.mask_, enc_type)) {
		err = "enc_type not specified";
		return false;
	}
	if (!BITMASK_BITTEST(lspInfo.mask_, gpid)) {
		err = "gpid not specified";
		return false;
	}
	if (!BITMASK_BITTEST(lspInfo.mask_, bw)) {
		err = "bw not specified";
		return false;
	}
	if (!lspInfo.bw) {
		err = "null bw value";
		return false;
	}
	if (BITMASK_BITTEST(lspInfo.mask_, times) &&
	    !isValid(lspInfo.times, err)) {
		return false;
	}

	// XXX more to add ??
	return true;
}

bool
isValid(const ero_hop_t & hop, std::string & err)
{
	err = "";

	if (hop.loose) {
		err = "loose ero hop not supported";
		return false;
	}
	if (!hop.node && is_addr_null(hop.te_link) != 0) {
		err = "Both node and te_link are null in this ero_hop";
		return false;
	}
	if (!hop.node && (hop.te_link.type == UNNUMBERED)) {
		err = "unnumbered te_link without node specification";
		return false;
	}

	// XXX more to add ??
	return true;
}

bool
isValid(const std::list<ero_hop_t> & ero, std::string & err)
{

	err = "";

	std::list<ero_hop_t>::const_iterator iter;
	for (iter = ero.begin(); iter != ero.end(); iter++) {
		if (!isValid(*iter, err)) {
			err += " in ERO";
			return false;
		}
	}

	return true;
}
