//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//



#include <zebra.h>

#include "thread.h"
#include "vty.h"
#include "command.h"
#include "linklist.h"
#include "prefix.h"
#include "table.h"
#include "memory.h"
#include "stream.h"
#include "log.h"
#include "sockunion.h"
#include "network.h"

#include "g2pcerad.hh"
#include "g2pcera_trace.hh"

//
// Statics initialization
//
G2PCERA_Master *		G2PCERA_Master::instance_  = 0;
struct thread_master *		G2PCERA_Master::master_    = 0;
time_t				G2PCERA_Master::startTime_ = 0;
bool				G2PCERA_Master::dryrun_    = false;
uint32_t			G2PCERA_Master::routerId_  = 0;

std::list<PCERA::G2PCERA *>	G2PCERA_Master::g2pceras_;

#if HAVE_OMNIORB
LRM::Info_var				G2PCERA_Master::lrmProxy_  = 0;
#endif // HAVE_OMNIORB


//
// CLASS G2PCERA_Master implementation
//
G2PCERA_Master::G2PCERA_Master(void)
{
	METIN();

	G2PCERA_WARNING("Called G2PCERA_Master constructor");

	METOUT();
}

G2PCERA_Master::~G2PCERA_Master(void)
{
	METIN();

	G2PCERA_WARNING("Called G2PCERA_Master destructor");

	METOUT();
}

bool
G2PCERA_Master::init(void)
{
	METIN();

	G2PCERA_DEBUG("Initialising G2PCERA_Master");

	instance_ = new G2PCERA_Master;
	if (!instance_) {
		G2PCERA_ERROR("Cannot allocate enough memory ...");
		METOUT();
		assert(0);
	}

	master_ = thread_master_create();
	if (!master_) {
		G2PCERA_ERROR("Cannot create ZEBRA thread master ");
		assert(0);
	}

	g2pceras_.clear();

	startTime_ = quagga_time(NULL);
	dryrun_    = false;
	routerId_  = G2PCERA_DEFAULT_RID;

	METOUT();

	return true;
}

bool
G2PCERA_Master::destroy(void)
{
	METIN();

	G2PCERA_DEBUG("Finalising G2PCERA_Master");

	startTime_ = 0;

#if HAVE_OMNIORB
	lrmProxy_ = 0;
#endif // HAVE_OMNIORB

	std::list<PCERA::G2PCERA *>::iterator iter;
	for (iter = g2pceras_.begin(); iter != g2pceras_.end(); iter++) {
		delete (*iter);
	}
	g2pceras_.clear();

	delete instance_;
	instance_ = 0;

	thread_master_free(master_);
	master_ = 0;

	METOUT();

	return true;
}

G2PCERA_Master &
G2PCERA_Master::instance(void)
{
	METIN();

	if (instance_ == 0) {
		G2PCERA_DEBUG("G2PCERA_Master singleton instance is empty\n");

		instance_ = new G2PCERA_Master;
		if (!instance_) {
			G2PCERA_ERROR("Cannot allocate enough memory ...");
			METOUT();
			assert(0);
		}
	}

	METOUT();

	return * instance_;
}

struct thread_master *
G2PCERA_Master::getMaster(void)
{
	METIN();

	//G2PCERA_DEBUG("returning PCERA master of threads");
	METOUT();

	return master_;
}

std::list<PCERA::G2PCERA *>
G2PCERA_Master::getPceList(void)
{
	METIN();
	std::list<PCERA::G2PCERA *> tmp;

	tmp.clear();

	tmp = g2pceras_;

	METOUT();

	return tmp;
}

PCERA::G2PCERA *
G2PCERA_Master::getPceInstance(void)
{
	METIN();

	if (g2pceras_.empty()) {
		return 0;
	}

	PCERA::G2PCERA * pce;

	pce = *(g2pceras_.begin());

	METOUT();

	return pce;
}

PCERA::G2PCERA *
G2PCERA_Master::getPceInstance(uint32_t  rId)
{
	METIN();

	if (rId == G2PCERA_DEFAULT_INSTANCE) {
		// default instance is always the first in the list
		return getPceInstance();
	}

	if (g2pceras_.empty()) {
		return 0;
	}

	std::list<PCERA::G2PCERA *>::iterator iter;

	for (iter = g2pceras_.begin(); iter != g2pceras_.end(); iter++) {
		if ((*iter)->rootNodeId() == rId) {

			METOUT();
			return (*iter);
		}
	}

	METOUT();

	return 0;
}


bool
G2PCERA_Master::attach(PCERA::G2PCERA * proto_instance)
{
	METIN();

	assert(proto_instance);

	std::list<PCERA::G2PCERA *>::iterator iter;

	for (iter = g2pceras_.begin(); iter != g2pceras_.end(); iter++) {
		if ((*iter) == proto_instance) {
			G2PCERA_ERROR("Proto instance already "
				      "attached to G2PCERA_Master");
			METOUT();
			return false;
		}
	}

	g2pceras_.push_back(proto_instance);

	G2PCERA_DEBUG("Proto instance attached to G2PCERA_Master");

	if ((g2pceras_.size() == 1) && (routerId_ == G2PCERA_DEFAULT_RID)) {
		// get the router ID from the first instance
		// if not previously retrieved from LRM
		routerId_ = proto_instance->rootNodeId();
	}

	METOUT();

	return true;
}

bool
G2PCERA_Master::detach(PCERA::G2PCERA * proto_instance)
{
	METIN();

	assert(proto_instance);

	std::list<PCERA::G2PCERA *>::iterator iter;

	for (iter = g2pceras_.begin(); iter != g2pceras_.end(); iter++) {
		if ((*iter) == proto_instance) {
			g2pceras_.erase(iter);
			G2PCERA_DEBUG("Proto instance detached "
				      "from G2PCERA_Master");
			METOUT();
			return true;
		}
	}

	G2PCERA_DEBUG("Proto instance not found in G2PCERA_Master");

	METOUT();

	return false;
}

#if HAVE_OMNIORB
LRM::Info_var &
G2PCERA_Master::getLrmProxy(void)
{
	METIN();

	METOUT();

	return lrmProxy_;
}
#endif // HAVE_OMNIORB

bool
G2PCERA_Master::dryrun(void)
{
	METIN();

	METOUT();

	return dryrun_;
}

void
G2PCERA_Master::dryrun(bool value)
{
	METIN();

	dryrun_ = value;

	METOUT();
}

uint32_t
G2PCERA_Master::routerId(void)
{
	METIN();

	METOUT();

	return routerId_;
}

void
G2PCERA_Master::routerId(uint32_t value)
{
	METIN();

	routerId_ = value;

	METOUT();
}
