//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//



#ifndef __G2PCERAD_HH__
#define __G2PCERAD_HH__

#include <list>

#include "g2pcera_config.hh"
#include "g2pcera_dm.hh"

#if HAVE_OMNIORB
#include "g2pcera_corba.hh"
#endif // HAVE_OMNIORB

// PCERA  master for system wide configuration
class G2PCERA_Master {
 public:
	static bool				init(void);
	static bool				destroy(void);
	static G2PCERA_Master &			instance(void);

	void					init_vty(void);
	struct thread_master *			getMaster();

	bool					dryrun(void);
	void					dryrun(bool value);
	uint32_t				routerId(void);
	void					routerId(uint32_t value);

	static std::list<PCERA::G2PCERA *>	getPceList(void);
	static PCERA::G2PCERA *			getPceInstance(void);
	static PCERA::G2PCERA *			getPceInstance(uint32_t	rId);
	static bool				attach(PCERA::G2PCERA *	pce);
	static bool				detach(PCERA::G2PCERA *	pce);

#if HAVE_OMNIORB
	LRM::Info_var &				getLrmProxy(void);
#endif // HAVE_OMNIORB

 protected:
	G2PCERA_Master& operator=(const G2PCERA_Master& j);
	G2PCERA_Master(const G2PCERA_Master & j);
	G2PCERA_Master(void);
	~G2PCERA_Master(void);

 private:
	static G2PCERA_Master *			instance_;
	static struct thread_master *		master_;    // thread master
	static std::list<PCERA::G2PCERA *>	g2pceras_;  // instances list
	static time_t				startTime_; // start time
	static bool				dryrun_;
	static uint32_t				routerId_;
#if HAVE_OMNIORB
	static LRM::Info_var			lrmProxy_;
#endif // HAVE_OMNIORB

};

#define G2PCERA_MASTER    (G2PCERA_Master::instance())


#endif // __G2PCERAD_HH__
