//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//



#include "g2rsvpted/g2rsvpte_cc_api.h"
#include "g2rsvpted/g2rsvpte_lsp.h"
#include "g2rsvpted/g2rsvpte_dm.h"
#include "g2rsvpted/g2rsvpted.h"
#include "g2rsvpted/g2rsvpte_utils.h"

#include "lib/corba.h"
#include "g2mpls_corba_utils.h"
#include "g2rsvpte_corba.h"

#include <stdexcept>


bool postCallCreate(RSVP::LSP * lsp)
{
	assert(lsp);
	assert(lsp->psb());
	assert(lsp->psb()->pathIn());

#ifdef G2RSVPTE2CC_DEBUG

	G2RSVPTE_DBG("Posting CallCreate fake event to CC ... debugging");

	return true;

#else
        if (lsp->isTail()) {
                try {
			bool                            res = false;
			g2mplsTypes::callIdent_var      callId;
			call_ident_t                    callIdTmp;
			g2mplsTypes::callParams_var     callInfo;
			call_info_t                     callInfoTmp;
			g2mplsTypes::recoveryParams_var recoveryInfo;
			recovery_info_t                 recoveryInfoTmp;
			g2mplsTypes::lspParams_var      lspInfo;
			lsp_info_t                      lspInfoTmp;
			g2mplsTypes::aaaParams_var	aaaInfo;
			std::string                     authz = "";
			g2mplsTypes::actorInfo_var      actor;

			G2RSVPTE_DBG("Posting CallCreate event to CC");

			//get call id
			callIdTmp = lsp->callId();
			callId   << callIdTmp;

			//get call info
			memset(&callInfoTmp, 0, sizeof(callInfoTmp));
			if (!lsp->getCallInfo(callInfoTmp)) {
				G2RSVPTE_ERR("Impossible to retrieve "
					     "call info parameters");
				return false;
			}
			callInfo << callInfoTmp;

			//get recovery info
			////TMP - FIXME
			memset(&recoveryInfoTmp, 0, sizeof(recoveryInfoTmp));
			recoveryInfo << recoveryInfoTmp;
			////

			//get lsp parameters
			memset(&lspInfoTmp, 0, sizeof(lspInfoTmp));
			if (!lsp->getInfo(lspInfoTmp)) {
				G2RSVPTE_ERR("Impossible to retrieve "
					     "LSP info parameters");
				return false;
			}
			lspInfo << lspInfoTmp;

			//get aaa info
			aaaInfo = new g2mplsTypes::aaaParams;
			if (lsp->psb()->pathIn()->policy_data_ && !lsp->getPolicy(authz)) {
				G2RSVPTE_ERR("Impossible to retrieve "
					     "Policy info parameters");
				return false;
			}
			aaaInfo->authzToken = authz.c_str();

			actor = new g2mplsTypes::actorInfo;
			actor->issuer = (lsp->getInstance()->adjType() == UNI) ?
				g2mplsTypes::ISSUERTYPE_UNI_IF :
				g2mplsTypes::ISSUERTYPE_ENNI_IF;
			actor->forceCommand = false;

			res = cc_mgmt_proxy->callCreate(callId,
							callInfo,
							recoveryInfo,
							lspInfo,
							aaaInfo,
							actor);
			//get updated callId
			callIdTmp << callId;
			//update callId in lsp
			if (!lsp->updateCallId(callIdTmp)) {
				G2RSVPTE_ERR("Impossible to update "
					     "callId in LSP");
				return false;
			}

		} catch (Types::InternalProblems) {
			G2RSVPTE_ERR("Posting CallCreate event to CC failed: "
				     "Internal Problems");
			return false;
		} catch (std::string e) {
			G2RSVPTE_ERR("Posting CallCreate event to CC failed: %s",
				     e.c_str());
			return false;
		} catch (std::out_of_range & e) {
                        G2RSVPTE_ERR("Posting CallCreate event to CC failed: %s",
				     e.what());
                        return false;
		} catch (CORBA::SystemException & e) {
			G2RSVPTE_ERR("Posting CallCreate event to CC failed: "
				     "CORBA::SystemException (%d)", (uint32_t) e.minor());
			return false;
		} catch (CORBA::Exception & e) {
			G2RSVPTE_ERR("Posting CallCreate event to CC failed: "
				     "CORBA::Exception");
			return false;
		} catch (omniORB::fatalException & e) {
			G2RSVPTE_ERR("Posting CallCreate event to CC failed: "
				     "omniORB::fatalException:");
			G2RSVPTE_ERR("  file: %s\n", e.file());
			G2RSVPTE_ERR("  line: %d\n", e.line());
			G2RSVPTE_ERR("  mesg: %s\n", e.errmsg());
			return false;
		} catch (...) {
			G2RSVPTE_ERR("Posting CallCreate event to CC failed: "
				     "UNKNOWN REASON");
			return false;
		}

		G2RSVPTE_DBG("CallCreate event posted");

	}

#endif // G2RSVPTE2CC_DEBUG

	return true;
}

bool postCallSetTna(RSVP::LSP * lsp)
{
	assert(lsp);

#ifdef G2RSVPTE2CC_DEBUG

	G2RSVPTE_DBG("Posting CallSetTna fake event to CC ... debugging");

	return true;

#else
        if (lsp->isTail()) {
                try {
			bool                            res = false;
			g2mplsTypes::callIdent_var      callId;
			call_ident_t                    callIdTmp;
			net_res_spec_t                  tnaResTmp;
			g2mplsTypes::tnaResource_var    tnaRes;
			g2mplsTypes::resourcePosition	pos;
			g2mplsTypes::actorInfo_var      actor;

			G2RSVPTE_DBG("Posting CallSetTna event to CC");

			//get call id
			callIdTmp = lsp->callId();
			callId   << callIdTmp;

			//get actor
			actor = new g2mplsTypes::actorInfo;
			actor->issuer = (lsp->getInstance()->adjType() == UNI) ?
				g2mplsTypes::ISSUERTYPE_UNI_IF :
				g2mplsTypes::ISSUERTYPE_ENNI_IF;
			actor->forceCommand = false;

			tnaResTmp = lsp->iRes();
			if (!isNetResSpecNull(tnaResTmp)) {
				tnaRes    << tnaResTmp;
				pos = g2mplsTypes::RESOURCEPOSITION_INGRESS;

				res = cc_mgmt_proxy->callSetTna(callId,
								pos,
								tnaRes,
								actor);
			}

			tnaResTmp = lsp->eRes();
			if (!isNetResSpecNull(tnaResTmp)) {
				tnaRes    << tnaResTmp;
				pos = g2mplsTypes::RESOURCEPOSITION_EGRESS;

				res = cc_mgmt_proxy->callSetTna(callId,
								pos,
								tnaRes,
								actor);
			}
		} catch (Types::InternalProblems) {
			G2RSVPTE_ERR("Posting CallSetTna event to CC failed: "
				     "Internal Problems");
			return false;
		} catch (Types::CannotFetch) {
			G2RSVPTE_ERR("Posting CallSetTna event to CC failed: "
				     "Cannot Fetch");
			return false;
		} catch (std::string e) {
			G2RSVPTE_ERR("Posting CallSetTna event to CC failed: %s",
				     e.c_str());
			return false;
		} catch (std::out_of_range & e) {
                        G2RSVPTE_ERR("Posting CallSetTna event to CC failed: %s",
				     e.what());
                        return false;
		} catch (CORBA::SystemException & e) {
			G2RSVPTE_ERR("Posting CallSetTna event to CC failed: "
				     "CORBA::SystemException (%d)", (uint32_t) e.minor());
			return false;
		} catch (CORBA::Exception & e) {
			G2RSVPTE_ERR("Posting CallSetTna event to CC failed: "
				     "CORBA::Exception");
			return false;
		} catch (omniORB::fatalException & e) {
			G2RSVPTE_ERR("Posting CallSetTna event to CC failed: "
				     "omniORB::fatalException:");
			G2RSVPTE_ERR("  file: %s\n", e.file());
			G2RSVPTE_ERR("  line: %d\n", e.line());
			G2RSVPTE_ERR("  mesg: %s\n", e.errmsg());
			return false;
		} catch (...) {
			G2RSVPTE_ERR("Posting CallSetTna event to CC failed: "
				     "UNKNOWN REASON");
			return false;
		}

		G2RSVPTE_DBG("CallSetTna event posted");
	}
#endif  //G2RSVPTE2CC_DEBUG

	return true;
}

bool postCallSetGnsTna(RSVP::LSP * lsp, bool ingress)
{
	assert(lsp);
	if (lsp->isTail()) {
		assert(lsp->psb());
		assert(lsp->psb()->pathIn());
	} else if (lsp->isHead()) {
		assert(lsp->rsb());
		assert(lsp->rsb()->resvIn());
	}

#ifdef G2RSVPTE2CC_DEBUG

	G2RSVPTE_DBG("Posting CallSetGnsTna fake event to CC ... debugging");

	return true;

#else
	try {
		bool                            res = false;
		g2mplsTypes::callIdent_var      callId;
		call_ident_t                    callIdTmp;
		grid_res_spec_t                 gnsTnaTmp;
		g2mplsTypes::gridParams_var     gnsTna;
		g2mplsTypes::resourcePosition	pos;
		g2mplsTypes::actorInfo_var      actor;

		G2RSVPTE_DBG("Posting CallSetGnsTna event to CC");

		//get call id
		if (lsp->isHead() &&
		    lsp->getInstance()->adjType() == UNI &&
		    lsp->getInstance()->isUniClient()) {
			callIdTmp = lsp->callId_old();
		} else {
			callIdTmp = lsp->callId();
		}
		callId   << callIdTmp;

		//get gnsTna
		memset(&gnsTnaTmp, 0, sizeof(gnsTnaTmp));
		if (lsp->isTail()) {
			if (!lsp->psb()->pathIn()->getGnsTna(gnsTnaTmp, ingress)) {
				G2RSVPTE_ERR("Impossible to retrieve GNS-TNA "
					     "parameters from incoming Path message");
				return false;
			}
		} else if (lsp->isHead()) {
			if (!lsp->rsb()->resvIn()->getGnsTna(gnsTnaTmp, ingress)) {
				G2RSVPTE_ERR("Impossible to retrieve GNS-TNA"
					     "parameters from incoming Resv message");
				return false;
			}
		}
		gnsTna << gnsTnaTmp;

		//get actor
		actor = new g2mplsTypes::actorInfo;
		actor->issuer = (lsp->getInstance()->adjType() == UNI) ?
			g2mplsTypes::ISSUERTYPE_UNI_IF :
			g2mplsTypes::ISSUERTYPE_ENNI_IF;
		actor->forceCommand = false;

		pos = (ingress ? g2mplsTypes::RESOURCEPOSITION_INGRESS :
		       g2mplsTypes::RESOURCEPOSITION_EGRESS);

		res = cc_mgmt_proxy->callSetGnsTna(callId,
						   pos,
						   gnsTna,
						   actor);

	} catch (Types::InternalProblems) {
		G2RSVPTE_ERR("Posting CallSetGnsTna event to CC failed: "
			     "Internal Problems");
		return false;
	} catch (Types::CannotFetch) {
		G2RSVPTE_ERR("Posting CallSetGnsTna event to CC failed: "
			     "Cannot Fetch");
		return false;
	} catch (std::string e) {
		G2RSVPTE_ERR("Posting CallSetGnsTna event to CC failed: %s",
			     e.c_str());
		return false;
	} catch (std::out_of_range & e) {
		G2RSVPTE_ERR("Posting CallSetGnsTna event to CC failed: %s",
			     e.what());
		return false;
	} catch (CORBA::SystemException & e) {
		G2RSVPTE_ERR("Posting CallSetGnsTna event to CC failed: "
			     "CORBA::SystemException (%d)", (uint32_t) e.minor());
		return false;
	} catch (CORBA::Exception & e) {
		G2RSVPTE_ERR("Posting CallSetGnsTna event to CC failed: "
			     "CORBA::Exception");
		return false;
	} catch (omniORB::fatalException & e) {
		G2RSVPTE_ERR("Posting CallSetGnsTna event to CC failed: "
			     "omniORB::fatalException:");
		G2RSVPTE_ERR("  file: %s\n", e.file());
		G2RSVPTE_ERR("  line: %d\n", e.line());
		G2RSVPTE_ERR("  mesg: %s\n", e.errmsg());
		return false;
	} catch (...) {
		G2RSVPTE_ERR("Posting CallSetGnsTna event to CC failed: "
			     "UNKNOWN REASON");
		return false;
	}

	G2RSVPTE_DBG("CallSetGnsTna event posted");
#endif  //G2RSVPTE2CC_DEBUG

	return true;
}

bool postCallAddEroPart(RSVP::LSP * lsp)
{
	assert(lsp);
	assert(lsp->psb());
	assert(lsp->psb()->pathOut());

#ifdef G2RSVPTE2CC_DEBUG

	G2RSVPTE_DBG("Posting CallAddEroPart fake event to CC ... debugging");

	return true;

#else
        if (lsp->isTail()) {
                try {
			bool                                res = false;
			g2mplsTypes::callIdent_var          callId;
			call_ident_t                        callIdTmp;
			std::list<lsp_ero_sobj_t>           ero;
			std::list<lsp_ero_sobj_t>::iterator it;
			g2mplsTypes::actorInfo_var          actor;
			size_t                              eroSize;

			G2RSVPTE_DBG("Posting CallAddEroPart event to CC");

			//get call id
			callIdTmp = lsp->callId();
			callId   << callIdTmp;

			//get ERO
			ero.clear();
			if (!lsp->psb()->pathOut()->getEro(ero)) {
				G2RSVPTE_ERR("Impossible to get ERO");
				return false;
			}
			eroSize = ero.size();
			// if ENNI (always verified) increment by one ERO list
			// size to add one extra element (required by NCC)
			if (lsp->getInstance()->adjType() == ENNI) {
				eroSize++;
			}
			g2mplsTypes::eroSeq_var eroItem;
			{
				g2mplsTypes::eroSeq * tmp;

				tmp = new g2mplsTypes::eroSeq(eroSize);
				if (!tmp) {
					G2RSVPTE_ERR("Impossible to allocate "
						     "a new eroSeq sequence");
					return false;
				}
				eroItem = tmp;
			}

			eroItem->length(eroSize);
			int i = 0;
			lsp_ero_sobj_t sObj;
			// if ENNI (always verified) add one element on top
			// of ERO list, containing this node id (required by NCC)
			if (lsp->getInstance()->adjType() == ENNI) {
				memset(&sObj, 0, sizeof(sObj));
				sObj.type = ERO_SUBOBJ_NODE;
				sObj.hop.node.node_id = lsp->getInstance()->nodeId();
				// set loose falg to false
				sObj.loose = ERO_SUBOBJ_STRICT;
				ero.push_front(sObj);
			}
			for (it = ero.begin(); it != ero.end(); it++) {
				memset(&sObj, 0, sizeof(sObj));
				sObj = *it;
				eroItem[i] = getEroItem(sObj);
				i++;
			}
			//get actor
			actor = new g2mplsTypes::actorInfo;
			actor->issuer = (lsp->getInstance()->adjType() == UNI) ?
				g2mplsTypes::ISSUERTYPE_UNI_IF :
				g2mplsTypes::ISSUERTYPE_ENNI_IF;
			actor->forceCommand = false;

			res = cc_mgmt_proxy->callAddEroPart(callId,
							    eroItem,
							    actor);

		} catch (Types::InternalProblems) {
			G2RSVPTE_ERR("Posting CallAddEroPart event to CC failed: "
				     "Internal Problems");
			return false;
		} catch (Types::CannotFetch) {
			G2RSVPTE_ERR("Posting CallAddEroPart event to CC failed: "
				     "Cannot Fetch");
			return false;
		} catch (std::string e) {
			G2RSVPTE_ERR("Posting CallAddEroPart event to CC failed: %s",
				     e.c_str());
			return false;
		} catch (std::out_of_range & e) {
                        G2RSVPTE_ERR("Posting CallAddEroPart event to CC failed: %s",
				     e.what());
                        return false;
		} catch (CORBA::SystemException & e) {
			G2RSVPTE_ERR("Posting CallAddEroPart event to CC failed: "
				     "CORBA::SystemException (%d)", (uint32_t) e.minor());
			return false;
		} catch (CORBA::Exception & e) {
			G2RSVPTE_ERR("Posting CallAddEroPart event to CC failed: "
				     "CORBA::Exception");
			return false;
		} catch (omniORB::fatalException & e) {
			G2RSVPTE_ERR("Posting CallAddEroPart event to CC failed: "
				     "omniORB::fatalException:");
			G2RSVPTE_ERR("  file: %s\n", e.file());
			G2RSVPTE_ERR("  line: %d\n", e.line());
			G2RSVPTE_ERR("  mesg: %s\n", e.errmsg());
			return false;
		} catch (...) {
			G2RSVPTE_ERR("Posting CallAddEroPart event to CC failed: "
				     "UNKNOWN REASON");
			return false;
		}

		G2RSVPTE_DBG("CallAddEroPart event posted");
	}
#endif  //G2RSVPTE2CC_DEBUG

	return true;
}

bool notifyLspRecvPathEvent(RSVP::LSP * lsp)
{
	assert(lsp);
	assert(lsp->getInstance());

#ifdef G2RSVPTE2CC_DEBUG

	G2RSVPTE_DBG("Posting PathEvent fake event to CC ... debugging");

	// LSP MUST call a sendResv automatically
	if (lsp->isTail()) {
		G2RSVPTE_DBG("Posting sendResv Event ... (AUTO-POST)");
		return lsp->sendResv(true);
	}

	G2RSVPTE_DBG("PathEvent ... without action");
#else
        if (lsp->isTail()) {
                try {
			bool                        res = false;
			g2mplsTypes::lspIdent_var   lspIdent;
			lsp_ident_t                 lspIdentTmp;
			g2mplsTypes::callIdent_var  callId;
			call_ident_t                callIdTmp;
			g2mplsTypes::callMessageType msgType;
			g2mplsTypes::actorInfo_var  actor;

			G2RSVPTE_DBG("Posting PathEvent to CC");

			//get lsp ident
			lspIdentTmp   = lsp->getIdent();
			lspIdent     << lspIdentTmp;

			//get call id
			callIdTmp = lsp->callId();
			callId   << callIdTmp;

			msgType = g2mplsTypes::CALLMESSAGETYPE_SETUPREQUEST;

			actor = new g2mplsTypes::actorInfo;
			actor->issuer = (lsp->getInstance()->adjType() == UNI) ?
				g2mplsTypes::ISSUERTYPE_UNI_IF :
				g2mplsTypes::ISSUERTYPE_ENNI_IF;
			actor->forceCommand = false;

			res = cc_ls_proxy->recvLspMessage(callId,
							  lspIdent,
							  msgType,
							  actor);
		} catch (Types::InternalProblems) {
			G2RSVPTE_ERR("Posting PathEvent to CC failed: "
				     "LSP already present in CC");
			return false;
		} catch (std::string e) {
			G2RSVPTE_ERR("Posting PathEvent to CC failed: %s",
				     e.c_str());
			return false;
		} catch (std::out_of_range & e) {
                        G2RSVPTE_ERR("Posting PathEvent to CC failed: %s",
				     e.what());
                        return false;
		} catch (CORBA::SystemException & e) {
			G2RSVPTE_ERR("Posting PathEvent to CC failed: "
				     "CORBA::SystemException (%d)", (uint32_t) e.minor());
			return false;
		} catch (CORBA::Exception & e) {
			G2RSVPTE_ERR("Posting PathEvent to CC failed: "
				     "CORBA::Exception");
			return false;
		} catch (omniORB::fatalException & e) {
			G2RSVPTE_ERR("Posting PathEvent to CC failed: "
				     "omniORB::fatalException:");
			G2RSVPTE_ERR("  file: %s\n", e.file());
			G2RSVPTE_ERR("  line: %d\n", e.line());
			G2RSVPTE_ERR("  mesg: %s\n", e.errmsg());
			return false;
		} catch (...) {
			G2RSVPTE_ERR("Posting PathEvent to CC failed: "
				     "UNKNOWN REASON");
			return false;
		}

		G2RSVPTE_DBG("PathEvent posted");
	}
#endif // G2RSVPTE2CC_DEBUG

	return true;
}

bool notifyLspRecvResvEvent(RSVP::LSP * lsp)
{
	assert(lsp);
	assert(lsp->getInstance());

#ifdef G2RSVPTE2CC_DEBUG

	G2RSVPTE_DBG("Posting ResvEvent fake event to CC ... debugging");

	// LSP MUST call a sendConfirm automatically
	if (lsp->isTail()) {
		G2RSVPTE_DBG("Posting ResvEvent Event ... (AUTO-POST)");
		return lsp->sendConfirm(true);
	}

	G2RSVPTE_DBG("ResvEvent ... without action");
#else
        if (lsp->isHead()) {
                try {
			bool                        res = false;
			g2mplsTypes::lspIdent_var   lspIdent;
			lsp_ident_t                 lspIdentTmp;
			g2mplsTypes::callIdent_var  callId;
			call_ident_t                callIdTmp;
			g2mplsTypes::callMessageType msgType;
			g2mplsTypes::actorInfo_var  actor;

			G2RSVPTE_DBG("Posting ResvEvent to CC");

			//get lsp ident
			lspIdentTmp   = lsp->getIdent();
			lspIdent     << lspIdentTmp;

			//get call id
			callIdTmp = lsp->callId();
			callId   << callIdTmp;

			msgType = g2mplsTypes::CALLMESSAGETYPE_SETUPINDICATION;

			actor = new g2mplsTypes::actorInfo;
			actor->issuer = (lsp->getInstance()->adjType() == UNI) ?
				g2mplsTypes::ISSUERTYPE_UNI_IF :
				g2mplsTypes::ISSUERTYPE_ENNI_IF;
			actor->forceCommand = false;

			res = cc_ls_proxy->recvLspMessage(callId,
							  lspIdent,
							  msgType,
							  actor);
		} catch (Types::InternalProblems) {
			G2RSVPTE_ERR("Posting ResvEvent to CC failed: "
				     "Internal Problems");
			return false;
		} catch (std::string e) {
			G2RSVPTE_ERR("Posting ResvEvent to CC failed: %s",
				     e.c_str());
			return false;
		} catch (std::out_of_range & e) {
                        G2RSVPTE_ERR("Posting ResvEvent to CC failed: %s",
				     e.what());
                        return false;
		} catch (CORBA::SystemException & e) {
			G2RSVPTE_ERR("Posting ResvEvent to CC failed: "
				     "CORBA::SystemException (%d)", (uint32_t) e.minor());
			return false;
		} catch (CORBA::Exception & e) {
			G2RSVPTE_ERR("Posting ResvEvent to CC failed: "
				     "CORBA::Exception");
			return false;
		} catch (omniORB::fatalException & e) {
			G2RSVPTE_ERR("Posting ResvEvent to CC failed: "
				     "omniORB::fatalException:");
			G2RSVPTE_ERR("  file: %s\n", e.file());
			G2RSVPTE_ERR("  line: %d\n", e.line());
			G2RSVPTE_ERR("  mesg: %s\n", e.errmsg());
			return false;
		} catch (...) {
			G2RSVPTE_ERR("Posting ResvEvent to CC failed: "
				     "UNKNOWN REASON");
			return false;
		}

		G2RSVPTE_DBG("ResvEvent posted");
	}
#endif // G2RSVPTE2CC_DEBUG

	return true;
}

bool notifyLspRecvResvConfEvent(RSVP::LSP * lsp)
{
	assert(lsp);
	assert(lsp->getInstance());

#ifdef G2RSVPTE2CC_DEBUG

	G2RSVPTE_DBG("Notifying fake RecvResvConfEvent to CC ... debugging !");
#else
        if (lsp->isTail()) {
                try {
			bool                        res = false;
			g2mplsTypes::lspIdent_var   lspIdent;
			lsp_ident_t                 lspIdentTmp;
			g2mplsTypes::callIdent_var  callId;
			call_ident_t                callIdTmp;
			g2mplsTypes::callMessageType msgType;
			g2mplsTypes::actorInfo_var  actor;

			G2RSVPTE_DBG("Notifying RecvResvConfEvent to CC");

			//get lsp ident
			lspIdentTmp   = lsp->getIdent();
			lspIdent     << lspIdentTmp;

			//get call id
			callIdTmp = lsp->callId();
			callId   << callIdTmp;

			msgType = g2mplsTypes::CALLMESSAGETYPE_SETUPCONFIRM;

			actor = new g2mplsTypes::actorInfo;
			actor->issuer = (lsp->getInstance()->adjType() == UNI) ?
				g2mplsTypes::ISSUERTYPE_UNI_IF :
				g2mplsTypes::ISSUERTYPE_ENNI_IF;
			actor->forceCommand = false;

			res = cc_ls_proxy->recvLspMessage(callId,
							  lspIdent,
							  msgType,
							  actor);
		} catch (Types::InternalProblems) {
			G2RSVPTE_ERR("Posting RecvResvConfEvent to CC failed: "
				     "Internal Problems");
			return false;
		} catch (std::string e) {
			G2RSVPTE_ERR("Posting RecvResvConfEvent to CC failed: %s",
				     e.c_str());
			return false;
		} catch (std::out_of_range & e) {
                        G2RSVPTE_ERR("Posting RecvResvConfEvent to CC failed: %s",
				     e.what());
                        return false;
		} catch (CORBA::SystemException & e) {
			G2RSVPTE_ERR("Posting RecvResvConfEvent to CC failed: "
				     "CORBA::SystemException (%d)", (uint32_t) e.minor());
			return false;
		} catch (CORBA::Exception & e) {
			G2RSVPTE_ERR("Posting RecvResvConfEvent to CC failed: "
				     "CORBA::Exception");
			return false;
		} catch (omniORB::fatalException & e) {
			G2RSVPTE_ERR("Posting RecvResvConfEvent to CC failed: "
				     "omniORB::fatalException:");
			G2RSVPTE_ERR("  file: %s\n", e.file());
			G2RSVPTE_ERR("  line: %d\n", e.line());
			G2RSVPTE_ERR("  mesg: %s\n", e.errmsg());
			return false;
		} catch (...) {
			G2RSVPTE_ERR("Posting RecvResvConfEvent to CC failed: "
				     "UNKNOWN REASON");
			return false;
		}

		G2RSVPTE_DBG("RecvResvConfEvent notified");
	}
#endif // G2RSVPTE2CC_DEBUG

	return true;
}

bool notifyLspRecvPathDownEvent(RSVP::LSP * lsp)
{
	assert(lsp);
	assert(lsp->getInstance());

#ifdef G2RSVPTE2CC_DEBUG

	G2RSVPTE_DBG("Notifying fake RecvPathDownEvent to CC ... debugging !");
#else
        if (lsp->isTail()) {
                try {
			bool                        res = false;
			g2mplsTypes::lspIdent_var   lspIdent;
			lsp_ident_t                 lspIdentTmp;
			g2mplsTypes::callIdent_var  callId;
			call_ident_t                callIdTmp;
			g2mplsTypes::callMessageType msgType;
			g2mplsTypes::actorInfo_var  actor;

			G2RSVPTE_DBG("Notifying RecvPathDownEvent to CC");

			//get lsp ident
			lspIdentTmp   = lsp->getIdent();
			lspIdent     << lspIdentTmp;

			//get call id
			callIdTmp = lsp->callId();
			callId   << callIdTmp;

			msgType = g2mplsTypes::CALLMESSAGETYPE_RELEASEREQUEST;

			actor = new g2mplsTypes::actorInfo;
			actor->issuer = (lsp->getInstance()->adjType() == UNI) ?
				g2mplsTypes::ISSUERTYPE_UNI_IF :
				g2mplsTypes::ISSUERTYPE_ENNI_IF;
			actor->forceCommand = false;

			res = cc_ls_proxy->recvLspMessage(callId,
							  lspIdent,
							  msgType,
							  actor);
		} catch (Types::InternalProblems) {
			G2RSVPTE_ERR("Posting RecvPathDownEvent to CC failed: "
				     "Internal Problems");
			return false;
		} catch (std::string e) {
			G2RSVPTE_ERR("Posting RecvPathDownEvent to CC failed: %s",
				     e.c_str());
			return false;
		} catch (std::out_of_range & e) {
                        G2RSVPTE_ERR("Posting RecvPathDownEvent to CC failed: %s",
				     e.what());
                        return false;
		} catch (CORBA::SystemException & e) {
			G2RSVPTE_ERR("Posting RecvPathDownEvent to CC failed: "
				     "CORBA::SystemException (%d)", (uint32_t) e.minor());
			return false;
		} catch (CORBA::Exception & e) {
			G2RSVPTE_ERR("Posting RecvPathDownEvent to CC failed: "
				     "CORBA::Exception");
			return false;
		} catch (omniORB::fatalException & e) {
			G2RSVPTE_ERR("Posting RecvPathDownEvent to CC failed: "
				     "omniORB::fatalException:");
			G2RSVPTE_ERR("  file: %s\n", e.file());
			G2RSVPTE_ERR("  line: %d\n", e.line());
			G2RSVPTE_ERR("  mesg: %s\n", e.errmsg());
			return false;
		} catch (...) {
			G2RSVPTE_ERR("Posting RecvPathDownEvent to CC failed: "
				     "UNKNOWN REASON");
			return false;
		}

		G2RSVPTE_DBG("RecvPathDownEvent notified");
	}
#endif // G2RSVPTE2CC_DEBUG

	return true;
}

bool notifyLspRecvResvDownEvent(RSVP::LSP * lsp)
{
	assert(lsp);
	assert(lsp->getInstance());

#ifdef G2RSVPTE2CC_DEBUG

	G2RSVPTE_DBG("Notifying fake RecvResvDownEvent to CC ... debugging !");
#else
        if (lsp->isHead()) {
                try {
			bool                        res = false;
			g2mplsTypes::lspIdent_var   lspIdent;
			lsp_ident_t                 lspIdentTmp;
			g2mplsTypes::callIdent_var  callId;
			call_ident_t                callIdTmp;
			g2mplsTypes::callMessageType msgType;
			g2mplsTypes::actorInfo_var  actor;

			G2RSVPTE_DBG("Notifying RecvResvDownEvent to CC");

			//get lsp ident
			lspIdentTmp   = lsp->getIdent();
			lspIdent     << lspIdentTmp;

			//get call id
			callIdTmp = lsp->callId();
			callId   << callIdTmp;

			msgType = g2mplsTypes::CALLMESSAGETYPE_RELEASEREQUEST;

			actor = new g2mplsTypes::actorInfo;
			actor->issuer = (lsp->getInstance()->adjType() == UNI) ?
				g2mplsTypes::ISSUERTYPE_UNI_IF :
				g2mplsTypes::ISSUERTYPE_ENNI_IF;
			actor->forceCommand = false;

			res = cc_ls_proxy->recvLspMessage(callId,
							  lspIdent,
							  msgType,
							  actor);
		} catch (Types::InternalProblems) {
			G2RSVPTE_ERR("Posting RecvResvDownEvent to CC failed: "
				     "Internal Problems");
			return false;
		} catch (std::string e) {
			G2RSVPTE_ERR("Posting RecvResvDownEvent to CC failed: %s",
				     e.c_str());
			return false;
		} catch (std::out_of_range & e) {
                        G2RSVPTE_ERR("Posting RecvResvDownEvent to CC failed: %s",
				     e.what());
                        return false;
		} catch (CORBA::SystemException & e) {
			G2RSVPTE_ERR("Posting RecvResvDownEvent to CC failed: "
				     "CORBA::SystemException (%d)", (uint32_t) e.minor());
			return false;
		} catch (CORBA::Exception & e) {
			G2RSVPTE_ERR("Posting RecvResvDownEvent to CC failed: "
				     "CORBA::Exception");
			return false;
		} catch (omniORB::fatalException & e) {
			G2RSVPTE_ERR("Posting RecvResvDownEvent to CC failed: "
				     "omniORB::fatalException:");
			G2RSVPTE_ERR("  file: %s\n", e.file());
			G2RSVPTE_ERR("  line: %d\n", e.line());
			G2RSVPTE_ERR("  mesg: %s\n", e.errmsg());
			return false;
		} catch (...) {
			G2RSVPTE_ERR("Posting RecvResvDownEvent to CC failed: "
				     "UNKNOWN REASON");
			return false;
		}

		G2RSVPTE_DBG("RecvResvDownEvent notified");
	}
#endif // G2RSVPTE2CC_DEBUG

	return true;
}

bool notifyLspRecvPathErrEvent(RSVP::LSP * lsp)
{
	assert(lsp);
	assert(lsp->getInstance());

#ifdef G2RSVPTE2CC_DEBUG

	G2RSVPTE_DBG("Notifying fake RecvPathErrEvent to CC ... debugging !");
#else
        if (lsp->isHead()) {
                try {
			bool                        res = false;
			g2mplsTypes::lspIdent_var   lspIdent;
			lsp_ident_t                 lspIdentTmp;
			g2mplsTypes::callIdent_var  callId;
			call_ident_t                callIdTmp;
			g2mplsTypes::actorInfo_var  actor;
			error_info_t                errorInfo;
			std::list<lsp_ero_sobj_t>   excludeEro;

			G2RSVPTE_DBG("Notifying RecvPathErrEvent to CC");

			//get lsp ident
			lspIdentTmp   = lsp->getIdent();
			lspIdent     << lspIdentTmp;

			//get call id
			callIdTmp = lsp->callId();
			callId   << callIdTmp;

			//get Err info
			if (!lsp->getErrInfo(errorInfo,
					     excludeEro,
					     true)) {
				G2RSVPTE_ERR("Posting RecvPathErrEvent to CC failed: "
					     "Impossible to get error info");
				return false;
			}

			actor = new g2mplsTypes::actorInfo;
			actor->issuer = (lsp->getInstance()->adjType() == UNI) ?
				g2mplsTypes::ISSUERTYPE_UNI_IF :
				g2mplsTypes::ISSUERTYPE_ENNI_IF;
			actor->forceCommand = false;

			if (errorInfo.err_code == G2RSVPTE_ERROR_SPEC_ZERO &&
			    errorInfo.err_val  == G2RSVPTE_ERROR_SPEC_VAL_ZERO) {
				g2mplsTypes::callMessageType msgType;

				msgType = g2mplsTypes::CALLMESSAGETYPE_RELEASEINDICATION;

				res = cc_ls_proxy->recvLspMessage(callId,
								  lspIdent,
								  msgType,
								  actor);
			} else {
				g2mplsTypes::errorInfo_var error;

				error << errorInfo;

				res = cc_ls_proxy->recvLspError(callId,
								lspIdent,
								error,
								actor);
			}

		} catch (Types::InternalProblems) {
			G2RSVPTE_ERR("Posting RecvPathErrEvent to CC failed: "
				     "Internal Problems");
			return false;
		} catch (std::string e) {
			G2RSVPTE_ERR("Posting RecvPathErrEvent to CC failed: %s",
				     e.c_str());
			return false;
		} catch (std::out_of_range & e) {
                        G2RSVPTE_ERR("Posting RecvPathErrEvent to CC failed: %s",
				     e.what());
                        return false;
		} catch (CORBA::SystemException & e) {
			G2RSVPTE_ERR("Posting RecvPathErrEvent to CC failed: "
				     "CORBA::SystemException (%d)", (uint32_t) e.minor());
			return false;
		} catch (CORBA::Exception & e) {
			G2RSVPTE_ERR("Posting RecvPathErrEvent to CC failed: "
				     "CORBA::Exception");
			return false;
		} catch (omniORB::fatalException & e) {
			G2RSVPTE_ERR("Posting RecvPathErrEvent to CC failed: "
				     "omniORB::fatalException:");
			G2RSVPTE_ERR("  file: %s\n", e.file());
			G2RSVPTE_ERR("  line: %d\n", e.line());
			G2RSVPTE_ERR("  mesg: %s\n", e.errmsg());
			return false;
		} catch (...) {
			G2RSVPTE_ERR("Posting RecvPathErrEvent to CC failed: "
				     "UNKNOWN REASON");
			return false;
		}

		G2RSVPTE_DBG("RecvPathErrEvent notified");
	}
#endif // G2RSVPTE2CC_DEBUG

	return true;
}

bool notifyLspRecvResvErrEvent(RSVP::LSP * lsp)
{
	assert(lsp);
	assert(lsp->getInstance());

#ifdef G2RSVPTE2CC_DEBUG

	G2RSVPTE_DBG("Notifying fake RecvResvErrEvent to CC ... debugging !");
#else
        if (lsp->isTail()) {
                try {
			bool                        res = false;
			g2mplsTypes::lspIdent_var   lspIdent;
			lsp_ident_t                 lspIdentTmp;
			g2mplsTypes::callIdent_var  callId;
			call_ident_t                callIdTmp;
			g2mplsTypes::actorInfo_var  actor;
			error_info_t                errorInfo;
			std::list<lsp_ero_sobj_t>   excludeEro;

			G2RSVPTE_DBG("Notifying RecvResvErrEvent to CC");

			//get lsp ident
			lspIdentTmp   = lsp->getIdent();
			lspIdent     << lspIdentTmp;

			//get call id
			callIdTmp = lsp->callId();
			callId   << callIdTmp;

			//get Err info
			if (!lsp->getErrInfo(errorInfo,
					     excludeEro,
					     false)) {
				G2RSVPTE_ERR("Posting RecvResvErrEvent to CC failed: "
					     "Impossible to get error info");
				return false;
			}

			actor = new g2mplsTypes::actorInfo;
			actor->issuer = (lsp->getInstance()->adjType() == UNI) ?
				g2mplsTypes::ISSUERTYPE_UNI_IF :
				g2mplsTypes::ISSUERTYPE_ENNI_IF;
			actor->forceCommand = false;

			g2mplsTypes::errorInfo_var error;

			error << errorInfo;

			res = cc_ls_proxy->recvLspError(callId,
							lspIdent,
							error,
							actor);

		} catch (Types::InternalProblems) {
			G2RSVPTE_ERR("Posting RecvResvErrEvent to CC failed: "
				     "Internal Problems");
			return false;
		} catch (std::string e) {
			G2RSVPTE_ERR("Posting RecvResvErrEvent to CC failed: %s",
				     e.c_str());
			return false;
		} catch (std::out_of_range & e) {
                        G2RSVPTE_ERR("Posting RecvResvErrEvent to CC failed: %s",
				     e.what());
                        return false;
		} catch (CORBA::SystemException & e) {
			G2RSVPTE_ERR("Posting RecvResvErrEvent to CC failed: "
				     "CORBA::SystemException (%d)", (uint32_t) e.minor());
			return false;
		} catch (CORBA::Exception & e) {
			G2RSVPTE_ERR("Posting RecvResvErrEvent to CC failed: "
				     "CORBA::Exception");
			return false;
		} catch (omniORB::fatalException & e) {
			G2RSVPTE_ERR("Posting RecvResvErrEvent to CC failed: "
				     "omniORB::fatalException:");
			G2RSVPTE_ERR("  file: %s\n", e.file());
			G2RSVPTE_ERR("  line: %d\n", e.line());
			G2RSVPTE_ERR("  mesg: %s\n", e.errmsg());
			return false;
		} catch (...) {
			G2RSVPTE_ERR("Posting RecvResvErrEvent to CC failed: "
				     "UNKNOWN REASON");
			return false;
		}

		G2RSVPTE_DBG("RecvResvErrEvent notified");
	}
#endif // G2RSVPTE2CC_DEBUG

	return true;
}

bool notifyLspRecvPathTearEvent(RSVP::LSP * lsp)
{
	assert(lsp);
	assert(lsp->getInstance());

#ifdef G2RSVPTE2CC_DEBUG

	G2RSVPTE_DBG("Notifying fake RecvPathTearEvent to CC ... debugging !");
#else
        if (lsp->isTail()) {
                try {
			bool                        res = false;
			g2mplsTypes::lspIdent_var   lspIdent;
			lsp_ident_t                 lspIdentTmp;
			g2mplsTypes::callIdent_var  callId;
			call_ident_t                callIdTmp;
			g2mplsTypes::callMessageType msgType;
			g2mplsTypes::actorInfo_var  actor;

			G2RSVPTE_DBG("Notifying RecvPathTearEvent to CC");

			//get lsp ident
			lspIdentTmp   = lsp->getIdent();
			lspIdent     << lspIdentTmp;

			//get call id
			callIdTmp = lsp->callId();
			callId   << callIdTmp;

			msgType = g2mplsTypes::CALLMESSAGETYPE_RELEASEINDICATION;

			actor = new g2mplsTypes::actorInfo;
			actor->issuer = (lsp->getInstance()->adjType() == UNI) ?
				g2mplsTypes::ISSUERTYPE_UNI_IF :
				g2mplsTypes::ISSUERTYPE_ENNI_IF;
			actor->forceCommand = false;

			res = cc_ls_proxy->recvLspMessage(callId,
							  lspIdent,
							  msgType,
							  actor);
		} catch (Types::InternalProblems) {
			G2RSVPTE_ERR("Posting RecvPathTearEvent to CC failed: "
				     "Internal Problems");
			return false;
		} catch (std::string e) {
			G2RSVPTE_ERR("Posting RecvPathTearEvent to CC failed: %s",
				     e.c_str());
			return false;
		} catch (std::out_of_range & e) {
                        G2RSVPTE_ERR("Posting RecvPathTearEvent to CC failed: %s",
				     e.what());
                        return false;
		} catch (CORBA::SystemException & e) {
			G2RSVPTE_ERR("Posting RecvPathTearEvent to CC failed: "
				     "CORBA::SystemException (%d)", (uint32_t) e.minor());
			return false;
		} catch (CORBA::Exception & e) {
			G2RSVPTE_ERR("Posting RecvPathTearEvent to CC failed: "
				     "CORBA::Exception");
			return false;
		} catch (omniORB::fatalException & e) {
			G2RSVPTE_ERR("Posting RecvPathTearEvent to CC failed: "
				     "omniORB::fatalException:");
			G2RSVPTE_ERR("  file: %s\n", e.file());
			G2RSVPTE_ERR("  line: %d\n", e.line());
			G2RSVPTE_ERR("  mesg: %s\n", e.errmsg());
			return false;
		} catch (...) {
			G2RSVPTE_ERR("Posting RecvPathTearEvent to CC failed: "
				     "UNKNOWN REASON");
			return false;
		}

		G2RSVPTE_DBG("RecvPathTearEvent notified");
	}
#endif // G2RSVPTE2CC_DEBUG

	return true;
}

bool notifyLspRecvResvTearEvent(RSVP::LSP * lsp)
{
	assert(lsp);
	assert(lsp->getInstance());

#ifdef G2RSVPTE2CC_DEBUG

	G2RSVPTE_DBG("Notifying fake RecvResvTearEvent to CC ... debugging !");
#else
        if (lsp->isHead()) {
                try {
			bool                        res = false;
			g2mplsTypes::lspIdent_var   lspIdent;
			lsp_ident_t                 lspIdentTmp;
			g2mplsTypes::callIdent_var  callId;
			call_ident_t                callIdTmp;
			g2mplsTypes::callMessageType msgType;
			g2mplsTypes::actorInfo_var  actor;

			G2RSVPTE_DBG("Notifying RecvResvTearEvent to CC");

			//get lsp ident
			lspIdentTmp   = lsp->getIdent();
			lspIdent     << lspIdentTmp;

			//get call id
			callIdTmp = lsp->callId();
			callId   << callIdTmp;

			msgType = g2mplsTypes::CALLMESSAGETYPE_RELEASEINDICATION;

			actor = new g2mplsTypes::actorInfo;
			actor->issuer = (lsp->getInstance()->adjType() == UNI) ?
				g2mplsTypes::ISSUERTYPE_UNI_IF :
				g2mplsTypes::ISSUERTYPE_ENNI_IF;
			actor->forceCommand = false;

			res = cc_ls_proxy->recvLspMessage(callId,
							  lspIdent,
							  msgType,
							  actor);
		} catch (Types::InternalProblems) {
			G2RSVPTE_ERR("Posting RecvResvTearEvent to CC failed: "
				     "Internal Problems");
			return false;
		} catch (std::string e) {
			G2RSVPTE_ERR("Posting RecvResvTearEvent to CC failed: %s",
				     e.c_str());
			return false;
		} catch (std::out_of_range & e) {
                        G2RSVPTE_ERR("Posting RecvResvTearEvent to CC failed: %s",
				     e.what());
                        return false;
		} catch (CORBA::SystemException & e) {
			G2RSVPTE_ERR("Posting RecvResvTearEvent to CC failed: "
				     "CORBA::SystemException (%d)", (uint32_t) e.minor());
			return false;
		} catch (CORBA::Exception & e) {
			G2RSVPTE_ERR("Posting RecvResvTearEvent to CC failed: "
				     "CORBA::Exception");
			return false;
		} catch (omniORB::fatalException & e) {
			G2RSVPTE_ERR("Posting RecvResvTearEvent to CC failed: "
				     "omniORB::fatalException:");
			G2RSVPTE_ERR("  file: %s\n", e.file());
			G2RSVPTE_ERR("  line: %d\n", e.line());
			G2RSVPTE_ERR("  mesg: %s\n", e.errmsg());
			return false;
		} catch (...) {
			G2RSVPTE_ERR("Posting RecvResvTearEvent to CC failed: "
				     "UNKNOWN REASON");
			return false;
		}

		G2RSVPTE_DBG("RecvResvTearEvent notified");
	}
#endif // G2RSVPTE2CC_DEBUG

	return true;
}
