//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//



#include "g2rsvpted/g2rsvpte_g2pcera_api.h"
#include "g2rsvpted/g2rsvpte_lsp.h"
#include "g2rsvpted/g2rsvpte_dm.h"
#include "g2rsvpted/g2rsvpted.h"
#include "g2rsvpted/g2rsvpte_config.h"

#include "lib/corba.h"
#include "g2mpls_corba_utils.h"
#include "g2rsvpte_corba.h"

#include <stdexcept>

bool connectionRouteDisjoint(RSVP::LSP *                 lsp,
			     std::list<lsp_ero_sobj_t> & ero)
{
	assert(lsp);
	assert(lsp->getInstance());

#ifdef CRANKBACK_ON
	G2RSVPTE_DBG("Requesting connectionRouteDisjoint to G2PCERA");

	try {
		bool                          res = false;
		lsp_ident_t                   lspIdent;
		g2mplsTypes::callIdent_var    callId;
		call_ident_t                  callIdTmp;
		g2mplsTypes::lspParams_var    lspInfo;
		lsp_info_t                    lspInfoTmp;
		g2mpls_addr_t                 nullAddr;
		g2mplsTypes::linkId_var       nullLinkId;
		g2mplsTypes::connEndPoint_var src;
		g2mplsTypes::connEndPoint_var dst;
		g2mplsTypes::eroSeq_var       xro;
		std::list<lsp_ero_sobj_t>     xroTmp;
		g2mplsTypes::eroSeq_var       wEro;
		g2mplsTypes::eroSeq_var       pEro;
		g2mplsTypes::eroItem_var      eroItTmp;

		//get LSP ident
		lspIdent = lsp->getIdent();

		nullAddr.type = IPv4;
		nullAddr.preflen = 32;
		nullAddr.value.ipv4.s_addr = 0;
		nullLinkId << nullAddr;
		//get src connection end point (LSP src nodeId)
		src = new g2mplsTypes::connEndPoint;
		src->node = lsp->getInstance()->nodeId();
		src->teLink   = nullLinkId;
		src->dataLink = nullLinkId;
		src->label.label32(0);

		//get dst connection end point (LSP dst nodeId)
		dst = new g2mplsTypes::connEndPoint;
		dst->node = lspIdent.dst_nid;
		dst->teLink   = nullLinkId;
		dst->dataLink = nullLinkId;
		dst->label.label32(0);

		//get call id
		callIdTmp = lsp->callId();
		callId   << callIdTmp;

		//get lsp parameters
		memset(&lspInfoTmp, 0, sizeof(lspInfoTmp));
		if (!lsp->getInfo(lspInfoTmp)) {
			G2RSVPTE_ERR("Impossible to get "
				     "LSP info parameters");
			return false;
		}
		lspInfo << lspInfoTmp;

		//get exclude Ero
		xroTmp = lsp->getExcludeEro();
		xro = getExclEroSeq(xroTmp);
		//build multiXro (for now only 1 element)
		g2mplsTypes::multiEroSeq_var  multiXro;
		{
			g2mplsTypes::multiEroSeq * tmp;

			tmp = new g2mplsTypes::multiEroSeq(1);
			if (!tmp) {
				G2RSVPTE_ERR("Impossible to allocate "
					     "a new multiEroSeq sequence");
				return false;
			}
			multiXro = tmp;
		}
		multiXro->length(1);
		multiXro[0] = xro;

		//call G2PCERA method
		res = g2pcera_proxy->connectionRouteDisjoint(src,
							     dst,
							     callId,
							     lspInfo,
							     multiXro,
							     wEro,
							     pEro);
		if (!res) {
			G2RSVPTE_ERR("Impossible to complete "
				     "connectionRouteDisjoint");
			return false;
		}

		// return back ERO
		ero.clear();
		for (size_t i = 0; i < wEro->length(); i++) {
			eroItTmp   = wEro[i];
			eroItemProcess(eroItTmp, ero);
		}

	} catch (G2PCERA::InternalProblems & e) {
		std::string err;
		err = std::string(e.what);
		G2RSVPTE_ERR("Requesting connectionRouteDisjoint to G2PCERA failed: "
			     "Internal Problems (%s)", err.c_str());
		return false;
	} catch (G2PCERA::NoRoute & e) {
		G2RSVPTE_ERR("Requesting connectionRouteDisjoint to G2PCERA failed: "
			     "No Route");
		return false;
	} catch (G2PCERA::CannotFetchConnEndPoint & e) {
		G2RSVPTE_ERR("Requesting connectionRouteDisjoint to G2PCERA failed: "
			     "Cannot Fetch Connection End Point");
		return false;
	} catch (G2PCERA::ConnectionParamsMismatch & e) {
		G2RSVPTE_ERR("Requesting connectionRouteDisjoint to G2PCERA failed: "
			     "Connection Params Mismatch");
		return false;
	} catch (G2PCERA::ConnectionEroMismatch & e) {
		G2RSVPTE_ERR("Requesting connectionRouteDisjoint to G2PCERA failed: "
			     "Connection Ero Mismatch");
		return false;
	} catch (std::string e) {
		G2RSVPTE_ERR("Requesting connectionRouteDisjoint to G2PCERA failed: %s",
			     e.c_str());
		return false;
	} catch (std::out_of_range & e) {
		G2RSVPTE_ERR("Requesting connectionRouteDisjoint to G2PCERA failed: %s",
			     e.what());
		return false;
	} catch (CORBA::SystemException & e) {
		G2RSVPTE_ERR("Requesting connectionRouteDisjoint to G2PCERA failed: "
			     "CORBA::SystemException (0x%x)", (uint32_t) e.minor());
		return false;
	} catch (CORBA::Exception & e) {
		G2RSVPTE_ERR("Requesting connectionRouteDisjoint to G2PCERA failed: "
			     "CORBA::Exception");
		return false;
	} catch (omniORB::fatalException & e) {
		G2RSVPTE_ERR("Requesting connectionRouteDisjoint to G2PCERA failed: "
			     "omniORB::fatalException:");
		G2RSVPTE_ERR("  file: %s\n", e.file());
		G2RSVPTE_ERR("  line: %d\n", e.line());
		G2RSVPTE_ERR("  mesg: %s\n", e.errmsg());
		return false;
	} catch (...) {
		G2RSVPTE_ERR("Requesting connectionRouteDisjoint to G2PCERA failed: "
			     "UNKNOWN REASON");
		return false;
	}

	G2RSVPTE_DBG("connectionRouteDisjoint successfully completed");
	return true;
#else
	G2RSVPTE_DBG("Crankback is disabled: cannot request connectionRouteDisjoint");
	return false;
#endif // CRANKBACK_ON
}
