/*
 *  This file is part of phosphorus-g2mpls.
 *
 *  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Authors:
 *
 *  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
 *  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
 *  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
 *  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
 *  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
 */



#ifndef G2RSVPTE_MESSAGE_UTILS_H
#define G2RSVPTE_MESSAGE_UTILS_H

#include "g2rsvpted/g2rsvpte_config.h"

//
// Utils
//

// Utils for (hdr) TLV
#define TLV_OBJ2STREAM(STREAM,OBJ)					      \
{									      \
	switch (OBJ.type()) {						      \
		case IF_ID_TLV_IPV4: {					      \
			STREAM << static_cast<const IPv4_TLV &>(OBJ);	      \
			break;						      \
		}							      \
		case IF_ID_TLV_IPV6: {					      \
			STREAM << static_cast<const IPv6_TLV &>(OBJ);	      \
			break;						      \
		}							      \
		case IF_ID_TLV_IF_INDEX: {				      \
			STREAM << static_cast<const IF_IDX_TLV &>(OBJ);	      \
			break;						      \
		}							      \
		case IF_ID_TLV_COMP_IF_DOWN: {				      \
			STREAM << static_cast<const COMP_IF_DOWN_TLV &>(OBJ); \
			break;						      \
		}							      \
		case IF_ID_TLV_COMP_IF_UP: {				      \
			STREAM << static_cast<const COMP_IF_UP_TLV &>(OBJ);   \
			break;						      \
		}                                                             \
		case IF_ID_TLV_DOWN_LABEL: {				      \
			STREAM << static_cast<const DOWN_LABEL_TLV &>(OBJ);   \
			break;						      \
		}                                                             \
	        case IF_ID_TLV_UP_LABEL: {				      \
			STREAM << static_cast<const UP_LABEL_TLV &>(OBJ);   \
			break;						      \
		}                                                             \
		default:						      \
			throw(std::string("Invalid TLV obj type"));	      \
	}								      \
}

// Utils for (hdr) ERO SubObject
#define EROSUBOBJ_OBJ2STREAM(STREAM,OBJ)				     \
{									     \
	switch (OBJ.type()) {						     \
		case IPV4_PREF_SUBOBJECT: {				     \
			STREAM << static_cast<const IPv4Pref_SubObj &>(OBJ); \
			break;						     \
		}							     \
		case IPV6_PREF_SUBOBJECT: {				     \
			STREAM << static_cast<const IPv6Pref_SubObj &>(OBJ); \
			break;						     \
		}							     \
		case AS_SUBOBJECT: {					     \
			STREAM << static_cast<const ASNum_SubObj &>(OBJ);    \
			break;						     \
		}							     \
		case UNNUM_LINK_SUBOBJECT: {				     \
			STREAM << static_cast<const Unnum_SubObj &>(OBJ);    \
			break;						     \
		}							     \
		case LABEL_ERO_SUBOBJECT: {				     \
			STREAM << static_cast<const LabelERO_SubObj &>(OBJ); \
			break;						     \
		}							     \
		default:						     \
			throw(std::string("Invalid ERO subObj type"));	     \
	}								     \
}

// Utils for (hdr) RRO SubObject
#define RROSUBOBJ_OBJ2STREAM(STREAM,OBJ)				      \
{									      \
	switch (OBJ.type()) {						      \
		case IPV4_ADDR_SUBOBJ: {				      \
			STREAM << static_cast<const IPv4Addr_SubObj &>(OBJ);  \
			break;						      \
		}							      \
		case IPV6_ADDR_SUBOBJ: {				      \
			STREAM << static_cast<const IPv6Addr_SubObj &>(OBJ);  \
			break;						      \
		}							      \
		case UNNUM_LINK_SUBOBJ: {				      \
			STREAM << static_cast<const Unnumbered_SubObj &>(OBJ);\
			break;						      \
		}							      \
		case LABEL_SUBOBJ: {					      \
			STREAM << static_cast<const Label_SubObj &>(OBJ);     \
			break;						      \
		}							      \
		default:						      \
			throw(std::string("Invalid RRO subObj type"));	      \
	}								      \
}


// Utils for (hdr) GUNI SubObject
#define GUNI_SUBOBJ_OBJ2STREAM(STREAM,OBJ)				      \
{									      \
  switch (OBJ.type()) {							      \
	case TYPE_SRC_TNA: {						      \
		switch (OBJ.subType()) {				      \
			case SUBTYPE_IPv4_SRC_TNA: {			      \
				STREAM << static_cast			      \
				     <const IPv4SourceTnaAddr_SubObj &>(OBJ); \
				break;					      \
			}						      \
			case SUBTYPE_IPv6_SRC_TNA: {			      \
				STREAM << static_cast			      \
				     <const IPv6SourceTnaAddr_SubObj &>(OBJ); \
				break;					      \
			}						      \
			case SUBTYPE_NSAP_SRC_TNA: {			      \
				STREAM << static_cast			      \
				     <const NSAPSourceTnaAddr_SubObj &>(OBJ); \
				break;					      \
			}						      \
			default:					      \
				throw(std::string("Invalid subObj subType")); \
		}							      \
		break;							      \
	}								      \
	case TYPE_DST_TNA: {						      \
		switch (OBJ.subType()) {				      \
			case SUBTYPE_IPv4_DST_TNA: {			      \
				STREAM << static_cast			      \
				      <const IPv4DestTnaAddr_SubObj &>(OBJ);  \
				break;					      \
			}						      \
			case SUBTYPE_IPv6_DST_TNA: {			      \
				STREAM << static_cast			      \
				      <const IPv6DestTnaAddr_SubObj &>(OBJ);  \
				break;					      \
			}						      \
			case SUBTYPE_NSAP_DST_TNA: {			      \
				STREAM << static_cast			      \
				      <const NSAPDestTnaAddr_SubObj &>(OBJ);  \
				break;					      \
			}						      \
			default:					      \
				throw(std::string("Invalid subObj subType")); \
		}							      \
		break;							      \
	}								      \
	case TYPE_DIVERSITY: {						      \
		switch (OBJ.subType()) {				      \
			case SUBTYPE_DIVERSITY: {			      \
				STREAM << static_cast			      \
					<const Diversity_SubObj &>(OBJ);      \
				break;					      \
			}						      \
			default:					      \
				throw(std::string("Invalid subObj subType")); \
		}							      \
		break;							      \
	}								      \
	case TYPE_EGR_LABEL: {						      \
		switch (OBJ.subType()) {				      \
			case SUBTYPE_EGR_LABEL: {			      \
				STREAM << static_cast			      \
					<const EgressLabel_SubObj &>(OBJ);    \
				break;					      \
			}						      \
			case SUBTYPE_EGR_SPC_LABEL: {			      \
				STREAM << static_cast			      \
					<const SpcLabel_SubObj &>(OBJ);	      \
				break;					      \
			}						      \
			default:					      \
				throw(std::string("Invalid subObj subType")); \
		}							      \
		break;							      \
	}								      \
	case TYPE_SERVICE_LVL: {					      \
		switch (OBJ.subType()) {				      \
			case SUBTYPE_SERVICE_LVL: {			      \
				STREAM << static_cast			      \
					<const ServiceLevel_SubObj &>(OBJ);   \
				break;					      \
			}						      \
			default:					      \
				throw(std::string("Invalid subObj subType")); \
		}							      \
		break;							      \
	}								      \
	default:							      \
		throw(std::string("Invalid subObj type"));		      \
  }									      \
}

// Utils for (hdr) GNS SubObject
#define GNS_SUBOBJ_OBJ2STREAM(STREAM,OBJ)				      \
{									      \
  switch (OBJ.type()) {							      \
	case TYPE_APPLICATION: {					      \
		switch (OBJ.subType()) {				      \
			case SUBTYPE_APPLICATION: {			      \
				STREAM << static_cast			      \
					<const Application_SubObj &>(OBJ);    \
				break;					      \
			}						      \
			default:					      \
				throw(std::string("Invalid subObj subType")); \
		}							      \
		break;							      \
	}								      \
	case TYPE_CANDIDATE_HOSTS: {					      \
		switch (OBJ.subType()) {				      \
			case SUBTYPE_IPv4_CANDIDATE_HOSTS: {		      \
				STREAM << static_cast			      \
				      <const IPv4CandidateHost_SubObj &>(OBJ);\
				break;					      \
			}						      \
			case SUBTYPE_IPv6_CANDIDATE_HOSTS: {		      \
				STREAM << static_cast			      \
				      <const IPv6CandidateHost_SubObj &>(OBJ);\
				break;					      \
			}						      \
			case SUBTYPE_NSAP_CANDIDATE_HOSTS: {		      \
				STREAM << static_cast			      \
				      <const NsapCandidateHost_SubObj &>(OBJ);\
				break;					      \
			}						      \
			default:					      \
				throw(std::string("Invalid subObj subType")); \
		}							      \
		break;							      \
	}								      \
	case TYPE_FILE_SYSTEM_RES: {					      \
		switch (OBJ.subType()) {				      \
			case SUBTYPE_FILE_SYSTEM_RES: {			      \
				STREAM << static_cast			      \
					<const FsResources_SubObj &>(OBJ);    \
				break;					      \
			}						      \
			default:					      \
				throw(std::string("Invalid subObj subType")); \
		}							      \
		break;							      \
	}								      \
	case TYPE_SYSTEM_CAPS: {					      \
		switch (OBJ.subType()) {				      \
			case SUBTYPE_SYSTEM_CAPS: {			      \
				STREAM << static_cast			      \
					<const SystemCaps_SubObj &>(OBJ);     \
				break;					      \
			}						      \
			default:					      \
				throw(std::string("Invalid subObj subType")); \
		}							      \
		break;							      \
	}								      \
	case TYPE_INDIV_CPU_SPEED: {					      \
		switch (OBJ.subType()) {				      \
			case SUBTYPE_INDIV_CPU_SPEED: {			      \
				STREAM << static_cast			      \
					<const IndvCpuSpeed_SubObj &>(OBJ);   \
				break;					      \
			}						      \
			default:					      \
				throw(std::string("Invalid subObj subType")); \
		}							      \
		break;							      \
	}								      \
	case TYPE_INDIV_CPU_TIME: {					      \
		switch (OBJ.subType()) {				      \
			case SUBTYPE_INDIV_CPU_TIME: {			      \
				STREAM << static_cast			      \
					<const IndvCpuTime_SubObj &>(OBJ);    \
				break;					      \
			}						      \
			default:					      \
				throw(std::string("Invalid subObj subType")); \
		}							      \
		break;							      \
	}								      \
	case TYPE_INDIV_CPU_COUNT: {					      \
		switch (OBJ.subType()) {				      \
			case SUBTYPE_INDIV_CPU_COUNT: {			      \
				STREAM << static_cast			      \
					<const IndvCpuCount_SubObj &>(OBJ);   \
				break;					      \
			}						      \
			default:					      \
				throw(std::string("Invalid subObj subType")); \
		}							      \
		break;							      \
	}								      \
	case TYPE_INDIV_NET_BW: {					      \
		switch (OBJ.subType()) {				      \
			case SUBTYPE_INDIV_NET_BW: {			      \
				STREAM << static_cast			      \
					<const IndvNetBw_SubObj &>(OBJ);      \
				break;					      \
			}						      \
			default:					      \
				throw(std::string("Invalid subObj subType")); \
		}							      \
		break;							      \
	}								      \
	case TYPE_INDIV_PHY_MEM: {					      \
		switch (OBJ.subType()) {				      \
			case SUBTYPE_INDIV_PHY_MEM: {			      \
				STREAM << static_cast			      \
					<const IndvPhyMem_SubObj &>(OBJ);     \
				break;					      \
			}						      \
			default:					      \
				throw(std::string("Invalid subObj subType")); \
		}							      \
		break;							      \
	}								      \
	case TYPE_INDIV_VIR_MEM: {					      \
		switch (OBJ.subType()) {				      \
			case SUBTYPE_INDIV_VIR_MEM: {			      \
				STREAM << static_cast			      \
					<const IndvVirMem_SubObj &>(OBJ);     \
				break;					      \
			}						      \
			default:					      \
				throw(std::string("Invalid subObj subType")); \
		}							      \
		break;							      \
	}								      \
	case TYPE_INDIV_DISK_SPACE: {					      \
		switch (OBJ.subType()) {				      \
			case SUBTYPE_INDIV_DISK_SPACE: {		      \
				STREAM << static_cast			      \
					<const IndvDiskSpace_SubObj &>(OBJ);  \
				break;					      \
			}						      \
			default:					      \
				throw(std::string("Invalid subObj subType")); \
		}							      \
		break;							      \
	}								      \
	case TYPE_TOTAL_CPU_TIME: {					      \
		switch (OBJ.subType()) {				      \
			case SUBTYPE_TOTAL_CPU_TIME: {			      \
				STREAM << static_cast			      \
					<const TotCpuTime_SubObj &>(OBJ);     \
				break;					      \
			}						      \
			default:					      \
				throw(std::string("Invalid subObj subType")); \
		}							      \
		break;							      \
	}								      \
	case TYPE_TOTAL_CPU_COUNT: {					      \
		switch (OBJ.subType()) {				      \
			case SUBTYPE_TOTAL_CPU_COUNT: {			      \
				STREAM << static_cast			      \
					<const TotCpuCount_SubObj &>(OBJ);    \
				break;					      \
			}						      \
			default:					      \
				throw(std::string("Invalid subObj subType")); \
		}							      \
		break;							      \
	}								      \
	case TYPE_TOTAL_PHY_MEM: {					      \
		switch (OBJ.subType()) {				      \
			case SUBTYPE_TOTAL_PHY_MEM: {			      \
				STREAM << static_cast			      \
					<const TotPhyMem_SubObj &>(OBJ);      \
				break;					      \
			}						      \
			default:					      \
				throw(std::string("Invalid subObj subType")); \
		}							      \
		break;							      \
	}								      \
	case TYPE_TOTAL_VIR_MEM: {					      \
		switch (OBJ.subType()) {				      \
			case SUBTYPE_TOTAL_VIR_MEM: {			      \
				STREAM << static_cast			      \
					<const TotVirMem_SubObj &>(OBJ);      \
				break;					      \
			}						      \
			default:					      \
				throw(std::string("Invalid subObj subType")); \
		}							      \
		break;							      \
	}								      \
	case TYPE_TOTAL_DISK_SPACE: {					      \
		switch (OBJ.subType()) {				      \
			case SUBTYPE_TOTAL_DISK_SPACE: {		      \
				STREAM << static_cast			      \
					<const TotDiskSpace_SubObj &>(OBJ);   \
				break;					      \
			}						      \
			default:					      \
				throw(std::string("Invalid subObj subType")); \
		}							      \
		break;							      \
	}								      \
	case TYPE_TOTAL_RES_COUNT: {					      \
		switch (OBJ.subType()) {				      \
			case SUBTYPE_TOTAL_RES_COUNT: {			      \
				STREAM << static_cast			      \
					<const TotResCount_SubObj &>(OBJ);    \
				break;					      \
			}						      \
			default:					      \
				throw(std::string("Invalid subObj subType")); \
		}							      \
		break;							      \
	}								      \
	case TYPE_DATA_STAGING: {					      \
		switch (OBJ.subType()) {				      \
			case SUBTYPE_DATA_STAGING: {			      \
				STREAM << static_cast			      \
					<const DataStaging_SubObj &>(OBJ);    \
				break;					      \
			}						      \
			default:					      \
				throw(std::string("Invalid subObj subType")); \
		}							      \
		break;							      \
	}								      \
	default:							      \
		throw(std::string("Invalid subObj type"));		      \
  }									      \
}


//
// Objects
//
#define Session_Class_OBJ2STREAM(STREAM,OBJ)				    \
{									    \
	switch (OBJ.cType()) {						    \
		case CTYPE_SESSION_IPV4: {				    \
			STREAM << static_cast				    \
				<const IPv4Session_Object &>(OBJ);	    \
			break;						    \
		}							    \
		case CTYPE_SESSION_IPV6: {				    \
			STREAM << static_cast				    \
				<const IPv6Session_Object &>(OBJ);	    \
			break;						    \
		}							    \
		case CTYPE_SESSION_LSP_TUNNEL_IPV4: {			    \
			STREAM << static_cast				    \
				<const LSPTunnelIPv4Session_Object &>(OBJ); \
			break;						    \
		}							    \
		case CTYPE_SESSION_LSP_TUNNEL_IPV6: {			    \
			STREAM << static_cast				    \
				<const LSPTunnelIPv6Session_Object &>(OBJ); \
			break;						    \
		}							    \
		case CTYPE_SESSION_UNI_IPV4: {				    \
			STREAM << static_cast				    \
				<const UNIIPv4Session_Object &>(OBJ);	    \
			break;						    \
		}							    \
		default:						    \
			throw(std::string("Invalid obj cType"));	    \
	}								    \
}

#define Label_Class_OBJ2STREAM(STREAM,OBJ)				      \
{									      \
	switch (OBJ.cType()) {						      \
		case CTYPE_LABEL: {					      \
			STREAM << static_cast<const LabelMpls_Object &>(OBJ); \
			break;						      \
		}							      \
		case CTYPE_GENERALISED_LABEL: {				      \
			STREAM << static_cast<const GLabel_Object &>(OBJ);    \
			break;						      \
		}							      \
		case CTYPE_WBS_LABEL: {					      \
			STREAM << static_cast<const WBSLabel_Object &>(OBJ);  \
			break;						      \
		}							      \
		default:						      \
			throw(std::string("Invalid obj cType"));	      \
	}								      \
}

#ifdef CRANKBACK_ON
#define LSPAttr_Class_OBJ2STREAM(STREAM,OBJ)			              \
{									      \
	switch (OBJ.cType()) {						      \
		case CTYPE_LSP_ATTRIBUTES: {				      \
			STREAM << static_cast<const LSPAttr_Object &>(OBJ);   \
			break;						      \
		}							      \
		default:						      \
			throw(std::string("Invalid obj cType"));	      \
	}								      \
}
#endif // CRANKBACK_ON

#define Session_Attr_Class_OBJ2STREAM(STREAM,OBJ)			    \
{									    \
	switch (OBJ.cType()) {						    \
		case CTYPE_SESSION_ATTRIBUTE_LSP_TUNNEL: {		    \
			STREAM << static_cast				    \
				<const LSPTunnelSessionAttr_Object &>(OBJ); \
			break;						    \
		}							    \
		case CTYPE_SESSION_ATTRIBUTE_LSP_TUNNEL_RA: {		    \
			STREAM << static_cast				    \
			       <const LSPTunnelRASessionAttr_Object &>(OBJ);\
			break;						    \
		}							    \
		default:						    \
			throw(std::string("Invalid obj cType"));	    \
	}								    \
}

#define Rsvp_Hop_Class_OBJ2STREAM(STREAM,OBJ)				\
{									\
	switch (OBJ.cType()) {						\
		case CTYPE_RSVP_HOP_IPV4: {				\
			STREAM << static_cast				\
				<const IPv4RsvpHop_Object &>(OBJ);	\
			break;						\
		}							\
		case CTYPE_RSVP_HOP_IPV6: {				\
			STREAM << static_cast				\
				<const IPv6RsvpHop_Object &>(OBJ);	\
			break;						\
		}							\
		case CTYPE_RSVP_HOP_IF_ID_IPV4: {			\
			STREAM << static_cast				\
				<const IFIDIPv4RsvpHop_Object &>(OBJ);	\
			break;						\
		}							\
		case CTYPE_RSVP_HOP_IF_ID_IPV6: {			\
			STREAM << static_cast				\
				<const IFIDIPv6RsvpHop_Object &>(OBJ);	\
			break;						\
		}							\
		default:						\
			throw(std::string("Invalid obj cType"));	\
	}								\
}

#define TimeValues_Class_OBJ2STREAM(STREAM,OBJ)				\
{									\
	switch (OBJ.cType()) {						\
		case CTYPE_TIME_VALUES: {				\
			STREAM << static_cast				\
				<const TimeValues_Object &>(OBJ);	\
			break;						\
		}							\
		default:						\
			throw(std::string("Invalid obj cType"));	\
	}								\
}

#define Error_Spec_Class_OBJ2STREAM(STREAM,OBJ)				 \
{									 \
	switch (OBJ.cType()) {						 \
		case CTYPE_ERROR_SPEC_IPV4: {				 \
			STREAM << static_cast				 \
				<const IPv4ErrorSpec_Object &>(OBJ);	 \
			break;						 \
		}							 \
		case CTYPE_ERROR_SPEC_IPV6: {				 \
			STREAM << static_cast				 \
				<const IPv6ErrorSpec_Object &>(OBJ);	 \
			break;						 \
		}							 \
		case CTYPE_ERROR_SPEC_IF_ID_IPV4: {			 \
			STREAM << static_cast				 \
				<const IFIDIPv4ErrorSpec_Object &>(OBJ); \
			break;						 \
		}							 \
		case CTYPE_ERROR_SPEC_IF_ID_IPV6: {			 \
			STREAM << static_cast				 \
				<const IFIDIPv6ErrorSpec_Object &>(OBJ); \
			break;						 \
		}							 \
		default:						 \
			throw(std::string("Invalid obj cType"));	 \
	}								 \
}

#define Scope_Class_OBJ2STREAM(STREAM,OBJ)				      \
{									      \
	switch (OBJ.cType()) {						      \
		case CTYPE_IPV4_SCOPE: {				      \
			STREAM << static_cast<const IPv4Scope_Object &>(OBJ); \
			break;						      \
		}							      \
		case CTYPE_IPV6_SCOPE: {				      \
			STREAM << static_cast<const IPv6Scope_Object &>(OBJ); \
			break;						      \
		}							      \
		default:						      \
			throw(std::string("Invalid obj cType"));	      \
	}								      \
}

#define FlowSpec_Class_OBJ2STREAM(STREAM,OBJ)				\
{									\
	switch (OBJ.cType()) {						\
		case CTYPE_INT_SERV_FLOWSPEC: {				\
			STREAM << static_cast				\
				<const IntServFlowSpec_Object &>(OBJ);	\
			break;						\
		}							\
		default:						\
			throw(std::string("Invalid obj cType"));	\
	}								\
}

#define MessageId_Class_OBJ2STREAM(STREAM,OBJ)				 \
{									 \
	switch (OBJ.cType()) {						 \
		case CTYPE_MESSAGE_ID: {				 \
			STREAM << static_cast				 \
				<const MessageId_Object &>(OBJ);	 \
			break;						 \
		}							 \
		default:						 \
			throw(std::string("Invalid obj cType"));	 \
	}								 \
}

#define MessageIdAckNack_Class_OBJ2STREAM(STREAM,OBJ)			 \
	switch (OBJ.cType()) {						 \
		case CTYPE_MESSAGE_ID_ACK: {				 \
			STREAM << static_cast				 \
				<const MessageIdAck_Object &>(OBJ);	 \
			break;						 \
		}							 \
		case CTYPE_MESSAGE_ID_NACK: {				 \
			STREAM << static_cast				 \
				<const MessageIdNack_Object &>(OBJ);	 \
			break;						 \
		}							 \
		default:						 \
			throw(std::string("Invalid obj cType"));	 \
	}

#define MessageIdList_Class_OBJ2STREAM(STREAM,OBJ)

#define Integrity_Class_OBJ2STREAM(STREAM,OBJ)				\
{									\
	switch (OBJ.cType()) {						\
		case CTYPE_INTEGRITY: {					\
			STREAM << static_cast				\
				<const Integrity_Object &>(OBJ);	\
			break;						\
		}							\
		default:						\
			throw(std::string("Invalid obj cType"));	\
	}								\
}

#define ExplicitRoute_Class_OBJ2STREAM(STREAM,OBJ)			\
{									\
	switch (OBJ.cType()) {						\
		case CTYPE_ERO: {					\
			STREAM << static_cast				\
				<const ExplicitRoute_Object &>(OBJ);	\
			break;						\
		}							\
		default:						\
			throw(std::string("Invalid obj cType"));	\
	}								\
}

#define LabelRequest_Class_OBJ2STREAM(STREAM,OBJ)			\
{									\
	switch (OBJ.cType()) {						\
		case CTYPE_GENERALISED_LABEL_REQ: {			\
			STREAM << static_cast				\
				<const GLabelRequest_Object &>(OBJ);	\
			break;						\
		}							\
		default:						\
			throw(std::string("Invalid obj cType"));	\
	}								\
}

#define CallOptions_Class_OBJ2STREAM(STREAM,OBJ)			 \
{									 \
	switch (OBJ.cType()) {						 \
		case CTYPE_CALL_OPS: {					 \
			STREAM << static_cast				 \
				<const CallOptions_Object &>(OBJ);	 \
			break;						 \
		}							 \
		default:						 \
			throw(std::string("Invalid obj cType"));	 \
	}								 \
}

#define CallId_Class_OBJ2STREAM(STREAM,OBJ)				\
{									\
	switch (OBJ.cType()) {						\
		case CTYPE_CALL_ID_OPERATOR_SPECIFIC: {			\
			STREAM << static_cast				\
				<const CallIdOpSpec_Object &>(OBJ);	\
			break;						\
		}							\
		case CTYPE_CALL_ID_GLOBALLY_UNIQUE: {			\
			STREAM << static_cast				\
				<const CallIdGlobal_Object &>(OBJ);	\
			break;						\
		}                                                       \
		case CTYPE_CALL_ID_NULL: {			        \
			STREAM << static_cast				\
				<const CallIdNull_Object &>(OBJ);	\
			break;						\
		}							\
		default:						\
			throw(std::string("Invalid obj cType"));	\
	}								\
}

#define AdminStatus_Class_OBJ2STREAM(STREAM,OBJ)			      \
{									      \
	switch (OBJ.cType()) {						      \
		case CTYPE_ADMIN_STATUS: {				      \
			STREAM << static_cast<const AdminStatus_Object&>(OBJ);\
			break;						      \
		}							      \
		default:						      \
			throw(std::string("Invalid obj cType"));	      \
	}								      \
}

#define Protection_Class_OBJ2STREAM(STREAM,OBJ)				      \
{									      \
	switch (OBJ.cType()) {						      \
		case CTYPE_PROTECTION: {				      \
			STREAM << static_cast<const Protection_Object &>(OBJ);\
			break;						      \
		}							      \
		default:						      \
			throw(std::string("Invalid obj cType"));	      \
	}								      \
}

#define Style_Class_OBJ2STREAM(STREAM,OBJ)				      \
{									      \
	switch (OBJ.cType()) {						      \
		case CTYPE_STYLE: {					      \
			STREAM << static_cast<const Style_Object &>(OBJ);     \
			break;						      \
		}							      \
		default:						      \
			throw(std::string("Invalid obj cType"));	      \
	}								      \
}

#define FilterSpec_Class_OBJ2STREAM(STREAM,OBJ)				  \
{									  \
	switch (OBJ.cType()) {						  \
		case CTYPE_FLOW_LABEL_FILTER_SPEC: {			  \
			STREAM << static_cast				  \
				<const IPV6FlowFilterSpec_Object &>(OBJ); \
			break;						  \
		}							  \
		case CTYPE_IPV4_FILTER_SPEC: {				  \
			STREAM << static_cast				  \
				<const IPV4FilterSpec_Object &>(OBJ);	  \
			break;						  \
		}							  \
		case CTYPE_IPV6_FILTER_SPEC: {				  \
			STREAM << static_cast				  \
				<const IPV6FilterSpec_Object &>(OBJ);	  \
			break;						  \
		}							  \
		case CTYPE_LSP_IPV4_FILTER_SPEC: {			  \
			STREAM << static_cast				  \
				<const IPV4LSPFilterSpec_Object &>(OBJ);  \
			break;						  \
		}							  \
		case CTYPE_LSP_IPV6_FILTER_SPEC: {			  \
			STREAM << static_cast				  \
				<const IPV6LSPFilterSpec_Object &>(OBJ);  \
			break;						  \
		}							  \
		default:						  \
			throw(std::string("Invalid obj cType"));	  \
	}								  \
}

#define Policy_Class_OBJ2STREAM(STREAM,OBJ)				    \
{									    \
	switch (OBJ.cType()) {						    \
		case  CTYPE_POLICY: {					    \
			STREAM << static_cast <const Policy_Object &>(OBJ); \
			break;						    \
		}							    \
		default:						    \
			throw(std::string("Invalid obj cType"));	    \
	}								    \
}

#define SenderTemplate_Class_OBJ2STREAM(STREAM,OBJ)			      \
{									      \
	switch (OBJ.cType()) {						      \
		case CTYPE_FLOW_LABEL_SENDER_TEMPLATE: {		      \
			STREAM << static_cast				      \
				<const IPV6FlowSenderTemplate_Object &>(OBJ); \
			break;						      \
		}							      \
		case CTYPE_IPV4_SENDER_TEMPLATE: {			      \
			STREAM << static_cast				      \
				<const IPV4SenderTemplate_Object &>(OBJ);     \
			break;						      \
		}							      \
		case CTYPE_IPV6_SENDER_TEMPLATE: {			      \
			STREAM << static_cast				      \
				<const IPV6SenderTemplate_Object &>(OBJ);     \
			break;						      \
		}							      \
		case CTYPE_LSP_IPV4_SENDER_TEMPLATE: {			      \
			STREAM << static_cast				      \
				<const IPV4LSPSenderTemplate_Object &>(OBJ);  \
			break;						      \
		}							      \
		case CTYPE_LSP_IPV6_SENDER_TEMPLATE: {			      \
			STREAM << static_cast				      \
				<const IPV6LSPSenderTemplate_Object &>(OBJ);  \
			break;						      \
		}							      \
		default:						      \
			throw(std::string("Invalid obj cType"));	      \
	}								      \
}

#define NotifyRequest_Class_OBJ2STREAM(STREAM,OBJ)			  \
{									  \
	switch (OBJ.cType()) {						  \
		case CTYPE_IPV4_NOTIFY_REQUEST: {			  \
			STREAM << static_cast				  \
				<const IPV4NotifyRequest_Object &>(OBJ);  \
			break;						  \
		}							  \
		case CTYPE_IPV6_NOTIFY_REQUEST: {			  \
			STREAM << static_cast				  \
				<const IPV6NotifyRequest_Object &>(OBJ);  \
			break;						  \
		}							  \
		default:						  \
			throw(std::string("Invalid obj cType"));	  \
	}								  \
}

#define LabelSet_Class_OBJ2STREAM(STREAM,OBJ)				\
{									\
	switch (OBJ.cType()) {						\
		case CTYPE_LABEL_SET: {					\
			STREAM << static_cast				\
				<const LabelSet_Object &>(OBJ);		\
			break;						\
		}							\
		default:						\
			throw(std::string("Invalid obj cType"));	\
	}								\
}

#define AcceptableLabelSet_Class_OBJ2STREAM(STREAM,OBJ)			  \
	{								  \
	switch (OBJ.cType()) {						  \
			case CTYPE_ACCEPTABLE_LABEL_SET: {		  \
			STREAM << static_cast				  \
				<const AcceptableLabelSet_Object &>(OBJ); \
			break;						  \
		}							  \
		default:						  \
			throw(std::string("Invalid obj cType"));	  \
	}								  \
}

#define SuggestedLabel_Class_OBJ2STREAM(STREAM,OBJ)			\
{									\
	switch (OBJ.cType()) {						\
		case CTYPE_SUGGESTED_GENERALISED_LABEL: {		\
			STREAM << static_cast				\
				<const SuggestedLabel_Object &>(OBJ);	\
			break;						\
		}							\
		default:						\
			throw(std::string("Invalid obj cType"));	\
	}								\
}

#define UpstreamLabel_Class_OBJ2STREAM(STREAM,OBJ)			\
{									\
	switch (OBJ.cType()) {						\
		case CTYPE_UPSTREAM_GENERALISED_LABEL: {		\
			STREAM << static_cast				\
				<const UpstreamLabel_Object &>(OBJ);	\
			break;						\
		}							\
		default:						\
			throw(std::string("Invalid obj cType"));	\
	}								\
}

#define RecoveryLabel_Class_OBJ2STREAM(STREAM,OBJ)			\
{									\
	switch (OBJ.cType()) {						\
		case CTYPE_RECOVERY_GENERALISED_LABEL: {		\
			STREAM << static_cast				\
				<const RecoveryLabel_Object &>(OBJ);	\
			break;						\
		}							\
		default:						\
			throw(std::string("Invalid obj cType"));	\
	}								\
}

#define ADSpec_Class_OBJ2STREAM(STREAM,OBJ)

#define SenderTSpec_Class_OBJ2STREAM(STREAM,OBJ)			\
{									\
	switch (OBJ.cType()) {						\
		case CTYPE_SENDER_TSPEC: {				\
			STREAM << static_cast				\
				<const SenderTSpec_Object &>(OBJ);	\
			break;						\
		}							\
		default:						\
			throw(std::string("Invalid obj cType"));	\
	}								\
}

#define RecordedRoute_Class_OBJ2STREAM(STREAM,OBJ)			\
{									\
	switch (OBJ.cType()) {						\
		case CTYPE_RECORDED_ROUTE: {				\
			STREAM << static_cast				\
				<const RecordedRoute_Object &>(OBJ);	\
			break;						\
		}							\
		default:						\
			throw(std::string("Invalid obj cType"));	\
	}								\
}

#define ResvConfirm_Class_OBJ2STREAM(STREAM,OBJ)			 \
{									 \
	switch (OBJ.cType()) {						 \
		case CTYPE_RESVCONF_IPV4: {				 \
			STREAM << static_cast				 \
				<const ResvConfirmIPv4_Object &>(OBJ);	 \
			break;						 \
		}							 \
		case CTYPE_RESVCONF_IPV6: {				 \
			STREAM << static_cast				 \
				<const ResvConfirmIPv6_Object &>(OBJ);	 \
			break;						 \
		}							 \
		default:						 \
			throw(std::string("Invalid obj cType"));	 \
	}								 \
}

#define Hello_Class_OBJ2STREAM(STREAM,OBJ)
#define RestartCap_Class_OBJ2STREAM(STREAM,OBJ)

#define GnsCallExt_Class_OBJ2STREAM(STREAM,OBJ)				      \
{									      \
	switch (OBJ.cType()) {						      \
		case CTYPE_GNS_CALL: {					      \
			STREAM << static_cast<const GnsCallExt_Object &>(OBJ);\
			break;						      \
		}							      \
		default:						      \
			throw(std::string("Invalid obj cType"));	      \
	}								      \
}

#define GnsUni_Class_OBJ2STREAM(STREAM,OBJ)				\
{									\
	switch (OBJ.cType()) {						\
		case CTYPE_GNS_UNI: {					\
			STREAM << static_cast				\
				<const GnsUni_Object &>(OBJ);		\
			break;						\
		}							\
		default:						\
			throw(std::string("Invalid obj cType"));	\
	}								\
}

#define GeneralisedUni_Class_OBJ2STREAM(STREAM,OBJ)			\
{									\
	switch (OBJ.cType()) {						\
		case CTYPE_GENERALIZED_UNI: {				\
			STREAM << static_cast				\
				<const GeneralisedUni_Object &>(OBJ);	\
			break;						\
		}							\
		default:						\
			throw(std::string("Invalid obj cType"));	\
	}								\
}

#define Times_Class_OBJ2STREAM(STREAM,OBJ)				\
{									\
	switch (OBJ.cType()) {						\
		case CTYPE_TIMES: {				\
			STREAM << static_cast				\
				<const Times_Object &>(OBJ);	\
			break;						\
		}							\
		default:						\
			throw(std::string("Invalid obj cType"));	\
	}								\
}



//
// Utils
//

#define SenderTemplate2FilterSpec_Class(SENDER,FILTER)			\
{									\
	assert(SENDER);							\
	switch (SENDER->cType()) {					\
		case CTYPE_FLOW_LABEL_SENDER_TEMPLATE: {		\
			FILTER = new IPV6FlowFilterSpec_Object((static_cast \
			  <IPV6FlowSenderTemplate_Object *>(SENDER))->get()); \
			assert(FILTER);					\
			break;						\
		}							\
		case CTYPE_IPV4_SENDER_TEMPLATE: {			\
			FILTER = new IPV4FilterSpec_Object((static_cast \
			  <IPV4SenderTemplate_Object *>(SENDER))->get()); \
			assert(FILTER);					\
			break;						\
		}							\
		case CTYPE_IPV6_SENDER_TEMPLATE: {			\
			FILTER = new IPV6FilterSpec_Object((static_cast \
			  <IPV6SenderTemplate_Object *>(SENDER))->get()); \
			assert(FILTER);					\
			break;						\
		}							\
		case CTYPE_LSP_IPV4_SENDER_TEMPLATE: {			\
			FILTER = new IPV4LSPFilterSpec_Object((static_cast \
			  <IPV4LSPSenderTemplate_Object *>(SENDER))->get()); \
			assert(FILTER);					\
			break;						\
		}							\
		case CTYPE_LSP_IPV6_SENDER_TEMPLATE: {			\
			FILTER = new IPV6LSPFilterSpec_Object((static_cast \
			  <IPV6LSPSenderTemplate_Object *>(SENDER))->get()); \
			assert(FILTER);					\
			break;						\
		}							\
		default:						\
			throw(std::string("Invalid obj cType"));	\
	}								\
}

#define SenderTSpec2FlowSpec_Class(SENDER,FLOW)			        \
{									\
	assert(SENDER);							\
	switch (SENDER->cType()) {					\
		case CTYPE_SENDER_TSPEC: {				\
			FLOW = new IntServFlowSpec_Object((static_cast	\
			<SenderTSpec_Object *>(SENDER))->get());        \
			assert(FLOW);                                   \
			break;						\
		}							\
		default:						\
			throw(std::string("Invalid obj cType"));	\
	}								\
}

// Utils for (hdr) TLV
#define TLV_STREAM2OBJ(STREAM,OBJ,TYPE)					   \
{									   \
	switch (TYPE) {							   \
		case IF_ID_TLV_IPV4: {					   \
			OBJ = new IPv4_TLV();				   \
			assert(OBJ);					   \
			STREAM >> static_cast<IPv4_TLV &>(*(OBJ));	   \
			break;						   \
		}							   \
		case IF_ID_TLV_IPV6: {					   \
			OBJ = new IPv6_TLV();				   \
			assert(OBJ);					   \
			STREAM >> static_cast<IPv6_TLV &>(*(OBJ));	   \
			break;						   \
		}							   \
		case IF_ID_TLV_IF_INDEX: {				   \
			OBJ = new IF_IDX_TLV();				   \
			assert(OBJ);					   \
			STREAM >> static_cast<IF_IDX_TLV &>(*(OBJ));	   \
			break;						   \
		}							   \
		case IF_ID_TLV_COMP_IF_DOWN: {				   \
			OBJ = new COMP_IF_DOWN_TLV();			   \
			assert(OBJ);					   \
			STREAM >> static_cast<COMP_IF_DOWN_TLV &>(*(OBJ)); \
			break;						   \
		}							   \
		case IF_ID_TLV_COMP_IF_UP: {				   \
			OBJ = new COMP_IF_UP_TLV();			   \
			assert(OBJ);					   \
			STREAM >> static_cast<COMP_IF_UP_TLV &>(*(OBJ));   \
			break;						   \
		}							   \
		case IF_ID_TLV_DOWN_LABEL: {				   \
			OBJ = new DOWN_LABEL_TLV();			   \
			assert(OBJ);					   \
			STREAM >> static_cast<DOWN_LABEL_TLV &>(*(OBJ));   \
			break;						   \
		}							   \
		case IF_ID_TLV_UP_LABEL: {				   \
			OBJ = new UP_LABEL_TLV();			   \
			assert(OBJ);					   \
			STREAM >> static_cast<UP_LABEL_TLV &>(*(OBJ));     \
			break;						   \
		}							   \
		default:						   \
			throw(std::string("Invalid TLV obj type"));	   \
	}								   \
}

// Utils for (hdr) ERO SubObject
#define EROSUBOBJ_STREAM2OBJ(STREAM,OBJ,TYPE)				  \
{									  \
	switch (TYPE) {							  \
		case IPV4_PREF_SUBOBJECT: {				  \
			OBJ = new IPv4Pref_SubObj();			  \
			assert(OBJ);					  \
			STREAM >> static_cast<IPv4Pref_SubObj &>(*(OBJ)); \
			break;						  \
		}							  \
		case IPV6_PREF_SUBOBJECT: {				  \
			OBJ = new IPv6Pref_SubObj();			  \
			assert(OBJ);					  \
			STREAM >> static_cast<IPv6Pref_SubObj &>(*(OBJ)); \
			break;						  \
		}							  \
		case AS_SUBOBJECT: {					  \
			OBJ = new ASNum_SubObj();			  \
			assert(OBJ);					  \
			STREAM >> static_cast<ASNum_SubObj &>(*(OBJ));	  \
			break;						  \
		}							  \
		case UNNUM_LINK_SUBOBJECT: {				  \
			OBJ = new Unnum_SubObj();			  \
			assert(OBJ);					  \
			STREAM >> static_cast<Unnum_SubObj &>(*(OBJ));	  \
			break;						  \
		}							  \
		case LABEL_ERO_SUBOBJECT: {				  \
			OBJ = new LabelERO_SubObj();			  \
			assert(OBJ);					  \
			STREAM >> static_cast<LabelERO_SubObj &>(*(OBJ)); \
			break;						  \
		}							  \
		default:						  \
			throw(std::string("Invalid ERO subObj type"));	  \
	}								  \
}

// Utils for (hdr) RRO SubObject
#define RROSUBOBJ_STREAM2OBJ(STREAM,OBJ,TYPE)				    \
{									    \
	switch (TYPE) {							    \
		case IPV4_ADDR_SUBOBJ: {				    \
			OBJ = new IPv4Addr_SubObj();			    \
			assert(OBJ);					    \
			STREAM >> static_cast<IPv4Addr_SubObj &>(*(OBJ));   \
			break;						    \
		}							    \
		case IPV6_ADDR_SUBOBJ: {				    \
			OBJ = new IPv6Addr_SubObj();			    \
			assert(OBJ);					    \
			STREAM >> static_cast<IPv6Addr_SubObj &>(*(OBJ));   \
			break;						    \
		}							    \
		case UNNUM_LINK_SUBOBJ: {				    \
			OBJ = new Unnumbered_SubObj();			    \
			assert(OBJ);					    \
			STREAM >> static_cast<Unnumbered_SubObj &>(*(OBJ)); \
			break;						    \
		}							    \
		case LABEL_SUBOBJ: {					    \
			OBJ = new Label_SubObj();			    \
			assert(OBJ);					    \
			STREAM >> static_cast<Label_SubObj &>(*(OBJ));	    \
			break;						    \
		}							    \
		default:						    \
			throw(std::string("Invalid RRO subObj type"));	    \
	}								    \
}


// Utils for (hdr) GUNI SubObject
#define GUNI_SUBOBJ_STREAM2OBJ(STREAM,OBJ,TYPE,SUBTYPE)			      \
{									      \
  switch (TYPE) {							      \
	case TYPE_SRC_TNA: {						      \
		switch (SUBTYPE) {					      \
			case SUBTYPE_IPv4_SRC_TNA: {			      \
				OBJ = new IPv4SourceTnaAddr_SubObj();	      \
				assert(OBJ);				      \
				STREAM >> static_cast			      \
					<IPv4SourceTnaAddr_SubObj &>(*(OBJ)); \
				break;					      \
			}						      \
			case SUBTYPE_IPv6_SRC_TNA: {			      \
				OBJ = new IPv6SourceTnaAddr_SubObj();	      \
				assert(OBJ);				      \
				STREAM >> static_cast			      \
					<IPv6SourceTnaAddr_SubObj &>(*(OBJ)); \
				break;					      \
			}						      \
			case SUBTYPE_NSAP_SRC_TNA: {			      \
				OBJ = new NSAPSourceTnaAddr_SubObj();	      \
				assert(OBJ);				      \
				STREAM >> static_cast			      \
					<NSAPSourceTnaAddr_SubObj &>(*(OBJ)); \
				break;					      \
			}						      \
			default:					      \
				throw(std::string("Invalid subObj subType")); \
		}							      \
		break;							      \
	}								      \
	case TYPE_DST_TNA: {						      \
		switch (SUBTYPE) {					      \
			case SUBTYPE_IPv4_DST_TNA: {			      \
				OBJ = new IPv4DestTnaAddr_SubObj();	      \
				assert(OBJ);				      \
				STREAM >> static_cast			      \
					<IPv4DestTnaAddr_SubObj &>(*(OBJ));   \
				break;					      \
			}						      \
			case SUBTYPE_IPv6_DST_TNA: {			      \
				OBJ = new IPv6DestTnaAddr_SubObj();	      \
				assert(OBJ);				      \
				STREAM >> static_cast			      \
					<IPv6DestTnaAddr_SubObj &>(*(OBJ));   \
				break;					      \
			}						      \
			case SUBTYPE_NSAP_DST_TNA: {			      \
				OBJ = new NSAPDestTnaAddr_SubObj();	      \
				assert(OBJ);				      \
				STREAM >> static_cast			      \
					<NSAPDestTnaAddr_SubObj &>(*(OBJ));   \
				break;					      \
			}						      \
			default:					      \
				throw(std::string("Invalid subObj subType")); \
		}							      \
		break;							      \
	}								      \
	case TYPE_DIVERSITY: {						      \
		switch (SUBTYPE) {					      \
			case SUBTYPE_DIVERSITY: {			      \
				OBJ = new Diversity_SubObj();		      \
				assert(OBJ);				      \
				STREAM >> static_cast			      \
					<Diversity_SubObj &>(*(OBJ));	      \
				break;					      \
			}						      \
			default:					      \
				throw(std::string("Invalid subObj subType")); \
		}							      \
		break;							      \
	}								      \
	case TYPE_EGR_LABEL: {						      \
		switch (SUBTYPE) {					      \
			case SUBTYPE_EGR_LABEL: {			      \
				OBJ = new EgressLabel_SubObj();		      \
				assert(OBJ);				      \
				STREAM >> static_cast			      \
					<EgressLabel_SubObj &>(*(OBJ));	      \
				break;					      \
			}						      \
			case SUBTYPE_EGR_SPC_LABEL: {			      \
				OBJ = new SpcLabel_SubObj();		      \
				assert(OBJ);				      \
				STREAM >> static_cast			      \
					<SpcLabel_SubObj &>(*(OBJ));	      \
				break;					      \
			}						      \
			default:					      \
				throw(std::string("Invalid subObj subType")); \
		}							      \
		break;							      \
	}								      \
	case TYPE_SERVICE_LVL: {					      \
		switch (SUBTYPE) {					      \
			case SUBTYPE_SERVICE_LVL: {			      \
				OBJ = new ServiceLevel_SubObj();	      \
				assert(OBJ);				      \
				STREAM >> static_cast			      \
					<ServiceLevel_SubObj &>(*(OBJ));      \
				break;					      \
			}						      \
			default:					      \
				throw(std::string("Invalid subObj subType")); \
		}							      \
		break;							      \
	}								      \
	default:							      \
		throw(std::string("Invalid subObj type"));		      \
  }									      \
}

// Utils for (hdr) GNS SubObject
#define GNS_SUBOBJ_STREAM2OBJ(STREAM,OBJ,TYPE,SUBTYPE)			      \
{									      \
	switch (TYPE) {							      \
		case TYPE_APPLICATION: {				      \
			switch (SUBTYPE) {				      \
				case SUBTYPE_APPLICATION: {		      \
					OBJ = new Application_SubObj();	      \
					assert(OBJ);			      \
					STREAM >> static_cast		      \
					<Application_SubObj &>(*(OBJ));	      \
					break;				      \
				}					      \
				default:				      \
				throw(std::string("Invalid subObj subType")); \
			}						      \
			break;						      \
		}							      \
		case TYPE_CANDIDATE_HOSTS: {				      \
			switch (SUBTYPE) {				      \
				case SUBTYPE_IPv4_CANDIDATE_HOSTS: {	      \
					OBJ = new IPv4CandidateHost_SubObj(); \
					assert(OBJ);			      \
					STREAM >> static_cast		      \
					<IPv4CandidateHost_SubObj &>(*(OBJ)); \
					break;				      \
				}					      \
				case SUBTYPE_IPv6_CANDIDATE_HOSTS: {	      \
					OBJ = new IPv6CandidateHost_SubObj(); \
					assert(OBJ);			      \
					STREAM >> static_cast		      \
					<IPv6CandidateHost_SubObj &>(*(OBJ)); \
					break;				      \
				}					      \
				case SUBTYPE_NSAP_CANDIDATE_HOSTS: {	      \
					OBJ = new NsapCandidateHost_SubObj(); \
					assert(OBJ);			      \
					STREAM >> static_cast		      \
					<NsapCandidateHost_SubObj &>(*(OBJ)); \
					break;				      \
				}					      \
				default:				      \
				throw(std::string("Invalid subObj subType")); \
			}						      \
			break;						      \
		}							      \
		case TYPE_FILE_SYSTEM_RES: {				      \
			switch (SUBTYPE) {				      \
				case SUBTYPE_FILE_SYSTEM_RES: {		      \
					OBJ = new FsResources_SubObj();	      \
					assert(OBJ);			      \
					STREAM >> static_cast		      \
					<FsResources_SubObj &>(*(OBJ));	      \
					break;				      \
				}					      \
				default:				      \
				throw(std::string("Invalid subObj subType")); \
			}						      \
			break;						      \
		}							      \
		case TYPE_SYSTEM_CAPS: {				      \
			switch (SUBTYPE) {				      \
				case SUBTYPE_SYSTEM_CAPS: {		      \
					OBJ = new SystemCaps_SubObj();	      \
					assert(OBJ);			      \
					STREAM >> static_cast		      \
					<SystemCaps_SubObj &>(*(OBJ));	      \
					break;				      \
				}					      \
				default:				      \
				throw(std::string("Invalid subObj subType")); \
			}						      \
			break;						      \
		}							      \
		case TYPE_INDIV_CPU_SPEED: {				      \
			switch (SUBTYPE) {				      \
				case SUBTYPE_INDIV_CPU_SPEED: {		      \
					OBJ = new IndvCpuSpeed_SubObj();      \
					assert(OBJ);			      \
					STREAM >> static_cast		      \
					<IndvCpuSpeed_SubObj &>(*(OBJ));      \
					break;				      \
				}					      \
				default:				      \
				throw(std::string("Invalid subObj subType")); \
			}						      \
			break;						      \
		}							      \
		case TYPE_INDIV_CPU_TIME: {				      \
			switch (SUBTYPE) {				      \
				case SUBTYPE_INDIV_CPU_TIME: {		      \
					OBJ = new IndvCpuTime_SubObj();	      \
					assert(OBJ);			      \
					STREAM >> static_cast		      \
					<IndvCpuTime_SubObj &>(*(OBJ));	      \
					break;				      \
				}					      \
				default:				      \
				throw(std::string("Invalid subObj subType")); \
			}						      \
			break;						      \
		}							      \
		case TYPE_INDIV_CPU_COUNT: {				      \
			switch (SUBTYPE) {				      \
				case SUBTYPE_INDIV_CPU_COUNT: {		      \
					OBJ = new IndvCpuCount_SubObj();      \
					assert(OBJ);			      \
					STREAM >> static_cast		      \
					<IndvCpuCount_SubObj &>(*(OBJ));      \
					break;				      \
				}					      \
				default:				      \
				throw(std::string("Invalid subObj subType")); \
			}						      \
			break;						      \
		}							      \
		case TYPE_INDIV_NET_BW: {				      \
			switch (SUBTYPE) {				      \
				case SUBTYPE_INDIV_NET_BW: {		      \
					OBJ = new IndvNetBw_SubObj();	      \
					assert(OBJ);			      \
					STREAM >> static_cast		      \
					<IndvNetBw_SubObj &>(*(OBJ));	      \
					break;				      \
				}					      \
				default:				      \
				throw(std::string("Invalid subObj subType")); \
			}						      \
			break;						      \
		}							      \
		case TYPE_INDIV_PHY_MEM: {				      \
			switch (SUBTYPE) {				      \
				case SUBTYPE_INDIV_PHY_MEM: {		      \
					OBJ = new IndvPhyMem_SubObj();	      \
					assert(OBJ);			      \
					STREAM >> static_cast		      \
					<IndvPhyMem_SubObj &>(*(OBJ));	      \
					break;				      \
				}					      \
				default:				      \
				throw(std::string("Invalid subObj subType")); \
			}						      \
			break;						      \
		}							      \
		case TYPE_INDIV_VIR_MEM: {				      \
			switch (SUBTYPE) {				      \
				case SUBTYPE_INDIV_VIR_MEM: {		      \
					OBJ = new IndvVirMem_SubObj();	      \
					assert(OBJ);			      \
					STREAM >> static_cast		      \
					<IndvVirMem_SubObj &>(*(OBJ));	      \
					break;				      \
				}					      \
				default:				      \
				throw(std::string("Invalid subObj subType")); \
			}						      \
			break;						      \
		}							      \
		case TYPE_INDIV_DISK_SPACE: {				      \
			switch (SUBTYPE) {				      \
				case SUBTYPE_INDIV_DISK_SPACE: {	      \
					OBJ = new IndvDiskSpace_SubObj();     \
					assert(OBJ);			      \
					STREAM >> static_cast		      \
					<IndvDiskSpace_SubObj &>(*(OBJ));     \
					break;				      \
				}					      \
				default:				      \
				throw(std::string("Invalid subObj subType")); \
			}						      \
			break;						      \
		}							      \
		case TYPE_TOTAL_CPU_TIME: {				      \
			switch (SUBTYPE) {				      \
				case SUBTYPE_TOTAL_CPU_TIME: {		      \
					OBJ = new TotCpuTime_SubObj();	      \
					assert(OBJ);			      \
					STREAM >> static_cast		      \
					<TotCpuTime_SubObj &>(*(OBJ));	      \
					break;				      \
				}					      \
				default:				      \
				throw(std::string("Invalid subObj subType")); \
			}						      \
			break;						      \
		}							      \
		case TYPE_TOTAL_CPU_COUNT: {				      \
			switch (SUBTYPE) {				      \
				case SUBTYPE_TOTAL_CPU_COUNT: {		      \
					OBJ = new TotCpuCount_SubObj();	      \
					assert(OBJ);			      \
					STREAM >> static_cast		      \
					<TotCpuCount_SubObj &>(*(OBJ));	      \
					break;				      \
				}					      \
				default:				      \
				throw(std::string("Invalid subObj subType")); \
			}						      \
			break;						      \
		}							      \
		case TYPE_TOTAL_PHY_MEM: {				      \
			switch (SUBTYPE) {				      \
				case SUBTYPE_TOTAL_PHY_MEM: {		      \
					OBJ = new TotPhyMem_SubObj();	      \
					assert(OBJ);			      \
					STREAM >> static_cast		      \
					<TotPhyMem_SubObj &>(*(OBJ));	      \
					break;				      \
				}					      \
				default:				      \
				throw(std::string("Invalid subObj subType")); \
			}						      \
			break;						      \
		}							      \
		case TYPE_TOTAL_VIR_MEM: {				      \
			switch (SUBTYPE) {				      \
				case SUBTYPE_TOTAL_VIR_MEM: {		      \
					OBJ = new TotVirMem_SubObj();	      \
					assert(OBJ);			      \
					STREAM >> static_cast		      \
					<TotVirMem_SubObj &>(*(OBJ));	      \
					break;				      \
				}					      \
				default:				      \
				throw(std::string("Invalid subObj subType")); \
			}						      \
			break;						      \
		}							      \
		case TYPE_TOTAL_DISK_SPACE: {				      \
			switch (SUBTYPE) {				      \
				case SUBTYPE_TOTAL_DISK_SPACE: {	      \
					OBJ = new TotDiskSpace_SubObj();      \
					assert(OBJ);			      \
					STREAM >> static_cast		      \
					<TotDiskSpace_SubObj &>(*(OBJ));      \
					break;				      \
				}					      \
				default:				      \
				throw(std::string("Invalid subObj subType")); \
			}						      \
			break;						      \
		}							      \
		case TYPE_TOTAL_RES_COUNT: {				      \
			switch (SUBTYPE) {				      \
				case SUBTYPE_TOTAL_RES_COUNT: {		      \
					OBJ = new TotResCount_SubObj();	      \
					assert(OBJ);			      \
					STREAM >> static_cast		      \
					<TotResCount_SubObj &>(*(OBJ));	      \
					break;				      \
				}					      \
				default:				      \
				throw(std::string("Invalid subObj subType")); \
			}						      \
			break;						      \
		}							      \
		case TYPE_DATA_STAGING: {				      \
			switch (SUBTYPE) {				      \
				case SUBTYPE_DATA_STAGING: {		      \
					OBJ = new DataStaging_SubObj();	      \
					assert(OBJ);			      \
					STREAM >> static_cast		      \
					<DataStaging_SubObj &>(*(OBJ));	      \
					break;				      \
				}					      \
				default:				      \
				throw(std::string("Invalid subObj subType")); \
			}						      \
			break;						      \
		}							      \
		default:						      \
			throw(std::string("Invalid subObj type"));	      \
	}								      \
}


//
// Objects
//
#define Session_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)			     \
{									     \
	switch (CTYPE) {						     \
		case CTYPE_SESSION_IPV4: {				     \
			OBJ = new IPv4Session_Object();			     \
			assert(OBJ);					     \
			STREAM >> static_cast<IPv4Session_Object &>(*(OBJ)); \
			break;						     \
		}							     \
		case CTYPE_SESSION_IPV6: {				     \
			OBJ = new IPv6Session_Object();			     \
			assert(OBJ);					     \
			STREAM >> static_cast<IPv6Session_Object &>(*(OBJ)); \
			break;						     \
		}							     \
		case CTYPE_SESSION_LSP_TUNNEL_IPV4: {			     \
			OBJ = new LSPTunnelIPv4Session_Object();	     \
			assert(OBJ);					     \
			STREAM >> static_cast				     \
				<LSPTunnelIPv4Session_Object &>(*(OBJ));     \
			break;						     \
		}							     \
		case CTYPE_SESSION_LSP_TUNNEL_IPV6: {			     \
			OBJ = new LSPTunnelIPv6Session_Object();	     \
			assert(OBJ);					     \
			STREAM >> static_cast				     \
				<LSPTunnelIPv6Session_Object &>(*(OBJ));     \
			break;						     \
		}							     \
		case CTYPE_SESSION_UNI_IPV4: {				     \
			OBJ = new UNIIPv4Session_Object();		     \
			assert(OBJ);					     \
			STREAM >> static_cast				     \
				<UNIIPv4Session_Object &>(*(OBJ));	     \
			break;						     \
		}							     \
		default:						     \
			throw(std::string("Invalid obj cType"));	     \
	}								     \
}

#define Label_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)			   \
{									   \
	switch (CTYPE) {						   \
		case CTYPE_LABEL: {					   \
			OBJ = new LabelMpls_Object();			   \
			assert(OBJ);					   \
			STREAM >> static_cast<LabelMpls_Object &>(*(OBJ)); \
			break;						   \
		}							   \
		case CTYPE_GENERALISED_LABEL: {				   \
			OBJ = new GLabel_Object();			   \
			assert(OBJ);					   \
			STREAM >> static_cast<GLabel_Object &>(*(OBJ));	   \
			break;						   \
		}							   \
		case CTYPE_WBS_LABEL: {					   \
			OBJ = new WBSLabel_Object();			   \
			assert(OBJ);					   \
			STREAM >> static_cast<WBSLabel_Object &>(*(OBJ));  \
			break;						   \
		}							   \
		default:						   \
			throw(std::string("Invalid obj cType"));	   \
	}								   \
}

#ifdef CRANKBACK_ON
#define LSPAttr_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)			  \
{									  \
	switch (CTYPE) {						  \
		case CTYPE_LSP_ATTRIBUTES: {		                  \
			OBJ = new LSPAttr_Object();	                  \
			assert(OBJ);					  \
			STREAM >> static_cast<LSPAttr_Object &>(*(OBJ));  \
			break;						  \
		}                                                         \
		default:						  \
			throw(std::string("Invalid obj cType"));	  \
	}								  \
}
#endif // CRANKBACK_ON

#define Session_Attr_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)			  \
{									  \
	switch (CTYPE) {						  \
		case CTYPE_SESSION_ATTRIBUTE_LSP_TUNNEL: {		  \
			OBJ = new LSPTunnelSessionAttr_Object();	  \
			assert(OBJ);					  \
			STREAM >> static_cast				  \
				<LSPTunnelSessionAttr_Object &>(*(OBJ));  \
			break;						  \
		}							  \
		case CTYPE_SESSION_ATTRIBUTE_LSP_TUNNEL_RA: {		  \
			OBJ = new LSPTunnelRASessionAttr_Object();	  \
			assert(OBJ);					  \
			STREAM >> static_cast				  \
			       <LSPTunnelRASessionAttr_Object &>(*(OBJ)); \
			break;						  \
		}							  \
		default:						  \
			throw(std::string("Invalid obj cType"));	  \
	}								  \
}

#define Rsvp_Hop_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)			      \
{									      \
	switch (CTYPE) {						      \
		case CTYPE_RSVP_HOP_IPV4: {				      \
			OBJ = new IPv4RsvpHop_Object();			      \
			assert(OBJ);					      \
			STREAM >> static_cast<IPv4RsvpHop_Object &>(*(OBJ));  \
			break;						      \
		}							      \
		case CTYPE_RSVP_HOP_IPV6: {				      \
			OBJ = new IPv6RsvpHop_Object();			      \
			assert(OBJ);					      \
			STREAM >> static_cast<IPv6RsvpHop_Object &>(*(OBJ));  \
			break;						      \
		}							      \
		case CTYPE_RSVP_HOP_IF_ID_IPV4: {			      \
			OBJ = new IFIDIPv4RsvpHop_Object();		      \
			assert(OBJ);					      \
			STREAM >> static_cast				      \
				<IFIDIPv4RsvpHop_Object &>(*(OBJ));	      \
			break;						      \
		}							      \
		case CTYPE_RSVP_HOP_IF_ID_IPV6: {			      \
			OBJ = new IFIDIPv6RsvpHop_Object();		      \
			assert(OBJ);					      \
			STREAM >> static_cast				      \
				<IFIDIPv6RsvpHop_Object &>(*(OBJ));	      \
			break;						      \
		}							      \
		default:						      \
			throw(std::string("Invalid obj cType"));	      \
	}								      \
}

#define TimeValues_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)			    \
{									    \
	switch (CTYPE) {						    \
		case CTYPE_TIME_VALUES: {				    \
			OBJ = new TimeValues_Object();			    \
			assert(OBJ);					    \
			STREAM >> static_cast<TimeValues_Object &>(*(OBJ)); \
			break;						    \
		}							    \
		default:						    \
			throw(std::string("Invalid obj cType"));	    \
	}								    \
}

#define Error_Spec_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)			      \
{									      \
	switch (CTYPE) {						      \
		case CTYPE_ERROR_SPEC_IPV4: {				      \
			OBJ = new IPv4ErrorSpec_Object();		      \
			assert(OBJ);					      \
			STREAM >> static_cast<IPv4ErrorSpec_Object &>(*(OBJ));\
			break;						      \
		}							      \
		case CTYPE_ERROR_SPEC_IPV6: {				      \
			OBJ = new IPv6ErrorSpec_Object();		      \
			assert(OBJ);					      \
			STREAM >> static_cast<IPv6ErrorSpec_Object &>(*(OBJ));\
			break;						      \
		}							      \
		case CTYPE_ERROR_SPEC_IF_ID_IPV4: {			      \
			OBJ = new IFIDIPv4ErrorSpec_Object();		      \
			assert(OBJ);					      \
			STREAM >> static_cast				      \
				<IFIDIPv4ErrorSpec_Object &>(*(OBJ));	      \
			break;						      \
		}							      \
		case CTYPE_ERROR_SPEC_IF_ID_IPV6: {			      \
			OBJ = new IFIDIPv6ErrorSpec_Object();		      \
			assert(OBJ);					      \
			STREAM >> static_cast				      \
				<IFIDIPv6ErrorSpec_Object &>(*(OBJ));	      \
			break;						      \
		}							      \
		default:						      \
			throw(std::string("Invalid obj cType"));	      \
	}								      \
}

#define Scope_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)			   \
{									   \
	switch (CTYPE) {						   \
		case CTYPE_IPV4_SCOPE: {				   \
			OBJ = new IPv4Scope_Object();			   \
			assert(OBJ);					   \
			STREAM >> static_cast<IPv4Scope_Object &>(*(OBJ)); \
			break;						   \
		}							   \
		case CTYPE_IPV6_SCOPE: {				   \
			OBJ = new IPv6Scope_Object();			   \
			assert(OBJ);					   \
			STREAM >> static_cast<IPv6Scope_Object &>(*(OBJ)); \
			break;						   \
		}							   \
		default:						   \
			throw(std::string("Invalid obj cType"));	   \
	}								   \
}

#define FlowSpec_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)			\
{									\
	switch (CTYPE) {						\
		case CTYPE_INT_SERV_FLOWSPEC: {				\
			OBJ = new IntServFlowSpec_Object();		\
			assert(OBJ);					\
			STREAM >> static_cast				\
				<IntServFlowSpec_Object &>(*(OBJ));	\
			break;						\
		}							\
		default:						\
			throw(std::string("Invalid obj cType"));	\
	}								\
}

#define MessageId_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)			 \
{									 \
	switch (CTYPE) {						 \
		case CTYPE_MESSAGE_ID: {				 \
			OBJ = new MessageId_Object();			 \
			assert(OBJ);					 \
			STREAM >> static_cast				 \
				<MessageId_Object &>(*(OBJ));		 \
			break;						 \
		}							 \
		default:						 \
			throw(std::string("Invalid obj cType"));	 \
	}								 \
}

#define MessageIdAckNack_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)		 \
{									 \
	switch (CTYPE) {						 \
		case CTYPE_MESSAGE_ID_ACK: {				 \
			OBJ = new MessageIdAck_Object();		 \
			assert(OBJ);					 \
			STREAM >> static_cast				 \
				<MessageIdAck_Object &>(*(OBJ));	 \
			break;						 \
		}							 \
		case CTYPE_MESSAGE_ID_NACK: {				 \
			OBJ = new MessageIdNack_Object();		 \
			assert(OBJ);					 \
			STREAM >> static_cast				 \
				<MessageIdNack_Object &>(*(OBJ));	 \
			break;						 \
		}							 \
		default:						 \
			throw(std::string("Invalid obj cType"));	 \
	}								 \
}

#define MessageIdList_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)

#define Integrity_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)			   \
{									   \
	switch (CTYPE) {						   \
		case CTYPE_INTEGRITY: {					   \
			OBJ = new Integrity_Object();			   \
			assert(OBJ);					   \
			STREAM >> static_cast<Integrity_Object &>(*(OBJ)); \
			break;						   \
		}							   \
		default:						   \
			throw(std::string("Invalid obj cType"));	   \
	}								   \
}

#define ExplicitRoute_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)		      \
{									      \
	switch (CTYPE) {						      \
		case CTYPE_ERO: {					      \
			OBJ = new ExplicitRoute_Object();		      \
			assert(OBJ);					      \
			STREAM >> static_cast<ExplicitRoute_Object &>(*(OBJ));\
			break;						      \
		}							      \
		default:						      \
			throw(std::string("Invalid obj cType"));	      \
	}								      \
}

#define LabelRequest_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)			      \
{									      \
	switch (CTYPE) {						      \
		case CTYPE_GENERALISED_LABEL_REQ: {			      \
			OBJ = new GLabelRequest_Object();		      \
			assert(OBJ);					      \
			STREAM >> static_cast<GLabelRequest_Object &>(*(OBJ));\
			break;						      \
		}							      \
		default:						      \
			throw(std::string("Invalid obj cType"));	      \
	}								      \
}

#define CallOptions_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)			      \
{									      \
	switch (CTYPE) {						      \
		case CTYPE_CALL_OPS: {					      \
			OBJ = new CallOptions_Object();			      \
			assert(OBJ);					      \
			STREAM >> static_cast<CallOptions_Object &>(*(OBJ));  \
			break;						      \
		}							      \
		default:						      \
			throw(std::string("Invalid obj cType"));	      \
	}								      \
}

#define CallId_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)			      \
{									      \
	switch (CTYPE) {						      \
		case CTYPE_CALL_ID_OPERATOR_SPECIFIC: {			      \
			OBJ = new CallIdOpSpec_Object();		      \
			assert(OBJ);					      \
			STREAM >> static_cast<CallIdOpSpec_Object &>(*(OBJ)); \
			break;						      \
		}							      \
		case CTYPE_CALL_ID_GLOBALLY_UNIQUE: {			      \
			OBJ = new CallIdGlobal_Object();		      \
			assert(OBJ);					      \
			STREAM >> static_cast<CallIdGlobal_Object &>(*(OBJ)); \
			break;						      \
		}                                                             \
		case CTYPE_CALL_ID_NULL: {			              \
			OBJ = new CallIdNull_Object();		              \
			assert(OBJ);					      \
			STREAM >> static_cast<CallIdNull_Object &>(*(OBJ));   \
			break;						      \
		}							      \
		default:						      \
			throw(std::string("Invalid obj cType"));	      \
	}								      \
}

#define AdminStatus_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)			     \
{									     \
	switch (CTYPE) {						     \
		case CTYPE_ADMIN_STATUS: {				     \
			OBJ = new AdminStatus_Object();			     \
			assert(OBJ);					     \
			STREAM >> static_cast<AdminStatus_Object &>(*(OBJ)); \
			break;						     \
		}							     \
		default:						     \
			throw(std::string("Invalid obj cType"));	     \
	}								     \
}

#define Protection_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)			    \
{									    \
	switch (CTYPE) {						    \
		case CTYPE_PROTECTION: {				    \
			OBJ = new Protection_Object();			    \
			assert(OBJ);					    \
			STREAM >> static_cast<Protection_Object &>(*(OBJ)); \
			break;						    \
		}							    \
		default:						    \
			throw(std::string("Invalid obj cType"));	    \
	}								    \
}

#define Style_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)			\
{									\
	switch (CTYPE) {						\
		case CTYPE_STYLE: {					\
			OBJ = new Style_Object();			\
			assert(OBJ);					\
			STREAM >> static_cast<Style_Object &>(*(OBJ));	\
			break;						\
		}							\
		default:						\
			throw(std::string("Invalid obj cType"));	\
	}								\
}

#define FilterSpec_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)			\
{									\
	switch (CTYPE) {						\
		case CTYPE_FLOW_LABEL_FILTER_SPEC: {			\
			OBJ = new IPV6FlowFilterSpec_Object();		\
			assert(OBJ);					\
			STREAM >> static_cast				\
				<IPV6FlowFilterSpec_Object &>(*(OBJ));	\
			break;						\
		}							\
		case CTYPE_IPV4_FILTER_SPEC: {				\
			OBJ = new IPV4FilterSpec_Object();		\
			assert(OBJ);					\
			STREAM >> static_cast				\
				<IPV4FilterSpec_Object &>(*(OBJ));	\
			break;						\
		}							\
		case CTYPE_IPV6_FILTER_SPEC: {				\
			OBJ = new IPV6FilterSpec_Object();		\
			assert(OBJ);					\
			STREAM >> static_cast				\
				<IPV6FilterSpec_Object &>(*(OBJ));	\
			break;						\
		}							\
		case CTYPE_LSP_IPV4_FILTER_SPEC: {			\
			OBJ = new IPV4LSPFilterSpec_Object();		\
			assert(OBJ);					\
			STREAM >> static_cast				\
				<IPV4LSPFilterSpec_Object &>(*(OBJ));	\
			break;						\
		}							\
		case CTYPE_LSP_IPV6_FILTER_SPEC: {			\
			OBJ = new IPV6LSPFilterSpec_Object();		\
			assert(OBJ);					\
			STREAM >> static_cast				\
				<IPV6LSPFilterSpec_Object &>(*(OBJ));	\
			break;						\
		}							\
		default:						\
			throw(std::string("Invalid obj cType"));	\
	}								\
}

#define Policy_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)			 \
{									 \
	switch (CTYPE) {						 \
		case  CTYPE_POLICY: {					 \
			OBJ = new Policy_Object();			 \
			assert(OBJ);					 \
			STREAM >> static_cast <Policy_Object &>(*(OBJ)); \
			break;						 \
		}							 \
		default:						 \
			throw(std::string("Invalid obj cType"));	 \
	}								 \
}

#define SenderTemplate_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)		   \
{									   \
	switch (CTYPE) {						   \
		case CTYPE_FLOW_LABEL_SENDER_TEMPLATE: {		   \
			OBJ = new IPV6FlowSenderTemplate_Object();	   \
			assert(OBJ);					   \
			STREAM >> static_cast				   \
				<IPV6FlowSenderTemplate_Object &>(*(OBJ)); \
			break;						   \
		}							   \
		case CTYPE_IPV4_SENDER_TEMPLATE: {			   \
			OBJ = new IPV4SenderTemplate_Object();		   \
			assert(OBJ);					   \
			STREAM >> static_cast				   \
				<IPV4SenderTemplate_Object &>(*(OBJ));	   \
			break;						   \
		}							   \
		case CTYPE_IPV6_SENDER_TEMPLATE: {			   \
			OBJ = new IPV6SenderTemplate_Object();		   \
			assert(OBJ);					   \
			STREAM >> static_cast				   \
				<IPV6SenderTemplate_Object &>(*(OBJ));	   \
			break;						   \
		}							   \
		case CTYPE_LSP_IPV4_SENDER_TEMPLATE: {			   \
			OBJ = new IPV4LSPSenderTemplate_Object();	   \
			assert(OBJ);					   \
			STREAM >> static_cast				   \
				<IPV4LSPSenderTemplate_Object &>(*(OBJ));  \
			break;						   \
		}							   \
		case CTYPE_LSP_IPV6_SENDER_TEMPLATE: {			   \
			OBJ = new IPV6LSPSenderTemplate_Object();	   \
			assert(OBJ);					   \
			STREAM >> static_cast				   \
				<IPV6LSPSenderTemplate_Object &>(*(OBJ));  \
			break;						   \
		}							   \
		default:						   \
			throw(std::string("Invalid obj cType"));	   \
	}								   \
}

#define NotifyRequest_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)		      \
{									      \
	switch (CTYPE) {						      \
		case CTYPE_IPV4_NOTIFY_REQUEST: {			      \
			OBJ = new IPV4NotifyRequest_Object();		      \
			assert(OBJ);					      \
			STREAM >> static_cast				      \
				<IPV4NotifyRequest_Object &>(*(OBJ));	      \
			break;						      \
		}							      \
		case CTYPE_IPV6_NOTIFY_REQUEST: {			      \
			OBJ = new IPV6NotifyRequest_Object();		      \
			assert(OBJ);					      \
			STREAM >> static_cast				      \
				<IPV6NotifyRequest_Object &>(*(OBJ));	      \
			break;						      \
		}							      \
		default:						      \
			throw(std::string("Invalid obj cType"));	      \
	}								      \
}


#define LabelSet_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)			  \
{									  \
	switch (CTYPE) {						  \
		case CTYPE_LABEL_SET: {					  \
			OBJ = new LabelSet_Object();			  \
			assert(OBJ);					  \
			STREAM >> static_cast<LabelSet_Object &>(*(OBJ)); \
			break;						  \
		}							  \
		default:						  \
			throw(std::string("Invalid obj cType"));	  \
	}								  \
}

#define AcceptableLabelSet_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)		  \
{									  \
	switch (CTYPE) {						  \
		case CTYPE_ACCEPTABLE_LABEL_SET: {			  \
			OBJ = new AcceptableLabelSet_Object();		  \
			assert(OBJ);					  \
			STREAM >> static_cast				  \
				<AcceptableLabelSet_Object &>(*(OBJ));	  \
			break;						  \
		}							  \
		default:						  \
			throw(std::string("Invalid obj cType"));	  \
	}								  \
}

#define SuggestedLabel_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)		\
{									\
	switch (CTYPE) {						\
		case CTYPE_SUGGESTED_GENERALISED_LABEL: {		\
			OBJ = new SuggestedLabel_Object();		\
			assert(OBJ);					\
			STREAM >> static_cast				\
				<SuggestedLabel_Object &>(*(OBJ));	\
			break;						\
		}							\
		default:						\
			throw(std::string("Invalid obj cType"));	\
	}								\
}

#define UpstreamLabel_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)		\
{									\
	switch (CTYPE) {						\
		case CTYPE_UPSTREAM_GENERALISED_LABEL: {		\
			OBJ = new UpstreamLabel_Object();		\
			assert(OBJ);					\
			STREAM >> static_cast				\
				<UpstreamLabel_Object &>(*(OBJ));	\
			break;						\
		}							\
		default:						\
			throw(std::string("Invalid obj cType"));	\
	}								\
}

#define RecoveryLabel_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)		\
{									\
	switch (CTYPE) {						\
		case CTYPE_RECOVERY_GENERALISED_LABEL: {		\
			OBJ = new RecoveryLabel_Object();		\
			assert(OBJ);					\
			STREAM >> static_cast				\
				<RecoveryLabel_Object &>(*(OBJ));	\
			break;						\
		}							\
		default:						\
			throw(std::string("Invalid obj cType"));	\
	}								\
}

#define ADSpec_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)

#define SenderTSpec_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)			     \
{									     \
	switch (CTYPE) {						     \
		case CTYPE_SENDER_TSPEC: {				     \
			OBJ = new SenderTSpec_Object();			     \
			assert(OBJ);					     \
			STREAM >> static_cast<SenderTSpec_Object &>(*(OBJ)); \
			break;						     \
		}							     \
		default:						     \
			throw(std::string("Invalid obj cType"));	     \
	}								     \
}

#define RecordedRoute_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)		      \
{									      \
	switch (CTYPE) {						      \
		case CTYPE_RECORDED_ROUTE: {				      \
			OBJ = new RecordedRoute_Object();		      \
			assert(OBJ);					      \
			STREAM >> static_cast<RecordedRoute_Object &>(*(OBJ));\
			break;						      \
		}							      \
		default:						      \
			throw(std::string("Invalid obj cType"));	      \
	}								      \
}

#define ResvConfirm_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)			\
{									\
	switch (CTYPE) {						\
		case CTYPE_RESVCONF_IPV4: {				\
			OBJ = new ResvConfirmIPv4_Object();		\
			assert(OBJ);					\
			STREAM >> static_cast				\
				<ResvConfirmIPv4_Object &>(*(OBJ));	\
			break;						\
		}							\
		case CTYPE_RESVCONF_IPV6: {				\
			OBJ = new ResvConfirmIPv6_Object();		\
			assert(OBJ);					\
			STREAM >> static_cast				\
				<ResvConfirmIPv6_Object &>(*(OBJ));	\
			break;						\
		}							\
		default:						\
			throw(std::string("Invalid obj cType"));	\
	}								\
}




#define Hello_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)
#define RestartCap_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)

#define GnsCallExt_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)			    \
{									    \
	switch (CTYPE) {						    \
		case CTYPE_GNS_CALL: {					    \
			OBJ = new GnsCallExt_Object();			    \
			assert(OBJ);					    \
			STREAM >> static_cast<GnsCallExt_Object &>(*(OBJ)); \
			break;						    \
		}							    \
		default:						    \
			throw(std::string("Invalid obj cType"));	    \
	}								    \
}

#define GnsUni_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)			\
{									\
	switch (CTYPE) {						\
		case CTYPE_GNS_UNI: {					\
			OBJ = new GnsUni_Object();			\
			assert(OBJ);					\
			STREAM >> static_cast<GnsUni_Object &>(*(OBJ));	\
			break;						\
		}							\
		default:						\
			throw(std::string("Invalid obj cType"));	\
	}								\
}

#define GeneralisedUni_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)		      \
{									      \
	switch (CTYPE) {						      \
		case CTYPE_GENERALIZED_UNI: {				      \
			OBJ = new GeneralisedUni_Object();		      \
			assert(OBJ);					      \
			STREAM >> static_cast<GeneralisedUni_Object&>(*(OBJ));\
			break;						      \
		}							      \
		default:						      \
			throw(std::string("Invalid obj cType"));	      \
	}								      \
}

#define Times_Class_STREAM2OBJ(STREAM,OBJ,CTYPE)			    \
{									    \
	switch (CTYPE) {						    \
		case CTYPE_TIMES: {				    \
			OBJ = new Times_Object();			    \
			assert(OBJ);					    \
			STREAM >> static_cast<Times_Object &>(*(OBJ)); \
			break;						    \
		}							    \
		default:						    \
			throw(std::string("Invalid obj cType"));	    \
	}								    \
}

//
// Utils for Messages
//
#define CAST_MSG2STREAM(STREAM,MSG)					     \
{									     \
	switch (MSG.getType()) {					     \
		case Message::PATH: {					     \
			STREAM << static_cast<const PathMessage &>(MSG);     \
			break;						     \
		}							     \
		case Message::RESV: {					     \
			STREAM << static_cast<const ResvMessage &>(MSG);     \
			break;						     \
		}							     \
		case Message::PATH_ERR: {				     \
			STREAM << static_cast<const PathErrMessage &>(MSG);  \
			break;						     \
		}							     \
		case Message::RESV_ERR: {				     \
			STREAM << static_cast<const ResvErrMessage &>(MSG);  \
			break;						     \
		}							     \
		case Message::PATH_TEAR: {				     \
			STREAM << static_cast<const PathTearMessage &>(MSG); \
			break;						     \
		}							     \
		case Message::RESV_TEAR: {				     \
			STREAM << static_cast<const ResvTearMessage &>(MSG); \
			break;						     \
		}							     \
		case Message::RESV_CONF: {				     \
			STREAM << static_cast<const ResvConfMessage &>(MSG); \
			break;						     \
		}							     \
		case Message::ACK:					     \
		case Message::SREFRESH:					     \
		case Message::HELLO:					     \
		case Message::NOTIFY:					     \
		case Message::RECOVERY_PATH:				     \
			throw(std::string("Msg type ")   +		     \
			      MSGTYPE2STR(MSG.getType()) +		     \
			      std::string(" not supported yet"));	     \
		default:						     \
			throw(std::string("Invalid msg type"));		     \
	}								     \
}

#define CAST_STREAM2MSG(STREAM,MSG,TYPE)				   \
{									   \
	switch (TYPE) {							   \
		case Message::PATH: {					   \
			MSG = new PathMessage();			   \
			assert(MSG);					   \
			STREAM >> static_cast<PathMessage &>(*(MSG));	   \
			break;						   \
		}							   \
		case Message::RESV: {					   \
			MSG = new ResvMessage();			   \
			assert(MSG);					   \
			STREAM >> static_cast<ResvMessage &>(*(MSG));	   \
			break;						   \
		}							   \
		case Message::PATH_ERR: {				   \
			MSG = new PathErrMessage();			   \
			assert(MSG);					   \
			STREAM >> static_cast<PathErrMessage &>(*(MSG));   \
			break;						   \
		}							   \
		case Message::RESV_ERR: {				   \
			MSG = new ResvErrMessage();			   \
			assert(MSG);					   \
			STREAM >> static_cast<ResvErrMessage &>(*(MSG));   \
			break;						   \
		}							   \
		case Message::PATH_TEAR: {				   \
			MSG = new PathTearMessage();			   \
			assert(MSG);					   \
			STREAM >> static_cast<PathTearMessage &>(*(MSG));  \
			break;						   \
		}							   \
		case Message::RESV_TEAR: {				   \
			MSG = new ResvTearMessage();			   \
			assert(MSG);					   \
			STREAM >> static_cast<ResvTearMessage &>(*(MSG));  \
			break;						   \
		}							   \
		case Message::RESV_CONF: {				   \
			MSG = new ResvConfMessage();			   \
			assert(MSG);					   \
			STREAM >> static_cast<ResvConfMessage &>(*(MSG));  \
			break;						   \
		}							   \
		case Message::ACK:					   \
		case Message::SREFRESH:					   \
		case Message::HELLO:					   \
		case Message::NOTIFY:					   \
		case Message::RECOVERY_PATH:				   \
			throw(std::string("Msg type ") + MSGTYPE2STR(TYPE) \
			      + std::string(" not supported yet"));	   \
		default:						   \
			throw(std::string("Invalid msg type"));		   \
	}								   \
}

//
// Utils for Objects
//
#define OBJ2CURSOR(OBJ,CURSOR)					\
	if (OBJ) {						\
		CURSOR = CURSOR.neighbor((OBJ)->length());	\
		CURSOR << *(OBJ);				\
	}

#define OBJ2CURSOR2(OBJ,CURSOR,FATHER_CUR,START)	\
	if (OBJ) {					\
		uint16_t len = (OBJ)->length();		\
		CURSOR = FATHER_CUR;			\
		CURSOR.resize(START, START + len - 1);	\
		CURSOR << *(OBJ);			\
		START = START + len;			\
	}

#define OBJ2STREAM(OBJ,STREAM)			\
	if (OBJ) {				\
		STREAM << *(OBJ);		\
	}

#define ADD_LENGTH(OBJ,LEN)			\
	if (OBJ) { LEN += (OBJ)->length(); }

#define CLONE_OBJ_INTO_MSG(OBJ,MSG)			\
	if (OBJ) { (MSG)->OBJ = (OBJ)->clone(); }

#define CLONE_OBJ_FROM_OBJ(OBJ_SRC,OBJ_DST)		\
	if (OBJ_SRC) { (OBJ_DST) = (OBJ_SRC)->clone(); }

#define DELETE_OBJ(OBJ)				\
	if (OBJ) { delete OBJ; }

#define DELETE_ARR_OBJ(OBJ,SIZE)				\
	if ((OBJ) && ((SIZE) > 1))       { delete [] OBJ; }	\
	else if ((OBJ) && ((SIZE) == 1)) { delete OBJ;    }

#define CHECK_MANDATORY_OBJ(OBJ)					\
	if (!(OBJ) || !(OBJ)->isConsistent()) {				\
		G2RSVPTE_ERR("Wrong %s mandatory object", #OBJ);	\
		return false;						\
	}

#define CHECK_OPTIONAL_OBJ(OBJ)					\
	if ((OBJ) && !(OBJ)->isConsistent()) {			\
		G2RSVPTE_ERR("Wrong %s optional object", #OBJ);	\
		return false;					\
	}

//
// Utils for handling objects
//
#define CHECK_FLOW_DESC(FLOW_DESC_P)					\
{									\
	if (!FLOW_DESC_P) {						\
		throw(std::string("Probably a STYLE obj has not been"	\
				  " found yet ... malformed MSG"));	\
	}								\
}

#define SETOBJ_INTO_FLOW_DESC(OBJECT)					      \
{									      \
	if (!OBJECT) {							      \
		G2RSVPTE_ERR("Empty " #OBJECT " object");		      \
		return false;						      \
	}								      \
									      \
	ff_flow_item_desc_t * tmpDesc = 0;				      \
	if (ff_desc->filters.empty()) {					      \
		tmpDesc = new ff_flow_item_desc_t();			      \
		assert(tmpDesc);					      \
									      \
		/* reset all */						      \
		tmpDesc->flow_spec   = 0;				      \
		tmpDesc->filter_spec = 0;				      \
		tmpDesc->label       = 0;				      \
		tmpDesc->rro         = 0;				      \
									      \
		/* inserto into head ??	*/				      \
		ff_desc->filters.push_front(tmpDesc);			      \
	} else {							      \
		tmpDesc = ff_desc->filters.front();			      \
		assert(tmpDesc);					      \
	}								      \
									      \
	if (tmpDesc->OBJECT) {						      \
		if (tmpDesc->OBJECT != OBJECT) {			      \
			G2RSVPTE_ERR("There is already a " #OBJECT " into FF" \
				     " Desc obj (only 1 elem is supported)"); \
			return false;					      \
		}							      \
		G2RSVPTE_WRN("The same " #OBJECT			      \
			     " is already present into RSVP msg");	      \
		return true;						      \
	}								      \
									      \
	G2RSVPTE_DBG("Set " #OBJECT " into FF FlowDesc obj");		      \
	tmpDesc->OBJECT = OBJECT;					      \
}

#define GETLSPINFO_FROM_SESSION(SESSION, INFO_P)			       \
{									       \
	switch((SESSION)->cType()) {					       \
		case CTYPE_SESSION_LSP_TUNNEL_IPV4:			       \
			LSPTunnelIPv4Session_Object * tmp;		       \
			tmp = static_cast<LSPTunnelIPv4Session_Object *>       \
				((SESSION));				       \
			(INFO_P)->dst_nid = ntohl(tmp->get().ipv4_addr.s_addr);\
			(INFO_P)->tun_id    = tmp->get().tunnelID;	       \
			(INFO_P)->ext_tun_id = tmp->get().extendedTunnelID;    \
			break;						       \
									       \
		case CTYPE_SESSION_IPV4:				       \
		case CTYPE_SESSION_IPV6:				       \
		case CTYPE_SESSION_LSP_TUNNEL_IPV6:			       \
		default:						       \
			(INFO_P)->dst_nid   = 0;			       \
			(INFO_P)->tun_id    = 0;			       \
			(INFO_P)->ext_tun_id = 0;			       \
			G2RSVPTE_ERR("Session type not yet supported!");       \
			break;						       \
	}								       \
}

#define GETLSPINFO_FROM_SENDTEMP(SEND_TEMP, INFO_P)			       \
{									       \
	switch((SEND_TEMP)->cType()) {					       \
		case CTYPE_LSP_IPV4_SENDER_TEMPLATE:			       \
			IPV4LSPSenderTemplate_Object * tmp;		       \
			tmp = static_cast<IPV4LSPSenderTemplate_Object *>      \
				(SEND_TEMP);				       \
			(INFO_P)->src_nid = ntohl(tmp->get().ipv4_addr.s_addr);\
			(INFO_P)->lsp_id  = tmp->get().lsp_id;		       \
			break;						       \
									       \
		case CTYPE_IPV4_SENDER_TEMPLATE:			       \
		case CTYPE_IPV6_SENDER_TEMPLATE:			       \
		case CTYPE_LSP_IPV6_SENDER_TEMPLATE:			       \
		default:						       \
			(INFO_P)->src_nid   = 0;			       \
			(INFO_P)->lsp_id    = 0;			       \
			G2RSVPTE_ERR("Sender Template type "		       \
				     "not yet supported!");		       \
			break;						       \
	}								       \
}

#define GETLSPINFO_FROM_FILTERSPEC(FILTER_SPEC, INFO_P)			       \
{									       \
	switch((FILTER_SPEC)->cType()) {				       \
		case CTYPE_LSP_IPV4_FILTER_SPEC:			       \
			IPV4LSPFilterSpec_Object * tmp;			       \
			tmp = static_cast<IPV4LSPFilterSpec_Object *>	       \
				(FILTER_SPEC);				       \
			(INFO_P)->src_nid = ntohl(tmp->get().ipv4_addr.s_addr);\
			(INFO_P)->lsp_id  = tmp->get().lsp_id;		       \
			break;						       \
									       \
		case CTYPE_FLOW_LABEL_FILTER_SPEC:			       \
		case CTYPE_IPV4_FILTER_SPEC:				       \
		case CTYPE_IPV6_FILTER_SPEC:				       \
		case CTYPE_LSP_IPV6_FILTER_SPEC:			       \
		default:						       \
			(INFO_P)->src_nid   = 0;			       \
			(INFO_P)->lsp_id    = 0;			       \
			G2RSVPTE_ERR("Filter Spec type not yet supported!");   \
			break;						       \
	}								       \
}

#endif // G2RSVPTE_MESSAGE_UTILS_H
