/*
 *  This file is part of phosphorus-g2mpls.
 *
 *  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Authors:
 *
 *  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
 *  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
 *  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
 *  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
 *  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
 */

#ifndef G2RSVPTE_PERSISTENCY_H
#define G2RSVPTE_PERSISTENCY_H

#include <assert.h>

#include <stdint.h>
#include <list>

#include "g2mpls_types.h"
#include "g2mpls_addr.h"

namespace RSVP {
	class LSP;
}

typedef struct {
	uint32_t iGnsTna:1;
	uint32_t eGnsTna:1;
	uint32_t authz:1;
} storedLspDataMask_t;

typedef struct {
	uint32_t      nodeId;
	g2mpls_addr_t locTelId;
	g2mpls_addr_t locDlId;
	g2mpls_addr_t remDlId;
	uint32_t      labelId;
} storedLspStateBlockInfo_t;

class PersistencyMngr {

 public:

	PersistencyMngr(void) {};
	virtual ~PersistencyMngr(void) {};

	virtual bool initDatabase(adj_type_t adjType)  = 0;
	virtual bool closeDatabase(adj_type_t adjType) = 0;
	virtual bool flushDatabase(adj_type_t adjType) = 0;
	virtual bool storeLsp(RSVP::LSP * lsp) = 0;
	virtual bool unstoreLsp(RSVP::LSP * lsp) = 0;
	virtual bool fetchLsp(const lsp_ident_t           ident,
			      adj_type_t                  adjType,
			      lsp_ident_t  &              nextIdent,
			      lsp_info_t &                lspInfo,
			      storedLspStateBlockInfo_t & psbInfo,
			      storedLspStateBlockInfo_t & rsbInfo,
			      net_res_spec_t &            iTnaRes,
			      net_res_spec_t &            eTnaRes,
			      call_ident_t &              callId,
			      call_info_t &               callInfo,
			      storedLspDataMask_t &       mask,
			      grid_res_spec_t &           iGnsTna,
			      grid_res_spec_t &           eGnsTna,
			      std::string &               authz,
			      std::list<lsp_ero_sobj_t> & ero,
			      std::string &               resp) = 0;

 protected:

 private:

};


#endif // G2RSVPTE_PERSISTENCY_H
