//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//



#include "g2rsvpted/g2rsvpte_rc_api.h"
#include "g2rsvpted/g2rsvpte_lsp.h"
#include "g2rsvpted/g2rsvpte_dm.h"
#include "g2rsvpted/g2rsvpted.h"

#include "lib/corba.h"
#include "g2mpls_corba_utils.h"
#include "g2rsvpte_corba.h"

#include <stdexcept>

bool postLspPathEvent(RSVP::LSP * lsp, bool enqueueEvent)
{
	assert(lsp);
	if (!lsp->getInstance()) {
		G2RSVPTE_DBG("Null instance pointer");
	}

#ifdef G2RSVPTE2RC_DEBUG

	G2RSVPTE_DBG("Posting PathEvent fake event to RC ... debugging");

#else
	if (!lsp->getInstance()->isCore() && lsp->isTail()) {

		G2RSVPTE_DBG("Posting PathEvent event to RC");

                try {
			bool                       res = false;
			g2mplsTypes::lspIdent_var  lspIdent;
			lsp_ident_t                lspIdentTmp;
			g2mplsTypes::callIdent_var callId;
			call_ident_t               callIdTmp;
			g2mplsTypes::lspParams_var lspInfo;
			lsp_info_t                 lspInfoTmp;

			//get lsp ident
			lspIdentTmp   = lsp->getIdent();
			lspIdent     << lspIdentTmp;

			//get call id
			callIdTmp = lsp->callId();
			callId   << callIdTmp;

			//get lsp parameters
			memset(&lspInfoTmp, 0, sizeof(lspInfoTmp));
			if (!lsp->getInfo(lspInfoTmp)) {
				G2RSVPTE_ERR("Impossible to retrieve "
					     "LSP info parameters");
			}
			lspInfo << lspInfoTmp;

			res = rc_proxy->notifyLspNew(lspIdent,
						     callId,
						     lspInfo);
		} catch (Types::InternalProblems) {
			G2RSVPTE_ERR("Posting PathEvent event to RC failed: "
				     "LSP already present in RC");
			return false;
		} catch (Types::CannotFetch) {
			G2RSVPTE_ERR("Posting PathEvent event to RC failed: "
				     "RC cannot associate LSP to a "
				     "recovery bundle");
			return false;
		} catch (std::string e) {
			G2RSVPTE_ERR("Posting PathEvent event to RC failed: %s",
				     e.c_str());
			return false;
		} catch (std::out_of_range & e) {
                        G2RSVPTE_ERR("Posting PathEvent event to RC failed: %s",
				     e.what());
                        return false;
		} catch (...) {
			G2RSVPTE_ERR("Posting PathEvent event to RC failed: "
				     "UNKNOWN REASON");
			return false;
		}
	}

	G2RSVPTE_DBG("PathEvent posted");
#endif // G2RSVPTE2RC_DEBUG

	// LSP MUST call a sendResv automatically
	if (lsp->isTail()) {
		G2RSVPTE_DBG("Posting sendResv Event ... (AUTO-POST)");
		return lsp->sendResv(enqueueEvent);
	}

	G2RSVPTE_DBG("PathEvent ... without action");

	return true;
}

bool postLspResvEvent(RSVP::LSP * lsp, bool enqueueEvent)
{
	assert(lsp);

#ifdef G2RSVPTE2RC_DEBUG

	G2RSVPTE_DBG("Posting ResvEvent fake event to RC ... debugging");

#else
        if (!lsp->getInstance()->isCore() && lsp->isHead()) {

		G2RSVPTE_DBG("Posting ResvEvent to RC");

                try {
			bool                                          res = false;
			RecoveryController::SouthBound::eventInfo_var evInfo;
			g2mplsTypes::lspIdent_var                     lspIdent;
			lsp_ident_t                                   lspIdentTmp;
			evInfo = new RecoveryController::SouthBound::eventInfo;

			//get lsp ident
			lspIdentTmp = lsp->getIdent();

			lspIdent     << lspIdentTmp;
			evInfo->event = RecoveryController::SouthBound::
				LSPEVENT_READY;

			res = rc_proxy->notifyLspEvent(lspIdent,
						       evInfo);
		} catch (Types::CannotFetch) {
			G2RSVPTE_ERR("Posting ResvEvent to RC failed: "
				     "LSP doesn't exist in RC");
			return false;
		} catch (std::string e) {
			G2RSVPTE_ERR("Posting ResvEvent to RC failed: %s",
				     e.c_str());
			return false;
		} catch (...) {
			G2RSVPTE_ERR("Posting ResvEvent to RC failed: "
				     "UNKNOWN REASON");
			return false;
		}
	}

	G2RSVPTE_DBG("ResvEvent posted");
#endif // G2RSVPTE2RC_DEBUG

	// LSP call a sendConfirm automatically
	if (lsp->isHead()) {
		G2RSVPTE_DBG("Posting sendConfirm Event ... (AUTO-POST)");
		return lsp->sendConfirm(enqueueEvent);
	}

	G2RSVPTE_DBG("ResvEvent ... without action");
	return true;
}

bool postLspResvConfEvent(RSVP::LSP * lsp, bool enqueueEvent)
{
	assert(lsp);

#ifdef G2RSVPTE2RC_DEBUG

	G2RSVPTE_DBG("Posting ResvConfEvent fake event to RC ... debugging");
	return true;
#else
	bool res = false;

        if (!lsp->getInstance()->isCore() && lsp->isTail()) {

		G2RSVPTE_DBG("Posting ResvConfEvent to RC");

                try {
			RecoveryController::SouthBound::eventInfo_var evInfo;
			g2mplsTypes::lspIdent_var                     lspIdent;
			lsp_ident_t                                   lspIdentTmp;

			evInfo = new RecoveryController::SouthBound::eventInfo;

			//get lsp ident
			lspIdentTmp = lsp->getIdent();

			lspIdent     << lspIdentTmp;
			evInfo->event = RecoveryController::SouthBound::
				LSPEVENT_READY;

			res = rc_proxy->notifyLspEvent(lspIdent,
						       evInfo);
		} catch (Types::CannotFetch) {
			G2RSVPTE_ERR("Posting ResvConfEvent to RC failed: "
				     "LSP doesn't exist in RC");
			return false;
		} catch (std::string e) {
			G2RSVPTE_ERR("Posting ResvConfEvent to RC failed: %s",
				     e.c_str());
			return false;
		} catch (...) {
			G2RSVPTE_ERR("Posting ResvConfEvent to RC failed: "
				     "UNKNOWN REASON");
			return false;
		}
	}

	G2RSVPTE_DBG("ResvConfEvent posted");

	return res;

#endif // G2RSVPTE2RC_DEBUG
}

bool postLspNotifyEvent(RSVP::LSP * lsp, bool enqueueEvent)
{
	assert(lsp);
	bool res = false;

#ifdef G2RSVPTE2RC_DEBUG
	G2RSVPTE_DBG("Posting NotifyEvent fake event to RC ... debugging");
	res = true;
#else
	//FIXME TODO add connection with RC -> INFORM RC about this event
#endif
	return res;
}

bool postLspPathDownEvent(RSVP::LSP * lsp, bool enqueueEvent)
{
	assert(lsp);

	bool res = false;

#ifdef G2RSVPTE2RC_DEBUG

	G2RSVPTE_DBG("Posting PathDown fake event to RC ... debugging");
#else

        if (!lsp->getInstance()->isCore() && lsp->isTail()) {

		G2RSVPTE_DBG("Posting PathDown event to RC");

                try {
			RecoveryController::SouthBound::eventInfo_var evInfo;
			g2mplsTypes::lspIdent_var                     lspIdent;
			lsp_ident_t                                   lspIdentTmp;

			evInfo = new RecoveryController::SouthBound::eventInfo;

			//get lsp ident
			lspIdentTmp = lsp->getIdent();

			lspIdent     << lspIdentTmp;
			evInfo->event = RecoveryController::SouthBound::
				LSPEVENT_GOINGDOWN;

			res = rc_proxy->notifyLspEvent(lspIdent,
						       evInfo);
		} catch (Types::CannotFetch) {
			G2RSVPTE_ERR("Posting PathDown event to RC failed: "
				     "LSP doesn't exist in RC");
			return false;
		} catch (std::string e) {
			G2RSVPTE_ERR("Posting PathDown event to RC failed: %s",
				     e.c_str());
			return false;
		} catch (...) {
			G2RSVPTE_ERR("Posting PathDown event to RC failed: "
				     "UNKNOWN REASON");
			return false;
		}
	}

	G2RSVPTE_DBG("PathDown event posted");
#endif // G2RSVPTE2RC_DEBUG

        if (lsp->isTail()) {
		G2RSVPTE_DBG("Posting sendDown Event ... (AUTO-POST)");
		return lsp->sendDown(enqueueEvent);
	}

	return res;
}

bool postLspResvDownEvent(RSVP::LSP * lsp, bool enqueueEvent)
{
	assert(lsp);

	bool res = false;

#ifdef G2RSVPTE2RC_DEBUG

	G2RSVPTE_DBG("Posting ResvDown fake event to RC ... debugging");
#else
        if (!lsp->getInstance()->isCore() && lsp->isHead()) {

		G2RSVPTE_DBG("Posting ResvDown event to RC");

                try {
			RecoveryController::SouthBound::eventInfo_var evInfo;
			g2mplsTypes::lspIdent_var                     lspIdent;
			lsp_ident_t                                   lspIdentTmp;

			evInfo = new RecoveryController::SouthBound::eventInfo;

			//get lsp ident
			lspIdentTmp = lsp->getIdent();

			lspIdent     << lspIdentTmp;
			evInfo->event = RecoveryController::SouthBound::
				LSPEVENT_GOINGDOWN;

			res = rc_proxy->notifyLspEvent(lspIdent,
						       evInfo);
		} catch (Types::CannotFetch) {
			G2RSVPTE_ERR("Posting ResvDown event to RC failed: "
				     "LSP doesn't exist in RC");
			return false;
		} catch (std::string e) {
			G2RSVPTE_ERR("Posting ResvDown event to RC failed: %s",
				     e.c_str());
			return false;
		} catch (...) {
			G2RSVPTE_ERR("Posting ResvDown event to RC failed: "
				     "UNKNOWN REASON");
			return false;
		}
	}

	G2RSVPTE_DBG("ResvDown event posted");
#endif // G2RSVPTE2RC_DEBUG

	G2RSVPTE_DBG("Posting sendTear Event ... (AUTO-POST)");

        if (lsp->isHead()) {
		return lsp->sendTear(enqueueEvent);
	}

	return res;
}

bool postLspErrEvent(RSVP::LSP * lsp, bool enqueueEvent)
{
	assert(lsp);
	bool res = false;

#ifdef G2RSVPTE2RC_DEBUG
	G2RSVPTE_DBG("Posting ErrEvent fake event to RC ... debugging");
	res = true;
#else
	if (!lsp->getInstance()->isCore()) {

		G2RSVPTE_DBG("Posting ErrEvent to RC");

		try {
			RecoveryController::SouthBound::eventInfo_var evInfo;
			g2mplsTypes::lspIdent_var                     lspIdent;
			lsp_ident_t                                   lspIdentTmp;

			evInfo = new RecoveryController::SouthBound::eventInfo;

			//get lsp ident
			lspIdentTmp = lsp->getIdent();

			lspIdent     << lspIdentTmp;
			evInfo->event = RecoveryController::SouthBound::
				LSPEVENT_SIGERROR;

			res = rc_proxy->notifyLspEvent(lspIdent,
						       evInfo);

		} catch (Types::CannotFetch) {
			G2RSVPTE_ERR("Posting ErrEvent to RC failed: "
				     "LSP doesn't exist in RC");
			return false;
		} catch (std::string e) {
			G2RSVPTE_ERR("Posting ErrEvent to RC failed: %s",
				     e.c_str());
			return false;
		} catch (...) {
			G2RSVPTE_ERR("Posting ErrEvent to RC failed: "
				     "UNKNOWN REASON");
			return false;
		}
	}

	G2RSVPTE_DBG("ErrEvent posted");
#endif
	return res;
}

bool postLspTearEvent(RSVP::LSP * lsp, bool post, bool enqueueEvent)
{
	assert(lsp);
	bool res = false;

	if (post) {
		G2RSVPTE_DBG("Posting sendTear Event ... (AUTO-POST)");
		res = lsp->sendTear(enqueueEvent);
	}

#ifdef G2RSVPTE2RC_DEBUG
	G2RSVPTE_DBG("Posting TearEvent fake event to RC ... debugging");
	res = true;
#else
	//FIXME TODO add connection with RC -> INFORM RC this LSP has gone DOWN
#endif
	//-> this LSP calls a postLspDeletion automatically (into FSM trans.)
	return res;
}

///////////////////////////////////////////////////////////////////////////////
struct delInfo {
	RSVP::G2RSVPTE * g2rsvpte;
	lsp_ident_t      ident;
};

int lsp_deletion(struct thread *t)
{
	bool             res;
	struct delInfo * info;

	info = (struct delInfo *)THREAD_ARG(t);
	assert(info);
	assert(info->g2rsvpte);

	res = info->g2rsvpte->destroyLSP(info->ident); //internal = true!!!
	//TODO: if (!res)  schedule lsp_deletion again???

	if (info->g2rsvpte->adjType() == INNI) {
#ifndef G2RSVPTE2RC_DEBUG
		try {
			g2mplsTypes::lspIdent_var  lspIdent;
			lsp_ident_t                lspIdentTmp;

			G2RSVPTE_DBG("Posting LSP deletion event to RC");

			//get lsp ident
			lspIdentTmp   = info->ident;
			lspIdent     << lspIdentTmp;

			res = rc_proxy->notifyLspDeleted(lspIdent);

		} catch (Types::CannotFetch) {
			G2RSVPTE_ERR("Posting event to RC failed: "
				     "RC cannot associate LSP to a "
				     "recovery bundle");
			return false;
		} catch (std::string e) {
			G2RSVPTE_ERR("Posting event to RC failed: %s",
				     e.c_str());
			return false;
		} catch (std::out_of_range & e) {
			G2RSVPTE_ERR("Posting event to RC failed: %s",
				     e.what());
			return false;
		} catch (...) {
			G2RSVPTE_ERR("Posting event to RC failed: "
				     "UNKNOWN REASON");
			return false;
		}

		G2RSVPTE_DBG("Event posted");
#endif
	}

	delete info;

	return 0;
}
///////////////////////////////////////////////////////////////////////////////

bool postLspDeletion(lsp_ident_t lspIdent, adj_type_t adjType, bool enqueueEvent)
{
	bool res = false;


	G2RSVPTE_DBG("Calling lspDestroy Event ... (ADD_EVENT: lsp_deletion)");
	struct delInfo * info;
	RSVP::G2RSVPTE * proto;
	info = new delInfo();
	assert(info);
	proto = RSVP_MASTER.getProtoInstance(adjType);
	if (!proto) {
	  G2RSVPTE_WRN("Cannot destroy LSP: instance not found");
	  return false;
	}
	info->g2rsvpte = proto;
	info->ident    = lspIdent;

	// !!! No thread_add_event because it has a priority greater than
	// !!! thraed_add_write function ... so if a thread_add_write has been
	// !!! already added we should not respect the order!
	thread_add_event(RSVP_MASTER.getMaster(), lsp_deletion, info, 0);
	//thread_add_write(RSVP_MASTER.getMaster(), lsp_deletion, info, 0);

#ifdef G2RSVPTE2RC_DEBUG
	G2RSVPTE_DBG("Posting deletion fake event to RC ... debugging");
	res = true;
#endif

	return res;
}
