/*
 *  This file is part of phosphorus-g2mpls.
 *
 *  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Authors:
 *
 *  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
 *  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
 *  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
 *  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
 *  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
 */



#ifndef G2RSVPTE_UTILS_H
#define G2RSVPTE_UTILS_H

#include "g2rsvpted/g2rsvpte_config.h"
#include "log.h"
#include "zebra.h"
#include "zassert.h"

#include <iostream>
#include <list>
#include "g2mpls_addr.h"
#include "g2mpls_types.h"
#include "idl/g2mplsTypes.hh"

#ifdef VERBOSE_DEBUG
#define METIN() {					\
	zlog_debug("Entering %s", __PRETTY_FUNCTION__);	\
}


#define METOUT() {					\
	zlog_debug("Exiting %s",  __PRETTY_FUNCTION__);	\
}
#else
#define METIN()
#define METOUT()
#endif // VERBOSE_DEBUG

#define HBEAT()					\
	zlog_debug("[HBEAT]> %s:%d (%s)\n",	\
		   __FILE__,			\
		   __LINE__,			\
		   __PRETTY_FUNCTION__)

#define BUG()					\
	zlog_err("Bug hit in %s:%d (%s)\n",	\
		 __FILE__,			\
		 __LINE__,			\
		 __PRETTY_FUNCTION__);

//
// Operator utils
//

namespace RSVP {
	extern inline std::ostream & operator << (std::ostream &  os,
						  const uint8_t & data)	{
		os << (uint16_t)data;
		return os;
	}

	extern inline std::ostream & operator << (std::ostream & os,
						  const ipv4_t & addr) {
		//os << numb_htoa(ntohl(addr.s_addr));
		os << inet_ntoa(addr);
		return os;
	}

	extern inline std::ostream & operator << (std::ostream & os,
						  const ipv6_t & addr) {
		throw("IPv6 address not supported yet");
	}
}

//
// host2net/net2host utils
//
uint64_t htonll(uint64_t val);
uint64_t ntohll(uint64_t val);


//
// Timer utils
//
#define RAND_MAX_INT   2147483647

uint32_t generateRefreshInterval(uint32_t interval);

//
// ERO utils
//

namespace RSVP {
	class EroSubObject;

	bool fill_ero(uint32_t                    src,
		      uint32_t                    dst,
		      std::list<EroSubObject *> & subObjs);
}
#if HAVE_OMNIORB
g2mplsTypes::eroItem_var createNullEroItem(void);
g2mplsTypes::eroItem getEroItem(const lsp_ero_sobj_t & sObj);
void eroItemProcess(const g2mplsTypes::eroItem_var & eroItem,
		    std::list<lsp_ero_sobj_t> &      ero);
void linkId2eroSubObj(const g2mplsTypes::nodeId & nodeId,
		      const g2mplsTypes::linkId & linkId,
		      bool                        loose,
		      lsp_ero_sobj_t &            eroSubObj);
void label2eroSubObj(const g2mplsTypes::labelId & labelId,
		     bool                         upstream,
		     bool                         loose,
		     lsp_ero_sobj_t &             eroSubObj);
bool eroSubobjIsConsistent(const lsp_ero_sobj_t & eroSubObj);
g2mplsTypes::eroSeq_var getExclEroSeq(std::list<lsp_ero_sobj_t> exclEro);
#endif

#endif // G2RSVPTE_UTILS_H
