//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 University of Essex.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Eduard Escalona       (Univ. of Essex)   <eescal_at_essex.ac.uk>
//



#include "GridResourceRegistryBindingServer.hh"
#include "g2mpls_types.h"

int
GRR_thread()
{
    GridResourceRegistryBindingService grr;
    
    GUNIGW_DBG("Starting GRR Service...");
    printf("Starting GRR Service...\n");
    if (grr.run(8888))
    {
        GUNIGW_ERR("Error starting GRR process");
        grr.soap_stream_fault(std::cerr);
        exit(-1);
    }
    
    GUNIGW_DBG("Exiting GRR_thread...");
    
    return GRR_OK;

}

int
init_GRR()
{
    pthread_t tid;
                
    if (!pthread_create(&tid, NULL, (void*(*)(void*))GRR_thread, NULL))
    {
        GUNIGW_ERR("Error creating GRR thread");
        return GRR_ERR;
    }
    
    return GRR_OK;
}

int
GridResourceRegistryBindingService::publishOperation(glue__SiteType *PO,
        grr2__BoolType *POR)
{ 
    struct soap soap;
    GUNIGW_DBG("GRR Call: publishOperation (IN)");
    
    soap_init(&soap);
    
    std::string     err;
     
#if HAVE_OMNIORB
     
    g2mplsTopology::Info_var    ospfMgmtProxy;

    ospfMgmtProxy = GUNIGW_MASTER.getospfProxy();
 
    if (CORBA::is_nil(ospfMgmtProxy)){
        GUNIGW_ERR("Cannot get Corba client object "
            "previously set up");
	assert(0);
    }
   
    GUNIGW_DBG("Site->Name: %s",PO->Name);
    GUNIGW_DBG("Site->UniqueID: %s",PO->UniqueID);
    //GUNIGW_DBG("Site->Cluster->Name: %s",PO->Cluster[0]->Name);
    GUNIGW_DBG("Site->Cluster->UniqueID: %s",PO->Cluster[0]->UniqueID);
    GUNIGW_DBG("Site->Cluster->ComputingElement->UniqueID: %s",PO->Cluster[0]->ComputingElement[0]->UniqueID);
    GUNIGW_DBG("Site->Cluster->ComputingElement->GRAMVersion: %s",PO->Cluster[0]->ComputingElement[0]->GRAMVersion);
    GUNIGW_DBG("Site->Cluster->ComputingElement->HostName: %s",PO->Cluster[0]->ComputingElement[0]->HostName);
    GUNIGW_DBG("Site->Cluster->ComputingElement->GateKeeperPort: %s",PO->Cluster[0]->ComputingElement[0]->GateKeeperPort);
    GUNIGW_DBG("Site->Cluster->ComputingElement->FreeJobSlots: %s",PO->Cluster[0]->ComputingElement[0]->FreeJobSlots);
    GUNIGW_DBG("Site->Cluster->SubCluster->UniqueID: %s",PO->Cluster[0]->SubCluster[0]->UniqueID);
    GUNIGW_DBG("Site->Cluster->SubCluster->PhysicalCPUs: %s",PO->Cluster[0]->SubCluster[0]->PhysicalCPUs);
    // BUG: Problem with gsoap integers. gsoap converts them to char*

    // Update Site
    g2mplsTypes::nodeId nodeId;
    g2mplsTypes::nodeIdent nodeIdent;
    g2mplsTypes::gridSiteParams siteInfo;
    nodeId = htonl((uint32_t) atoi (PO->UniqueID));
    GUNIGW_MASTER.APserverId((uint32_t) atoi (PO->UniqueID));
    nodeIdent.id = htonl((uint32_t) atoi (PO->UniqueID));
    nodeIdent.type = g2mplsTypes::NODETYPE_GRID;
    siteInfo.name = (::CORBA::String_member) PO->Name;
    /*struct in_addr address_ip4;
    inet_aton ("10.20.30.40", &address_ip4);
    siteInfo.peRouterId = address_ip4.s_addr;*/
    try {
        ospfMgmtProxy->nodeAdd(nodeIdent);
    } catch(Types::InternalProblems & e) {
        err += "inodeAdd through CORBA Mgmt ";
	err += "interface failed for internal problems (reason= ";
	err += e.what;
	err += ")";
        GUNIGW_ERR("%s", err.c_str());
	return false;
    } catch (Types::CannotFetch & e) {
        err += "nodeAdd through CORBA Mgmt ";
        err += "interface failed for fetching problems (reason= ";
        err += e.what;
        err += ")";
        GUNIGW_ERR("%s", err.c_str());
        return false;
    } catch (...) {
        err += "nodeAdd through CORBA Mgmt ";
        err += "interface failed for unknown reason";
        GUNIGW_ERR("%s", err.c_str());
        //return false;
	int grr_err = 1;
    }

    GUNIGW_DBG("Sent nodeAdd through CORBA Mgmt interface");

    // Update Computing Element
    g2mplsTypes::gridSubNodeId subNodeId;
    subNodeId = htonl((uint32_t) (atoi) (PO->Cluster[0]->ComputingElement[0]->UniqueID));
    // GUNIGW_DBG("ceInfo.lrmsInfo");
    g2mplsTypes::gridCEParams ceInfo;
    ceInfo.lrmsInfo.type = g2mplsTypes::GRIDLRMSTYPE_UNKNOWN;
    uint32_t value = 0; uint32_t temp = 0;
    uint16_t lrmsVersion = 0x1083;
    value = (uint32_t) lrmsVersion;
    temp = value>>12;
    temp &= 0xf;
    ceInfo.lrmsInfo.mjrRev = temp;
    temp = value>>6;
    temp &= 0x3f;
    ceInfo.lrmsInfo.mnrRev = temp;
    temp = value;
    temp &= 0x3f;
    ceInfo.lrmsInfo.bldFix = temp;
    ceInfo.lrmsInfo.valid = false;
    // GUNIGW_DBG("ceInfo.hostAddr");
    struct in_addr *address_ip4;
    hostent *HostAddress;
    HostAddress = gethostbyname(PO->Cluster[0]->ComputingElement[0]->HostName); 
    address_ip4 = (in_addr *) HostAddress->h_addr;
    g2mpls_addr_t addr;
    addr.type = IPv4;
    addr.value.ipv4.s_addr = address_ip4->s_addr;
    g2mplsTypes::gridHostId_var host;
    host << addr;
    ceInfo.hostAddr = host;
    // GUNIGW_DBG("ceInfo.gatekeeperPort");
    ceInfo.gatekeeperPort =  (uint32_t) (atoi) (PO->Cluster[0]->ComputingElement[0]->GateKeeperPort);
    // GUNIGW_DBG("ceInfo.jobManager");
    ceInfo.jobManager = "jobManager";
    // GUNIGW_DBG("ceInfo.dataDir");
    ceInfo.dataDir = "dataDirectory";
    // GUNIGW_DBG("ceInfo.defaultStorageElemId");
    ceInfo.defaultStorageElemId = 0;
    // GUNIGW_DBG("ceInfo.jobsState");
    ceInfo.jobsState.freeJobSlots = (uint32_t) atoi (PO->Cluster[0]->ComputingElement[0]->FreeJobSlots);
    ceInfo.jobsState.state = g2mplsTypes::GRIDCESESTATE_DRAINING;
    ceInfo.jobsState.valid = true;
    // GUNIGW_DBG("ceInfo.jobsStats");
    ceInfo.jobsStats.runningJobs = 0;
    ceInfo.jobsStats.waitingJobs = 0;
    ceInfo.jobsStats.totalJobs = 0;
    ceInfo.jobsStats.valid = false;
    // GUNIGW_DBG("ceInfo.jobsTimePerf");
    ceInfo.jobsTimePerf.estimatedResponseTime = 0;
    ceInfo.jobsTimePerf.worstResponseTime = 0;
    ceInfo.jobsTimePerf.valid = false;
    // GUNIGW_DBG("ceInfo.jobsTimePolicy");
    ceInfo.jobsTimePolicy.maxWallclockTime = 0;
    ceInfo.jobsTimePolicy.maxObtainableWallclockTime = 0;
    ceInfo.jobsTimePolicy.maxCpuTime = 0;
    ceInfo.jobsTimePolicy.maxObtainableCpuTime = 0;
    ceInfo.jobsTimePolicy.valid = false;
    // GUNIGW_DBG("ceInfo.jobsLoadPolicy");
    ceInfo.jobsLoadPolicy.maxTotalJobs = 0;
    ceInfo.jobsLoadPolicy.maxRunningJobs = 0;
    ceInfo.jobsLoadPolicy.maxWaitingJobs = 0;
    ceInfo.jobsLoadPolicy.assignedJobSlots = 0;
    ceInfo.jobsLoadPolicy.maxSlotsPerJob = 0;
    char priorityPreemptionFlag = 8;
    uint8_t tempo = priorityPreemptionFlag;
    tempo >>= 1; tempo &= 0x7f;
    ceInfo.jobsLoadPolicy.priority = tempo;
    tempo = priorityPreemptionFlag & 0x1;
    ceInfo.jobsLoadPolicy.preemptionFlag = tempo;
    ceInfo.jobsLoadPolicy.valid = false;
    // GUNIGW_DBG("ceInfo.freeJobSlotsCalendar()");
   try {
        ospfMgmtProxy->gridCompElemUpdate(nodeId, subNodeId, ceInfo);
    } catch(Types::InternalProblems & e) {
        err += "gridCompElemUpdate through CORBA Mgmt ";
	err += "interface failed for internal problems (reason= ";
	err += e.what;
	err += ")";
        GUNIGW_ERR("%s", err.c_str());
	return false;
    } catch (Types::CannotFetch & e) {
        err += "gridCompElemUpdate through CORBA Mgmt ";
        err += "interface failed for fetching problems (reason= ";
        err += e.what;
        err += ")";
        GUNIGW_ERR("%s", err.c_str());
        return false;
    } catch (...) {
        err += "gridCompElemUpdate through CORBA Mgmt ";
        err += "interface failed for unknown reason";
        GUNIGW_ERR("%s", err.c_str());
        return false;
    }

    GUNIGW_DBG("Sent gridCompElemUpdate through CORBA Mgmt interface");

    /*
    // Update SubCluster
    subNodeId = (uint32_t) atoi (PO->Cluster[0]->UniqueID);
    g2mplsTypes::gridSubClusterParams subClusterInfo;
    // GUNIGW_DBG("subClusterInfo.cpu");
    subClusterInfo.cpu.cpuCounts.physical = (uint32_t) atoi (PO->Cluster[0]->SubCluster[0]->PhysicalCPUs);
    subClusterInfo.cpu.cpuCounts.logical = 0;
    subClusterInfo.cpu.cpuArch = g2mplsTypes::GRIDCPUARCH_OTHER;
    // GUNIGW_DBG("subClusterInfo.os");
    subClusterInfo.os.mjrRev = 0;
    subClusterInfo.os.mnrRev = 0;
    subClusterInfo.os.bldFix = 0;
    subClusterInfo.os.valid = false;
    // GUNIGW_DBG("subClusterInfo.memory");
    subClusterInfo.memory.ramSize = 0;
    subClusterInfo.memory.virtualMemorySize = 0;
    subClusterInfo.memory.valid = false;
    // GUNIGW_DBG("subClusterInfo.subClusterCalendar()");
    try {
        ospfMgmtProxy->gridSubClusterUpdate(nodeId, subNodeId, subClusterInfo);
    } catch(Types::InternalProblems & e) {
        err += "gridSubClusterUpdate through CORBA Mgmt ";
	err += "interface failed for internal problems (reason= ";
	err += e.what;
	err += ")";
        GUNIGW_ERR("%s", err.c_str());
	return false;
    } catch (Types::CannotFetch & e) {
        err += "gridSubClusterUpdate through CORBA Mgmt ";
        err += "interface failed for fetching problems (reason= ";
        err += e.what;
        err += ")";
        GUNIGW_ERR("%s", err.c_str());
        return false;
    } catch (...) {
        err += "gridSubClusterUpdate through CORBA Mgmt ";
        err += "interface failed for unknown reason";
        GUNIGW_ERR("%s", err.c_str());
        return false;
    }

    GUNIGW_DBG("Sent gridSubClusterUpdate through CORBA Mgmt interface");
    */
#endif // HAVE_OMNIORB

    POR->boolean = true;
    
    GUNIGW_DBG("GRR Call: publishOperation (OUT)");
    
    return SOAP_OK;
}

int
GridResourceRegistryBindingService::getResourcesOperation(grr2__SiteDocumentArrayType *GRO)
{
    struct soap soap;

    GUNIGW_DBG("GRR Call: getResourcesOperation (IN)");

    soap_init(&soap);

    GUNIGW_DBG("GRR Call: getResourcesOperation (OUT)");

    return SOAP_OK;
}

