//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//  Copyright (C) 2006, 2007, 2008, 2009 University of Essex.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//  Eduard Escalona       (Univ. of Essex)   <eescal_at_essex.ac.uk>
//



#include "zebra.h"

#include "lib/corba.h"
#include "g2mpls_corba_utils.h"
#include "log.h"
#include "gunigw_trace.hh"
#include "gunigw_corba.hh"
#include "../idl/lrm.hh"
#include "../idl/CallController.hh"
#include "../idl/RecoveryController.hh"
#include "../idl/g2mplsTopology.hh"
#include "../idl/lrm.hh"
#include <iostream>
#include <string>
#include <stdlib.h>

#if HAVE_OMNIORB

const char server[] = "localhost:8080";
//const char server[] = "http://packcs-e0.scai.fraunhofer.de/phosphorus-wp2/services/BESFactoryService";
BESFactoryBindingProxy bes;
bes__CreateActivityType *CA = new bes__CreateActivityType;
bes__CreateActivityResponseType *CAR = new bes__CreateActivityResponseType;

using namespace std;

class GUNIGW_i : public POA_CallController::EastWest,
                 public PortableServer::RefCountServantBase
{
    public:
    inline GUNIGW_i();
    virtual ~GUNIGW_i();

    CORBA::Boolean
    callCreate(g2mplsTypes::callIdent&,
               const g2mplsTypes::callParams&,
               const g2mplsTypes::recoveryParams&,
               const g2mplsTypes::lspParams&,
               const g2mplsTypes::aaaParams&,
               const g2mplsTypes::actorInfo&);

    CORBA::Boolean
    callSetTna(const g2mplsTypes::callIdent&,
               g2mplsTypes::resourcePosition,
               const g2mplsTypes::tnaResource&,
               const g2mplsTypes::actorInfo&);

    CORBA::Boolean
    callSetGnsTna(const g2mplsTypes::callIdent&,
                  g2mplsTypes::resourcePosition,
                  const g2mplsTypes::gridParams&,
                  const g2mplsTypes::actorInfo&);

    CORBA::Boolean
    callDestroy(const g2mplsTypes::callIdent&,
                const g2mplsTypes::actorInfo&);

    CORBA::Boolean
    callRaiseMsgEvent(const g2mplsTypes::callIdent& id,
                      g2mplsTypes::callMessageType msgType,
                      const char* reason,
                      const g2mplsTypes::actorInfo& actor);
};

GUNIGW_i::GUNIGW_i()
{
}

GUNIGW_i::~GUNIGW_i()
{
}

GUNIGW_i*     servant      = 0;

CORBA::Boolean
GUNIGW_i::callCreate(g2mplsTypes::callIdent& id,
           const g2mplsTypes::callParams& callPar,
           const g2mplsTypes::recoveryParams& recPar,
           const g2mplsTypes::lspParams& lspPar,
           const g2mplsTypes::aaaParams& aaaPar,
           const g2mplsTypes::actorInfo& actor)
{
    STACK_LOCK();
    GUNIGW_DBG("Received callCreate through "
               "CORBA EW interface");
    STACK_UNLOCK();
}

CORBA::Boolean
GUNIGW_i::callSetTna(const g2mplsTypes::callIdent& id,
           g2mplsTypes::resourcePosition resPos,
           const g2mplsTypes::tnaResource& tnaRes,
           const g2mplsTypes::actorInfo& actor)
{
    STACK_LOCK();
    GUNIGW_DBG("Received callSetTna through "
               "CORBA EW interface");
    STACK_UNLOCK();
}

CORBA::Boolean
GUNIGW_i::callSetGnsTna(const g2mplsTypes::callIdent& id,
              g2mplsTypes::resourcePosition resPos,
              const g2mplsTypes::gridParams& gridPar,
              const g2mplsTypes::actorInfo& actor)
{
    STACK_LOCK();
    GUNIGW_DBG("Received callSetGnsTna through "
               "CORBA EW interface");
    STACK_UNLOCK();
}

CORBA::Boolean
GUNIGW_i::callDestroy(const g2mplsTypes::callIdent& id,
            const g2mplsTypes::actorInfo& actor)
{
    STACK_LOCK();
    GUNIGW_DBG("Received callDestroy through "
               "CORBA EW interface");
    STACK_UNLOCK();
}

CORBA::Boolean
GUNIGW_i::callRaiseMsgEvent(const g2mplsTypes::callIdent& id,
                           g2mplsTypes::callMessageType msgType,
                           const char* reason,
                           const g2mplsTypes::actorInfo& actor)
{
    STACK_LOCK();

    //struct timeval  timeout;
    //struct timeval  time_tmp;
    g2mplsTypes::callIdent_var idtemp;
    call_ident_t id2;
    
    idtemp = id;
    id2 << idtemp;

    call_ident_t * id3 = new call_ident_t;
    *id3 = id2;

    switch (msgType) {
	case g2mplsTypes::CALLMESSAGETYPE_SETUPREQUEST:

	    GUNIGW_DBG("Received callRaiseMsgEvent(SetupRequest) through "
                       "CORBA EW interface");

	    thread_add_event(GUNIGW_MASTER.getMaster(),
	    		     sendCallSetGnsTna,
			     id3,
			     0);
	    thread_add_event(GUNIGW_MASTER.getMaster(),
	    		     sendSetupIndication,
			     id3,
			     0);
	    //gunigw_lock(true, 1);	
	    //gettimeofday(&time_tmp, NULL);


 	    // Wait for Event
 	    //gettimeofday(&timeout, NULL);
            //while(is_gunigw_locked(1) && (timeout.tv_sec - time_tmp.tv_sec) < 20)
            //{
            //    gettimeofday(&timeout, NULL);
            //}
	    //if (!is_gunigw_locked(1))
            //{
	        // Setup Confirm received
	    //}
	    //else
	    //{
	    //    GUNIGW_DBG("Timeout! (SetupConfirm expected)", err.c_str());
	    //    gunigw_lock(false, 1);
	    //}
	    break;
        case g2mplsTypes::CALLMESSAGETYPE_SETUPINDICATION:
	    //gunigw_lock(false, 0);

	    GUNIGW_DBG("Received callRaiseMsgEvent(SetupIndication) through "
	               "CORBA EW interface");

	    thread_add_event(GUNIGW_MASTER.getMaster(),
	    		     sendSetupConfirm,
			     id3,
			     0);
	    break;
	case g2mplsTypes::CALLMESSAGETYPE_SETUPCONFIRM:
	    //gunigw_lock(false, 1);
            GUNIGW_DBG("Received callRaiseMsgEvent(SetupConfirm) through "
	               "CORBA EW interface");
	    break;
        case g2mplsTypes::CALLMESSAGETYPE_RELEASEREQUEST:
	    //gunigw_lock(false, 2);

	    GUNIGW_DBG("Received callRaiseMsgEvent(ReleaseRequest) through "
	               "CORBA EW interface");

	    thread_add_event(GUNIGW_MASTER.getMaster(),
	    		     sendReleaseIndication,
			     id3,
			     0);
	    break;
        case g2mplsTypes::CALLMESSAGETYPE_RELEASEINDICATION:
	    //gunigw_lock(false, 3);
	    GUNIGW_DBG("Received callRaiseMsgEvent(ReleaseIndication) through "
	               "CORBA EW interface");
	    break;
        case g2mplsTypes::CALLMESSAGETYPE_SIGERROR:
	    GUNIGW_DBG("Received callRaiseMsgEvent(SigError) through "
	               "CORBA EW interface");
	    break;
	default:
	    GUNIGW_DBG("Received unknown message through "
	               "CORBA EW interface");
	    break;
    }

    STACK_UNLOCK();
    return true;
}


bool corba_client_setup(CallController::Mgmt_var & l)
{
	CORBA::ORB_var orb;
	orb = corba_orb();

	if (CORBA::is_nil(orb)) {
		throw string("Cannot get ORB");
		GUNIGW_ERR("Cannot get ORB");
		return false;
	}

	string ior;
	if (!corba_fetch_ior(CORBA_SERVANT_CLIENT_CALLCONTROLLER, ior)) {
		throw string("Cannot fetch IOR");
		GUNIGW_ERR("Cannot fetch IOR");
		return false;
	}

        try {
		CORBA::Object_var obj;
		obj = orb->string_to_object(ior.c_str());
		if (CORBA::is_nil(obj)) {
		    throw string("Cannot get object");
		    GUNIGW_ERR("Cannot get object");
		    return false;
		}

		l = CallController::Mgmt::_narrow(obj);
		if (CORBA::is_nil(l)) {
		    throw string("Cannot invoke on a nil object reference");
		    GUNIGW_ERR("cannot invoke on a nil object reference");
		    return false;
		}

	} catch (CORBA::SystemException & e) {
		GUNIGW_ERR("Caught CORBA::SystemException");
		return false;
	} catch (CORBA::Exception & e) {
		GUNIGW_ERR("Caught CORBA::Exception");
		return false;
	} catch (...) {
		GUNIGW_ERR("CLIENT_SETUP: caught unknown exception");
		return false;
	}

	return true;
}

bool corba_client_setup(CallController::EastWest_var & l)
{
    CORBA::ORB_var orb;
    orb = corba_orb();

    if (CORBA::is_nil(orb)) {
        throw string("Cannot get ORB");
        GUNIGW_ERR("Cannot get ORB");
        return false;
    }

    string ior;
    if (!corba_fetch_ior(CORBA_SERVANT_CLIENT_CALLCONTROLLER_EW, ior)) {
        throw string("Cannot fetch IOR");
        GUNIGW_ERR("Cannot fetch IOR");
        return false;
    }

    try {
        CORBA::Object_var obj;
        obj = orb->string_to_object(ior.c_str());
        if (CORBA::is_nil(obj)) {
            throw string("Cannot get object");
            GUNIGW_ERR("Cannot get object");
            return false;
        }

        l = CallController::EastWest::_narrow(obj);
        if (CORBA::is_nil(l)) {
            throw string("Cannot invoke on a nil object reference");
            GUNIGW_ERR("cannot invoke on a nil object reference");
            return false;
        }

    } catch (CORBA::SystemException & e) {
        GUNIGW_ERR("Caught CORBA::SystemException");
        return false;
    } catch (CORBA::Exception & e) {
        GUNIGW_ERR("Caught CORBA::Exception");
        return false;
    } catch (...) {
        GUNIGW_ERR("CLIENT_SETUP: caught unknown exception");
        return false;
    }

    return true;
}


bool corba_client_setup(g2mplsTopology::Info_var & l)
{
        CORBA::ORB_var orb;
        orb = corba_orb();

        if (CORBA::is_nil(orb)) {
		throw string("Cannot get ORB");
		GUNIGW_ERR("Cannot get ORB");
		return false;
        }

        string ior;
        if (!corba_fetch_ior(CORBA_SERVANT_G2TOPOLOGY_UNI, ior)) {
		throw string("Cannot fetch IOR");
		GUNIGW_ERR("Cannot fetch IOR");
		return false;
        }

        try {
		CORBA::Object_var obj;
		obj = orb->string_to_object(ior.c_str());
		if (CORBA::is_nil(obj)) {
		    throw string("Cannot get object");
		    GUNIGW_ERR("Cannot get object");
		    return false;
		}

		l = g2mplsTopology::Info::_narrow(obj);
		if (CORBA::is_nil(l)) {
		    throw string("Cannot invoke on a nil object reference");
		    GUNIGW_ERR("cannot invoke on a nil object reference");
		    return false;
		}

	} catch (CORBA::SystemException & e) {
		GUNIGW_ERR("Caught CORBA::SystemException");
		return false;
	} catch (CORBA::Exception & e) {
		GUNIGW_ERR("Caught CORBA::Exception");
		return false;
	} catch (...) {
		GUNIGW_ERR("CLIENT_SETUP: caught unknown exception");
		return false;
	}

	return true;
}

bool corba_client_setup(LRM::Info_var & l)
{
    CORBA::ORB_var orb;
    orb = corba_orb();

    if (CORBA::is_nil(orb)) {
        throw string("Cannot get ORB");
        GUNIGW_ERR("Cannot get ORB");
        return false;
    }

    string ior;
    if (!corba_fetch_ior(CORBA_SERVANT_LRM, ior)) {
        throw string("Cannot fetch IOR");
        GUNIGW_ERR("Cannot fetch IOR");
        return false;
    }

    try {
        CORBA::Object_var obj;
    	obj = orb->string_to_object(ior.c_str());
    	if (CORBA::is_nil(obj)) {
		    throw string("Cannot get object");
		    GUNIGW_ERR("Cannot get object");
		    return false;
    	}

	l = LRM::Info::_narrow(obj);
	if (CORBA::is_nil(l)) {
	    throw string("Cannot invoke on a nil object reference");
	    GUNIGW_ERR("cannot invoke on a nil object reference");
	    return false;
	}
    } catch (CORBA::SystemException & e) {
        GUNIGW_ERR("Caught CORBA::SystemException");
        return false;
    } catch (CORBA::Exception & e) {
        GUNIGW_ERR("Caught CORBA::Exception");
        return false;
    } catch (...) {
        GUNIGW_ERR("CLIENT_SETUP: caught unknown exception");
        return false;
    }

    return true;
}

bool corba_client_shutdown(corba_servant_t s)
{
	return 1;
}

bool corba_server_setup(void)
{
    GUNIGW_DBG("Setting up CORBA server side");

    try {
        servant = new GUNIGW_i();
        if (!servant) {
            GUNIGW_ERR("Cannot create servant");
            throw string("Cannot create servant");
        }

        PortableServer::POA_var poa;
        poa = corba_poa();
        if (CORBA::is_nil(poa)) {
            GUNIGW_ERR("Cannot get POA");
            throw string("Cannot get POA");
        }

        PortableServer::ObjectId_var servant_id;
        servant_id = poa->activate_object(servant);

        CORBA::Object_var obj;
        obj = servant->_this();
        if (CORBA::is_nil(obj)) {
            GUNIGW_ERR("Cannot get object");
            throw string("Cannot get object");
        }

        CORBA::ORB_var orb;
        orb = corba_orb();
        if (CORBA::is_nil(orb)) {
            GUNIGW_ERR("Cannot get ORB");
            throw string("Cannot get ORB");
        }

        CORBA::String_var sior(orb->object_to_string(obj));
        zlog_debug("IOR = '%s'", (char*) sior);

        if (!corba_dump_ior(CORBA_SERVANT_GUNIGW, string(sior))) {
            GUNIGW_ERR("Cannot dump IOR");
            throw string("Cannot dump IOR");
        }

        servant->_remove_ref();

        PortableServer::POAManager_var poa_manager;
        poa_manager = corba_poa_manager();
        if (CORBA::is_nil(poa_manager)) {
            GUNIGW_ERR("Cannot get POA Manager");
            throw string("Cannot get POA Manager");
        }

        poa_manager->activate();
    } catch (CORBA::SystemException & e) {
        GUNIGW_ERR("Caught CORBA::SystemException");
        fprintf(stderr, "Caught CORBA::SystemException\n");
        return 0;
    } catch (CORBA::Exception & e) {
        GUNIGW_ERR("Caught CORBA::Exception");
        fprintf(stderr, "Caught CORBA::Exception\n");
        return 0;
    } catch (omniORB::fatalException & e) {
        fprintf(stderr, "Caught omniORB::fatalException:\n");
        fprintf(stderr, "  file: %s\n", e.file());
        fprintf(stderr, "  line: %d\n", e.line());
        fprintf(stderr, "  mesg: %s\n", e.errmsg());
        return 0;
    } catch (string & e) {
        fprintf(stderr, "Caught exception: %s\n", e.c_str());
        return 0;
    } catch (...) {
        fprintf(stderr, "Caught unknown exception\n");
        return 0;
    }
	return 1;
}

bool corba_server_shutdown(void)
{
    try {
        zlog_debug("Shutting down CORBA server side");

        if (!corba_remove_ior(CORBA_SERVANT_GUNIGW)) {
            throw string("Cannot remove IOR");
        }
    } catch (...) {
            zlog_err("Caught unknown exception");
            return 0;
    }
    return 1;
}

int
sendSetupIndication(struct thread *thread)
{
    call_ident_t * id3;
    call_ident_t id2;
 
    g2mplsTypes::callIdent_var id;
    std::string     err;

    CallController::EastWest_var         nccEastWestProxy;
    nccEastWestProxy = GUNIGW_MASTER.getNccEWProxy();

    if (CORBA::is_nil(nccEastWestProxy)){
        GUNIGW_ERR("Cannot get Corba client object "
       		   "previously set up");
        assert(0);
    }

    g2mplsTypes::actorInfo_var actor2;
    actor2 = new g2mplsTypes::actorInfo;
    actor2->issuer       = g2mplsTypes::ISSUERTYPE_UNIGW;
    actor2->forceCommand = false;

    id3 = (call_ident_t *) THREAD_ARG(thread);
    id2 = *id3;
    id << id2;

    try {
        nccEastWestProxy->callRaiseMsgEvent(id,
        g2mplsTypes::CALLMESSAGETYPE_SETUPINDICATION,
            "",
            actor2);
    } catch (Types::InternalProblems & e) {
        err += "callRaiseMsgEvent through CORBA EastWest ";
        err += "interface failed for internal problems ";
        err += "(reason= ";
        err += e.what;
        err += ")";
        GUNIGW_ERR("%s", err.c_str());
        return false;
    } catch (Types::CannotFetch & e) {
        err += "callRaiseMsgEvent through CORBA EastWest ";
        err += "interface failed for fetching problems ";
        err += "(reason= ";
        err += e.what;
        err += ")";
        GUNIGW_ERR("%s", err.c_str());
        return false;
    } catch (...) {
        err += "callRaiseMsgEvent through CORBA EastWest ";
        err += "interface failed for unknown reason";
        GUNIGW_ERR("%s", err.c_str());
        return false;
    }

    GUNIGW_DBG("Sent callRaiseMsgEvent(SetupIndication) through "
               "CORBA EW interface");

    delete id3;
    return 0;
}

int
sendSetupConfirm(struct thread *thread)
{
    call_ident_t * id3;
    call_ident_t id2;
    
    g2mplsTypes::callIdent_var id;
    std::string     err;

    CallController::EastWest_var         nccEastWestProxy;
    nccEastWestProxy = GUNIGW_MASTER.getNccEWProxy();

    if (CORBA::is_nil(nccEastWestProxy)){
        GUNIGW_ERR("Cannot get Corba client object "
       		   "previously set up");
        assert(0);
    }

    g2mplsTypes::actorInfo_var actor2;
    actor2 = new g2mplsTypes::actorInfo;
    actor2->issuer       = g2mplsTypes::ISSUERTYPE_UNIGW;
    actor2->forceCommand = false;

    id3 = (call_ident_t *) THREAD_ARG(thread);
    id2 = *id3;
    id << id2;

    try {
        nccEastWestProxy->callRaiseMsgEvent(id,
        g2mplsTypes::CALLMESSAGETYPE_SETUPCONFIRM,
            "",
            actor2);
    } catch (Types::InternalProblems & e) {
        err += "callRaiseMsgEvent through CORBA EastWest ";
        err += "interface failed for internal problems ";
        err += "(reason= ";
        err += e.what;
        err += ")";
        GUNIGW_ERR("%s", err.c_str());
        return false;
    } catch (Types::CannotFetch & e) {
        err += "callRaiseMsgEvent through CORBA EastWest ";
        err += "interface failed for fetching problems ";
        err += "(reason= ";
        err += e.what;
        err += ")";
        GUNIGW_ERR("%s", err.c_str());
        return false;
    } catch (...) {
        err += "callRaiseMsgEvent through CORBA EastWest ";
        err += "interface failed for unknown reason";
        GUNIGW_ERR("%s", err.c_str());
        return false;
    }

    GUNIGW_DBG("Sent callRaiseMsgEvent(SetupConfirm) through "
               "CORBA EW interface");

    delete id3;
    return 0;
}

int
sendReleaseIndication(struct thread *thread)
{
    call_ident_t * id3;
    call_ident_t id2;
 
    g2mplsTypes::callIdent_var id;
    std::string     err;

    CallController::EastWest_var         nccEastWestProxy;
    nccEastWestProxy = GUNIGW_MASTER.getNccEWProxy();

    if (CORBA::is_nil(nccEastWestProxy)){
        GUNIGW_ERR("Cannot get Corba client object "
       		   "previously set up");
        assert(0);
    }

    g2mplsTypes::actorInfo_var actor2;
    actor2 = new g2mplsTypes::actorInfo;
    actor2->issuer       = g2mplsTypes::ISSUERTYPE_UNIGW;
    actor2->forceCommand = false;

    id3 = (call_ident_t *) THREAD_ARG(thread);
    id2 = *id3;
    id << id2;

    try {
        nccEastWestProxy->callRaiseMsgEvent(id,
        g2mplsTypes::CALLMESSAGETYPE_RELEASEINDICATION,
            "",
            actor2);
    } catch (Types::InternalProblems & e) {
        err += "callRaiseMsgEvent through CORBA EastWest ";
        err += "interface failed for internal problems ";
        err += "(reason= ";
        err += e.what;
        err += ")";
        GUNIGW_ERR("%s", err.c_str());
        return false;
    } catch (Types::CannotFetch & e) {
        err += "callRaiseMsgEvent through CORBA EastWest ";
        err += "interface failed for fetching problems ";
        err += "(reason= ";
        err += e.what;
        err += ")";
        GUNIGW_ERR("%s", err.c_str());
        return false;
    } catch (...) {
        err += "callRaiseMsgEvent through CORBA EastWest ";
        err += "interface failed for unknown reason";
        GUNIGW_ERR("%s", err.c_str());
        return false;
    }

    GUNIGW_DBG("Sent callRaiseMsgEvent(ReleaseIndication) through "
               "CORBA EW interface");
    
    delete id3;
    return 0;
}

int
sendCallSetGnsTna(struct thread *thread)
{
    call_ident_t	*id3;
    call_ident_t	id2;
    grid_res_spec_t     gres;
    call_data_t		*data;
 
    g2mplsTypes::callIdent_var id;
    std::string     err;

    CallController::EastWest_var         nccEastWestProxy;
    nccEastWestProxy = GUNIGW_MASTER.getNccEWProxy();

    if (CORBA::is_nil(nccEastWestProxy)){
        GUNIGW_ERR("Cannot get Corba client object "
       		   "previously set up");
        assert(0);
    }

    g2mplsTypes::actorInfo_var actor2;
    actor2 = new g2mplsTypes::actorInfo;
    actor2->issuer       = g2mplsTypes::ISSUERTYPE_UNIGW;
    actor2->forceCommand = false;

    id3 = (call_ident_t *) THREAD_ARG(thread);
    id3->type = CALLID_OPSPEC;
    id2 = *id3;
    id << id2;

    memset(&data->cdata, 0, sizeof(call_info_t));
    memset(&gres, 0, sizeof(gres));

    BITMASK_BITSET(data->cdata.eTNA_res.mask_, grid);
    BITMASK_BITSET(gres.mask_, tot_cpu_count);
    BITMASK_BITSET(gres.mask_, cand_host);
    gres.tot_cpu_count.lb_included = 1;
    gres.tot_cpu_count.ub_included = 1;
    gres.tot_cpu_count.lower_bound = 10;
    gres.tot_cpu_count.upper_bound = 10;
    BITMASK_BITSET(gres.mask_, cand_host);
    GUNIGW_DBG("EDU: CandHost = %d", GUNIGW_MASTER.APserverId()); 
    gres.cand_host.value.ipv4.s_addr = GUNIGW_MASTER.APserverId();
    memcpy(&(data->cdata.eTNA_res.grid), &gres, sizeof(gres));

    g2mplsTypes::gridParams_var   gnsTnaRes;
    try {
        gnsTnaRes << data->cdata.eTNA_res.grid;
    } catch (std::out_of_range & e) {
        err  = "Exception occurred during ";
        err += "translation of iTNA_res ";
        err += "grid params(";
        err += e.what();
        err += ")";
        return false;
    }


    g2mplsTypes::resourcePosition  pos;
    pos = g2mplsTypes::RESOURCEPOSITION_EGRESS;

    try {
        nccEastWestProxy->callSetGnsTna(id,
					pos,
					gnsTnaRes,
					actor2);
    } catch (Types::InternalProblems & e) {
        err += "callSetGnsTna through CORBA EastWest ";
        err += "interface failed for internal problems ";
        err += "(reason= ";
        err += e.what;
        err += ")";
        GUNIGW_ERR("%s", err.c_str());
        return false;
    } catch (Types::CannotFetch & e) {
        err += "callSetGnsTna through CORBA EastWest ";
        err += "interface failed for fetching problems ";
        err += "(reason= ";
        err += e.what;
        err += ")";
        GUNIGW_ERR("%s", err.c_str());
        return false;
    } catch (...) {
        err += "callSetGnsTna through CORBA EastWest ";
        err += "interface failed for unknown reason";
        GUNIGW_ERR("%s", err.c_str());
        return false;
    }

    GUNIGW_DBG("Sent callSetGnaTna through "
               "CORBA EW interface");
    
    //delete id3;
    return 0;
}
#endif // HAVE_OMNIORB
