//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//  Copyright (C) 2006, 2007, 2008, 2009 University of Essex.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//  Eduard Escalona       (Univ. of Essex)   <eescal_at_essex.ac.uk>
//


#include <config.h>
#include <map>
#include <list>
#include <string>
#include <sstream>
#include <iomanip>
#include <iostream>
#include <stdexcept>

#include "gunigw_utils.hh"

bool GUNIGW_LOCK_SR = 0;
bool GUNIGW_LOCK_SI = 0;

std::map<mapKey_t, call_data_t *>  vtyCallContainer;


#define CALLMAP_FIND(MAP, TYPE, KEY, RES) {		\
	std::map<mapKey_t, TYPE >::iterator itmp;	\
							\
	(RES) = 0;					\
	itmp = (MAP).find(KEY);				\
	if (itmp != (MAP).end()) {			\
		(RES) = (*itmp).second;			\
	}						\
}

#define CALLMAP_COND_CREATE_AND_BIND(MAP, TYPE, KEY, RES, ERR) {	\
	if (!RES) {							\
		(RES) = new (TYPE);					\
		if (!RES) {						\
			(ERR) += "Cannot allocate a VTY CALL container";\
			(MAP).erase(KEY);				\
			return false;					\
		}							\
									\
		memset((RES), 0, sizeof(*(RES)));			\
		(MAP)[KEY] = (RES);					\
	}								\
}
char *
hostsrv_ipv4_addr()
{
    char hostname[255];
    
    gethostname (hostname, sizeof(hostname));
    struct in_addr* address_ipv4;
    struct hostent *HostAddress;
    HostAddress = gethostbyname(hostname); 
    address_ipv4 = (in_addr *) HostAddress->h_addr;
    
    return (inet_ntoa(address_ipv4[0]));
}


std::string key2string(const mapKey_t & key)
{
	std::string        tmp;
	std::ostringstream localId;

	tmp = std::string("");

	localId << "0x" << std::hex
		<< std::setw(16) << std::setfill('0')  << key;

	tmp += localId.str();

	return tmp;
}

static std::string version2string(const version_t & ver)
{
	std::string        tmp;
	std::ostringstream verStr;

	tmp = std::string("");

	verStr << "v" << ver.major_revision
	       << "." << ver.minor_revision
	       << "." << ver.build_fix;

	tmp += verStr.str();

	return tmp;
}

static std::string range2string(const range_set_t & range)
{
	std::string        tmp;
	std::ostringstream rangeStr;

	tmp = std::string("");

	rangeStr << ((range.lb_included == 1) ? "[" : "(")
		 << range.lower_bound
		 << ","
		 << range.upper_bound
		 << ((range.ub_included == 1) ? "]" : ")");

	tmp += rangeStr.str();

	return tmp;
}

#define SET_RANGE(DST, LB, UB, IS_L, IS_U)	\
	DST.lower_bound = LB;			\
	DST.upper_bound = UB;			\
	DST.lb_included = IS_L ? 1 : 0;		\
	DST.ub_included = IS_U ? 1 : 0;

bool
defaultGridResSpec(res_spec_t & res)
{

	res.grid.application.type                   = GRID_APPNAME_KODAVIS;
	res.grid.application.version.major_revision = 1;
	res.grid.application.version.minor_revision = 2;
	res.grid.application.version.build_fix      = 0;

	res.grid.cand_host.type    = IPv4;
	res.grid.cand_host.preflen = 32;
	inet_aton("192.168.1.1", &(res.grid.cand_host.value.ipv4));

	res.grid.fs_resources.fs_name      = FS_NAME_HOME;
	res.grid.fs_resources.fs_type      = FS_TYPE_NORMAL;
	SET_RANGE(res.grid.fs_resources.disk_space, 10, 100, true, false);

	delete res.grid.fs_resources.mount_point;
	res.grid.fs_resources.mount_point =
		new std::string("/dev/remote");
	delete res.grid.fs_resources.mount_source;
	res.grid.fs_resources.mount_source =
		new std::string("nfs://cluster.man.poznan.pl");

	res.grid.sys_caps.os_type                   = OS_LINUX;
	res.grid.sys_caps.os_version.major_revision = 2;
	res.grid.sys_caps.os_version.minor_revision = 6;
	res.grid.sys_caps.os_version.build_fix      = 20;
	res.grid.sys_caps.cpu_arch                  = CPU_X86_64;
	res.grid.sys_caps.exclusive_access          = 1;

	SET_RANGE(res.grid.ind_cpu_speed , 1 ,  100, false, true);
	SET_RANGE(res.grid.ind_cpu_time  , 2 ,  200, true,  true);
	SET_RANGE(res.grid.ind_cpu_count , 3 ,  300, false, true);
	SET_RANGE(res.grid.ind_net_bw    , 4 ,  400, false, true);
	SET_RANGE(res.grid.ind_phy_mem   , 5 ,  500, false, true);
	SET_RANGE(res.grid.ind_vir_mem   , 6 ,  600, false, true);
	SET_RANGE(res.grid.ind_disk_space, 7 ,  700, false, true);
	SET_RANGE(res.grid.tot_cpu_time  , 8 ,  800, false, true);
	SET_RANGE(res.grid.tot_cpu_count , 9 ,  900, false, true);
	SET_RANGE(res.grid.tot_phy_mem   , 10, 1000, false, true);
	SET_RANGE(res.grid.tot_vir_mem   , 11, 1100, false, true);
	SET_RANGE(res.grid.tot_disk_space, 12, 1200, false, true);
	SET_RANGE(res.grid.tot_res_count , 13, 1300, false, true);


	res.grid.data_staging.fs_name            = FS_NAME_TMP;
	res.grid.data_staging.creation_flag      = CF_DONTOVERWRITE;
	res.grid.data_staging.del_on_termination = 0;
	delete res.grid.data_staging.filename;
	res.grid.data_staging.filename =
		new std::string("phosphorus.staging.log");
	delete res.grid.data_staging.source;
	res.grid.data_staging.source =
		new std::string("nfs://cluster.man.poznan.pl");
	delete res.grid.data_staging.target;
	res.grid.data_staging.target =
		new std::string("nfs://storage.man.poznan.pl");

	res.grid.grid_site_id = 0XFACAAAAA;
	BITMASK_BITSET(res.mask_     , grid);         // never unset
	BITMASK_BITSET(res.grid.mask_, grid_site_id); // never unset


	BITMASK_BITSET(res.grid.mask_, application);
	BITMASK_BITSET(res.grid.mask_, cand_host);
	BITMASK_BITSET(res.grid.mask_, fs_resources);
	BITMASK_BITSET(res.grid.mask_, sys_caps);
	BITMASK_BITSET(res.grid.mask_, ind_cpu_speed);
	BITMASK_BITSET(res.grid.mask_, ind_cpu_time);
	BITMASK_BITSET(res.grid.mask_, ind_cpu_count);
	BITMASK_BITSET(res.grid.mask_, ind_net_bw);
	BITMASK_BITSET(res.grid.mask_, ind_phy_mem);
	BITMASK_BITSET(res.grid.mask_, ind_vir_mem);
	BITMASK_BITSET(res.grid.mask_, ind_disk_space);
	BITMASK_BITSET(res.grid.mask_, tot_cpu_time);
	BITMASK_BITSET(res.grid.mask_, tot_cpu_count);
	BITMASK_BITSET(res.grid.mask_, tot_phy_mem);
	BITMASK_BITSET(res.grid.mask_, tot_vir_mem);
	BITMASK_BITSET(res.grid.mask_, tot_disk_space);
	BITMASK_BITSET(res.grid.mask_, tot_res_count);
	BITMASK_BITSET(res.grid.mask_, data_staging);

	return true;
}

bool
vtyCallCommittable(const mapKey_t & callKey)
{
	call_data_t *   data;

	CALLMAP_FIND(vtyCallContainer, call_data_t *, callKey, data);

	return (data &&
		isResSpecValid(data->cdata.iTNA_res) &&
		isResSpecValid(data->cdata.eTNA_res) &&
		(data->ldata.sw_cap   != SWCAP_UNKNOWN) &&
		(data->ldata.enc_type != ENCT_UNKNOWN) &&
		(data->ldata.bw       != BwEnc_Unknown));
}

void
logDump(const call_ident_t & cid)
{
	GUNIGW_DBG("CALL identity [%s]",
		  SHOW_CALL_ID_TYPE(cid.type));
	GUNIGW_DBG("\t - srcAddr             : %s",
		  addr_ntoa(cid.src_addr));
	GUNIGW_DBG("\t - localId             : %s",
		  key2string(cid.local_id).c_str());

	if (cid.type == CALLID_GLOBUNIQ) {
		GUNIGW_DBG("\t - Globally unique attributes");
		GUNIGW_DBG("\t     - ituCountryCode: 0x%06x",
			  cid.itu_country_code);
		GUNIGW_DBG("\t     - ituCarrierCode    : %s",
			  key2string(cid.itu_carrier_code).c_str());
		GUNIGW_DBG("\t     - uniqueAP          : %s",
			  key2string(cid.unique_ap).c_str());
	}
}

void
logDump(const res_spec_t & res)
{
	GUNIGW_DBG("\t   - Network data      : %s present",
		  BITMASK_BITTEST(res.mask_, net) ?
		  "is" : "not");
	GUNIGW_DBG("\t       - tna           : %s",
		  BITMASK_BITTEST(res.net.mask_, tna) ?
		  addr_ntoa(res.net.tna) : "undeclared");
	GUNIGW_DBG("\t       - data link     : %s",
		  BITMASK_BITTEST(res.net.mask_, data_link) ?
		  addr_ntoa(res.net.data_link) : "undeclared");
	GUNIGW_DBG("\t       - label         : %s",
		  BITMASK_BITTEST(res.net.mask_, label) ?
		  label2string(res.net.label).c_str() :
		  "undeclared");

#ifdef G2MPLS
	GUNIGW_DBG("\t   - Grid data         : %s present",
		  BITMASK_BITTEST(res.mask_, grid) ?
		  "is" : "not");
	// APPLICATION
	GUNIGW_DBG("\t       - APP  type     : %s",
		  BITMASK_BITTEST(res.grid.mask_, application) ?
		  SHOW_GRID_APPLICATION(res.grid.application.type) :
		  "undeclared");
	GUNIGW_DBG("\t       - APP  version  : %s",
		  BITMASK_BITTEST(res.grid.mask_, application) ?
		  version2string(res.grid.application.version).c_str() :
		  "undeclared");

	// CAND HOST
	GUNIGW_DBG("\t       - CAND HOST     : %s",
		  BITMASK_BITTEST(res.grid.mask_, cand_host) ?
		  addr_ntoa(res.grid.cand_host) : "undeclared");

	// FS_RESOURCES
	GUNIGW_DBG("\t       - FS name       : %s",
		  BITMASK_BITTEST(res.grid.mask_, fs_resources) ?
		  SHOW_GRID_FS_NAME(res.grid.fs_resources.fs_name) :
		  "undeclared");
	GUNIGW_DBG("\t       - FS type       : %s",
		  BITMASK_BITTEST(res.grid.mask_, fs_resources) ?
		  SHOW_GRID_FS_TYPE(res.grid.fs_resources.fs_type) :
		  "undeclared");
	GUNIGW_DBG("\t       - FS disk space : %s",
		  BITMASK_BITTEST(res.grid.mask_, fs_resources) ?
		  range2string(res.grid.fs_resources.disk_space).c_str() :
		  "undeclared");
	GUNIGW_DBG("\t       - FS mount point: %s",
		  BITMASK_BITTEST(res.grid.mask_, fs_resources) ?
		  (res.grid.fs_resources.mount_point)->c_str() :
		  "undeclared");
	GUNIGW_DBG("\t       - FS mount src  : %s",
		  BITMASK_BITTEST(res.grid.mask_, fs_resources) ?
		  (res.grid.fs_resources.mount_source)->c_str() :
		  "undeclared");

	// SYSTEM CAPS
	GUNIGW_DBG("\t       - SYS OS        : %s",
		  BITMASK_BITTEST(res.grid.mask_, sys_caps) ?
		  SHOW_GRID_OS(res.grid.sys_caps.os_type) :
		  "undeclared");
	GUNIGW_DBG("\t       - SYS OS version: %s",
		  BITMASK_BITTEST(res.grid.mask_, sys_caps) ?
		  version2string(res.grid.sys_caps.os_version).c_str() :
		  "undeclared");
	GUNIGW_DBG("\t       - SYS CPU arch  : %s",
		  BITMASK_BITTEST(res.grid.mask_, sys_caps) ?
		  SHOW_GRID_CPU_ARCH(res.grid.sys_caps.cpu_arch) :
		  "undeclared");
	GUNIGW_DBG("\t       - SYS exclAccess: %s",
		  BITMASK_BITTEST(res.grid.mask_, sys_caps) ?
		  ((res.grid.sys_caps.exclusive_access == 1) ? "yes" : "no")  :
		  "undeclared");
	// ranges
	GUNIGW_DBG("\t       - ind CPU speed : %s MHz",
		  BITMASK_BITTEST(res.grid.mask_, ind_cpu_speed) ?
		  range2string(res.grid.ind_cpu_speed).c_str() :
		  "undeclared");
	GUNIGW_DBG("\t       - ind CPU time  : %s seconds",
		  BITMASK_BITTEST(res.grid.mask_, ind_cpu_time) ?
		  range2string(res.grid.ind_cpu_time).c_str() :
		  "undeclared");
	GUNIGW_DBG("\t       - ind CPU count : %s cores",
		  BITMASK_BITTEST(res.grid.mask_, ind_cpu_count) ?
		  range2string(res.grid.ind_cpu_count).c_str() :
		  "undeclared");
	GUNIGW_DBG("\t       - ind NET bw    : %s Mbps",
		  BITMASK_BITTEST(res.grid.mask_, ind_net_bw) ?
		  range2string(res.grid.ind_net_bw).c_str() :
		  "undeclared");
	GUNIGW_DBG("\t       - ind MEM phy   : %s MB",
		  BITMASK_BITTEST(res.grid.mask_, ind_phy_mem) ?
		  range2string(res.grid.ind_phy_mem).c_str() :
		  "undeclared");
	GUNIGW_DBG("\t       - ind MEM vir   : %s MB",
		  BITMASK_BITTEST(res.grid.mask_, ind_phy_mem) ?
		  range2string(res.grid.ind_phy_mem).c_str() :
		  "undeclared");
	GUNIGW_DBG("\t       - ind DISK space: %s MB",
		  BITMASK_BITTEST(res.grid.mask_, ind_disk_space) ?
		  range2string(res.grid.ind_disk_space).c_str() :
		  "undeclared");
	GUNIGW_DBG("\t       - tot CPU time  : %s seconds",
		  BITMASK_BITTEST(res.grid.mask_, tot_cpu_time) ?
		  range2string(res.grid.tot_cpu_time).c_str() :
		  "undeclared");
	GUNIGW_DBG("\t       - tot CPU count : %s cores",
		  BITMASK_BITTEST(res.grid.mask_, tot_cpu_count) ?
		  range2string(res.grid.tot_cpu_count).c_str() :
		  "undeclared");
	GUNIGW_DBG("\t       - tot MEM phy   : %s MB",
		  BITMASK_BITTEST(res.grid.mask_, tot_phy_mem) ?
		  range2string(res.grid.tot_phy_mem).c_str() :
		  "undeclared");
	GUNIGW_DBG("\t       - tot MEM vir   : %s MB",
		  BITMASK_BITTEST(res.grid.mask_, tot_phy_mem) ?
		  range2string(res.grid.tot_phy_mem).c_str() :
		  "undeclared");
	GUNIGW_DBG("\t       - tot DISK space: %s MB",
		  BITMASK_BITTEST(res.grid.mask_, tot_disk_space) ?
		  range2string(res.grid.tot_disk_space).c_str() :
		  "undeclared");
	GUNIGW_DBG("\t       - tot RES count : %s",
		  BITMASK_BITTEST(res.grid.mask_, tot_res_count) ?
		  range2string(res.grid.tot_res_count).c_str() :
		  "undeclared");
	// DATA STAGING
	GUNIGW_DBG("\t       - STAG FS name  : %s",
		  BITMASK_BITTEST(res.grid.mask_, data_staging) ?
		  SHOW_GRID_FS_NAME(res.grid.data_staging.fs_name) :
		  "undeclared");
	GUNIGW_DBG("\t       - STAG createFlg: %s",
		  BITMASK_BITTEST(res.grid.mask_, data_staging) ?
		  SHOW_GRID_CREATION_FLAG(res.grid.data_staging.creation_flag):
		  "undeclared");
	GUNIGW_DBG("\t       - STAG delOnEnd : %s",
		  BITMASK_BITTEST(res.grid.mask_, data_staging) ?
		  ((res.grid.data_staging.del_on_termination == 1) ?
		   "yes" : "no")  :  "undeclared");
	GUNIGW_DBG("\t       - STAG filename : %s",
		  BITMASK_BITTEST(res.grid.mask_, data_staging) ?
		  (res.grid.data_staging.filename)->c_str() :
		  "undeclared");
	GUNIGW_DBG("\t       - STAG source   : %s",
		  BITMASK_BITTEST(res.grid.mask_, data_staging) ?
		  (res.grid.data_staging.source)->c_str() :
		  "undeclared");
	GUNIGW_DBG("\t       - STAG target   : %s",
		  BITMASK_BITTEST(res.grid.mask_, data_staging) ?
		  (res.grid.data_staging.target)->c_str() :
		  "undeclared");

	// GRID SITE ID
	GUNIGW_DBG("\t       - GRID SITE addr: %s",
		  BITMASK_BITTEST(res.grid.mask_, grid_site_id) ?
		  numb_htoa(res.grid.grid_site_id) : "undeclared");

#endif // G2MPLS
}

void
logDump(const call_info_t & cdata)
{
	tm *               curTime;
	const char *       timeStringFormat = "%a, %d %b %Y %H:%M:%S %zGMT";
	const int          timeStringLength = 40;
	char               timeString[timeStringLength];

	GUNIGW_DBG("CALL data");
	GUNIGW_DBG("\t - Symbolic name       : %s",
		  (cdata.call_name) ?
		  (cdata.call_name)->c_str() : "==UNKNOWN==");
	curTime = localtime((time_t *) &cdata.times.start_time);
	strftime(timeString, timeStringLength, timeStringFormat, curTime);
	GUNIGW_DBG("\t - startTime           : %s",
		  timeString);
	curTime = localtime((time_t *) &cdata.times.end_time);
	strftime(timeString, timeStringLength, timeStringFormat, curTime);
	GUNIGW_DBG("\t - endTime             : %s",
		  timeString);
	double duration = difftime(cdata.times.end_time,
				   cdata.times.start_time);
	GUNIGW_DBG("\t     - duration        : %.0f seconds",
		  (float)duration);
#ifdef G2MPLS
	GUNIGW_DBG("\t - jobName             : %s",
		  (cdata.job_name) ?
		  (cdata.job_name)->c_str() : "==UNKNOWN==");
	GUNIGW_DBG("\t - jobProject          : %s",
		  (cdata.job_project) ?
		  (cdata.job_project)->c_str() : "==UNKNOWN==");
#endif // G2MPLS

	GUNIGW_DBG("\t - iTNA resources");
	logDump(cdata.iTNA_res);

	GUNIGW_DBG("\t - eTNA resources");
	logDump(cdata.eTNA_res);
}

void
logDump(const recovery_info_t & rdata)
{
	GUNIGW_DBG("Recovery data");
	GUNIGW_DBG("\t - recovery type       : %s",
		  SHOW_GMPLS_RECOVERYTYPE(rdata.rec_type));
	GUNIGW_DBG("\t - disjointness        : %s",
		  SHOW_GMPLS_DISJOINTNESS(rdata.disj_type));
}

void
logDump(const lsp_info_t & ldata)
{
	GUNIGW_DBG("LSP data");
	GUNIGW_DBG("\t - swcap               : %s",
		  SHOW_SW_CAP(ldata.sw_cap));
	GUNIGW_DBG("\t - enctype             : %s",
		  SHOW_ENC_TYPE(ldata.enc_type));
	GUNIGW_DBG("\t - gpid                : %s",
		  SHOW_GPID(ldata.gpid));
	GUNIGW_DBG("\t - bandwidth           : %.3fMbps (signal %s)",
		  BW_HEX2BPS(ldata.bw) / 1000000,
		  SHOW_GMPLS_BWENC(ldata.bw));
	GUNIGW_DBG("\t - setupPrio           : %d",
		  ldata.setup_prio);
	GUNIGW_DBG("\t - holdingPrio         : %d",
		  ldata.holding_prio);
	GUNIGW_DBG("\t - excludeAny          : 0x%08x",
		  ldata.exclude_any);
	GUNIGW_DBG("\t - includeAny          : 0x%08x",
		  ldata.include_any);
	GUNIGW_DBG("\t - includeAll          : 0x%08x",
		  ldata.include_all);
	GUNIGW_DBG("\t - linkProtMask        : %s",
		  SHOW_GMPLS_PROTTYPE(ldata.link_prot_mask));
	GUNIGW_DBG("\t - crankback scope     : %s",
		  SHOW_CBACK_SCOPE(ldata.crankback));
	GUNIGW_DBG("\t     - maxRetrSrc      : %d",
		  ldata.max_cback_retries_src);
	GUNIGW_DBG("\t     - maxRetrInt      : %d",
		  ldata.max_cback_retries_intmd);
	GUNIGW_DBG("\t - type                : %s",
		  SHOW_LSP_TYPE(ldata.type));
	GUNIGW_DBG("\t - role                : %s",
		  SHOW_LSP_ROLE(ldata.role));
	GUNIGW_DBG("\t - action              : %s",
		  SHOW_LSP_RES_ACTION(ldata.action));
	GUNIGW_DBG("\t - rroMode             : %s",
		  SHOW_LSP_RRO_MODE(ldata.rro_mode));
	GUNIGW_DBG("\t - refresh             : %d ms",
		  ldata.refresh_interval);
	GUNIGW_DBG("\t - activateAck         : %s",
		  ldata.activate_ack ? "YES" : "NO");
	GUNIGW_DBG("\t - rapidRetrInt        : %d ms",
		  ldata.rapid_retransm_interval);
	GUNIGW_DBG("\t - rapidRetryLimit     : %d",
		  ldata.rapid_retry_limit);
	GUNIGW_DBG("\t - incrValueDelta      : %d",
		  ldata.increment_value_delta);
#ifdef G2MPLS
	tm *               curTime;
	const char *       timeStringFormat = "%a, %d %b %Y %H:%M:%S %zGMT";
	const int          timeStringLength = 40;
	char               timeString[timeStringLength];

	curTime = localtime((time_t *) &ldata.times.start_time);
	strftime(timeString, timeStringLength, timeStringFormat, curTime);
	GUNIGW_DBG("\t - startTime           : %s",
		  timeString);
	curTime = localtime((time_t *) &ldata.times.end_time);
	strftime(timeString, timeStringLength, timeStringFormat, curTime);
	GUNIGW_DBG("\t - endTime             : %s",
		  timeString);
	double duration = difftime(ldata.times.end_time,
				   ldata.times.start_time);
	GUNIGW_DBG("\t     - duration        : %.0f seconds",
		  (float)duration);
#endif // G2MPLS
}

void
vtyCallDumpLog(const mapKey_t & callKey)
{
	call_data_t *      data;

	CALLMAP_FIND(vtyCallContainer, call_data_t *, callKey, data);

	if (!data) {
		GUNIGW_ERR("CALL with %s not present in VTY containers",
			  key2string(callKey).c_str());
		vtyCallContainer.erase(callKey);
		return;
	}

	GUNIGW_DBG("%s", BAR1);
	GUNIGW_DBG("VTY CALL container:");
	GUNIGW_DBG("\t - localId             : %s",
		  key2string(callKey).c_str());
	GUNIGW_DBG("\t - committable         : %s",
		  vtyCallCommittable(callKey) ? "YES" : "NO");

	logDump(data->cid);
	logDump(data->cdata);
	logDump(data->rdata);
	logDump(data->ldata);

	GUNIGW_DBG("%s", BAR2);
	GUNIGW_DBG("");
}

bool
vtyCallUpdate(const mapKey_t &       callKey,
	      const uint32_t &       ituCountryCode,
	      const uint64_t &       ituCarrierCode,
	      const uint64_t &       uniqueAP,
	      std::string &          err)
{
	call_data_t * data;

	err      = "";


	CALLMAP_FIND(vtyCallContainer, call_data_t *, callKey, data);

	CALLMAP_COND_CREATE_AND_BIND(vtyCallContainer, call_data_t,
				     callKey, data, err);

	data->cid.type                       = CALLID_OPSPEC;
	data->cid.local_id                   = callKey;
	data->cid.src_addr.type              = IPv4;
	data->cid.src_addr.preflen           = 32;
	data->cid.src_addr.value.ipv4.s_addr = htonl(GUNIGW_MASTER.routerId());

	if (ituCountryCode) {
		data->cid.type             = CALLID_GLOBUNIQ;
		data->cid.itu_country_code = ituCountryCode;
		data->cid.itu_carrier_code = ituCarrierCode;
		data->cid.unique_ap        = uniqueAP;
	}

	//	vtyCallDumpLog(callKey);
	return true;
}

bool
vtyCallUpdate(const mapKey_t &         callKey,
	      const call_info_t &      cinfo,
	      std::string &            err)
{
	call_data_t * data;

	err = "";
	CALLMAP_FIND(vtyCallContainer, call_data_t *, callKey, data);
	CALLMAP_COND_CREATE_AND_BIND(vtyCallContainer, call_data_t,
				     callKey, data, err);

	if (BITMASK_BITTEST(cinfo.mask_, call_name)) {
		if (data->cdata.call_name) {

			delete data->cdata.call_name;
		}
		BITMASK_BITRESET(data->cdata.mask_, call_name);
		data->cdata.call_name = cinfo.call_name;
		BITMASK_BITSET(data->cdata.mask_, call_name);
	}

	SELECTIVE_UPDATE2(data->cdata, cinfo, times);

	if (BITMASK_BITTEST(cinfo.mask_, job_name)) {
		if (data->cdata.job_name) {
			delete data->cdata.job_name;
		}
		BITMASK_BITRESET(data->cdata.mask_, job_name);
		data->cdata.job_name = cinfo.job_name;
		BITMASK_BITSET(data->cdata.mask_, job_name);
	}

	if (BITMASK_BITTEST(cinfo.mask_, job_project)) {
		if (data->cdata.job_project) {
			delete data->cdata.job_project;
		}
		BITMASK_BITRESET(data->cdata.mask_, job_project);
		data->cdata.job_project = cinfo.job_project;
		BITMASK_BITSET(data->cdata.mask_, job_project);
	}

	if (BITMASK_BITTEST(cinfo.mask_, iTNA_res)) {
		BITMASK_BITRESET(data->cdata.mask_, iTNA_res);

		if (!ResSpecUpdate(data->cdata.iTNA_res, cinfo.iTNA_res)) {
			err = "Cannot copy iTNA resource specifications";
			return false;
		}

		BITMASK_BITSET(data->cdata.mask_, iTNA_res);
	}

	if (BITMASK_BITTEST(cinfo.mask_, eTNA_res)) {
		BITMASK_BITRESET(data->cdata.mask_, eTNA_res);

		if (!ResSpecUpdate(data->cdata.eTNA_res, cinfo.eTNA_res)) {
			err = "Cannot copy eTNA resource specifications";
			return false;
		}

		BITMASK_BITSET(data->cdata.mask_, eTNA_res);
	}
	//	vtyCallDumpLog(callKey);

	return true;
}

bool
vtyCallUpdate(const mapKey_t &             callKey,
	      const recovery_info_t &      rinfo,
	      std::string &                err)
{
	call_data_t * data;

	err = "";
	CALLMAP_FIND(vtyCallContainer, call_data_t *, callKey, data);
	CALLMAP_COND_CREATE_AND_BIND(vtyCallContainer, call_data_t,
				     callKey, data, err);

	SELECTIVE_UPDATE2(data->rdata, rinfo, rec_type);
	SELECTIVE_UPDATE2(data->rdata, rinfo, disj_type);

	//	vtyCallDumpLog(callKey);
	return true;
}

bool
vtyCallUpdate(const mapKey_t &        callKey,
	      const lsp_info_t &      linfo,
	      std::string &           err)
{
	call_data_t * data;

	err = "";
	CALLMAP_FIND(vtyCallContainer, call_data_t *, callKey, data);
	CALLMAP_COND_CREATE_AND_BIND(vtyCallContainer, call_data_t,
				     callKey, data, err);

	SELECTIVE_UPDATE2(data->ldata, linfo, sw_cap);
	SELECTIVE_UPDATE2(data->ldata, linfo, enc_type);
	SELECTIVE_UPDATE2(data->ldata, linfo, gpid);
	SELECTIVE_UPDATE2(data->ldata, linfo, bw);
	SELECTIVE_UPDATE2(data->ldata, linfo, setup_prio);
	SELECTIVE_UPDATE2(data->ldata, linfo, holding_prio);
	SELECTIVE_UPDATE2(data->ldata, linfo, exclude_any);
	SELECTIVE_UPDATE2(data->ldata, linfo, include_any);
	SELECTIVE_UPDATE2(data->ldata, linfo, include_all);
	SELECTIVE_UPDATE2(data->ldata, linfo, link_prot_mask);
	SELECTIVE_UPDATE2(data->ldata, linfo, crankback);
	SELECTIVE_UPDATE2(data->ldata, linfo, max_cback_retries_src);
	SELECTIVE_UPDATE2(data->ldata, linfo, max_cback_retries_intmd);
	SELECTIVE_UPDATE2(data->ldata, linfo, type);
	SELECTIVE_UPDATE2(data->ldata, linfo, role);
	SELECTIVE_UPDATE2(data->ldata, linfo, action);
	SELECTIVE_UPDATE2(data->ldata, linfo, rro_mode);
	SELECTIVE_UPDATE2(data->ldata, linfo, refresh_interval);

	if (BITMASK_BITTEST(linfo.mask_, activate_ack)) {
		BITMASK_BITRESET(data->ldata.mask_, activate_ack);
		data->ldata.activate_ack = linfo.activate_ack;
		BITMASK_BITRESET(data->ldata.mask_, activate_ack);
	}

	SELECTIVE_UPDATE2(data->ldata, linfo, rapid_retransm_interval);
	SELECTIVE_UPDATE2(data->ldata, linfo, rapid_retry_limit);
	SELECTIVE_UPDATE2(data->ldata, linfo, increment_value_delta);
	SELECTIVE_UPDATE2(data->ldata, linfo, times);

	if (!BITMASK_BITTEST(data->ldata.mask_, times)) {
		data->ldata.mask_.times = data->cdata.mask_.times;
		data->ldata.times       = data->cdata.times;
	}

	//	vtyCallDumpLog(callKey);
	return true;
}

bool
vtyCallCommit(const mapKey_t &	callKey,
	      std::string &     err)
{
	call_data_t * data;

	err = "";

	CALLMAP_FIND(vtyCallContainer, call_data_t *, callKey, data);

	vtyCallDumpLog(callKey);

	if (!data) {
		err  = "CALL with ";
		err += key2string(callKey);
		err += " not present in VTY containers";
		vtyCallContainer.erase(callKey);
		return false;
	}

	if (!vtyCallCommittable(callKey)) {
		err  = "CALL with ";
		err += key2string(callKey);
		err += " not yet committable to py2NCCd";
		return false;
	}

	if (GUNIGW_MASTER.dryrun()) {
		// remove item from container
		delete data->cdata.job_name;
		delete data->cdata.job_project;
		delete data;
		vtyCallContainer.erase(callKey);

		return true;
	}

#if HAVE_OMNIORB
	GUNIGW_DBG("Sending callCreate through CORBA Mgmt interface");

	CallController::Mgmt_var         nccMgmtProxy;

	nccMgmtProxy = GUNIGW_MASTER.getNccProxy();

	if (CORBA::is_nil(nccMgmtProxy)) {
		GUNIGW_ERR("Cannot get Corba client object "
			  "previously set up");
		assert(0);
	}

	g2mplsTypes::callIdent_var id;

	try {
		id << data->cid;
	} catch (std::out_of_range & e) {
		err  = "Exception occurred during callId translation (";
		err += e.what();
		err += ")";
		return false;
	}

	g2mplsTypes::callParams_var callInfo;

	try {
		callInfo << data->cdata;
	} catch (std::out_of_range & e) {
		err  = "Exception occurred during callInfo translation (";
		err += e.what();
		err += ")";
		return false;
	}

	g2mplsTypes::recoveryParams_var recoveryInfo;

	try {
		recoveryInfo << data->rdata;
	} catch (std::out_of_range & e) {
		err  = "Exception occurred during recoveryInfo translation (";
		err += e.what();
		err += ")";
		return false;
	}

	g2mplsTypes::lspParams_var lspInfo;

	try {
		lspInfo << data->ldata;
	} catch (std::out_of_range & e) {
		err  = "Exception occurred during lspInfo translation (";
		err += e.what();
		err += ")";
		return false;
	}

	g2mplsTypes::aaaParams_var aaaInfo;
	//get aaa info
	////TMP - EDUARD FIXME
	aaaInfo = new g2mplsTypes::aaaParams;
	aaaInfo->authzToken = "";
	////

	g2mplsTypes::actorInfo_var actor;
	actor = new g2mplsTypes::actorInfo;

	actor->issuer       = g2mplsTypes::ISSUERTYPE_MANAGEMENT_IF;
	actor->forceCommand = false;

	try {
		nccMgmtProxy->callCreate(id          ,
					 callInfo    ,
					 recoveryInfo,
					 lspInfo     ,
					 aaaInfo     ,
					 actor       );
	} catch (Types::InternalProblems & e) {
		err += "callCreate through CORBA Mgmt ";
		err += "interface failed for internal problems (reason= ";
		err += e.what;
		err += ")";
		return false;
	} catch (Types::CannotFetch & e) {
		err += "callCreate through CORBA Mgmt ";
		err += "interface failed for fetching problems (reason= ";
		err += e.what;
		err += ")";
		return false;
	} catch (...) {
		err += "callCreate through CORBA Mgmt ";
		err += "interface failed for unknown reason";
		return false;
	}

	GUNIGW_DBG("Sent callCreate through CORBA Mgmt interface");

	GUNIGW_DBG("Created call with identity [%s]",
		  ((id->idType == g2mplsTypes::CALLIDTYPE_GLOBUNIQ) ?
		   "GLOBALLY UNIQUE" : "OPERATOR SPECIFIC"));
	GUNIGW_DBG("\t - srcAddr             : %s",
		  addr_ntoa(data->cid.src_addr));
	GUNIGW_DBG("\t - localId             : %s",
		  key2string(data->cid.local_id).c_str());

#if NCC_GLOBAL_CALLID
	if (id->idType == g2mplsTypes::CALLIDTYPE_GLOBUNIQ) {
		GUNIGW_DBG("\t - Globally unique segments");
		GUNIGW_DBG("\t     - international : 0x%02x%02x%02x",
			  id->segs.intlSeg[0],
			  id->segs.intlSeg[1],
			  id->segs.intlSeg[2]);
		GUNIGW_DBG("\t     - national      : 0x%08x%08x%08x",
			  id->segs.natlSeg[0],
			  id->segs.natlSeg[1],
			  id->segs.natlSeg[2]);
	}
#endif

	if (BITMASK_BITTEST(data->cdata.mask_, iTNA_res)) {
		g2mplsTypes::resourcePosition  pos;

		pos = g2mplsTypes::RESOURCEPOSITION_INGRESS;

		if (BITMASK_BITTEST(data->cdata.iTNA_res.mask_, net)) {
			g2mplsTypes::tnaResource_var  tnaRes;

			try {
				tnaRes << data->cdata.iTNA_res.net;
			} catch (std::out_of_range & e) {
				err  = "Exception occurred during ";
				err += "translation of iTNA_res ";
				err += "network params(";
				err += e.what();
				err += ")";
				return false;
			}

			try {
				nccMgmtProxy->callSetTna(id    ,
							 pos   ,
							 tnaRes,
							 actor );
			} catch (Types::InternalProblems & e) {
				err += "callSetTna through CORBA Mgmt ";
				err += "interface failed for internal ";
				err += "problems (reason= ";
				err += e.what;
				err += ")";
				return false;
			} catch (Types::CannotFetch & e) {
				err += "callSetTna through CORBA Mgmt ";
				err += "interface failed for fetching ";
				err += "problems (reason= ";
				err += e.what;
				err += ")";
				return false;
			} catch (...) {
				err += "callSetTna through CORBA Mgmt ";
				err += "interface failed for unknown reason";
				return false;
			}

			GUNIGW_DBG("Sent callSetTna for iTNA "
				  "through CORBA Mgmt interface");
		}

		if (BITMASK_BITTEST(data->cdata.iTNA_res.mask_, grid)) {
			g2mplsTypes::gridParams_var   gnsTnaRes;

			try {
				gnsTnaRes << data->cdata.iTNA_res.grid;
			} catch (std::out_of_range & e) {
				err  = "Exception occurred during ";
				err += "translation of iTNA_res ";
				err += "grid params(";
				err += e.what();
				err += ")";
				return false;
			}

			try {
				nccMgmtProxy->callSetGnsTna(id       ,
							    pos      ,
							    gnsTnaRes,
							    actor    );
			} catch (Types::InternalProblems & e) {
				err += "callSetGnsTna through CORBA Mgmt ";
				err += "interface failed for internal ";
				err += "problems (reason= ";
				err += e.what;
				err += ")";
				return false;
			} catch (Types::CannotFetch & e) {
				err += "callSetGnsTna through CORBA Mgmt ";
				err += "interface failed for fetching ";
				err += "problems (reason= ";
				err += e.what;
				err += ")";
				return false;
			} catch (...) {
				err += "callSetGnsTna through CORBA Mgmt ";
				err += "interface failed for unknown reason";
				return false;
			}

			GUNIGW_DBG("Sent callSeGnstTna for iTNA "
				  "through CORBA Mgmt interface");
		}
	}

	if (BITMASK_BITTEST(data->cdata.mask_, eTNA_res)) {
		g2mplsTypes::resourcePosition  pos;

		pos = g2mplsTypes::RESOURCEPOSITION_EGRESS;

		if (BITMASK_BITTEST(data->cdata.eTNA_res.mask_, net)) {
			g2mplsTypes::tnaResource_var  tnaRes;

			try {
				tnaRes << data->cdata.eTNA_res.net;
			} catch (std::out_of_range & e) {
				err  = "Exception occurred during ";
				err += "translation of eTNA_res ";
				err += "network params(";
				err += e.what();
				err += ")";
				return false;
			}

			try {
				nccMgmtProxy->callSetTna(id    ,
							 pos   ,
							 tnaRes,
							 actor );
			} catch (Types::InternalProblems & e) {
				err += "callSetTna through CORBA Mgmt ";
				err += "interface failed for internal ";
				err += "problems (reason= ";
				err += e.what;
				err += ")";
				return false;
			} catch (Types::CannotFetch & e) {
				err += "callSetTna through CORBA Mgmt ";
				err += "interface failed for fetching ";
				err += "problems (reason= ";
				err += e.what;
				err += ")";
				return false;
			} catch (...) {
				err += "callSetTna through CORBA Mgmt ";
				err += "interface failed for unknown reason";
				return false;
			}

			GUNIGW_DBG("Sent callSetTna for eTNA "
				  "through CORBA Mgmt interface");
		}

		if (BITMASK_BITTEST(data->cdata.eTNA_res.mask_, grid)) {
			g2mplsTypes::gridParams_var   gnsTnaRes;

			try {
				gnsTnaRes << data->cdata.eTNA_res.grid;
			} catch (std::out_of_range & e) {
				err  = "Exception occurred during ";
				err += "translation of iTNA_res ";
				err += "grid params(";
				err += e.what();
				err += ")";
				return false;
			}

			try {
				nccMgmtProxy->callSetGnsTna(id       ,
							    pos      ,
							    gnsTnaRes,
							    actor    );
			} catch (Types::InternalProblems & e) {
				err += "callSetGnsTna through CORBA Mgmt ";
				err += "interface failed for internal ";
				err += "problems (reason= ";
				err += e.what;
				err += ")";
				return false;
			} catch (Types::CannotFetch & e) {
				err += "callSetGnsTna through CORBA Mgmt ";
				err += "interface failed for fetching ";
				err += "problems (reason= ";
				err += e.what;
				err += ")";
				return false;
			} catch (...) {
				err += "callSetGnsTna through CORBA Mgmt ";
				err += "interface failed for unknown reason";
				return false;
			}
			GUNIGW_DBG("Sent callSetGnsTna for eTNA "
				  "through CORBA Mgmt interface");
		}
	}

#endif // HAVE_OMNIORB

	// remove item from container
	if (data->cdata.call_name) {
		delete data->cdata.call_name;
	}
	if (data->cdata.job_name) {
		delete data->cdata.job_name;
	}
	if (data->cdata.job_project) {
		delete data->cdata.job_project;
	}
	if (data->cdata.iTNA_res.grid.fs_resources.mount_point) {
		delete data->cdata.iTNA_res.grid.fs_resources.mount_point;
	}
	if (data->cdata.iTNA_res.grid.fs_resources.mount_source) {
		delete data->cdata.iTNA_res.grid.fs_resources.mount_source;
	}
	if (data->cdata.iTNA_res.grid.data_staging.filename) {
		delete data->cdata.iTNA_res.grid.data_staging.filename;
	}
	if (data->cdata.iTNA_res.grid.data_staging.source) {
		delete data->cdata.iTNA_res.grid.data_staging.source;
	}
	if (data->cdata.iTNA_res.grid.data_staging.target) {
		delete data->cdata.iTNA_res.grid.data_staging.target;
	}
	delete data;
	vtyCallContainer.erase(callKey);

	return true;
}

void
vtyCallDumpVty(struct vty *      vty,
	       const mapKey_t &  key)
{
	call_data_t *      data;

	CALLMAP_FIND(vtyCallContainer, call_data_t *, key, data);

	if (!data) {
		vty_out(vty,
			"CALL with %s not present in VTY containers%s",
			key2string(key).c_str(),
			VTY_NEWLINE);
		vtyCallContainer.erase(key);
		return;
	}

	vty_out(vty, "%s%s", BAR1, VTY_NEWLINE);
	vty_out(vty, "VTY CALL container%s", VTY_NEWLINE);
	vty_out(vty, "\t - committable     : %s%s",
		vtyCallCommittable(key) ? "YES" : "NO",	VTY_NEWLINE);

	assert(data);

	vtyDump(vty, data->cid);
	vtyDump(vty, data->cdata);
	vtyDump(vty, data->rdata);
	vtyDump(vty, data->ldata);

	vty_out(vty, "%s%s", BAR2, VTY_NEWLINE);
	vty_out(vty, "%s", VTY_NEWLINE);

}

void
vtyCallDumpVty(struct vty * vty)
{
	int i;

	vty_out(vty, "NCC CALL(s) pending in VTY%s", VTY_NEWLINE);

	if (vtyCallContainer.empty()) {
		vty_out(vty, "   ***EMPTY***%s", VTY_NEWLINE);
		vty_out(vty, "%s", VTY_NEWLINE);
		return;
	}

	std::map<mapKey_t, call_data_t * >::const_iterator iter;
	for (iter = vtyCallContainer.begin(), i = 0;
	     iter != vtyCallContainer.end();
	     iter++, i++) {
		vty_out(vty, "   - [p] #%02d: src-LSR = %s / local-ID = %s%s",
			i,
			numb_htoa(GUNIGW_MASTER.routerId()),
			key2string((*iter).first).c_str(),
			VTY_NEWLINE);
		if (i == SHOWLIST_DEPTH) {
			break;
		}
	}
	vty_out(vty, "%s", VTY_NEWLINE);
	vty_out(vty, "   printed %d VTY pending CALL(s)%s", i, VTY_NEWLINE);
}

void
vtyDump(struct vty * vty, const call_ident_t & cid)
{
	vty_out(vty, "CALL identity [%s]%s",
		((cid.type == CALLID_GLOBUNIQ) ?
		 "GLOBALLY UNIQUE" : "OPERATOR SPECIFIC"),
		VTY_NEWLINE);
	vty_out(vty, "\t - srcAddr         : %s %s",
		addr_ntoa(cid.src_addr),
		VTY_NEWLINE);
	vty_out(vty, "\t - localId         : %s%s",
		key2string(cid.local_id).c_str(),
		VTY_NEWLINE);

	if (cid.type == CALLID_GLOBUNIQ) {
		vty_out(vty, "\t - Globally unique attributes%s",
			VTY_NEWLINE);
		vty_out(vty, "\t     - ituCountryCode: 0x%06x%s",
			cid.itu_country_code,
			VTY_NEWLINE);
		vty_out(vty, "\t     - ituCarrierCode: %s %s",
			key2string(cid.itu_carrier_code).c_str(),
			VTY_NEWLINE);
		vty_out(vty, "\t     - uniqueAP      : %s %s",
			key2string(cid.unique_ap).c_str(),
			VTY_NEWLINE);
	}
}

void
vtyDump(struct vty * vty, const reco_bundle_ident_t & rid)
{
	vty_out(vty, "Recovery Bundle identity%s",
		VTY_NEWLINE);
	vty_out(vty, "\t - Tunnel id       : 0x%08x%s",
		rid.tun_id, VTY_NEWLINE);
	vty_out(vty, "\t - src LSR address : %s%s",
		numb_htoa(rid.src_lsr), VTY_NEWLINE);
	vty_out(vty, "\t - dst LSR address : %s%s",
		numb_htoa(rid.dst_lsr), VTY_NEWLINE);
}

void
vtyDump(struct vty * vty, const lsp_ident_t & lid)
{
	vty_out(vty, "LSP identity%s",
		VTY_NEWLINE);
	vty_out(vty, "\t - Tunnel id       : 0x%08x%s",
		lid.tun_id, VTY_NEWLINE);
	vty_out(vty, "\t - Ext. Tunnel id  : 0x%08x%s",
		lid.ext_tun_id, VTY_NEWLINE);
	vty_out(vty, "\t - LSP id          : 0x%08x%s",
		lid.lsp_id, VTY_NEWLINE);
	vty_out(vty, "\t - src LSR address : %s%s",
		numb_htoa(lid.src_nid), VTY_NEWLINE);
	vty_out(vty, "\t - dst LSR address : %s%s",
		numb_htoa(lid.dst_nid), VTY_NEWLINE);
}

void
vtyDump(struct vty * vty, const res_spec_t & res)
{
	vty_out(vty, "\t   - Network data      : %s present%s",
		BITMASK_BITTEST(res.mask_, net) ? "is" : "not",
		VTY_NEWLINE);
	vty_out(vty, "\t       - tna           : %s%s",
		BITMASK_BITTEST(res.net.mask_, tna) ?
		addr_ntoa(res.net.tna) : "undeclared",
		VTY_NEWLINE);
	vty_out(vty, "\t       - data link     : %s%s",
		BITMASK_BITTEST(res.net.mask_, data_link) ?
		addr_ntoa(res.net.data_link) : "undeclared",
		VTY_NEWLINE);
	vty_out(vty, "\t       - label         : %s%s",
		BITMASK_BITTEST(res.net.mask_, label) ?
		label2string(res.net.label).c_str() : "undeclared",
		VTY_NEWLINE);

#ifdef G2MPLS
	vty_out(vty, "\t   - Grid data         : %s present%s",
		BITMASK_BITTEST(res.mask_, grid) ?
		"is" : "not",
		VTY_NEWLINE);
	// APPLICATION
	vty_out(vty, "\t       - APP  type     : %s%s",
		BITMASK_BITTEST(res.grid.mask_, application) ?
		SHOW_GRID_APPLICATION(res.grid.application.type) :
		"undeclared",
		VTY_NEWLINE);
	vty_out(vty, "\t       - APP  version  : %s%s",
		BITMASK_BITTEST(res.grid.mask_, application) ?
		version2string(res.grid.application.version).c_str() :
		"undeclared",
		VTY_NEWLINE);

	// CAND HOST
	vty_out(vty, "\t       - CAND HOST     : %s%s",
		BITMASK_BITTEST(res.grid.mask_, cand_host) ?
		addr_ntoa(res.grid.cand_host) : "undeclared",
		VTY_NEWLINE);

	// FS_RESOURCES
	vty_out(vty, "\t       - FS name       : %s%s",
		BITMASK_BITTEST(res.grid.mask_, fs_resources) ?
		SHOW_GRID_FS_NAME(res.grid.fs_resources.fs_name) :
		"undeclared",
		VTY_NEWLINE);
	vty_out(vty, "\t       - FS type       : %s%s",
		BITMASK_BITTEST(res.grid.mask_, fs_resources) ?
		SHOW_GRID_FS_TYPE(res.grid.fs_resources.fs_type) :
		"undeclared",
		VTY_NEWLINE);
	vty_out(vty, "\t       - FS disk space : %s%s",
		BITMASK_BITTEST(res.grid.mask_, fs_resources) ?
		range2string(res.grid.fs_resources.disk_space).c_str() :
		"undeclared",
		VTY_NEWLINE);
	vty_out(vty, "\t       - FS mount point: %s%s",
		BITMASK_BITTEST(res.grid.mask_, fs_resources) ?
		(res.grid.fs_resources.mount_point)->c_str() :
		"undeclared",
		VTY_NEWLINE);
	vty_out(vty, "\t       - FS mount src  : %s%s",
		BITMASK_BITTEST(res.grid.mask_, fs_resources) ?
		(res.grid.fs_resources.mount_source)->c_str() :
		"undeclared",
		VTY_NEWLINE);

	// SYSTEM CAPS
	vty_out(vty, "\t       - SYS OS        : %s%s",
		BITMASK_BITTEST(res.grid.mask_, sys_caps) ?
		SHOW_GRID_OS(res.grid.sys_caps.os_type) :
		"undeclared",
		VTY_NEWLINE);
	vty_out(vty, "\t       - SYS OS version: %s%s",
		BITMASK_BITTEST(res.grid.mask_, sys_caps) ?
		version2string(res.grid.sys_caps.os_version).c_str() :
		"undeclared",
		VTY_NEWLINE);
	vty_out(vty, "\t       - SYS CPU arch  : %s%s",
		BITMASK_BITTEST(res.grid.mask_, sys_caps) ?
		SHOW_GRID_CPU_ARCH(res.grid.sys_caps.cpu_arch) :
		"undeclared",
		VTY_NEWLINE);
	vty_out(vty, "\t       - SYS exclAccess: %s%s",
		BITMASK_BITTEST(res.grid.mask_, sys_caps) ?
		((res.grid.sys_caps.exclusive_access == 1) ? "yes" : "no")  :
		"undeclared",
		VTY_NEWLINE);
	// ranges
	vty_out(vty, "\t       - ind CPU speed : %s MHz%s",
		BITMASK_BITTEST(res.grid.mask_, ind_cpu_speed) ?
		range2string(res.grid.ind_cpu_speed).c_str() :
		"undeclared",
		VTY_NEWLINE);
	vty_out(vty, "\t       - ind CPU time  : %s seconds%s",
		BITMASK_BITTEST(res.grid.mask_, ind_cpu_time) ?
		range2string(res.grid.ind_cpu_time).c_str() :
		"undeclared",
		VTY_NEWLINE);
	vty_out(vty, "\t       - ind CPU count : %s cores%s",
		BITMASK_BITTEST(res.grid.mask_, ind_cpu_count) ?
		range2string(res.grid.ind_cpu_count).c_str() :
		"undeclared",
		VTY_NEWLINE);
	vty_out(vty, "\t       - ind NET bw    : %s Mbps%s",
		BITMASK_BITTEST(res.grid.mask_, ind_net_bw) ?
		range2string(res.grid.ind_net_bw).c_str() :
		"undeclared",
		VTY_NEWLINE);
	vty_out(vty, "\t       - ind MEM phy   : %s MB%s",
		BITMASK_BITTEST(res.grid.mask_, ind_phy_mem) ?
		range2string(res.grid.ind_phy_mem).c_str() :
		"undeclared",
		VTY_NEWLINE);
	vty_out(vty, "\t       - ind MEM vir   : %s MB%s",
		BITMASK_BITTEST(res.grid.mask_, ind_phy_mem) ?
		range2string(res.grid.ind_phy_mem).c_str() :
		"undeclared",
		VTY_NEWLINE);
	vty_out(vty, "\t       - ind DISK space: %s MB%s",
		BITMASK_BITTEST(res.grid.mask_, ind_disk_space) ?
		range2string(res.grid.ind_disk_space).c_str() :
		"undeclared",
		VTY_NEWLINE);
	vty_out(vty, "\t       - tot CPU time  : %s seconds%s",
		BITMASK_BITTEST(res.grid.mask_, tot_cpu_time) ?
		range2string(res.grid.tot_cpu_time).c_str() :
		"undeclared",
		VTY_NEWLINE);
	vty_out(vty, "\t       - tot CPU count : %s cores%s",
		BITMASK_BITTEST(res.grid.mask_, tot_cpu_count) ?
		range2string(res.grid.tot_cpu_count).c_str() :
		"undeclared",
		VTY_NEWLINE);
	vty_out(vty, "\t       - tot MEM phy   : %s MB%s",
		BITMASK_BITTEST(res.grid.mask_, tot_phy_mem) ?
		range2string(res.grid.tot_phy_mem).c_str() :
		"undeclared",
		VTY_NEWLINE);
	vty_out(vty, "\t       - tot MEM vir   : %s MB%s",
		BITMASK_BITTEST(res.grid.mask_, tot_phy_mem) ?
		range2string(res.grid.tot_phy_mem).c_str() :
		"undeclared",
		VTY_NEWLINE);
	vty_out(vty, "\t       - tot DISK space: %s MB%s",
		BITMASK_BITTEST(res.grid.mask_, tot_disk_space) ?
		range2string(res.grid.tot_disk_space).c_str() :
		"undeclared",
		VTY_NEWLINE);
	vty_out(vty, "\t       - tot RES count : %s%s",
		BITMASK_BITTEST(res.grid.mask_, tot_res_count) ?
		range2string(res.grid.tot_res_count).c_str() :
		"undeclared",
		VTY_NEWLINE);
	// DATA STAGING
	vty_out(vty, "\t       - STAG FS name  : %s%s",
		BITMASK_BITTEST(res.grid.mask_, data_staging) ?
		SHOW_GRID_FS_NAME(res.grid.data_staging.fs_name) :
		"undeclared",
		VTY_NEWLINE);
	vty_out(vty, "\t       - STAG createFlg: %s%s",
		BITMASK_BITTEST(res.grid.mask_, data_staging) ?
		SHOW_GRID_CREATION_FLAG(res.grid.data_staging.creation_flag):
		"undeclared",
		VTY_NEWLINE);
	vty_out(vty, "\t       - STAG delOnEnd : %s%s",
		BITMASK_BITTEST(res.grid.mask_, data_staging) ?
		((res.grid.data_staging.del_on_termination == 1) ?
		 "yes" : "no")  :  "undeclared",
		VTY_NEWLINE);
	vty_out(vty, "\t       - STAG filename : %s%s",
		BITMASK_BITTEST(res.grid.mask_, data_staging) ?
		(res.grid.data_staging.filename)->c_str() :
		"undeclared",
		VTY_NEWLINE);
	vty_out(vty, "\t       - STAG source   : %s%s",
		BITMASK_BITTEST(res.grid.mask_, data_staging) ?
		(res.grid.data_staging.source)->c_str() :
		"undeclared",
		VTY_NEWLINE);
	vty_out(vty, "\t       - STAG target   : %s%s",
		BITMASK_BITTEST(res.grid.mask_, data_staging) ?
		(res.grid.data_staging.target)->c_str() :
		"undeclared",
		VTY_NEWLINE);

	// GRID SITE ID
	vty_out(vty, "\t       - GRID SITE addr: %s%s",
		BITMASK_BITTEST(res.grid.mask_, grid_site_id) ?
		numb_htoa(res.grid.grid_site_id) : "undeclared",
		VTY_NEWLINE);

#endif // G2MPLS
}

void
vtyDump(struct vty * vty, const call_info_t & cdata)
{
	tm *               curTime;
	const char *       timeStringFormat = "%a, %d %b %Y %H:%M:%S %zGMT";
	const int          timeStringLength = 40;
	char               timeString[timeStringLength];

	vty_out(vty, "CALL data%s", VTY_NEWLINE);
	curTime = localtime((time_t *) &cdata.times.start_time);
	strftime(timeString, timeStringLength, timeStringFormat, curTime);
	vty_out(vty, "\t - startTime       : %s%s",
		timeString, VTY_NEWLINE);
	curTime = localtime((time_t *) &cdata.times.end_time);
	strftime(timeString, timeStringLength, timeStringFormat, curTime);
	vty_out(vty, "\t - endTime         : %s%s",
		timeString, VTY_NEWLINE);
	double duration = difftime(cdata.times.end_time,
				   cdata.times.start_time);
	vty_out(vty, "\t     - duration    : %.0f seconds%s",
		(float)duration, VTY_NEWLINE);
	vty_out(vty, "\t - jobName         : %s%s",
		(cdata.job_name) ?
		(cdata.job_name)->c_str() : "==UNKNOWN==",
		VTY_NEWLINE);
	vty_out(vty, "\t - jobProject      : %s%s",
		(cdata.job_project) ?
		(cdata.job_project)->c_str() : "==UNKNOWN==",
		VTY_NEWLINE);
	vty_out(vty, "\t - iTNA resources %s",
		VTY_NEWLINE);
	vtyDump(vty, cdata.iTNA_res);
	vty_out(vty, "\t - eTNA resources %s",
		VTY_NEWLINE);
	vtyDump(vty, cdata.eTNA_res);
}

void
vtyDump(struct vty * vty, const recovery_info_t & rdata)
{
	vty_out(vty, "   Recovery data%s", VTY_NEWLINE);
	vty_out(vty, "\t - recovery type   : %s%s",
		SHOW_GMPLS_RECOVERYTYPE(rdata.rec_type), VTY_NEWLINE);
	vty_out(vty, "\t - disjointness    : %s%s",
		SHOW_GMPLS_DISJOINTNESS(rdata.disj_type), VTY_NEWLINE);
}

void
vtyDump(struct vty * vty, const lsp_info_t & ldata)
{
	vty_out(vty, "   LSP data%s", VTY_NEWLINE);
	vty_out(vty, "\t - swcap           : %s%s",
		SHOW_SW_CAP(ldata.sw_cap), VTY_NEWLINE);
	vty_out(vty, "\t - enctype         : %s%s",
		SHOW_ENC_TYPE(ldata.enc_type), VTY_NEWLINE);
	vty_out(vty, "\t - gpid            : %s%s",
		SHOW_GPID(ldata.gpid), VTY_NEWLINE);
	vty_out(vty, "\t - bandwidth       : %.3fMbps (signal %s)%s",
		BW_HEX2BPS(ldata.bw) / 1000000,
		SHOW_GMPLS_BWENC(ldata.bw), VTY_NEWLINE);
	vty_out(vty, "\t - setupPrio       : %d%s",
		ldata.setup_prio, VTY_NEWLINE);
	vty_out(vty, "\t - holdingPrio     : %d%s",
		ldata.holding_prio, VTY_NEWLINE);
	vty_out(vty, "\t - excludeAny      : 0x%08x%s",
		ldata.exclude_any, VTY_NEWLINE);
	vty_out(vty, "\t - includeAny      : 0x%08x%s",
		ldata.include_any, VTY_NEWLINE);
	vty_out(vty, "\t - includeAll      : 0x%08x%s",
		ldata.include_all, VTY_NEWLINE);
	vty_out(vty, "\t - linkProtMask    : %s%s",
		SHOW_GMPLS_PROTTYPE(ldata.link_prot_mask),
		VTY_NEWLINE);
	vty_out(vty, "\t - crankback scope : %s%s",
		SHOW_CBACK_SCOPE(ldata.crankback),
		VTY_NEWLINE);
	vty_out(vty, "\t     - maxRetrSrc  : %d%s",
		ldata.max_cback_retries_src, VTY_NEWLINE);
	vty_out(vty, "\t     - maxRetrInt  : %d%s",
		ldata.max_cback_retries_intmd, VTY_NEWLINE);
	vty_out(vty, "\t - type            : %s%s",
		SHOW_LSP_TYPE(ldata.type), VTY_NEWLINE);
	vty_out(vty, "\t - role            : %s%s",
		SHOW_LSP_ROLE(ldata.role), VTY_NEWLINE);
	vty_out(vty, "\t - action          : %s%s",
		SHOW_LSP_RES_ACTION(ldata.action), VTY_NEWLINE);
	vty_out(vty, "\t - rro_mode        : %s%s",
		SHOW_LSP_RRO_MODE(ldata.rro_mode), VTY_NEWLINE);
	vty_out(vty, "\t - refresh         : %d ms%s",
		ldata.refresh_interval, VTY_NEWLINE);
	vty_out(vty, "\t - activateAck     : %s%s",
		ldata.activate_ack ? "YES" : "NO", VTY_NEWLINE);
	vty_out(vty, "\t - rapidRetrInt    : %d ms%s",
		ldata.rapid_retransm_interval, VTY_NEWLINE);
	vty_out(vty, "\t - rapidRetryLimit : %d%s",
		ldata.rapid_retry_limit, VTY_NEWLINE);
	vty_out(vty, "\t - incrValueDelta  : %d%s",
		ldata.increment_value_delta, VTY_NEWLINE);
#ifdef G2MPLS
	tm *               curTime;
	const char *       timeStringFormat = "%a, %d %b %Y %H:%M:%S %zGMT";
	const int          timeStringLength = 40;
	char               timeString[timeStringLength];

	curTime = localtime((time_t *) &ldata.times.start_time);
	strftime(timeString, timeStringLength, timeStringFormat, curTime);
	vty_out(vty, "\t - startTime           : %s%s",
		  timeString, VTY_NEWLINE);
	curTime = localtime((time_t *) &ldata.times.end_time);
	strftime(timeString, timeStringLength, timeStringFormat, curTime);
	vty_out(vty, "\t - endTime             : %s%s",
		  timeString, VTY_NEWLINE);
	double duration = difftime(ldata.times.end_time,
				   ldata.times.start_time);
	vty_out(vty, "\t     - duration        : %.0f seconds%s",
		  (float)duration, VTY_NEWLINE);

#endif // G2MPLS
}


bool
getCalls(size_t & vtyCalls, size_t & nccCalls, std::string & err)
{
	vtyCalls = vtyCallContainer.size();

	nccCalls = 0;

	err = "";

#if HAVE_OMNIORB
	GUNIGW_DBG("Requesting list of Calls");

	CallController::Mgmt_var  nccMgmtProxy;

	nccMgmtProxy = GUNIGW_MASTER.getNccProxy();

	if (CORBA::is_nil(nccMgmtProxy)) {
		GUNIGW_ERR("Cannot get Corba nccMgmtProxy client object "
			  "previously set up");
		assert(0);
	}

	CallController::Mgmt::callIdentSeq_var s;

	try {
		s = nccMgmtProxy->getCalls();
	} catch (Types::InternalProblems & e) {
		err += "getCalls through CORBA Mgmt ";
		err += "interface failed for internal problems ";
		err += "(reason= ";
		err += e.what;
		err += ")";
		return false;
	} catch (Types::CannotFetch & e) {
		err += "getCalls through CORBA Mgmt ";
		err += "interface failed for fetching problems ";
		err += "(reason= ";
		err += e.what;
		err += ")";
		return false;
	} catch (...) {
		err += "getCalls through CORBA Mgmt ";
		err += "interface failed for unknown reason";
		return false;
	}

	nccCalls = s->length();
#endif // HAVE_OMNIORB

	return true;
}



bool
getTna(const call_ident_t & cid,
       call_info_t &        cdata,
       bool                 isEgress,
       std::string &        err)
{
#if HAVE_OMNIORB
	CallController::Mgmt_var       nccMgmtProxy;
	g2mplsTypes::callIdent_var     id;
	g2mplsTypes::tnaResource_var   tnaRes;
	g2mplsTypes::gridParams_var    gnsTnaRes;
	g2mplsTypes::resourcePosition  pos;
	bool                           found;
	res_spec_t *                   myTna;

	err   = "";
	found = false;
	myTna = &(cdata.iTNA_res);
	pos   = g2mplsTypes::RESOURCEPOSITION_INGRESS;

	nccMgmtProxy = GUNIGW_MASTER.getNccProxy();

	if (CORBA::is_nil(nccMgmtProxy)) {
		GUNIGW_ERR("Cannot get Corba nccMgmtProxy client object "
			  "previously set up");
		assert(0);
	}

	try {
		id << cid;
	} catch (std::out_of_range & e) {
		err += "Exception occurred during callId translation (%s)";
		err += e.what();
		return false;
	}

	if (isEgress) {
		pos = g2mplsTypes::RESOURCEPOSITION_EGRESS;
		myTna = &(cdata.eTNA_res);
	}

	// at first try to get net data
	try {
		found = nccMgmtProxy->callGetTna(id, pos, tnaRes);
	} catch (...) {
		BITMASK_BITRESET(myTna->mask_, net);
		found = false;
	}

	if (found) {
		try {
			myTna->net  << tnaRes;
		} catch (...) {
			err += "Exception occurred during tna translation";
			return false;
		}

		BITMASK_BITSET(myTna->mask_, net);
	}

	// then try to get grid data
	found = false;
	try {
		found = nccMgmtProxy->callGetGnsTna(id, pos, gnsTnaRes);
	} catch (...) {
		BITMASK_BITRESET(myTna->mask_, grid);
		found = false;
	}

	if (found) {
		try {
			myTna->grid  << gnsTnaRes;
		} catch (...) {
			err += "Exception occurred during "
				"grid tna translation";
			return false;
		}

		BITMASK_BITSET(myTna->mask_, grid);
	}

	if (isEgress) {
		if (BITMASK_BITTEST(cdata.eTNA_res.mask_, net) ||
		    BITMASK_BITTEST(cdata.eTNA_res.mask_, grid)  ) {
			BITMASK_BITSET(cdata.mask_, eTNA_res);
		}
	} else {
		if (BITMASK_BITTEST(cdata.iTNA_res.mask_, net) ||
		    BITMASK_BITTEST(cdata.iTNA_res.mask_, grid)  ) {
			BITMASK_BITSET(cdata.mask_, iTNA_res);
		}
	}

	return true;
#else
	return false;
#endif // HAVE_OMNIORB
}

bool gunigw_lock(bool b, int lock)
{
    if (lock == 0)
	GUNIGW_LOCK_SR = b;
    if (lock == 1)
	GUNIGW_LOCK_SI = b;
    return true;
}

bool is_gunigw_locked(int lock)
{
    if (lock == 0)
        return GUNIGW_LOCK_SR;
    if (lock == 1)
        return GUNIGW_LOCK_SI;
}

