//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//  Copyright (C) 2006, 2007, 2008, 2009 University of Essex.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//  Eduard Escalona       (Univ. of Essex)   <eescal_at_essex.ac.uk>
//



#ifndef __GUNIGW_UTILS_HH__
#define __GUNIGW_UTILS_HH__

#include "g2mpls_types.h"
#include "g2mpls_utils.h"

#include "vty.h"

#include "gunigwd.hh"
#include "gunigw_trace.hh"

typedef struct call_data {
	call_ident_t		cid;
	call_info_t		cdata;
	recovery_info_t		rdata;
	lsp_info_t		ldata;
} call_data_t;

//
//
//  CALL container
//
//
#define CALLCTRL_SCOPE "NULL, selected by CallCtrl"
#define CALLCTRL_WORK "(if " CALLCTRL_SCOPE")"

#define BAR1 "========================================"	\
             "========================================"
#define BAR2 "----------------------------------------"	\
             "----------------------------------------"
#define SHOWLIST_DEPTH 50


typedef uint64_t mapKey_t;	// local_id

char *              hostsrv_ipv4_addr();

std::string		key2string(const mapKey_t &            key);

bool			vtyCallCommittable(const mapKey_t &    callKey);

void			vtyDump(struct vty *                   vty,
				const call_ident_t &           cid);
void			vtyDump(struct vty *                   vty,
				const reco_bundle_ident_t &    rid);
void			vtyDump(struct vty *                   vty,
				const lsp_ident_t &            lid);
void			vtyDump(struct vty *                   vty,
				const res_spec_t &             res);
void			vtyDump(struct vty *                   vty,
				const call_info_t &            cdata);
void			vtyDump(struct vty *                   vty,
				const recovery_info_t &        rdata);
void			vtyDump(struct vty *                   vty,
				const lsp_info_t &             ldata);

void			vtyCallDumpVty(struct vty *            vty);
void			vtyCallDumpVty(struct vty *            vty,
				       const mapKey_t &        callKey);

bool			vtyCallUpdate(const mapKey_t &         callKey,
				      const uint32_t &         ituCountryCode,
				      const uint64_t &         ituCarrierCode,
				      const uint64_t &         uniqueAP,
				      std::string &            err);
bool			vtyCallUpdate(const mapKey_t &         callKey,
				      const call_info_t &      cinfo,
				      std::string &            err);
bool			vtyCallUpdate(const mapKey_t &         callKey,
				      const recovery_info_t &  rinfo,
				      std::string &            err);
bool			vtyCallUpdate(const mapKey_t &         callKey,
				      const lsp_info_t &       linfo,
				      std::string &            err);
bool			vtyCallCommit(const mapKey_t &	       callKey,
				      std::string &            err);

bool			getCalls(size_t &           vtyCalls,
				 size_t &           nccCalls,
				 std::string &      err);
bool			getTna(const call_ident_t & cid,
			       call_info_t &        cdata,
			       bool                 isEgress,
			       std::string &        err);
bool			defaultGridResSpec(res_spec_t & res);
bool            gunigw_lock(bool, int);
bool            is_gunigw_locked(int);
#endif /* __GUNIGW_UTILS_HH__ */

