//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//  Copyright (C) 2006, 2007, 2008, 2009 University of Essex.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//  Eduard Escalona       (Univ. of Essex)   <eescal_at_essex.ac.uk>
//



#include "hg2gwd.hh"

//
// Statics initialization
//
HG2GW_Master *                  HG2GW_Master::instance_     = 0;
struct thread_master *          HG2GW_Master::master_       = 0; // thread master
time_t                          HG2GW_Master::startTime_    = 0; // start time
bool                            HG2GW_Master::dryrun_       = false;
uint32_t                        HG2GW_Master::routerId_     = 0;
#if HAVE_OMNIORB
CallController::Mgmt_var        HG2GW_Master::nccMgmtProxy_ = 0;
CallController::EastWest_var	HG2GW_Master::nccEWProxy_   = 0;
g2mplsTopology::Info_var	HG2GW_Master::ospfProxy_    = 0;
LRM::Info_var                   HG2GW_Master::lrmProxy_     = 0;
#endif // HAVE_OMNIORB

//
// CLASS HG2GW_Master implementation
//
HG2GW_Master::HG2GW_Master(void)
{
	METIN();

	HG2GW_DBG("Called HG2GW_Master constructor");

	METOUT();
}

HG2GW_Master::~HG2GW_Master(void)
{
	METIN();

	HG2GW_DBG("Called HG2GW_Master destructor");

	METOUT();
}

bool
HG2GW_Master::init(void)
{
	METIN();

	HG2GW_DBG("Initialising HG2GW_Master");

	instance_ = new HG2GW_Master;
	if (!instance_) {
		HG2GW_ERR("Cannot allocate enough memory ...");
		METOUT();
		assert(0);
	}

	master_ = thread_master_create();
	if (!master_) {
		HG2GW_ERR("Cannot create ZEBRA thread master ");
		assert(0);
	}

	startTime_ = quagga_time(NULL);

	METOUT();

	return true;
}

bool
HG2GW_Master::destroy(void)
{
	METIN();

	HG2GW_DBG("Finalising HG2GW_Master");

	startTime_    = 0;

#if HAVE_OMNIORB
	nccMgmtProxy_   = 0;
	nccEWProxy_     = 0;
	ospfProxy_      = 0;
	lrmProxy_	= 0;
#endif // HAVE_OMNIORB

	delete instance_;
	instance_ = 0;

	thread_master_free(master_);
	master_ = 0;

	METOUT();

	return true;
}

HG2GW_Master &
HG2GW_Master::instance(void)
{
	METIN();

	if (instance_ == 0) {
		HG2GW_ERR("HG2GW_Master singleton instance is empty\n");

		instance_ = new HG2GW_Master;
		if (!instance_) {
			//HG2GW_ERR("Cannot allocate enough memory ...");
			METOUT();
			assert(0);
		}
	}

	METOUT();

	return * instance_;
}

struct thread_master *
HG2GW_Master::getMaster(void)
{
	METIN();

	//HG2GW_DBG("returning HG2GW master of threads");
	
        METOUT();

	return master_;
}

#if HAVE_OMNIORB
CallController::Mgmt_var &
HG2GW_Master::getNccProxy(void)
{
	METIN();

	METOUT();

	return nccMgmtProxy_;
}

CallController::EastWest_var &
HG2GW_Master::getNccEWProxy(void)
{
	METIN();

	METOUT();

	return nccEWProxy_;
}

g2mplsTopology::Info_var &
HG2GW_Master::getospfProxy(void)
{
	METIN();

	METOUT();

	return ospfProxy_;
}

LRM::Info_var &
HG2GW_Master::getLrmProxy(void)
{
	METIN();

	METOUT();

	return lrmProxy_;
}
#endif // HAVE_OMNIORB

bool
HG2GW_Master::dryrun(void)
{
	METIN();

	METOUT();

	return dryrun_;
}

void
HG2GW_Master::dryrun(bool value)
{
	METIN();

	dryrun_ = value;

	METOUT();
}

uint32_t
HG2GW_Master::routerId(void)
{
	METIN();

	METOUT();

	return routerId_;
}

void
HG2GW_Master::routerId(uint32_t value)
{
	METIN();

	routerId_ = value;

	METOUT();
}
