
#include "CallController.hh"

//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 University of Essex.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Eduard Escalona       (Univ. of Essex)   <eescal_at_essex.ac.uk>
//


#include "networkReservationBindingServer.hh"

int
nRes_thread()
{
    networkReservationBindingService nres;
    
    HG2GW_DBG("Starting nRes Service in Port 8080");
    if (nres.run(8080))
    {
        HG2GW_ERR("Error starting nRes process");
        printf("ERROR\n");
        nres.soap_stream_fault(std::cerr);
        exit(-1);
    }
    
    HG2GW_DBG("Exiting nRes_thread...");
    
    return nRes_OK;
}

int
init_nRes()
{
    pthread_t tid;
                
    if (!pthread_create(&tid, NULL, (void*(*)(void*))nRes_thread, NULL))
    {
        HG2GW_ERR("Error creating inRes thread");
        return nRes_ERR;
    }
    
    return nRes_OK;
}

int
networkReservationBindingService::createReservation(_rsv__createReservation *CR, _rsv__createReservationResponse *CRR)
{
    struct soap soap;
    call_data_t     *data = new call_data_t;
    net_res_spec_t  nres;
    std::string     err;
    struct timeval  timeout;
    struct timeval  time_tmp;

    HG2GW_DBG("nRes Call: createReservation");
    soap_init(&soap);

    memset(&data->cdata, 0, sizeof(call_info_t));

    if (CR->createReservation)
    {
        if (CR->createReservation->createReservation)
        {
            if (CR->createReservation->createReservation->Service)
            {
                HG2GW_DBG("ServiceID = %d", CR->createReservation->createReservation->Service[0]->ServiceID);
                memset(&data->cid, 0, sizeof(call_ident_t));
                data->cid.type                       = CALLID_OPSPEC;
                data->cid.local_id                   = CR->createReservation->createReservation->Service[0]->ServiceID;
                data->cid.src_addr.type              = IPv4;
                data->cid.src_addr.preflen           = 32;
                data->cid.src_addr.value.ipv4.s_addr = htonl(HG2GW_MASTER.routerId());

		data->cdata.call_type	 = CALL_TYPE_aEGWzUGW;

                // iTNA res data
                memset(&nres, 0, sizeof(nres));
                BITMASK_BITSET(data->cdata.mask_, iTNA_res);
                BITMASK_BITSET(data->cdata.iTNA_res.mask_, net);
                nres.tna.type       = IPv4;
                nres.tna.preflen    = 32;
                inet_aton(CR->createReservation->createReservation->Service[0]->Connections[0]->Source->EndpointId, &(nres.tna.value.ipv4));
                BITMASK_BITSET(nres.mask_, tna);
                memcpy(&(data->cdata.iTNA_res.net), &nres, sizeof(nres));

                // eTNA res data
                memset(&nres, 0, sizeof(nres));
                BITMASK_BITSET(data->cdata.mask_, eTNA_res);
                // eTNA res data (net)
                BITMASK_BITSET(data->cdata.eTNA_res.mask_, net);
                nres.tna.type       = IPv4;
                nres.tna.preflen    = 32;
                //inet_aton(HOST_CANDIDATE((bes__CreateActivityType *)CC), &(nres.tna.value.ipv4));
                inet_aton(CR->createReservation->createReservation->Service[0]->Connections[0]->Target[0]->EndpointId, &(nres.tna.value.ipv4));
                BITMASK_BITSET(nres.mask_, tna);
                memcpy(&(data->cdata.eTNA_res.net), &nres, sizeof(nres));

                // Advance Reservations
                BITMASK_BITSET(data->cdata.mask_, times);
                data->cdata.times.start_time = CR->createReservation->createReservation->Service[0]->union_ServiceConstraintType.FixedReservationConstraints->StartTime;
                data->cdata.times.end_time   = CR->createReservation->createReservation->Service[0]->union_ServiceConstraintType.FixedReservationConstraints->StartTime +
                                               CR->createReservation->createReservation->Service[0]->union_ServiceConstraintType.FixedReservationConstraints->Duration;

                // recovery info data
                memset(&data->rdata, 0, sizeof(recovery_info_t));

                // lsp info data
                memset(&data->ldata, 0, sizeof(lsp_info_t));
                BITMASK_BITSET(data->ldata.mask_, sw_cap);
                data->ldata.sw_cap          = SWCAP_FSC;
                BITMASK_BITSET(data->ldata.mask_, enc_type);
                data->ldata.enc_type        = ENCT_FIBER;
                BITMASK_BITSET(data->ldata.mask_, gpid);
                data->ldata.gpid            = GPID_ETHERNET;
                BITMASK_BITSET(data->ldata.mask_, link_prot_mask);
                data->ldata.link_prot_mask  = PROTTYPE_NONE;
                BITMASK_BITSET(data->ldata.mask_, crankback);
                data->ldata.crankback       = CB_NONE;
                BITMASK_BITSET(data->ldata.mask_, type);
                data->ldata.type            = LSP_TYPE_SC;
                BITMASK_BITSET(data->ldata.mask_, role);
                data->ldata.role            = LSP_ROLE_PRIMARY;
                BITMASK_BITSET(data->ldata.mask_, action);
                data->ldata.action          = RESOURCE_XCONNECT;
                BITMASK_BITSET(data->ldata.mask_, rro_mode);
                data->ldata.rro_mode        = LSP_RRO_OFF;

                // Bandwidth
                BITMASK_BITSET(data->ldata.mask_, bw);
                HG2GW_DBG("Bandwidth = %d", CR->createReservation->createReservation->Service[0]->Connections[0]->MinBW);
                data->ldata.bw = CR->createReservation->createReservation->Service[0]->Connections[0]->MinBW;
                HG2GW_DBG("Going to create a HG2GW CALL with localID %d", data->cid.local_id);

#if HAVE_OMNIORB
                CallController::Mgmt_var         nccMgmtProxy;

                nccMgmtProxy = HG2GW_MASTER.getNccProxy();

                if (CORBA::is_nil(nccMgmtProxy)){
                    HG2GW_ERR("Cannot get Corba client object "
                        "previously set up");
                    assert(0);
                }

                g2mplsTypes::callIdent_var id;

                try {
                    id << data->cid;
                } catch (std::out_of_range & e) {
                    err  = "Exception occurred during callId translation (";
                    err += e.what();
                    err += ")";
                    HG2GW_ERR("%s", err.c_str());
                    return false;
                } catch (...) {
                    err += "Unknown exception during callId translation";
                    HG2GW_ERR("%s", err.c_str());
                    return false;
                }

                g2mplsTypes::callParams_var callInfo;

                try {
                    callInfo << data->cdata;
                } catch (std::out_of_range & e) {
                    err  = "Exception occurred during callInfo translation (";
                    err += e.what();
                    err += ")";
                    HG2GW_ERR("%s", err.c_str());
                    return false;
                } catch (...) {
                    err += "Unknown exception during callInfo translation";
                    HG2GW_ERR("%s", err.c_str());
                    return false;
                }

                g2mplsTypes::recoveryParams_var recoveryInfo;

                try {
                    recoveryInfo << data->rdata;
                } catch (std::out_of_range & e) {
                    err  = "Exception occurred during recoveryInfo translation (";
                    err += e.what();
                    err += ")";
                    HG2GW_ERR("%s", err.c_str());
                    return false;
                } catch (...) {
                    err += "Unknown exception during recoveryInfo translation";
                    HG2GW_ERR("%s", err.c_str());
                    return false;
                }

                g2mplsTypes::lspParams_var lspInfo;

                try {
                    lspInfo << data->ldata;
                } catch (std::out_of_range & e) {
                    err  = "Exception occurred during lspInfo translation (";
                    err += e.what();
                    err += ")";
                    HG2GW_ERR("%s", err.c_str());
                    return false;
                } catch (...) {
                    err += "Unknown exception during lspInfo translation";
                    HG2GW_ERR("%s", err.c_str());
                    return false;
                }

               	g2mplsTypes::aaaParams_var aaaInfo;
               	aaaInfo = new g2mplsTypes::aaaParams;
		if (CR->createReservation->createReservation->Token)
		{	
			aaaInfo->authzToken = CORBA::string_dup(CR->createReservation->createReservation->Token);
		}
		else
			aaaInfo->authzToken = "";

                g2mplsTypes::actorInfo_var actor;
                actor = new g2mplsTypes::actorInfo;

                actor->issuer       = g2mplsTypes::ISSUERTYPE_MANAGEMENT_IF;
                actor->forceCommand = false;

                try {
                    nccMgmtProxy->callCreate(id          ,
                                             callInfo    ,
                                             recoveryInfo,
                                             lspInfo     ,
                                             aaaInfo     ,
                                             actor       );
                } catch (Types::InternalProblems & e) {
                    err += "callCreate through CORBA Mgmt ";
                    err += "interface failed for internal problems (reason= ";
                    err += e.what;
                    err += ")";
                    HG2GW_ERR("%s", err.c_str());
                    return false;
                } catch (Types::CannotFetch & e) {
                    err += "callCreate through CORBA Mgmt ";
                    err += "interface failed for fetching problems (reason= ";
                    err += e.what;
                    err += ")";
                    HG2GW_ERR("%s", err.c_str());
                    return false;
                } catch (...) {
                    err += "callCreate through CORBA Mgmt ";
                    err += "interface failed for unknown reason";
                    HG2GW_ERR("%s", err.c_str());
                    return false;
                }
                HG2GW_DBG("Sent callCreate through CORBA Mgmt interface");

                HG2GW_DBG("Created call with identity [%s]",
                          ((id->idType == g2mplsTypes::CALLIDTYPE_GLOBUNIQ) ?
                           "GLOBALLY UNIQUE" : "OPERATOR SPECIFIC"));
                HG2GW_DBG("\t - srcAddr             : %s",
                          addr_ntoa(data->cid.src_addr));
                HG2GW_DBG("\t - localId             : %s",
                          key2string(data->cid.local_id).c_str());

#if NCC_GLOBAL_CALLID
                if (id->idType == g2mplsTypes::CALLIDTYPE_GLOBUNIQ) {
                        HG2GW_DBG("\t - Globally unique segments");
                        HG2GW_DBG("\t     - international : 0x%02x%02x%02x",
                                  id->segs.intlSeg[0],
                                  id->segs.intlSeg[1],
                                  id->segs.intlSeg[2]);
                        HG2GW_DBG("\t     - national      : 0x%08x%08x%08x",
                                  id->segs.natlSeg[0],
                                  id->segs.natlSeg[1],
                                  id->segs.natlSeg[2]);
                }
#endif

                if (BITMASK_BITTEST(data->cdata.mask_, iTNA_res))
                {
                    g2mplsTypes::resourcePosition  pos;
                    pos = g2mplsTypes::RESOURCEPOSITION_INGRESS;
                    if (BITMASK_BITTEST(data->cdata.iTNA_res.mask_, net))
                    {
                        g2mplsTypes::tnaResource_var  tnaRes;
                        try {
                            tnaRes << data->cdata.iTNA_res.net;
                        } catch (std::out_of_range & e) {
                            err  = "Exception occurred during ";
                            err += "translation of iTNA_res ";
                            err += "network params(";
                            err += e.what();
                            err += ")";
                            HG2GW_ERR("%s", err.c_str());
                            return false;
                        }

                        try {
                            nccMgmtProxy->callSetTna(id    ,
                                                     pos   ,
                                                     tnaRes,
                                                     actor );
                        } catch (Types::InternalProblems & e) {
                            err += "callSetTna through CORBA Mgmt ";
                            err += "interface failed for internal ";
                            err += "problems (reason= ";
                            err += e.what;
                            err += ")";
                            HG2GW_ERR("%s", err.c_str());
                            return false;
                        } catch (Types::CannotFetch & e) {
                                err += "callSetTna through CORBA Mgmt ";
                            err += "interface failed for fetching ";
                            err += "problems (reason= ";
                            err += e.what;
                            err += ")";
                            HG2GW_ERR("%s", err.c_str());
                            return false;
                        } catch (...) {
                            err += "callSetTna through CORBA Mgmt ";
                            err += "interface failed for unknown reason";
                            HG2GW_ERR("%s", err.c_str());
                            return false;
                        }

                        HG2GW_DBG("Sent callSetTna for iTNA "
                                  "through CORBA Mgmt interface");
                    }
                }

                if (BITMASK_BITTEST(data->cdata.mask_, eTNA_res))
                {
                    g2mplsTypes::resourcePosition  pos;

                    pos = g2mplsTypes::RESOURCEPOSITION_EGRESS;

                    if (BITMASK_BITTEST(data->cdata.eTNA_res.mask_, net)) {
                        g2mplsTypes::tnaResource_var  tnaRes;

                        try {
                            tnaRes << data->cdata.eTNA_res.net;
                        } catch (std::out_of_range & e) {
                            err  = "Exception occurred during ";
                            err += "translation of eTNA_res ";
                            err += "network params(";
                            err += e.what();
                            err += ")";
                            HG2GW_ERR("%s", err.c_str());
                            return false;
                        }

                        try {
                            nccMgmtProxy->callSetTna(id    ,
                                                     pos   ,
                                                     tnaRes,
                                                     actor );
                        } catch (Types::InternalProblems & e) {
                            err += "callSetTna through CORBA Mgmt ";
                            err += "interface failed for internal ";
                            err += "problems (reason= ";
                            err += e.what;
                            err += ")";
                            HG2GW_ERR("%s", err.c_str());
                            return false;
                        } catch (Types::CannotFetch & e) {
                            err += "callSetTna through CORBA Mgmt ";
                            err += "interface failed for fetching ";
                            err += "problems (reason= ";
                            err += e.what;
                            err += ")";
                            HG2GW_ERR("%s", err.c_str());
                            return false;
                        } catch (...) {
                            err += "callSetTna through CORBA Mgmt ";
                            err += "interface failed for unknown reason";
                            HG2GW_ERR("%s", err.c_str());
                            return false;
                        }

                        HG2GW_DBG("Sent callSetTna for eTNA "
                                   "through CORBA Mgmt interface");
                    }

                    hg2gw_lock(true);
                    gettimeofday(&time_tmp, NULL);

                    try {
                        nccMgmtProxy->callSetUp(id, actor);
                    } catch (Types::InternalProblems & e) {
                            err += "callSetUp through CORBA Mgmt ";
                            err += "interface failed for internal problems ";
                            err += "(reason= ";
                            err += e.what;
                            err += ")";
                            HG2GW_ERR("%s", err.c_str());
                            return false;
                    } catch (Types::CannotFetch & e) {
                            err += "callSetUp through CORBA Mgmt ";
                            err += "interface failed for fetching problems ";
                            err += "(reason= ";
                            err += e.what;
                            err += ")";
                            HG2GW_ERR("%s", err.c_str());
                            return false;
                    } catch (...) {
                            err += "callSetUp through CORBA Mgmt ";
                            err += "interface failed for unknown reason";
                            HG2GW_ERR("%s", err.c_str());
                            return false;
                    }

                    HG2GW_DBG("Sent callSetUp through "
                               "CORBA Mgmt interface");
                }

                // EastWest Interface
                // Wait for Event
                gettimeofday(&timeout, NULL);
                while(is_hg2gw_locked() && (timeout.tv_sec - time_tmp.tv_sec) < 20)
                {
                    gettimeofday(&timeout, NULL);
                }

                if (!is_hg2gw_locked())
                {
                    /*CallController::EastWest_var         nccEastWestProxy;

                    nccEastWestProxy = HG2GW_MASTER.getNccEWProxy();

                    if (CORBA::is_nil(nccEastWestProxy)){
                        HG2GW_ERR("Cannot get Corba client object "
                            "previously set up");
                        assert(0);
                    }
        	    
		    g2mplsTypes::actorInfo_var actor2;
		    actor2 = new g2mplsTypes::actorInfo;

		    actor2->issuer       = g2mplsTypes::ISSUERTYPE_MANAGEMENT_IF;
		    actor2->forceCommand = false;

                    try {
                            nccEastWestProxy->callRaiseMsgEvent(id,
                                    g2mplsTypes::CALLMESSAGETYPE_SETUPCONFIRM,
                                    "",
                                    actor2);
                    } catch (Types::InternalProblems & e) {
                            err += "callRaiseMsgEvent through CORBA EastWest ";
                            err += "interface failed for internal problems ";
                            err += "(reason= ";
                            err += e.what;
                            err += ")";
                            HG2GW_ERR("%s", err.c_str());
                            return false;
                    } catch (Types::CannotFetch & e) {
                            err += "callRaiseMsgEvent through CORBA EastWest ";
                            err += "interface failed for fetching problems ";
                            err += "(reason= ";
                            err += e.what;
                            err += ")";
                            HG2GW_ERR("%s", err.c_str());
                            return false;
                    } catch (...) {
                            err += "callRaiseMsgEvent through CORBA EastWest ";
                            err += "interface failed for unknown reason";
                            HG2GW_ERR("%s", err.c_str());
                            return false;
                    }

		    HG2GW_DBG("Sent callRaiseMsgEvent(Confirm) through "
		              "CORBA Mgmt interface");*/

                }
                else
                {
                    HG2GW_DBG("Timeout!", err.c_str());

		    try {
		        nccMgmtProxy->callSetDown(id, actor);
		    } catch (Types::InternalProblems & e) {
		        err += "callSetDown through CORBA Mgmt ";
		        err += "interface failed for internal problems ";
		        err += "(reason= ";
		        err += e.what;
		        err += ")";
		       	HG2GW_ERR("%s", err.c_str());
		        return false;
		    } catch (Types::CannotFetch & e) {
		        err += "callSetDown through CORBA Mgmt ";
		        err += "interface failed for fetching problems ";
		        err += "(reason= ";
		        err += e.what;
		        err += ")";
		       	HG2GW_ERR("%s", err.c_str());
		        return false;
		    } catch (...) {
		        err += "callSetDown through CORBA Mgmt ";
		        err += "interface failed for unknown reason";
		       	HG2GW_ERR("%s", err.c_str());
		        return false;
		    }

		    HG2GW_DBG("Sent callDestroy through CORBA Mgmt interface");
                }
#endif // HAVE_OMNIORB

                /*CRR->createReservationResponse = new rsv__CreateReservationResponseEncapsulated;
                CRR->createReservationResponse->createReservationResponse = new rsv__CreateReservationResponseType;
                CRR->createReservationResponse->createReservationResponse->ReservationID = new char[15];
                sprintf(CRR->createReservationResponse->createReservationResponse->ReservationID, "%d", data->cid.local_id);*/
		CRR->__size = 1;
		CRR->__any = new char*;
		CRR->__any[0] = new char[255];
		sprintf(CRR->__any[0], "<rsv:createReservationResponse><rsv:createReservationResponse><rsv:JobID>%d</rsv:JobID><rsv:ReservationID>%d</rsv:ReservationID><rsv:GRI></rsv:GRI><rsv:Token></rsv:Token></rsv:createReservationResponse></rsv:createReservationResponse>", CR->createReservation->createReservation->Service[0]->ServiceID, CR->createReservation->createReservation->Service[0]->ServiceID);
            }
        }
    }

    HG2GW_DBG("nRes Call: Returning createReservation");
    
    return SOAP_OK;
}

int
networkReservationBindingService::getReservation(_rsv__getReservation *GR, _rsv__getReservationResponse *GRR)
{
    struct soap soap;
    HG2GW_DBG("nRes Call: getReservation");
    soap_init(&soap);

    HG2GW_DBG("nRes Call: Returning getReservation");
    return SOAP_OK;
}

int
networkReservationBindingService::getStatus(_rsv__getStatus *GS, _rsv__getStatusResponse *GSR)
{
    struct soap soap;
    HG2GW_DBG("nRes Call: getStatus");
    soap_init(&soap);

    GSR->__size = 1;
    GSR->__any = new char*;
    GSR->__any[0] = new char[255];
    //sprintf(GSR->__any[0], "<rsv:getStatusResponse><rsv:getStatusResponse><rsv:ServiceStatus><rsv:ServiceID>%d</rsv:ServiceID><rsv:Status>active</rsv:Status><rsv:DomainStatus><rsv:Domain></rsv:Domain><rsv:Status>active</rsv:Status></rsv:DomainStatus><rsv:Connections><rsv:Status>active</rsv:Status></rsv:Connections></rsv:ServiceStatus></rsv:getStatusResponse></rsv:getStatusResponse>", CR->createReservation->createReservation->Service[0]->ServiceID, CR->createReservation->createReservation->Service[0]->ServiceID);
    sprintf(GSR->__any[0], "<rsv:getStatusResponse><rsv:getStatusResponse></rsv:getStatusResponse></rsv:getStatusResponse>");

    HG2GW_DBG("nRes Call: Returning getStatus");
    return SOAP_OK;
}

int
networkReservationBindingService::cancelReservation(_rsv__cancelReservation *CR, _rsv__cancelReservationResponse *CRR)
{
    struct soap soap;
    HG2GW_DBG("nRes Call: cancelReservation");
    soap_init(&soap);

    call_data_t     *data = new call_data_t;
    std::string     err;

    CRR->__size = 1;
    CRR->__any = new char*;
    CRR->__any[0] = new char[255];
    sprintf(CRR->__any[0], "<rsv:cancelReservationResponse><rsv:cancelReservationResponse><rsv:Success>false</rsv:Success></rsv:cancelReservationResponse></rsv:cancelReservationResponse>");

    memset(&data->cid, 0, sizeof(call_ident_t));
    data->cid.type                       = CALLID_OPSPEC;
    data->cid.local_id                   = atoi(CR->cancelReservation->cancelReservation->ReservationID);
    data->cid.src_addr.type              = IPv4;
    data->cid.src_addr.preflen           = 32;
    data->cid.src_addr.value.ipv4.s_addr = htonl(HG2GW_MASTER.routerId());

    HG2GW_DBG("Going to destroy a HG2GW CALL with localID %d", data->cid.local_id);

#if HAVE_OMNIORB

    CallController::Mgmt_var nccMgmtProxy;

    nccMgmtProxy = HG2GW_MASTER.getNccProxy();

    if (CORBA::is_nil(nccMgmtProxy)) {
            HG2GW_ERR("Cannot get Corba nccMgmtProxy client "
                      "object previously set up");
            assert(0);
    }

    g2mplsTypes::callIdent_var id;
    // id = new g2mplsTypes::callIdent;
    try {
        id << data->cid;
    } catch (std::out_of_range & e) {
        err  = "Exception occurred during "
               "callId translation (";
        err += e.what();
        err += ")";
        HG2GW_ERR("%s", err.c_str());
        return false;
    } catch (...) {
        err += "Unknown exception during callId translation";
        HG2GW_ERR("%s", err.c_str());
        return false;
    }

    g2mplsTypes::actorInfo_var actor;
    actor = new g2mplsTypes::actorInfo;

    actor->issuer = g2mplsTypes::ISSUERTYPE_MANAGEMENT_IF;
    actor->forceCommand = true;

    try {
        nccMgmtProxy->callSetDown(id, actor);
    } catch (Types::InternalProblems & e) {
        err += "callSetDown through CORBA Mgmt ";
        err += "interface failed for internal problems ";
        err += "(reason= ";
        err += e.what;
        err += ")";
       	HG2GW_ERR("%s", err.c_str());
        return false;
    } catch (Types::CannotFetch & e) {
        err += "callSetDown through CORBA Mgmt ";
        err += "interface failed for fetching problems ";
        err += "(reason= ";
        err += e.what;
        err += ")";
       	HG2GW_ERR("%s", err.c_str());
        return false;
    } catch (...) {
        err += "callSetDown through CORBA Mgmt ";
        err += "interface failed for unknown reason";
       	HG2GW_ERR("%s", err.c_str());
        return false;
    }
#endif // HAVE_OMNIORB

    HG2GW_DBG("Sent callDestroy through CORBA Mgmt interface");

    sprintf(CRR->__any[0], "<rsv:cancelReservationResponse><rsv:cancelReservationResponse><rsv:Success>true</rsv:Success></rsv:cancelReservationResponse></rsv:cancelReservationResponse>");

    HG2GW_DBG("nRes Call: Returning cancelReservation");
    return SOAP_OK;
}

int
networkReservationBindingService::bind(_rsv__bind*, _rsv__bindResponse*)
{
    struct soap soap;
    return SOAP_OK;
}

int
networkReservationBindingService::activate(_rsv__activate*, _rsv__activateResponse*)
{
    struct soap soap;
    return SOAP_OK;
}

int
networkReservationBindingService::completeJob(_rsv__completeJob*, _rsv__completeJobResponse*)
{
    struct soap soap;
    return SOAP_OK;
}

int
networkReservationBindingService::notification(_rsv__notification*, _rsv__notificationResponse*)
{
    struct soap soap;
    return SOAP_OK;
}

int
networkReservationBindingService::getReservations(_rsv__getReservations*, _rsv__getReservationsResponse*)
{
    struct soap soap;
    return SOAP_OK;
}

int
networkReservationBindingService::cancelJob(_rsv__cancelJob*, _rsv__cancelJobResponse*)
{
    struct soap soap;
    return SOAP_OK;
}

int
networkReservationBindingService::isAvailable(_rsv__isAvailable *iA, _rsv__isAvailableResponse *iAR)
{
    struct soap soap;
    HG2GW_DBG("nRes Call: isAvailable");
    soap_init(&soap);
    iAR->__size = 1;
    iAR->__any = new char*;
    iAR->__any[0] = new char[255];
    sprintf(iAR->__any[0], "<rsv:isAvailableResponse><rsv:isAvailableResponse><rsv:Availability>available</rsv:Availability></rsv:isAvailableResponse></rsv:isAvailableResponse>");
    HG2GW_DBG("nRes Call: Returning isAvailable");
    return SOAP_OK;
}

