//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//



#include <zebra.h>
#include "log.h"


#ifdef __cplusplus

#include "g2mpls_types.h"

#include <iostream>
#include <fstream>
#include <stdlib.h>
#include <sstream>
#include <iomanip>

using namespace std;

bool vtyGridResSpecUpdate(grid_res_spec_t & dst, const grid_res_spec_t & src)
{
	SELECTIVE_UPDATE2(dst, src, application);
	SELECTIVE_UPDATE2(dst, src, cand_host);

	if (BITMASK_BITTEST(src.mask_, fs_resources)) {
		BITMASK_BITRESET(dst.mask_, fs_resources);
		dst.fs_resources.fs_name    = src.fs_resources.fs_name;
		dst.fs_resources.fs_type    = src.fs_resources.fs_type;
		dst.fs_resources.disk_space = src.fs_resources.disk_space;

		if (src.fs_resources.mount_point) {
			delete dst.fs_resources.mount_point;
		}
		dst.fs_resources.mount_point = src.fs_resources.mount_point;

		if (src.fs_resources.mount_source) {
			delete dst.fs_resources.mount_source;
		}
		dst.fs_resources.mount_source = src.fs_resources.mount_source;

		BITMASK_BITSET(dst.mask_, fs_resources);
	}

	SELECTIVE_UPDATE2(dst, src, sys_caps);
	SELECTIVE_UPDATE2(dst, src, ind_cpu_speed);
	SELECTIVE_UPDATE2(dst, src, ind_cpu_time);
	SELECTIVE_UPDATE2(dst, src, ind_cpu_count);
	SELECTIVE_UPDATE2(dst, src, ind_net_bw);
	SELECTIVE_UPDATE2(dst, src, ind_phy_mem);
	SELECTIVE_UPDATE2(dst, src, ind_vir_mem);
	SELECTIVE_UPDATE2(dst, src, ind_disk_space);
	SELECTIVE_UPDATE2(dst, src, tot_cpu_time);
	SELECTIVE_UPDATE2(dst, src, tot_cpu_count);
	SELECTIVE_UPDATE2(dst, src, tot_phy_mem);
	SELECTIVE_UPDATE2(dst, src, tot_vir_mem);
	SELECTIVE_UPDATE2(dst, src, tot_disk_space);
	SELECTIVE_UPDATE2(dst, src, tot_res_count);

	if (BITMASK_BITTEST(src.mask_, data_staging)) {
		BITMASK_BITRESET(dst.mask_, data_staging);
		dst.data_staging.fs_name	    =
			src.data_staging.fs_name;
		dst.data_staging.creation_flag	    =
			src.data_staging.creation_flag;
		dst.data_staging.del_on_termination =
			src.data_staging.del_on_termination;

		if (src.data_staging.filename) {
			delete dst.data_staging.filename;
		}
		dst.data_staging.filename = src.data_staging.filename;

			if (src.data_staging.source) {
				delete dst.data_staging.source;
			}
			dst.data_staging.source = src.data_staging.source;

			if (src.data_staging.target) {
				delete dst.data_staging.target;
			}
			dst.data_staging.target = src.data_staging.target;

			BITMASK_BITSET(dst.mask_, data_staging);
	}

	SELECTIVE_UPDATE2(dst, src, grid_site_id);

	return true;
}

bool vtyNetResSpecUpdate(net_res_spec_t & dst, const net_res_spec_t & src)
{
	SELECTIVE_UPDATE2(dst, src, tna);
	SELECTIVE_UPDATE2(dst, src, data_link);
	SELECTIVE_UPDATE2(dst, src, label);

	return true;
}

bool vtyResSpecUpdate(res_spec_t & dst, const res_spec_t & src)
{

	if (BITMASK_BITTEST(src.mask_, net)) {
		BITMASK_BITRESET(dst.mask_, net);
		vtyNetResSpecUpdate(dst.net, src.net);
		BITMASK_BITSET(dst.mask_, net);
	}

	if (BITMASK_BITTEST(src.mask_, grid)) {
		BITMASK_BITRESET(dst.mask_, grid);

		vtyGridResSpecUpdate(dst.grid, src.grid);

		BITMASK_BITSET(dst.mask_, grid);
	}

	return true;
}


#endif // __cplusplus
