//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//



#include "zebra.h"
#include "log.h"

#include "lib/corba.h"
#include "idl/lrm.hh"
#include "idl/scngw.hh"
#include "idl/tnrc.hh"

#include <iostream>
#include <string>
#include <stdlib.h>

using namespace std;

#include "lrmd.h"

#if HAVE_OMNIORB
#include "g2mpls_corba_utils.h"
#include "lrm_corba.h"

extern "C" {
int  send_msg_to_scngw(struct thread *t);
void fill_adv_rsrv_calendar(const g2mplsTypes::dataLinkCalendarSeq & calendar,
			    datalink_t *                             dl);
void fill_lambdas_bitmap(const g2mplsTypes::DLinkWdmLambdasBitmap & src,
			 wdm_link_lambdas_bitmap_t &                dst);
}

class LRM_i : public POA_LRM::Info,
	      public PortableServer::RefCountServantBase
{
public:
	inline LRM_i();
	virtual ~LRM_i();

	void localDLinkIdFromRemoteDLinkId(g2mplsTypes::nodeId          nodeId,
					   const g2mplsTypes::DLinkId & remoteDLink,
					   g2mplsTypes::DLinkId &       localDLink,
					   g2mplsTypes::operState &     operState,
					   g2mplsTypes::adminState &    adminState);

	g2mplsTypes::TELinkId TELinkFromDLink(const g2mplsTypes::DLinkId & datalink);

	g2mplsTypes::DLinkId DLinksInDLink(const g2mplsTypes::DLinkIdSeq & datalinks);

	g2mplsTypes::DLinkId DLinkFromTELink(const g2mplsTypes::TELinkId  & telink);

	void TELinksData(g2mplsTypes::TELinkDataSeq & telinks);

	void DLinksData(g2mplsTypes::DLinkDataSeq & datalinks);

	g2mplsTypes::TELinkIdSeq * allTELinkIds();

	g2mplsTypes::TELinkDataSeq * allTELinks(g2mplsTypes::adjType type);

	CORBA::Boolean checkTELinkId(const g2mplsTypes::TELinkId & telink);

	CORBA::Boolean getTELinkStatus(const  g2mplsTypes::TELinkId & telink,
				       g2mplsTypes::operState &       operState,
				       g2mplsTypes::adminState &      adminState);

	CORBA::Boolean getDataLinkStatus(const  g2mplsTypes::DLinkId & datalink,
					 g2mplsTypes::operState &      operState,
					 g2mplsTypes::adminState &     adminState);

	g2mplsTypes::TELinkIdSeq * adjTELinks(g2mplsTypes::adjacencyId     adjacency);
	g2mplsTypes::nodeId NodeIdFromTELink(const g2mplsTypes::TELinkId & telink);

	g2mplsTypes::TELinkId TELinkFromTna(const g2mplsTypes::tnaId & tna);

	g2mplsTypes::TELinkId TELinkFromRemNodeId(const g2mplsTypes::nodeId node);

	g2mplsTypes::nodeId NbrNodeIdFromTna(const g2mplsTypes::tnaId & tna);

	g2mplsTypes::nodeId SingleUniNbrNodeId(void);

	g2mplsTypes::nodeId nodeId();

	void scngw_isup(void);

	void updateDLinkDetails(const g2mplsTypes::linkId &          datalink,
				const g2mplsTypes::DLinkParameters & parms);

};

LRM_i::LRM_i()
{
}

LRM_i::~LRM_i()
{
}

void
LRM_i::localDLinkIdFromRemoteDLinkId(g2mplsTypes::nodeId nodeId,
				     const g2mplsTypes::DLinkId & remoteDLink,
				     g2mplsTypes::DLinkId & localDLink,
				     g2mplsTypes::operState & operState,
				     g2mplsTypes::adminState & adminState)
{
	STACK_LOCK();

	g2mpls_addr_t            tmpAddr;
	g2mplsTypes::DLinkId_var remDLink;
	g2mplsTypes::DLinkId_var lclDLink;
	std::string              remDlinkString;
	std::string              locDlinkString;
	lrm_t *                  lrm;
	struct zlist *           anylist;
	te_link_t *              tel;
	datalink_t *             dl;
	uint32_t                 nodeAddr;
	struct zlistnode *       node_tel;
	struct zlistnode *       node_dl;
	void *                   data_tel;
	void *                   data_dl;

	// As the DL is a remoteDL we need to check also the nodeId!!!
	remDLink = remoteDLink;
	tmpAddr << remDLink;
	//if (tmpAddr.type == UNNUMBERED) {
	//	tmpAddr.value.unnum.node_id = nodeId;
	//}

	remDlinkString << remDLink;
	nodeAddr        = nodeId;

	zlog_debug("Someone is requesting Local DataLink "
		   "from Remote DataLink %s on node %s",
		   remDlinkString.c_str(), numb_htoa(nodeAddr));

	// Check LRM presence
	lrm = lrm_lookup();
	if (lrm == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	anylist = lrm->telink_list;
	if (anylist == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	for (ALL_LIST_ELEMENTS_RO(anylist, node_tel, data_tel)) {
		tel = (te_link_t *) data_tel;
		assert(tel != 0);
		if (tel->rem_node_id == nodeAddr) {
			for (ALL_LIST_ELEMENTS_RO(tel->dl_list, node_dl, data_dl)) {
				dl = (datalink_t *) data_dl;
				assert(dl != 0);
				//if (dl->rem_id == remDLink) {
				if (addr_equal(dl->rem_id, tmpAddr)) {
					lclDLink   << dl->lcl_id;
					localDLink  = lclDLink;
					operState  << dl->op_state;
					adminState << dl->adm_state;

					locDlinkString << lclDLink;

					zlog_debug("Sent Local DataLink %s",
						   locDlinkString.c_str());

					STACK_UNLOCK();
					return;
				}
			}
		}
	}

	zlog_debug("Remote DataLink not found");

	STACK_UNLOCK();
	throw(LRM::Info::InternalProblems());

}

g2mplsTypes::TELinkId
LRM_i::TELinkFromDLink(const g2mplsTypes::DLinkId & datalink)
{
	STACK_LOCK();

	g2mplsTypes::DLinkId_var dlink;
	std::string              dlinkString;
	std::string              telinkString;
	lrm_t *                  lrm;
	struct zlist *           anylist;
	datalink_t *             dl;
	te_link_t *              tel;
	struct zlistnode *       node_tel;
	struct zlistnode *       node_dl;
	void *                   data_tel;
	void *                   data_dl;

	dlink       = datalink;
	dlinkString << dlink;

	zlog_debug("Someone is requesting TELink from DataLink %s",
		   dlinkString.c_str());

	g2mplsTypes::TELinkId_var telink;
	{
		g2mplsTypes::TELinkId * tmp;

		tmp = new g2mplsTypes::TELinkId;
		if (!tmp) {
			STACK_UNLOCK();
			throw(LRM::Info::InternalProblems());
		}
		telink = tmp;
	}

	// Check LRM presence
	lrm = lrm_lookup();
	if (lrm == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	anylist = lrm->telink_list;
	if (anylist == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	for (ALL_LIST_ELEMENTS_RO(anylist, node_tel, data_tel)) {
		tel = (te_link_t *) data_tel;
		assert(tel != 0);
		for (ALL_LIST_ELEMENTS_RO(tel->dl_list, node_dl, data_dl)) {
			dl = (datalink_t *) data_dl;
			assert(dl != 0);
			if (dl->lcl_id == dlink) {
				telink << tel->lcl_id;

				telinkString << telink;

				zlog_debug("Sent TELink %s",
					   telinkString.c_str());
				STACK_UNLOCK();
				return telink._retn();
			}
		}
	}
	zlog_debug("DataLink not found");
	STACK_UNLOCK();
	throw (LRM::Info::UnknownDLink());
}

g2mplsTypes::DLinkId
LRM_i::DLinksInDLink(const g2mplsTypes::DLinkIdSeq & datalinks)
{
	STACK_LOCK();
	g2mplsTypes::DLinkId_var id;
	{
		g2mplsTypes::DLinkId * tmp;

		tmp = new g2mplsTypes::DLinkId;
		if (!tmp) {
			STACK_UNLOCK();
			throw(LRM::Info::InternalProblems());
		}
		id = tmp;
	}
	id->ipv4(0);
	STACK_UNLOCK();
	return id;
}

g2mplsTypes::DLinkId
LRM_i::DLinkFromTELink(const g2mplsTypes::TELinkId  & telink)
{
	STACK_LOCK();

	g2mplsTypes::TELinkId_var corbaTel;
	std::string               telinkString;
	std::string               dlinkString;
	struct zlistnode *        node_tel;
	struct zlistnode *        node_dl;
	void *                    data_tel;
	void *                    data_dl;
	lrm_t *                   lrm;
	struct zlist *            anylist;
	datalink_t *              dl;
	te_link_t *               tel;

	corbaTel      = telink;
	telinkString << corbaTel;

	zlog_debug("Someone is requesting DataLink from TELink %s",
		   telinkString.c_str());

	g2mplsTypes::DLinkId_var dlink;
	{
		g2mplsTypes::DLinkId * tmp;

		tmp = new g2mplsTypes::DLinkId;
		if (!tmp) {
			STACK_UNLOCK();
			throw(LRM::Info::InternalProblems());
		}
		dlink = tmp;
	}

	// Check LRM presence
	lrm = lrm_lookup();
	if (lrm == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	anylist = lrm->telink_list;
	if (anylist == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	for (ALL_LIST_ELEMENTS_RO(anylist, node_tel, data_tel)) {
		tel = (te_link_t *) data_tel;
		assert(tel != 0);
		if (tel->lcl_id == corbaTel) {
			if (!tel->dl_list || list_isempty(tel->dl_list)) {
				zlog_err("NO DataLink for this TELink");
				STACK_UNLOCK();
				throw(LRM::Info::NoDLinks());
			}
			for (ALL_LIST_ELEMENTS_RO(tel->dl_list, node_dl, data_dl)) {
				dl = (datalink_t *) data_dl;
				assert(dl);
				if (dl->op_state == UP) {
					dlink << dl->lcl_id;
					dlinkString << dlink;

					zlog_debug("Sent DataLink %s", dlinkString.c_str());
					STACK_UNLOCK();
					return dlink._retn();
				}
			}
			zlog_debug("No DataLink UP");
			STACK_UNLOCK();
			throw(LRM::Info::NoDLinks());
		}
	}
	zlog_debug("TELink not found");
	STACK_UNLOCK();
	throw (LRM::Info::UnknownTELink());
}

void
LRM_i::TELinksData(g2mplsTypes::TELinkDataSeq & telinks)
{
	STACK_LOCK();

	lrm_t *            lrm;
	struct zlist *     anylist;
	te_link_t *        interface;
	struct zlistnode * node;
	void		 * data;
	int                found;

	zlog_debug("Someone is requesting TELinks Data");

	// Check LRM presence
	lrm = lrm_lookup();
	if (lrm == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	anylist = lrm->telink_list;
	if (anylist == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	for (int i = 0; i < telinks.length(); i++) {
		found = 0;
		for (ALL_LIST_ELEMENTS_RO(anylist, node, data)) {
			interface = (te_link_t *) data;
			assert(interface != 0);

			g2mplsTypes::TELinkId_var lclId;
			g2mplsTypes::TELinkId_var remId;

			lclId = telinks[i].localId;

			if (!(interface->lcl_id == lclId)) {
				continue;
			}
			found = 1;
			//Set correctly the Parameters for current TE-Link
			remId               << interface->rem_id;
			telinks[i].remoteId  = remId;
			telinks[i].neighbour = interface->rem_node_id;
			telinks[i].parms.states.opState  << interface->op_state;
			telinks[i].parms.states.admState << interface->adm_state;

			break;
		}
		if (!found) {
			zlog_debug("TELink not found");
			STACK_UNLOCK();
			throw(LRM::Info::UnknownTELinkIdentity(telinks[i].localId));
		}
	}

	zlog_debug("Sent TELinks Data");

	STACK_UNLOCK();
}

void
LRM_i::DLinksData(g2mplsTypes::DLinkDataSeq & datalinks)
{
	STACK_LOCK();

	lrm_t *            lrm;
	struct zlist *     anylist;
	datalink_t *       dl;
	struct zlistnode * node;
	void *             data;
	int                found;

	zlog_debug("Someone is requesting DLinks Data");

	// Check LRM presence
	lrm = lrm_lookup();
	if (lrm == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	anylist = lrm->datalink_list;
	if (anylist == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	for (int i = 0; i < datalinks.length(); i++) {
		found = 0;
		for (ALL_LIST_ELEMENTS_RO(anylist, node, data)) {
			dl = (datalink_t *) data;
			assert(dl != 0);

			g2mplsTypes::DLinkId_var lclId;
			g2mplsTypes::DLinkId_var remId;

			lclId  = datalinks[i].localId;

			if (!(dl->lcl_id == lclId)) {
				continue;
			}
			found = 1;
			//Set correctly the Parameters for current DataLink
			remId                              << dl->rem_id;
			datalinks[i].remoteId               = remId;
			datalinks[i].parms.states.opState  << dl->op_state;
			datalinks[i].parms.states.admState << dl->adm_state;
			datalinks[i].parms.swCap           << dl->swcap;
			datalinks[i].parms.encType         << dl->enctype;
			datalinks[i].parms.maxBandwidth     = dl->max_bw;
			datalinks[i].parms.maxResBandwidth  = dl->max_res_bw;
			for (int j = 0; i < 8; i++) {
				datalinks[i].parms.availBandwidthPerPrio[j]   =
					dl->avail_bw_per_prio[j];

				datalinks[i].parms.maxLSPbandwidth[j] =
					dl->max_LSP_bw[j];
			}
			datalinks[i].parms.minLSPbandwidth  = dl->min_LSP_bw;

			break;
		}
		if (!found) {
			zlog_debug("DLink not found");
			STACK_UNLOCK();
			throw(LRM::Info::UnknownDLinkIdentity(datalinks[i].localId));
		}
	}
	zlog_debug("Sent DLinks Data");

	STACK_UNLOCK();
}

g2mplsTypes::TELinkIdSeq *
LRM_i::allTELinkIds()
{
	STACK_LOCK();

	lrm_t *            lrm;
	struct zlist *     anylist;
	te_link_t *        interface;
	struct zlistnode * node;
	void *             data;
	int                i;

	zlog_debug("Someone is requesting all TELink Ids");

	// Check LRM presence
	lrm = lrm_lookup();
	if (lrm == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	anylist = lrm->telink_list;
	if (anylist == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	g2mplsTypes::TELinkIdSeq_var seq;
	{
		g2mplsTypes::TELinkIdSeq * tmp;

		tmp = new g2mplsTypes::TELinkIdSeq(listcount(anylist));
		if (!tmp) {
			STACK_UNLOCK();
			throw(LRM::Info::InternalProblems());
		}
		seq = tmp;
	}
	seq->length(listcount(anylist));

	i = 0;
	for (ALL_LIST_ELEMENTS_RO(anylist, node, data)) {
		interface = (te_link_t *) data;
		assert(interface != 0);

		g2mplsTypes::TELinkId_var lclId;

		lclId << interface->lcl_id;
		seq[i] = lclId;

		i++;
	}

	zlog_debug("Sent all TELink Ids");

	STACK_UNLOCK();

	return seq._retn();

}

g2mplsTypes::TELinkDataSeq *
LRM_i::allTELinks(g2mplsTypes::adjType type)
{
	STACK_LOCK();

	lrm_t *            lrm;
	struct zlist *     anylist;
	te_link_t *        interface;
	struct zlistnode * node;
	void *             data;
	int                count = 0;
	std::string        typeStr;

	typeStr << type;

	zlog_debug("Someone is requesting %s all TELinks", typeStr.c_str());

	// Check LRM presence
	lrm = lrm_lookup();
	if (lrm == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	anylist = lrm->telink_list;
	if (anylist == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	//TO BE OPTIMIZED WITH A SMART LIST
	for (ALL_LIST_ELEMENTS_RO(anylist, node, data)) {
		interface = (te_link_t *) data;
		assert(interface != 0);
		if (!(interface->adj_type == type)) {
			continue;
		}
		count++;
	}
	if (count == 0) {
		STACK_UNLOCK();
		throw(LRM::Info::NoTELinks());
	}

	g2mplsTypes::TELinkDataSeq_var seq;
	{
		g2mplsTypes::TELinkDataSeq * tmp;

		tmp = new g2mplsTypes::TELinkDataSeq(count);
		if (!tmp) {
			STACK_UNLOCK();
			throw(LRM::Info::InternalProblems());
		}
		seq = tmp;
	}
	seq->length(count);

	int i = 0;

	for (ALL_LIST_ELEMENTS_RO(anylist, node, data)) {
		interface = (te_link_t *) data;
		assert(interface != 0);
		if (!(interface->adj_type == type)) {
			continue;
		}
		g2mplsTypes::TELinkId_var lclId;
		g2mplsTypes::TELinkId_var remId;

		lclId            << interface->lcl_id;
		remId            << interface->rem_id;

		seq[i].localId   = lclId;
		seq[i].remoteId  = remId;

		seq[i].neighbour = interface->rem_node_id;

		seq[i].parms.states.opState  << interface->op_state;
		seq[i].parms.states.admState << interface->adm_state;
		i++;
	}

	zlog_debug("Sent all TELinks");

	STACK_UNLOCK();

	return seq._retn();

}

CORBA::Boolean
LRM_i::checkTELinkId(const g2mplsTypes::TELinkId & telink)
{
	STACK_LOCK();

	g2mplsTypes::TELinkId_var corbaTel;
	std::string               telinkString;
	lrm_t *                   lrm;
	struct zlist *            anylist;
	te_link_t *               tel;
	struct zlistnode *        node;
	void *                    data;

	corbaTel      = telink;
	telinkString << corbaTel;

	zlog_debug("Someone is requesting check TELink %s",
		   telinkString.c_str());

	// Check LRM presence
	lrm = lrm_lookup();
	if (lrm == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	anylist = lrm->telink_list;
	if (anylist == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	for (ALL_LIST_ELEMENTS_RO(anylist, node, data)) {
		tel = (te_link_t *) data;
		assert(tel != 0);
		//zlog_debug("TELinkId %s", addr_ntoa(tel->lcl_id));
		if (tel->lcl_id == corbaTel) {
			if (tel->op_state  == DOWN ||
			    tel->adm_state == DISABLED) {
				zlog_debug("TELink is Down");
				STACK_UNLOCK();
				throw(LRM::Info::TELinkDown());
			}
			zlog_debug("Found TELink");
			STACK_UNLOCK();
			return true;
		}
	}
	zlog_debug("TELink not found");
	STACK_UNLOCK();
	return false;
}

CORBA::Boolean
LRM_i::getTELinkStatus(const  g2mplsTypes::TELinkId & telink,
		       g2mplsTypes::operState &       operState,
		       g2mplsTypes::adminState &      adminState)
{
	STACK_LOCK();

	g2mplsTypes::TELinkId_var corbaTel;
	std::string               telinkString;
	std::string               operStateString;
	std::string               adminStateString;
	lrm_t *                   lrm;
	struct zlist *            anylist;
	te_link_t *               tel;
	struct zlistnode *        node;
	void *                    data;

	corbaTel      = telink;
	telinkString << corbaTel;

	zlog_debug("Someone is requesting status of TELink %s",
		   telinkString.c_str());

	// Check LRM presence
	lrm = lrm_lookup();
	if (lrm == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	anylist = lrm->telink_list;
	if (anylist == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	for (ALL_LIST_ELEMENTS_RO(anylist, node, data)) {
		tel = (te_link_t *) data;
		assert(tel != 0);
		if (tel->lcl_id == corbaTel) {
			operState       << tel->op_state;
			operStateString << operState;

			adminState       << tel->adm_state;
			adminStateString << adminState;

			zlog_debug("Got TELink %s status:", telinkString.c_str());
			zlog_debug("   adminStatus: %s", adminStateString.c_str());
			zlog_debug("   operStatus : %s", operStateString.c_str());
			STACK_UNLOCK();
			return true;
		}
	}

	zlog_debug("TELink %s not found", telinkString.c_str());
	STACK_UNLOCK();
	throw (LRM::Info::UnknownTELink());
}

CORBA::Boolean
LRM_i::getDataLinkStatus(const  g2mplsTypes::DLinkId & datalink,
			 g2mplsTypes::operState &      operState,
			 g2mplsTypes::adminState &     adminState)
{
	STACK_LOCK();

	g2mplsTypes::DLinkId_var corbaDl;
	std::string              datalinkString;
	std::string              operStateString;
	std::string              adminStateString;
	lrm_t *                  lrm;
	struct zlist *           anylist;
        datalink_t *             dl;
	struct zlistnode *       node;
	void *                   data;

	corbaDl         = datalink;
	datalinkString << corbaDl;

	zlog_debug("Someone is requesting status of DataLink %s",
		   datalinkString.c_str());

	// Check LRM presence
	lrm = lrm_lookup();
	if (lrm == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	anylist = lrm->datalink_list;
	if (anylist == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	for (ALL_LIST_ELEMENTS_RO(anylist, node, data)) {
		dl = (datalink_t *) data;
		assert(dl != 0);
		if (dl->lcl_id == corbaDl) {
			operState       << dl->op_state;
			operStateString << operState;

			adminState       << dl->adm_state;
			adminStateString << adminState;

			zlog_debug("Got DataLink %s status:", datalinkString.c_str());
			zlog_debug("   adminStatus: %s", adminStateString.c_str());
			zlog_debug("   operStatus : %s", operStateString.c_str());
			STACK_UNLOCK();
			return true;
		}
	}

	zlog_debug("DataLink %s not found", datalinkString.c_str());
	STACK_UNLOCK();
	throw (LRM::Info::UnknownDLink());
}

g2mplsTypes::TELinkIdSeq *
LRM_i::adjTELinks(g2mplsTypes::adjacencyId adjacency)
{
	STACK_LOCK();

	std::string        adjString;
	lrm_t *            lrm;
	struct zlist *     anylist;
	adj_t *            adj;
	te_link_t *        tel;
	struct zlistnode * node;
	void *             data_adj;
	void *             data_tel;
	int                i;

	adjString << adjacency;

	zlog_debug("Someone is requesting TELinks in adjacency %s",
		   adjString.c_str());

	// Check LRM presence
	lrm = lrm_lookup();
	if (lrm == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	anylist = lrm->adj_list;
	if (anylist == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	i = 0;
	for (ALL_LIST_ELEMENTS_RO(anylist, node, data_adj)) {
		adj = (adj_t *) data_adj;
		assert(adj != 0);
		if (adj->adj_id == adjacency) {
			if (adj->tel_list->count == 0) {
				zlog_debug("No TELinks for this adjacency");
				STACK_UNLOCK();
				throw(LRM::Info::InternalProblems());
			}
			g2mplsTypes::TELinkIdSeq_var seq;
			{
				g2mplsTypes::TELinkIdSeq * tmp;

				tmp = new g2mplsTypes::TELinkIdSeq(adj->
								   tel_list->
								   count);
				if (!tmp) {
					STACK_UNLOCK();
					throw(LRM::Info::InternalProblems());
				}
				seq = tmp;
			}
			seq->length(adj->tel_list->count);
			for (ALL_LIST_ELEMENTS_RO(adj->tel_list, node, data_tel)) {
				tel = (te_link_t *) data_tel;
				assert(tel != 0);

				g2mplsTypes::TELinkId_var lclId;

				lclId << tel->lcl_id;
				seq[i] = lclId;

				i++;
			}
			zlog_debug("Sent all TELinks");

			STACK_UNLOCK();
			return seq._retn();
		}
	}
	zlog_debug("Adjacency not found");

	STACK_UNLOCK();
	throw (LRM::Info::UnknownAdjId());
}

g2mplsTypes::nodeId
LRM_i::NodeIdFromTELink(const g2mplsTypes::TELinkId & telink)
{
	STACK_LOCK();

	g2mplsTypes::TELinkId_var	corbaTel;
	std::string			telinkString;
	g2mplsTypes::nodeId		id;
	std::string			nodeIdString;
	struct zlistnode *		node;
	void *				data;
	lrm_t *				lrm;
	struct zlist *			anylist;
	te_link_t *			tel;
	bool				found;

	found         = false;
	id            = 0;
	corbaTel      = telink;
	telinkString << corbaTel;

	zlog_debug("Someone is requesting nodeId from TELink %s",
		   telinkString.c_str());

	// Check LRM presence
	lrm = lrm_lookup();
	if (lrm == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	anylist = lrm->telink_list;
	if (anylist == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	for (ALL_LIST_ELEMENTS_RO(anylist, node, data)) {
		tel = (te_link_t *) data;
		assert(tel != 0);

		if (tel->lcl_id == corbaTel) {
			// it is a my local TE-link
			id = lrm->router_id;
			found = true;
		} else if (tel->rem_id == corbaTel) {
			// it is a my remote TE-link
			id = tel->rem_node_id;
			found = true;
		}

		if (found) {
			nodeIdString << id;

			zlog_debug("Sent nodeId %s for telink %s",
				   nodeIdString.c_str(),
				   telinkString.c_str());
			STACK_UNLOCK();
			return id;
		}
	}

	zlog_debug("TELink %s not found", telinkString.c_str());
	STACK_UNLOCK();
	throw (LRM::Info::UnknownTELink());
}

g2mplsTypes::TELinkId
LRM_i::TELinkFromTna(const g2mplsTypes::tnaId & tna)
{
	STACK_LOCK();

	g2mpls_addr_t                   tna_id;
	g2mplsTypes::tnaId_var		corbaTna;
	g2mplsTypes::TELinkId_var	corbaTel;
	std::string			tnaString;
	std::string			telinkString;
	struct zlistnode *		node;
	void *				data;
	lrm_t *				lrm;
	struct zlist *			anylist;
	te_link_t *			tel;
	bool				found;

	memset(&tna_id, 0, sizeof(tna_id));
	found      = false;
	corbaTna   = tna;
	tnaString << corbaTna;
	tna_id    << corbaTna;

	zlog_debug("Someone is requesting TelinkId from TNA %s",
		   tnaString.c_str());

	// Check LRM presence
	lrm = lrm_lookup();
	if (lrm == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	if (corbaTna->_d() != g2mplsTypes::TNAIDTYPE_IPV4) {
		zlog_err("Unsupported TNA types different from "
			 "TNAIDTYPE_IPV4 (%s)",
			 tnaString.c_str());

		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	anylist = lrm->telink_list;
	if (anylist == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	for (ALL_LIST_ELEMENTS_RO(anylist, node, data)) {
		tel = (te_link_t *) data;
		assert(tel != 0);

		if (tel->adj && tel->adj->link_type != UNI &&
		    tel->adj->link_type != ENNI) {
			continue;
		}

		if (addr_equal(tel->tna_id, tna_id)) {

			corbaTel     << tel->lcl_id;
			telinkString << corbaTel;

			zlog_debug("Sent teLink %s for tna %s",
				   telinkString.c_str(),
				   tnaString.c_str());
			STACK_UNLOCK();
			return corbaTel._retn();
		}
	}

	zlog_debug("Tna %s not found", tnaString.c_str());
	STACK_UNLOCK();
	throw (LRM::Info::UnknownTna());
}

g2mplsTypes::TELinkId
LRM_i::TELinkFromRemNodeId(const g2mplsTypes::nodeId node)
{
	STACK_LOCK();

	uint32_t                        nodeId;
	g2mplsTypes::TELinkId_var	corbaTel;
	std::string			telinkString;
	struct zlistnode *		znode;
	void *				data;
	lrm_t *				lrm;
	struct zlist *			anylist;
	te_link_t *			tel;
	adj_t *                         adj;
	bool				found;

	found    = false;
	nodeId   = node;

	zlog_debug("Someone is requesting TelinkId from Node %s",
		   numb_htoa(nodeId));

	// Check LRM presence
	lrm = lrm_lookup();
	if (lrm == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	anylist = lrm->adj_list;
	if (anylist == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	for (ALL_LIST_ELEMENTS_RO(anylist, znode, data)) {
		adj = (adj_t *) data;
		assert(adj != 0);

		if (adj->adj_addr == nodeId) {
			if (adj->tel_list->count) {
				tel = (te_link_t *) listnode_head(adj->tel_list);
				corbaTel     << tel->lcl_id;
				telinkString << corbaTel;

				zlog_debug("Sent teLink %s for Node %s",
					   telinkString.c_str(),
					   numb_htoa(nodeId));
				STACK_UNLOCK();
				return corbaTel._retn();
			} else {
				zlog_err("There are no TeLinks to Node %s",
					  numb_htoa(nodeId));
				STACK_UNLOCK();
				throw (LRM::Info::NoTELinks());
			}
		}
	}

	zlog_debug("Node %s not found", numb_htoa(nodeId));
	STACK_UNLOCK();
	throw (LRM::Info::UnknownNodeId());
}


g2mplsTypes::nodeId
LRM_i::NbrNodeIdFromTna(const g2mplsTypes::tnaId & tna)
{
	STACK_LOCK();

	g2mplsTypes::tnaId_var		corbaTna;
	std::string			tnaString;
	g2mpls_addr_t                   tna_id;
	g2mplsTypes::nodeId		id;
	std::string			nodeIdString;
	struct zlistnode *		node;
	void *				data;
	lrm_t *				lrm;
	struct zlist *			anylist;
	te_link_t *			tel;

	memset(&tna_id, 0, sizeof(tna_id));
	id         = 0;
	corbaTna   = tna;
	tnaString << corbaTna;
	tna_id    << corbaTna;

	zlog_debug("Someone is requesting nbrNodeId from Tna %s",
		   tnaString.c_str());

	// Check LRM presence
	lrm = lrm_lookup();
	if (lrm == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	if (corbaTna->_d() != g2mplsTypes::TNAIDTYPE_IPV4) {
		zlog_err("Unsupported TNA types different from "
			 "TNAIDTYPE_IPV4 (%s)",
			 tnaString.c_str());

		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	anylist = lrm->telink_list;
	if (anylist == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	for (ALL_LIST_ELEMENTS_RO(anylist, node, data)) {
		tel = (te_link_t *) data;
		assert(tel != 0);

		if (tel->adj && tel->adj->link_type != UNI) {
			continue;
		}

		if (addr_equal(tel->tna_id, tna_id)) {
			id = tel->rem_node_id;

			nodeIdString << id;

			zlog_debug("Sent nodeId %s for TNA %s",
				   nodeIdString.c_str(),
				   tnaString.c_str());
			STACK_UNLOCK();
			return id;
		}
	}

	zlog_debug("Tna %s not found", tnaString.c_str());
	STACK_UNLOCK();
	throw (LRM::Info::UnknownTna());
}

g2mplsTypes::nodeId
LRM_i::SingleUniNbrNodeId(void)
{
	STACK_LOCK();

	g2mplsTypes::nodeId		id;
	std::string			nodeIdString;
	struct zlistnode *		node;
	void *				data;
	lrm_t *				lrm;
	struct zlist *			anylist;
	adj_t *				adj;

	zlog_debug("Someone is requesting SingleUniNbrNodeId");

	// Check LRM presence
	lrm = lrm_lookup();
	if (lrm == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	anylist = lrm->adj_list;
	if (anylist == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	id = 0;

	for (ALL_LIST_ELEMENTS_RO(anylist, node, data)) {
		adj = (adj_t *) data;
		assert(adj != 0);

		if (adj->link_type == UNI) {
			if (id && id != adj->adj_addr) {
				zlog_debug("More than one UNI adjacency... "
					   "no single solution");
				STACK_UNLOCK();
				throw (LRM::Info::NoSingleUniNbr());
			}

			id = adj->adj_addr;

			nodeIdString << id;

			zlog_debug("Found nodeId %s under UNI adjacency",
				   nodeIdString.c_str());
		}
	}

	zlog_debug("Single UNI nbr node Id found (%s)", nodeIdString.c_str());
	STACK_UNLOCK();
	return id;
}

g2mplsTypes::nodeId
LRM_i::nodeId()
{
	STACK_LOCK();

	lrm_t               * lrm;
	g2mplsTypes::nodeId   id;
	std::string           nodeIdString;

	zlog_debug("Someone is requesting nodeId");

	// Check LRM presence
	lrm = lrm_lookup();
	if (lrm == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	id = lrm->router_id;

	nodeIdString << id;

	zlog_debug("Sent nodeId %s", nodeIdString.c_str());
	STACK_UNLOCK();
	return id;
}


void
LRM_i::scngw_isup(void)
{
	STACK_LOCK();

	zlog_debug("Received a SCNGW_IS_UP message from SCNGW");

	// corba_client_shutdown before setup ???
	if (!corba_scngw_client_setup()) {
		zlog_err("Cannot setup SCNGW client object");
		STACK_UNLOCK();
		return;
	}

	//Check LRM-SCNGW connection status
	if (lrm_scn.status == UP) {
		reset_lrm_scn();
		lrm_scn.err_counter = 0;
	}

	//Schedule a thread to send LRM_BEGIN_SYNC message to SCNGW
	thread_add_event(master, send_msg_to_scngw, NULL, LRM_BEGIN_SYNC);

	STACK_UNLOCK();
}

void
LRM_i::updateDLinkDetails(const g2mplsTypes::DLinkId &         datalink,
			  const g2mplsTypes::DLinkParameters & parms)
{
	STACK_LOCK();

	g2mplsTypes::DLinkId_var               dlink;
	g2mplsTypes::DLinkWdmLambdasBitmap_var bitmap;
	std::string                            dlinkString;
	std::string                            telinkString;
	lrm_t *                                lrm;
	struct zlist *                         anylist;
	struct zlist *                         telist;
	datalink_t *                           dl;
	datalink_t *                           dlTmp;
	te_link_t *	                       tel;
	struct zlistnode *                     node;
	struct zlistnode *                     node_dl;
	struct zlistnode *                     node_tel;
	void *                                 data;
	void *                                 data_dl;
	void *                                 data_tel;

	dlink       = datalink;
	dlinkString << dlink;

	zlog_debug("TNRC is updating details of DataLink %s",
		   dlinkString.c_str());

	// Check LRM presence
	lrm = lrm_lookup();
	if (lrm == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	anylist = lrm->datalink_list;
	if (anylist == NULL) {
		STACK_UNLOCK();
		throw(LRM::Info::InternalProblems());
	}

	for (ALL_LIST_ELEMENTS_RO(anylist, node, data)) {
		dl = (datalink_t *) data;
		assert(dl != 0);
		if (dl->lcl_id == dlink) {
			dl->op_state   << parms.states.opState;
			dl->swcap      << parms.swCap;
			dl->enctype    << parms.encType;
			dl->max_bw      = parms.maxBandwidth;
			dl->max_res_bw  = parms.maxResBandwidth;
			dl->min_LSP_bw  = parms.minLSPbandwidth;
			for (int i = 0; i < 8; i++) {
				dl->avail_bw_per_prio[i] =
					parms.
					availBandwidthPerPrio[i];
				dl->max_LSP_bw[i]        =
					parms.maxLSPbandwidth[i];
			}
			fill_adv_rsrv_calendar(parms.advRsrvCalendar, dl);

			if (dl->swcap == SWCAP_LSC) {
				fill_lambdas_bitmap(parms.lambdasBitmap,
						    dl->lambdas_bitmap);
			}
			zlog_debug("DataLink details updated");
			//call te bundle
			telist = lrm->telink_list;
			if (anylist == NULL) {
				zlog_debug("Cannot rebundle "
					   "correspondent TELink");
				STACK_UNLOCK();
				throw(LRM::Info::InternalProblems());
			}
			for (ALL_LIST_ELEMENTS_RO(telist, node_tel, data_tel)) {
				tel = (te_link_t *) data_tel;
				assert(tel != 0);
				for (ALL_LIST_ELEMENTS_RO(tel->dl_list,
							  node_dl,
							  data_dl)) {
					dlTmp = (datalink_t *) data_dl;
					assert(dlTmp != 0);
					if (addr_equal(dlTmp->lcl_id,
						       dl->lcl_id) == 1) {
						te_bundle(tel);
						zlog_debug("Correspondent TELink "
							   "rebundled");
						//advertise OSPF
						zlog_debug("Send update to OSPF");
						struct interface * ifp;
						ifp = TEL2interface(tel);
						lrm_interface_update_notify(ifp, BW_UPDATE);
						STACK_UNLOCK();
						return;
					}
				}
			}
			zlog_debug("Cannot rebundle correspondent TELink");
			STACK_UNLOCK();
			throw(LRM::Info::InternalProblems());
		}
	}
	STACK_UNLOCK();
	throw(LRM::Info::UnknownDLink());
}

SCNGW::Sync_var        scngw_proxy;
TNRC::NorthBound_var   tnrc_proxy;
LRM_i*     servant      = 0;
#endif // HAVE_OMNIORB

extern "C" {
	void schedule_next_msg_to_send();
	SCNGW::Sync::CC_Seq * create_cc_seq(te_link_t * tel);
	void sync_err_handle();

	int corba_scngw_client_setup()
	{
#if HAVE_OMNIORB
		CORBA::ORB_var orb;
		orb = corba_orb();
		if (CORBA::is_nil(orb)) {
			zlog_err("Cannot get ORB");
			return 0;
		}

		string ior;
		if (!corba_fetch_ior(CORBA_SERVANT_SCNGW, ior)) {
			zlog_err("Cannot fetch IOR");
			return 0;
		}

		try {
			CORBA::Object_var obj;
			obj = orb->string_to_object(ior.c_str());
			if (CORBA::is_nil(obj)) {
				zlog_err("Cannot get object");
				return 0;
			}

			scngw_proxy = SCNGW::Sync::_narrow(obj);
			if (CORBA::is_nil(scngw_proxy)) {
				zlog_err("cannot invoke on a nil object "
					 "reference");
				return 0;
			}
		} catch (CORBA::SystemException & e) {
			zlog_err("Caught CORBA::SystemException");
			return false;
		} catch (CORBA::Exception & e) {
			zlog_err("Caught CORBA::Exception");
			return false;
		} catch (...) {
			zlog_err("CLIENT_SETUP: caught unknown exception");
			return false;
		}
#endif // HAVE_OMNIORB
		return 1;
	}

	int corba_tnrc_client_setup()
	{
#if HAVE_OMNIORB
		CORBA::ORB_var orb;
		orb = corba_orb();
		if (CORBA::is_nil(orb)) {
			fprintf(stderr, "Cannot get ORB\n");
			return 0;
		}

		string ior;
		if (!corba_fetch_ior(CORBA_SERVANT_TNRC, ior)) {
			fprintf(stderr, "Cannot fetch IOR\n");
			return 0;
		}

		try {
			CORBA::Object_var obj;
			obj = orb->string_to_object(ior.c_str());
			if (CORBA::is_nil(obj)) {
				fprintf(stderr, "Cannot get object\n");
				return 0;
			}

			tnrc_proxy = TNRC::NorthBound::_narrow(obj);
			if (CORBA::is_nil(tnrc_proxy)) {
				fprintf(stderr, "cannot invoke on a nil object "
					 "reference\n");
				return 0;
			}
		} catch (CORBA::SystemException & e) {
			fprintf(stderr, "Caught CORBA::SystemException\n");
			return false;
		} catch (CORBA::Exception & e) {
		        fprintf(stderr, "Caught CORBA::Exception\n");
			return false;
		} catch (...) {
			fprintf(stderr, "CLIENT_SETUP: caught unknown "
				"exception\n");
			return false;
		}
#endif // HAVE_OMNIORB
		return 1;
	}

	int corba_scngw_client_shutdown(void)
	{
#if HAVE_OMNIORB
#endif // HAVE_OMNIORB
		return 1;
	}

	int corba_tnrc_client_shutdown(void)
	{
#if HAVE_OMNIORB
#endif // HAVE_OMNIORB
		return 1;
	}

	int corba_server_setup(void)
	{
#if HAVE_OMNIORB
		fprintf(stderr, "Setting up CORBA server side\n");

		try {
			servant = new LRM_i();
			if (!servant) {
				throw string("Cannot create servant");
			}

			PortableServer::POA_var poa;
			poa = corba_poa();
			if (CORBA::is_nil(poa)) {
				throw string("Cannot get POA");
			}

			PortableServer::ObjectId_var servant_id;
			servant_id = poa->activate_object(servant);

			CORBA::Object_var obj;
			obj = servant->_this();
			if (CORBA::is_nil(obj)) {
				throw string("Cannot get object");
			}

			CORBA::ORB_var orb;
			orb = corba_orb();
			if (CORBA::is_nil(orb)) {
				throw string("Cannot get ORB");
			}

			CORBA::String_var sior(orb->object_to_string(obj));
			zlog_debug("IOR = '%s'", (char*) sior);

			if (!corba_dump_ior(CORBA_SERVANT_LRM, string(sior))) {
				throw string("Cannot dump IOR");
			}

			servant->_remove_ref();

			PortableServer::POAManager_var poa_manager;
			poa_manager = corba_poa_manager();
			if (CORBA::is_nil(poa_manager)) {
				throw string("Cannot get POA Manager");
			}

			poa_manager->activate();
	} catch (CORBA::SystemException & e) {
		fprintf(stderr, "Caught CORBA::SystemException\n");
		return 0;
	} catch (CORBA::Exception & e) {
		fprintf(stderr, "Caught CORBA::Exception\n");
		return 0;
	} catch (omniORB::fatalException & e) {
		fprintf(stderr, "Caught omniORB::fatalException:\n");
		fprintf(stderr, "  file: %s\n", e.file());
		fprintf(stderr, "  line: %d\n", e.line());
		fprintf(stderr, "  mesg: %s\n", e.errmsg());
		return 0;
	} catch (string & e) {
		fprintf(stderr, "Caught exception: %s\n", e.c_str());
		return 0;
	} catch (...) {
		fprintf(stderr, "Caught unknown exception\n");
		return 0;
	}
#endif // HAVE_OMNIORB

		return 1;
	}

	int corba_lrm_isup(void)
	{
#if HAVE_OMNIORB
		try {
			tnrc_proxy->lrmIsUp();
		} catch (TNRC::InternalProblems) {
			fprintf(stderr,"ERR: TNRC internal problems "
				"on lrmIsUp\n");
			return 0;
		} catch (...) {
			fprintf(stderr,"ERR: Unexpected error on lrmIsUp\n");
			return 0;
		}
#endif // HAVE_OMNIORB
		return 1;
	}

#if HAVE_OMNIORB
	void
	fill_adv_rsrv_calendar(const g2mplsTypes::dataLinkCalendarSeq & calendar,
			       datalink_t *                             dl)
	{
		size_t             size;
		calendar_event_t * event;

		list_delete_all_node(dl->calendar);

		for (size_t i = 0; i < calendar.length(); i++) {
			event = (calendar_event_t *) malloc(sizeof(*event));
			if (event == NULL) {
				zlog_err("Cannot allocate a calendar event data structure");
				return;
			}
			event->time_stamp  = calendar[i].unixTime;
			for (size_t j = 0; j < MAX_BW_PRIORITIES; j++) {
				event->avail_bw[j] = calendar[i].availBw[j];
			}
			listnode_add(dl->calendar, event);
		}
	}

	void
	fill_lambdas_bitmap(const g2mplsTypes::DLinkWdmLambdasBitmap & src,
			    wdm_link_lambdas_bitmap_t &                dst)
	{
		size_t    size;
		uint8_t * mask;

		if (dst.bitmap_word) {
			free(dst.bitmap_word);
		}

		memset(&dst, 0, sizeof(dst));

		dst.base_lambda_label = src.baseLambda;
		dst.num_wavelengths   = src.numLambdas;

		dst.bitmap_size = (src.numLambdas % 32 ?
				   (src.numLambdas / 32) + 1 :
				   src.numLambdas / 32);

		size = (size_t) (dst.bitmap_size * 4);
		dst.bitmap_word = (uint32_t *) malloc(size);
		memset(dst.bitmap_word, 0, size);

		mask = (uint8_t *) dst.bitmap_word;

		for (size_t i = 0; i < src.bitmap.length(); i++) {
			*mask = src.bitmap[i];
			mask++;
		}
	}

	int tnrc_get_dlink_details(datalink_t * dl)
	{
		g2mplsTypes::DLinkId_var               localDLink;
		std::string                            localDLinkString;
		g2mplsTypes::DLinkParameters_var       params;
		g2mplsTypes::DLinkWdmLambdasBitmap_var bitmapTmp;

		localDLink << dl->lcl_id;

		localDLinkString << localDLink;

		zlog_debug("Retrieve TE info from TNRC for data link %s",
			   localDLinkString.c_str());

		try {
			bool res = tnrc_proxy->getDLinkDetails(localDLink,
							       params);
			if (res) {
				dl->op_state   << params->states.opState;
				dl->swcap      << params->swCap;
				dl->enctype    << params->encType;
				dl->max_bw      = params->maxBandwidth;
				dl->max_res_bw  = params->maxResBandwidth;
				dl->min_LSP_bw  = params->minLSPbandwidth;

				for (int i = 0; i < 8; i++) {
					dl->avail_bw_per_prio[i] =
						params->
						availBandwidthPerPrio[i];
					dl->max_LSP_bw[i]        =
						params->maxLSPbandwidth[i];
				}
				fill_adv_rsrv_calendar(params->advRsrvCalendar, dl);
				if (dl->swcap == SWCAP_LSC) {
					//bitmapTmp = params->lambdasBitmap;
					//dl->lambdas_bitmap << bitmapTmp;
					fill_lambdas_bitmap(params->lambdasBitmap,
							    dl->lambdas_bitmap);
				}
			} else {
				return 0;
			}
		} catch (TNRC::CannotFetch) {

			zlog_debug("Data link not found");
			return 0;
		} catch (...) {

			zlog_err("Problems in the communication with TNRC");
			return 0;
		}

		zlog_debug("Got TE info from TNRC");
		return 1;
	}

	int scngw_begin_sync(void)
	{
		zlog_debug("**Start of synchronization with SCNGW**");

		try {
			scngw_proxy->begin_sync(lrm_scn.scnif_number,
						lrm_scn.cc_number,
						lrm_scn.tel_number);

		} catch (SCNGW::Sync::SyncErr) {

			zlog_err("Synchronization error");

			sync_err_handle();

			return 0;
		} catch (...) {

			zlog_err("Problems in the communication with SCNGW");

			return 0;
		}
		//Schedule next thread for next mesaage
		schedule_next_msg_to_send();

		zlog_debug("**End of synchronization func with SCNGW**");
		return 1;
	}

	int scngw_sync_fatal_error(void)
	{
		zlog_debug("Sending a SYNC_FATAL_ERROR message to SCNGW");

		try {
			scngw_proxy->sync_fatal_error();

		} catch (...) {

			zlog_err("Problems in the communication with SCNGW");

			return 0;
		}

		return 1;
	}

	int scngw_end_sync(void)
	{
		zlog_debug("Sending an END_SYNC message to SCNGW");

		try {
			scngw_proxy->end_sync();
		} catch (SCNGW::Sync::SyncErr) {

			zlog_err("Synchronization error");

			sync_err_handle();

			return 0;
		} catch (...) {

			zlog_err("Problems in the communication with SCNGW");

			return 0;
		}
		zlog_debug("**End of synchronization with SCNGW**");
		lrm_scn.status = UP;

		return 1;
	}

	int scngw_scnif_add(ctrl_intf_t * scnif)
	{
		zlog_debug("Sending a SCNIF_ADD message to SCNGW");

		g2mplsTypes::addr_var address;
		{
			g2mplsTypes::addr * tmp;

			tmp = new g2mplsTypes::addr;
			if (!tmp) {
				throw(SCNGW::Sync::InternalProblems());
			}
			address = tmp;
		}

		address->ipv4(scnif->addr.value.ipv4.s_addr);

		try {
			scngw_proxy->scnif_add(address);
		} catch (SCNGW::Sync::SyncErr) {

			zlog_err("Synchronization error");

			sync_err_handle();

			return 0;
		} catch (...) {

			zlog_err("Problems in the communication with SCNGW");

			return 0;
		}
		if (lrm_scn.status == SYNC) {
			scnif->sync_status = SENT;
			lrm_scn.scnif_counter--;
			lrm_scn.tot_counter--;
			//Schedule next thread for next message
			schedule_next_msg_to_send();
		}
		return 1;
	}

	int scngw_scnif_delete(ctrl_intf_t * scnif)
	{
		zlog_debug("Sending a SCNIF_DELETE message to SCNGW");

		g2mplsTypes::addr_var address;
		{
			g2mplsTypes::addr * tmp;

			tmp = new g2mplsTypes::addr;
			if (!tmp) {
				throw(SCNGW::Sync::InternalProblems());
			}
			address = tmp;
		}

		address->ipv4(scnif->addr.value.ipv4.s_addr);

		try {
			scngw_proxy->scnif_delete(address);
		} catch (...) {

			zlog_err("Problems in the communication with SCNGW");

			return 0;
		}

		return 1;
	}

	int scngw_cc_scnif_assoc_add(cc_t * cc)
	{
		int       n_assoc;

		n_assoc = 1;

		SCNGW::Sync::CC_Add_AssocSeq_var assocs;
		{
			SCNGW::Sync::CC_Add_AssocSeq * tmp;

			tmp = new SCNGW::Sync::CC_Add_AssocSeq(n_assoc);
			if (!tmp) {
				throw(SCNGW::Sync::InternalProblems());
			}
			assocs = tmp;
		}
		assocs->length(n_assoc);

		for(size_t i = 0; i < n_assoc; i++) {
			assocs[i].cc_id       = cc->cc_id;
			assocs[i].local_addr.ipv4(cc->lcl_scn_addr.value.ipv4.s_addr);
			assocs[i].remote_addr.ipv4(cc->rem_scn_addr.value.ipv4.s_addr);
		}

		zlog_debug("Sending a CC_SCNIF_ASSOC_ADD message to SCNGW");

		try {
			scngw_proxy->cc_scnif_assoc_add(assocs);
		} catch (SCNGW::Sync::SyncErr) {

			zlog_err("Synchronization error");

			sync_err_handle();

			return 0;
		} catch (...) {

			zlog_err("Problems in the communication with SCNGW");

			return 0;
		}
		if (lrm_scn.status == SYNC) {
			cc->sync_status = SENT;
			lrm_scn.cc_counter--;
			lrm_scn.tot_counter--;
			//Schedule next thread for next message
			schedule_next_msg_to_send();
		}
		return 1;
	}

	int scngw_cc_scnif_assoc_delete(cc_t * cc)
	{
		int       n_assoc;

		n_assoc = 1;

		SCNGW::Sync::CC_Delete_AssocSeq_var assocs;
		{
			SCNGW::Sync::CC_Delete_AssocSeq * tmp;

			tmp = new SCNGW::Sync::CC_Delete_AssocSeq(n_assoc);
			if (!tmp) {
				throw(SCNGW::Sync::InternalProblems());
			}
			assocs = tmp;
		}
		assocs->length(n_assoc);

		for(size_t i = 0; i < n_assoc; i++) {
			assocs[i].cc_id = cc->cc_id;
		}

		zlog_debug("Sending a CC_SCNIF_ASSOC_DELETE message to SCNGW");

		try {
			scngw_proxy->cc_scnif_assoc_delete(assocs);
		} catch (...) {

			zlog_err("Problems in the communication with SCNGW");

			return 0;
		}

		return 1;
	}

	int scngw_cc_scnif_assoc_update(void *arg)
	{
		return 1;
	}

	int scngw_tel_cc_assoc_add(te_link_t * tel)
	{
		int             n_assoc;
		SCNGW::Sync::CC_Seq * ccs;

		n_assoc = 1;

		SCNGW::Sync::TELCC_Add_AssocSeq_var assocs;
		{
			SCNGW::Sync::TELCC_Add_AssocSeq * tmp;

			tmp = new SCNGW::Sync::TELCC_Add_AssocSeq(n_assoc);
			if (!tmp) {
				throw(SCNGW::Sync::InternalProblems());
			}
			assocs = tmp;
		}
		assocs->length(n_assoc);

		for(size_t i = 0; i < n_assoc; i++) {
			assocs[i].key_id      = tel->tel_key;
			assocs[i].iftype      = ((tel->adj_type == INNI) ? SCNGW::Sync::IF_INNI:
						 (tel->adj_type == ENNI) ? SCNGW::Sync::IF_ENNI:
									   SCNGW::Sync::IF_UNI);
			assocs[i].local_addr.ipv4(tel->lcl_id.value.ipv4.s_addr);
			assocs[i].remote_addr.ipv4(tel->rem_id.value.ipv4.s_addr);

			ccs = create_cc_seq(tel);

			assocs[i].ccs         = *ccs;
		}

		zlog_debug("Sending a TEL_CC_ASSOC_ADD message to SCNGW");

		try {
			scngw_proxy->tel_cc_assoc_add(assocs);
		} catch (SCNGW::Sync::SyncErr) {

			zlog_err("Synchronization error");

			sync_err_handle();

			return 0;
		} catch (...) {

			zlog_err("Problems in the communication with SCNGW");

			return 0;
		}
		if (lrm_scn.status == SYNC) {
			tel->sync_status = SENT;
			lrm_scn.tel_counter--;
			lrm_scn.tot_counter--;
			//Schedule next thread for next mesaage
			schedule_next_msg_to_send();
		}
		return 1;
	}

	int scngw_tel_cc_assoc_delete(te_link_t * tel)
	{
		int       n_assoc;

		n_assoc = 1;

		SCNGW::Sync::TELCC_Delete_AssocSeq_var assocs;
		{
			SCNGW::Sync::TELCC_Delete_AssocSeq * tmp;

			tmp = new SCNGW::Sync::TELCC_Delete_AssocSeq(n_assoc);
			if (!tmp) {
				throw(SCNGW::Sync::InternalProblems());
			}
			assocs = tmp;
		}
		assocs->length(n_assoc);

		for(size_t i = 0; i < n_assoc; i++) {
			assocs[i].key_id = tel->tel_key;
		}

		zlog_debug("Sending a TEL_CC_ASSOC_DELETE message to SCNGW");

		try {
			scngw_proxy->tel_cc_assoc_delete(assocs);
		} catch (...) {

			zlog_err("Problems in the communication with SCNGW");

			return 0;
		}

		return 1;
	}

	int scngw_tel_cc_assoc_update(te_link_t * tel, int op, u_int32_t cc_id)
	{
		int n_upd;

		n_upd = 1;

		SCNGW::Sync::UpdateSeq_var updates;
		{
			SCNGW::Sync::UpdateSeq * tmp;

			tmp = new SCNGW::Sync::UpdateSeq(n_upd);
			if (!tmp) {
				throw(SCNGW::Sync::InternalProblems());
			}
			updates = tmp;
		}
		updates->length(n_upd);

		for(size_t i = 0; i < n_upd; i++) {
			updates[i].type  = ((op == ADD_OP) ? SCNGW::Sync::TELCC_ADD :
							     SCNGW::Sync::TELCC_REMOVE);
			updates[i].cc_id = cc_id;
		}

		zlog_debug("Sending a TEL_CC_ASSOC_UPDATE message to SCNGW");

		try {
			scngw_proxy->tel_cc_assoc_update(tel->tel_key, updates);
		} catch (...) {

			zlog_err("Problems in the communication with SCNGW");

			return 0;
		}

		return 1;
	}

	void sync_err_handle()
	{
		//Reset lrm-scn
		reset_lrm_scn();
		if(lrm_scn.err_counter < 5){
			lrm_scn.err_counter++;
			//Schedule a thread to send again a
			//LRM_BEGIN_SYNC msg
			thread_add_event(master, send_msg_to_scngw,
						NULL, LRM_BEGIN_SYNC);
		} else {
			zlog_debug("too many synchronization errors");
			//Schedule a thread to send a
			//LRM_SYNC_FATAL_ERROR msg
			thread_add_event(master, send_msg_to_scngw,
						NULL, LRM_SYNC_FATAL_ERROR);

		}
	}

	SCNGW::Sync::CC_Seq * create_cc_seq(te_link_t * tel)
	{
		cc_t             * cc;
		struct zlistnode * ccnode;
		struct zlistnode * ccnnode;
		void             * p;
		int		   i;

		SCNGW::Sync::CC_Seq_var ccs;
		{
			SCNGW::Sync::CC_Seq * tmp;

			tmp = new SCNGW::Sync::CC_Seq(tel->num_cc_up);
			if (!tmp) {
				throw(SCNGW::Sync::InternalProblems());
			}
			ccs = tmp;
		}
		ccs->length(tel->num_cc_up);

		i = 0;

		for(ALL_LIST_ELEMENTS(tel->cc_list, ccnode, ccnnode, p)){
			cc = (cc_t *) p;
			if((cc->op_state  == UP     ) &&
			   (cc->adm_state == ENABLED)) {
				ccs[i].id = cc->cc_id;
			}
			i++;
		}
		return ccs._retn();
	}


	void schedule_next_msg_to_send()
	{
		lrm_t            * lrm;
		struct zlist     * tel_list;
		struct zlist     * cc_list;
		struct zlist     * scnif_list;
		struct zlistnode * ccnode;
		struct zlistnode * ccnnode;
		struct zlistnode * telnode;
		struct zlistnode * telnnode;
		struct zlistnode * scnifnode;
		struct zlistnode * scnifnnode;
		te_link_t        * tel;
		cc_t             * cc;
		ctrl_intf_t      * scnif;
		void             * p;

		lrm        = lrm_lookup();
		assert(lrm);
		tel_list   = lrm->telink_list;
		cc_list    = lrm->cc_list;
		scnif_list = lrm->scn_if_list;

		if(lrm_scn.status == SYNC){
			if(lrm_scn.tot_counter > 0){
				if(lrm_scn.scnif_counter > 0){
					for(ALL_LIST_ELEMENTS(scnif_list,
							      scnifnode,
							      scnifnnode,
							      p)){
						scnif = (ctrl_intf_t *) p;
						if((scnif->adm_state   == ENABLED) &&
						   (scnif->sync_status == NOT_SENT)){
							//Schedule next message
							thread_add_event(master,
									 send_msg_to_scngw,
									 scnif,
									 LRM_SCNIF_ADD);
							break;
						}
					}
				}
				else if(lrm_scn.cc_counter > 0){
					for(ALL_LIST_ELEMENTS(cc_list, ccnode,
							      ccnnode, p)){
						cc = (cc_t *) p;
						if((cc->op_state    == UP) &&
						   (cc->sync_status == NOT_SENT)){
							//Schedule next message
							thread_add_event(master,
									 send_msg_to_scngw,
									 cc,
									 LRM_CC_SCNIF_ASSOC_ADD);
							break;
						}
					}
				}
				else if(lrm_scn.tel_counter > 0){
					for(ALL_LIST_ELEMENTS(tel_list, telnode,
							      telnnode, p)){
						tel = (te_link_t *) p;
						if((tel->op_state == UP) &&
						   (tel->sync_status == NOT_SENT)){
							//Schedule next message
							thread_add_event(master,
									 send_msg_to_scngw,
									 tel,
									 LRM_TEL_CC_ASSOC_ADD);
							break;
						}
					}
				}
			} else {
				//Schedule next message
				thread_add_event(master, send_msg_to_scngw,
						 NULL, LRM_END_SYNC);
			}
		}
		//No messages to send. No thread scheduled
	}

	int send_msg_to_scngw(struct thread * t)
	{
		int    command;
		void * arg;

		command = THREAD_VAL(t);
		arg     = THREAD_ARG(t);

		switch (command) {
			case LRM_BEGIN_SYNC:
				scngw_begin_sync();
				break;
			case LRM_SCNIF_ADD:
				ctrl_intf_t * scnif;
				scnif = (ctrl_intf_t *) arg;
				scngw_scnif_add(scnif);
				break;
			case LRM_CC_SCNIF_ASSOC_ADD:
				cc_t * cc;
				cc = (cc_t *) arg;
				scngw_cc_scnif_assoc_add(cc);
				break;
			case LRM_TEL_CC_ASSOC_ADD:
				te_link_t * tel;
				tel = (te_link_t *) arg;
				scngw_tel_cc_assoc_add(tel);
				break;
			case LRM_END_SYNC:
				scngw_end_sync();
				break;
			case LRM_SYNC_FATAL_ERROR:
				scngw_sync_fatal_error();
				break;
			default:
				break;
		}

		return 0;
	}

	int corba_server_shutdown(void)
	{
#if HAVE_OMNIORB
		try {
			zlog_debug("Shutting down CORBA server side");

			if (!corba_remove_ior(CORBA_SERVANT_LRM)) {
				throw string("Cannot remove IOR");
			}
		} catch (...) {
			zlog_err("Caught unknown exception");
			return 0;
		}
#endif // HAVE_OMNIORB

		return 1;
	}

#endif // HAVE_OMNIORB
} //extern "C"
