//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 PSNC
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Adam Kaliszan         (PSNC)             <kaliszan_at_man.poznan.pl>
//  Damian Parniewicz     (PSNC)             <damianp_at_man.poznan.pl>
//  Lukasz Lopatowski     (PSNC)             <llopat_at_man.poznan.pl>
//  Jakub Gutkowski       (PSNC)             <jgutkow_at_man.poznan.pl>
//

#include "zebra.h"
#include "log.h"
#include "linklist.h"
#include "prefix.h"
#include "memory.h"

#include "lib/corba.h"
#include "g2mpls_corba_utils.h"
#include "ospfd/ospf_corba.h"
#include "ospfd/ospf_corba_utils.h"
#include "idl/g2mplsTopology.hh"
#include "ospfd/ospf_te.h"
#include "ospfd/ospf_grid.h"

#include <iostream>
#include <string>
#include <stdlib.h>

using namespace std;

#if HAVE_OMNIORB

g2mplsTopology::Info_var  g2pcera_proxy;
g2mplsTopology::Info_var  gunigw_proxy;

#define UPDATE_G2PCERA      1
#define UPDATE_GUNIGW       2

#define REMOVE_TNA_ADDRESS  0
#define ADD_TNA_ADDRESS     1

#define REMOVE_TE_LINK      0
#define ADD_TE_LINK         1

extern "C" {

int corba_g2pcera_client_setup()
{
  if (IS_DEBUG_GRID_NODE(CORBA))
    zlog_debug("[DBG] CORBA: Setting up G2PCERA client side");
  CORBA::ORB_var orb;
  orb = corba_orb();
  if (CORBA::is_nil(orb)) {
      zlog_err("[ERR] CORBA: Cannot get ORB");
    return 0;
  }

  string ior;
  if (!corba_fetch_ior(CORBA_SERVANT_G2MPLS_TOPOLOGY, ior)) {
      zlog_err("[ERR] CORBA: Cannot fetch IOR");
    return 0;
  }

  CORBA::Object_var obj;
  obj = orb->string_to_object(ior.c_str());
  if (CORBA::is_nil(obj)) {
      zlog_err("[ERR] CORBA: Cannot get object");
    return 0;
  }

  g2pcera_proxy = g2mplsTopology::Info::_narrow(obj);
  if (CORBA::is_nil(g2pcera_proxy)) {
      zlog_err("[ERR] CORBA: Cannot invoke on a nil object reference");
    return 0;
  }
    if (IS_DEBUG_GRID_NODE(CORBA))
      zlog_debug("[DBG] CORBA: G2PCERA client side is up");
  return 1;
}

}

extern "C" {

int corba_gunigw_client_setup()
{
  if (IS_DEBUG_GRID_NODE(CORBA))
    zlog_debug("[DBG] CORBA: Setting up GUNIGW client side");
  CORBA::ORB_var orb;
  orb = corba_orb();
  if (CORBA::is_nil(orb)) {
      zlog_err("[ERR] CORBA: Cannot get ORB");
    return 0;
  }

  string ior;
  if (!corba_fetch_ior(CORBA_SERVANT_G2TOPOLOGY_UNI, ior)) {
      zlog_err("[ERR] CORBA: Cannot fetch IOR");
    return 0;
  }

  CORBA::Object_var obj;
  obj = orb->string_to_object(ior.c_str());
  if (CORBA::is_nil(obj)) {
      zlog_err("[ERR] CORBA: Cannot get object");
    return 0;
  }

  gunigw_proxy = g2mplsTopology::Info::_narrow(obj);
  if (CORBA::is_nil(gunigw_proxy)) {
      zlog_err("[ERR] CORBA: Cannot invoke on a nil object reference");
    return 0;
  }
  if (IS_DEBUG_GRID_NODE(CORBA))
    zlog_debug("[DBG] CORBA: GUNIGW client side is up");
  return 1;
}

}

g2mplsTypes::nodeIdent nodeIdent;
g2mplsTypes::gridSubNodeIdent subNodeIdent;
g2mplsTypes::nodeId nodeId;
g2mplsTypes::gridSubNodeId subNodeId;
g2mplsTypes::gridSiteParams siteInfo;
g2mplsTypes::gridServiceParams serviceInfo;
g2mplsTypes::gridCEParams ceInfo;
g2mplsTypes::gridSubClusterParams subClusterInfo;
g2mplsTypes::gridSEParams seInfo;

g2mplsTypes::tnaIdent emptyTnaIdent;
g2mplsTypes::teLinkIdent emptyLinkIdent;

extern "C" {

void node_add(uint8_t server, int id, node_type_t type)
{
  try {
    if (id == 0) return;

    if (id != -1) nodeIdent.id = id;
    else nodeIdent.id = nodeId;
    nodeIdent.type = nodeType_from_node_type_t(type);

    struct in_addr nId;
    nId.s_addr = htonl(nodeIdent.id);

    if (!isNodeInSeq(nodeIdent, g2pcera_proxy->nodeGetAll()))
    {
      g2pcera_proxy->nodeAdd(nodeIdent);

      if(IS_DEBUG_GRID_NODE(CORBA))
      {
        zlog_debug("[DBG] CORBA: Sent NODE_ADD message to G2PCERA");
        const char* str;
        switch(type)
        {
          case NTYPE_UNKNOWN: str = "UNKNOWN"; break;
          case NTYPE_NETWORK: str = "NETWORK"; break;
          case NTYPE_GRID: str = "GRID"; break;
        }
        zlog_debug("[DBG]        Node type: %s, Node id: %s", str, inet_ntoa(nId));
      }
    }
    else
    {
      if (IS_DEBUG_GRID_NODE(CORBA_ALL))
        zlog_debug("[DBG] CORBA: Aborted sending NODE_ADD message to G2PCERA (Node %s already exists)", inet_ntoa(nId));
      return;
    }
  } catch (g2mplsTopology::NodeAlreadyExists) {

    zlog_debug("[ERR] CORBA: Exception NodeAlreadyExists (method nodeAdd)");
    return;
  } catch (g2mplsTopology::InvocationNotAllowed) {

    zlog_debug("[ERR] CORBA: Exception InvocationNotAllowed (method nodeAdd)");
    return;
  } catch (g2mplsTopology::InternalProblems & e) {

    zlog_debug("[ERR] CORBA: Exception InternalProblems (method nodeAdd): %s", (char *) e.what);
  } catch (...) {

    zlog_debug("[ERR] CORBA: Exception Unknown (method nodeAdd)");
    return;
  }
  return;
}

void node_del(uint8_t server, uint32_t id, node_type_t type)
{
  try {
    if (id == 0) return;

    if (id != -1) nodeIdent.id = id;
    else nodeIdent.id = nodeId;
    nodeIdent.type = nodeType_from_node_type_t(type);

    struct in_addr nId;
    nId.s_addr = htonl(nodeIdent.id);

    int length = 0;
    g2mplsTypes::gridSiteParams* sinfo;
    g2mplsTypes::gridSubNodes* snodes;
    g2mplsTypes::teLinkIdentSeq* linkSeq;

    if (isNodeInSeq(nodeIdent, g2pcera_proxy->nodeGetAll()))
    {
      if (type == NTYPE_GRID)
      {
        g2pcera_proxy->gridSiteGet(nodeIdent.id, sinfo, snodes);
        if (snodes->services.length() > 0 || snodes->compElems.length() > 0
          || snodes->subClusters.length() > 0 || snodes->storageElems.length() > 0)
          length = 1;
      }
      else
      {
        linkSeq = g2pcera_proxy->teLinkGetAllFromNode(nodeIdent);
        length = linkSeq->length();
      }

      if (length == 0)
      {
        g2pcera_proxy->nodeDel(nodeIdent);

        if (IS_DEBUG_GRID_NODE(CORBA))
        {
          zlog_debug("[DBG] CORBA: Sent NODE_DEL message to G2PCERA");
          const char* str;
          switch(type)
          {
            case NTYPE_UNKNOWN: str = "UNKNOWN"; break;
            case NTYPE_NETWORK: str = "NETWORK"; break;
            case NTYPE_GRID: str = "GRID"; break;
          }
          zlog_debug("[DBG]        Node type: %s, Node id: %s", str, inet_ntoa(nId));
        }
      }
      else
      {
        if (type == NTYPE_GRID)
        {
        if (IS_DEBUG_GRID_NODE(CORBA_ALL))
          zlog_debug("[DBG] CORBA: Aborted sending NODE_DEL message to G2PCERA (There are still subNodes attached to node %s)", inet_ntoa(nId));
        }
        else
        {
        if (IS_DEBUG_GRID_NODE(CORBA_ALL))
          zlog_debug("[DBG] CORBA: Aborted sending NODE_DEL message to G2PCERA (There are still TE-Links attached to node %s)", inet_ntoa(nId));
        }
        return;
      }
    }
    else
    {
      if (IS_DEBUG_GRID_NODE(CORBA_ALL))
        zlog_debug("[DBG] CORBA: Aborted sending NODE_DEL message to G2PCERA (Node %s doesn't exist)", inet_ntoa(nId));
      return;
    }
  } catch (g2mplsTopology::CannotFetchNode) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchNode (method nodeDel)");
    return;
  } catch (g2mplsTopology::InvocationNotAllowed) {

    zlog_debug("[ERR] CORBA: Exception InvocationNotAllowed (method nodeDel)");
    return;
  } catch (g2mplsTopology::InternalProblems & e) {

    zlog_debug("[ERR] CORBA: Exception InternalProblems (method nodeDel): %s", (char *) e.what);
  } catch (...) {

    zlog_debug("[ERR] CORBA: Exception Unknown (method nodeDel)");
    return;
  }
  return;
}

void update_net_node(int id, uint8_t isDomain)
{
  struct zlistnode * node; void *data;
  struct in_addr * addr;

  try {
    g2mplsTypes::netNodeParams nodeParams;
    g2mplsTypes::netNodeParams* serverNetNodeParams;
    nodeParams.isDomain = isDomain;
    nodeParams.state.opState = g2mplsTypes::OPERSTATE_UP;             // TODO Lukasz
    nodeParams.state.admState = g2mplsTypes::ADMINSTATE_ENABLED;      // TODO Lukasz
    nodeParams.colors = 0;                                            // TODO Lukasz

    g2mplsTypes::areaSeq_var seq;
    {
      g2mplsTypes::areaSeq * tmp;
      tmp = new g2mplsTypes::areaSeq(1);
      if (!tmp) {
        zlog_debug("[ERR] CORBA: update_net_node: tmp == NULL");
        return;
      }
      seq = tmp;
    }
    seq->length(1);

    seq[0] = 0;                // TODO Lukasz: one area (0.0.0.0) in this moment
    nodeParams.areas = seq;

    if (id != -1) g2pcera_proxy->netNodeGet(id, serverNetNodeParams);  
    else          g2pcera_proxy->netNodeGet(nodeId, serverNetNodeParams);
    if (!equalNetNodes(&nodeParams, serverNetNodeParams))
    {
      if (id != -1) g2pcera_proxy->netNodeUpdate(id, nodeParams);
      else          g2pcera_proxy->netNodeUpdate(nodeId, nodeParams);
      if(IS_DEBUG_GRID_NODE(CORBA))
        zlog_debug("[DBG] CORBA: Sent NET_NODE_UPDATE message to G2PCERA");
    }
    else {
      if(IS_DEBUG_GRID_NODE(CORBA_ALL))
        zlog_debug("[DBG] CORBA: Aborted sending NET_NODE_UPDATE message to G2PCERA");
    }
  } catch (g2mplsTopology::CannotFetchNode) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchNode (method netNodeUpdate)");
    return;
  } catch (g2mplsTopology::NodeParamsMismatch) {

    zlog_debug("[ERR] CORBA: Exception NodeParamsMismatch (method netNodeUpdate)");
    return;
  } catch (g2mplsTopology::InvocationNotAllowed) {

    zlog_debug("[ERR] CORBA: Exception InvocationNotAllowed (method netNodeUpdate)");
    return;
  } catch (g2mplsTopology::InternalProblems & e) {

    zlog_debug("[ERR] CORBA: Exception InternalProblems (method netNodeUpdate): %s", (char *) e.what);
    return;
  } catch (...) {

    zlog_debug("[ERR] CORBA: Exception Unknown (method netNodeUpdate)");
    return;
  }
  return;
}

void grid_subnode_del(uint32_t siteId, uint32_t id, grid_s_node_type_t type)
{
  const char* str;
  try {
    nodeId = siteId;
    subNodeIdent.id = id;
    subNodeIdent.type = gridSubNodeType_from_grid_s_node_type_t(type);
    bool subNodeExists = false;

    g2mplsTypes::gridSiteParams* serverSiteInfo;
    g2mplsTypes::gridSubNodes* snodes;

    struct in_addr nId;
    nId.s_addr = siteId;
    struct in_addr snId;
    snId.s_addr = id;

    if (isNodeInSeq(nodeIdent, g2pcera_proxy->nodeGetAll()))
    {
      g2pcera_proxy->gridSiteGet(nodeId, serverSiteInfo, snodes);
      switch (type)
      {
        case GRIDSUBNTYPE_UNKNOWN:
          str = "UNKNOWN";
          break;
        case GRIDSUBNTYPE_SERVICE:
          subNodeExists = isSubNodeInSeq(subNodeIdent, &snodes->services);
          str = "SERVICE";
          break;
        case GRIDSUBNTYPE_COMPUTINGELEMENT:
          subNodeExists = isSubNodeInSeq(subNodeIdent, &snodes->compElems);
          str = "COMPUTING_ELEMENT";
          break;
        case GRIDSUBNTYPE_SUBCLUSTER:
          subNodeExists = isSubNodeInSeq(subNodeIdent, &snodes->subClusters);
          str = "SUBCLUSTER";
          break;
        case GRIDSUBNTYPE_STORAGEELEMENT:
          subNodeExists = isSubNodeInSeq(subNodeIdent, &snodes->storageElems);
          str = "STORAGE_ELEMENT";
          break;
      }

      if (subNodeExists)
      {
        g2pcera_proxy->gridSubNodeDel(nodeId, subNodeIdent);

        if(IS_DEBUG_GRID_NODE(CORBA))
        {
          zlog_debug("[DBG] CORBA: Sent GRID_SUB_NODE_DEL message to G2PCERA");
          zlog_debug("[DBG]        Site id: %d, SubNode id: %d, SubNode type: %s", siteId, id, str);
        }
        return;
      }
      else
      {
        if(IS_DEBUG_GRID_NODE(CORBA_ALL))
        {
          zlog_debug("[DBG] CORBA: Aborted sending GRID_SUB_NODE_DEL message to G2PCERA (SubNode %s doesn't exist)", inet_ntoa(snId));
          zlog_debug("[DBG]        Site id: %d, SubNode id: %d, SubNode type: %s", siteId, id, str);
        }
      }
    }
    else
    {
      if(IS_DEBUG_GRID_NODE(CORBA_ALL))
      {
        zlog_debug("[DBG] CORBA: Aborted sending GRID_SUB_NODE_DEL message to G2PCERA (Node %s doesn't exist)", inet_ntoa(nId));
        zlog_debug("[DBG]        Site id: %d, SubNode id: %d, SubNode type: %s", siteId, id, str);
      }
    }
  } catch (g2mplsTopology::CannotFetchNode) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchNode (method gridSubNodeDel)");
    return;
  } catch (g2mplsTopology::CannotFetchSubNode) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchSubNode (method gridSubNodeDel)");
    return;
  } catch (g2mplsTopology::InvocationNotAllowed) {

    zlog_debug("[ERR] CORBA: Exception InvocationNotAllowed (method gridSubNodeDel)");
    return;
  } catch (g2mplsTopology::InternalProblems & e) {

    zlog_debug("[ERR] CORBA: Exception InternalProblems (method gridSubNodeDel): %s", (char *) e.what);
  } catch (...) {

    zlog_debug("[ERR] CORBA: Exception Unknown (method gridSubNodeDel)");
    return;
  }
  return;
}

void corba_update_gn_GridSite_ID(uint32_t id)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridSite ID");
  nodeId = ntohl(id);
  return;
}

void corba_update_gn_GridSite_Name(const char* name)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridSite NAME");
  siteInfo.name = name;
  return;
}

void corba_update_gn_GridSite_Latitude(uint8_t *latitude)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridSite LATITUDE");
  uint8_t res = latitude[4] >> 2;
  res &= 0x3f;
  siteInfo.location.latResolution = res;
  u_int64_t value = latitude[4]; value <<= 8; value &= 0xff00;
  value |= latitude[3]; value <<= 8; value &= 0xffff00;
  value |= latitude[2]; value <<= 8; value &= 0xffffff00;
  value |= latitude[1]; value <<= 8; value &= 0xffffff00;
  value |= latitude[0];
  u_int64_t temp = 0;
  temp = value & 0xffffffff;
  siteInfo.location.latitute = temp; //wrong field name "latitude"
  siteInfo.location.valid = true;
  return;
}

void corba_update_gn_GridSite_Longitude(uint8_t *longitude)
{
  if(IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridSite LONGITUDE");
  uint8_t res = longitude[4] >> 2;
  res &= 0x3f;
  siteInfo.location.lonResolution = res;
  u_int64_t value = longitude[4]; value <<= 8; value &= 0xff00;
  value |= longitude[3]; value <<= 8; value &= 0xffff00;
  value |= longitude[2]; value <<= 8; value &= 0xffffff00;
  value |= longitude[1]; value <<= 8; value &= 0xffffff00;
  value |= longitude[0];
  u_int64_t temp = 0;
  temp = value & 0xffffffff;
  siteInfo.location.longitude = temp;
  siteInfo.location.valid = true;
  return;
}

void corba_update_gn_GridSite_PERouter_ID(struct in_addr ipv4Id)
{
  if(IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridSite PE_ROUTER_ID");
  siteInfo.peRouterId =  ntohl(ipv4Id.s_addr);
  return;
}

void init_grid_GridSite()
{
  if(IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridSite");

  const char* name = "";
  siteInfo.name = name;
  siteInfo.location.latResolution = 0;
  siteInfo.location.latitute = 0;
  siteInfo.location.lonResolution = 0;
  siteInfo.location.longitude = 0;
  siteInfo.location.valid = false;
  siteInfo.peRouterId =  0;

  return;
}

void corba_update_gn_GridSite(int server)
{
  g2mplsTypes::gridSiteParams* serverSiteInfo;
  g2mplsTypes::gridSubNodes* snodes;
  try {
    switch(server)
    {
      case UPDATE_G2PCERA:
        g2pcera_proxy->gridSiteGet(nodeId, serverSiteInfo, snodes);
        if (!equalGridSites(&siteInfo, serverSiteInfo))
          g2pcera_proxy->gridSiteUpdate(nodeId, siteInfo);
        else
        {
          if (IS_DEBUG_GRID_NODE(CORBA_ALL))
            OSPF_CORBA_DEBUG(OCD_TYPE_GRID, OCD_ACTION_ABORTED, OCD_SERV_G2PCERA, "GRID_SITE_UPDATE",
                             emptyTnaIdent, emptyLinkIdent);
          return;
        }
        if(IS_DEBUG_GRID_NODE(CORBA))
            OSPF_CORBA_DEBUG(OCD_TYPE_GRID, OCD_ACTION_SENT, OCD_SERV_G2PCERA, "GRID_SITE_UPDATE",
                             emptyTnaIdent, emptyLinkIdent);
        break;
      case UPDATE_GUNIGW:
        if (1)
          gunigw_proxy->gridSiteUpdate(nodeId, siteInfo);
        else
        {
          if(IS_DEBUG_GRID_NODE(CORBA_ALL))
            OSPF_CORBA_DEBUG(OCD_TYPE_GRID, OCD_ACTION_ABORTED, OCD_SERV_GUNIGW, "GRID_SITE_UPDATE",
                             emptyTnaIdent, emptyLinkIdent);
          return;
        }
        if(IS_DEBUG_GRID_NODE(CORBA))
          OSPF_CORBA_DEBUG(OCD_TYPE_GRID, OCD_ACTION_SENT, OCD_SERV_GUNIGW, "GRID_SITE_UPDATE",
                           emptyTnaIdent, emptyLinkIdent);
        break;
    }
  } catch (g2mplsTopology::CannotFetchNode) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchNode (method gridSiteUpdate)");
    return;
  } catch (g2mplsTopology::NodeParamsMismatch) {

    zlog_debug("[ERR] CORBA: Exception NodeParamsMismatch (method gridSiteUpdate)");
    return;
  } catch (g2mplsTopology::InvocationNotAllowed) {

    zlog_debug("[ERR] CORBA: Exception InvocationNotAllowed (method gridSiteUpdate)");
    return;
  } catch (g2mplsTopology::InternalProblems & e) {

    zlog_debug("[ERR] CORBA: Exception InternalProblems (method gridSiteUpdate): %s", (char *) e.what);
  } catch (...) {

    zlog_debug("[ERR] CORBA: Exception Unknown (method gridSiteUpdate)");
    return;
  }
  return;
}

void corba_update_gn_GridService_ID(uint32_t id)
{
  if(IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridService ID");
  subNodeId = ntohl(id);
  return;
}

void corba_update_gn_GridService_ParentSite_ID(uint32_t id)
{
  if(IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridService PARENT_SITE_ID");
  nodeId = ntohl(id);
  return;
}

void corba_update_gn_GridService_ServiceInfo(uint16_t type, uint16_t version)
{
  if(IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridService SERVICE_INFO");
  serviceInfo.data.type =  service_type_from_uint(ntohs(type));

  uint32_t value = 0; uint32_t temp = 0;
  value = (uint32_t) ntohs(version);
  temp = value>>12;
  temp &= 0xf;
  serviceInfo.data.mjrRev = temp;
  temp = value>>6;
  temp &= 0x3f;
  serviceInfo.data.mnrRev = temp;
  temp = value;
  temp &= 0x3f;
  serviceInfo.data.bldFix = temp;
  serviceInfo.data.valid = true;
  return;
}

void corba_update_gn_GridService_Status(char status)
{
  if(IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridService STATUS");
  serviceInfo.state = service_state_from_uint(status);
  return;
}

char addrLength;

void corba_update_gn_GridService_AddressLength(u_char addressLength)
{
  if(addressLength == 0)
    addrLength = 32;
  else
    addrLength = addressLength;
  if(IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridService ADDRESS_LENGTH: %d", (u_char) addrLength);
  return;
}

void corba_update_gn_GridService_IPv4Endpoint(struct in_addr ipv4Endp)
{
  if((u_char)addrLength == 32)
  {
    if(IS_DEBUG_GRID_NODE(CORBA_ALL))
      zlog_debug("[DBG] CORBA: Update GridService IPV4_ENDPOINT");
    g2mpls_addr_t addr;
    addr.type = IPv4;
    addr.value.ipv4 = ipv4Endp;
    g2mplsTypes::gridHostId_var host;
    host << addr;
    serviceInfo.endPointAddr = host;
  }
  return;
}

void corba_update_gn_GridService_IPv6Endpoint(struct in6_addr ipv6Endp)
{
  if((u_char)addrLength == 128)
  {
    if(IS_DEBUG_GRID_NODE(CORBA_ALL))
      zlog_debug("[DBG] CORBA: Update GridService IPV6_ENDPOINT");
    g2mpls_addr_t addr;
    addr.type = IPv6;
    addr.value.ipv6 = ipv6Endp;
    g2mplsTypes::gridHostId_var host;
    host << addr;
    serviceInfo.endPointAddr = host;
  }
  return;
}

void corba_update_gn_GridService_NsapEndpoint(uint32_t nsapEndp[])
{
  if((u_char)addrLength == 160)
  {
    if(IS_DEBUG_GRID_NODE(CORBA_ALL))
      zlog_debug("[DBG] CORBA: Update GridService NSAP_ENDPOINT");
    g2mpls_addr_t addr;
    addr.type = NSAP;
    uint32_t value[5];
    for(int j = 0; j < 5; j++)
      value[j] = ntohl(nsapEndp[4-j]);
    uint8_t *part = (uint8_t *) value;
    for (int i = 0; i < 20; i++)
      addr.value.nsap.nsap_addr8[i] = *part++;
    g2mplsTypes::gridHostId_var host;
    host << addr;
    serviceInfo.endPointAddr = host;
  }
  return;
}

void init_grid_GridService()
{
  if(IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridService");

  serviceInfo.data.type =  g2mplsTypes::SERVICE_UNKNOWN;
  serviceInfo.data.mjrRev = 0;
  serviceInfo.data.mnrRev = 0;
  serviceInfo.data.bldFix = 0;
  serviceInfo.data.valid = false;
  serviceInfo.state = g2mplsTypes::GRIDSERVICESTATE_UNKNOWN;

  g2mpls_addr_t addr;
  addr.type = IPv4;
  addr.value.ipv4.s_addr = 0;
  g2mplsTypes::gridHostId_var host;
  host << addr;
  serviceInfo.endPointAddr = host;
  return;
}

void corba_update_gn_GridService(int server)
{
  g2mplsTypes::gridServiceParams serverServiceInfo;
  g2mplsTypes::gridSiteParams* serverSiteInfo;
  g2mplsTypes::gridSubNodes* snodes;
  g2mplsTypes::gridSubNodeIdent sNodeIdent;

  g2mplsTypes::nodeIdent ident;
  ident.id = nodeId;
  ident.type = g2mplsTypes::NODETYPE_GRID;

  sNodeIdent.id = subNodeId;
  sNodeIdent.type = g2mplsTypes::GRIDSUBNODETYPE_SERVICE;

  try {
    switch(server)
    {
      case UPDATE_G2PCERA:

        if (!isNodeInSeq(ident, g2pcera_proxy->nodeGetAll()))
          node_add(UPDATE_G2PCERA, nodeId, NTYPE_GRID);

        g2pcera_proxy->gridSiteGet(nodeId, serverSiteInfo, snodes);
        if (isSubNodeInSeq(sNodeIdent, &snodes->services))
        {
          g2pcera_proxy->gridServiceGet(nodeId, subNodeId, serverServiceInfo);
          if (!equalGridServices(&serviceInfo, &serverServiceInfo))
            g2pcera_proxy->gridServiceUpdate(nodeId, subNodeId, serviceInfo);
          else
          {
            if (IS_DEBUG_GRID_NODE(CORBA_ALL))
              OSPF_CORBA_DEBUG(OCD_TYPE_GRID, OCD_ACTION_ABORTED, OCD_SERV_G2PCERA, "GRID_SERVICE_UPDATE",
                               emptyTnaIdent, emptyLinkIdent);
            return;
          }
        }
        else
          g2pcera_proxy->gridServiceUpdate(nodeId, subNodeId, serviceInfo);

        if (IS_DEBUG_GRID_NODE(CORBA))
          OSPF_CORBA_DEBUG(OCD_TYPE_GRID, OCD_ACTION_SENT, OCD_SERV_G2PCERA, "GRID_SERVICE_UPDATE",
                           emptyTnaIdent, emptyLinkIdent);
        break;
      case UPDATE_GUNIGW:

        if (!isNodeInSeq(ident, gunigw_proxy->nodeGetAll()))
          node_add(UPDATE_GUNIGW, nodeId, NTYPE_GRID);

        gunigw_proxy->gridSiteGet(nodeId, serverSiteInfo, snodes);
        if (isSubNodeInSeq(sNodeIdent, &snodes->services))
        {
          gunigw_proxy->gridServiceGet(nodeId, subNodeId, serverServiceInfo);
          if (!equalGridServices(&serviceInfo, &serverServiceInfo))
            gunigw_proxy->gridServiceUpdate(nodeId, subNodeId, serviceInfo);
          else
          {
            if (IS_DEBUG_GRID_NODE(CORBA_ALL))
              OSPF_CORBA_DEBUG(OCD_TYPE_GRID, OCD_ACTION_ABORTED, OCD_SERV_GUNIGW, "GRID_SERVICE_UPDATE",
                               emptyTnaIdent, emptyLinkIdent);
            return;
          }
        }
        else
          gunigw_proxy->gridServiceUpdate(nodeId, subNodeId, serviceInfo);

        if (IS_DEBUG_GRID_NODE(CORBA))
          OSPF_CORBA_DEBUG(OCD_TYPE_GRID, OCD_ACTION_SENT, OCD_SERV_GUNIGW, "GRID_SERVICE_UPDATE",
                           emptyTnaIdent, emptyLinkIdent);
        break;
    }
  } catch (g2mplsTopology::CannotFetchNode) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchNode (method gridServiceUpdate)");
    return;
  }  catch (g2mplsTopology::CannotFetchSubNode) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchSubNode (method gridServiceUpdate)");
    return;
  } catch (g2mplsTopology::SubNodeParamsMismatch) {

    zlog_debug("[ERR] CORBA: Exception NodeParamsMismatch (method gridServiceUpdate)");
    return;
  } catch (g2mplsTopology::InvocationNotAllowed) {

    zlog_debug("[ERR] CORBA: Exception InvocationNotAllowed (method gridServiceUpdate)");
    return;
  } catch (g2mplsTopology::InternalProblems & e) {

    zlog_debug("[ERR] CORBA: Exception InternalProblems (method gridServiceUpdate): %s", (char *) e.what);
  } catch (...) {

    zlog_debug("[ERR] CORBA: Exception Unknown (method gridServiceUpdate)");
    return;
  }
  return;
}

void corba_update_gn_GCE_ID(uint32_t id)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridComputingElement ID");
  subNodeId = ntohl(id);
  return;
}

void corba_update_gn_GCE_ParentSiteID(uint32_t id)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridComputingElement PARENT_SITE_ID");
  nodeId = ntohl(id);
  return;
}

void corba_update_gn_GCE_LrmsInfo(uint16_t lrmsType, uint16_t lrmsVersion)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridComputingElement LRMS_INFO");
  ceInfo.lrmsInfo.type =  lrms_type_from_uint(ntohs(lrmsType));

  uint32_t value = 0; uint32_t temp = 0;
  value = (uint32_t) ntohs(lrmsVersion);
  temp = value>>12;
  temp &= 0xf;
  ceInfo.lrmsInfo.mjrRev = temp;
  temp = value>>6;
  temp &= 0x3f;
  ceInfo.lrmsInfo.mnrRev = temp;
  temp = value;
  temp &= 0x3f;
  ceInfo.lrmsInfo.bldFix = temp;
  ceInfo.lrmsInfo.valid = true;
  return;
}

void corba_update_gn_GCE_AddressLength(u_char addressLength)
{
  if(addressLength == 0)
    addrLength = 32;
  else
    addrLength = addressLength;
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridComputingElement ADDRESS_LENGTH: %d", (u_char) addrLength);
  return;
}

void corba_update_gn_GCE_IPv4HostName(struct in_addr ipv4HostNam)
{
  if((u_char)addrLength == 32)
  {
    if (IS_DEBUG_GRID_NODE(CORBA_ALL))
      zlog_debug("[DBG] CORBA: Update GridComputingElement IPV4_HOSTNAME");
    g2mpls_addr_t addr;
    addr.type = IPv4;
    addr.value.ipv4 = ipv4HostNam;
    g2mplsTypes::gridHostId_var host;
    host << addr;
    ceInfo.hostAddr = host;
  }
  return;
}

void corba_update_gn_GCE_IPv6HostName(struct in6_addr ipv6HostNam)
{
  if((u_char)addrLength == 128)
  {
    if (IS_DEBUG_GRID_NODE(CORBA_ALL))
      zlog_debug("[DBG] CORBA: Update GridComputingElement IPV6_HOSTNAME");
    g2mpls_addr_t addr;
    addr.type = IPv6;
    addr.value.ipv6 = ipv6HostNam;
    g2mplsTypes::gridHostId_var host;
    host << addr;
    ceInfo.hostAddr = host;
  }
  return;
}

void corba_update_gn_GCE_NsapHostName(uint32_t nsapHostNam[])
{
  if((u_char)addrLength == 160)
  {
    if (IS_DEBUG_GRID_NODE(CORBA_ALL))
      zlog_debug("[DBG] CORBA: Update GridComputingElement NSAP_HOSTNAME");
    g2mpls_addr_t addr;
    addr.type = NSAP;
    uint32_t value[5];
    for(int j = 0; j < 5; j++)
      value[j] = ntohl(nsapHostNam[4-j]);
    uint8_t *part = (uint8_t *) value;
    for (int i = 0; i < 20; i++)
      addr.value.nsap.nsap_addr8[i] = *part++;
    g2mplsTypes::gridHostId_var host;
    host << addr;
    ceInfo.hostAddr = host;
  }
  return;
}

void corba_update_gn_GCE_GatekeeperPort(uint32_t gateKPort)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridComputingElement GATEKEEPER_PORT");
  ceInfo.gatekeeperPort = ntohl (gateKPort);
  return;
}

void corba_update_gn_GCE_JobManager(const char* jman)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridComputingElement JOB_MANAGER");
  ceInfo.jobManager = jman;
  return;
}

void corba_update_gn_GCE_DataDir(const char* datd)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridComputingElement DATA_DIR");
  ceInfo.dataDir = datd;
  return;
}

void corba_update_gn_GCE_DefaultStorageElement(uint32_t defaultSelement)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridComputingElement DEFAULT_STORAGE_ELEMENT");
  ceInfo.defaultStorageElemId = ntohl (defaultSelement);
  return;
}

void corba_update_gn_GCE_JobsStates(uint16_t freeJobSlots, char status)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridComputingElement JOBS_STATES");
  ceInfo.jobsState.freeJobSlots = ntohs (freeJobSlots);
  ceInfo.jobsState.state = cese_state_from_uint(status);
  ceInfo.jobsState.valid = true;
  return;
}

void corba_update_gn_GCE_JobsStats(uint32_t runningJobs, uint32_t waitingJobs, uint32_t totalJobs)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridComputingElement JOBS_STATES");
  ceInfo.jobsStats.runningJobs =  ntohl(runningJobs);
  ceInfo.jobsStats.waitingJobs = ntohl(waitingJobs);
  ceInfo.jobsStats.totalJobs = ntohl (totalJobs);
  ceInfo.jobsStats.valid = true;
  return;
}

void corba_update_gn_GCE_JobsTimePerformances(uint32_t estRespTime, uint32_t worstRespTime)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridComputingElement JOBS_TIME_PERFORMANCES");
  ceInfo.jobsTimePerf.estimatedResponseTime = ntohl(estRespTime);
  ceInfo.jobsTimePerf.worstResponseTime = ntohl(worstRespTime);
  ceInfo.jobsTimePerf.valid = true;
  return;
}

void corba_update_gn_GCE_JobsTimePolicy(uint32_t maxWcTime, uint32_t maxObtWcTime, uint32_t maxCpuTime, uint32_t maxObtCpuTime)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridComputingElement JOBS_TIME_POLICY");
  ceInfo.jobsTimePolicy.maxWallclockTime = ntohl (maxWcTime);
  ceInfo.jobsTimePolicy.maxObtainableWallclockTime = ntohl (maxObtWcTime);
  ceInfo.jobsTimePolicy.maxCpuTime = ntohl (maxCpuTime);
  ceInfo.jobsTimePolicy.maxObtainableCpuTime = ntohl (maxObtCpuTime);
  ceInfo.jobsTimePolicy.valid = true;
  return;
}

void corba_update_gn_GCE_JobsLoadPolicy(uint32_t maxTotalJobs, uint32_t maxRunJobs, uint32_t maxWaitJobs, uint16_t assignJobSlots, uint16_t maxSlotsPerJob, char priorityPreemptionFlag)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridComputingElement JOBS_LOAD_POLICY");
  ceInfo.jobsLoadPolicy.maxTotalJobs = ntohl (maxTotalJobs);
  ceInfo.jobsLoadPolicy.maxRunningJobs = ntohl (maxRunJobs);
  ceInfo.jobsLoadPolicy.maxWaitingJobs = ntohl (maxWaitJobs);
  ceInfo.jobsLoadPolicy.assignedJobSlots = ntohs (assignJobSlots);
  ceInfo.jobsLoadPolicy.maxSlotsPerJob = ntohs (maxSlotsPerJob);
  uint8_t temp = priorityPreemptionFlag;
  temp >>= 1; temp &= 0x7f;
  ceInfo.jobsLoadPolicy.priority = temp;
  temp = priorityPreemptionFlag & 0x1;
  ceInfo.jobsLoadPolicy.preemptionFlag = temp;
  ceInfo.jobsLoadPolicy.valid = true;
  return;
}

void corba_update_gn_GCE_CeCalendar(struct zlist *ceCalList)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridComputingElement CE_CALENDAR");
  struct zlistnode * node;
  void *data;
  struct ce_calendar * elem;

  g2mplsTypes::JobSlotsCalendarSeq_var seq;
  {
    g2mplsTypes::JobSlotsCalendarSeq * tmp;
    tmp = new g2mplsTypes::JobSlotsCalendarSeq(listcount(ceCalList));
    if (!tmp) {
      zlog_debug("[ERR] CORBA: corba_update_gn_GCE_CeCalendar: tmp == NULL");
      return;
    }
    seq = tmp;
  }
  seq->length(listcount(ceCalList));

  int i = 0;
  for (ALL_LIST_ELEMENTS_RO(ceCalList, node, data)) {
    elem = (struct ce_calendar *) data;
    assert(elem != 0);

    g2mplsTypes::JobSlotsCalendarEvent event;
    event.unixTime = ntohl (elem->time);
    event.JobSlots = ntohs (elem->freeJobSlots);
    seq[i] = event;

    i++;
  }
  ceInfo.freeJobSlotsCalendar = seq;
  return;
}

void corba_update_gn_GCE_Name(const char* name)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridComputingElement NAME");
  ceInfo.name = name;
  return;
}

void init_grid_GridComputingElement()
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridComputingElement");
  addrLength = 32;

  const char* str = "";
  ceInfo.name = str;
  ceInfo.dataDir = str;
  ceInfo.jobManager = str;

  ceInfo.lrmsInfo.type = g2mplsTypes::GRIDLRMSTYPE_UNKNOWN;
  ceInfo.lrmsInfo.mjrRev = 0;
  ceInfo.lrmsInfo.mnrRev = 0;
  ceInfo.lrmsInfo.bldFix = 0;
  ceInfo.lrmsInfo.valid = false;

  g2mpls_addr_t addr;
  addr.type = IPv4;
  addr.value.ipv4.s_addr = 0;
  g2mplsTypes::gridHostId_var host;
  host << addr;
  ceInfo.hostAddr = host;

  ceInfo.gatekeeperPort = 0;

  ceInfo.defaultStorageElemId = 0;
  ceInfo.jobsState.freeJobSlots = 0;
  ceInfo.jobsState.state = g2mplsTypes::GRIDCESESTATE_UNKNOWN;
  ceInfo.jobsState.valid = false;
  ceInfo.jobsStats.runningJobs = 0;
  ceInfo.jobsStats.waitingJobs = 0;
  ceInfo.jobsStats.totalJobs = 0;
  ceInfo.jobsStats.valid = false;
  ceInfo.jobsTimePerf.estimatedResponseTime = 0;
  ceInfo.jobsTimePerf.worstResponseTime = 0;
  ceInfo.jobsTimePerf.valid = false;
  ceInfo.jobsTimePolicy.maxWallclockTime = 0;
  ceInfo.jobsTimePolicy.maxObtainableWallclockTime = 0;
  ceInfo.jobsTimePolicy.maxCpuTime = 0;
  ceInfo.jobsTimePolicy.maxObtainableCpuTime = 0;
  ceInfo.jobsTimePolicy.valid = false;
  ceInfo.jobsLoadPolicy.maxTotalJobs = 0;
  ceInfo.jobsLoadPolicy.maxRunningJobs = 0;
  ceInfo.jobsLoadPolicy.maxWaitingJobs = 0;
  ceInfo.jobsLoadPolicy.assignedJobSlots = 0;
  ceInfo.jobsLoadPolicy.maxSlotsPerJob = 0;
  ceInfo.jobsLoadPolicy.priority = 
  ceInfo.jobsLoadPolicy.preemptionFlag = 0;
  ceInfo.jobsLoadPolicy.valid = false;

  g2mplsTypes::JobSlotsCalendarSeq_var seq;
  {
    g2mplsTypes::JobSlotsCalendarSeq * tmp;
    tmp = new g2mplsTypes::JobSlotsCalendarSeq(0);
    if (!tmp) {
      zlog_debug("[ERR] CORBA: init_grid_GridComputingElement: tmp == NULL");
      return;
    }
    seq = tmp;
  }
  seq->length(0);
  ceInfo.freeJobSlotsCalendar = seq;
  return;
}

void corba_update_gn_GCE(int server)
{
  g2mplsTypes::gridCEParams* serverCeInfo;
  g2mplsTypes::gridSiteParams* serverSiteInfo;
  g2mplsTypes::gridSubNodes* snodes;
  g2mplsTypes::gridSubNodeIdent sNodeIdent;

  g2mplsTypes::nodeIdent ident;
  ident.id = nodeId;
  ident.type = g2mplsTypes::NODETYPE_GRID;

  sNodeIdent.id = subNodeId;
  sNodeIdent.type = g2mplsTypes::GRIDSUBNODETYPE_COMPUTINGELEMENT;

  try {
    switch(server)
    {
      case UPDATE_G2PCERA:

        if (!isNodeInSeq(ident, g2pcera_proxy->nodeGetAll()))
          node_add(UPDATE_G2PCERA, nodeId, NTYPE_GRID);

        g2pcera_proxy->gridSiteGet(nodeId, serverSiteInfo, snodes);
        if (isSubNodeInSeq(sNodeIdent, &snodes->compElems))
        {
          g2pcera_proxy->gridCompElemGet(nodeId, subNodeId, serverCeInfo);
          if (!equalGridCompElems(&ceInfo, serverCeInfo))
            g2pcera_proxy->gridCompElemUpdate(nodeId, subNodeId, ceInfo);
          else
          {
            if (IS_DEBUG_GRID_NODE(CORBA_ALL))
              OSPF_CORBA_DEBUG(OCD_TYPE_GRID, OCD_ACTION_ABORTED, OCD_SERV_G2PCERA, "GRID_COMP_ELEM_UPDATE",
                               emptyTnaIdent, emptyLinkIdent);
            return;
          }
        }
        else
          g2pcera_proxy->gridCompElemUpdate(nodeId, subNodeId, ceInfo);

        if (IS_DEBUG_GRID_NODE(CORBA))
          OSPF_CORBA_DEBUG(OCD_TYPE_GRID, OCD_ACTION_SENT, OCD_SERV_G2PCERA, "GRID_COMP_ELEM_UPDATE",
                           emptyTnaIdent, emptyLinkIdent);
        break;
      case UPDATE_GUNIGW:

        if (!isNodeInSeq(ident, gunigw_proxy->nodeGetAll()))
          node_add(UPDATE_GUNIGW, nodeId, NTYPE_GRID);

        gunigw_proxy->gridSiteGet(nodeId, serverSiteInfo, snodes);
        if (isSubNodeInSeq(sNodeIdent, &snodes->compElems))
        {
          gunigw_proxy->gridCompElemGet(nodeId, subNodeId, serverCeInfo);
          if (!equalGridCompElems(&ceInfo, serverCeInfo))
            gunigw_proxy->gridCompElemUpdate(nodeId, subNodeId, ceInfo);
          else
          {
            if (IS_DEBUG_GRID_NODE(CORBA_ALL))
              OSPF_CORBA_DEBUG(OCD_TYPE_GRID, OCD_ACTION_ABORTED, OCD_SERV_GUNIGW, "GRID_COMP_ELEM_UPDATE",
                               emptyTnaIdent, emptyLinkIdent);
            return;
          }
        }
        else
          gunigw_proxy->gridCompElemUpdate(nodeId, subNodeId, ceInfo);

        if (IS_DEBUG_GRID_NODE(CORBA))
          OSPF_CORBA_DEBUG(OCD_TYPE_GRID, OCD_ACTION_SENT, OCD_SERV_GUNIGW, "GRID_COMP_ELEM_UPDATE",
                           emptyTnaIdent, emptyLinkIdent);
        break;
    }
  } catch (g2mplsTopology::CannotFetchNode) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchNode (method gridCompElemUpdate)");
    return;
  }  catch (g2mplsTopology::CannotFetchSubNode) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchSubNode (method gridCompElemUpdate)");
    return;
  } catch (g2mplsTopology::SubNodeParamsMismatch) {

    zlog_debug("[ERR] CORBA: Exception NodeParamsMismatch (method gridCompElemUpdate)");
    return;
  } catch (g2mplsTopology::InvocationNotAllowed) {

    zlog_debug("[ERR] CORBA: Exception InvocationNotAllowed (method gridCompElemUpdate)");
    return;
  } catch (g2mplsTopology::InternalProblems & e) {

    zlog_debug("[ERR] CORBA: Exception InternalProblems (method gridCompElemUpdate): %s", (char *) e.what);
  } catch (...) {

    zlog_debug("[ERR] CORBA: Exception Unknown (method gridCompElemUpdate)");
    return;
  }
  return;
}

void corba_update_gn_GSCluster_ID(uint32_t id)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridSubCluster ID");
  subNodeId = ntohl(id);
  return;
}

void corba_update_gn_GSCluster_ParentSiteID(uint32_t parSiteId)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridSubCluster PARENT_SITE_ID");
  nodeId = ntohl(parSiteId);
  return;
}

void corba_update_gn_GSCluster_CpuInfo(uint32_t physicalCpus, uint32_t logicalCpus, char cpuArch)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridSubCluster CPU_INFO");
  subClusterInfo.cpu.cpuCounts.physical = ntohl (physicalCpus);
  subClusterInfo.cpu.cpuCounts.logical = ntohl (logicalCpus);
  subClusterInfo.cpu.cpuArch = cpu_arch_from_uint(cpuArch);
  subClusterInfo.cpu.valid = true;
  return;
}

void corba_update_gn_GSCluster_OsInfo(uint16_t osType, uint16_t osVersion)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridSubCluster OS_INFO");
  subClusterInfo.os.type = os_type_from_uint(ntohs (osType));

  uint32_t value = 0; uint32_t temp = 0;
  value = (uint32_t) ntohs (osVersion);
  temp = value>>12;
  temp &= 0xf;
  subClusterInfo.os.mjrRev = temp;
  temp = value>>6;
  temp &= 0x3f;
  subClusterInfo.os.mnrRev = temp;
  temp = value;
  temp &= 0x3f;
  subClusterInfo.os.bldFix = temp;
  subClusterInfo.os.valid = true;
  return;
}

void corba_update_gn_GSCluster_MemoryInfo(uint32_t ramSize, uint32_t virtualMemorySize)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridSubCluster MEMORY_INFO");
  subClusterInfo.memory.ramSize = ntohl (ramSize);
  subClusterInfo.memory.virtualMemorySize = ntohl (virtualMemorySize);
  subClusterInfo.memory.valid = true;
  return;
}

typedef struct grid_subcluster_softpac{
  uint16_t                  soft_type;
  uint16_t                  soft_version;
  char *                    soft_environment_setup;
} grid_subcluster_softpac_t;

struct zlist SoftwarePackeges;

void corba_update_gn_GSCluster_NewSoftwarePackage(uint16_t softType, uint16_t softVersion, char *env)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridSubCluster New SOFTWARE_PACKAGE");
  grid_subcluster_softpac_t* softPac = (grid_subcluster_softpac_t *) XMALLOC(0, sizeof(grid_subcluster_softpac_t));
  softPac->soft_type = softType;
  softPac->soft_version = softVersion;
  softPac->soft_environment_setup = env;
  listnode_add(&SoftwarePackeges, softPac);
  return;
}

void corba_update_gn_GSCluster_SoftwarePackages()
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridSubCluster SOFTWARE_PACKAGE");

  void *data; 
  struct zlistnode *node, *nnode;

  g2mplsTypes::softwarePackageSeq_var seq;
  {
    g2mplsTypes::softwarePackageSeq * tmp;
    tmp = new g2mplsTypes::softwarePackageSeq(listcount(&SoftwarePackeges));
    if (!tmp) {
      zlog_debug("[ERR] CORBA: corba_update_gn_GSCluster_SoftwarePackages: tmp == NULL");
      return;
    }
    seq = tmp;
  }
  seq->length(listcount(&SoftwarePackeges));

  grid_subcluster_softpac_t * elem;
  uint32_t value, temp, z;
  z = 0;
  for (ALL_LIST_ELEMENTS (&SoftwarePackeges, node, nnode, data)) {

    elem = (grid_subcluster_softpac_t *) data;

    g2mplsTypes::gridSoftwarePackage softPac;
    softPac.software.type = appl_type_from_uint(ntohs (elem->soft_type));
    value = 0; temp = 0;
    value = (uint32_t) ntohs (elem->soft_version);
    temp = value>>12;
    temp &= 0xf;
    softPac.software.mjrRev = temp;
    temp = value>>6;
    temp &= 0x3f;
    softPac.software.mnrRev = temp;
    temp = value;
    temp &= 0x3f;
    softPac.software.bldFix = temp;
    softPac.software.valid = true;
    softPac.softwareEnvironmentSetup = elem->soft_environment_setup;
    seq[z] = softPac;
    z++;
  }
  subClusterInfo.softwarePackages = seq;

  return;
}

void corba_update_gn_GSCluster_SubClusterCalendar(struct zlist *scCalList)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridSubCluster SUBCLUSTER_CALENDAR");
  struct zlistnode * node;
  void *data;
  struct sc_calendar * elem;
  int i;

  g2mplsTypes::subClusterCalendarSeq_var seq;
  {
    g2mplsTypes::subClusterCalendarSeq * tmp;
    tmp = new g2mplsTypes::subClusterCalendarSeq(listcount(scCalList));
    if (!tmp) {
      zlog_debug("[ERR] CORBA: corba_update_gn_GridStorage_SubClusterCalendar: tmp == NULL");
      return;
    }
    seq = tmp;
  }
  seq->length(listcount(scCalList));

  i = 0;
  for (ALL_LIST_ELEMENTS_RO(scCalList, node, data)) {
    elem = (struct sc_calendar *) data;
    assert(elem != 0);

    g2mplsTypes::subClusterCalendarEvent event;

    event.unixTime = ntohl (elem->time);
    event.cpuCount.physical = ntohs (elem->physical_cpus);
    event.cpuCount.logical = ntohs (elem->logical_cpus);
    seq[i] = event;
    i++;
  }
  subClusterInfo.subClusterCalendar = seq;
  return;
}

void corba_update_gn_GSCluster_Name(const char* name)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridSubCluster NAME");
  subClusterInfo.name = name;
  return;
}

void init_grid_GridSubCluster()
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridSubCluster");

  memset (&SoftwarePackeges, 0, sizeof (struct zlist));

  const char* str = "";
  subClusterInfo.name = str;

  subClusterInfo.cpu.cpuCounts.physical = 0;
  subClusterInfo.cpu.cpuCounts.logical = 0;
  subClusterInfo.cpu.cpuArch = g2mplsTypes::GRIDCPUARCH_UNKNOWN;
  subClusterInfo.cpu.valid = false;
  subClusterInfo.os.type = g2mplsTypes::GRIDOSTYPE_UNKNOWN;
  subClusterInfo.os.mjrRev = 0;
  subClusterInfo.os.mnrRev = 0;
  subClusterInfo.os.bldFix = 0;
  subClusterInfo.os.valid = false;
  subClusterInfo.memory.ramSize = 0;
  subClusterInfo.memory.virtualMemorySize = 0;
  subClusterInfo.memory.valid = false;

  g2mplsTypes::softwarePackageSeq_var seq;
  {
    g2mplsTypes::softwarePackageSeq * tmp;
    tmp = new g2mplsTypes::softwarePackageSeq(0);
    if (!tmp) {
      zlog_debug("[ERR] CORBA: init_grid_GridSubCluster: tmp == NULL");
      return;
    }
    seq = tmp;
  }
  seq->length(0);
  subClusterInfo.softwarePackages = seq;

  g2mplsTypes::subClusterCalendarSeq_var seq2;
  {
    g2mplsTypes::subClusterCalendarSeq * tmp;
    tmp = new g2mplsTypes::subClusterCalendarSeq(0);
    if (!tmp) {
      zlog_debug("[ERR] CORBA: init_grid_GridSubCluster: tmp == NULL");
      return;
    }
    seq2 = tmp;
  }
  seq2->length(0);
  subClusterInfo.subClusterCalendar = seq2;

  return;
}

void corba_update_gn_GSCluster(int server)
{
  g2mplsTypes::gridSubClusterParams* serverSubClusterInfo;
  g2mplsTypes::gridSiteParams* serverSiteInfo;
  g2mplsTypes::gridSubNodes* snodes;
  g2mplsTypes::gridSubNodeIdent sNodeIdent;

  g2mplsTypes::nodeIdent ident;
  ident.id = nodeId;
  ident.type = g2mplsTypes::NODETYPE_GRID;

  sNodeIdent.id = subNodeId;
  sNodeIdent.type = g2mplsTypes::GRIDSUBNODETYPE_SUBCLUSTER;

  try {
    switch(server)
    {
      case UPDATE_G2PCERA:

        if (!isNodeInSeq(ident, g2pcera_proxy->nodeGetAll()))
          node_add(UPDATE_G2PCERA, nodeId, NTYPE_GRID);

        g2pcera_proxy->gridSiteGet(nodeId, serverSiteInfo, snodes);
        if (isSubNodeInSeq(sNodeIdent, &snodes->subClusters))
        {
          g2pcera_proxy->gridSubClusterGet(nodeId, subNodeId, serverSubClusterInfo);
          if (!equalGridSubClusters(&subClusterInfo, serverSubClusterInfo))
            g2pcera_proxy->gridSubClusterUpdate(nodeId, subNodeId, subClusterInfo);
          else
          {
            if (IS_DEBUG_GRID_NODE(CORBA_ALL))
              OSPF_CORBA_DEBUG(OCD_TYPE_GRID, OCD_ACTION_ABORTED, OCD_SERV_G2PCERA, "GRID_SUBCLUSTER_UPDATE",
                               emptyTnaIdent, emptyLinkIdent);
            return;
          }
        }
        else
          g2pcera_proxy->gridSubClusterUpdate(nodeId, subNodeId, subClusterInfo);

        if (IS_DEBUG_GRID_NODE(CORBA))
          OSPF_CORBA_DEBUG(OCD_TYPE_GRID, OCD_ACTION_SENT, OCD_SERV_G2PCERA, "GRID_SUBCLUSTER_UPDATE",
                           emptyTnaIdent, emptyLinkIdent);

        break;
      case UPDATE_GUNIGW:

        if (!isNodeInSeq(ident, gunigw_proxy->nodeGetAll()))
          node_add(UPDATE_GUNIGW, nodeId, NTYPE_GRID);

        gunigw_proxy->gridSiteGet(nodeId, serverSiteInfo, snodes);
        if (isSubNodeInSeq(sNodeIdent, &snodes->subClusters))
        {
          gunigw_proxy->gridSubClusterGet(nodeId, subNodeId, serverSubClusterInfo);
          if (!equalGridSubClusters(&subClusterInfo, serverSubClusterInfo))
            gunigw_proxy->gridSubClusterUpdate(nodeId, subNodeId, subClusterInfo);
          else
          {
            if (IS_DEBUG_GRID_NODE(CORBA_ALL))
              OSPF_CORBA_DEBUG(OCD_TYPE_GRID, OCD_ACTION_ABORTED, OCD_SERV_GUNIGW, "GRID_SUBCLUSTER_UPDATE",
                               emptyTnaIdent, emptyLinkIdent);
            return;
          }
        }
        else
          gunigw_proxy->gridSubClusterUpdate(nodeId, subNodeId, subClusterInfo);

        if (IS_DEBUG_GRID_NODE(CORBA))
          OSPF_CORBA_DEBUG(OCD_TYPE_GRID, OCD_ACTION_SENT, OCD_SERV_GUNIGW, "GRID_SUBCLUSTER_UPDATE",
                           emptyTnaIdent, emptyLinkIdent);
        break;
    }
  } catch (g2mplsTopology::CannotFetchNode) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchNode (method gridSubClusterUpdate)");
    return;
  }  catch (g2mplsTopology::CannotFetchSubNode) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchSubNode (method gridSubClusterUpdate)");
    return;
  } catch (g2mplsTopology::SubNodeParamsMismatch) {

    zlog_debug("[ERR] CORBA: Exception NodeParamsMismatch (method gridSubClusterUpdate)");
    return;
  } catch (g2mplsTopology::InvocationNotAllowed) {

    zlog_debug("[ERR] CORBA: Exception InvocationNotAllowed (method gridSubClusterUpdate)");
    return;
  } catch (g2mplsTopology::InternalProblems & e) {

    zlog_debug("[ERR] CORBA: Exception InternalProblems (method gridSubClusterUpdate): %s", (char *) e.what);
  } catch (...) {

    zlog_debug("[ERR] CORBA: Exception Unknown (method gridSubClusterUpdate)");
    return;
  }
  return;
}

void corba_update_gn_GridStorage_ID(uint32_t id)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridStorage ID");
  subNodeId = ntohl(id);
  return;
}

void corba_update_gn_GridStorage_ParentSiteID(uint32_t parSiteId)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridStorage PARENT_SITE_ID");
  nodeId = ntohl(parSiteId);
  return;
}

void corba_update_gn_GridStorage_StorageInfo(uint32_t storInfo)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridStorage STORAGE_INFO");
  uint32_t value = ntohl(storInfo);
  uint32_t temp = value >> 28;
  temp &= 0xf;
  seInfo.storageInfo.arch = storage_arch_from_uint((uint8_t) temp);
  temp = value >> 24;
  temp &= 0xf;
  seInfo.storageInfo.state = cese_state_from_uint((uint8_t) temp);
  temp = value >> 12;
  temp &= 0xfff;
  seInfo.storageInfo.accessProtocolsMask = temp;
  temp = value & 0xfff;
  seInfo.storageInfo.controlProtocolsMask = temp;
  seInfo.storageInfo.valid = true;
  return;
}

void corba_update_gn_GridStorage_OnlineSize(uint32_t totalSize, uint32_t usedSize)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridStorage ONLINE_SIZE");
  seInfo.onlineSize.total = ntohl(totalSize);
  seInfo.onlineSize.used = ntohl(usedSize);
  seInfo.onlineSize.valid = true;
  return;
}

void corba_update_gn_GridStorage_NearlineSize(uint32_t totalSize, uint32_t usedSize)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridStorage NEARLINE_SIZE");
  seInfo.nearlineSize.total = ntohl(totalSize);
  seInfo.nearlineSize.used = ntohl(usedSize);
  seInfo.nearlineSize.valid = true;
  return;
}

typedef struct grid_storage_area{
  char *                    storage_area_name;
  char *                    storage_area_path;
  grid_storage_area_info_t  storage_area_info;
} grid_storage_area_t;

struct zlist StorageAreas;

void corba_update_gn_GridStorage_NewStorageArea(char *name, char *path, uint32_t totalOnlineSize, uint32_t freeOnlineSize, uint32_t resTotalOnlineSize, uint32_t totalNearlineSize, uint32_t freeNearlineSize, uint32_t resNearlineSize, char retPolAccLat, char expirationMode)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridStorage New STORAGE_AREA");

  grid_storage_area_t* storageArea = (grid_storage_area_t *) XMALLOC(0, sizeof(grid_storage_area_t));
  storageArea->storage_area_name = name;
  storageArea->storage_area_path = path;
  storageArea->storage_area_info.total_online_size = ntohl(totalOnlineSize);
  storageArea->storage_area_info.free_online_size = ntohl(freeOnlineSize);
  storageArea->storage_area_info.reserved_total_online_size = ntohl(resTotalOnlineSize);
  storageArea->storage_area_info.total_nearline_size = ntohl(totalNearlineSize);
  storageArea->storage_area_info.free_nearline_size = ntohl(freeNearlineSize);
  storageArea->storage_area_info.reserved_nearline_size = ntohl(resNearlineSize);
  uint8_t temp = retPolAccLat;
  temp >>= 4; temp &= 0xf;
  storageArea->storage_area_info.retention_policy = (grid_storage_retention_policy_t)temp;
  temp = retPolAccLat; temp &= 0xf;
  storageArea->storage_area_info.access_latency = (grid_storage_access_latency_t)temp;
  temp = expirationMode >> 4; temp &= 0xf;
  storageArea->storage_area_info.expiration_mode = (grid_storage_expiration_mode_t)temp;
  listnode_add(&StorageAreas, storageArea);
  return;
}

void corba_update_gn_GridStorage_StorageAreas()
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridStorage STORAGE_AREA");

  void *data; 
  struct zlistnode *node, *nnode;;

  g2mplsTypes::StorageAreaSeq_var seq;
  {
    g2mplsTypes::StorageAreaSeq * tmp;
    tmp = new g2mplsTypes::StorageAreaSeq(listcount(&StorageAreas));
    if (!tmp) {
      zlog_debug("[ERR] CORBA: corba_update_gn_GridStorage_StorageAreas: tmp == NULL");
      return;
    }
    seq = tmp;
  }
  seq->length(listcount(&StorageAreas));

  grid_storage_area_t *elem;
  uint32_t z = 0;
  for (ALL_LIST_ELEMENTS (&StorageAreas, node, nnode, data)) {

    elem = (grid_storage_area_t *) data; 

    g2mplsTypes::gridStorageArea stArea;
    stArea.storageAreaName = elem->storage_area_name;
    stArea.storageAreaPath = elem->storage_area_path;
    stArea.storageAreaInfo.totalOnlineSize = elem->storage_area_info.total_online_size;
    stArea.storageAreaInfo.freeOnlineSize = elem->storage_area_info.free_online_size;
    stArea.storageAreaInfo.reservedTotalOnlineSize = elem->storage_area_info.reserved_total_online_size;
    stArea.storageAreaInfo.totalNearlineSize = elem->storage_area_info.total_nearline_size;
    stArea.storageAreaInfo.freeNearlineSize = elem->storage_area_info.free_nearline_size;
    stArea.storageAreaInfo.reservedNearlineSize = elem->storage_area_info.reserved_nearline_size;
    stArea.storageAreaInfo.retentionPolicy =  storage_retpol_from_uint((int)elem->storage_area_info.retention_policy);
    stArea.storageAreaInfo.accessLatency = storage_acclat_from_uint((int)elem->storage_area_info.access_latency);
    stArea.storageAreaInfo.expirationMode = storage_expmode_from_uint((int)elem->storage_area_info.expiration_mode);
    stArea.storageAreaInfo.valid = true;
    seq[z] = stArea;
    z++;
  }
  seInfo.storageAreas = seq;

  return;
}

void corba_update_gn_GridStorage_SeCalendar(struct zlist *seCalList)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridStorage SE_CALENDAR");
  struct zlistnode * node;
  void *data;
  struct se_calendar * elem;
  int i;
  g2mplsTypes::seCalendarSeq_var seq;
  {
    g2mplsTypes::seCalendarSeq * tmp;
    tmp = new g2mplsTypes::seCalendarSeq(listcount(seCalList));
    if (!tmp) {
      zlog_debug("[ERR] CORBA: corba_update_gn_GridStorage_SeCalendar: tmp == NULL");
      return;
    }
    seq = tmp;
  }
  seq->length(listcount(seCalList));
  i = 0;
  for (ALL_LIST_ELEMENTS_RO(seCalList, node, data)) {
    elem = (struct se_calendar *) data;
    assert(elem != 0);

    g2mplsTypes::seCalendarEvent event;

    event.unixTime = ntohl(elem->time);
    event.storageCount.freeOnlineSize = ntohl(elem->freeOnlineSize);
    event.storageCount.logicalCpus = ntohl(elem->freeNearlineSize);        //wrong field name ??
    seq[i] = event;

    i++;
  }
  seInfo.seCalendar = seq;
  return;
}

void corba_update_gn_GridStorage_Name(const char* name)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridStorage NAME");
  seInfo.name = name;
  return;
}

void init_grid_GridStorage()
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update GridStorage");

  memset (&StorageAreas, 0, sizeof (struct zlist));

  const char* str = "";
  seInfo.name = str;;
  seInfo.storageInfo.arch = g2mplsTypes::GRIDSTORAGEARCH_UNKNOWN;
  seInfo.storageInfo.state = g2mplsTypes::GRIDCESESTATE_UNKNOWN;
  seInfo.storageInfo.accessProtocolsMask = 0;
  seInfo.storageInfo.controlProtocolsMask = 0;
  seInfo.storageInfo.valid = false;
  seInfo.onlineSize.total = 0;
  seInfo.onlineSize.used = 0;
  seInfo.onlineSize.valid = false;
  seInfo.nearlineSize.total = 0;
  seInfo.nearlineSize.used = 0;
  seInfo.nearlineSize.valid = false;

  g2mplsTypes::StorageAreaSeq_var seq;
  {
    g2mplsTypes::StorageAreaSeq * tmp;
    tmp = new g2mplsTypes::StorageAreaSeq(0);
    if (!tmp) {
      zlog_debug("[ERR] CORBA: init_grid_GridStorage: tmp == NULL");
      return;
    }
    seq = tmp;
  }
  seq->length(0);
  seInfo.storageAreas = seq;

  g2mplsTypes::seCalendarSeq_var seq2;
  {
    g2mplsTypes::seCalendarSeq * tmp;
    tmp = new g2mplsTypes::seCalendarSeq(0);
    if (!tmp) {
      zlog_debug("[ERR] CORBA: init_grid_GridStorage: tmp == NULL");
      return;
    }
    seq2 = tmp;
  }
  seq2->length(0);
  seInfo.seCalendar = seq2;

  return;
}

void corba_update_gn_GridStorage(int server)
{
  g2mplsTypes::gridSEParams* serverSeInfo;
  g2mplsTypes::gridSiteParams* serverSiteInfo;
  g2mplsTypes::gridSubNodes* snodes;
  g2mplsTypes::gridSubNodeIdent sNodeIdent;

  g2mplsTypes::nodeIdent ident;
  ident.id = nodeId;
  ident.type = g2mplsTypes::NODETYPE_GRID;

  sNodeIdent.id = subNodeId;
  sNodeIdent.type = g2mplsTypes::GRIDSUBNODETYPE_STORAGEELEMENT;

  try {
    switch(server)
    {
      case UPDATE_G2PCERA:

        if (!isNodeInSeq(ident, g2pcera_proxy->nodeGetAll()))
          node_add(UPDATE_G2PCERA, nodeId, NTYPE_GRID);

        g2pcera_proxy->gridSiteGet(nodeId, serverSiteInfo, snodes);
        if (isSubNodeInSeq(sNodeIdent, &snodes->storageElems))
        {
          g2pcera_proxy->gridStorageElemGet(nodeId, subNodeId, serverSeInfo);
          if (!equalGridStorages(&seInfo, serverSeInfo))
            g2pcera_proxy->gridStorageElemUpdate(nodeId, subNodeId, seInfo);
          else
          {
            if (IS_DEBUG_GRID_NODE(CORBA_ALL))
              OSPF_CORBA_DEBUG(OCD_TYPE_GRID, OCD_ACTION_ABORTED, OCD_SERV_G2PCERA, "GRID_STORAGE_ELEM_UPDATE",
                               emptyTnaIdent, emptyLinkIdent);
            return;
          }
        }
        else
          g2pcera_proxy->gridStorageElemUpdate(nodeId, subNodeId, seInfo);

        if (IS_DEBUG_GRID_NODE(CORBA))
          OSPF_CORBA_DEBUG(OCD_TYPE_GRID, OCD_ACTION_SENT, OCD_SERV_G2PCERA, "GRID_STORAGE_ELEM_UPDATE",
                           emptyTnaIdent, emptyLinkIdent);
        break;
      case UPDATE_GUNIGW:

        if (!isNodeInSeq(ident, gunigw_proxy->nodeGetAll()))
          node_add(UPDATE_GUNIGW, nodeId, NTYPE_GRID);

        gunigw_proxy->gridSiteGet(nodeId, serverSiteInfo, snodes);
        if (isSubNodeInSeq(sNodeIdent, &snodes->storageElems))
        {
          gunigw_proxy->gridStorageElemGet(nodeId, subNodeId, serverSeInfo);
          if (!equalGridStorages(&seInfo, serverSeInfo))
            gunigw_proxy->gridStorageElemUpdate(nodeId, subNodeId, seInfo);
          else
          {
            if (IS_DEBUG_GRID_NODE(CORBA_ALL))
              OSPF_CORBA_DEBUG(OCD_TYPE_GRID, OCD_ACTION_ABORTED, OCD_SERV_GUNIGW, "GRID_STORAGE_ELEM_UPDATE",
                               emptyTnaIdent, emptyLinkIdent);
            return;
          }
        }
        else
          gunigw_proxy->gridStorageElemUpdate(nodeId, subNodeId, seInfo);

        if (IS_DEBUG_GRID_NODE(CORBA))
          OSPF_CORBA_DEBUG(OCD_TYPE_GRID, OCD_ACTION_SENT, OCD_SERV_GUNIGW, "GRID_STORAGE_ELEM_UPDATE",
                           emptyTnaIdent, emptyLinkIdent);
        break;
    }
  } catch (g2mplsTopology::CannotFetchNode) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchNode (method gridStorageElemUpdate)");
    return;
  }  catch (g2mplsTopology::CannotFetchSubNode) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchSubNode (method gridStorageElemUpdate)");
    return;
  } catch (g2mplsTopology::SubNodeParamsMismatch) {

    zlog_debug("[ERR] CORBA: Exception NodeParamsMismatch (method gridStorageElemUpdate)");
    return;
  } catch (g2mplsTopology::InvocationNotAllowed) {

    zlog_debug("[ERR] CORBA: Exception InvocationNotAllowed (method gridStorageElemUpdate)");
    return;
  } catch (g2mplsTopology::InternalProblems & e) {

    zlog_debug("[ERR] CORBA: Exception InternalProblems (method gridStorageElemUpdate): %s", (char *) e.what);
  } catch (...) {

    zlog_debug("[ERR] CORBA: Exception Unknown (method gridStorageElemUpdate)");
    return;
  }
  return;
}

} //extern "C"

extern "C" {

void corba_update_te_ra_router_addr(struct in_addr id)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update ROUTER ADDRESS");

  nodeId = ntohl(id.s_addr);
  return;
}

// CORBA Update TE TNA related

struct in_addr  rc;       // Routing Controller ID used in tna_id_update
struct in_addr  node;     // Node Id used in tna_id_update

typedef struct tna_entry
{
  struct in_addr  node;     // Node Id used in tna_id_update
  g2mpls_addr_t   tna_addr; // TNA address used in tna_id_update
} tna_entry;

struct zlist tnaIds;

void init_tna_ident()
{
  rc.s_addr = 0;
  node.s_addr = 0;
  memset (&tnaIds, 0, sizeof (struct zlist));
  return;
}

// from TNA Address IPv4 | TNA Address IPv6 | TNA Address NSAP TNA Address Sub-TLV
void corba_update_te_tna_addr(g2mpls_addr_t tna)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update TNA ADDRESS");

  tna_entry* entry = (tna_entry *) XMALLOC(0, sizeof(tna_entry));
  entry->node = node;
  entry->tna_addr = tna;

  listnode_add(&tnaIds, entry);
  return;
}

// from Node ID TNA Address Sub-TLV
void corba_update_te_tna_node(struct in_addr node_id)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update TNA NODE");

  node = node_id;
  return;
}

// from Ancestor RC Id TNA Address Sub-TLV
void corba_update_te_tna_anc_rc_id(struct in_addr value)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update TNA ANC_RC_ID");

  rc = value;
  return;
}

void tna_ids_update(uint8_t option)
{
  g2mplsTypes::tnaIdent ident;

  bool isDomain = false;
  void *data; 
  struct zlistnode *node, *nnode;

  tna_entry * item;
  for (ALL_LIST_ELEMENTS (&tnaIds, node, nnode, data)) {

    item = (tna_entry *) data;

    ident.rc = ntohl(rc.s_addr);
    ident.node = ntohl(item->node.s_addr);

    g2mplsTypes::tnaId_var tnaAddr;
    tnaAddr << item->tna_addr;
    ident.tna = tnaAddr;
    ident.prefix = item->tna_addr.preflen;

    switch(option)
    {
      case ADD_TNA_ADDRESS:
        try {

          nodeIdent.type = g2mplsTypes::NODETYPE_NETWORK;

          if (!ident.rc) {
            isDomain = false;
            nodeIdent.id = ident.node;
          }
          else {
            isDomain = true;
            nodeIdent.id = ident.rc;
          }

          if (!isNodeInSeq(nodeIdent, g2pcera_proxy->nodeGetAll()))
            node_add(UPDATE_G2PCERA, nodeIdent.id, NTYPE_NETWORK);
          update_net_node(nodeIdent.id, isDomain);

          if (!isTnaIdinSeq(ident, g2pcera_proxy->tnaIdsGetAllFromNode(nodeIdent.id, isDomain)))
          {
            g2pcera_proxy->tnaIdAdd(ident);
            if (IS_DEBUG_GRID_NODE(CORBA))
              OSPF_CORBA_DEBUG(OCD_TYPE_TNA, OCD_ACTION_SENT, OCD_SERV_G2PCERA, "TNA_ID_ADD",
                               ident, emptyLinkIdent);
          }
          else
          {
            if (IS_DEBUG_GRID_NODE(CORBA_ALL))
              OSPF_CORBA_DEBUG(OCD_TYPE_TNA, OCD_ACTION_ABORTED, OCD_SERV_G2PCERA, "TNA_ID_ADD",
                               ident, emptyLinkIdent);
          }
        } catch (g2mplsTopology::CannotFetchNode & e) {

          zlog_debug("[ERR] CORBA: Exception CannotFetchNode (method tnaIdAdd)");
          zlog_debug("[ERR]        %s", (char *) e.what);
          return;
        } catch (g2mplsTopology::TnaAlreadyExists & e) {

          zlog_debug("[ERR] CORBA: Exception TnaAlreadyExists (method tnaIdAdd)");
          zlog_debug("[ERR]        %s", (char *) e.what);
          return;
        } catch (g2mplsTopology::InvocationNotAllowed & e) {

          zlog_debug("[ERR] CORBA: Exception InvocationNotAllowed (method tnaIdAdd)");
          return;
        } catch (g2mplsTopology::InternalProblems & e) {

          zlog_debug("[ERR] CORBA: Exception InternalProblems (method tnaIdAdd");
          zlog_debug("[ERR]        %s", (char *) e.what);
          return;
        } catch (...) {

          zlog_debug("[ERR] CORBA: Exception Unknown (method tnaIdAdd)");
          return;
        }
        break;
      case REMOVE_TNA_ADDRESS:
        try {

          nodeIdent.type = g2mplsTypes::NODETYPE_NETWORK;

          if (!ident.rc) {
            isDomain = false;
            nodeIdent.id = ident.node;
          }
          else {
            isDomain = true;
            nodeIdent.id = ident.rc;
          }

          if (isNodeInSeq(nodeIdent, g2pcera_proxy->nodeGetAll()))
            if (isTnaIdinSeq(ident, g2pcera_proxy->tnaIdsGetAllFromNode(nodeIdent.id, isDomain)))
          {
            g2pcera_proxy->tnaIdDel(ident);
            if (IS_DEBUG_GRID_NODE(CORBA))
              OSPF_CORBA_DEBUG(OCD_TYPE_TNA, OCD_ACTION_SENT, OCD_SERV_G2PCERA, "TNA_ID_DEL",
                               ident, emptyLinkIdent);
            }
            else
            {
              if (IS_DEBUG_GRID_NODE(CORBA_ALL))
                OSPF_CORBA_DEBUG(OCD_TYPE_TNA, OCD_ACTION_ABORTED, OCD_SERV_G2PCERA, "TNA_ID_DEL",
                                 ident, emptyLinkIdent);
            }
          break;
        } catch (g2mplsTopology::CannotFetchNode & e) {

          zlog_debug("[ERR] CORBA: Exception CannotFetchNode (method tnaIdDel)");
          zlog_debug("[ERR]        %s", (char *) e.what);
          return;
        } catch (g2mplsTopology::CannotFetchTna & e) {

          zlog_debug("[ERR] CORBA: Exception CannotFetchTna (method tnaIdDel)");
          zlog_debug("[ERR]        %s", (char *) e.what);
          return;
        } catch (g2mplsTopology::InvocationNotAllowed & e) {

          zlog_debug("[ERR] CORBA: Exception InvocationNotAllowed (method tnaIdDel)");
          return;
        } catch (g2mplsTopology::InternalProblems & e) {

          zlog_debug("[ERR] CORBA: Exception InternalProblems (method tnaIdDel)");
          zlog_debug("[ERR]        %s", (char *) e.what);
          return;
        } catch (...) {

          zlog_debug("[ERR] CORBA: Exception Unknown (method tnaIdDel)");
          return;
        }
        break;
    }
  }

  return;
}

}

extern "C" {

g2mplsTypes::teLinkIdent linkIdent;

struct in_addr localNodeId;     // local node id (interdomain telink)
struct in_addr remoteNodeId;    // remote node id (interdomain telink)
uint32_t       localId;         // local id (interdomain telink)
uint32_t       remoteId;        // remote id (interdomain telink)
struct in_addr ancestorRcId;    // local rc id (interdomain telink)
struct in_addr linkId;          // remote rc id (interdomain telink) & remote node id (intradomain telink)
struct in_addr lclIfAddr;       // local id (intradomain telink)
struct in_addr rmtIfAddr;       // remote id (intradomain telink)
struct in_addr advertisingRId;  // local node id (intradomain telink)
uint8_t        ltype;           // converted to link mode (interdomain & intradomain telink)

void init_link_tmp_values()
{
  localNodeId.s_addr = 0;
  remoteNodeId.s_addr = 0;
  localId = 0;
  remoteId = 0;
  ancestorRcId.s_addr = 0;
  linkId.s_addr = 0;
  lclIfAddr.s_addr = 0;
  rmtIfAddr.s_addr = 0;
  ltype = 0;
}

void init_link_ident()
{
  linkIdent.localNodeId = 0;
  linkIdent.remoteNodeId = 0;

  g2mpls_addr_t addr;
  addr.type = IPv4;
  addr.value.ipv4.s_addr = 0;
  g2mplsTypes::TELinkId_var initId;
  initId << addr;

  linkIdent.localId = initId;
  linkIdent.remoteId = initId;
  linkIdent.localRcId = 0;
  linkIdent.remoteRcId = 0;
  linkIdent.mode = g2mplsTypes::LINKMODE_UNKNOWN;
}

g2mplsTypes::linkMode link_type_2_link_mode(telink_type_t teltype);

void set_link_ident(telink_type_t type)
{
  init_link_ident();

  g2mplsTypes::linkId_var clinkId;
  g2mpls_addr_t addr;
  switch (type)
  {
    case INTERDOM_TEL:
      linkIdent.localNodeId = ntohl(localNodeId.s_addr);
      linkIdent.remoteNodeId = ntohl(remoteNodeId.s_addr);

      addr.type = IPv4;
      addr.value.ipv4.s_addr = localId;
      clinkId << addr;

      linkIdent.localId  = (g2mplsTypes::TELinkId) clinkId;

      addr.value.ipv4.s_addr = remoteId;
      clinkId << addr;

      linkIdent.remoteId = (g2mplsTypes::TELinkId) clinkId;

      linkIdent.localRcId = ntohl(ancestorRcId.s_addr);
      linkIdent.remoteRcId = ntohl(linkId.s_addr);

      linkIdent.mode = link_type_2_link_mode(INTERDOM_TEL);
      break;

    case INTRADOM_TEL:
      linkIdent.localNodeId = ntohl(advertisingRId.s_addr);
      linkIdent.remoteNodeId = ntohl(linkId.s_addr);

      addr.type = IPv4;
      addr.value.ipv4 = lclIfAddr;
      clinkId << addr;

      linkIdent.localId = (g2mplsTypes::TELinkId) clinkId;

      addr.value.ipv4 = rmtIfAddr;
      clinkId << addr;

      linkIdent.remoteId = (g2mplsTypes::TELinkId) clinkId;

      linkIdent.localRcId = 0;
      linkIdent.remoteRcId = 0;

      linkIdent.mode = link_type_2_link_mode(INTRADOM_TEL);
      break;
  }

  return;
}

// from Local Node ID Link Sub-TLVs
void corba_update_te_link_lcl_node_id(struct in_addr localNId)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update TELink LCL_NODE_ID");

  localNodeId = localNId;
  return;
}

// from Remote Node ID Link Sub-TLV
void corba_update_te_link_rmt_node_id(struct in_addr remoteNId)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update TELink RMT_NODE_ID");

  remoteNodeId = remoteNId;
  return;
}

// from Link Local/Remote Identifiers Link Sub-TLV
void corba_update_te_link_lcl_rmt_ids(uint32_t lclId, uint32_t rmtId)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update TELink LCL_RMT_IDS");

  localId = lclId;
  remoteId = rmtId;
  return;
}

// from Link Id Link Sub-TLV
void corba_update_te_link_id(struct in_addr telinkId)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update TELink LINK_ID");

  linkId = telinkId;
  return;
}

// from Ancestor RC Id Link Sub-TLV
void corba_update_te_link_anc_rc_id(struct in_addr value)
{
  if(IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update TELink ANC_RC_ID");

  ancestorRcId = value;
  return;
}

// from Local Interface IP Address Link Sub-TLV
void corba_update_te_link_lclif_ipaddr(struct in_addr value)
{
  if(IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update TELink LCLIF_IPADDR");

  lclIfAddr = value;
  return;
}

// from Remote Interface IP Address Link Sub-TLV
void corba_update_te_link_rmtif_ipaddr(struct in_addr value)
{
  if(IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update TELink RMTIF_IPADDR");

  rmtIfAddr = value;
  return;
}

void corba_update_advertising_router(struct in_addr value)
{
  if(IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update TELink ADV_ROUTER");

  advertisingRId = value;
  return;
}

// from Link Type Link Sub-TLV
void corba_update_te_link_type(uint8_t type)
{
  if (IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update TELink TYPE");

  ltype = type;
  return;
}

g2mplsTypes::linkMode link_type_2_link_mode(telink_type_t teltype)
{
  g2mplsTypes::linkMode mode;
  switch (ltype)
  {
    case LINK_TYPE_SUBTLV_VALUE_PTP:
      if (teltype == INTRADOM_TEL) 
        mode = g2mplsTypes::LINKMODE_P2P_NUMBERED;
      else
      {
        if ((linkIdent.localRcId != 0) && (linkIdent.localRcId == linkIdent.remoteRcId))
          mode = g2mplsTypes::LINKMODE_ENNI_INTRADOMAIN;
        else
          mode = g2mplsTypes::LINKMODE_ENNI_INTERDOMAIN;
      }
      break;
    case LINK_TYPE_SUBTLV_VALUE_MA:
      mode = g2mplsTypes::LINKMODE_MULTIACCESS;
      break;
    default:
      mode = g2mplsTypes::LINKMODE_UNKNOWN;
      break;
  }

  return mode;
}

uint8_t link_update(uint8_t option, telink_type_t type)
{
  set_link_ident(type);

  g2mplsTypes::nodeIdent ident;
  ident.type = g2mplsTypes::NODETYPE_NETWORK;

  switch(option)
  {
    case ADD_TE_LINK:
      try {

        if ((!linkIdent.localRcId && !linkIdent.remoteRcId)
          || (!linkIdent.localRcId && linkIdent.remoteRcId))
        {
          ident.id = linkIdent.localNodeId;
          if (!isNodeInSeq(ident, g2pcera_proxy->nodeGetAll()))
          {
            node_add(UPDATE_G2PCERA, ident.id, NTYPE_NETWORK);
            update_net_node(ident.id, false);
          }
        }

        if ((linkIdent.localRcId && linkIdent.remoteRcId)
          || (linkIdent.localRcId && !linkIdent.remoteRcId))
        {
          ident.id = linkIdent.localRcId;
          if (!isNodeInSeq(ident, g2pcera_proxy->nodeGetAll()))
          {
            node_add(UPDATE_G2PCERA, ident.id, NTYPE_NETWORK);
            update_net_node(ident.id, true);
          }
        }

        if (!isTELinkInSeq(linkIdent, g2pcera_proxy->teLinkGetAllFromNode(ident)))
        {
          g2pcera_proxy->linkAdd(linkIdent);
          if (IS_DEBUG_GRID_NODE(CORBA))
            OSPF_CORBA_DEBUG(OCD_TYPE_TELINK, OCD_ACTION_SENT, OCD_SERV_G2PCERA, "LINK_ADD",
                             emptyTnaIdent, linkIdent);
        }
        else
        {
          if (IS_DEBUG_GRID_NODE(CORBA_ALL))
            OSPF_CORBA_DEBUG(OCD_TYPE_TELINK, OCD_ACTION_ABORTED, OCD_SERV_G2PCERA, "LINK_ADD",
                             emptyTnaIdent, linkIdent);
        }
      } catch (g2mplsTopology::CannotFetchNode & e) {

        zlog_debug("[ERR] CORBA: Exception CannotFetchNode (method linkAdd)");
        zlog_debug("[ERR]        %s", (char *) e.what);
        return 0;
      } catch (g2mplsTopology::LinkAlreadyExists & e) {

        zlog_debug("[ERR] CORBA: Exception LinkAlreadyExists (method linkAdd)");
        zlog_debug("[ERR]        %s", (char *) e.what);
        return 0;
      } catch (g2mplsTopology::InvocationNotAllowed) {

        zlog_debug("[ERR] CORBA: Exception InvocationNotAllowed (method linkAdd)");
        return 0;
      } catch (g2mplsTopology::InternalProblems & e) {

        zlog_debug("[ERR] CORBA: Exception InternalProblems (method linkAdd)");
        zlog_debug("[ERR]        %s", (char *) e.what);
        return 0;
      } catch (...) {

        zlog_debug("[ERR] CORBA: Exception Unknown (method linkAdd)");
        return 0;
      }
      break;
    case REMOVE_TE_LINK:
      try {

        if ((!linkIdent.localRcId && !linkIdent.remoteRcId)
          || (!linkIdent.localRcId && linkIdent.remoteRcId))
          ident.id = linkIdent.localNodeId;

        if ((linkIdent.localRcId && linkIdent.remoteRcId)
          || (linkIdent.localRcId && !linkIdent.remoteRcId))
          ident.id = linkIdent.localRcId;

        if (isNodeInSeq(ident, g2pcera_proxy->nodeGetAll()))
          if (isTELinkInSeq(linkIdent, g2pcera_proxy->teLinkGetAllFromNode(ident)))
          {
            g2pcera_proxy->linkDel(linkIdent);
            if (IS_DEBUG_GRID_NODE(CORBA))
              OSPF_CORBA_DEBUG(OCD_TYPE_TELINK, OCD_ACTION_SENT, OCD_SERV_G2PCERA, "LINK_DEL",
                               emptyTnaIdent, linkIdent);
          }
        else
        {
          if (IS_DEBUG_GRID_NODE(CORBA_ALL))
            OSPF_CORBA_DEBUG(OCD_TYPE_TELINK, OCD_ACTION_ABORTED, OCD_SERV_G2PCERA, "LINK_DEL",
                             emptyTnaIdent, linkIdent);
        }
      } catch (g2mplsTopology::CannotFetchNode & e) {

        zlog_debug("[ERR] CORBA: Exception CannotFetchNode (method linkDel)");
        zlog_debug("[ERR]        %s", (char *) e.what);
        return 0;
      } catch (g2mplsTopology::LinkAlreadyExists & e) {

        zlog_debug("[ERR] CORBA: Exception LinkAlreadyExists (method linkDel)");
        zlog_debug("[ERR]        %s", (char *) e.what);
        return 0;
      } catch (g2mplsTopology::CannotFetchLink & e) {

        zlog_debug("[ERR] CORBA: Exception CannotFetchLink (method linkDel)");
        zlog_debug("[ERR]        %s", (char *) e.what);
        return 0;
      } catch (g2mplsTopology::InvocationNotAllowed) {

        zlog_debug("[ERR] CORBA: Exception InvocationNotAllowed (method linkDel)");
        return 0;
      } catch (g2mplsTopology::InternalProblems & e) {

        zlog_debug("[ERR] CORBA: Exception InternalProblems (method linkDel)");
        zlog_debug("[ERR]        %s", (char *) e.what);
        return 0;
      } catch (...) {

        zlog_debug("[ERR] CORBA: Exception Unknown (method linkDel)");
        return 0;
      }
      break;
  }

  return 1;
}

g2mplsTypes::teLinkComParams linkComParams;

void corba_update_te_link_metric(uint32_t teMetric)
{
  if(IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update TELink METRIC");
  linkComParams.teMetric = teMetric;
  return;
}

void corba_update_te_link_max_bw(float teMaxBw)
{
  if(IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update TELink MAX_BW");
  uint32_t tmp = 0;
  memcpy(&tmp, &teMaxBw, 4);
  linkComParams.teMaxBw = tmp;
  return;
}

void corba_update_te_link_max_res_bw(float teMaxResvBw)
{
  if(IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update TELink MAX_RES_BW");
  uint32_t tmp = 0;
  memcpy(&tmp, &teMaxResvBw, 4);
  linkComParams.teMaxResvBw = tmp;
  return;
}

g2mplsTypes::availBwPerPrio availBw;

void corba_update_te_link_unrsv_bw(float avBand[])
{
  if(IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update TELink UNRSV_BW");
  uint32_t tmp;
  for (int i=0; i< 8; i++)
  {
    memcpy(&tmp, &avBand[i], 4);
    availBw[i] = tmp;
  }
  return;
}

void corba_update_te_link_rsc_clsclr(uint32_t teColorMask)
{
  if(IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update TELink RSC_CLSCLR");
  linkComParams.teColorMask = teColorMask;
  return;
}

void corba_update_te_link_protect_type(uint8_t teProtectionTypeMask)
{
  if(IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update TELink PROTECT_TYPE");
  linkComParams.teProtectionTypeMask = teProtectionTypeMask;
  return;
}

void link_update_com()
{
  try {
    g2mplsTypes::teLinkComParams serverLinkComParams;

    linkComParams.adminMetric = 0;      //FIXME Lukasz: where is this information?

    g2pcera_proxy->teLinkGetCom(linkIdent, serverLinkComParams);
    if (!equalComParams(&linkComParams, &serverLinkComParams))
    {
      g2pcera_proxy->teLinkUpdateCom(linkIdent, linkComParams);

      if(IS_DEBUG_GRID_NODE(CORBA))
        OSPF_CORBA_DEBUG(OCD_TYPE_TELINK, OCD_ACTION_SENT, OCD_SERV_G2PCERA, "TE_LINK_UPDATE_COM",
                         emptyTnaIdent, linkIdent);
    }
    else
    {
      if (IS_DEBUG_GRID_NODE(CORBA_ALL))
        OSPF_CORBA_DEBUG(OCD_TYPE_TELINK, OCD_ACTION_ABORTED, OCD_SERV_G2PCERA, "TE_LINK_UPDATE_COM",
                         emptyTnaIdent, linkIdent);
    }
  } catch (g2mplsTopology::CannotFetchNode) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchNode (method teLinkUpdateCom)");
    return;
  } catch (g2mplsTopology::CannotFetchLink) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchLink (method teLinkUpdateCom)");
    return;
  } catch (g2mplsTopology::LinkParamsMismatch) {

    zlog_debug("[ERR] CORBA: Exception LinkParamsMismatch (method teLinkUpdateCom)");
    return;
  } catch (g2mplsTopology::InvocationNotAllowed) {

    zlog_debug("[ERR] CORBA: Exception InvocationNotAllowed (method teLinkUpdateCom)");
    return;
  } catch (g2mplsTopology::InternalProblems & e) {

    zlog_debug("[ERR] CORBA: Exception InternalProblems (method teLinkUpdateCom): %s", (char *) e.what);
  } catch (...) {

    zlog_debug("[ERR] CORBA: Exception Unknown (method teLinkUpdateCom)");
    return;
  }

  return;
}

void link_update_tdm()
{
  try {
    g2mplsTypes::teLinkTdmParams serverLinkTdmParams;

    g2mplsTypes::teLinkTdmParams linkTdmParams;
    linkTdmParams.hoMuxCapMask     = 0;         //FIXME Lukasz: where is this information?
    linkTdmParams.loMuxCapMask     = 0;         //FIXME Lukasz: where is this information?
    linkTdmParams.transparencyMask = 0;         //FIXME Lukasz: where is this information?
    linkTdmParams.blsrRingId       = 0;         //FIXME Lukasz: where is this information?

    g2pcera_proxy->teLinkGetTdm(linkIdent, serverLinkTdmParams);
    if (!equalTdmParams(&linkTdmParams, &serverLinkTdmParams))
    {
      g2pcera_proxy->teLinkUpdateTdm(linkIdent, linkTdmParams);

      if(IS_DEBUG_GRID_NODE(CORBA))
        OSPF_CORBA_DEBUG(OCD_TYPE_TELINK, OCD_ACTION_SENT, OCD_SERV_G2PCERA, "TE_LINK_UPDATE_TDM",
                         emptyTnaIdent, linkIdent);
    }
    else
    {
      if (IS_DEBUG_GRID_NODE(CORBA_ALL))
        OSPF_CORBA_DEBUG(OCD_TYPE_TELINK, OCD_ACTION_ABORTED, OCD_SERV_G2PCERA, "TE_LINK_UPDATE_TDM",
                         emptyTnaIdent, linkIdent);
    }
  } catch (g2mplsTopology::CannotFetchNode) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchNode (method teLinkUpdateTdm)");
    return;
  } catch (g2mplsTopology::CannotFetchLink) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchLink (method teLinkUpdateTdm)");
    return;
  } catch (g2mplsTopology::LinkParamsMismatch) {

    zlog_debug("[ERR] CORBA: Exception LinkParamsMismatch (method teLinkUpdateTdm)");
    return;
  } catch (g2mplsTopology::InvocationNotAllowed) {

    zlog_debug("[ERR] CORBA: Exception InvocationNotAllowed (method teLinkUpdateTdm)");
    return;
  } catch (g2mplsTopology::InternalProblems & e) {

    zlog_debug("[ERR] CORBA: Exception InternalProblems (method teLinkUpdateTdm): %s", (char *) e.what);
  } catch (...) {

    zlog_debug("[ERR] CORBA: Exception Unknown (method teLinkUpdateTdm)");
    return;
  }

  return;
}

void link_update_lscG709()
{
  try {
    g2mplsTypes::teLinkLscG709Params serverLinkLscG709Params;

    g2mplsTypes::teLinkLscG709Params linkLscG709Params;
    linkLscG709Params.odukMuxCapMask = 0;      //FIXME Lukasz: where is this information?

    g2pcera_proxy->teLinkGetLscG709(linkIdent, serverLinkLscG709Params);
    if (!equalLscG709Params(&linkLscG709Params, &serverLinkLscG709Params))
    {
      g2pcera_proxy->teLinkUpdateLscG709(linkIdent, linkLscG709Params);

      if(IS_DEBUG_GRID_NODE(CORBA))
        OSPF_CORBA_DEBUG(OCD_TYPE_TELINK, OCD_ACTION_SENT, OCD_SERV_G2PCERA, "TE_LINK_UPDATE_LSCG709",
                         emptyTnaIdent, linkIdent);
    }
    else
    {
      if (IS_DEBUG_GRID_NODE(CORBA_ALL))
        OSPF_CORBA_DEBUG(OCD_TYPE_TELINK, OCD_ACTION_ABORTED, OCD_SERV_G2PCERA, "TE_LINK_UPDATE_LSCG709",
                         emptyTnaIdent, linkIdent);
    }
  } catch (g2mplsTopology::CannotFetchNode) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchNode (method teLinkUpdateLscG709)");
    return;
  } catch (g2mplsTopology::CannotFetchLink) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchLink (method teLinkUpdateLscG709)");
    return;
  } catch (g2mplsTopology::LinkParamsMismatch) {

    zlog_debug("[ERR] CORBA: Exception LinkParamsMismatch (method teLinkUpdateLscG709)");
    return;
  } catch (g2mplsTopology::InvocationNotAllowed) {

    zlog_debug("[ERR] CORBA: Exception InvocationNotAllowed (method teLinkUpdateLscG709)");
    return;
  } catch (g2mplsTopology::InternalProblems & e) {

    zlog_debug("[ERR] CORBA: Exception InternalProblems (method teLinkUpdateLscG709): %s", (char *) e.what);
  } catch (...) {

    zlog_debug("[ERR] CORBA: Exception Unknown (method teLinkUpdateLscG709)");
    return;
  }

  return;
}

g2mplsTypes::teLinkLscWdmParams linkLscWdmParams;

// from Dpdm Link Sub-TLV
void corba_update_te_link_d_pdm(uint32_t dispersionPMD)
{
  if(IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update TELink DPDM");
  linkLscWdmParams.dispersionPMD = dispersionPMD;
  return;
}

// from Span Length Link Sub-TLV
void corba_update_te_link_span_length(uint32_t spanLength)
{
  if(IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update TELink SPAN_LENGTH");
  linkLscWdmParams.spanLength = spanLength;
  return;
}

// from Amplifiers List Link Sub-TLV
void corba_update_te_link_amp_list(struct amp_par *amplist, uint16_t len)
{
  if(IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update TELink AMP_LIST");

  struct amp_par * elem;
  g2mplsTypes::amplifiersSeq_var seq;

  g2mplsTypes::amplifiersSeq * tmp;
  tmp = new g2mplsTypes::amplifiersSeq(len);
  if (!tmp) {
    zlog_debug("[ERR] CORBA: corba_update_te_link_amp_list: tmp == NULL");
    return;
  }
  seq = tmp;
  seq->length(len);

  float tmpNoise;
  u_int32_t lu1, lu2;
  for (uint16_t i = 0; i< len; i++)
  {
    g2mplsTypes::teLinkWdmAmplifierEntry entry;
    entry.gain = ntohl(amplist[i].gain);
    memcpy (&lu1, &amplist[i].noise, 4);
    lu2 = ntohl (lu1);
    memcpy (&tmpNoise, &lu2, 4);
    entry.noiseFigure = (int)tmpNoise;

    seq[i] = entry;
  }
  linkLscWdmParams.amplifiers = seq;
  return;
}

void link_update_lscwdm()
{
  try {
    g2mplsTypes::teLinkLscWdmParams_var serverLinkLscWdmParams;

    g2pcera_proxy->teLinkGetLscWdm(linkIdent, serverLinkLscWdmParams);
    if (!equalLscWdmParams(linkLscWdmParams, serverLinkLscWdmParams))
    {
      g2pcera_proxy->teLinkUpdateLscWdm(linkIdent, linkLscWdmParams);

      if(IS_DEBUG_GRID_NODE(CORBA))
        OSPF_CORBA_DEBUG(OCD_TYPE_TELINK, OCD_ACTION_SENT, OCD_SERV_G2PCERA, "TE_LINK_UPDATE_LSCWDM",
                         emptyTnaIdent, linkIdent);
    }
    else
    {
      if (IS_DEBUG_GRID_NODE(CORBA_ALL))
        OSPF_CORBA_DEBUG(OCD_TYPE_TELINK, OCD_ACTION_ABORTED, OCD_SERV_G2PCERA, "TE_LINK_UPDATE_LSCWDM",
                         emptyTnaIdent, linkIdent);
    }
  } catch (g2mplsTopology::CannotFetchNode) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchNode (method teLinkUpdateLscWdm)");
    return;
  } catch (g2mplsTopology::CannotFetchLink) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchLink (method teLinkUpdateLscWdm)");
    return;
  } catch (g2mplsTopology::LinkParamsMismatch) {

    zlog_debug("[ERR] CORBA: Exception LinkParamsMismatch (method teLinkUpdateLscWdm)");
    return;
  } catch (g2mplsTopology::InvocationNotAllowed) {

    zlog_debug("[ERR] CORBA: Exception InvocationNotAllowed (method teLinkUpdateLscWdm)");
    return;
  } catch (g2mplsTopology::InternalProblems & e) {

    zlog_debug("[ERR] CORBA: Exception InternalProblems (method teLinkUpdateLscWdm): %s", (char *) e.what);
  } catch (...) {

    zlog_debug("[ERR] CORBA: Exception Unknown (method teLinkUpdateLscWdm)");
    return;
  }

  return;
}

void link_update_states()
{
  try {
    g2mplsTypes::statesBundle serverStates;

    g2mplsTypes::statesBundle states;
    states.opState = g2mplsTypes::OPERSTATE_UP;
    states.admState = g2mplsTypes::ADMINSTATE_ENABLED;

    g2pcera_proxy->teLinkGetStates(linkIdent, serverStates);
    if (!equalLinkStates(&states, &serverStates))
    {
      g2pcera_proxy->teLinkUpdateStates(linkIdent, states);

      if(IS_DEBUG_GRID_NODE(CORBA))
        OSPF_CORBA_DEBUG(OCD_TYPE_TELINK, OCD_ACTION_SENT, OCD_SERV_G2PCERA, "TE_LINK_UPDATE_STATES",
                         emptyTnaIdent, linkIdent);
    }
    else
    {
      if (IS_DEBUG_GRID_NODE(CORBA_ALL))
        OSPF_CORBA_DEBUG(OCD_TYPE_TELINK, OCD_ACTION_ABORTED, OCD_SERV_G2PCERA, "TE_LINK_UPDATE_STATES",
                         emptyTnaIdent, linkIdent);
    }
  } catch (g2mplsTopology::CannotFetchNode) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchNode (method teLinkUpdateStates)");
    return;
  } catch (g2mplsTopology::CannotFetchLink) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchLink (method teLinkUpdateStates)");
    return;
  } catch (g2mplsTopology::LinkParamsMismatch) {

    zlog_debug("[ERR] CORBA: Exception LinkParamsMismatch (method teLinkUpdateStates)");
    return;
  } catch (g2mplsTopology::InvocationNotAllowed) {

    zlog_debug("[ERR] CORBA: Exception InvocationNotAllowed (method teLinkUpdateStates)");
    return;
  } catch (g2mplsTopology::InternalProblems & e) {

    zlog_debug("[ERR] CORBA: Exception InternalProblems (method teLinkUpdateStates): %s", (char *) e.what);
  } catch (...) {

    zlog_debug("[ERR] CORBA: Exception Unknown (method teLinkUpdateStates)");
    return;
  }

  return;
}

void link_update_genbw()
{
  try {
    g2mplsTypes::availBwPerPrio serverAvailBw;

    g2pcera_proxy->teLinkGetGenBw(linkIdent, serverAvailBw);
    if (!equalGenBw(availBw, serverAvailBw))
    {
      g2pcera_proxy->teLinkUpdateGenBw(linkIdent, availBw);

      if(IS_DEBUG_GRID_NODE(CORBA))
        OSPF_CORBA_DEBUG(OCD_TYPE_TELINK, OCD_ACTION_SENT, OCD_SERV_G2PCERA, "TE_LINK_UPDATE_GEN_BW",
                         emptyTnaIdent, linkIdent);
    }
    else
    {
      if (IS_DEBUG_GRID_NODE(CORBA_ALL))
        OSPF_CORBA_DEBUG(OCD_TYPE_TELINK, OCD_ACTION_ABORTED, OCD_SERV_G2PCERA, "TE_LINK_UPDATE_GEN_BW",
                         emptyTnaIdent, linkIdent);
    }
  } catch (g2mplsTopology::CannotFetchNode) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchNode (method teLinkUpdateGenBw)");
    return;
  } catch (g2mplsTopology::CannotFetchLink) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchLink (method teLinkUpdateGenBw)");
    return;
  } catch (g2mplsTopology::LinkParamsMismatch) {

    zlog_debug("[ERR] CORBA: Exception LinkParamsMismatch (method teLinkUpdateGenBw)");
    return;
  } catch (g2mplsTopology::InvocationNotAllowed) {

    zlog_debug("[ERR] CORBA: Exception InvocationNotAllowed (method teLinkUpdateGenBw)");
    return;
  } catch (g2mplsTopology::InternalProblems & e) {

    zlog_debug("[ERR] CORBA: Exception InternalProblems (method teLinkUpdateGenBw): %s", (char *) e.what);
  } catch (...) {

    zlog_debug("[ERR] CORBA: Exception Unknown (method teLinkUpdateGenBw)");
    return;
  }

  return;
}

g2mplsTypes::freeCTPSeq freeCTP;

void corba_update_te_link_ssdh_if_sw_cap_desc(struct zlist* freeTS)
{
  if(IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update TELink SSDH_IF_SW_CAP_DESC");

  struct zlistnode *node;
  void *data;
  struct signal_unalloc_tslots *elem;
  int i = 0;

  g2mplsTypes::freeCTPSeq_var seq;
  g2mplsTypes::freeCTPSeq * tmp;
  tmp = new g2mplsTypes::freeCTPSeq(listcount(freeTS));
  if (!tmp) {
    zlog_debug("[ERR] CORBA: corba_update_te_link_ssdh_if_sw_cap_desc: tmp == NULL");
    return;
  }
  seq = tmp;
  seq->length(listcount(freeTS));

  uint32_t value = 0;
  for (ALL_LIST_ELEMENTS_RO(freeTS, node, data))
  {
    elem = (struct signal_unalloc_tslots *) data;
    assert(elem != 0);

    g2mplsTypes::freeCTPEntry entry;
    entry.sigType = (uint8_t) elem->signal_type;
    value  = 0;
    value  = elem->unalloc_tslots[0]; value <<= 8; value &= 0xff00;
    value |= elem->unalloc_tslots[1]; value <<= 8; value &= 0xffff00;
    value |= elem->unalloc_tslots[2]; value &= 0xffffff;
    entry.ctps = value;
    seq[i] = entry;

    i++;
  }
  freeCTP = seq;

  return;
}

void link_update_tdmbw()
{
  try {
    g2mplsTypes::freeCTPSeq_var serverFreeCTP;

    g2pcera_proxy->teLinkGetTdmBw(linkIdent, serverFreeCTP);
    if (!equalTdmBw(freeCTP, serverFreeCTP))
    {
      g2pcera_proxy->teLinkUpdateTdmBw(linkIdent, freeCTP);

      if(IS_DEBUG_GRID_NODE(CORBA))
        OSPF_CORBA_DEBUG(OCD_TYPE_TELINK, OCD_ACTION_SENT, OCD_SERV_G2PCERA, "TE_LINK_UPDATE_TDM_BW",
                         emptyTnaIdent, linkIdent);
    }
    else
    {
      if (IS_DEBUG_GRID_NODE(CORBA_ALL))
        OSPF_CORBA_DEBUG(OCD_TYPE_TELINK, OCD_ACTION_ABORTED, OCD_SERV_G2PCERA, "TE_LINK_UPDATE_TDM_BW",
                         emptyTnaIdent, linkIdent);
    }
  } catch (g2mplsTopology::CannotFetchNode) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchNode (method teLinkUpdateTdmBw)");
    return;
  } catch (g2mplsTopology::CannotFetchLink) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchLink (method teLinkUpdateTdmBw)");
    return;
  } catch (g2mplsTopology::LinkParamsMismatch) {

    zlog_debug("[ERR] CORBA: Exception LinkParamsMismatch (method teLinkUpdateTdmBw)");
    return;
  } catch (g2mplsTopology::InvocationNotAllowed) {

    zlog_debug("[ERR] CORBA: Exception InvocationNotAllowed (method teLinkUpdateTdmBw)");
    return;
  } catch (g2mplsTopology::InternalProblems & e) {

    zlog_debug("[ERR] CORBA: Exception InternalProblems (method teLinkUpdateTdmBw): %s", (char *) e.what);
  } catch (...) {

    zlog_debug("[ERR] CORBA: Exception Unknown (method teLinkUpdateTdmBw)");
    return;
  }

  return;
}

void corba_update_te_link_band_account(uint32_t *band_account, uint32_t list_len)
{
  //TODO finish it
  if(IS_DEBUG_GRID_NODE(CORBA_ALL))
    zlog_debug("[DBG] CORBA: Update TELink BAND_ACCOUNT");
  return;
}

void link_update_lscG709bw(struct zlist *fODUk, struct zlist *fOCh)
{
  //TODO finish it
  try {
    g2mplsTypes::freeCTPSeq* serverFreeODUk;
    g2mplsTypes::freeCTPSeq* serverFreeOCh;

    g2mplsTypes::freeCTPSeq_var seq;
    g2mplsTypes::freeCTPSeq * tmp;
    tmp = new g2mplsTypes::freeCTPSeq(0);
    if (!tmp) {
      zlog_debug("[ERR] CORBA: link_update_lscG709bw: tmp == NULL");
      return;
    }
    seq = tmp;
    seq->length(0);

    g2mplsTypes::freeCTPSeq freeODUk = seq;      //FIXME Lukasz: where is this information?
    g2mplsTypes::freeCTPSeq freeOCh = seq;       //FIXME Lukasz: where is this information?

    g2pcera_proxy->teLinkGetLscG709Bw(linkIdent, serverFreeODUk, serverFreeOCh);
    if (!equalLscG709Bw(freeODUk, freeOCh, *serverFreeODUk, *serverFreeOCh))
    {
      g2pcera_proxy->teLinkUpdateLscG709Bw(linkIdent, freeODUk, freeOCh);

      if(IS_DEBUG_GRID_NODE(CORBA))
        OSPF_CORBA_DEBUG(OCD_TYPE_TELINK, OCD_ACTION_SENT, OCD_SERV_G2PCERA, "TE_LINK_UPDATE_LSCG709_BW",
                         emptyTnaIdent, linkIdent);
    }
    else
    {
      if (IS_DEBUG_GRID_NODE(CORBA_ALL))
        OSPF_CORBA_DEBUG(OCD_TYPE_TELINK, OCD_ACTION_ABORTED, OCD_SERV_G2PCERA, "TE_LINK_UPDATE_LSCG709_BW",
                         emptyTnaIdent, linkIdent);
    }
  } catch (g2mplsTopology::CannotFetchNode) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchNode (method teLinkUpdateLscG709Bw)");
    return;
  } catch (g2mplsTopology::CannotFetchLink) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchLink (method teLinkUpdateLscG709Bw)");
    return;
  } catch (g2mplsTopology::LinkParamsMismatch) {

    zlog_debug("[ERR] CORBA: Exception LinkParamsMismatch (method teLinkUpdateLscG709Bw)");
    return;
  } catch (g2mplsTopology::InvocationNotAllowed) {

    zlog_debug("[ERR] CORBA: Exception InvocationNotAllowed (method teLinkUpdateLscG709Bw)");
    return;
  } catch (g2mplsTopology::InternalProblems & e) {

    zlog_debug("[ERR] CORBA: Exception InternalProblems (method teLinkUpdateLscG709Bw): %s", (char *) e.what);
  } catch (...) {

    zlog_debug("[ERR] CORBA: Exception Unknown (method teLinkUpdateLscG709Bw)");
    return;
  }

  return;
}

g2mplsTypes::teLinkWdmLambdasBitmap_var lambdasBitmap;

void corba_update_te_link_av_wave_mask(u_int16_t num_wavelengths, u_int32_t label_set_desc, uint32_t *bitmap, uint16_t bitmap_len)
{
  wdm_link_lambdas_bitmap_t lamBitmap;

  lamBitmap.base_lambda_label = label_set_desc;
  lamBitmap.num_wavelengths = num_wavelengths;
  lamBitmap.bitmap_size = (num_wavelengths/32) + 1;
  uint32_t *bitmaps = new uint32_t[lamBitmap.bitmap_size];

  for (uint16_t i = 0; i<bitmap_len; i++)
    bitmaps[i] = bitmap[i];

  lamBitmap.bitmap_word = bitmaps;
  lambdasBitmap << lamBitmap;

  return;
}

void link_update_lscwdm_bw()
{
  g2mplsTypes::teLinkWdmLambdasBitmap_var serverLambdasBitmap;

  try {
    g2pcera_proxy->teLinkGetLscWdmBw(linkIdent, serverLambdasBitmap);
    if (!equalLscWdmBw(lambdasBitmap, serverLambdasBitmap))
    {
      g2pcera_proxy->teLinkUpdateLscWdmBw(linkIdent, lambdasBitmap);

      if (IS_DEBUG_GRID_NODE(CORBA))
        OSPF_CORBA_DEBUG(OCD_TYPE_TELINK, OCD_ACTION_SENT, OCD_SERV_G2PCERA, "TE_LINK_UPDATE_LSCWDM_BW",
                         emptyTnaIdent, linkIdent);
    }
    else
    {
      if (IS_DEBUG_GRID_NODE(CORBA_ALL))
        OSPF_CORBA_DEBUG(OCD_TYPE_TELINK, OCD_ACTION_ABORTED, OCD_SERV_G2PCERA, "TE_LINK_UPDATE_LSCWDM_BW",
                         emptyTnaIdent, linkIdent);
    }
  } catch (g2mplsTopology::CannotFetchNode) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchNode (method teLinkUpdateLscWdmBw)");
    return;
  } catch (g2mplsTopology::CannotFetchLink) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchLink (method teLinkUpdateLscWdmBw)");
    return;
  } catch (g2mplsTopology::LinkParamsMismatch) {

    zlog_debug("[ERR] CORBA: Exception LinkParamsMismatch (method teLinkUpdateLscWdmBw)");
    return;
  } catch (g2mplsTopology::InvocationNotAllowed) {

    zlog_debug("[ERR] CORBA: Exception InvocationNotAllowed (method teLinkUpdateLscWdmBw)");
    return;
  } catch (g2mplsTopology::InternalProblems & e) {

    zlog_debug("[ERR] CORBA: Exception InternalProblems (method teLinkUpdateLscWdmBw): %s", (char *) e.what);
  } catch (...) {

    zlog_debug("[ERR] CORBA: Exception Unknown (method teLinkUpdateLscWdmBw)");
    return;
  }

  return;
}

g2mplsTypes::srlgSeq srlgs;

void corba_update_te_link_shared_risk_link_grp(uint32_t *srlg, uint16_t len)
{
  g2mplsTypes::srlgSeq_var seq;
  g2mplsTypes::srlgSeq * tmp;

  tmp = new g2mplsTypes::srlgSeq(len);
  if (!tmp) {
    zlog_debug("[ERR] CORBA: corba_update_te_link_shared_risk_link_grp: tmp == NULL");
    return;
  }
  seq = tmp;
  seq->length(len);

  for (uint16_t i = 0; i < len; i++)
    seq[i] = srlg[i];

  srlgs = seq;

  return;
}

void corba_update_te_link_srlg()
{
  try {
    g2mplsTypes::srlgSeq uniqueSrlgs;
    g2mplsTypes::srlgSeq * serverSrlgs;

    g2pcera_proxy->teLinkGetSrlgs(linkIdent, serverSrlgs);
    uniqueSrlgs = diffSrlgs(srlgs, *serverSrlgs);
    if (uniqueSrlgs.length() > 0)
    {
      g2pcera_proxy->teLinkAppendSrlgs(linkIdent, uniqueSrlgs);

      if(IS_DEBUG_GRID_NODE(CORBA))
        OSPF_CORBA_DEBUG(OCD_TYPE_TELINK, OCD_ACTION_SENT, OCD_SERV_G2PCERA, "TE_LINK_APPEND_SRLGS",
                         emptyTnaIdent, linkIdent);
    }
    else
    {
      if (IS_DEBUG_GRID_NODE(CORBA_ALL))
        OSPF_CORBA_DEBUG(OCD_TYPE_TELINK, OCD_ACTION_ABORTED, OCD_SERV_G2PCERA, "TE_LINK_APPEND_SRLGS",
                         emptyTnaIdent, linkIdent);
    }
  } catch (g2mplsTopology::CannotFetchNode) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchNode (method teLinkAppendSrlgs)");
    return;
  } catch (g2mplsTopology::CannotFetchLink) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchLink (method teLinkAppendSrlgs)");
    return;
  } catch (g2mplsTopology::LinkParamsMismatch) {

    zlog_debug("[ERR] CORBA: Exception LinkParamsMismatch (method teLinkAppendSrlgs)");
    return;
  } catch (g2mplsTopology::InvocationNotAllowed) {

    zlog_debug("[ERR] CORBA: Exception InvocationNotAllowed (method teLinkAppendSrlgs)");
    return;
  } catch (g2mplsTopology::InternalProblems & e) {

    zlog_debug("[ERR] CORBA: Exception InternalProblems (method teLinkAppendSrlgs): %s", (char *) e.what);
  } catch (...) {

    zlog_debug("[ERR] CORBA: Exception Unknown (method teLinkAppendSrlgs)");
    return;
  }

  return;
}

g2mplsTypes::teLinkCalendarSeq cal;

void corba_update_te_link_callendar(struct te_link_calendar *te_calendar, uint16_t te_calendar_len)
{
  void *data;
  struct te_link_calendar *elem;
  g2mplsTypes::teLinkCalendarSeq_var seq;

  g2mplsTypes::teLinkCalendarSeq * tmp = new g2mplsTypes::teLinkCalendarSeq(te_calendar_len);
  if (!tmp) {
    zlog_debug("[ERR] CORBA: corba_update_te_link_callendar: tmp == NULL");
    return;
  }
  seq = tmp;
  seq->length(te_calendar_len);

  uint32_t tmp32 = 0;
  int i = 0;
  for (uint16_t i=0; i<te_calendar_len; i++)
  {
    g2mplsTypes::linkCalendarEvent entry;
    entry.unixTime = te_calendar[i].time;
    for(int j=0; j<8; j++)
    {
      memcpy(&tmp32, &te_calendar[i].value[j], 4);
      entry.availBw[j] = tmp32;
    }
    seq[i] = entry;
  }
  cal = seq;

  return;
}

void corba_update_te_link_tecal()
{
  try {
    g2mplsTypes::teLinkCalendarSeq uniqueCal;
    g2mplsTypes::teLinkCalendarSeq_var serverCal;

    g2pcera_proxy->teLinkGetCalendar(linkIdent, serverCal);
    uniqueCal = diffCalendars(cal, serverCal);
    if (uniqueCal.length() > 0)
    {
      g2pcera_proxy->teLinkAppendCalendar(linkIdent, uniqueCal);

      if(IS_DEBUG_GRID_NODE(CORBA))
        OSPF_CORBA_DEBUG(OCD_TYPE_TELINK, OCD_ACTION_SENT, OCD_SERV_G2PCERA, "TE_LINK_APPEND_CALENDAR",
                         emptyTnaIdent, linkIdent);
    }
    else
    {
      if (IS_DEBUG_GRID_NODE(CORBA_ALL))
        OSPF_CORBA_DEBUG(OCD_TYPE_TELINK, OCD_ACTION_ABORTED, OCD_SERV_G2PCERA, "TE_LINK_APPEND_CALENDAR",
                         emptyTnaIdent, linkIdent);
    }
  } catch (g2mplsTopology::CannotFetchNode) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchNode (method teLinkAppendCalendar)");
    return;
  } catch (g2mplsTopology::CannotFetchLink) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchLink (method teLinkAppendCalendar)");
    return;
  } catch (g2mplsTopology::LinkParamsMismatch) {

    zlog_debug("[ERR] CORBA: Exception LinkParamsMismatch (method teLinkAppendCalendar)");
    return;
  } catch (g2mplsTopology::InvocationNotAllowed) {

    zlog_debug("[ERR] CORBA: Exception InvocationNotAllowed (method teLinkAppendCalendar)");
    return;
  } catch (g2mplsTopology::InternalProblems & e) {

    zlog_debug("[ERR] CORBA: Exception InternalProblems (method teLinkAppendCalendar): %s", (char *) e.what);
  } catch (...) {

    zlog_debug("[ERR] CORBA: Exception Unknown (method teLinkAppendCalendar)");
    return;
  }

  return;
}

struct zlist teLinkIscs;

void init_grid_TELink_Iscs()
{
  memset (&teLinkIscs, 0, sizeof (struct zlist));
}

void corba_update_te_link_if_sw_cap_desc_genisc(uint8_t switching_cap, uint8_t encoding, float maxLSPbw[])
{
  te_link_if_sw_cap_t* isc = (te_link_if_sw_cap *) XMALLOC(0, sizeof(te_link_if_sw_cap));

  isc->switching_cap = switching_cap;
  isc->encoding = encoding;
  for(int i=0; i<LINK_MAX_PRIORITY; i++)
    isc->maxLSPbw[i] = maxLSPbw[i];

  listnode_add(&teLinkIscs, isc);
  return;
}

void corba_update_te_link_if_sw_cap_desc_tdmisc(uint8_t switching_cap, uint8_t encoding, float maxLSPbw[], float minLSPbw, uint8_t indication)
{
  te_link_if_sw_cap_t* isc = (te_link_if_sw_cap *) XMALLOC(0, sizeof(te_link_if_sw_cap));

  isc->switching_cap = switching_cap;
  isc->encoding = encoding;
  for(int i=0; i<LINK_MAX_PRIORITY; i++)
    isc->maxLSPbw[i] = maxLSPbw[i];
  isc->min_lsp_bw = minLSPbw;
  isc->indication = indication;

  listnode_add(&teLinkIscs, isc);
  return;
}

void corba_update_te_link_if_sw_cap_desc_pscisc(uint8_t switching_cap, uint8_t encoding, float maxLSPbw[], float minLSPbw, uint16_t interfaceMTU)
{
  te_link_if_sw_cap_t* isc = (te_link_if_sw_cap *) XMALLOC(0, sizeof(te_link_if_sw_cap));

  isc->switching_cap = switching_cap;
  isc->encoding = encoding;
  for(int i=0; i<LINK_MAX_PRIORITY; i++)
    isc->maxLSPbw[i] = maxLSPbw[i];
  isc->min_lsp_bw = minLSPbw;
  isc->mtu = interfaceMTU;

  listnode_add(&teLinkIscs, isc);
  return;
}

void corba_update_te_link_if_sw_cap_desc()
{
  void *data; 
  struct zlistnode *node, *nnode;

  g2mplsTypes::iscSeq_var seq;
  {
    g2mplsTypes::iscSeq * tmp;
    tmp = new g2mplsTypes::iscSeq(listcount(&teLinkIscs));
    if (!tmp) {
      zlog_debug("[ERR] CORBA: corba_update_te_link_if_sw_cap_desc: tmp == NULL");
    }
    seq = tmp;
  }
  seq->length(listcount(&teLinkIscs));

  te_link_if_sw_cap_t * elem;
  uint32_t value, temp, z;
  z = 0;
  for (ALL_LIST_ELEMENTS (&teLinkIscs, node, nnode, data)) {

    elem = (te_link_if_sw_cap *) data;
    g2mplsTypes::isc isc;

    g2mplsTypes::iscParamsGen gen;
    g2mplsTypes::iscParamsTdm tdm;
    g2mplsTypes::iscParamsPsc psc;

    uint32_t tmp = 0;
    switch(elem->switching_cap)
    {
      case CAPABILITY_PSC1:
      case CAPABILITY_PSC2:
      case CAPABILITY_PSC3:
      case CAPABILITY_PSC4:
      case CAPABILITY_L2SC:
        psc.swCap   = switching_cap_from_uchar(elem->switching_cap);
        psc.encType = encoding_type_from_uchar(elem->encoding);
        for(int i=0; i<LINK_MAX_PRIORITY; i++)
        {
          memcpy(&tmp, &elem->maxLSPbw[i], 4);
          psc.maxLSPbandwidth[i] = tmp;
        }
        memcpy(&tmp, &elem->min_lsp_bw, 4);
        psc.minLSPbandwidth = tmp;
        psc.interfaceMTU = elem->mtu;
        isc.psc(psc);
        break;
      case CAPABILITY_TDM:
        tdm.swCap   = switching_cap_from_uchar(elem->switching_cap);
        tdm.encType = encoding_type_from_uchar(elem->encoding);
        for(int i=0; i<LINK_MAX_PRIORITY; i++)
        {
          memcpy(&tmp, &elem->maxLSPbw[i], 4);
          tdm.maxLSPbandwidth[i] = tmp;
        }
        memcpy(&tmp, &elem->min_lsp_bw, 4);
        tdm.minLSPbandwidth = tmp;
        tdm.indication = elem->indication;
        isc.tdm(tdm);
        break;
      case CAPABILITY_LSC:
      case CAPABILITY_FSC:
        gen.swCap   = switching_cap_from_uchar(elem->switching_cap);
        gen.encType = encoding_type_from_uchar(elem->encoding);
        for(int i=0; i<LINK_MAX_PRIORITY; i++)
        {
          memcpy(&tmp, &elem->maxLSPbw[i], 4);
          gen.maxLSPbandwidth[i] = tmp;
        }
        isc.gen(gen);
        break;
      default:
        break;
    }

    seq[z] = isc;
    z++;
  }

  g2mplsTypes::iscSeq iscs;
  iscs = seq;

  try {
    g2mplsTypes::iscSeq uniqueIscs;
    g2mplsTypes::iscSeq_var serverIscs;

    g2pcera_proxy->teLinkGetIsc(linkIdent, serverIscs);
    uniqueIscs = diffIscs(iscs, serverIscs);
    if (uniqueIscs.length() > 0)
    {
      g2pcera_proxy->teLinkAppendIsc(linkIdent, uniqueIscs);

      if(IS_DEBUG_GRID_NODE(CORBA))
        OSPF_CORBA_DEBUG(OCD_TYPE_TELINK, OCD_ACTION_SENT, OCD_SERV_G2PCERA, "TE_LINK_APPEND_ISC",
                         emptyTnaIdent, linkIdent);
    }
    else
    {
      if (IS_DEBUG_GRID_NODE(CORBA_ALL))
        OSPF_CORBA_DEBUG(OCD_TYPE_TELINK, OCD_ACTION_ABORTED, OCD_SERV_G2PCERA, "TE_LINK_APPEND_ISC",
                         emptyTnaIdent, linkIdent);
    }
  } catch (g2mplsTopology::CannotFetchNode) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchNode (method teLinkAppendIsc)");
    return;
  } catch (g2mplsTopology::CannotFetchLink) {

    zlog_debug("[ERR] CORBA: Exception CannotFetchLink (method teLinkAppendIsc)");
    return;
  } catch (g2mplsTopology::LinkParamsMismatch) {

    zlog_debug("[ERR] CORBA: Exception LinkParamsMismatch (method teLinkAppendIsc)");
    return;
  } catch (g2mplsTopology::InvocationNotAllowed) {

    zlog_debug("[ERR] CORBA: Exception InvocationNotAllowed (method teLinkAppendIsc)");
    return;
  } catch (g2mplsTopology::InternalProblems & e) {

    zlog_debug("[ERR] CORBA: Exception InternalProblems (method teLinkAppendIsc): %s", (char *) e.what);
    return;
  } catch (...) {

    zlog_debug("[ERR] CORBA: Exception Unknown (method teLinkAppendIsc)");
    return;
  }

  return;
}

} //extern "C"

#endif // HAVE_OMNIORB
