//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//



#include "zebra.h"

#if HAVE_OMNIORB

#include <iostream>
#include <string>
#include <stdlib.h>

#include "log.h"
#include "scngws.h"
#include "scngws_lrm.h"

#include "idl/scngw.hh"
#include "idl/lrm.hh"
#include "lib/corba.h"

// SCNGWs process wide configuration pointer.
extern struct scn_master *scn_m;

// SCNGWs master of the threads.
extern struct thread_master *master;

using namespace std;

extern "C" {
int timer_expired(struct thread * t);
}

class SCNGW_i : public POA_SCNGW::Sync,
		public PortableServer::RefCountServantBase
{
public:
	inline SCNGW_i();
	virtual ~SCNGW_i();

	void end_sync();
	void begin_sync(CORBA::Long scnif_count,
			CORBA::Long cc_count,
			CORBA::Long telink_count);
	void sync_fatal_error();
	void scnif_add(const g2mplsTypes::addr & addr);
	void scnif_delete(const  g2mplsTypes::addr & addr);
	void tel_cc_assoc_add(const SCNGW::Sync::TELCC_Add_AssocSeq& assocs);
	void tel_cc_assoc_delete(const SCNGW::Sync::TELCC_Delete_AssocSeq& assocs);
	void tel_cc_assoc_update(CORBA::Long key_id,
				 const SCNGW::Sync::UpdateSeq& updates);
	void cc_scnif_assoc_add(const SCNGW::Sync::CC_Add_AssocSeq& assocs);
	void cc_scnif_assoc_delete(const SCNGW::Sync::CC_Delete_AssocSeq& assocs);
	void cc_scnif_assoc_update(CORBA::Long cc_id,
				   const  g2mplsTypes::addr & local_addr,
				   const  g2mplsTypes::addr & remote_addr);

	bool syncTimedOut();
	void fireTimeOut();
	void syncTimeOutClear();

private:
	void set_scnif(struct scnif *  scn_if,
		       uint32_t        address);
	bool set_cc_scnif_assoc(struct cc_scnif_assoc *    cc_scnif,
				struct SCNGW::Sync::CC_Add_Assoc assoc);
	bool set_tel_cc_assoc(struct tel_cc_assoc *         tel_cc,
			      struct SCNGW::Sync::TELCC_Add_Assoc assoc);
	void exec_tel_cc_assoc_update(struct tel_cc_assoc * tel_cc,
				      struct SCNGW::Sync::Update  update);
	void reset_associations();
	void sync_err();

	u_int32_t n_scnif;
	u_int32_t n_assoc_tel;
	u_int32_t n_assoc_scnif;
	bool      TimedOut;
};

void
SCNGW_i::set_scnif(struct scnif * scn_if,
		   uint32_t       address)
{
	scn_if->loc_addr.s_addr = (unsigned long) address;
	scn_if->status          = STATUS_UP;
}

bool
SCNGW_i::set_cc_scnif_assoc(struct cc_scnif_assoc *    cc_scnif,
			    struct SCNGW::Sync::CC_Add_Assoc assoc)
{
	if ((assoc.local_addr._d()  != g2mplsTypes::ADDRTYPE_IPV4) ||
	    (assoc.remote_addr._d() != g2mplsTypes::ADDRTYPE_IPV4)) {
		zlog_debug("Uncorrect address type");
		return false;
	}

	cc_scnif->status           = STATUS_UP;
	cc_scnif->cc_id            = (u_int32_t)assoc.cc_id;
	cc_scnif->scnif_loc.s_addr = (unsigned long)assoc.local_addr.ipv4();
	cc_scnif->scnif_rem.s_addr = (unsigned long)assoc.remote_addr.ipv4();

	return true;
}

bool
SCNGW_i::set_tel_cc_assoc(struct tel_cc_assoc *         tel_cc,
			  struct SCNGW::Sync::TELCC_Add_Assoc assoc)
{
	u_int32_t * ccid;

	if ((assoc.local_addr._d()  != g2mplsTypes::ADDRTYPE_IPV4) ||
	    (assoc.remote_addr._d() != g2mplsTypes::ADDRTYPE_IPV4)) {
		zlog_debug("Uncorrect address type");
		return false;
	}

	tel_cc->status         = STATUS_UP;
	tel_cc->key            = (u_int32_t)assoc.key_id;
	tel_cc->interface_type = (( assoc.iftype == SCNGW::Sync::IF_INNI) ? SCN_INNI:
				  ((assoc.iftype == SCNGW::Sync::IF_ENNI) ? SCN_ENNI:
				   SCN_UNI));
	tel_cc->tel_loc.s_addr = (unsigned long)assoc.local_addr.ipv4();
	tel_cc->tel_rem.s_addr = (unsigned long)assoc.remote_addr.ipv4();

	for (size_t i = 0; i < assoc.ccs.length(); i++) {
		ccid  = (u_int32_t*) malloc(sizeof(u_int32_t));
		*ccid = (u_int32_t) assoc.ccs[i].id;
		if (check_cc (*ccid)) {
			listnode_add (tel_cc->cclist, ccid);
		} else {
			zlog_debug("\t CC %u is not present in the list of "
				   "CC/SCN-if associations", *ccid);
			free(ccid);
		}
	}
	return true;
}

void
SCNGW_i::exec_tel_cc_assoc_update(struct tel_cc_assoc * tel_cc,
				  struct SCNGW::Sync::Update  update)
{
	u_int32_t          op;
	u_int32_t        * cc_id;
	u_int32_t        * match;
	struct zlistnode * cc_node;
	struct zlistnode * cc_nnode;
	void		 * p;
	int		   found;

	found = 0;

	cc_id  = (u_int32_t *) malloc(sizeof(u_int32_t));

	op     = ((update.type == SCNGW::Sync::TELCC_ADD) ? ADD_OP : DEL_OP);
	*cc_id = (u_int32_t) update.cc_id;
	if (!check_cc (*cc_id)) {
		zlog_debug ("\t CC %u is not present in the list of CC/SCN-if "
			    "associations", *cc_id);
		free(cc_id);
		return;
	}

	switch (op) {
		case ADD_OP:
			for (ALL_LIST_ELEMENTS (tel_cc->cclist, cc_node,
						cc_nnode, p)) {
				match = (u_int32_t *) p;
				if (*match == *cc_id) {
					found = 1;
					zlog_debug ("\t CC %u is is already "
						    "associated at TE-link "
						    "%s", *cc_id,
						    inet_ntoa(tel_cc->tel_loc));
					break;
				}
			}
			if (!found) {
				listnode_add (tel_cc->cclist, cc_id);
				zlog_debug ("\t Added a CC (id = %u) to the "
					    "list of associated CC of TE-link "
					    "%s", *cc_id,
					    inet_ntoa(tel_cc->tel_loc));
			}
			return;

		case DEL_OP:
			for (ALL_LIST_ELEMENTS (tel_cc->cclist, cc_node,
						cc_nnode, p)) {
				match = (u_int32_t *) p;
				if (*match == *cc_id) {
					found = 1;
					zlog_debug ("\t Deleted an associated "
						    "CC (id = %u) from TE-link "
						    "%s", *cc_id,
						    inet_ntoa(tel_cc->tel_loc));
					listnode_delete (tel_cc->cclist, match);
					free(match);
					break;
				}
			}
			if (!found) {
				zlog_debug("\t CC %u is not associated at "
					   "TE-link %s and cannot be deleted",
					   *cc_id, inet_ntoa(tel_cc->tel_loc));
			}
			break;
	}

	free(cc_id);
}

void
SCNGW_i::reset_associations()
{
	struct scnif           * scn_if;
	struct zlistnode       * scnif_node;
	struct zlistnode       * scnif_nnode;
	struct tel_cc_assoc    * tel_cc;
	struct zlistnode       * telcc_node;
	struct zlistnode       * telcc_nnode;
	struct cc_scnif_assoc  * cc_scnif;
	struct zlistnode       * ccscnif_node;
	struct zlistnode       * ccscnif_nnode;
	struct zlist	       * scnif_list;
	struct zlist	       * tel_cc_list;
	struct zlist	       * cc_scnif_list;
	void		       * p;

	scnif_list    = get_scnif_list();
	tel_cc_list   = get_tel_cc_list();
	cc_scnif_list = get_cc_scnif_list();

	scn_m->lrm_conn_status = STATUS_DOWN;

	for (ALL_LIST_ELEMENTS (scnif_list, scnif_node, scnif_nnode, p)) {
		scn_if = (scnif *) p;
		scn_if->status = STATUS_DOWN;
	}
	for (ALL_LIST_ELEMENTS (tel_cc_list, telcc_node, telcc_nnode, p)) {
		tel_cc = (tel_cc_assoc *) p;
		tel_cc->status = STATUS_DOWN;
	}
	for (ALL_LIST_ELEMENTS (cc_scnif_list, ccscnif_node, ccscnif_nnode, p)) {
		cc_scnif = (cc_scnif_assoc *) p;
		cc_scnif->status = STATUS_DOWN;
	}

	n_scnif = 0;
	n_assoc_tel = 0;
	n_assoc_scnif = 0;
}

void
SCNGW_i::sync_err()
{
	delete_tmp_associations();
	reset_associations();
	syncTimeOutClear();
}

bool
SCNGW_i::syncTimedOut()
{
	return TimedOut;
}

void
SCNGW_i::fireTimeOut()
{
	TimedOut = true;

	zlog_debug("Timeout expired waiting END_SYNC message from LRM. "
		   "Synchronization must be restarted.");

}

void
SCNGW_i::syncTimeOutClear()
{
	thread_cancel (scn_m->timer_thread);
	TimedOut = false;
}

SCNGW_i::SCNGW_i()
{
	TimedOut = false;
}

SCNGW_i::~SCNGW_i()
{
}

void SCNGW_i::begin_sync(CORBA::Long scnif_count,
			 CORBA::Long cc_count,
			 CORBA::Long telink_count)
{
	STACK_LOCK();

	zlog_debug ("Received a BEGIN SYNC message from LRM");

	switch (scn_m->lrm_conn_status) {
		case STATUS_UP:
			zlog_debug ("Connection with LRM is up. Expected "
				    "another kind of command.");
			//throw wrong message unack;
			STACK_UNLOCK();
			return;
		case STATUS_SYNC:
			if(!syncTimedOut()) {
				zlog_debug ("Connection with LRM is in "
					    "synchronization phase. Expected "
					    "another kind of command.");
			}
			sync_err();
			STACK_UNLOCK();
			throw SCNGW::Sync::SyncErr();
		case STATUS_DOWN:
			zlog_debug ("The communication with LRM is again "
				    "available");
			zlog_debug ("Synchronizing the TE-link/CC/SCN-if "
				    "associations");
			scn_m->lrm_conn_status = STATUS_SYNC;
			break;
		case STATUS_NEVER_CONN:
			zlog_debug ("Synchronizing the TE-link/CC/SCN-if "
				    "associations");
			scn_m->lrm_conn_status = STATUS_SYNC;
			tel_cc_scnif_association_init ();
			tmp_tel_cc_scnif_association_init ();
			break;
	}
	// Set timeout waiting END_SYNC from LRM
	scn_m->timer_thread = thread_add_timer_msec(master,
						    timer_expired,
						    NULL,
						    scn_m->timeout_lrmsync);
	// Get the number of TE-link/CC and CC/SCn-if associatons
	n_scnif       = (u_int32_t) scnif_count;
	n_assoc_scnif = (u_int32_t) cc_count;
	n_assoc_tel   = (u_int32_t) telink_count;

	zlog_debug ("\t number of SCN-if: %u", n_scnif);
	zlog_debug ("\t number of CC: %u", n_assoc_scnif);
	zlog_debug ("\t number of TE-link: %u", n_assoc_tel);

	STACK_UNLOCK();
}

void SCNGW_i::end_sync()
{
	STACK_LOCK();

	zlog_debug ("Received an END_SYNC message from LRM");

	syncTimeOutClear();

	switch (scn_m->lrm_conn_status) {
		case STATUS_SYNC:
			if ((n_assoc_scnif == 0)&&(n_assoc_tel == 0)) {
				scn_m->lrm_conn_status = STATUS_UP;
				realign_associations ();
				delete_old_associations ();
				zlog_debug ("End of synchronization of the "
					    "TE-link/CC/SCN-if associations");
				STACK_UNLOCK();
				return;
			}
			else {
				zlog_debug ("Connection with LRM is in "
					    "synchronization phase. Expected "
					    "another kind of command.");
				sync_err();
				STACK_UNLOCK();
				throw SCNGW::Sync::SyncErr();
			}
			break;
		default:
			zlog_debug ("Connection with LRM is not in "
				    "synchronization phase. Expected another "
				    "kind of command.");
			//throw wrong message unack
			STACK_UNLOCK();
			return;
	}

	STACK_UNLOCK();
}

void SCNGW_i::sync_fatal_error()
{
	STACK_LOCK();

	zlog_debug ("---------------------------------------");
	zlog_debug ("Received a SYNC_FATAL_ERROR message from LRM");

	reset_associations();

	zlog_debug ("FATAL ERROR: loop in the LRM-SCNGW communication. Aborting..");
	zlog_debug ("---------------------------------------");

	STACK_UNLOCK();

	exit(1);
}

void SCNGW_i::scnif_add(const g2mplsTypes::addr & addr)
{
	STACK_LOCK();

	struct scnif     * scn_if;
	struct zlistnode * scnif_node, * scnif_nnode;
	struct scnif     * match;
	struct zlist     * scnif_list;
	struct zlist     * tmp_scnif_list;
	uint32_t           address;
	void		 * p;
	bool		   err = false;

	zlog_debug("Received a SCNIF_ADD message from LRM");

	scnif_list     = get_scnif_list();
	tmp_scnif_list = get_tmp_scnif_list();

	if(addr._d() != g2mplsTypes::ADDRTYPE_IPV4) {
		zlog_debug("Uncorrect address type");
		err = true;
	}

	switch (scn_m->lrm_conn_status) {
		case STATUS_UP:
			for (ALL_LIST_ELEMENTS (scnif_list, scnif_node,
						scnif_nnode, p)) {
				match = (struct scnif *) p;
				if (err == true) {
					STACK_UNLOCK();
					throw SCNGW::Sync::InternalProblems();
				}
				address = addr.ipv4();
				if (address == match->loc_addr.s_addr) {
					zlog_debug("\t SCN-if %s is already "
						   "present in SCNGW",
						   inet_ntoa(match->loc_addr));
					//throw scnif already present
					STACK_UNLOCK();
					return;
				}
			}
			scn_if = scnif_new ();
			set_scnif(scn_if, address);
			listnode_add (scnif_list, scn_if);
			zlog_debug ("\t Added a new SCN-if from LRM");
			zlog_debug ("\t\t address: %s",
				    inet_ntoa(scn_if->loc_addr));
			break;
		case STATUS_SYNC:
			if(syncTimedOut()) {
				sync_err();
				STACK_UNLOCK();
				throw SCNGW::Sync::SyncErr();
			}
			if (n_scnif == 0) {
				if ((n_assoc_scnif == 0) &&
				    (n_assoc_tel == 0)) {
					zlog_debug ("Connection with LRM is "
						    "in synchronization phase."
						    "Expected command "
						    "END_SYNC. Error.");
				} else {
					zlog_debug ("Connection with LRM is "
						    "in synchronization phase."
						    "Expected another kind of "
						    "command. Error.");
				}
				sync_err();
				STACK_UNLOCK();
				throw SCNGW::Sync::SyncErr();
			} else {
				if (err == true) {
					STACK_UNLOCK();
					throw SCNGW::Sync::InternalProblems();
				}
				address = addr.ipv4();
				n_scnif --;
				scn_if = scnif_new ();
				set_scnif(scn_if, address);
				listnode_add (tmp_scnif_list, scn_if);
				zlog_debug ("\t Added an SCN-if to the "
					    "temporary list");
			}
			break;
		case STATUS_NEVER_CONN:
		case STATUS_DOWN:
			zlog_debug ("Connection with LRM is down. Expected "
				    "command BEGIN_SYNC. Error.");
			reset_associations();
			STACK_UNLOCK();
			throw SCNGW::Sync::SyncErr();
		default:
			break;
	}

	STACK_UNLOCK();
}

void SCNGW_i::scnif_delete(const g2mplsTypes::addr & addr)
{
	STACK_LOCK();

	u_int32_t          found;
	int		   count;
	struct in_addr     address;
	struct zlistnode * scnif_node;
	struct zlistnode * scnif_nnode;
	struct scnif     * match;
	struct zlist     * scnif_list;
	void             * p;


	zlog_debug ("Received a SCNIF_DELETE message from LRM");

	scnif_list = get_scnif_list();

	switch (scn_m->lrm_conn_status) {
		case STATUS_SYNC:
			if(!syncTimedOut()) {
				zlog_debug ("Connection with LRM is in "
					    "synchronization phase. Expected "
					    "another kind of command");
			}
			sync_err();
			STACK_UNLOCK();
			throw SCNGW::Sync::SyncErr();
			break;
		case STATUS_NEVER_CONN:
		case STATUS_DOWN:
			zlog_debug ("Connection with LRM is down. Expected "
				    "command BEGIN_SYNC. Error.");
			reset_associations();
			STACK_UNLOCK();
			throw SCNGW::Sync::SyncErr();
			break;
		default:
			break;
	}

	if(addr._d() != g2mplsTypes::ADDRTYPE_IPV4) {
		zlog_debug("Uncorrect address type");
		STACK_UNLOCK();
		throw SCNGW::Sync::InternalProblems();
	}

	address.s_addr = addr.ipv4();
	for (ALL_LIST_ELEMENTS (scnif_list, scnif_node, scnif_nnode, p)) {
		match = (struct scnif *) p;
		if (match->loc_addr.s_addr == address.s_addr) {
			zlog_debug ("\t Deleted a SCN-if");
			zlog_debug ("\t\t address: %s",
				    inet_ntoa(match->loc_addr));
			listnode_delete (scnif_list, match);
			free(match);
			count = delete_scnif_from_cc(address);
			if (count) {
				zlog_debug ("\t\t This SCN-if was associated "
					    "with %d CCs", count);
				zlog_debug ("\t\t Deleted these CCs from "
					    "correspondent TE-links");
			}
			STACK_UNLOCK();
			return;
		}
	}
	zlog_debug("\t SCN-if %s is not present in SCNGW and cannot be "
		   "deleted", inet_ntoa(address));

	STACK_UNLOCK();
}

void SCNGW_i::tel_cc_assoc_add(const SCNGW::Sync::TELCC_Add_AssocSeq& assocs)
{
	STACK_LOCK();

	struct tel_cc_assoc  * tel_cc;
	struct zlistnode     * cc_node;
	struct zlistnode     * cc_nnode;
	u_int32_t            * cc_id;
	struct zlistnode     * telcc_node;
	struct zlistnode     * telcc_nnode;
	struct tel_cc_assoc  * match;
	struct zlist	     * tel_cc_list;
	struct zlist	     * tmp_tel_cc_list;
	void		     * p;
	int		       found;

	zlog_debug ("Received a TEL_CC_ASSOC_ADD message from LRM");

	found = 0;

	tel_cc_list     = get_tel_cc_list();
	tmp_tel_cc_list = get_tmp_tel_cc_list();

	switch (scn_m->lrm_conn_status) {
		case STATUS_UP:
			for (size_t i = 0; i < assocs.length(); i++) {
				for (ALL_LIST_ELEMENTS (tel_cc_list, telcc_node,
							telcc_nnode, p)) {
					match = (struct tel_cc_assoc *) p;
					if (match->key == (u_int32_t)assocs[i].key_id) {
						zlog_debug("\t TE-link with key"
							   " %u is already "
							   "present in SCNGW",
							   match->key);
						found = 1;
						break;
					}
				}
				if (found) {
					found = 0;
					continue;
				}
				tel_cc = tel_cc_new ();
				if(!set_tel_cc_assoc (tel_cc, assocs[i])) {
					free(tel_cc);
					STACK_UNLOCK();
					throw SCNGW::Sync::InternalProblems();
				}
				listnode_add (tel_cc_list, tel_cc);
				zlog_debug ("\t Added a new TE-link/CC "
					    "association (key = %u, %s) from "
					    "LRM", tel_cc->key,
					    if_type_string(tel_cc->interface_type));
				zlog_debug ("\t\t TE-link loc: %s",
					    inet_ntoa(tel_cc->tel_loc));
				zlog_debug ("\t\t TE-link rem: %s (%u CC's "
					    "associated)",
					    inet_ntoa(tel_cc->tel_rem),
					    tel_cc->cclist->count);
				for (ALL_LIST_ELEMENTS (tel_cc->cclist,
							cc_node, cc_nnode,
							p)) {
					cc_id = (u_int32_t *) p;
					zlog_debug ("\t\t\t CCid: %u", *cc_id);
				}
			}
			break;
		case STATUS_SYNC:
			if(syncTimedOut()) {
				sync_err();
				STACK_UNLOCK();
				throw SCNGW::Sync::SyncErr();
			}
			if ((n_scnif != 0) || (n_assoc_scnif != 0)) {
				zlog_debug ("Connection with LRM is in "
					    "synchronization phase. Expected "
					    "another kind of command. Error.");
				sync_err();
				STACK_UNLOCK();
				throw SCNGW::Sync::SyncErr();
			}
			else if (n_assoc_tel == 0) {
				zlog_debug ("Connection with LRM is in "
					    "synchronization phase. Expected "
					    "command END_SYNC. Error.");
				sync_err();
				STACK_UNLOCK();
				throw SCNGW::Sync::SyncErr();
			}
			n_assoc_tel --;
			for (size_t i = 0; i < assocs.length(); i++) {
				tel_cc = tel_cc_new ();
				if(!set_tel_cc_assoc (tel_cc, assocs[i])) {
					free(tel_cc);
					STACK_UNLOCK();
					throw SCNGW::Sync::InternalProblems();
				}
				listnode_add (tmp_tel_cc_list, tel_cc);
				zlog_debug ("\t Added a TE-link/CC association "
					    "to the temporary list of "
					    "associations");
			}
			break;
		case STATUS_NEVER_CONN:
		case STATUS_DOWN:
			zlog_debug ("Connection with LRM is down. Expected "
				    "command LRM_BEGIN_SYNC. Error.");
			reset_associations();
			STACK_UNLOCK();
			throw SCNGW::Sync::SyncErr();
			break;
		default:
			break;
	}
	STACK_UNLOCK();
}

void SCNGW_i::tel_cc_assoc_delete(const SCNGW::Sync::TELCC_Delete_AssocSeq & assocs)
{
	STACK_LOCK();

	u_int32_t             key;
	struct zlistnode    * telcc_node;
	struct zlistnode    * telcc_nnode;
	struct tel_cc_assoc * match;
	struct zlist        * tel_cc_list;
	void		    * p;
	int		      found;

	zlog_debug ("Received a TEL_CC_ASSOC_DELETE message from LRM");

	tel_cc_list = get_tel_cc_list();

	switch (scn_m->lrm_conn_status) {
		case STATUS_SYNC:
			if(!syncTimedOut()) {
				zlog_debug ("Connection with LRM is in "
					    "synchronization phase. Expected "
					    "another kind of command");
			}
			sync_err();
			STACK_UNLOCK();
			throw SCNGW::Sync::SyncErr();
		case STATUS_NEVER_CONN:
		case STATUS_DOWN:
			zlog_debug ("Connection with LRM is down. Expected "
				    "command BEGIN_SYNC. Error.");
			reset_associations();
			STACK_UNLOCK();
			throw SCNGW::Sync::SyncErr();
		default:
			break;
	}

	for (size_t i = 0; i < assocs.length(); i++) {
		found = 0;
		key = assocs[i].key_id;
		for (ALL_LIST_ELEMENTS (tel_cc_list, telcc_node,
					telcc_nnode, p)) {
			match = (struct tel_cc_assoc *) p;
			if (match->key == key) {
				found = 1;
				zlog_debug ("\t Deleted a  TE-link/CC "
					    "association");
				zlog_debug ("\t\t TE-link loc: %s",
					    inet_ntoa(match->tel_loc));
				zlog_debug ("\t\t TE-link rem: %s",
					    inet_ntoa(match->tel_rem));
				listnode_delete (tel_cc_list, match);
				free (match);
				break;
			}
		}
		if (!found) {
			zlog_debug("\t TE-link with key %u is not present in "
				   "SCNGW and cannot be deleted", key);
		}
	}

	STACK_UNLOCK();
}

void SCNGW_i::tel_cc_assoc_update(CORBA::Long key_id,
				  const SCNGW::Sync::UpdateSeq& updates)
{
	STACK_LOCK();

	struct zlistnode    * telcc_node;
	struct zlistnode    * telcc_nnode;
	struct tel_cc_assoc * match;
	u_int32_t             key;
	u_int32_t             n_upd;
	struct zlist        * tel_cc_list;
	void                * p;

	zlog_debug ("Received a TEL_CC_ASSOC_UPDATE message from LRM");

	tel_cc_list = get_tel_cc_list();

	switch (scn_m->lrm_conn_status) {
		case STATUS_SYNC:
			if(syncTimedOut()) {
				sync_err();
				STACK_UNLOCK();
				throw SCNGW::Sync::SyncErr();
			}
			zlog_debug ("Connection with LRM is in synchronization "
				    "phase. Expected another kind of command");
			sync_err();
			STACK_UNLOCK();
			throw SCNGW::Sync::SyncErr();
		case STATUS_NEVER_CONN:
		case STATUS_DOWN:
			zlog_debug ("Connection with LRM is down. Expected "
				    "command BEGIN_SYNC. Error.");
			reset_associations();
			STACK_UNLOCK();
			throw SCNGW::Sync::SyncErr();
		default:
			break;
	}
	// Get the key of TE-link/CC association to update
	key = (u_int32_t) key_id;
	// Get the number of CC to add/delete
	n_upd = (u_int32_t) updates.length();

	for (ALL_LIST_ELEMENTS (tel_cc_list, telcc_node, telcc_nnode, p)) {
		match = (struct tel_cc_assoc *) p;
		if (match->key == key) {
			for (size_t i = 0; i < n_upd; i++) {
				exec_tel_cc_assoc_update(match, updates[i]);
			}
			STACK_UNLOCK();
			return;
		}
	}
	zlog_debug ("\t TE-link with key %u is not present in SCNGW "
		    "and cannot be updated", key);

	//throw tel cc assoc not found
	STACK_UNLOCK();
}

void SCNGW_i::cc_scnif_assoc_add(const SCNGW::Sync::CC_Add_AssocSeq& assocs)
{
	STACK_LOCK();

	struct cc_scnif_assoc * cc_scnif;
	struct cc_scnif_assoc * match;
	struct zlistnode      * ccscnif_node;
	struct zlistnode      * ccscnif_nnode;
	struct zlist	      * cc_scnif_list;
	struct zlist	      * tmp_cc_scnif_list;
	void		      * p;
	int		        found;

	zlog_debug ("Received a CC_SCNIF_ASSOC_ADD message from LRM");

	found = 0;

	cc_scnif_list     = get_cc_scnif_list();
	tmp_cc_scnif_list = get_tmp_cc_scnif_list();

	switch (scn_m->lrm_conn_status) {
		case STATUS_UP:
			for (size_t i = 0; i < assocs.length(); i++) {
				for (ALL_LIST_ELEMENTS (cc_scnif_list,
							ccscnif_node,
							ccscnif_nnode,
							p)) {
					match = (struct cc_scnif_assoc *) p;
					if (match->cc_id == (u_int32_t)assocs[i].cc_id) {
						zlog_debug("\t CC with id %u "
							   "is already present "
							   "in SCNGW",
							   match->cc_id);
						found = 1;
						break;
					}
				}
				if (found) {
					found = 0;
					continue;
				}
				cc_scnif = cc_scnif_new ();
				if(!set_cc_scnif_assoc(cc_scnif, assocs[i])) {
					free(cc_scnif);
					STACK_UNLOCK();
					throw SCNGW::Sync::InternalProblems();
				}
				listnode_add (cc_scnif_list, cc_scnif);
				zlog_debug ("\t Added a new CC/SCN-if "
					    "association from LRM");
				zlog_debug ("\t\t CCid: %u", cc_scnif->cc_id);
				zlog_debug ("\t\t\t SCN-if loc: %s",
					    inet_ntoa(cc_scnif->scnif_loc));
				zlog_debug ("\t\t\t SCN-if rem: %s",
					    inet_ntoa(cc_scnif->scnif_rem));
			}
			break;
		case STATUS_SYNC:
			if(syncTimedOut()) {
				sync_err();
				STACK_UNLOCK();
				throw SCNGW::Sync::SyncErr();
			}
			if (n_scnif != 0) {
				zlog_debug ("Connection with LRM is in "
					    "synchronization phase. Expected "
					    "command SCNIF_ADD. Error.");
				sync_err();
				STACK_UNLOCK();
				throw SCNGW::Sync::SyncErr();
			} else if (n_assoc_scnif == 0) {
				if (n_assoc_tel == 0) {
					zlog_debug ("Connection with LRM is "
						    "in synchronization phase. "
						    "Expected command END_SYNC."
						    "Error.");
				} else {
					zlog_debug ("Connection with LRM is in "
						    "synchronization phase. "
						    "Expected command "
						    "TEL_CC_ASSOC_ADD. Error.");
				}
				sync_err();
				STACK_UNLOCK();
				throw SCNGW::Sync::SyncErr();
			}
			n_assoc_scnif --;
			for (size_t i = 0; i < assocs.length(); i++) {
				cc_scnif = cc_scnif_new ();
				if(!set_cc_scnif_assoc(cc_scnif, assocs[i])) {
					free(cc_scnif);
					STACK_UNLOCK();
					throw SCNGW::Sync::InternalProblems();
				}
				listnode_add (tmp_cc_scnif_list, cc_scnif);
				zlog_debug ("\t Added CC/SCN-if association "
					    "to the temporary list of "
					    "associations");
			}
			break;
		case STATUS_NEVER_CONN:
		case STATUS_DOWN:
			zlog_debug ("Connection with LRM is down. Expected "
				    "command BEGIN_SYNC. Error.");
			reset_associations();
			STACK_UNLOCK();
			throw SCNGW::Sync::SyncErr();
		default:
			break;
	}

	STACK_UNLOCK();
}

void SCNGW_i::cc_scnif_assoc_delete(const SCNGW::Sync::CC_Delete_AssocSeq& assocs)
{
	STACK_LOCK();

	u_int32_t               cc_id;
	int                     count;
	struct zlistnode      * ccscnif_node;
	struct zlistnode      * ccscnif_nnode;
	struct cc_scnif_assoc * match;
	struct zlist          * cc_scnif_list;
	void                  * p;
	int		        found;

	zlog_debug ("Received a CC_SCNIF_ASSOC_DELETE message from LRM");

	cc_scnif_list = get_cc_scnif_list();

	switch (scn_m->lrm_conn_status) {
		case STATUS_SYNC:
			if(syncTimedOut()) {
				zlog_debug ("Connection with LRM is in "
					    "synchronization phase. Expected "
					    "another kind of command");
			}
			sync_err();
			STACK_UNLOCK();
			throw SCNGW::Sync::SyncErr();
		case STATUS_NEVER_CONN:
		case STATUS_DOWN:
			zlog_debug ("Connection with LRM is down. Expected "
				    "command BEGIN_SYNC. Error.");
			reset_associations();
			STACK_UNLOCK();
			throw SCNGW::Sync::SyncErr();
		default:
			break;
	}
	for (size_t i = 0; i < assocs.length(); i++) {
		found = 0;
		cc_id = (u_int32_t) assocs[i].cc_id;
		for (ALL_LIST_ELEMENTS (cc_scnif_list, ccscnif_node,
					ccscnif_nnode, p)) {
			match = (struct cc_scnif_assoc *) p;
			if (match->cc_id == cc_id) {
				found  = 1;
				zlog_debug ("\t Deleted a CC/SCN-if "
					    "association");
				zlog_debug ("\t\t CCid: %u", match->cc_id);
				zlog_debug ("\t\t\t SCN-if loc: %s",
					    inet_ntoa(match->scnif_loc));
				zlog_debug ("\t\t\t SCN-if rem: %s",
					    inet_ntoa(match->scnif_rem));
				listnode_delete (cc_scnif_list, match);
				free(match);
				count = delete_cc_from_tel (cc_id);
				if (count) {
					zlog_debug ("\t\t This CC was "
						    "associated with %d "
						    "TE-links", count);
				}
				break;
			}
		}
		if (!found) {
			zlog_debug("\t CC with id %u is not present in SCNGW "
				   "and cannot be deleted", cc_id);
		}
	}

	STACK_UNLOCK();

}

void SCNGW_i::cc_scnif_assoc_update(CORBA::Long cc_id,
				    const g2mplsTypes::addr & local_addr,
				    const g2mplsTypes::addr & remote_addr)
{
	STACK_LOCK();

	struct zlistnode      * ccscnif_node;
	struct zlistnode      * ccscnif_nnode;
	struct cc_scnif_assoc * cc_scnif;
	u_int32_t	        ccid;
	struct zlist          * cc_scnif_list;
	void                  * p;

	zlog_debug ("Received a CC_SCNIF_ASSOC_UPDATE message from LRM");

	cc_scnif_list = get_cc_scnif_list();

	switch (scn_m->lrm_conn_status) {
		case STATUS_SYNC:
			if(syncTimedOut()) {
				zlog_debug ("Connection with LRM is in "
					    "synchronization phase. Expected "
					    "another kin of command");
			}
			sync_err();
			STACK_UNLOCK();
			throw SCNGW::Sync::SyncErr();
		case STATUS_NEVER_CONN:
		case STATUS_DOWN:
			zlog_debug ("Connection with LRM is down. Expected "
				    "command BEGIN_SYNC. Error.");
			reset_associations();
			STACK_UNLOCK();
			throw SCNGW::Sync::SyncErr();
		default:
			break;
	}

	if ((local_addr._d()  != g2mplsTypes::ADDRTYPE_IPV4) ||
	    (remote_addr._d() != g2mplsTypes::ADDRTYPE_IPV4)) {
		zlog_debug("Uncorrect address type");
		STACK_UNLOCK();
		throw SCNGW::Sync::InternalProblems();
	}

	// Get the id of CC/SCN-if associaton to update
	ccid = (u_int32_t) cc_id;

	for (ALL_LIST_ELEMENTS (cc_scnif_list, ccscnif_node,
				ccscnif_nnode, p)) {
		cc_scnif = (struct cc_scnif_assoc *) p;
		if (cc_scnif->cc_id == ccid) {
			cc_scnif->scnif_loc.s_addr = (unsigned long)local_addr.ipv4();
			cc_scnif->scnif_rem.s_addr = (unsigned long)remote_addr.ipv4();

			zlog_debug ("\t Updated a CC/SCN-if association "
				    "(CCid = %u)", cc_scnif->cc_id);
			zlog_debug ("\t\t SCN-if loc: %s",
				    inet_ntoa(cc_scnif->scnif_loc));
			zlog_debug ("\t\t SCN-if rem: %s",
				    inet_ntoa(cc_scnif->scnif_rem));

			STACK_UNLOCK();
			return;
		}
	}
	zlog_debug ("\t CC with id %u is not present in SCNGW and "
		    "cannot be updated", ccid);

	STACK_UNLOCK();
	throw SCNGW::Sync::CCNotFound();
}

SCNGW_i* servant    = 0;
LRM::Info_var  lrm_proxy;

extern "C" {
	int timer_expired(struct thread * t)
	{
		servant->fireTimeOut();

		return 0;
	}
	int corba_client_setup(void)
	{
		CORBA::ORB_var orb;
		orb = corba_orb();
		if (CORBA::is_nil(orb)) {
			zlog_err("Cannot get ORB");
			return 0;
		}

		string ior;
		if (!corba_fetch_ior(CORBA_SERVANT_LRM, ior)) {
			zlog_err("Cannot fetch IOR");
			return 0;
		}

		try {
			CORBA::Object_var obj;
			obj = orb->string_to_object(ior.c_str());
			if (CORBA::is_nil(obj)) {
				zlog_err("Cannot get object");
				return 0;
			}

			lrm_proxy = LRM::Info::_narrow(obj);
			if (CORBA::is_nil(lrm_proxy)) {
				zlog_err("cannot invoke on a nil object reference");
				return 0;
			}

		} catch (CORBA::SystemException & e) {
			zlog_err("Caught CORBA::SystemException");
			return 0;
		} catch (CORBA::Exception & e) {
			zlog_err("Caught CORBA::Exception");
			return 0;
		} catch (...) {
			zlog_err("CLIENT_SETUP: caught unknown exception");
			return 0;
		}

		return 1;
	}

	int corba_client_shutdown(void)
	{
		return 1;
	}

	int corba_server_setup(void)
	{
		zlog_debug("Setting up CORBA server side");

		try {
			servant = new SCNGW_i();
			if (!servant) {
				throw string("Cannot create servant");
			}

			PortableServer::POA_var poa;
			poa = corba_poa();
			if (CORBA::is_nil(poa)) {
				throw string("Cannot get POA");
			}

			PortableServer::ObjectId_var servant_id;
			servant_id = poa->activate_object(servant);

			CORBA::Object_var obj;
			obj = servant->_this();
			if (CORBA::is_nil(obj)) {
				throw string("Cannot get object");
			}

			CORBA::ORB_var orb;
			orb = corba_orb();
			if (CORBA::is_nil(orb)) {
				throw string("Cannot get ORB");
			}

			CORBA::String_var sior(orb->object_to_string(obj));
			zlog_debug("IOR = '%s'", (char*) sior);

			if (!corba_dump_ior(CORBA_SERVANT_SCNGW,string(sior))) {
				throw string("Cannot dump IOR");
			}

			servant->_remove_ref();

			PortableServer::POAManager_var poa_manager;
			poa_manager = corba_poa_manager();
			if (CORBA::is_nil(poa_manager)) {
				throw string("Cannot get POA Manager");
			}

			poa_manager->activate();
		} catch (CORBA::SystemException & e) {
			zlog_err("Caught CORBA::SystemException");
			return 0;
		} catch (CORBA::Exception & e) {
			zlog_err("Caught CORBA::Exception");
			return 0;
		} catch (omniORB::fatalException & e) {
			zlog_err("Caught omniORB::fatalException:");
			zlog_err("  file: %s", e.file());
			zlog_err("  line: %d", e.line());
			zlog_err("  mesg: %s", e.errmsg());
			return 0;
		} catch (string & e) {
			zlog_err("Caught exception: %s", e.c_str());
			return 0;
		} catch (...) {
			zlog_err("Caught unknown exception");
			return 0;
		}
		return 1;
	}

	int corba_signal_activity(void)
	{
		zlog_debug("Signaling activity");
		try {
			lrm_proxy->scngw_isup();
		} catch (...) {
			zlog_err("Problems");
			return 0;
		}
		zlog_debug("Done");

		return 1;
	}

	int corba_server_shutdown(void)
	{
	try {
		zlog_debug("Shutting down CORBA server side");

		if (!corba_remove_ior(CORBA_SERVANT_SCNGW)) {
			throw string("Cannot remove IOR");
		}
	} catch (...) {
		zlog_err("Caught unknown exception");
		return 0;
	}

		return 1;
	}
}

#endif // HAVE_OMNIORB
