/*
 *  This file is part of phosphorus-g2mpls.
 *
 *  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Authors:
 *
 *  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
 *  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
 *  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
 *  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
 *  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
 */




#ifndef _ZEBRA_SCNGWC_H
#define _ZEBRA_SCNGWC_H

#ifdef __cplusplus
extern "C" {
#endif

#include "stream.h"

#ifndef IPPROTO_OSPFIGP
#define IPPROTO_OSPFIGP         89
#endif /* IPPROTO_OSPFIGP */

#ifndef IPPROTO_RSVP
#define IPPROTO_RSVP            46
#endif /* IPPROTO_RSVP */

#define SCNGW_IP_TTL           100

#define SCNGW_HDR_SIZE         32U /* bytes */
#define REGISTRATION_MSG_SIZE  20U /* bytes */

#define PACKET_MESSAGE          1U
#define ACK_MESSAGE             2U
#define NACK_MESSAGE            3U
#define REGISTRATION_MESSAGE    4U

#define NO_TUNNEL                0 /* want no encapsulation in SCNGW server */
#define TUNNEL                   1 /* want encapsulation in SCNGW server */

#define WANT_NO_ACK              0 /* want no response on packet management from SCNGW */
#define WANT_ACK                 1 /* want response on packet management from SCNGW */

#define PACKETS_QUEUE_SIZE     200 /* number of packets */

#define TIMEOUT_ACK_DEFUALT   500L  /* msec */

/* structure containing the SCNGW header parameters */
struct scngw_hdr {
	u_int32_t           msg_type;  /* Message type */
	u_int32_t           hdr_len;   /* header length (bytes) */
	u_int32_t           sdu_len;   /* SDU length (bytes) */
	u_int32_t           msg_id;    /* Message ID */
	u_int32_t           flags;     /* flags used by OSPF */
        u_int32_t           src_addr;  /* TE-link local address */
	u_int32_t           dst_addr;  /* TE-link remote address */
	u_int32_t           cc;        /* Control channel */
};

typedef enum client_type {
	CLIENT_G2OSPFTE,
	CLIENT_G2RSVPTE,
	CLIENT_GUNIRSVP,
	CLIENT_GENNIRSVP
} client_type_t;

struct client_data {
	int                 fd;
	int                 proto;
	int                 encap;
	int                 want_ack;
	struct stream_fifo  *queue;
};

/* Protoypes */
extern int scngwc_init (int, int, int, int);
extern int scngwc_sendmsg (int, const void *, u_int16_t, struct in_addr, struct in_addr, int, int *);
extern int scngwc_stream_recvmsg (void *, int, struct ip **, size_t);
extern void scngwc_close (int);


#ifdef __cplusplus
}
#endif

#endif /* _ZEBRA_WRAP_H */
