/*
 *  This file is part of phosphorus-g2mpls.
 *
 *  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Authors:
 *
 *  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
 *  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
 *  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
 *  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
 *  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
 */



#ifndef _ZEBRA_SCNGWS_H
#define _ZEBRA_SCNGWS_H

#include "stream.h"
#include "linklist.h"
#include "scngws_packet.h"

#define SCNGWS_PORT                                 50000
#define OSPF_BASE_PORT                 SCNGWS_PORT + 8900
#define G2RSVPTE_BASE_PORT             SCNGWS_PORT + 4600
#define GUNIRSVP_BASE_PORT             SCNGWS_PORT + 5600
#define GENNIRSVP_BASE_PORT            SCNGWS_PORT + 6600

#define SCNGWSD_VTY_PORT                             2620
//#define SCNGWS_VTYSH_PATH   "/var/run/quagga/scngwsd.vty"
#define SCNGWS_CODE_VERSION "0.1"

#define STATUS_DOWN                                     0
#define STATUS_UP                                       1
#define STATUS_SYNC                                     2
#define STATUS_NEVER_CONN                               3
#define STATUS_NEW                                      4

#define SCN_INNI                                        0
#define SCN_ENNI                                        1
#define SCN_UNI                                         2

#define TIMEOUT_LRMSYNC_DEFUALT                     7500L  /* msec */

/* Default configuration file name for scngwsd. */
#define SCNGWS_DEFAULT_CONFIG              "scngwsd.conf"

#ifdef GNU_LINUX
#define WANT_SCNGW_WRITE_FRAGMENT
#endif

/* SCNGWs  master for system wide configuration and variables. */
struct scn_master {

	/* SCNGW thread master. */
	struct thread_master *master;
	struct zlist *client_list;

	/* Thread for END_SYNC timeout in LRM communication */
	struct thread *timer_thread;

	/* Status of the connection with LRM */
	int lrm_conn_status;

	/* Timeout for synchronization phase with LRM (sec) */
	long int timeout_lrmsync;

	/* SCNGWs start time. */
	time_t start_time;
};

/* Structure for the SCNGWs client. */
struct scn_client {

	/* Client protocol */
	int proto;
	/* Client interface type */
	int interface_type;
	/* Encapsulation */
	int encap;
	/* ACK / NO ACK */
	int want_ack;
	/* Number of packets sent */
	int pckts_sent;
	/* Number of packets received */
	int pckts_rcvd;

	/* Socket */
	int fd_cl;
	int fd_net;

	/* Input buffers*/
	struct stream *ibuf_cl;
	/* Output queues. */
	struct scngws_fifo *obufq_cl;
	struct scngws_fifo *obufq_net;

	/* threads. */
	struct thread *t_read_cl;
	struct thread *t_write_cl;
	struct thread *t_read_net;
	struct thread *t_write_net;

};
/* Structure containing one SCN-if */
struct scnif {

	int status;                /* Status of SCN-if */

	struct in_addr loc_addr;   /*local SCN-if address*/

};

/* Structure containing one TE-link/CC association */
struct tel_cc_assoc {

	int status;               /* Status of the TE-link */

	u_int16_t interface_type;  /* INNI / ENNI / UNI */

	u_int32_t key;            /* key used to update association */

	struct in_addr tel_loc;   /* local TE-link address*/

	struct in_addr tel_rem;   /* remote TE-link address*/

	struct zlist *cclist;      /* control channels (id) associated*/
};

/* Structure containing one CC/SCN-if association */
struct cc_scnif_assoc {

	int status;               /* Status of the CC */

	u_int32_t cc_id;          /* control channel id*/

	struct in_addr scnif_loc; /*local SCN-interface address*/
	struct in_addr scnif_rem; /*remote SCN-interface address*/
};

/* SCN-if structure */
struct scn_if_addrs {
	struct in_addr loc_addr;  /*local SCN-interface address*/
	struct in_addr rem_addr;  /*remote SCN-interface address*/
	int mtu;
};

/* Protoypes */
extern void scngws_master_init (void);
extern struct scngws *scngw_server_init (char *);
extern int server_sock_init (void);
extern int client_set_thread_read_net(int, int, struct thread *);
extern int accept_sock (struct thread *);
extern void scngws_client_close (struct scn_client *);

#endif /*_ZEBRA_SCNGWS_H */
