/*
 *  This file is part of phosphorus-g2mpls.
 *
 *  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Authors:
 *
 *  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
 *  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
 *  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
 *  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
 *  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
 */



#include "stream.h"
#include "thread.h"

#ifndef _ZEBRA_SCNGWS_PACKET_H
#define _ZEBRA_SCNGWS_PACKET_H

#define MAX_PACKET_SIZE       65535U

/*structure containing the client packet to send with additional useful parameters*/
struct scngws_packet {

	struct scngws_packet *next;
	struct stream *s;

	int flags;                  /* flags used by OSPF */

	struct in_addr src_addr;    /* TE-link local address */
	struct in_addr dst_addr;    /* TE-link remote address */

	int cc;                     /* Control Channel */
	u_int32_t length;	    /* Packet length (bytes) */
	int client_type;            /* Type of client */
	int msg_id;                 /* Message ID */
};

/* FIFO queue for SCNGWS packets */
struct scngws_fifo {
	u_int32_t count;

	struct scngws_packet *head;
	struct scngws_packet *tail;
};


/* Protoypes */
extern int scngwc_packet_send (struct thread *);
extern int scngwc_packet_read (struct thread *);
extern int scngws_recv_hdr (int, struct stream *, u_int32_t, int);
extern int scngws_send_hdr (int, struct stream *, u_int32_t, int);
extern int net_packet_send (struct thread *);
extern int net_packet_read (struct thread *);
extern struct scngws_fifo* scngws_fifo_new(void);
extern void scngws_fifo_flush (struct scngws_fifo *);


#endif /* _ZEBRA_SCNGWS_PACKET_H  */
