/*
 *  This file is part of phosphorus-g2mpls.
 *
 *  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Authors:
 *
 *  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
 *  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
 *  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
 *  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
 *  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
 */



#include <zebra.h>
#include "vty.h"
#include "command.h"
#include "linklist.h"
#include "stream.h"
#include "log.h"
#include "scngwc.h"
#include "scngws.h"
#include "scngws_lrm.h"
#include "scngws_vty.h"

/* SCNGWs master of the threads. */
extern struct scn_master *scn_m;

/* SCNGWs process wide configuration pointer. */
extern struct thread_master *master;

/* SCNGWs tel_cc and cc_scnif associations lists*/
/*extern struct zlist *scnif_list;*/
/*extern struct zlist *tel_cc_list;*/
/*extern struct zlist *cc_scnif_list;*/

struct cmd_node scngws_node = {
	SCNGWS_NODE,
	"%s(config-scngws)# ",
	1
};

DEFUN(scngws,
      scngws_cmd,
      "scngws",
      "Enable a SCNGWS process\n"
      "Start SCNGWS configuration\n")
{
	vty->node  = SCNGWS_NODE;
	return CMD_SUCCESS;
}

static void
scngws_scnif_show_sub (struct vty *vty, struct scnif *scn_if)
{
	vty_out (vty,"\t  %s ", inet_ntoa(scn_if->loc_addr));
	switch (scn_if->status) {
		case STATUS_UP:
			vty_out (vty,"(status UP) %s", VTY_NEWLINE);
			break;
		case STATUS_DOWN:
			vty_out (vty,"(status DOWN) %s", VTY_NEWLINE);
			break;
	}
	vty_out (vty,"%s", VTY_NEWLINE);
}

DEFUN(scngws_scnif_show,
      scngws_scnif_show_cmd,
      "show scngws scn-if",
      "type \"show scngws ?\" for available options")
{
	struct zlistnode *scnif_node, *scnif_nnode;
	struct scnif *scn_if;

	struct zlist *scnif_list;
	scnif_list = get_scnif_list();

	if (scn_m->lrm_conn_status == STATUS_NEVER_CONN) {
		vty_out (vty, "Communication with LRM in status NEVER_CONN: there are no SCN-ifs, TE-links and CCs.%s", VTY_NEWLINE);
		return CMD_SUCCESS;
	}
	/* Show All SCN-ifs. */
	vty_out (vty, "Number of SCNI-if: %d.%s", scnif_list->count, VTY_NEWLINE);
	for (ALL_LIST_ELEMENTS (scnif_list, scnif_node, scnif_nnode, scn_if)) {
		scngws_scnif_show_sub (vty, scn_if);
	}

	return CMD_SUCCESS;
}


static struct cc_scnif_assoc *
cc_lookup_by_id (u_int32_t cc_id)
{
	struct zlistnode *ccscnif_node, *ccscnif_nnode;
	struct cc_scnif_assoc *cc_scnif;
	struct zlist *cc_scnif_list;

	cc_scnif_list = get_cc_scnif_list();

	for (ALL_LIST_ELEMENTS (cc_scnif_list, ccscnif_node, ccscnif_nnode, cc_scnif)) {
		if (cc_scnif->cc_id == cc_id)
			return cc_scnif;
	}
	return NULL;
}

static void
scngws_cc_show_sub (struct vty *vty, struct cc_scnif_assoc *cc_scnif)
{
	vty_out (vty,"CC id: %d ", cc_scnif->cc_id);
	switch (cc_scnif->status) {
		case STATUS_UP:
			vty_out (vty,"(status UP) %s", VTY_NEWLINE);
			break;
		case STATUS_DOWN:
			vty_out (vty,"(status DOWN) %s", VTY_NEWLINE);
			break;
	}
	vty_out (vty,"\t SCN-if local address : %s %s", inet_ntoa(cc_scnif->scnif_loc), VTY_NEWLINE);
	vty_out (vty,"\t SCN-if remote address: %s %s", inet_ntoa(cc_scnif->scnif_rem), VTY_NEWLINE);
	vty_out (vty,"%s", VTY_NEWLINE);
}

DEFUN(scngws_cc_show,
      scngws_cc_show_cmd,
      "show scngws cc [ID]",
      "type \"show scngws ?\" for available options")
{
	struct zlistnode *ccscnif_node, *ccscnif_nnode;
	struct cc_scnif_assoc *cc_scnif;
	struct zlist *cc_scnif_list;
	u_int32_t id;

	cc_scnif_list = get_cc_scnif_list();

	if (scn_m->lrm_conn_status == STATUS_NEVER_CONN) {
		vty_out (vty, "Communication with LRM in status NEVER_CONN: there are no TE-links and CCs.%s", VTY_NEWLINE);
		return CMD_SUCCESS;
	}
	/* Show All CCs. */
	if (argc == 0) {
		vty_out (vty, "Number of CC: %d.%s", cc_scnif_list->count, VTY_NEWLINE);
		for (ALL_LIST_ELEMENTS (cc_scnif_list, ccscnif_node, ccscnif_nnode, cc_scnif)) {
			scngws_cc_show_sub (vty, cc_scnif);
		}
	}
	/* CC ID is specified. */
	else {
		id = (u_int32_t) atoi(argv[0]);
		if ((cc_scnif = cc_lookup_by_id (id)) == NULL)
			vty_out (vty, "No such CC id.%s", VTY_NEWLINE);
		else
			 scngws_cc_show_sub (vty, cc_scnif);
	}

	return CMD_SUCCESS;
}

static void
scngws_tel_show_sub (struct vty *vty, struct tel_cc_assoc *tel_cc)
{
	struct zlistnode *cc_node, *cc_nnode;
        u_int32_t *cc_id;

	vty_out (vty,"TE-link key: %d (%s)", tel_cc->key, if_type_string(tel_cc->interface_type));
	switch (tel_cc->status) {
		case STATUS_UP:
			vty_out (vty,"(status UP) %s", VTY_NEWLINE);
			break;
		case STATUS_DOWN:
			vty_out (vty,"(status DOWN) %s", VTY_NEWLINE);
			break;
	}
	vty_out (vty,"\t TE-link local address: %s %s", inet_ntoa(tel_cc->tel_loc), VTY_NEWLINE);
	vty_out (vty,"\t\t Associated CCs:%s", VTY_NEWLINE);
	for (ALL_LIST_ELEMENTS (tel_cc->cclist, cc_node, cc_nnode, cc_id)) {
		vty_out(vty,"\t\t\t CC id :%d%s", *cc_id, VTY_NEWLINE);
	}
	vty_out (vty,"%s", VTY_NEWLINE);
}

DEFUN(scngws_tel_all_show,
      scngws_tel_all_show_cmd,
      "show scngws te-link",
      "type \"show scngws ?\" for available options")
{
	struct zlistnode *telcc_node, *telcc_nnode;
	struct tel_cc_assoc *tel_cc;
	struct zlist *tel_cc_list;

	tel_cc_list = get_tel_cc_list();

	if (scn_m->lrm_conn_status == STATUS_NEVER_CONN) {
		vty_out (vty, "Communication with LRM in status NEVER_CONN: there are no TE-links and CCs.%s", VTY_NEWLINE);
		return CMD_SUCCESS;
	}
	/* Show All TE-links */
	vty_out (vty, "Number of TE-link: %d.%s", tel_cc_list->count, VTY_NEWLINE);
	for (ALL_LIST_ELEMENTS (tel_cc_list, telcc_node, telcc_nnode, tel_cc)) {
		scngws_tel_show_sub (vty, tel_cc);
	}
	return CMD_SUCCESS;

}

DEFUN(scngws_tel_show,
      scngws_tel_show_cmd,
      "show scngws te-link (inni|enni|uni)",
      "type \"show scngws ?\" for available options")
{
	struct zlistnode *telcc_node, *telcc_nnode;
	struct tel_cc_assoc *tel_cc;
	struct zlist *tel_cc_list;
	int count;

	tel_cc_list = get_tel_cc_list();

	count = 0;

	if (scn_m->lrm_conn_status == STATUS_NEVER_CONN) {
		vty_out (vty, "Communication with LRM in status NEVER_CONN: there are no TE-links and CCs.%s", VTY_NEWLINE);
		return CMD_SUCCESS;
	}
	/* Show TE-links by interface type. */
	/* INNI */
	if(strcmp(argv[0],"inni") == 0){
		for (ALL_LIST_ELEMENTS (tel_cc_list, telcc_node, telcc_nnode, tel_cc)) {
			if (tel_cc->interface_type == SCN_INNI) {
				scngws_tel_show_sub (vty, tel_cc);
				count ++;
			}
		}
		vty_out (vty, "There are %d INNI TE-links.%s", count, VTY_NEWLINE);
		return CMD_SUCCESS;
	}
	/* ENNI */
	if(strcmp(argv[0],"enni") == 0){
		for (ALL_LIST_ELEMENTS (tel_cc_list, telcc_node, telcc_nnode, tel_cc)) {
			if (tel_cc->interface_type == SCN_ENNI) {
				scngws_tel_show_sub (vty, tel_cc);
				count ++;
			}
		}
		vty_out (vty, "There are %d ENNI TE-links.%s", count, VTY_NEWLINE);
		return CMD_SUCCESS;
	}
	/* UNI */
	if(strcmp(argv[0],"uni") == 0){
		for (ALL_LIST_ELEMENTS (tel_cc_list, telcc_node, telcc_nnode, tel_cc)) {
			if (tel_cc->interface_type == SCN_UNI) {
				scngws_tel_show_sub (vty, tel_cc);
				count ++;
			}
		}
		vty_out (vty, "There are %d UNI TE-links.%s", count, VTY_NEWLINE);
		return CMD_SUCCESS;
	}
	return CMD_WARNING;
}

static void
scngws_client_show_sub (struct vty *vty, struct scn_client *client)
{
	switch (client->proto) {
		case IPPROTO_OSPFIGP:
			switch (client->interface_type) {
				case SCN_INNI:
					vty_out (vty, "\tOSPF-INNI client");
					break;
				case SCN_ENNI:
					vty_out (vty, "\tOSPF-ENNI client");
					break;
				case SCN_UNI:
					vty_out (vty, "\tOSPF-UNI client");
					break;
			}
			break;
		case IPPROTO_RSVP:
			switch (client->interface_type) {
				case SCN_INNI:
					vty_out (vty, "\tRSVP-INNI client");
					break;
				case SCN_ENNI:
					vty_out (vty, "\tRSVP-ENNI client");
					break;
				case SCN_UNI:
					vty_out (vty, "\tRSVP-UNI client");
					break;
			}
			break;
	}
	vty_out (vty, "%s", VTY_NEWLINE);
	if (client->encap == TUNNEL) {
		vty_out (vty, "\t\tIPIP Encapsulation%s", VTY_NEWLINE);
	}
	else {
		vty_out (vty, "\t\tNo Encapsulation%s", VTY_NEWLINE);
	}
	if (client->want_ack == WANT_ACK) {
		vty_out (vty, "\t\tWants ACK from SCNGW server%s", VTY_NEWLINE);
	}
	else {
		vty_out (vty, "\t\tNo ACK from SCNGW server%s", VTY_NEWLINE);
	}
	vty_out (vty, "\t\tPackets sent    : %d%s", client->pckts_sent, VTY_NEWLINE);
	vty_out (vty, "\t\tPackets received: %d%s", client->pckts_rcvd, VTY_NEWLINE);
	vty_out (vty, "%s", VTY_NEWLINE);
}

DEFUN(scngws_clients_show,
      scngws_clients_show_cmd,
      "show scngws clients",
      "type \"show scngws ?\" for available options")
{
	struct zlistnode *client_node, *client_nnode;
	struct scn_client *client;

	vty_out (vty, "Number of clients registered on SCNGW server: %d.%s", scn_m->client_list->count, VTY_NEWLINE);
	for (ALL_LIST_ELEMENTS (scn_m->client_list, client_node, client_nnode, client)) {
		scngws_client_show_sub (vty, client);
	}
	return CMD_SUCCESS;
}

DEFUN(scngws_lrm_conn_status,
      scngws_lrm_conn_status_cmd,
      "show scngws lrm-conn-status",
      "type \"show scngws ?\" for available options")
{
	switch (scn_m->lrm_conn_status) {
		case STATUS_NEVER_CONN:
			vty_out (vty, "Communication with LRM is in status NEVER_CONN.%s", VTY_NEWLINE);
			return CMD_SUCCESS;
		case STATUS_DOWN:
			vty_out (vty, "Communication with LRM is in status DOWN.%s", VTY_NEWLINE);
			return CMD_SUCCESS;
		case STATUS_UP:
			vty_out (vty, "Communication with LRM is in status UP.%s", VTY_NEWLINE);
			return CMD_SUCCESS;
		case STATUS_SYNC:
			vty_out (vty, "Communication with LRM is in status SYNC.%s", VTY_NEWLINE);
			return CMD_SUCCESS;
		default:
			vty_out (vty, "Status unknown.%s", VTY_NEWLINE);
			return CMD_SUCCESS;
	}
}

DEFUN(scngws_timeout_sync,
      scngws_timeout_sync_cmd,
      "timeout lrm-sync [msec]",
      "Set timeout for synchronization with LRM")
{
	long int timeout;

	/* Timeout value is not specified. */
	if (argc == 0) {
		scn_m->timeout_lrmsync = TIMEOUT_LRMSYNC_DEFUALT;
		vty_out (vty, "Timeout for synchronization phase in SCNGWS-LRM intercommunication set to default value: %ld msec %s", TIMEOUT_LRMSYNC_DEFUALT, VTY_NEWLINE);
		return CMD_SUCCESS;
	}
	/* Timeout value is specified. */
	else {
		timeout = strtoul(argv[0], NULL, 10);
		scn_m->timeout_lrmsync = timeout;
		vty_out (vty, "Timeout for synchronization phase in SCNGWS-LRM intercommunication set to: %ld msec %s", timeout, VTY_NEWLINE);
		return CMD_SUCCESS;
	}
}

void scngws_vty_init ()
{
	/* Install scngws top node. */
	install_node (&scngws_node, NULL);

	install_element(CONFIG_NODE, &scngws_cmd);

	install_default(SCNGWS_NODE);

	/* Timeout commands  */
	install_element(SCNGWS_NODE, &scngws_timeout_sync_cmd);

	/* Show commands */
	install_element(VIEW_NODE, &scngws_scnif_show_cmd);
	install_element(ENABLE_NODE, &scngws_scnif_show_cmd);
	install_element(SCNGWS_NODE, &scngws_scnif_show_cmd);
	install_element(VIEW_NODE, &scngws_cc_show_cmd);
	install_element(ENABLE_NODE, &scngws_cc_show_cmd);
	install_element(SCNGWS_NODE, &scngws_cc_show_cmd);
	install_element(VIEW_NODE, &scngws_tel_show_cmd);
	install_element(ENABLE_NODE, &scngws_tel_show_cmd);
	install_element(SCNGWS_NODE, &scngws_tel_show_cmd);
	install_element(VIEW_NODE, &scngws_tel_all_show_cmd);
	install_element(ENABLE_NODE, &scngws_tel_all_show_cmd);
	install_element(SCNGWS_NODE, &scngws_tel_all_show_cmd);
	install_element(VIEW_NODE, &scngws_clients_show_cmd);
	install_element(ENABLE_NODE, &scngws_clients_show_cmd);
	install_element(SCNGWS_NODE, &scngws_clients_show_cmd);
	install_element(VIEW_NODE, &scngws_lrm_conn_status_cmd);
	install_element(ENABLE_NODE, &scngws_lrm_conn_status_cmd);
	install_element(SCNGWS_NODE, &scngws_lrm_conn_status_cmd);

}
