import os, re
from traceback import print_exc


#==================================================================================================

TEXT = """
 call: CALLIDTYPE_OPSPEC#(ipv4#192.168.100.10):0x1
   iTnaRes: TNA=ipv4#40.40.40.1; DL=ipv4#0.0.0.0; L=l32#0x00000000
   eTnaRes: TNA=ipv4#20.20.20.1; DL=ipv4#0.0.0.0; L=l32#0x00000000
   iGnsTna: None
   eGnsTna: {'dataStaging': {'target': '', 'cf': 'GRIDSTAGINGCF_UNKNOWN', 'fileName': '', 'source': '', 'delOnTermination': False, $
   callero:
     calleroitem: N=192.168.100.10; TEL=None; uDL=None; dDL=None; uL=None; dL=None; l=False
       rbitem: 192.168.100.10-192.168.100.8:0x0001
         rb-lsp: 192.168.100.10:0x0001-192.168.100.8:0x0001:0x00000000
           lsperoitem: N=0.0.0.0; TEL=ipv4#9.10.1.1; uDL=None; dDL=None; uL=None; dL=None; l=False
           lsperoitem: N=0.0.0.0; TEL=ipv4#8.9.1.1; uDL=None; dDL=None; uL=None; dL=None; l=False
     calleroitem: N=0.0.0.0; TEL=ipv4#8.9.1.1; uDL=None; dDL=None; uL=None; dL=None; l=False
"""
TEXT2 = """
  call: CALLIDTYPE_OPSPEC#(ipv4#192.168.100.137):0x1
   iTnaRes: TNA=ipv4#10.10.10.1; DL=ipv4#0.0.0.0; L=l32#0x00000000
   eTnaRes: TNA=ipv4#20.20.20.1; DL=ipv4#0.0.0.0; L=l32#0x00000000
   iGnsTna: None
   eGnsTna: None
   callero:
     calleroitem: N=192.168.100.137; TEL=None; uDL=None; dDL=None; uL=None; dL=None; l=False
       rbitem: 192.168.100.137-192.168.100.139:0x0001
         rb-lsp: 192.168.100.137:0x0001-192.168.100.139:0x0001:0x00000000
           lsperoitem: N=0.0.0.0; TEL=ipv4#1.3.1.2; uDL=None; dDL=None; uL=None; dL=None; l=False
     calleroitem: N=192.168.100.139; TEL=ipv4#0.0.0.0; uDL=None; dDL=None; uL=None; dL=None; l=False
     calleroitem: N=0.0.0.0; TEL=ipv4#6.9.1.2; uDL=None; dDL=None; uL=None; dL=None; l=False
       rbitem: 192.168.100.9-192.168.100.8:0x0001
         rb-lsp: 192.168.100.9:0x0001-192.168.100.8:0x0001:0x00000000
           lsperoitem: N=0.0.0.0; TEL=ipv4#9.10.1.2; uDL=None; dDL=None; uL=None; dL=None; l=False
           lsperoitem: N=0.0.0.0; TEL=ipv4#8.10.1.1; uDL=None; dDL=None; uL=None; dL=None; l=False
     calleroitem: N=192.168.100.8; TEL=ipv4#0.0.0.0; uDL=None; dDL=None; uL=None; dL=None; l=False
"""
TEXT3 = """

 call: CALLIDTYPE_OPSPEC#(ipv4#192.168.100.137):0x1
   iTnaRes: TNA=ipv4#10.10.10.1; DL=ipv4#0.0.0.0; L=l32#0x00000000
   eTnaRes: TNA=ipv4#20.20.20.1; DL=ipv4#0.0.0.0; L=l32#0x00000000
   iGnsTna: None
   eGnsTna: None
   callero:
     calleroitem: N=192.168.100.137; TEL=None; uDL=None; dDL=None; uL=None; dL=None; l=False
       rbitem: 192.168.100.137-192.168.100.139:0x0001
     calleroitem: N=192.168.100.139; TEL=ipv4#0.0.0.0; uDL=None; dDL=None; uL=None; dL=None; l=False
     calleroitem: N=0.0.0.0; TEL=ipv4#6.9.1.2; uDL=None; dDL=None; uL=None; dL=None; l=False
       rbitem: 192.168.100.9-192.168.100.8:0x0001
     calleroitem: N=192.168.100.8; TEL=ipv4#0.0.0.0; uDL=None; dDL=None; uL=None; dL=None; l=False

"""

#==================================================================================================
call_ero_patt = re.compile('calleroitem: N=(?P<node>[^;]*); TEL=(?P<telink>[^;]*); uDL=(?P<uDL>[^;]*); dDL=(?P<dDL>[^;]*); uL=(?P<uL>[^;]*); dL=(?P<dL>[^;]*); l=(?P<l>.*)')
rbitem_patt = re.compile('rbitem: (?P<first_node>[^-]*)-(?P<last_node>[^:]*):id=(?P<id>[.]*)')
rb_lsp_patt = re.compile('rb-lsp: (?P<first_node>[^:]*):(?P<first_id>[^-]*)-(?P<last_node>[^:]*):(?P<last_id>[^:]*):(?P<id>.*)')
lsp_ero_patt = re.compile('lsperoitem: N=(?P<node>[^;]*); TEL=(?P<telink>[^;]*); uDL=(?P<uDL>[^;]*); dDL=(?P<dDL>[^;]*); uL=(?P<uL>[^;]*); dL=(?P<dL>[^;]*); l=(?P<l>.*)')
iTna_patt = re.compile('iTnaRes: TNA=(?P<tna>[^;]*); DL=(?P<DL>[^;]*); L=(?P<label>.*)')
eTna_patt = re.compile('eTnaRes: TNA=(?P<tna>[^;]*); DL=(?P<DL>[^;]*); L=(?P<label>.*)')

addr_patt = re.compile('(?P<type>[^#]*)#(?P<addr>.*)')

#==================================================================================================

TElinks = {
    '2.20.1.1'   : '192.168.102.2',
    '2.20.1.2'   : '192.168.156.20',
    '3.30.1.1'   : '192.168.102.3',
    '3.30.1.2'   : '192.168.156.30',
    '4.40.1.1'   : '192.168.102.4',
    '4.40.1.2'   : '192.168.156.40',
            
    '1.2.1.1'    : '192.168.102.1',
    '1.2.1.2'    : '192.168.102.2',   
    '1.3.1.1'    : '192.168.102.1',
    '1.3.1.2'    : '192.168.102.3', 
    '2.3.1.1'    : '192.168.102.2',
    '2.3.1.2'    : '192.168.102.3', 
    '2.4.1.1'    : '192.168.102.2',
    '2.4.1.2'    : '192.168.102.4',   
    '1.4.1.1'    : '192.168.102.1',
    '1.4.1.2'    : '192.168.102.4', 
    '3.4.1.1'    : '192.168.102.3',
    '3.4.1.2'    : '192.168.102.4', 
        
}

#==================================================================================================

def parse_TNA(line, pattern):
    match = pattern.search(line)
    if match is None:
        return
    addr = match.groupdict().get('tna')
    addr_match = addr_patt.search(addr)
    if addr_match is None:
        return
    return addr_match.groupdict().get('addr')
    
def parse_eroitem(line, pattern):
    match_lsp = pattern.search(line)
    if not match_lsp:
        return
    node = match_lsp.groupdict().get('node')
    if node == '0.0.0.0':
        telink = match_lsp.groupdict().get('telink')
        addr_match = addr_patt.search(telink)
        if addr_match is None:
            return
        teaddr = addr_match.groupdict().get('addr')
        node = TElinks.get(teaddr, '0.0.0.0')
    return node

def parse_single_path(lines):
    path = {'id':'', 'iTna':None, 'eTna':None, 'call nodes':[], 'lsp nodes':[]}
    SEARCH_STATE = 'search_callID'

    for line in lines:
        if SEARCH_STATE == 'search_callID':
            if 'call: ' not in line:
                continue
            path['id'] = line[7:]
            SEARCH_STATE = 'search_iTNA'
            
        if SEARCH_STATE == 'search_iTNA':
            tna = parse_TNA(line, iTna_patt)
            if tna is None:
                continue
            path['iTna'] = tna
            SEARCH_STATE = 'search_eTNA'

        elif SEARCH_STATE == 'search_eTNA':
            tna = parse_TNA(line, eTna_patt)
            if tna is None:
                continue
            path['eTna'] = tna
            SEARCH_STATE = 'search_callOrLspEro'

        elif SEARCH_STATE == 'search_callOrLspEro':
            node = parse_eroitem(line, call_ero_patt)
            if node:
                path['call nodes'].append(node)
                continue
            
            rb_match = rb_lsp_patt.search(line)
            if rb_match:
                path['lsp nodes'].append(rb_match.groupdict().get('first_node'))
                continue
                
            node = parse_eroitem(line, lsp_ero_patt)
            if node:
                path['lsp nodes'].append(node)
                continue
                
    print ' - Path found:', path
    return path

def ispresent(path, pathes):
    for p in pathes:
        if p['id'] == path['id']:
            return True
    return False

def generate_path_infos(info):
    pathes = []
    line_number = 0
    prev_start_line = -1
    lines = info.splitlines()
    for line in lines:
        # next call found or last line
        if (' call: CALLIDTYPE_' in line) or (line_number == len(lines)-1):
            if prev_start_line != -1: # skip first call header, trigger on the next
                print 'Parsing next call'
                try:
                    path = parse_single_path(lines[prev_start_line:line_number+1])
                except:
                    print_exc()

                if ispresent(path, pathes) is False:
                    pathes.append(path)
                    yield path

            prev_start_line = line_number
        line_number += 1

def generate_pathes(text):
    pathes = []
    for path in generate_path_infos(text):
       if ispresent(path, pathes) is False:
           pathes.append(path)
       else:
           print ' = Path already found, skip it:', path

    return pathes

#==================================================================================================

def generate_telinks(data_telinks):
    telinks = []
    for line in data_telinks.splitlines():
        try:
            ip_addr, linkid_ipv4, oper, adm = line.split()
            rem_node = TElinks.get(linkid_ipv4, '0.0.0.0')
            telink = {'loc_node':ip_addr, 'rem_node':rem_node, 'oper':oper, 'adm':adm}
            telinks.append(telink)
        except:
            print_exc()
    return telinks 

#==================================================================================================
    
DOCUMENT_HTML_START = """
<html>
    <head>
        <meta http-equiv="Refresh" content="2" />
        <link rel="stylesheet" type="text/css" href="../css/style_main.css" />
        <meta http-equiv="content-type" content="text/html; charset=iso-8859-2" />
        <title>%(title)s</title>
    </head>
    <body>
"""
DOCUMENT_HTML_STOP = """
    </body>
</html>
"""

LINK            = '<a href="%(address)s">%(description)s</a>'
NEW_LINE        = '<br>'
BOLD            = '<b>%(text)s</b>'
START_PRE       = '<pre>'
STOP_PRE        = '</pre>'
HORIZONTAL_LINE = '<hr>'
LIST_ELEMENT    = '<li>%(text)s</li>'
LIST            = '<ul>%(list)s</ul>'

#==================================================================================================

class IO:
    def __init__(self):
        self.input = ''
    def write(self, data):
        self.input += data

def pathes_info(pathes):
    import pprint
    text_buffor = DOCUMENT_HTML_START
    text_buffor += START_PRE
    text_buffor += 'There are following pathes installed in G2MPLS Control Plane:\n\n'
    
    i = 0
    for path in pathes:
        text_buffor += 'Path #%i:\n' % i
        io = IO()
        pprint.pprint(path, io)
        text_buffor += io.input
        i += 1
        text_buffor += '\n\n'
    text_buffor += STOP_PRE
    text_buffor += DOCUMENT_HTML_STOP
    
    f = file('pathes_info.html', 'w')
    f.write(text_buffor)
    f.close()

#==================================================================================================
 
LINKS = {
    'client1-switch_8000_1'        :  ['192.168.156.20',  '192.168.102.2'],
    'switch_8000_1-switch_8000_3'  :  ['192.168.102.1',   '192.168.102.2'],
    'switch_8000_2-switch_9424T_1' :  ['192.168.102.3',   '192.168.102.4'],
    'switch_8000_1-switch_8000_2'  :  ['192.168.102.2',   '192.168.102.3'],
    'switch_8000_3-switch_9424T_1' :  ['192.168.102.1',   '192.168.102.4'  ],
    'switch_8000_2-switch_8000_3'  :  ['192.168.102.1',   '192.168.102.3' ],
    'switch_8000_1-switch_9424T_1' :  ['192.168.102.2',   '192.168.102.4' ],
    'switch_8000_2-server1'        :  ['192.168.102.3',   '192.168.156.30' ],
    'switch_9424T_1-server2'       :  ['192.168.102.4',   '192.168.156.40'  ],
}

TNA_LINKS = {
    '20.20.20.1'     : 'client1-switch_8000_1',
    '30.30.30.1'     : 'switch_8000_2-server1',
    '40.40.40.1'     : 'switch_9424T_1-server2'
}

#==================================================================================================

def create_topomap(pathes, telinks):
    """
    States:
    1		- 	Link			(black)
    20		-	Link disabled	(red)
    30		-	Link Down		(grey)n		(invisible-white)		(blue)
    """
    import copy
   
    #initialization
    topo_links = copy.deepcopy(LINKS)
    for k in LINKS:
        topo_links[k] = '1M'
	topo_links[k + '_call'] = '60M'
        
    def link_set_color(linkName, color):
        try:
            if linkName is None:
                return
            print 'Setting color %s for %s' % (color, linkName)
            topo_links[linkName] = color
        except Exception, e:
            print_exc()

    def search_link_name(node1, node2):
        for linkName, endPoints in LINKS.items():
            if node1 in endPoints and node2 in endPoints:
                return linkName
    
    for path in pathes:

        linkNameB1 = TNA_LINKS.get(path['iTna'])
        linkNameB2 = TNA_LINKS.get(path['eTna'])

        link_set_color(linkNameB1 + '_call','50M')
        link_set_color(linkNameB2 + '_call','50M')
        link_set_color(linkNameB1,'100M')
        link_set_color(linkNameB2,'100M')

        #present calls
        call = path['call nodes']
        while len(call) >= 2:
            node1, node2 = call[:2]
            del call[0]
            linkName = search_link_name(node1, node2)
            if linkName:
                link_set_color(linkName + '_call','50M')
        
        #present lsp        
        lsp = path['lsp nodes']
        while len(lsp) >= 2:
            node1, node2 = lsp[:2]
            del lsp[0]
            linkName = search_link_name(node1, node2)
            if linkName:
                link_set_color(linkName,'100M')

    # present telink status
    for telink in telinks:
        linkName = search_link_name(telink.get('loc_node'), telink.get('rem_node'))
        if linkName is None:
            continue
        if telink.get('adm') == 'ADMINSTATE_DISABLED':
            link_set_color(linkName,'20M')
        elif telink.get('oper') == 'OPERSTATE_DOWN':
            link_set_color(linkName,'30M')	        

    print topo_links
    f = file('dynamic_topo1.txt', 'w')    
    for linkName, color in topo_links.items():    
        f.write("%s\t1M\t%s\t#comm\n" % (linkName, color))
    f.close()
                                        
#==================================================================================================

if __name__ == '__main__':
    import sys
    try:
        # for pathes

        data_pathes = ''
        data_telinks = ''
        try:
              file_pathes, file_telinks = sys.argv[1:3]
        except:
              print_exc()

        try:
        	f1 = file(file_pathes, 'r')
        	data_pathes = f1.read()
		f1.close()
        except:
              print_exc()
              
        try:
        	f2 = file(file_telinks, 'r')
        	data_telinks = f2.read()
		print data_telinks
		f2.close()
        except:
              print_exc()
              
        #data_pathes = TEXT2
        pathes = generate_pathes(data_pathes)
        telinks = generate_telinks(data_telinks)
        pathes_info(pathes)
        create_topomap(pathes, telinks)
    except Exception, e:
        from traceback import print_exc
        print_exc()



