//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//



#include "g2rsvpted/g2rsvpte_config.h"
#include "g2rsvpted/g2rsvpte_lsp.h"
#include "g2rsvpted/g2rsvpted.h"
#include "g2rsvpted/g2rsvpte_apis.h"
#include "g2rsvpted/g2rsvpte_lrm_api.h"
#include "g2rsvpted/g2rsvpte_tnrc_api.h"
#include "g2rsvpted/g2rsvpte_rc_api.h"
#include "g2rsvpted/g2rsvpte_cc_api.h"
#ifdef PERSISTENCY_ON
#include "g2rsvpted/g2rsvpte_persistency_py.h"
#endif // PERSISTENCY_ON
#include "g2rsvpted/g2rsvpte_message_objects.h"
#include "g2rsvpted/g2rsvpte_message_constants.h"
#include "g2rsvpted/g2rsvpte_trace.h"

#define DUMP_STATE(STATE,EVENT)						   \
	std::cout << "STATE: '" << STATE << "' -> " << EVENT << std::endl;

#define DUMP_STATE2(STATE,EVENT,FROM_STATE)				   \
	std::cout << "* STATE: '" << STATE << "' -> after " << EVENT	   \
		  << " -> from " << FROM_STATE << std::endl;

//
// General
//

void
FSMtimerInit(RSVP::LSP * lsp)
{
	struct thread * thr;

	// Initialise timer for the FSM state

	G2RSVPTE_LSP_TIMER_context_t * timerCtxt;

	timerCtxt = new G2RSVPTE_LSP_TIMER_context_t;
	if (!timerCtxt) {
		G2RSVPTE_WRN("Impossible to initialize new timer in this FSM state");
		return;
	}
	timerCtxt->lsp = lsp;
	timerCtxt->adjType = lsp->getInstance()->adjType();

	thr = thread_add_timer(RSVP_MASTER.getMaster(),
			       lspFSMtimerElapsed,
			       timerCtxt,
			       LSP_FSM_TIMER);

	lsp->FSMtimerThr(thr);
}

void
FSMtimerDelete(RSVP::LSP * lsp)
{
	struct thread * thr;

	// Delete timer for the FSM state

	thr = lsp->FSMtimerThr();
	if (thr) {
		G2RSVPTE_LSP_TIMER_context_t * timerCtxt;
		timerCtxt = (G2RSVPTE_LSP_TIMER_context_t *) THREAD_ARG(thr);
		delete timerCtxt;

		thread_cancel(thr);
		lsp->FSMtimerThr(0);
	}
}

#ifdef CRANKBACK_ON
void
postNextCbackAttempt(RSVP::LSP * lsp)
{
	thread_add_event(RSVP_MASTER.getMaster(),
			 lspNextCbackAttemptStart,
			 lsp,
			 0);
	G2RSVPTE_DBG("Posted event to start next crankback attempt");
}

bool
prepareNextCbackAttempt(void * context)
{
	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	assert(ctxt->msg);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

	// post signalUpLsp event NOW only if
	// no release resource is being processed
	if (!lsp->cbackWaitReleaseRes()) {
		postNextCbackAttempt(lsp);
	}

	return true;
}
#endif // CRANKBACK_ON

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResv_t
handleFirstResv(void * context)
{
	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	assert(ctxt->msg);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->rsb());

	// XXXX FIXME: Check if msg is a ResvMessage
	//             Store parsed packet info into the LSP and check
	//             Set NOT_ERASE flags for this pkg
	lsp->rsb()->resvIn((RSVP::ResvMessage *)ctxt->msg);

	// Head node?
	bool headNode;
	headNode = lsp->isHead();

	// TODO: check if the outgoing resv parsed packet already points to a
	//       packet that is under retransmission in some ACK/NACK FSM

	RSVP::ResvMessage * tmpResv;

	if (headNode) {
		//lsp->rsb()->resvOut(lsp->rsb()->resvIn());
		tmpResv = static_cast<RSVP::ResvMessage *>(lsp->rsb()->resvIn()->clone());
	} else {
		// If not head node -> prepare skeleton of the outgoing parsed
		// Resv (copy the incoming to the outgoing and apply changes)

		if (lsp->rsb()->resvOut()) {
			G2RSVPTE_ERR("ARGHHHHH ... outgoing resv already exists!!!");
			lsp->toBeDestroyed(true);
			postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResv_to_InvalidEvent;
		}

		tmpResv = lsp->rsb()->resvIn()->partialClone();
	}

	lsp->rsb()->resvOut(tmpResv);

	if (!lsp->rsb()->resvOut()) {
		G2RSVPTE_ERR("Impossible to copy incoming resv ...");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResv_to_InvalidEvent;
	}

	if (lsp->rsb()->resvIn()->call_id_ && !lsp->processCallIdObj(false)) {
		G2RSVPTE_ERR("received CALL ID obj with problems !!!");

		//send a PathErr upstream
		if (!lsp->usAckNack() || !lsp->usAckNack()->errorHandle(G2RSVPTE_ERROR_SPEC_FLAG_PathStateRemoved,
									G2RSVPTE_ERROR_SPEC_CALL_MNGMT,
									G2RSVPTE_ERROR_SPEC_VAL_CALL_MNGMT_CALL_ID_CONTENTION)) {
			G2RSVPTE_ERR("could not send PathErr");
		}

		if ((lsp->getInstance()->adjType() == INNI) &&
		     lsp->tnrcCookie()) {
		        // Request a resource release at the TNRC
		        if (!releaseResource(lsp)) {
		                G2RSVPTE_ERR("Impossible to release resource");
		        }
			//set tnrcCookie to 0
			lsp->tnrcCookie(0);
		}
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResv_to_evRecvResvKo;
	}

	//
	// Process and check the NHOP obj in the incoming Resv
	//
	if (!lsp->processNHOPobj()) {
		G2RSVPTE_ERR("received NHOP obj with problems!!!");
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResv_to_evRecvResvKo;
	}

	//
	// Check that the label in LABEL obj matches the one sent in LABEL SET
	//
	if (!lsp->processLabelObj()) {
		G2RSVPTE_ERR("received LABEL obj with problems!!!");
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResv_to_evRecvResvVeryKo;
	}

	// Check that style is FF
	assert(lsp->rsb()->resvOut()->style_); // TMP???

	RSVP::Style_Class * style;
	style = lsp->rsb()->resvOut()->style_;

	if (style->cType() != CTYPE_STYLE) {
		G2RSVPTE_ERR("problems... STYLE CTYPE %d not supported",
			     style->cType());
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResv_to_evRecvResvVeryKo;
	}

	if (!(static_cast<RSVP::Style_Object *> (style))->isFF()) {
		G2RSVPTE_ERR("only Fixed Filter reservation style "
			     "is supported");

		//send a ResvErr downstream
		if (!lsp->dsAckNack() || !lsp->dsAckNack()->errorHandle(G2RSVPTE_ERROR_SPEC_FLAG_PathStateRemoved,
									G2RSVPTE_ERROR_SPEC_UNKNOWN_STYLE,
									0)) {
			G2RSVPTE_ERR("could not send ResvErr");
		}

		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResv_to_evRecvResvVeryKo;
	}

	G2RSVPTE_INF("Fixed Filter reservation selected");

	if (!headNode) {
		// prepare the NHOP in the outgoing Resv NHOP obj:
		if (!lsp->prepareNHOPobj()) {
			G2RSVPTE_ERR("problems preparing outgoing Resv NHOP obj");
			lsp->toBeDestroyed(true);
			postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResv_to_InvalidEvent;
		}

		// Prepare the label obj
		if (!lsp->prepareLabelObj()) {
			G2RSVPTE_ERR("problems preparing outgoing Resv LABEL obj");
			lsp->toBeDestroyed(true);
			postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResv_to_InvalidEvent;
		}
	}

	// Add this node to the RRO
	if (lsp->rsb()->resvOut()->flow_desc_              &&
	    lsp->rsb()->resvOut()->flow_desc_->getRROobj() &&
	    !lsp->insertIntoRRO(lsp->rsb()->resvOut()->flow_desc_->getRROobj(),
				lsp->rsb()->interface()->localId())) {
		G2RSVPTE_ERR("problems preparing outgoing Resv RRO obj");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResv_to_InvalidEvent;
	}

	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResv_to_evRecvResvOk;
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathErr_t
handleRecvPathErr(void * context)
{
	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	assert(ctxt->msg);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->psb());
	assert(lsp->rsb());
	assert(lsp->getInstance());

	// Check that the incoming msg is a PathErr
	if (ctxt->msg->getType() != RSVP::Message::PATH_ERR) {
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPathErr_to_evRecvPathErrKo;
	}
	RSVP::PathErrMessage * pathErr = static_cast<RSVP::PathErrMessage *>(ctxt->msg);
	// XXXX FIXME: Check if pathErr msg is ok

	// TODO: check if the outgoing parsed packet already points to a
	//       packet that is under retransmission in some ACK/NACK FSM

	// Get the error
	error_info_t              error;
	std::list<lsp_ero_sobj_t> excludeEro;

	memset(&error,0, sizeof(error));
	excludeEro.clear();

	if (!pathErr->error_spec_ ||
	    !pathErr->getErrorDetails(error, excludeEro)) {
		G2RSVPTE_ERR("Invalid received PathErr msg");
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPathErr_to_evRecvPathErrKo;
	}

	// TODO create a macro ERRCODE2STRING and ERRVAL2STRING for debugging
	G2RSVPTE_INF("PathErr: error %d, %d at NE %s",
		     error.err_code, error.err_val, numb_htoa(error.node_id));

	// Head node?
	bool headNode;
	headNode = lsp->isHead();

#ifdef CRANKBACK_ON
	if ( headNode                              &&
	     lsp->getInstance()->adjType() == INNI &&
	    (lsp->cbackScope() != CB_NONE)         &&
	    (excludeEro.size() > 0)) {
		G2RSVPTE_INF("         Crankback info:");
		std::list<lsp_ero_sobj_t>::iterator it;
		lsp_ero_sobj_t                      sObj;
		for (it = excludeEro.begin(); it != excludeEro.end(); it++) {
			memset(&sObj, 0, sizeof(sObj));
			sObj = *it;
			switch (sObj.type) {
				case ERO_SUBOBJ_NODE:
					G2RSVPTE_INF("              Node    %s",
						     numb_htoa(sObj.hop.node.node_id));
					break;
				case ERO_SUBOBJ_IPV4:
					G2RSVPTE_INF("              TE-link %s",
						     numb_htoa(sObj.hop.ipv4.addr.s_addr));
					break;
				case ERO_SUBOBJ_LABEL:
					G2RSVPTE_INF("              Label   0x%x",
						     sObj.hop.label.value);
					break;
				default:
					G2RSVPTE_ERR("Invalid exclude Ero subobject type");
					return fsm::base_G2RSVPTE_LSP_FSM::
						G2RSVPTE_LSP_FSM_from_RecvPathErr_to_evRecvPathErrKo;
			}
		}
		// check number of cnback attempts
		if (lsp->currentCbackAttempt() ==
		    lsp->maxCbackRetries()) {
			G2RSVPTE_ERR("Reached maximum number of"
				     "crankback retries attempts (%d)",
				     lsp->maxCbackRetries());
			postLspErrEvent(lsp, true);
		} else {
			lsp->attachExcludeEro(excludeEro);
		}

		// now we can throw away the received PathErr msg
		G2RSVPTE_DBG("Deleting the received PathErr msg");
		delete pathErr;

		if (lsp->tnrcCookie()) {
			// Request a resource release at the TNRC
			if (!releaseResource(lsp)) {
				G2RSVPTE_ERR("Impossible to release resource");
			}
			//set tnrcCookie to 0
			lsp->tnrcCookie(0);
			if (lsp->currentCbackAttempt() <
			    lsp->maxCbackRetries()) {
				//set cback wait release resource flag
				G2RSVPTE_DBG("Wait for release resource final response before trying "
					     "next crankback attempt");
				lsp->cbackWaitReleaseRes(true);
			}
		}

		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPathErr_to_evRecvPathErrOk;
	}
#endif // CRANKBACK_ON

	// TODO future (crankback) here ???
	//return G2RSVPTE_LSP_FSM_from_RecvPathErr_to_evRecvPathErrCrankback;

	if (headNode) {
		if (lsp->getInstance()->adjType() == INNI) {
			// notify an Err Event to RC
			G2RSVPTE_INF("posting an Err Event to RC");
			postLspErrEvent(lsp, true);
		} else {
			lsp->psb()->pathErr(static_cast<RSVP::PathErrMessage *>(pathErr->clone()));
			// notify an Err Event to CC
			G2RSVPTE_INF("posting a RecvPathErrEvent to CC");
			notifyLspRecvPathErrEvent(lsp);
		}

	} else {
		if (lsp->getInstance()->adjType() == INNI) {
			// Build the PathErr msg from the PSB PathOut
			if (!lsp->psb()->pathOut()) {
				G2RSVPTE_ERR("Invalid PathOut msg");
				return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPathErr_to_InvalidEvent;
			}

			if (lsp->psb()->pathErr()) {
				G2RSVPTE_WRN("Freeing path err msg before allocating");
				delete lsp->psb()->pathErr();
				lsp->psb()->pathErr(0);
			}

			// Set NOT_ERASE flags for this pkg
			lsp->psb()->pathErr(new RSVP::PathErrMessage(*lsp->psb()->pathOut()));

			if (!lsp->psb()->pathErr()) {
				G2RSVPTE_ERR("could not allocate outgoing PathErr msg");
				return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPathErr_to_InvalidEvent;
			}

			// Create Error Spec
			lsp->psb()->pathErr()->error_spec_ = pathErr->error_spec_->clone();
			if (!lsp->psb()->pathErr()->error_spec_) {
				G2RSVPTE_ERR("could not allocate outgoing ErrorSpec object");
				return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPathErr_to_InvalidEvent;
			}
			// Create RRO
			lsp->psb()->pathErr()->rro_ = pathErr->rro_->clone();
			if (!lsp->psb()->pathErr()->rro_) {
				G2RSVPTE_ERR("could not allocate outgoing RRO object");
				return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPathErr_to_InvalidEvent;
			}
			// add this node to the RRO
			RSVP::RecordedRoute_Object * genRRO;
			genRRO = static_cast<RSVP::RecordedRoute_Object *>(lsp->psb()->pathErr()->rro_);

			if (lsp->psb()->interface() &&
			    !lsp->insertIntoRRO(genRRO,
						lsp->psb()->interface()->localId())) {
				G2RSVPTE_ERR("Problems while preparing Path RRO obj for outgoing Path Err");
				return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPathErr_to_InvalidEvent;
			}

			// Clear all the timers
			// TODO: FIXME!!! clear timers:
			//          - lsp->psb()->refreshTimeout()
			//          - lsp->psb()->refreshTimer()
			//          - lsp->rsb()->refreshTimeout()
			//          - lsp->rsb()->refreshTimer()

			G2RSVPTE_INF("forwarding PathErr msg to NE %s through TE link %s",
				     numb_htoa(lsp->psb()->nodeId()),
				     addr_ntoa(lsp->psb()->interface()->localId()));

			if (!lsp->usAckNack() || !lsp->usAckNack()->sendMsg(lsp->psb()->pathErr())) {
				G2RSVPTE_ERR("could not deliver PathErr to the ACK FSM");
				return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPathErr_to_InvalidEvent;
			}

			//TODO: FIXME!!! cancel message (if flags NOT_ERASE is not set)
			//delete lsp->psb()->pathErr();
			//lsp->psb()->pathErr(0);
		}
	}
	// now we can throw away the received PathErr msg
	G2RSVPTE_DBG("Deleting the received PathErr msg");
	delete pathErr;

	if ((lsp->getInstance()->adjType() == INNI) &&
	     lsp->tnrcCookie()) {
		// Request a resource release at the TNRC
		if (!releaseResource(lsp)) {
			G2RSVPTE_ERR("Impossible to release resource");
		}
		//set tnrcCookie to 0
		lsp->tnrcCookie(0);
	}

	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPathErr_to_evRecvPathErrOk;
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResvErr_t
handleRecvResvErr(void * context)
{
	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	assert(ctxt->msg);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->psb());
	assert(lsp->rsb());
	assert(lsp->getInstance());

	// Check that the incoming msg is a ResvErr
	if (ctxt->msg->getType() != RSVP::Message::RESV_ERR) {
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResvErr_to_evRecvResvErrKo;
	}
	RSVP::ResvErrMessage * resvErr = static_cast<RSVP::ResvErrMessage *>(ctxt->msg);
	// XXXX FIXME: Check if pathErr msg is ok

	// TODO: check if the outgoing parsed packet already points to a
	//       packet that is under retransmission in some ACK/NACK FSM

	// Get the error
	error_info_t              error;

	memset(&error,0, sizeof(error));

	if (!resvErr->error_spec_ ||
	    !resvErr->getErrorDetails(error)) {
		G2RSVPTE_ERR("Invalid received ResvErr msg");
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResvErr_to_evRecvResvErrKo;
	}

	// TODO create a macro ERRCODE2STRING and ERRVAL2STRING for debugging
	G2RSVPTE_INF("ResvErr: error %d, %d at NE %s",
		     error.err_code, error.err_val, numb_htoa(error.node_id));

	// Head node?
	bool tailNode;
	tailNode = lsp->isTail();

	// TODO future (crankback) here ???
	//return G2RSVPTE_LSP_FSM_from_RecvPathErr_to_evRecvPathErrCrankback;

	if (tailNode) {
		if (lsp->getInstance()->adjType() == INNI) {
			// notify an Err Event to RC
			G2RSVPTE_INF("posting an Err Event to RC");
			postLspErrEvent(lsp, true);
		} else {
			lsp->rsb()->resvErr(static_cast<RSVP::ResvErrMessage *>(resvErr->clone()));
			// notify an Err Event to CC
			G2RSVPTE_INF("posting a RecvPathErrEvent to CC");
			notifyLspRecvResvErrEvent(lsp);
		}
		// now we can throw away the received PathErr msg
		G2RSVPTE_DBG("Deleting the received ResvErr msg");
		delete resvErr;

	} else {
		if (lsp->getInstance()->adjType() == INNI) {
			// Build the PathErr msg from the PSB PathOut
			if (!lsp->rsb()->resvOut()) {
				G2RSVPTE_ERR("Invalid ResvOut msg");
				return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResvErr_to_InvalidEvent;
			}

			if (lsp->rsb()->resvErr()) {
				G2RSVPTE_WRN("Freeing resv err msg before allocating");
				delete lsp->rsb()->resvErr();
				lsp->rsb()->resvErr(0);
			}

			// Set NOT_ERASE flags for this pkg
			lsp->rsb()->resvErr(new RSVP::ResvErrMessage(*lsp->rsb()->resvOut()));

			if (!lsp->rsb()->resvErr()) {
				G2RSVPTE_ERR("could not allocate outgoing ResvErr msg");
				return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResvErr_to_InvalidEvent;
			}

			// Create Error Spec, for now IPv4 but with cb IF_ID_IPv4 ?????
			RSVP::IPv4ErrorSpec_Object * tmpErrSpec;
			tmpErrSpec = new RSVP::IPv4ErrorSpec_Object(error.node_id,
								    error.flags,
								    error.err_code,
								    error.err_val);
			if (!tmpErrSpec) {
				G2RSVPTE_ERR("Impossible to allocate errorSpec subObj");

				delete lsp->rsb()->resvErr();
				lsp->rsb()->resvErr(0);

				return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResvErr_to_InvalidEvent;;
			}
			// Set error spec into PathErr msg
			lsp->rsb()->resvErr()->error_spec_ = tmpErrSpec;

			// Clear all the timers
			// TODO: FIXME!!! clear timers:
			//          - lsp->psb()->refreshTimeout()
			//          - lsp->psb()->refreshTimer()
			//          - lsp->rsb()->refreshTimeout()
			//          - lsp->rsb()->refreshTimer()

			G2RSVPTE_INF("forwarding ResvErr msg to NE %s through TE link %s",
				     numb_htoa(lsp->psb()->nodeId()),
				     addr_ntoa(lsp->psb()->interface()->localId()));

			if (!lsp->dsAckNack() || !lsp->dsAckNack()->sendMsg(lsp->rsb()->resvErr())) {
				G2RSVPTE_ERR("could not deliver ResvErr to the ACK FSM");
				return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResvErr_to_InvalidEvent;
			}

			//TODO: FIXME!!! cancel message (if flags NOT_ERASE is not set)
			//delete lsp->rsb()->resvErr();
			//lsp->rsb()->resvErr(0);
		}
	}

	if ((lsp->getInstance()->adjType() == INNI) &&
	     lsp->tnrcCookie()) {
		// Request a resource release at the TNRC
		if (!releaseResource(lsp)) {
			G2RSVPTE_ERR("Impossible to release resource");
		}
		//set tnrcCookie to 0
		lsp->tnrcCookie(0);
	}

	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResvErr_to_evRecvResvErrOk;
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathDown_t
handleRecvPathDown(void * context)
{
	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	assert(ctxt->msg);
	//assert(ctxt->intf);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->psb());
	assert(lsp->rsb());
	assert(lsp->getInstance());

	// Check that the incoming Path is a PathDown
	if (ctxt->msg->getType() != RSVP::Message::PATH) {
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPathDown_to_evRecvPathDownKo;
	}
	RSVP::PathMessage * pathDown = static_cast<RSVP::PathMessage *>(ctxt->msg);
	// XXXX FIXME: Check if msg is a PathDown and if it is ok
	//if (!pathDown->isPathDown()) {
	//	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPathDown_to_evRecvPathDownKo;
	//}


	// TODO: check if the outgoing parsed packet already points to a
	//       packet that is under retransmission in some ACK/NACK FSM

	// Tail node?
	bool tailNode;
	tailNode = lsp->isTail();

	// TODO disable alarms here
	//    - is this a release-to-mgmt?
	//NOTICE("disabling alarms");
	// if !cm_alarms_del(lsp, 0) => G2RSVPTE_WRN("failed requesting alarms disabling");

	// Clear all the timers
	// TODO: FIXME!!! clear timers:
	//          - lsp->psb()->refreshTimeout()
	//          - lsp->psb()->refreshTimer()
	//          - lsp->rsb()->refreshTimeout()
	//          - lsp->rsb()->refreshTimer()

	// TODO  ??????????

	if (tailNode) {
		//FIX!!! Change admin_status type
		//       (AdminStatus_Class, not AdminStatus_Object)
		assert(pathDown->admin_status_);
		assert(pathDown->admin_status_->cType() == CTYPE_ADMIN_STATUS);
		RSVP::AdminStatus_Object * adminStatus;
		adminStatus = static_cast<RSVP::AdminStatus_Object *>(pathDown->admin_status_);

		// notify a Down Event if the R bit is set
		if (adminStatus->get().R == 1) {
			if (lsp->getInstance()->adjType() == INNI) {
				G2RSVPTE_INF("posting a (Path)Down Event to RC");
				postLspPathDownEvent(lsp, true);
			} else {
				G2RSVPTE_ERR("ignoring the R bit");
				notifyLspRecvPathDownEvent(lsp);
			}
		} else {
			if (lsp->getInstance()->adjType() == INNI) {
				G2RSVPTE_INF("the R bit is unset: not Down Event notify");
			} else {
				G2RSVPTE_INF("posting a RecvPathDownEvent to CC");
				notifyLspRecvPathDownEvent(lsp);
			}
		}

		// now we can throw away the received PathDown msg
		G2RSVPTE_DBG("Deleting the received PathDown msg");
		delete pathDown;

	} else {
		// Prepare PathDown msg
		if (!lsp->psb()->pathOut()) {
			G2RSVPTE_ERR("Invalid PathOut msg");
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPathDown_to_InvalidEvent;
		}

		if (lsp->psb()->pathDown()) {
			G2RSVPTE_WRN("Freeing path down msg before allocating");
			delete lsp->psb()->pathDown();
			lsp->psb()->pathDown(0);
		}

		// Set NOT_ERASE flags for this pkg
		lsp->psb()->pathDown(new RSVP::PathMessage(*lsp->psb()->pathOut()));

		if (!lsp->psb()->pathDown()) {
			G2RSVPTE_ERR("could not allocate outgoing PathDown msg");
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPathDown_to_InvalidEvent;
		}

		// NB admin_status is optional but MUST be allocate
		if (!lsp->psb()->pathDown()->admin_status_) {
			lsp->psb()->pathDown()->admin_status_ = new RSVP::AdminStatus_Object();
			if (!lsp->psb()->pathDown()->admin_status_) {
				G2RSVPTE_ERR("could not allocate admin status Obj then PathDown msg");
				return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPathDown_to_InvalidEvent;
			}
		}

		//FIX!!! Change admin_status type
		//       (AdminStatus_Class, not AdminStatus_Object)
		assert(lsp->psb()->pathDown()->admin_status_->cType() == CTYPE_ADMIN_STATUS);
		RSVP::AdminStatus_Object * adminStatus;
		adminStatus = static_cast<RSVP::AdminStatus_Object *>(lsp->psb()->pathDown()->admin_status_);
		assert(pathDown->admin_status_);
		assert(pathDown->admin_status_->cType() == CTYPE_ADMIN_STATUS);

		// set D = 1 and R = 1 in Admin Status Obj
		adminStatus->get().D = 1;
		adminStatus->get().R = static_cast<RSVP::AdminStatus_Object *>(pathDown->admin_status_)->get().R;
		adminStatus->get().A = static_cast<RSVP::AdminStatus_Object *>(pathDown->admin_status_)->get().A; //0
		adminStatus->get().T = static_cast<RSVP::AdminStatus_Object *>(pathDown->admin_status_)->get().T; //0
		//adminStatus->get().XXX

		G2RSVPTE_INF("forwarding PathDown msg to NE %s through TE link %s",
			     numb_htoa(lsp->rsb()->nodeId()),
			     addr_ntoa(lsp->rsb()->interface()->localId()));

		if (!lsp->dsAckNack() || !lsp->dsAckNack()->sendMsg(lsp->psb()->pathDown())) {
			G2RSVPTE_ERR("could not deliver PathDown to the ACK FSM");
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPathDown_to_InvalidEvent;
		}

		//TODO: FIXME!!! cancel message (if flags NOT_ERASE is not set)
		//delete lsp->psb()->pathDown();
		//lsp->psb()->pathDown(0);
	}

	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPathDown_to_evRecvPathDownOk;
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendPathDown_t
handleSendPathDown(void * context)
{
	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	//assert(ctxt->msg);
	//assert(ctxt->intf);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

	assert(lsp->psb());
	assert(lsp->rsb());
	assert(lsp->psb()->pathDown());
	assert(lsp->psb()->pathDown()->admin_status_);

	// Tail node?
	bool tailNode;
	tailNode = lsp->isTail();

	// TODO disable alarms here
	//NOTICE("disabling alarms");
	// if !cm_alarms_del(lsp, 0) => G2RSVPTE_WRN("failed requesting alarms disabling");

	// Clear all the timers
	// TODO: FIXME!!! clear timers:
	//          - lsp->psb()->refreshTimeout()
	//          - lsp->psb()->refreshTimer()
	//          - lsp->rsb()->refreshTimeout()
	//          - lsp->rsb()->refreshTimer()

	if (tailNode) {
		//FIX!!! Change admin_status type
		//       (AdminStatus_Class, not AdminStatus_Object)
		assert(lsp->psb()->pathDown()->admin_status_->cType() == CTYPE_ADMIN_STATUS);
		RSVP::AdminStatus_Object * adminStatus;
		adminStatus = static_cast<RSVP::AdminStatus_Object *>(lsp->psb()->pathDown()->admin_status_);

		// notify a Down Event  if the R bit is set
		if (adminStatus->get().R == 1) {
//			G2RSVPTE_INF("posting a (Path)Down Event to RC");
//			postLspDownEvent(lsp, true);
		} else {
			G2RSVPTE_INF("the R bit is unset: not Down Event post");
		}
	} else {
		G2RSVPTE_INF("sending first PathDown msg to NE %s through TE link %s",
			     numb_htoa(lsp->rsb()->nodeId()),
			     addr_ntoa(lsp->rsb()->interface()->localId()));

		if (!lsp->dsAckNack() || !lsp->dsAckNack()->sendMsg(lsp->psb()->pathDown())) {
			G2RSVPTE_ERR("could not deliver PathDown to the ACK FSM");
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_SendPathDown_to_InvalidEvent;
		}

		//TODO: FIXME!!! cancel message (if flags NOT_ERASE is not set)
		//delete lsp->psb()->pathDown();
		//lsp->psb()->pathDown(0);
	}

	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_SendPathDown_to_evSendPathDown;
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResvTear_t
handleRecvResvTear(void * context)
{
	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	assert(ctxt->msg);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->psb());
	assert(lsp->rsb());
	assert(lsp->getInstance());

	// Check that the incoming Resv is a ResvTear
	if (ctxt->msg->getType() != RSVP::Message::RESV_TEAR) {
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResvTear_to_evRecvResvTearKo;
	}
	RSVP::ResvTearMessage * resvTear = static_cast<RSVP::ResvTearMessage *>(ctxt->msg);
	// XXXX FIXME: Check if msg is a ResvTear and if it is ok

	// TODO: check if the outgoing parsed packet already points to a
	//       packet that is under retransmission in some ACK/NACK FSM

	// Head node?
	bool headNode;
	headNode = lsp->isHead();

	// Clear all the timers
	// TODO: FIXME!!! clear timers:
	//          - lsp->psb()->refreshTimeout()
	//          - lsp->psb()->refreshTimer()
	//          - lsp->rsb()->refreshTimeout()
	//          - lsp->rsb()->refreshTimer()

	if (headNode) {
		if (lsp->getInstance()->adjType() == INNI) {
			// notify a Tear Event to RC
			G2RSVPTE_INF("posting a (Resv)Tear Event to RC");
			postLspTearEvent(lsp); // no action on this FSM
		} else {
			// notify a Tear Event to CC
			G2RSVPTE_INF("posting a RecvResvTearEvent to CC");
			notifyLspRecvResvTearEvent(lsp);
		}
		// now we can throw away the received ResvTear msg
		G2RSVPTE_DBG("Deleting the received ResvTear msg");
		delete resvTear;

	} else {
		// Build the ResvTear msg from the RSB ResvOut
		if (!lsp->rsb()->resvOut()) {
			G2RSVPTE_ERR("Invalid ResvOut msg");
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResvTear_to_InvalidEvent;
		}

		if (lsp->rsb()->resvTear()) {
			G2RSVPTE_WRN("Freeing resv tear msg before allocating");
			delete lsp->rsb()->resvTear();
			lsp->rsb()->resvTear(0);
		}

		// Set NOT_ERASE flags for this pkg
		lsp->rsb()->resvTear(new RSVP::ResvTearMessage(*lsp->rsb()->resvOut()));

		if (!lsp->rsb()->resvTear()) {
			G2RSVPTE_ERR("could not allocate outgoing ResvTear msg");
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResvTear_to_InvalidEvent;
		}

		G2RSVPTE_INF("forwarding ResvTear msg to NE %s through TE link %s",
			     numb_htoa(lsp->psb()->nodeId()),
			     addr_ntoa(lsp->psb()->interface()->localId()));

		if (!lsp->usAckNack() || !lsp->usAckNack()->sendMsg(lsp->rsb()->resvTear())) {
			G2RSVPTE_ERR("could not deliver ResvTear to the ACK FSM");
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResvTear_to_InvalidEvent;
		}

		//TODO: FIXME!!! cancel message (if flags NOT_ERASE is not set)
		//delete lsp->rsb()->resvTear();
		//lsp->rsb()->resvTear(0);
	}

	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResvTear_to_evRecvResvTearOk;
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendResvTear_t
handleSendResvTear(void * context)
{
	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	//assert(ctxt->msg);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

	assert(lsp->psb());
	assert(lsp->rsb());
	assert(lsp->rsb()->resvTear());
	assert(lsp->getInstance());

	// Head node?
	bool headNode;
	headNode = lsp->isHead();

	// Clear all the timers
	// TODO: FIXME!!! clear timers:
	//          - lsp->psb()->refreshTimeout()
	//          - lsp->psb()->refreshTimer()
	//          - lsp->rsb()->refreshTimeout()
	//          - lsp->rsb()->refreshTimer()

	if (headNode) {
		if (lsp->getInstance()->adjType() == INNI) {
			// notify a Tear Event
			G2RSVPTE_INF("posting a (Resv)Tear Event to RC");
			postLspTearEvent(lsp, true, true);
		}
	} else {
		G2RSVPTE_INF("sending first ResvTear msg to NE %s through TE link %s",
			     numb_htoa(lsp->rsb()->nodeId()),
			     addr_ntoa(lsp->rsb()->interface()->localId()));

		if (!lsp->usAckNack() || !lsp->usAckNack()->sendMsg(lsp->rsb()->resvTear())) {
			G2RSVPTE_ERR("could not deliver ResvTear to the ACK FSM");
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_SendResvTear_to_InvalidEvent;
		}

		//TODO: FIXME!!! cancel message (if flags NOT_ERASE is not set)
		//delete lsp->rsb()->resvTear();
		//lsp->rsb()->resvTear(0);
	}

	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_SendResvTear_to_evSendResvTear;
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResvDown_t
handleRecvResvDown(void * context)
{
	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	assert(ctxt->msg);
	//assert(ctxt->intf);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->psb());
	assert(lsp->rsb());
	assert(lsp->getInstance());

	// Check that the incoming Resv is a ResvDown
	if (ctxt->msg->getType() != RSVP::Message::RESV) {
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResvDown_to_evRecvResvDownKo;
	}
	RSVP::ResvMessage * resvDown = static_cast<RSVP::ResvMessage *>(ctxt->msg);
	// XXXX FIXME: Check if msg is a ResvDown and if it is ok
	if (!resvDown->isDown()) {
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResvDown_to_evRecvResvDownKo;
	}


	// TODO: check if the outgoing parsed packet already points to a
	//       packet that is under retransmission in some ACK/NACK FSM

	// Head node?
	bool headNode;
	headNode = lsp->isHead();

	// TODO disable alarms here
	//    - is this a release-to-mgmt?
	//NOTICE("disabling alarms");
	// if !cm_alarms_del(lsp, 0) => G2RSVPTE_WRN("failed requesting alarms disabling");

	// Clear all the timers
	// TODO: FIXME!!! clear timers:
	//          - lsp->psb()->refreshTimeout()
	//          - lsp->psb()->refreshTimer()
	//          - lsp->rsb()->refreshTimeout()
	//          - lsp->rsb()->refreshTimer()

	// TODO  ??????????

	if (headNode) {
		// notify a Down Event  // if the R bit is set ???
		//if (resvDown->admin_status_->get().R == 1) {
		if (lsp->getInstance()->adjType() == INNI) {
			G2RSVPTE_INF("posting a (Resv)Down Event to RC");
			postLspResvDownEvent(lsp, true);
		} else {
			G2RSVPTE_INF("posting a RecvResvDown Event to CC");
			notifyLspRecvResvDownEvent(lsp);
		}
		// now we can throw away the received ResvDown msg
		G2RSVPTE_DBG("Deleting the received ResvDown msg");
		delete resvDown;

	} else {
		// Prepare ResvDown msg
		if (!lsp->rsb()->resvOut()) {
			G2RSVPTE_ERR("Invalid ResvOut msg");
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResvDown_to_InvalidEvent;
		}

		if (lsp->rsb()->resvDown()) {
			G2RSVPTE_WRN("Freeing resv down msg before allocating");
			delete lsp->rsb()->resvDown();
			lsp->rsb()->resvDown(0);
		}

		// Set NOT_ERASE flags for this pkg
		lsp->rsb()->resvDown(new RSVP::ResvMessage(*lsp->rsb()->resvOut()));

		if (!lsp->rsb()->resvDown()) {
			G2RSVPTE_ERR("could not allocate outgoing ResvDown msg");
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResvDown_to_InvalidEvent;
		}

		// NB admin_status is optional but MUST be allocate
		if (!lsp->rsb()->resvDown()->admin_status_) {
			lsp->rsb()->resvDown()->admin_status_ = new RSVP::AdminStatus_Object();
			if (!lsp->rsb()->resvDown()->admin_status_) {
				G2RSVPTE_ERR("could not allocate admin status Obj then ResvDown msg");
				return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResvDown_to_InvalidEvent;
			}
		}

		//FIX!!! Change admin_status type
		//       (AdminStatus_Class, not AdminStatus_Object)
		assert(lsp->rsb()->resvDown()->admin_status_->cType() == CTYPE_ADMIN_STATUS);
		RSVP::AdminStatus_Object * adminStatus;
		adminStatus = static_cast<RSVP::AdminStatus_Object *>(lsp->rsb()->resvDown()->admin_status_);
		assert(resvDown->admin_status_);
		assert(resvDown->admin_status_->cType() == CTYPE_ADMIN_STATUS);

		// set D = 1 and R = 1 in Admin Status Obj
		adminStatus->get().D = 1;
		adminStatus->get().R = static_cast<RSVP::AdminStatus_Object *>(resvDown->admin_status_)->get().R;
		adminStatus->get().A = static_cast<RSVP::AdminStatus_Object *>(resvDown->admin_status_)->get().A; //0
		adminStatus->get().T = static_cast<RSVP::AdminStatus_Object *>(resvDown->admin_status_)->get().T; //0
		//adminStatus->get().XXX

		G2RSVPTE_INF("forwarding ResvDown msg to NE %s through TE link %s",
			     numb_htoa(lsp->psb()->nodeId()),
			     addr_ntoa(lsp->psb()->interface()->localId()));

		if (!lsp->usAckNack() || !lsp->usAckNack()->sendMsg(lsp->rsb()->resvDown())) {
			G2RSVPTE_ERR("could not deliver ResvDown to the ACK FSM");
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResvDown_to_InvalidEvent;
		}

		//TODO: FIXME!!! cancel message (if flags NOT_ERASE is not set)
		//delete lsp->rsb()->resvDown();
		//lsp->rsb()->resvDown(0);
	}

	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResvDown_to_evRecvResvDownOk;
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendResvDown_t
handleSendResvDown(void * context)
{
	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	//assert(ctxt->msg);
	//assert(ctxt->intf);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

	assert(lsp->psb());
	assert(lsp->rsb());
	assert(lsp->rsb()->resvDown());

	// Head node?
	bool headNode;
	headNode = lsp->isHead();

	// TODO disable alarms here
	//NOTICE("disabling alarms");
	// if !cm_alarms_del(lsp, 0) => G2RSVPTE_WRN("failed requesting alarms disabling");

	// Clear all the timers
	// TODO: FIXME!!! clear timers:
	//          - lsp->psb()->refreshTimeout()
	//          - lsp->psb()->refreshTimer()
	//          - lsp->rsb()->refreshTimeout()
	//          - lsp->rsb()->refreshTimer()

	if (headNode) {
		// notify a Down Event  ?? if the R bit is set
		if (lsp->rsb()->resvDown()->isDown()) {
//			G2RSVPTE_INF("posting a (Resv)Down Event to RC");
//			postLspDownEvent(lsp, true);
		}
	} else {
		G2RSVPTE_INF("sending first ResvDown msg to NE %s through TE link %s",
			     numb_htoa(lsp->psb()->nodeId()),
			     addr_ntoa(lsp->psb()->interface()->localId()));

		if (!lsp->usAckNack() || !lsp->usAckNack()->sendMsg(lsp->rsb()->resvDown())) {
			G2RSVPTE_ERR("could not deliver ResvDown to the ACK FSM");
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_SendResvDown_to_InvalidEvent;
		}

		//TODO: FIXME!!! cancel message (if flags NOT_ERASE is not set)
		//delete lsp->rsb()->resvDown();
		//lsp->rsb()->resvDown(0);
	}

	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_SendResvDown_to_evSendResvDown;
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathTear_t
handleRecvPathTear(void * context)
{
	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	assert(ctxt->msg);
	//assert(ctxt->intf);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->psb());
	assert(lsp->rsb());
	assert(lsp->getInstance());
	// Check that the incoming Path is a PathTear
	if (ctxt->msg->getType() != RSVP::Message::PATH_TEAR) {
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPathTear_to_evRecvPathTearKo;
	}
	RSVP::PathTearMessage * pathTear = static_cast<RSVP::PathTearMessage *>(ctxt->msg);
	// XXXX FIXME: Check if msg is a PathTear and if it is ok

	// TODO: check if the outgoing parsed packet already points to a
	//       packet that is under retransmission in some ACK/NACK FSM

	// Tail node?
	bool tailNode;
	tailNode = lsp->isTail();

	// Clear all the timers
	// TODO: FIXME!!! clear timers:
	//          - lsp->psb()->refreshTimeout()
	//          - lsp->psb()->refreshTimer()
	//          - lsp->rsb()->refreshTimeout()
	//          - lsp->rsb()->refreshTimer()

	if (tailNode) {
		if (lsp->getInstance()->adjType() == INNI) {
			// notify a Tear Event to RC
			G2RSVPTE_INF("posting a (Path)Tear Event to RC");
			postLspTearEvent(lsp);  // no action on this FSM
		} else {
			// notify a Tear Event to CC
			G2RSVPTE_INF("posting a RecvPathTear Event to CC");
			notifyLspRecvPathTearEvent(lsp);
		}
		// now we can throw away the received PathTear msg
		G2RSVPTE_DBG("Deleting the received PathTear msg");
		delete pathTear;

	} else {
		// Build the PathTear msg from the PSB PathOut
		if (!lsp->psb()->pathOut()) {
			G2RSVPTE_ERR("Invalid PathOut msg");
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPathTear_to_InvalidEvent;
		}

		if (lsp->psb()->pathTear()) {
			G2RSVPTE_WRN("Freeing path tear msg before allocating");
			delete lsp->psb()->pathTear();
			lsp->psb()->pathTear(0);
		}

		// Set NOT_ERASE flags for this pkg
		lsp->psb()->pathTear(new RSVP::PathTearMessage(*lsp->psb()->pathOut()));

		if (!lsp->psb()->pathTear()) {
			G2RSVPTE_ERR("could not allocate outgoing PathTear msg");
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPathTear_to_InvalidEvent;
		}

		G2RSVPTE_INF("forwarding PathTear msg to NE %s through TE link %s",
			     numb_htoa(lsp->rsb()->nodeId()),
			     addr_ntoa(lsp->rsb()->interface()->localId()));

		if (!lsp->dsAckNack() || !lsp->dsAckNack()->sendMsg(lsp->psb()->pathTear())) {
			G2RSVPTE_ERR("could not deliver PathTear to the ACK FSM");
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPathTear_to_InvalidEvent;
		}

		//TODO: FIXME!!! cancel message (if flags NOT_ERASE is not set)
		//delete lsp->psb()->pathTear();
		//lsp->psb()->pathTear(0);
	}

	if ((lsp->getInstance()->adjType() == INNI) &&
	     lsp->tnrcCookie()) {
		// Request a resource release at the TNRC
		if (!releaseResource(lsp)) {
			G2RSVPTE_ERR("Impossible to release resource");
		}
		//set tnrcCookie to 0
		lsp->tnrcCookie(0);
	}

	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPathTear_to_evRecvPathTearOk;
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendPathTear_t
handleSendPathTear(void * context)
{
	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	//assert(ctxt->msg);
	//assert(ctxt->intf);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

	assert(lsp->psb());
	assert(lsp->rsb());
	//assert(lsp->psb()->pathDown());
	assert(lsp->getInstance());

	// Tail node?
	bool tailNode;
	tailNode = lsp->isTail();

	// Clear all the timers
	// TODO: FIXME!!! clear timers:
	//          - lsp->psb()->refreshTimeout()
	//          - lsp->psb()->refreshTimer()
	//          - lsp->rsb()->refreshTimeout()
	//          - lsp->rsb()->refreshTimer()

	if (tailNode) {
		if (lsp->getInstance()->adjType() == INNI) {
			G2RSVPTE_INF("posting a (Path)Tear Event to RC");
			postLspTearEvent(lsp, true, true);
		}
	} else {
		G2RSVPTE_INF("sending first PathTear msg to NE %s through TE link %s",
			     numb_htoa(lsp->rsb()->nodeId()),
			     addr_ntoa(lsp->rsb()->interface()->localId()));

		if (!lsp->dsAckNack() || !lsp->dsAckNack()->sendMsg(lsp->psb()->pathTear())) {
			G2RSVPTE_ERR("could not deliver PathTear to the ACK FSM");
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_SendPathTear_to_InvalidEvent;
		}

		//TODO: FIXME!!! cancel message (if flags NOT_ERASE is not set)
		//delete lsp->psb()->pathTear();
		//lsp->psb()->pathTear(0);
	}

	if ((lsp->getInstance()->adjType() == INNI) &&
	     lsp->tnrcCookie()) {
		// Request a resource release at the TNRC
		if (!releaseResource(lsp)) {
			G2RSVPTE_ERR("Impossible to release resource");
		}
		//set tnrcCookie to 0
		lsp->tnrcCookie(0);
	}

	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_SendPathTear_to_evSendPathTear;
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_XConnErr_t
handleXConnErr(void * context)
{
	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->psb());

	G2RSVPTE_NOT("Error: the XC is failed");

	// Clear all the timers
	// TODO: FIXME!!! clear timers:
	//          - lsp->psb()->refreshTimeout()
	//          - lsp->psb()->refreshTimer()
	//          - lsp->rsb()->refreshTimeout()
	//          - lsp->rsb()->refreshTimer()

	// dispatch some error -> crank_info

	//send a ResvErr downstream
	if (!lsp->dsAckNack() || !lsp->dsAckNack()->errorHandle(G2RSVPTE_ERROR_SPEC_FLAG_PathStateRemoved,
								G2RSVPTE_ERROR_SPEC_SYSTEM,
								0,
								true,
								true)) {
		G2RSVPTE_ERR("could not send ResvErr");
	}

	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_XConnErr_to_evXConnErr;
}


/*****************************************************************************
 *                    Class WaitResvConf_i
 *****************************************************************************/

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResv_t
WaitResvConf_i::RecvResv(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "RecvResv");
	return (fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResv_t)1;
}

void
WaitResvConf_i::after_evRecvResvKo_from_virt_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvKo", "virt_WaitResvConf");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathErr_t
WaitResvConf_i::RecvPathErr(void * context)
{
	DUMP_STATE(name(), "RecvPathErr");
	return handleRecvPathErr(context);
}

void
WaitResvConf_i::after_evRecvPathErrKo_from_virt_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathErrKo", "virt_WaitResvConf");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPath_t
WaitResvConf_i::RecvPath(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "RecvPath");
	return (fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPath_t)1;
}

void
WaitResvConf_i::after_evRecvPathOk_from_virt_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathOk", "virt_WaitResvConf");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResvTimer_t
WaitResvConf_i::RecvResvTimer(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "RecvResvTimer");
	return (fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResvTimer_t)1;
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_XConnPreempt_t
WaitResvConf_i::XConnPreempt(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "XConnPreempt");
	return (fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_XConnPreempt_t)1;
}

void
WaitResvConf_i::after_evXConnPreempt_from_virt_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evXConnPreempt", "virt_WaitResvConf");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendPathTear_t
WaitResvConf_i::SendPathTear(void * context)
{
	DUMP_STATE(name(), "SendPathTear");
	return handleSendPathTear(context);
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendConfirm_t
WaitResvConf_i::SendConfirm(void * context)
{
	DUMP_STATE(name(), "SendConfirm");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	//assert(ctxt->msg);
	//assert(ctxt->intf);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->psb());
	assert(lsp->getInstance());

	//TODO (not now!!!)  good place to activate the alarms ???
	//if !cm_alarms_set(lsp, 0) then
	//return InvalidEvent;

	// Tail node?
	bool tailNode;
	tailNode = lsp->isTail();

	if (!tailNode) {
		G2RSVPTE_INF("sending first ResvConf msg to NE %s through TE link %s",
			     numb_htoa(lsp->rsb()->nodeId()),
			     addr_ntoa(lsp->rsb()->interface()->localId()));

		if (!lsp->dsAckNack() || !lsp->dsAckNack()->sendMsg(lsp->psb()->resvConf())) {

			G2RSVPTE_ERR("could not deliver ResvConf to the ACK FSM");
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_SendConfirm_to_InvalidEvent;
		}
		if (lsp->getInstance()->adjType() == INNI) {
			// Notify a ResvConf Event
			G2RSVPTE_INF("posting a ResvConfirm Event to RC");
			postLspResvConfEvent(lsp, true);
		}

		//TODO: FIXME!!! cancel message (if flags NOT_ERASE is not set)
		//delete lsp->psb()->resvConf();
		//lsp->psb()->resvConf(0);
	}

	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_SendConfirm_to_evSendConfirm;
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvConfirm_t
WaitResvConf_i::RecvConfirm(void * context)
{
	DUMP_STATE(name(), "RecvConfirm");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	assert(ctxt->msg);
	//assert(ctxt->intf);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->psb());


	// TODO: check if the outgoing parsed packet already points to a
	//       packet that is under retransmission in some ACK/NACK FSM

	// XXXX FIXME: Check if msg is a ResvConfMessage
	//             Store parsed packet info into the LSP and check
	//             Set NOT_ERASE flags for this pkg
	//             Set also the owner of the packet (not now)
	lsp->psb()->resvConf((RSVP::ResvConfMessage *)ctxt->msg);

	// Tail node?
	bool tailNode;
	tailNode = lsp->isTail();

	// Check that style is FF
	assert(lsp->psb()->resvConf()->style_); // TMP???

	RSVP::Style_Class * style;
	style = lsp->psb()->resvConf()->style_;

	if (style->cType() != CTYPE_STYLE) {
		G2RSVPTE_ERR("problems... STYLE CTYPE %d not supported",
			     style->cType());
		return fsm::
			base_G2RSVPTE_LSP_FSM::
			G2RSVPTE_LSP_FSM_from_RecvConfirm_to_evRecvConfirmKo;
	}

	if (!(static_cast<RSVP::Style_Object *> (style))->isFF()) {
		G2RSVPTE_ERR("only Fixed Filter reservation "
			     "style is supported");

		//send a PathErr upstream
		if (!lsp->usAckNack() || !lsp->usAckNack()->errorHandle(G2RSVPTE_ERROR_SPEC_FLAG_PathStateRemoved,
									G2RSVPTE_ERROR_SPEC_UNKNOWN_STYLE,
									0)) {
			G2RSVPTE_ERR("could not send PathErr");
		}

                if ((lsp->getInstance()->adjType() == INNI) &&
		     lsp->tnrcCookie()) {
		        // Request a resource release at the TNRC
		        if (!releaseResource(lsp)) {
		                G2RSVPTE_ERR("Impossible to release resource");
		        }
			//set tnrcCookie to 0
			lsp->tnrcCookie(0);
                }
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvConfirm_to_evRecvConfirmKo;
	}

	G2RSVPTE_INF("Fixed Filter reservation selected");

	//TODO (not now!!!)  good place to activate the alarms ???

	if (tailNode) {
		if (lsp->getInstance()->adjType() == INNI) {
			// Notify a ResvConf Event to RC
			G2RSVPTE_INF("posting a ResvConfirm Event to RC");
			postLspResvConfEvent(lsp, true);
		} else {
			// Notify a ResvConf Event to CC
			G2RSVPTE_INF("posting a ResvConfirm Event to CC");
			notifyLspRecvResvConfEvent(lsp);
		}
	} else {
		G2RSVPTE_INF("forwarding ResvConf msg to NE %s through TE link %s",
			     numb_htoa(lsp->rsb()->nodeId()),
			     addr_ntoa(lsp->rsb()->interface()->localId()));

		if (!lsp->dsAckNack() || !lsp->dsAckNack()->sendMsg(lsp->psb()->resvConf())) {

			G2RSVPTE_ERR("could not deliver ResvConf to the ACK FSM");

			//send a PathErr upstream
			if (!lsp->usAckNack() || !lsp->usAckNack()->errorHandle(G2RSVPTE_ERROR_SPEC_FLAG_PathStateRemoved,
										G2RSVPTE_ERROR_SPEC_SYSTEM,
										0)) {
				G2RSVPTE_ERR("could not send PathErr");
			}

			if ((lsp->getInstance()->adjType() == INNI) &&
			     lsp->tnrcCookie()) {
			        // Request a resource release at the TNRC
			        if (!releaseResource(lsp)) {
			                G2RSVPTE_ERR("Impossible to release resource");
			        }
				//set tnrcCookie to 0
				lsp->tnrcCookie(0);
			}
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvConfirm_to_InvalidEvent;
		}

		//TODO: FIXME!!! cancel message (if flags NOT_ERASE is not set)
		//delete lsp->psb()->resvConf();
		//lsp->psb()->resvConf(0);
	}

	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvConfirm_to_evRecvConfirmOk;
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathDown_t
WaitResvConf_i::RecvPathDown(void * context)
{
	DUMP_STATE(name(), "RecvPathDown");
	return handleRecvPathDown(context);
}

void
WaitResvConf_i::after_evRecvPathDownKo_from_virt_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathDownKo", "virt_WaitResvConf");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResvTear_t
WaitResvConf_i::RecvResvTear(void * context)
{
	DUMP_STATE(name(), "RecvResvTear");
	return handleRecvResvTear(context);
}

void
WaitResvConf_i::after_evRecvResvTearKo_from_virt_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvTearKo", "virt_WaitResvConf");
}

void
WaitResvConf_i::after_evRecvPathKo_from_virt_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathKo", "virt_WaitResvConf");
}

void
WaitResvConf_i::after_evRecvConfirmKo_from_virt_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvConfirmKo", "virt_WaitResvConf");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendResvTear_t
WaitResvConf_i::SendResvTear(void * context)
{
	DUMP_STATE(name(), "SendResvTear");
	return handleSendResvTear(context);
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResvErr_t
WaitResvConf_i::RecvResvErr(void * context)
{
	DUMP_STATE(name(), "RecvResvErr");
	return handleRecvResvErr(context);
}

void
WaitResvConf_i::after_evRecvResvErrKo_from_virt_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvErrKo", "virt_WaitResvConf");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathTear_t
WaitResvConf_i::RecvPathTear(void * context)
{
	DUMP_STATE(name(), "RecvPathTear");
	return handleRecvPathTear(context);
}

void
WaitResvConf_i::after_evRecvPathTearKo_from_virt_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathTearKo", "virt_WaitResvConf");
}

void
WaitResvConf_i::after_evRecvResvErrOk_from_virt_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvErrOk", "virt_WaitResvConf");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathTimer_t
WaitResvConf_i::RecvPathTimer(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "RecvPathTimer");
	return (fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathTimer_t)1;
}

void
WaitResvConf_i::after_evRecvResvOk_from_virt_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvOk", "virt_WaitResvConf");
}

void
WaitResvConf_i::after_evRecvResvOk_from_virt_WaitResv(void * context)
{
	DUMP_STATE2(name(), "evRecvResvOk", "virt_WaitResv");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
}

void
WaitResvConf_i::after_evSendResv_from_virt_WaitResv(void * context)
{
	DUMP_STATE2(name(), "evSendResv", "virt_WaitResv");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
}

void
WaitResvConf_i::after_evXConnCompleted_from_virt_WaitEqptConf(void * context)
{
	DUMP_STATE2(name(), "evXConnCompleted", "virt_WaitEqptConf");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendPathDown_t
WaitResvConf_i::SendPathDown(void* context)
{
	DUMP_STATE(name(), "SendPathDown");
	return handleSendPathDown(context);
};

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResvDown_t
WaitResvConf_i::RecvResvDown(void* context)
{
	DUMP_STATE(name(), "RecvResvDown");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->rsb());
	assert(lsp->psb()->pathIn());

	if (!lsp->rsb()->resvOut()) {
		RSVP::ResvMessage * resv;
		resv = new RSVP::ResvMessage(*lsp->psb()->pathIn());
		if (!resv) {
			G2RSVPTE_ERR("Impossible to create ResvOut "
				     "for LSP id %d", lsp->id());
				return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResvDown_to_evRecvResvDownKo;
		}
		// Initialise ResvOut for this new LSP
		lsp->rsb()->resvOut(resv);

		// prepare NHOP
		if (!lsp->prepareNHOPobj()) {
			G2RSVPTE_ERR("Impossible to prepare Resv Out NHOP obj");
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResvDown_to_evRecvResvDownKo;
		}
	}

	return handleRecvResvDown(context);
};

void
WaitResvConf_i::after_evRecvResvDownKo_from_virt_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvDownKo", "virt_WaitResvConf");
};

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendResvDown_t
WaitResvConf_i::SendResvDown(void* context)
{
	DUMP_STATE(name(), "SendResvDown");
	return handleSendResvDown(context);
};


/*****************************************************************************
 *                    Class WaitResv_i
 *****************************************************************************/

void
WaitResv_i::after_evRecvResvTimer_from_virt_WaitResvConf(void * context)
{
	DUMP_STATE2(name(), "evRecvResvTimer", "virt_WaitResvConf");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
}

void
WaitResv_i::after_evRecvPathTimer_from_virt_WaitResvConf(void * context)
{
	DUMP_STATE2(name(), "evRecvPathTimer", "virt_WaitResvConf");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathTimeout_t
WaitResv_i::RecvPathTimeout(void * context)
{
	DUMP_STATE(name(), "RecvPathTimeout");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

	lsp->toBeDestroyed(true);
	postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPathTimeout_to_evRecvPathTimeout;
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResv_t
WaitResv_i::RecvResv(void * context)
{
	DUMP_STATE(name(), "RecvResv");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->rsb());
	assert(lsp->getInstance());

	fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResv_t res;
	res = handleFirstResv(context);
	if (res != fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResv_to_evRecvResvOk) {
		return res;
	}

	assert(lsp->rsb()->resvIn());
	// Set the refresh timer timeout
	if (!lsp->rsb()->resvIn()->time_values_) {
		G2RSVPTE_ERR("problems setting refresh timer timeout ... empty TIME_VALUES obj");
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResv_to_InvalidEvent;
	}
	//FIX!!! Change time_values type
	//       (TimeValues_Class, not TimeValues_Object)
	if (lsp->rsb()->resvIn()->time_values_->cType() != CTYPE_TIME_VALUES) {
		G2RSVPTE_ERR("problems time values CTYPE %d not supported", lsp->rsb()->resvIn()->time_values_->cType());

		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResv_to_InvalidEvent;
	}
	lsp->rsb()->refreshTimeout(G2RSVPTE_DEFAULT_REFRESH_TIMEOUT *
				   (static_cast<RSVP::TimeValues_Object *>(lsp->rsb()->resvIn()->time_values_))->get().refresh_period);

	G2RSVPTE_INF("set a Resv refresh timeout = %u ms",
		     lsp->rsb()->refreshTimeout());

	// Start ZEBRA timer with the refresh timer timeout     => No, so far!!!
	// TODO: FIXME!!! start timer => lsp->rsb()->refreshTimeout()
	//                else => G2RSVPTE_WRN("could not start refresh-timeout timer; let's go on anyway");

	if (lsp->isHead()) {
		if (lsp->getInstance()->adjType() == INNI) {
			// notify a Resv Event to RC
			G2RSVPTE_INF("posting a Resv Event to RC");
			postLspResvEvent(lsp, true);
		} else {
			bool failed = false;
			G2RSVPTE_INF("posting a Resv Event to CC");
			//set eGNS-TNA in Mgmt interface of xCC
			if (!failed && lsp->rsb()->resvIn()->gns_uni_eg_ && !postCallSetGnsTna(lsp, false)) {
				failed = true;
			}
			//set iGNS-TNA in Mgmt interface of xCC
			if (!failed && lsp->rsb()->resvIn()->gns_uni_in_ && !postCallSetGnsTna(lsp, true)) {
				failed = true;
			}
			// notify a Resv Event to CC
			if (failed || !notifyLspRecvResvEvent(lsp)) {
				G2RSVPTE_ERR("Notify of recvResv not delivered to CC");
				//send a ResvErr downstream
				if (!lsp->dsAckNack() || !lsp->dsAckNack()->errorHandle(G2RSVPTE_ERROR_SPEC_FLAG_PathStateRemoved,
											G2RSVPTE_ERROR_SPEC_SYSTEM,
											0)) {
					G2RSVPTE_ERR("could not send ResvErr");
				}
				return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResv_to_InvalidEvent;
			}
		}
	} else {
		// Set a random refresh timer
		// If we want to use peer refresh period
		//lsp->rsb()->refreshInterval(lsp->rsb()->resvIn()->time_values_->get().refresh_period);
		// else
		lsp->rsb()->refreshInterval(lsp->refreshInterval());

		G2RSVPTE_INF("set a Resv refresh interval = %u ms",
			     lsp->rsb()->refreshInterval());

		// Start ZEBRA timer with the refresh timer   => No, so far!!!
		// TODO: FIXME!!! start timer => lsp->rsb()->refreshInterval()
		//                else => G2RSVPTE_WRN("could not start refresh-interval timer; let's go on anyway");

		// we won't send the message now, but when the XC is ready
		G2RSVPTE_INF("forwarding Resv msg to NE %s through TE link %s",
			     numb_htoa(lsp->psb()->nodeId()),
			     addr_ntoa(lsp->psb()->interface()->localId()));

		if (!lsp->usAckNack() || !lsp->usAckNack()->sendMsg(lsp->rsb()->resvOut())) {

			G2RSVPTE_WRN("could not deliver Resv to the ACK FSM");

			//send a ResvErr downstream
			if (!lsp->dsAckNack() || !lsp->dsAckNack()->errorHandle(G2RSVPTE_ERROR_SPEC_FLAG_PathStateRemoved,
										G2RSVPTE_ERROR_SPEC_SYSTEM,
										0)) {
				G2RSVPTE_ERR("could not send ResvErr");
			}

			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResv_to_InvalidEvent;
		}
	}

	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResv_to_evRecvResvOk;
}

void
WaitResv_i::after_evRecvResvKo_from_virt_WaitResv(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvKo", "virt_WaitResv");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathErr_t
WaitResv_i::RecvPathErr(void * context)
{
	DUMP_STATE(name(), "RecvPathErr");
	return handleRecvPathErr(context);
}

void
WaitResv_i::after_evRecvPathErrKo_from_virt_WaitResv(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathErrKo", "virt_WaitResv");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPath_t
WaitResv_i::RecvPath(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "RecvPath");
	return (fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPath_t)1;
}

void
WaitResv_i::after_evRecvPathOk_from_virt_WaitResv(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathOk", "virt_WaitResv");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_XConnPreempt_t
WaitResv_i::XConnPreempt(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "XConnPreempt");
	return (fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_XConnPreempt_t)1;
}

void
WaitResv_i::after_evXConnPreempt_from_virt_WaitResv(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evXConnPreempt", "virt_WaitResv");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendPathTear_t
WaitResv_i::SendPathTear(void * context)
{
	DUMP_STATE(name(), "SendPathTear");
	return handleSendPathTear(context);
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResvTear_t
WaitResv_i::RecvResvTear(void * context)
{
	DUMP_STATE(name(), "RecvResvTear");
	return handleRecvResvTear(context);
}

void
WaitResv_i::after_evRecvResvTearKo_from_virt_WaitResv(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvTearKo", "virt_WaitResv");
}

void
WaitResv_i::after_evRecvPathKo_from_virt_WaitResv(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathKo", "virt_WaitResv");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendResvTear_t
WaitResv_i::SendResvTear(void * context)
{
	DUMP_STATE(name(), "SendResvTear");
	return handleSendResvTear(context);
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathTear_t
WaitResv_i::RecvPathTear(void * context)
{
	DUMP_STATE(name(), "RecvPathTear");
	return handleRecvPathTear(context);
}

void
WaitResv_i::after_evRecvPathTearKo_from_virt_WaitResv(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathTearKo", "virt_WaitResv");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathTimer_t
WaitResv_i::RecvPathTimer(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "RecvPathTimer");
	return (fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathTimer_t)1;
}

void
WaitResv_i::after_evRecvPathTimer_from_virt_WaitResv(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathTimer", "virt_WaitResv");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendResv_t
WaitResv_i::SendResv(void * context)
{
	DUMP_STATE(name(), "SendResv");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->rsb());
	assert(lsp->rsb()->resvOut());
	assert(lsp->getInstance());

	if (lsp->isHead()) {
		if (lsp->getInstance()->adjType() == INNI) {
			// notify a Resv Event
			G2RSVPTE_INF("posting a Resv Event to RC");
			postLspResvEvent(lsp, true);
		}
	} else {
		lsp->rsb()->refreshInterval(lsp->refreshInterval());

		G2RSVPTE_INF("set a Resv refresh interval = %u ms",
			     lsp->rsb()->refreshInterval());

		// Start ZEBRA timer with the refresh timer   => No, so far!!!
		// TODO: FIXME!!! start timer => lsp->rsb()->refreshInterval()
		//                else => G2RSVPTE_WRN("could not start refresh-interval timer; let's go on anyway");

		// we won't send the message now, but when the XC is ready
		G2RSVPTE_INF("sending first Resv msg to NE %s through TE link %s",
			     numb_htoa(lsp->psb()->nodeId()),
			     addr_ntoa(lsp->psb()->interface()->localId()));

		if (!lsp->usAckNack() || !lsp->usAckNack()->sendMsg(lsp->rsb()->resvOut())) {
			G2RSVPTE_WRN("could not deliver Resv to the ACK FSM");
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_SendResv_to_InvalidEvent;
		}
	}

	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_SendResv_to_evSendResv;
}

void
WaitResv_i::after_evXConnCompleted_from_virt_PathReceived(void * context)
{
	DUMP_STATE2(name(), "evXConnCompleted", "virt_PathReceived");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendPathDown_t
WaitResv_i::SendPathDown(void* context)
{
	DUMP_STATE(name(), "SendPathDown");
	return handleSendPathDown(context);
};

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathDown_t
WaitResv_i::RecvPathDown(void* context)
{
	DUMP_STATE(name(), "RecvPathDown");
	return handleRecvPathDown(context);
};

void
WaitResv_i::after_evRecvPathDownKo_from_virt_WaitResv(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathDownKo", "virt_WaitResv");
};

void
WaitResv_i::after_evRecvResvDownKo_from_virt_WaitResv(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvReavDownKo", "virt_WaitResv");
};

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendResvDown_t
WaitResv_i::SendResvDown(void* context)
{
	DUMP_STATE(name(), "SendResvDown");
	return handleSendResvDown(context);
};

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResvDown_t
WaitResv_i::RecvResvDown(void* context)
{
	DUMP_STATE(name(), "RecvResvDown");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->rsb());
	assert(lsp->psb()->pathIn());

	if (!lsp->rsb()->resvOut()) {
		RSVP::ResvMessage * resv;
		resv = new RSVP::ResvMessage(*lsp->psb()->pathIn());
		if (!resv) {
			G2RSVPTE_ERR("Impossible to create ResvOut "
				     "for LSP id %d", lsp->id());
				return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResvDown_to_evRecvResvDownKo;
		}
		// Initialise ResvOut for this new LSP
		lsp->rsb()->resvOut(resv);

		// prepare NHOP
		if (!lsp->prepareNHOPobj()) {
			G2RSVPTE_ERR("Impossible to prepare Resv Out NHOP obj");
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResvDown_to_evRecvResvDownKo;
		}
	}

	return handleRecvResvDown(context);
};


/*****************************************************************************
 *                    Class WaitEqptConf_i
 *****************************************************************************/

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathTimeout_t
WaitEqptConf_i::RecvPathTimeout(void * context)
{
	DUMP_STATE(name(), "RecvPathTimeout");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

	lsp->toBeDestroyed(true);
	postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPathTimeout_to_evRecvPathTimeout;
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_XConnCompleted_t
WaitEqptConf_i::XConnCompleted(void * context)
{
	DUMP_STATE(name(), "XConnCompleted");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->psb());

	if (lsp->isHead()) {
		// Notify a Resv Event to RC
		G2RSVPTE_INF("posting a Resv Event to RC");
		postLspResvEvent(lsp, true);
	} else {
		assert(lsp->psb()->interface());
		G2RSVPTE_INF("forwarding Resv msg to NE %s through TE link %s",
			     numb_htoa(lsp->psb()->nodeId()),
			     addr_ntoa(lsp->psb()->interface()->localId()));

		if (!lsp->usAckNack() || !lsp->usAckNack()->sendMsg(lsp->rsb()->resvOut())) {

			G2RSVPTE_WRN("could not deliver Resv to the ACK FSM");

			//send a ResvErr downstream
			if (!lsp->dsAckNack() || !lsp->dsAckNack()->errorHandle(G2RSVPTE_ERROR_SPEC_FLAG_PathStateRemoved,
										G2RSVPTE_ERROR_SPEC_SYSTEM,
										0)) {
				G2RSVPTE_ERR("could not send ResvErr");
			}

			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_XConnCompleted_to_InvalidEvent;
		}

		//TODO: FIXME!!! cancel message (if flags NOT_ERASE is not set)
		//delete lsp->rsb()->resvOut();
		//lsp->rsb()->resvOut(0);
	}

	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_XConnCompleted_to_evXConnCompleted;
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResv_t
WaitEqptConf_i::RecvResv(void * context)
{
	DUMP_STATE(name(), "RecvResv");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	assert(ctxt->msg);
	//assert(ctxt->intf);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->rsb());

	// Check if msg is a ResvMessage
	// TODO: check msg type!!!
	RSVP::ResvMessage * resv = (RSVP::ResvMessage *)ctxt->msg;

	// check if this pkg is a refresh msg
	if (1) {
		G2RSVPTE_INF("received a refresh Resv");

		// Set the refresh timer timeout
		if (!resv->time_values_) {
			G2RSVPTE_ERR("problems setting refresh timer timeout ... empty TIME_VALUES obj");
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResv_to_InvalidEvent;
		}
		//FIX!!! Change time_values type
		//       (TimeValues_Class, not TimeValues_Object)
		if (resv->time_values_->cType() != CTYPE_TIME_VALUES) {
			G2RSVPTE_ERR("problems time values CTYPE %d not supported", resv->time_values_->cType());
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResv_to_InvalidEvent;
		}
		lsp->rsb()->refreshTimeout(G2RSVPTE_DEFAULT_REFRESH_TIMEOUT *
					   (static_cast<RSVP::TimeValues_Object *>(resv->time_values_))->get().refresh_period);

		// restart the refresh-timeout ZEBRA timer => No, so far!!!
		// TODO: FIXME!!! first clear/reset timer, after
		//                start timer => with lsp->rsb()->refreshTimeout()
		//                else => G2RSVPTE_WRN("could not start refresh-timeout timer; let's go on anyway");

		delete resv;

		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResv_to_evRecvResvOk;
	} else {

		G2RSVPTE_WRN("this is not refresh Resv!!!");
		delete resv;

		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResv_to_evRecvResvKo;
	}
}

void
WaitEqptConf_i::after_evRecvResvKo_from_virt_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvKo", "virt_WaitEqptConf");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathErr_t
WaitEqptConf_i::RecvPathErr(void * context)
{
	DUMP_STATE(name(), "RecvPathErr");
	return handleRecvPathErr(context);
}

void
WaitEqptConf_i::after_evRecvPathErrKo_from_virt_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathErrKo", "virt_WaitEqptConf");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPath_t
WaitEqptConf_i::RecvPath(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "RecvPath");

	return (fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPath_t)1;
}

void
WaitEqptConf_i::after_evRecvPathOk_from_virt_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathOk", "virt_WaitEqptConf");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResvTimer_t
WaitEqptConf_i::RecvResvTimer(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "RecvResvTimer");

	return (fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResvTimer_t)1;
}

void
WaitEqptConf_i::after_evRecvResvTimer_from_virt_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvTimer", "virt_WaitEqptConf");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendPathTear_t
WaitEqptConf_i::SendPathTear(void * context)
{
	DUMP_STATE(name(), "SendPathTear");
	return handleSendPathTear(context);
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResvTimeout_t
WaitEqptConf_i::RecvResvTimeout(void * context)
{
	DUMP_STATE(name(), "RecvResvTimeout");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

	lsp->toBeDestroyed(true);
	postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResvTimeout_to_evRecvResvTimeout;
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResvTear_t
WaitEqptConf_i::RecvResvTear(void * context)
{
	DUMP_STATE(name(), "RecvResvTear");
	return handleRecvResvTear(context);
}

void
WaitEqptConf_i::after_evRecvResvTearKo_from_virt_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvTearKo", "virt_WaitEqptConf");
}

void
WaitEqptConf_i::after_evRecvPathKo_from_virt_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathKo", "virt_WaitEqptConf");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendResvTear_t
WaitEqptConf_i::SendResvTear(void * context)
{
	DUMP_STATE(name(), "SendResvTear");
	return handleSendResvTear(context);
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathTear_t
WaitEqptConf_i::RecvPathTear(void * context)
{
	DUMP_STATE(name(), "RecvPathTear");
	return handleRecvPathTear(context);
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResvErr_t
WaitEqptConf_i::RecvResvErr(void * context)
{
	DUMP_STATE(name(), "RecvResvErr");
	return handleRecvResvErr(context);
}

void
WaitEqptConf_i::after_evRecvResvErrKo_from_virt_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvErrKo", "virt_WaitEqptConf");
}

void
WaitEqptConf_i::after_evRecvPathTearKo_from_virt_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathTearKo", "virt_WaitEqptConf");
}

void
WaitEqptConf_i::after_evRecvResvErrOk_from_virt_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvErrOk", "virt_WaitEqptConf");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathTimer_t
WaitEqptConf_i::RecvPathTimer(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "RecvPathTimer");

	return (fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathTimer_t)1;
}

void
WaitEqptConf_i::after_evRecvPathTimer_from_virt_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathTimer", "virt_WaitEqptConf");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_XConnErr_t
WaitEqptConf_i::XConnErr(void * context)
{
	DUMP_STATE(name(), "XConnErr");

	return handleXConnErr(context);
}

void
WaitEqptConf_i::after_evRecvResvOk_from_virt_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvOk", "virt_WaitEqptConf");
}

void
WaitEqptConf_i::after_evRecvResvOk_from_virt_PathReceived(void * context)
{
	DUMP_STATE2(name(), "evRecvResvOk", "virt_PathReceived");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
}

void
WaitEqptConf_i::after_evSendResv_from_virt_PathReceived(void * context)
{
	DUMP_STATE2(name(), "evSendResv", "virt_PathReceived");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendPathDown_t
WaitEqptConf_i::SendPathDown(void* context)
{
	DUMP_STATE(name(), "SendPathDown");
	return handleSendPathDown(context);
};

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResvDown_t
WaitEqptConf_i::RecvResvDown(void* context)
{
	DUMP_STATE(name(), "RecvResvDown");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->rsb());
	assert(lsp->psb()->pathIn());

	if (!lsp->rsb()->resvOut()) {
		RSVP::ResvMessage * resv;
		resv = new RSVP::ResvMessage(*lsp->psb()->pathIn());
		if (!resv) {
			G2RSVPTE_ERR("Impossible to create ResvOut "
				     "for LSP id %d", lsp->id());
				return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResvDown_to_evRecvResvDownKo;
		}
		// Initialise ResvOut for this new LSP
		lsp->rsb()->resvOut(resv);

		// prepare NHOP
		if (!lsp->prepareNHOPobj()) {
			G2RSVPTE_ERR("Impossible to prepare Resv Out NHOP obj");
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResvDown_to_evRecvResvDownKo;
		}
	}

	return handleRecvResvDown(context);
};

void
WaitEqptConf_i::after_evRecvResvDownKo_from_virt_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvDownKo", "virt_WaitEqptConf");
};

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathDown_t
WaitEqptConf_i::RecvPathDown(void* context)
{
	DUMP_STATE(name(), "RecvPathDown");
	return handleRecvPathDown(context);
};

void
WaitEqptConf_i::after_evRecvPathDownKo_from_virt_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathDownKo", "virt_WaitEqptConf");
};

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendResvDown_t
WaitEqptConf_i::SendResvDown(void* context)
{
	DUMP_STATE(name(), "SendResvDown");
	return handleSendResvDown(context);
};


/*****************************************************************************
 *                    Class Rebuild_i
 *****************************************************************************/

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathErr_t
Rebuild_i::RecvPathErr(void* context)
{
	DUMP_STATE(name(), "RecvPathErr");
	return handleRecvPathErr(context);
};

void
Rebuild_i::after_evLspRebuild_from_virt_Down(void * context)
{
	DUMP_STATE2(name(), "evLspRebuild", "virt_Down");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);

	// setResource for INNI
	if (lsp->getInstance()->adjType() == INNI) {
		// Request a resource setting at the TNRC
		if (!setResource(lsp, true)) {
			G2RSVPTE_ERR("Impossible to set resource ... failed requesting action on cross-connection");
			// Post a fake XConnFailed event
			lsp->xConnFailed(true);
			return;
		}
	} else {
		// Post a fake XConnCompleted event
		lsp->xConnCompleted(true);
	}

};

void
Rebuild_i::after_evRecvPathErrKo_from_virt_Rebuild(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathErrKo", "virt_Rebuild");
};

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathDown_t
Rebuild_i::RecvPathDown(void* context)
{
	DUMP_STATE(name(), "RecvPathDown");
	return handleRecvPathDown(context);
};

void
Rebuild_i::after_evRecvPathDownKo_from_virt_Rebuild(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathDownKo", "virt_Rebuild");
};

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResvDown_t
Rebuild_i::RecvResvDown(void* context)
{
	DUMP_STATE(name(), "RecvResvDown");
	return handleRecvResvDown(context);
};

void
Rebuild_i::after_evRecvResvDownKo_from_virt_Rebuild(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvDownKo", "virt_Rebuild");
};

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendResvDown_t
Rebuild_i::SendResvDown(void* context)
{
	DUMP_STATE(name(), "SendResvDown");
	return handleSendResvDown(context);
};

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendPathDown_t
Rebuild_i::SendPathDown(void* context)
{
	DUMP_STATE(name(), "SendPathDown");
	return handleSendPathDown(context);
};

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathTear_t
Rebuild_i::RecvPathTear(void* context)
{
	DUMP_STATE(name(), "RecvPathTear");
	return handleRecvPathTear(context);
};

void
Rebuild_i::after_evRecvPathTearKo_from_virt_Rebuild(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathTearKo", "virt_Rebuild");
};

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_XConnCompleted_t
Rebuild_i::XConnCompleted(void* context)
{
	DUMP_STATE(name(), "XConnCompleted");

	G2RSVPTE_NOT("Ok, the XC is ready");

	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_XConnCompleted_to_evXConnCompleted;
};

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_XConnErr_t
Rebuild_i::XConnErr(void* context)
{
        DUMP_STATE(name(), "XConnErr");

	return handleXConnErr(context);
};


/*****************************************************************************
 *                    Class Installed_i
 *****************************************************************************/

void
Installed_i::after_evSendConfirm_from_virt_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evSendConfirm", "virt_WaitResvConf");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

	//TODO - FIXME - TMP
	if (lsp->isHead() &&
	    lsp->times().start_time &&
	    lsp->times().end_time) {
		struct timeval  t;
		long            timer;
		//Get current time
		gettimeofday(&t, NULL);
		//Get timer
		timer = lsp->times().end_time - t.tv_sec;
		thread_add_timer(RSVP_MASTER.getMaster(),
				 destroyAdvRsrvLsp,
				 lsp,
				 timer);
	}

#ifdef CRANKBACK_ON
	if (lsp->getInstance()->adjType() == INNI &&
	    lsp->isHead()                         &&
	    lsp->cbackScope() != CB_NONE          &&
	    (lsp->getExcludeEro()).size() > 0) {
		G2RSVPTE_DBG("Going to clear LSP exclude ERO");
		lsp->clearExcludeEro();
		return;
	}
#endif // CRANKBACK_ON

#ifdef PERSISTENCY_ON
	if (!lsp->getInstance()->persistency_.storeLsp(lsp)) {
		G2RSVPTE_ERR("Cannot store LSP data in Persistency Controller");
	}
#endif // PERSISTENCY_ON

}

void
Installed_i::after_evRecvConfirmOk_from_virt_WaitResvConf(void * context)
{

	DUMP_STATE2(name(), "evRecvConfirmOk", "virt_WaitResvConf");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

#ifdef PERSISTENCY_ON
	if (!lsp->getInstance()->persistency_.storeLsp(lsp)) {
		G2RSVPTE_ERR("Cannot store LSP data in Persistency Controller");
	}
#endif // PERSISTENCY_ON

}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_XConnCompleted_t
Installed_i::XConnCompleted(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "XConnCompleted");

	return (fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_XConnCompleted_t)1;
}

void
Installed_i::after_evXConnCompleted_from_virt_Rebuild(void * context)
{
	DUMP_STATE2(name(), "XConnCompleted", "virt_Rebuild");

	G2RSVPTE_DBG("LSP successfully restored: FSM is in Installed status");
};

void
Installed_i::after_evXConnCompleted_from_virt_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evXConnCompleted", "virt_Installed");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendPathDown_t
Installed_i::SendPathDown(void * context)
{
	DUMP_STATE(name(), "SendPathDown");
	return handleSendPathDown(context);
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvConfirm_t
Installed_i::RecvConfirm(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "RecvConfirm");

	return (fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvConfirm_t)1;
}

void
Installed_i::after_evRecvConfirmOk_from_virt_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvConfirmOk", "virt_Installed");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathDown_t
Installed_i::RecvPathDown(void * context)
{
	DUMP_STATE(name(), "RecvPathDown");
	return handleRecvPathDown(context);
}

void
Installed_i::after_evRecvPathDownKo_from_virt_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathDownKo", "virt_Installed");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPath_t
Installed_i::RecvPath(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "RecvPath");

	return (fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPath_t)1;
}

void
Installed_i::after_evRecvPathKo_from_virt_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathKo", "virt_Installed");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResvTear_t
Installed_i::RecvResvTear(void * context)
{
	DUMP_STATE(name(), "RecvResvTear");
	return handleRecvResvTear(context);
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathErr_t
Installed_i::RecvPathErr(void * context)
{
	DUMP_STATE(name(), "RecvPathErr");

	fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathErr_t res;
	res = handleRecvPathErr(context);

//	if (res == fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPathErr_to_evRecvPathErrOk) {
//		// Get the error
//		uint32_t nodeId;
//		uint8_t  flags;
//		uint8_t  errCode;
//		uint16_t errVal;

//		G2RSVPTE_LSP_FSM_context_t * ctxt;
//		ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;

//		// if handleRecvPathErr return PathErrOk no other checks needed!!!
//		RSVP::PathErrMessage * pathErr = static_cast<RSVP::PathErrMessage *>(ctxt->msg);

//		if (!pathErr->getErrorDetails(nodeId, flags, errCode, errVal)) {
//			G2RSVPTE_ERR("Invalid received PathErr msg");
//			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPathErr_to_evRecvPathErrKo;
//		}

//	}

	return res;
}

void
Installed_i::after_evRecvPathAlarm_from_virt_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathAlarm", "virt_Installed");
}

void
Installed_i::after_evRecvConfirmKo_from_virt_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvConfirmKo", "virt_Installed");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathTear_t
Installed_i::RecvPathTear(void * context)
{
	DUMP_STATE(name(), "RecvPathTear");
	return handleRecvPathTear(context);
}

void
Installed_i::after_evRecvPathTearKo_from_virt_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathTearKo", "virt_Installed");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathTimer_t
Installed_i::RecvPathTimer(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "RecvPathTimer");

	return (fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathTimer_t)1;
}

void
Installed_i::after_evRecvPathTimer_from_virt_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathTimer", "virt_Installed");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_XConnDown_t
Installed_i::XConnDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "XConnDown");

	return (fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_XConnDown_t)1;
}

void
Installed_i::after_evXConnDown_from_virt_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evXConnDown", "virt_Installed");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResv_t
Installed_i::RecvResv(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "RecvResv");

	return (fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResv_t)1;
}

void
Installed_i::after_evRecvResvOk_from_virt_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvOk", "virt_Installed");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResvDown_t
Installed_i::RecvResvDown(void * context)
{
	DUMP_STATE(name(), "RecvResvDown");
	return handleRecvResvDown(context);
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvNotify_t
Installed_i::RecvNotify(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "RecvNotify");

	return (fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvNotify_t)1;
}

void
Installed_i::after_evRecvNotifyOk_from_virt_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvNotifyOk", "virt_Installed");
}

void
Installed_i::after_evRecvResvKo_from_virt_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvKo", "virt_Installed");
}

void
Installed_i::after_evRecvPathErrKo_from_virt_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathErrKo", "virt_Installed");
}

void
Installed_i::after_evRecvPathOk_from_virt_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathOk", "virt_Installed");
}

void
Installed_i::after_evRecvActivateErr_from_virt_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvActivateErr", "virt_Installed");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResvTimer_t
Installed_i::RecvResvTimer(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "RecvResvTimer");

	return (fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResvTimer_t)1;
}

void
Installed_i::after_evRecvResvTimer_from_virt_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvTimer", "virt_Installed");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_XConnPreempt_t
Installed_i::XConnPreempt(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "XConnPreempt");

	return (fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_XConnPreempt_t)1;
}

void
Installed_i::after_evXConnPreempt_from_virt_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evXConnPreempt", "virt_Installed");
}

void
Installed_i::after_evRecvResvDownKo_from_virt_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvDownKo", "virt_Installed");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvActivate_t
Installed_i::RecvActivate(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "RecvActivate");

	return (fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvActivate_t)1;
}

void
Installed_i::after_evRecvActivateOk_from_virt_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvActivateOk", "virt_Installed");
}

void
Installed_i::after_evRecvResvTearKo_from_virt_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvTearKo", "virt_Installed");
}

void
Installed_i::after_evRecvNotifyKo_from_virt_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvNotifyKo", "virt_Installed");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResvErr_t
Installed_i::RecvResvErr(void * context)
{
	DUMP_STATE(name(), "RecvResvErr");
	return handleRecvResvErr(context);
}

void
Installed_i::after_evRecvResvErrKo_from_virt_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvErrKo", "virt_Installed");
}

void
Installed_i::after_evRecvActivateKo_from_virt_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvActivateKo", "virt_Installed");
}

void
Installed_i::after_evRecvResvErrOk_from_virt_Installed(void * context)
{
	DUMP_STATE2(name(), "evRecvResvErrOk", "virt_Installed");

#ifdef PERSISTENCY_ON
	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	assert(ctxt->msg);
	//assert(ctxt->intf);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	if (!lsp->getInstance()->persistency_.unstoreLsp(lsp)) {
		G2RSVPTE_ERR("Cannot unstore LSP data in Persistency Controller");
	}
#endif // PERSISTENCY_ON
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_XConnErr_t
Installed_i::XConnErr(void * context)
{
	DUMP_STATE(name(), "XConnErr");

	G2RSVPTE_NOT("The reservation activation is failed!");

	//TODO: send a Err here...

	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_XConnErr_to_evXConnErr;
}

void
Installed_i::after_evXConnErr_from_virt_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evXConnErr", "virt_Installed");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendResvDown_t
Installed_i::SendResvDown(void * context)
{
	DUMP_STATE(name(), "SendResvDown");
	return handleSendResvDown(context);
}

/*****************************************************************************
 *                    Class RebuildTearDown_i
 *****************************************************************************/

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendPathDown_t
RebuildTearDown_i::SendPathDown(void* context)
{
	DUMP_STATE(name(), "SendPathDown");
	return handleSendPathDown(context);
};

void
RebuildTearDown_i::after_evSendPathDown_from_virt_RebuildTearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evSendPathDown", "virt_RebuildTearDown");
};

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_XConnCompleted_t
RebuildTearDown_i::XConnCompleted(void* context)
{
	DUMP_STATE(name(), "XConnCompleted");

	G2RSVPTE_DBG("The XC is ready, but the LSP is being teared down. "
		     "THe XC will be destroyed");

	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_XConnCompleted_to_evXConnCompleted;
};

void
RebuildTearDown_i::after_evXConnCompleted_from_virt_RebuildTearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evXConnCompleted", "virt_RebuildTearDown");
};

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathErr_t
RebuildTearDown_i::RecvPathErr(void* context)
{
	DUMP_STATE(name(), "RecvPathErr");
	return handleRecvPathErr(context);
};

void
RebuildTearDown_i::after_evRecvPathErrKo_from_virt_RebuildTearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathErrKo", "virt_RebuildTearDown");
};

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendPathTear_t
RebuildTearDown_i::SendPathTear(void* context)
{
	DUMP_STATE(name(), "SendPathTear");
	return handleSendPathTear(context);
};

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_XConnErr_t
RebuildTearDown_i::XConnErr(void* context)
{
	DUMP_STATE(name(), "XConnErr");

	G2RSVPTE_DBG("The XC hasn't been created, but the LSP is being teared down.");

	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_XConnErr_to_evXConnErr;
};

void
RebuildTearDown_i::after_evXConnErr_from_virt_RebuildTearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evXConnErr", "virt_RebuildTearDown");
};

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResvDown_t
RebuildTearDown_i::RecvResvDown(void* context)
{
	DUMP_STATE(name(), "RecvResvDown");
	return handleRecvResvDown(context);
};

void
RebuildTearDown_i::after_evRecvResvDownKo_from_virt_RebuildTearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvDownKo", "virt_RebuildTearDown");
};

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathDown_t
RebuildTearDown_i::RecvPathDown(void* context)
{
	DUMP_STATE(name(), "RecvPathDown");
	return handleRecvPathDown(context);
};

void
RebuildTearDown_i::after_evRecvPathDownOk_from_virt_RebuildTearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathDownOk", "virt_RebuildTearDown");
};

void
RebuildTearDown_i::after_evRecvPathDownKo_from_virt_RebuildTearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathDownKo", "virt_RebuildTearDown");
};

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResvTear_t
RebuildTearDown_i::RecvResvTear(void* context)
{
	DUMP_STATE(name(), "RecvResvTear");
	return handleRecvResvTear(context);
};

void
RebuildTearDown_i::after_evRecvResvTearKo_from_virt_RebuildTearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvTearKo", "virt_RebuildTearDown");
};

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendResvTear_t
RebuildTearDown_i::SendResvTear(void* context)
{
	DUMP_STATE(name(), "SendResvTear");
	return handleSendResvTear(context);
};

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathTear_t
RebuildTearDown_i::RecvPathTear(void* context)
{
	DUMP_STATE(name(), "RecvPathTear");
	return handleRecvPathTear(context);
};

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResvErr_t
RebuildTearDown_i::RecvResvErr(void* context)
{
	DUMP_STATE(name(), "RecvResvErr");
	return handleRecvResvErr(context);
};

void
RebuildTearDown_i::after_evRecvResvErrKo_from_virt_RebuildTearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvErrKo", "virt_RebuildTearDown");
};

void
RebuildTearDown_i::after_evRecvPathTearKo_from_virt_RebuildTearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathTearKo", "virt_RebuildTearDown");
};

void
RebuildTearDown_i::after_evRecvResvErrOk_from_virt_RebuildTearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvErrOk", "virt_RebuildTearDown");
};

void
RebuildTearDown_i::after_evRecvResvDownOk_from_virt_RebuildTearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvDownOk", "virt_RebuildTearDown");
};

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendResvDown_t
RebuildTearDown_i::SendResvDown(void* context)
{
	DUMP_STATE(name(), "SendResvDown");
	return handleSendResvDown(context);
};

void
RebuildTearDown_i::after_evSendResvDown_from_virt_RebuildTearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evSendResvDown", "virt_RebuildTearDown");
};

void
RebuildTearDown_i::after_evSendPathDown_from_virt_Rebuild(void * context)
{
	DUMP_STATE2(name(), "evSendPathDown", "virt_Rebuild");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
};

void
RebuildTearDown_i::after_evRecvPathDownOk_from_virt_Rebuild(void * context)
{
	DUMP_STATE2(name(), "evRecvPathDownOk", "virt_Rebuild");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
};

void
RebuildTearDown_i::after_evRecvResvDownOk_from_virt_Rebuild(void * context)
{
	DUMP_STATE2(name(), "evRecvResvDownOk", "virt_Rebuild");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
};

void
RebuildTearDown_i::after_evSendResvDown_from_virt_Rebuild(void * context)
{
	DUMP_STATE2(name(), "evSendResvDown", "virt_Rebuild");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
};


/*****************************************************************************
 *                    Class PathReceived_i
 *****************************************************************************/

void
PathReceived_i::after_evRecvPathErrCrankback_from_virt_WaitResv(void * context)
{
	DUMP_STATE2(name(), "evRecvPathErrCrankback", "virt_WaitResv");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
}

void
PathReceived_i::after_evXConnErr_from_virt_WaitEqptConf(void * context)
{
	DUMP_STATE2(name(), "evXConnErr", "virt_WaitEqptConf");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);

	if (lsp->isTail()) {
		lsp->xConnFailed(true);
	}
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathTimeout_t
PathReceived_i::RecvPathTimeout(void * context)
{
	DUMP_STATE(name(), "RecvPathTimeout");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

	lsp->toBeDestroyed(true);
	postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPathTimeout_to_evRecvPathTimeout;
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_XConnCompleted_t
PathReceived_i::XConnCompleted(void * context)
{
	DUMP_STATE(name(), "XConnCompleted");

	G2RSVPTE_NOT("Ok, the XC is ready");

	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_XConnCompleted_to_evXConnCompleted;
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResv_t
PathReceived_i::RecvResv(void * context)
{
	DUMP_STATE(name(), "RecvResv");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	//assert(ctxt->msg);
	//assert(ctxt->intf);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->rsb());

	// Get session
	RSVP::Session * session = lsp->parent();
	assert(session);

	//
	// XXX FIXME: Add all missing check
	//

	fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResv_t res;
	res = handleFirstResv(context);
	if (res != fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResv_to_evRecvResvOk) {
		return res;
	}

	assert(lsp->rsb()->resvIn());
	// Set the refresh timer timeout
	if (!lsp->rsb()->resvIn()->time_values_) {
		G2RSVPTE_ERR("problems setting refresh timer timeout ... empty TIME_VALUES obj");
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResv_to_InvalidEvent;
	}
	//FIX!!! Change time_values type
	//       (TimeValues_Class, not TimeValues_Object)
	if (lsp->rsb()->resvIn()->time_values_->cType() != CTYPE_TIME_VALUES) {
		G2RSVPTE_ERR("problems time values CTYPE %d not supported", lsp->rsb()->resvIn()->time_values_->cType());
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResv_to_InvalidEvent;
	}
	lsp->rsb()->refreshTimeout(G2RSVPTE_DEFAULT_REFRESH_TIMEOUT *
				   (static_cast<RSVP::TimeValues_Object *>(lsp->rsb()->resvIn()->time_values_))->get().refresh_period);

	G2RSVPTE_INF("set a Resv refresh timeout = %u ms",
		     lsp->rsb()->refreshTimeout());

	// Start ZEBRA timer with the refresh timer timeout     => No, so far!!!
	// TODO: FIXME!!! start timer => lsp->rsb()->refreshTimeout()
	//                else => G2RSVPTE_WRN("could not start refresh-timeout timer; let's go on anyway");

	if (lsp->isHead()) {
		G2RSVPTE_INF("I will notify a Resv Event to RC as soon as the XC is ready");
	} else {
		// Set a random refresh timer
		// If we want to use peer refresh period
		//lsp->rsb()->refreshInterval(lsp->rsb()->resvIn()->time_values_->get().refresh_period);
		// else
		lsp->rsb()->refreshInterval(lsp->refreshInterval());

		G2RSVPTE_INF("set a Resv refresh interval = %u ms",
			     lsp->rsb()->refreshInterval());

		// Start ZEBRA timer with the refresh timer   => No, so far!!!
		// TODO: FIXME!!! start timer => lsp->rsb()->refreshInterval()
		//                else => G2RSVPTE_WRN("could not start refresh-interval timer; let's go on anyway");

		// we won't send the message now, but when the XC is ready
		G2RSVPTE_INF("waiting for the XC before forwarding Resv msg to NE %s through TE link %s",
			     numb_htoa(lsp->psb()->nodeId()),
			     addr_ntoa(lsp->psb()->interface()->localId()));
	}

	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResv_to_evRecvResvOk;
}

void
PathReceived_i::after_evRecvResvKo_from_virt_PathReceived(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvKo", "virt_PathReceived");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathErr_t
PathReceived_i::RecvPathErr(void * context)
{
	DUMP_STATE(name(), "RecvPathErr");
	return handleRecvPathErr(context);
}

void
PathReceived_i::after_evRecvPathErrKo_from_virt_PathReceived(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathErrKo", "virt_PathReceived");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPath_t
PathReceived_i::RecvPath(void * context)
{
	DUMP_STATE(name(), "RecvPath");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	assert(ctxt->msg);
	//assert(ctxt->intf);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->psb());

	// Check if msg is a PathMessage
	// TODO: check msg type!!!
	RSVP::PathMessage * path = (RSVP::PathMessage *)ctxt->msg;

	// check if this pkg is a refresh msg
	if (1) {
		G2RSVPTE_INF("received a refresh Path");

		// Set the refresh timer timeout
		if (!path->time_values_) {
			G2RSVPTE_ERR("problems setting refresh timer timeout ... empty TIME_VALUES obj");
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_InvalidEvent;
		}
		//FIX!!! Change time_values type
		//       (TimeValues_Class, not TimeValues_Object)
		if (path->time_values_->cType() != CTYPE_TIME_VALUES) {
			G2RSVPTE_ERR("problems time values CTYPE %d not supported", path->time_values_->cType());
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_InvalidEvent;
		}
		lsp->psb()->refreshTimeout(G2RSVPTE_DEFAULT_REFRESH_TIMEOUT *
					   (static_cast<RSVP::TimeValues_Object *>(path->time_values_))->get().refresh_period);

		// restart the refresh-timeout ZEBRA timer => No, so far!!!
		// TODO: FIXME!!! first clear/reset timer, after
		//                start timer => with lsp->psb()->refreshTimeout()
		//                else => G2RSVPTE_WRN("could not start refresh-timeout timer; let's go on anyway");

		delete path;

		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_evRecvPathOk;
	} else {

		G2RSVPTE_WRN("this is not refresh Path!!!");
		delete path;

		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_evRecvPathKo;
	}
}

void
PathReceived_i::after_evRecvPathOk_from_virt_PathReceived(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathOk", "virt_PathReceived");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendPathTear_t
PathReceived_i::SendPathTear(void * context)
{
	DUMP_STATE(name(), "SendPathTear");
	return handleSendPathTear(context);
}

void
PathReceived_i::after_evRecvPathErrCrankback_from_virt_PathReceived(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathErrCrankback", "virt_PathReceived");
}

void
PathReceived_i::after_evRecvPathKo_from_virt_PathReceived(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathKo", "virt_PathReceived");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendResvTear_t
PathReceived_i::SendResvTear(void * context)
{
	DUMP_STATE(name(), "SendResvTear");
	return handleSendResvTear(context);
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathTear_t
PathReceived_i::RecvPathTear(void * context)
{
	DUMP_STATE(name(), "RecvPathTear");
	return handleRecvPathTear(context);
}

void
PathReceived_i::after_evRecvPathTearKo_from_virt_PathReceived(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathTearKo", "virt_PathReceived");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathTimer_t
PathReceived_i::RecvPathTimer(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "RecvPathTimer");

	return (fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathTimer_t)1;
}

void
PathReceived_i::after_evRecvPathTimer_from_virt_PathReceived(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathTimer", "virt_PathReceived");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_XConnErr_t
PathReceived_i::XConnErr(void * context)
{
	DUMP_STATE(name(), "XConnErr");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

	if (lsp->isTail()) {
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_XConnErr_to_evXConnErr;
	} else {
		return handleXConnErr(context);
	}
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendResv_t
PathReceived_i::SendResv(void * context)
{
	DUMP_STATE(name(), "SendResv");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->psb());
	assert(lsp->rsb());

	if (lsp->isHead()) {
		G2RSVPTE_INF("I will notify a Resv Event to RC as soon as the XC is ready");
	} else {
		// Set a random refresh timer
		lsp->rsb()->refreshInterval(lsp->refreshInterval());

		G2RSVPTE_INF("set a Resv refresh interval = %u ms",
			     lsp->rsb()->refreshInterval());

		// Start ZEBRA timer with the refresh timer   => No, so far!!!
		// TODO: FIXME!!! start timer => lsp->rsb()->refreshInterval()
		//                else => G2RSVPTE_WRN("could not start refresh-interval timer; let's go on anyway");

		G2RSVPTE_INF("I will send first Resv msg to NE %s through TE link %s as soon as the XC is ready",
			     numb_htoa(lsp->psb()->nodeId()),
			     addr_ntoa(lsp->psb()->interface()->localId()));
	}

	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_SendResv_to_evSendResv;
}

void
PathReceived_i::after_evSendPath_from_virt_Down(void * context)
{
	DUMP_STATE2(name(), "evSendPath", "virt_Down");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
}

void
PathReceived_i::after_evRecvPathOk_from_virt_Down(void * context)
{
	DUMP_STATE2(name(), "evRecvPathOk", "virt_Down");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendPathDown_t
PathReceived_i::SendPathDown(void* context)
{
	DUMP_STATE(name(), "SendPathDown");
	return handleSendPathDown(context);
};

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResvDown_t
PathReceived_i::RecvResvDown(void* context)
{
	DUMP_STATE(name(), "RecvResvDown");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->rsb());
	assert(lsp->psb()->pathIn());

	if (!lsp->rsb()->resvOut()) {
		RSVP::ResvMessage * resv;
		resv = new RSVP::ResvMessage(*lsp->psb()->pathIn());
		if (!resv) {
			G2RSVPTE_ERR("Impossible to create ResvOut "
				     "for LSP id %d", lsp->id());
				return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResvDown_to_evRecvResvDownKo;
		}
		// Initialise ResvOut for this new LSP
		lsp->rsb()->resvOut(resv);

		// prepare NHOP
		if (!lsp->prepareNHOPobj()) {
			G2RSVPTE_ERR("Impossible to prepare Resv Out NHOP obj");
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResvDown_to_evRecvResvDownKo;
		}
	}

	return handleRecvResvDown(context);
};

void
PathReceived_i::after_evRecvResvDownKo_from_virt_PathReceived(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvDownKo", "virt_PathReceived");
};

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathDown_t
PathReceived_i::RecvPathDown(void* context)
{
	DUMP_STATE(name(), "RecvPathDown");
	return handleRecvPathDown(context);
};

void
PathReceived_i::after_evRecvPathDownKo_from_virt_PathReceived(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathDownKo", "virt_PathReceived");
};

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendResvDown_t
PathReceived_i::SendResvDown(void* context)
{
	DUMP_STATE(name(), "SendResvDown");
	return handleSendResvDown(context);
};


/*****************************************************************************
 *                    Class TearDown_i
 *****************************************************************************/

void
TearDown_i::after_evRecvPathDownOk_from_virt_WaitResvConf(void * context)
{
	DUMP_STATE2(name(), "evRecvPathDownOk", "virt_WaitResvConf");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
}

void
TearDown_i::after_evSendPathDown_from_virt_Installed(void * context)
{
	DUMP_STATE2(name(), "evSendPathDown", "virt_Installed");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
}

void
TearDown_i::after_evRecvResvDownOk_from_virt_Installed(void * context)
{
	DUMP_STATE2(name(), "evRecvResvDownOk", "virt_Installed");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
}

void
TearDown_i::after_evRecvPathDownOk_from_virt_Installed(void * context)
{
	DUMP_STATE2(name(), "evRecvPathDownOk", "virt_Installed");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
}

void
TearDown_i::after_evSendResvDown_from_virt_Installed(void * context)
{
	DUMP_STATE2(name(), "evSendResvDown", "virt_Installed");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathTimeout_t
TearDown_i::RecvPathTimeout(void * context)
{
	DUMP_STATE(name(), "RecvPathTimeout");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

	lsp->toBeDestroyed(true);
	postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPathTimeout_to_evRecvPathTimeout;
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendPathDown_t
TearDown_i::SendPathDown(void * context)
{
	DUMP_STATE(name(), "SendPathDown");
	return handleSendPathDown(context);
}

void
TearDown_i::after_evSendPathDown_from_virt_TearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evSendPathDown", "virt_TearDown");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathErr_t
TearDown_i::RecvPathErr(void * context)
{
	DUMP_STATE(name(), "RecvPathErr");
	return handleRecvPathErr(context);
}

void
TearDown_i::after_evRecvPathErrKo_from_virt_TearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathErrKo", "virt_TearDown");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendPathTear_t
TearDown_i::SendPathTear(void * context)
{
	DUMP_STATE(name(), "SendPathTear");
	return handleSendPathTear(context);
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResvDown_t
TearDown_i::RecvResvDown(void * context)
{
	DUMP_STATE(name(), "RecvResvDown");
	return handleRecvResvDown(context);
}

void
TearDown_i::after_evRecvResvDownKo_from_virt_TearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvDownKo", "virt_TearDown");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathDown_t
TearDown_i::RecvPathDown(void * context)
{
	DUMP_STATE(name(), "RecvPathDown");
	return handleRecvPathDown(context);
}

void
TearDown_i::after_evRecvPathDownOk_from_virt_TearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathDownOk", "virt_TearDown");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResvTimeout_t
TearDown_i::RecvResvTimeout(void * context)
{
	DUMP_STATE(name(), "RecvResvTimeout");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

	lsp->toBeDestroyed(true);
	postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvResvTimeout_to_evRecvResvTimeout;
}

void
TearDown_i::after_evRecvPathDownKo_from_virt_TearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathDownKo", "virt_TearDown");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResvTear_t
TearDown_i::RecvResvTear(void * context)
{
	DUMP_STATE(name(), "RecvResvTear");
	return handleRecvResvTear(context);
}

void
TearDown_i::after_evRecvResvTearKo_from_virt_TearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvTearKo", "virt_TearDown");
}

void
TearDown_i::after_evRecvPathAlarm_from_virt_TearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathAlarm", "virt_TearDown");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendResvTear_t
TearDown_i::SendResvTear(void * context)
{
	DUMP_STATE(name(), "SendResvTear");
	return handleSendResvTear(context);
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPathTear_t
TearDown_i::RecvPathTear(void * context)
{
	DUMP_STATE(name(), "RecvPathTear");
	return handleRecvPathTear(context);
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvResvErr_t
TearDown_i::RecvResvErr(void * context)
{
	DUMP_STATE(name(), "RecvResvErr");
	return handleRecvResvErr(context);
}

void
TearDown_i::after_evRecvResvErrKo_from_virt_TearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvErrKo", "virt_TearDown");
}

void
TearDown_i::after_evRecvPathTearKo_from_virt_TearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathTearKo", "virt_TearDown");
}

void
TearDown_i::after_evRecvResvErrOk_from_virt_TearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvErrOk", "virt_TearDown");
}

void
TearDown_i::after_evRecvResvDownOk_from_virt_TearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvDownOk", "virt_TearDown");
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendResvDown_t
TearDown_i::SendResvDown(void * context)
{
	DUMP_STATE(name(), "SendResvDown");
	return handleSendResvDown(context);
}

void
TearDown_i::after_evSendResvDown_from_virt_TearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evSendResvDown", "virt_TearDown");
}

void
TearDown_i::after_evSendPathDown_from_virt_WaitResvConf(void * context)
{
	DUMP_STATE2(name(), "evSendPathDown", "virt_WaitResvConf");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
};

void
TearDown_i::after_evSendResvDown_from_virt_WaitResvConf(void * context)
{
	DUMP_STATE2(name(), "evSendResvDown", "virt_WaitResvConf");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
};

void
TearDown_i::after_evRecvResvDownOk_from_virt_WaitResvConf(void * context)
{
	DUMP_STATE2(name(), "evRecvResvDownOk", "virt_WaitResvConf");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
};

void
TearDown_i::after_evSendPathDown_from_virt_WaitEqptConf(void * context)
{
	DUMP_STATE2(name(), "evSendPathDown", "virt_WaitEqptConf");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
};

void
TearDown_i::after_evRecvPathDownOk_from_virt_WaitEqptConf(void * context)
{
	DUMP_STATE2(name(), "evRecvPathDownOk", "virt_WaitEqptConf");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
};

void
TearDown_i::after_evSendResvDown_from_virt_WaitEqptConf(void * context)
{
	DUMP_STATE2(name(), "evSendResvDown", "virt_WaitEqptConf");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
};

void
TearDown_i::after_evRecvResvDownOk_from_virt_WaitEqptConf(void * context)
{
	DUMP_STATE2(name(), "evRecvResvDownOk", "virt_WaitEqptConf");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
};

void
TearDown_i::after_evSendPathDown_from_virt_PathReceived(void * context)
{
	DUMP_STATE2(name(), "evSendPathDown", "virt_PathReceived");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
};

void
TearDown_i::after_evRecvPathDownOk_from_virt_PathReceived(void * context)
{
	DUMP_STATE2(name(), "evRecvPathDownOk", "virt_PathReceived");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
};

void
TearDown_i::after_evSendResvDown_from_virt_PathReceived(void * context)
{
	DUMP_STATE2(name(), "evSendResvDown", "virt_PathReceived");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
};

void
TearDown_i::after_evRecvResvDownOk_from_virt_PathReceived(void * context)
{
	DUMP_STATE2(name(), "evRecvResvDownOk", "virt_PathReceived");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
};

void
TearDown_i::after_evSendPathDown_from_virt_WaitResv(void * context)
{
	DUMP_STATE2(name(), "evSendPathDown", "virt_WaitResv");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
};

void
TearDown_i::after_evRecvPathDownOk_from_virt_WaitResv(void * context)
{
	DUMP_STATE2(name(), "evRecvPathDownOk", "virt_WaitResv");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
};

void
TearDown_i::after_evSendResvDown_from_virt_WaitResv(void * context)
{
	DUMP_STATE2(name(), "evSendResvDown", "virt_WaitResv");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
};

void
TearDown_i::after_evRecvResvDownOk_from_virt_WaitResv(void * context)
{
	DUMP_STATE2(name(), "evRecvResvDownOk", "virt_WaitResv");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Initialise timer for this FSM state
	FSMtimerInit(lsp);
};


/*****************************************************************************
 *                    Class Down_i
 *****************************************************************************/

void
Down_i::after_evSendPathTear_from_virt_WaitResvConf(void * context)
{
	DUMP_STATE2(name(), "evSendPathTear", "virt_WaitResvConf");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

	if (!lsp->isTail()) {
		G2RSVPTE_DBG("Deletion of LSP after SendPathTear in head/core node!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
}

void
Down_i::after_evRecvResvTearOk_from_virt_WaitResvConf(void * context)
{
	DUMP_STATE2(name(), "evRecvResvTearOk", "virt_WaitResvConf");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

	if (!lsp->isTail()) {
		G2RSVPTE_DBG("Deletion of LSP after RecvResvTearOK in head/core node!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
}

void
Down_i::after_evRecvPathAlarm_from_virt_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathAlarm", "virt_WaitResvConf");
}

void
Down_i::after_evSendResvTear_from_virt_WaitResvConf(void * context)
{
	DUMP_STATE2(name(), "evSendResvTear", "virt_WaitResvConf");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

	if (!lsp->isHead()) {
		G2RSVPTE_DBG("Deletion of LSP after SendResvTear in tail/core node!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
}

void
Down_i::after_evRecvPathTearOk_from_virt_WaitResvConf(void * context)
{
	DUMP_STATE2(name(), "evRecvPathTearOk", "virt_WaitResvConf");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

	if (!lsp->isHead()) {
		G2RSVPTE_DBG("Deletion of LSP after RecvPathTearOK in tail/core node!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
}

void
Down_i::after_evRecvPathErrOk_from_virt_WaitResvConf(void * context)
{
	DUMP_STATE2(name(), "evRecvPathErrOk", "virt_WaitResvConf");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

#ifdef CRANKBACK_ON
	if (lsp->getInstance()->adjType() == INNI &&
	    lsp->isHead()                         &&
	    lsp->cbackScope() != CB_NONE          &&
	    (lsp->getExcludeEro()).size() > 0) {
		if (lsp->currentCbackAttempt() <
		    lsp->maxCbackRetries()) {
			G2RSVPTE_DBG("LSP in Down state (not destroyed). "
				     "Going to try next crankback attempt");
			if (!prepareNextCbackAttempt(context)) {
				G2RSVPTE_WRN("Cannot try another crankback "
					     "attempt. Going to notify RC about failure.");
				postLspErrEvent(lsp, true);
				if (lsp->tnrcCookie()) {
					// Request a resource release at the TNRC
					if (!releaseResource(lsp)) {
						G2RSVPTE_ERR("Impossible to release resource");
					}
					//set tnrcCookie to 0
					lsp->tnrcCookie(0);
				}
			} else {
				return;
			}
		}
	}
#endif // CRANKBACK_ON

	//if isHead (not INNI) LSP destroy must be called manually (VTY/upper layers)
	if (!lsp->isTail() && ((lsp->getInstance()->adjType() == INNI) || !lsp->isHead())) {
		G2RSVPTE_DBG("Deletion of LSP after RecvPathErr!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
}

void
Down_i::after_evRecvPathTimeout_from_virt_WaitResv(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathTimeout", "virt_WaitResv");
}

void
Down_i::after_evSendPathTear_from_virt_WaitResv(void * context)
{
	DUMP_STATE2(name(), "evSendPathTear", "virt_WaitResv");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

	if (!lsp->isTail()) {
		G2RSVPTE_DBG("Deletion of LSP after SendPathTear in head/core node!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
}

void
Down_i::after_evRecvResvVeryKo_from_virt_WaitResv(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvVeryKo", "virt_WaitResv");
}

void
Down_i::after_evRecvResvTearOk_from_virt_WaitResv(void * context)
{
	DUMP_STATE2(name(), "evRecvResvTearOk", "virt_WaitResv");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

	if (!lsp->isTail()) {
		G2RSVPTE_DBG("Deletion of LSP after RecvResvTearOK in head/core node!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
}

void
Down_i::after_evRecvPathAlarm_from_virt_WaitResv(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathAlarm", "virt_WaitResv");
}

void
Down_i::after_evSendResvTear_from_virt_WaitResv(void * context)
{
	DUMP_STATE2(name(), "evSendResvTear", "virt_WaitResv");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

	if (!lsp->isHead()) {
		G2RSVPTE_DBG("Deletion of LSP after SendResvTear in tail/core node!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
}

void
Down_i::after_evRecvPathTearOk_from_virt_WaitResv(void * context)
{
	DUMP_STATE2(name(), "evRecvPathTearOk", "virt_WaitResv");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

	if (!lsp->isHead()) {
		G2RSVPTE_DBG("Deletion of LSP after RecvPathTearOK in tail/core node!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
}

void
Down_i::after_evRecvPathErrOk_from_virt_WaitResv(void * context)
{
	DUMP_STATE2(name(), "evRecvPathErrOk", "virt_WaitResv");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

#ifdef CRANKBACK_ON
	if (lsp->getInstance()->adjType() == INNI &&
	    lsp->isHead()                         &&
	    lsp->cbackScope() != CB_NONE          &&
	    (lsp->getExcludeEro()).size() > 0) {
		if (lsp->currentCbackAttempt() <
		    lsp->maxCbackRetries()) {
			G2RSVPTE_DBG("LSP in Down state (not destroyed). "
				     "Going to try next crankback attempt");
			if (!prepareNextCbackAttempt(context)) {
				G2RSVPTE_WRN("Cannot try another crankback "
					     "attempt. Going to notify RC about failure.");
				postLspErrEvent(lsp, true);
				if (lsp->tnrcCookie()) {
					// Request a resource release at the TNRC
					if (!releaseResource(lsp)) {
						G2RSVPTE_ERR("Impossible to release resource");
					}
					//set tnrcCookie to 0
					lsp->tnrcCookie(0);
				}
			} else {
				return;
			}
		}
	}
#endif // CRANKBACK_ON

	//if isHead (not INNI) LSP destroy must be called manually (VTY/upper layers)
	if (!lsp->isTail() && ((lsp->getInstance()->adjType() == INNI) || !lsp->isHead())) {
		G2RSVPTE_DBG("Deletion of LSP after RecvPathErr!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
}

void
Down_i::after_evRecvPathTimeout_from_virt_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathTimeout", "virt_WaitEqptConf");
}

void
Down_i::after_evSendPathTear_from_virt_WaitEqptConf(void * context)
{
	DUMP_STATE2(name(), "evSendPathTear", "virt_WaitEqptConf");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

	if (!lsp->isTail()) {
		G2RSVPTE_DBG("Deletion of LSP after SendPathTear in head/core node!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
}

void
Down_i::after_evRecvResvTimeout_from_virt_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvTimeout", "virt_WaitEqptConf");
}

void
Down_i::after_evRecvResvVeryKo_from_virt_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvVeryKo", "virt_WaitEqptConf");
}

void
Down_i::after_evRecvResvTearOk_from_virt_WaitEqptConf(void * context)
{
	DUMP_STATE2(name(), "evRecvResvTearOk", "virt_WaitEqptConf");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

	if (!lsp->isTail()) {
		G2RSVPTE_DBG("Deletion of LSP after RecvResvTearOK in head/core node!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
}

void
Down_i::after_evSendResvTear_from_virt_WaitEqptConf(void * context)
{
	DUMP_STATE2(name(), "evSendResvTear", "virt_WaitEqptConf");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

	if (!lsp->isHead()) {
		G2RSVPTE_DBG("Deletion of LSP after SendResvTear in tail/core node!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
}

void
Down_i::after_evRecvPathTearOk_from_virt_WaitEqptConf(void * context)
{
	DUMP_STATE2(name(), "evRecvPathTearOk", "virt_WaitEqptConf");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

	if (!lsp->isHead()) {
		G2RSVPTE_DBG("Deletion of LSP after RecvPathTearOK in tail/core node!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
}

void
Down_i::after_evRecvPathErrOk_from_virt_WaitEqptConf(void * context)
{
	DUMP_STATE2(name(), "evRecvPathErrOk", "virt_WaitEqptConf");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

#ifdef CRANKBACK_ON
	if (lsp->getInstance()->adjType() == INNI &&
	    lsp->isHead()                         &&
	    lsp->cbackScope() != CB_NONE          &&
	    (lsp->getExcludeEro()).size() > 0) {
		if (lsp->currentCbackAttempt() <
		    lsp->maxCbackRetries()) {
			G2RSVPTE_DBG("LSP in Down state (not destroyed). "
				     "Going to try next crankback attempt");
			if (!prepareNextCbackAttempt(context)) {
				G2RSVPTE_WRN("Cannot try another crankback "
					     "attempt. Going to notify RC about failure.");
				postLspErrEvent(lsp, true);
				if (lsp->tnrcCookie()) {
					// Request a resource release at the TNRC
					if (!releaseResource(lsp)) {
						G2RSVPTE_ERR("Impossible to release resource");
					}
					//set tnrcCookie to 0
					lsp->tnrcCookie(0);
				}
			} else {
				return;
			}
		}
	}
#endif // CRANKBACK_ON

	//if isHead (not INNI) LSP destroy must be called manually (VTY/upper layers)
	if (!lsp->isTail() && ((lsp->getInstance()->adjType() == INNI) || !lsp->isHead())) {
		G2RSVPTE_DBG("Deletion of LSP after RecvPathErr!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
}

void
Down_i::after_evRecvPathErrOk_from_virt_Rebuild(void * context)
{
	DUMP_STATE2(name(), "evRecvPathErrOk", "virt_Rebuild");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

#ifdef PERSISTENCY_ON
	if (!lsp->getInstance()->persistency_.unstoreLsp(lsp)) {
		G2RSVPTE_ERR("Cannot unstore LSP data in Persistency Controller");
	}
#endif // PERSISTENCY_ON

	//if isHead (not INNI) LSP destroy must be called manually (VTY/upper layers)
	if (!lsp->isTail() && ((lsp->getInstance()->adjType() == INNI) || !lsp->isHead())) {
		G2RSVPTE_DBG("Deletion of LSP after RecvPathErr!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
};

void
Down_i::after_evXConnErr_from_virt_Rebuild(void * context)
{
	DUMP_STATE2(name(), "XConnErr", "virt_Rebuild");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

#ifdef PERSISTENCY_ON
	if (!lsp->getInstance()->persistency_.unstoreLsp(lsp)) {
		G2RSVPTE_ERR("Cannot unstore LSP data in Persistency Controller");
	}
#endif // PERSISTENCY_ON

	G2RSVPTE_DBG("Deletion of LSP after XConnErr!");
	lsp->toBeDestroyed(true);
	postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
};

void
Down_i::after_evRecvResvTearOk_from_virt_Installed(void * context)
{
	DUMP_STATE2(name(), "evRecvResvTearOk", "virt_Installed");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

#ifdef PERSISTENCY_ON
	if (!lsp->getInstance()->persistency_.unstoreLsp(lsp)) {
		G2RSVPTE_ERR("Cannot unstore LSP data in Persistency Controller");
	}
#endif // PERSISTENCY_ON

	if (!lsp->isTail()) {
		G2RSVPTE_DBG("Deletion of LSP after RecvResvTearOK in head/core node!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
}

void
Down_i::after_evRecvPathErrOk_from_virt_Installed(void * context)
{
	DUMP_STATE2(name(), "evRecvPathErrOk", "virt_Installed");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

#ifdef PERSISTENCY_ON
	if (!lsp->getInstance()->persistency_.unstoreLsp(lsp)) {
		G2RSVPTE_ERR("Cannot unstore LSP data in Persistency Controller");
	}
#endif // PERSISTENCY_ON

	//if isHead (not INNI) LSP destroy must be called manually (VTY/upper layers)
	if (!lsp->isTail() && ((lsp->getInstance()->adjType() == INNI) || !lsp->isHead())) {
		G2RSVPTE_DBG("Deletion of LSP after RecvPathErr!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
}

void
Down_i::after_evRecvNotifyDown_from_virt_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvNotifyDown", "virt_Installed");
}

void
Down_i::after_evRecvResvVeryKo_from_virt_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvResvVeryKo", "virt_Installed");
}

void
Down_i::after_evRecvPathTearOk_from_virt_Installed(void * context)
{
	DUMP_STATE2(name(), "evRecvPathTearOk", "virt_Installed");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

#ifdef PERSISTENCY_ON
	if (!lsp->getInstance()->persistency_.unstoreLsp(lsp)) {
		G2RSVPTE_ERR("Cannot unstore LSP data in Persistency Controller");
	}
#endif // PERSISTENCY_ON

	if (!lsp->isHead()) {
		G2RSVPTE_DBG("Deletion of LSP after RecvPathTearOK in tail/core node!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_LspRebuild_t
Down_i::LspRebuild(void* context)
{
	DUMP_STATE(name(), "LspRebuild");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());
	assert(lsp->psb());
	assert(lsp->psb()->pathIn());
	assert(lsp->psb()->pathOut());

	// Complete pathOut message
	// prepare the PHOP in the outgoing Path
	if (!lsp->isTail() && !lsp->preparePHOPobj()) {
		G2RSVPTE_ERR("Cannot prepare PHOP Object");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_LspRebuild_to_InvalidEvent;
	}

	// prepare the LABEL_SET obj in the outgoing Path
	//  (using downstream tx label)
	if (!lsp->prepareLabelSetObj()) {
		G2RSVPTE_ERR("Cannot prepare LabelSet Object");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_LspRebuild_to_InvalidEvent;
	}

	// prepare the UPSTREAM_LABEL obj in the outgoing Path
	//  (using downstream rx label)
	if (!lsp->prepareUpstreamLabelObj()) {
		G2RSVPTE_ERR("Cannot prepare UpstreamLabel Object");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_LspRebuild_to_InvalidEvent;
	}

	// create ResvOut message
	if (!lsp->rsb()->resvOut()) {
		RSVP::ResvMessage * resv;
		resv = new RSVP::ResvMessage(*lsp->psb()->pathIn());
		if (!resv) {
			G2RSVPTE_ERR("Impossible to create ResvOut message");
			lsp->toBeDestroyed(true);
			postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_LspRebuild_to_InvalidEvent;
		}
		// Initialise ResvOut for this new LSP
		lsp->rsb()->resvOut(resv);
	}

	assert(lsp->rsb()->resvOut()->time_values_);
	// set time_values
	if (lsp->rsb()->resvOut()->time_values_->cType() != CTYPE_TIME_VALUES) {
		G2RSVPTE_ERR("cType %d not supported for TimeValue Obj",
			     lsp->rsb()->resvOut()->time_values_->cType());
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_LspRebuild_to_InvalidEvent;
	}
	static_cast<RSVP::TimeValues_Object *>
		(lsp->rsb()->resvOut()->time_values_)->get().refresh_period = lsp->refreshInterval();

	assert(lsp->rsb()->resvOut()->style_);
	// set style
	if (lsp->rsb()->resvOut()->style_->cType() != CTYPE_STYLE) {
		G2RSVPTE_ERR("cType %d not supported for Style Obj",
			     lsp->rsb()->resvOut()->style_->cType());
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_LspRebuild_to_InvalidEvent;
	}
	static_cast<RSVP::Style_Object *>(lsp->rsb()->resvOut()->style_)->
		setFFstyle();
	static_cast<RSVP::Style_Object *>(lsp->rsb()->resvOut()->style_)->
		flags(0);

	// prepare NHOP
	if (!lsp->isHead() && !lsp->prepareNHOPobj()) {
		G2RSVPTE_ERR("Impossible to prepare Resv NHOP obj");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_LspRebuild_to_InvalidEvent;
	}

	assert(lsp->rsb()->resvOut()->resv_confirm_);
	// set resv_confirm
	if (lsp->rsb()->resvOut()->resv_confirm_->cType() !=
	    CTYPE_RESVCONF_IPV4) {
		G2RSVPTE_ERR("cType %d not supported for ResvConf Obj",
			     lsp->rsb()->resvOut()->resv_confirm_->cType());
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_LspRebuild_to_InvalidEvent;
	}

	static_cast<RSVP::ResvConfirmIPv4_Object *>
		(lsp->rsb()->resvOut()->resv_confirm_)->get().ipv4_addr.
		s_addr = htonl(lsp->getInstance()->nodeId());

	// prepare label obj
	if (!lsp->prepareLabelObj()) {
		G2RSVPTE_ERR("Impossible to prepare Resv LABEL obj");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_LspRebuild_to_InvalidEvent;
	}

	assert(lsp->rsb()->resvOut()->notify_request_);
	if (lsp->rsb()->resvOut()->notify_request_->cType() !=
	    CTYPE_IPV4_NOTIFY_REQUEST) {
		G2RSVPTE_ERR("cType %d not supported for ResvConf Obj",
			     lsp->rsb()->resvOut()->notify_request_->cType());
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_LspRebuild_to_InvalidEvent;
	}
	static_cast<RSVP::IPV4NotifyRequest_Object *>
		(lsp->rsb()->resvOut()->notify_request_)->get().ipv4_addr.
		s_addr = htonl(lsp->getInstance()->nodeId());

	// create ResvIn message
	RSVP::ResvMessage * tmpResv;
	tmpResv = lsp->rsb()->resvOut()->partialClone();
	lsp->rsb()->resvIn(tmpResv);

	if (!lsp->rsb()->resvIn()) {
		G2RSVPTE_ERR("Impossible to copy ResvOut to get ResvIn ...");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_LspRebuild_to_InvalidEvent;
	}

	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_LspRebuild_to_evLspRebuild;
};

void
Down_i::after_evRecvPathTimeout_from_virt_PathReceived(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "evRecvPathTimeout", "virt_PathReceived");
}

void
Down_i::after_evSendPathTear_from_virt_PathReceived(void * context)
{
	DUMP_STATE2(name(), "evSendPathTear", "virt_PathReceived");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

	if (!lsp->isTail()) {
		G2RSVPTE_DBG("Deletion of LSP after SendPathTear in head/core node!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
}

void
Down_i::after_evSendResvTear_from_virt_PathReceived(void * context)
{
	DUMP_STATE2(name(), "evSendResvTear", "virt_PathReceived");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

	if (!lsp->isHead()) {
		G2RSVPTE_DBG("Deletion of LSP after SendResvTear in tail/core node!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
}

void
Down_i::after_evRecvPathTearOk_from_virt_PathReceived(void * context)
{
	DUMP_STATE2(name(), "evRecvPathTearOk", "virt_PathReceived");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

	if (!lsp->isHead()) {
		G2RSVPTE_DBG("Deletion of LSP after RecvPathTearOK in tail/core node!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
}

void
Down_i::after_evRecvPathErrOk_from_virt_PathReceived(void * context)
{
	DUMP_STATE2(name(), "evRecvPathErrOk", "virt_PathReceived");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

#ifdef CRANKBACK_ON
	if (lsp->getInstance()->adjType() == INNI &&
	    lsp->isHead()                         &&
	    lsp->cbackScope() != CB_NONE          &&
	    (lsp->getExcludeEro()).size() > 0) {
		if (lsp->currentCbackAttempt() <
		    lsp->maxCbackRetries()) {
			G2RSVPTE_DBG("LSP in Down state (not destroyed). "
				     "Going to try next crankback attempt");
			if (!prepareNextCbackAttempt(context)) {
				G2RSVPTE_WRN("Cannot try another crankback "
					     "attempt. Going to notify RC about failure.");
				postLspErrEvent(lsp, true);
				if (lsp->tnrcCookie()) {
					// Request a resource release at the TNRC
					if (!releaseResource(lsp)) {
						G2RSVPTE_ERR("Impossible to release resource");
					}
					//set tnrcCookie to 0
					lsp->tnrcCookie(0);
				}
			} else {
				return;
			}
		}
	}
#endif // CRANKBACK_ON

	//if isHead (not INNI) LSP destroy must be called manually (VTY/upper layers)
	if (!lsp->isTail() && ((lsp->getInstance()->adjType() == INNI) || !lsp->isHead())) {
		G2RSVPTE_DBG("Deletion of LSP after RecvPathErr!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
}

void
Down_i::after_evXConnErr_from_virt_PathReceived(void * context)
{
	DUMP_STATE2(name(), "evXConnErr", "virt_PathReceived");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

	//if isHead LSP (not INNI) destroy must be called manually (VTY/upper layers)
	if ((lsp->getInstance()->adjType() == INNI) || !lsp->isHead()) {
		G2RSVPTE_DBG("Deletion of LSP after evXConnErr in tail/core node!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
}

void
Down_i::after_evRecvPathTimeout_from_virt_TearDown(void * context)
{
	DUMP_STATE2(name(), "evRecvPathTimeout", "virt_TearDown");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

#ifdef PERSISTENCY_ON
	if (!lsp->getInstance()->persistency_.unstoreLsp(lsp)) {
		G2RSVPTE_ERR("Cannot unstore LSP data in Persistency Controller");
	}
#endif // PERSISTENCY_ON

	G2RSVPTE_DBG("Deletion of LSP after SendPathTear in head/core node!");
	lsp->toBeDestroyed(true);
	postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
}

void
Down_i::after_evSendPathTear_from_virt_TearDown(void * context)
{
	DUMP_STATE2(name(), "evSendPathTear", "virt_TearDown");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

#ifdef PERSISTENCY_ON
	if (!lsp->getInstance()->persistency_.unstoreLsp(lsp)) {
		G2RSVPTE_ERR("Cannot unstore LSP data in Persistency Controller");
	}
#endif // PERSISTENCY_ON

	if (!lsp->isTail()) {
		G2RSVPTE_DBG("Deletion of LSP after SendPathTear in head/core node!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
}

void
Down_i::after_evRecvResvTimeout_from_virt_TearDown(void * context)
{
	DUMP_STATE2(name(), "evRecvResvTimeout", "virt_TearDown");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

#ifdef PERSISTENCY_ON
	if (!lsp->getInstance()->persistency_.unstoreLsp(lsp)) {
		G2RSVPTE_ERR("Cannot unstore LSP data in Persistency Controller");
	}
#endif // PERSISTENCY_ON

	G2RSVPTE_DBG("Deletion of LSP after SendPathTear in head/core node!");
	lsp->toBeDestroyed(true);
	postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
}

void
Down_i::after_evRecvResvTearOk_from_virt_TearDown(void * context)
{
	DUMP_STATE2(name(), "evRecvResvTearOk", "virt_TearDown");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

#ifdef PERSISTENCY_ON
	if (!lsp->getInstance()->persistency_.unstoreLsp(lsp)) {
		G2RSVPTE_ERR("Cannot unstore LSP data in Persistency Controller");
	}
#endif // PERSISTENCY_ON

	if (!lsp->isTail()) {
		G2RSVPTE_DBG("Deletion of LSP after RecvResvTearOK in head/core node!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
}

void
Down_i::after_evSendResvTear_from_virt_TearDown(void * context)
{
	DUMP_STATE2(name(), "evSendResvTear", "virt_TearDown");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

#ifdef PERSISTENCY_ON
	if (!lsp->getInstance()->persistency_.unstoreLsp(lsp)) {
		G2RSVPTE_ERR("Cannot unstore LSP data in Persistency Controller");
	}
#endif // PERSISTENCY_ON

	if (!lsp->isHead()) {
		G2RSVPTE_DBG("Deletion of LSP after SendResvTear in tail/core node!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
}

void
Down_i::after_evRecvPathTearOk_from_virt_TearDown(void * context)
{
	DUMP_STATE2(name(), "evRecvPathTearOk", "virt_TearDown");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

#ifdef PERSISTENCY_ON
	if (!lsp->getInstance()->persistency_.unstoreLsp(lsp)) {
		G2RSVPTE_ERR("Cannot unstore LSP data in Persistency Controller");
	}
#endif // PERSISTENCY_ON

	if (!lsp->isHead()) {
		G2RSVPTE_DBG("Deletion of LSP after RecvPathTearOK in tail/core node!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
}

void
Down_i::after_evRecvPathErrOk_from_virt_TearDown(void * context)
{
	DUMP_STATE2(name(), "evRecvPathErrOk", "virt_TearDown");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

#ifdef PERSISTENCY_ON
	if (!lsp->getInstance()->persistency_.unstoreLsp(lsp)) {
		G2RSVPTE_ERR("Cannot unstore LSP data in Persistency Controller");
	}
#endif // PERSISTENCY_ON

	//if isHead (not INNI) LSP destroy must be called manually (VTY/upper layers)
	if (!lsp->isTail() && ((lsp->getInstance()->adjType() == INNI) || !lsp->isHead())) {
		G2RSVPTE_DBG("Deletion of LSP after RecvPathErr!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
}

fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_SendPath_t
Down_i::SendPath(void * context)
{
	DUMP_STATE(name(), "SendPath");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	//assert(ctxt->msg);
	//assert(ctxt->intf);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->psb());
	assert(lsp->rsb());
	assert(lsp->psb()->pathOut());
	assert(lsp->getInstance());

	if (lsp->getInstance()->adjType() == INNI) {
		// Request a resource setting at the TNRC (iTNA case)
		if (!setResource(lsp)) {
			G2RSVPTE_ERR("Impossible to set resource ... failed requesting action on cross-connection");
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_SendPath_to_InvalidEvent;
		}
	} else {
		// Post a fake XConnCompleted event
		lsp->xConnCompleted(true);
	}

	// Tail node?
	bool tailNode;
	tailNode = lsp->isTail();

	if (tailNode) {
		if (lsp->getInstance()->adjType() == INNI) {
			// Notify a Path Event to RC
			G2RSVPTE_INF("posting a Path Event to RC");
			postLspPathEvent(lsp, true);
		}
	} else {

		// Set a random refresh timer
		lsp->psb()->refreshInterval(lsp->refreshInterval());

		G2RSVPTE_INF("set a Path refresh interval = %u ms",
			     lsp->psb()->refreshInterval());

		// Start ZEBRA timer with the refresh timer   => No, so far!!!
		// TODO: FIXME!!! start timer => lsp->psb()->refreshInterval()
		//                else => G2RSVPTE_WRN("could not start refresh-interval timer; let's go on anyway");

		G2RSVPTE_INF("sending first Path msg to NE %s through TE link %s",
			     numb_htoa(lsp->rsb()->nodeId()),
			     addr_ntoa(lsp->rsb()->interface()->localId()));

		if (!lsp->dsAckNack() || !lsp->dsAckNack()->sendMsg(lsp->psb()->pathOut())) {

			G2RSVPTE_ERR("could not deliver Path to the ACK FSM");
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_SendPath_to_InvalidEvent;
		}

		//TODO: FIXME!!! cancel message (if flags NOT_ERASE is not set)
		//delete lsp->psb()->pathOut();
		//lsp->psb()->pathOut(0);
	}

	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_SendPath_to_evSendPath;
}


///////////////////////////////////////
fsm::base_G2RSVPTE_LSP_FSM::nextEvFor_RecvPath_t
Down_i::RecvPath(void * context)
{
	DUMP_STATE(name(), "RecvPath");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	assert(ctxt->msg);
	//assert(ctxt->intf);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->psb());

	// Get session
	RSVP::Session * session = lsp->parent();
	assert(session);

	//
	// XXX FIXME: Add all missing check
	//

	// XXXX FIXME: Check if msg is a PathMessage
	//             Store parsed packet info into the LSP and check
	//             Set NOT_ERASE flags for this pkg
	lsp->psb()->pathIn((RSVP::PathMessage *)ctxt->msg);

	// Tail node?
	bool tailNode;
	tailNode = lsp->isTail();

	// TODO: check if the outgoing parsed packet already points to a
	//       packet that is under retransmission in some ACK/NACK FSM


	RSVP::PathMessage * tmpPath;
	RSVP::G2RSVPTE *    proto;

	proto = lsp->getInstance();
	if (!proto) {
		G2RSVPTE_ERR("Null instance pointer !!!");
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_InvalidEvent;
	}

	if (tailNode) {
		tmpPath = static_cast<RSVP::PathMessage *>(lsp->psb()->pathIn()->clone());
		//tmpPath = lsp->psb()->pathIn();
	} else {
		// If not tail node -> prepare skeleton of the outgoing parsed
		// Path (copy the incoming to the outgoing and apply changes)

		if (lsp->psb()->pathOut()) {
			G2RSVPTE_ERR("ARGHHHHH ... outgoing path already exists!!!");
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_InvalidEvent;
		}

		tmpPath = lsp->psb()->pathIn()->partialClone();
	}

	lsp->psb()->pathOut(tmpPath);

	if (!lsp->psb()->pathOut()) {
		G2RSVPTE_ERR("Impossible to copy incoming path ...");
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_InvalidEvent;
	}

#ifdef CRANKBACK_ON
	//
	// Process LSP Attributes object
	//
	if (lsp->psb()->pathIn()->lsp_attributes_ && !lsp->processLSPAttrObj()) {
		G2RSVPTE_ERR("received LSP ATTRIBUTES obj with problems!!!");

		//send a PathErr upstream
		if (!lsp->usAckNack() || !lsp->usAckNack()->errorHandle(G2RSVPTE_ERROR_SPEC_FLAG_PathStateRemoved,
									G2RSVPTE_ERROR_SPEC_UNKNOWN_ATTRIBUTES_TLV,
									0)) {
			G2RSVPTE_ERR("could not send PathErr");
		}

		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_evRecvPathKo;
	}
#endif // CRANKBACK_ON

	//
	// Process and check the PHOP obj in the incoming Path
	//
	if (!lsp->processPHOPobj()) {
		G2RSVPTE_ERR("received PHOP obj with problems!!!");
		if (ctxt->intf) {
			// override PHOP intf setting
			lsp->psb()->interface(ctxt->intf);
			// set error params
			lsp->errTELink(lsp->psb()->interface()->localId());
			// send a PathErr upstream
			if (!lsp->usAckNack() || !lsp->usAckNack()->errorHandle(G2RSVPTE_ERROR_SPEC_FLAG_PathStateRemoved,
										G2RSVPTE_ERROR_SPEC_ROUTING,
										0,
										true)) {
				G2RSVPTE_ERR("could not send PathErr");
			}
		}
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_evRecvPathKo;
	}

	//
	// Check that PHOP TE link learnt from the PHOP is consistent
	// with the TE link along which the Path has been received
	//
	//TODO -> SCN info here, either from the dst ip if tunneling, or playing with CCs if not
	//
	if (ctxt->intf && (ctxt->intf != lsp->psb()->interface())) {
		G2RSVPTE_ERR("Path msg incoming TE link (%s) is different "
			     "from that derived from PHOP obj (%s)",
			     ((ctxt->intf) ?
			      addr_ntoa((ctxt->intf)->localId()) : "NULL"),
			     ((lsp->psb()->interface()) ?
			      addr_ntoa((lsp->psb()->interface())->localId()) :
			      "NULL"));

		// override PHOP intf setting
		lsp->psb()->interface(ctxt->intf);
		// set error params
		lsp->errTELink(lsp->psb()->interface()->localId());
		//send a PathErr upstream
		if (!lsp->usAckNack() || !lsp->usAckNack()->errorHandle(G2RSVPTE_ERROR_SPEC_FLAG_PathStateRemoved,
									G2RSVPTE_ERROR_SPEC_ROUTING,
									0,
									true)) {
			G2RSVPTE_ERR("could not send PathErr");
		}

		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_evRecvPathKo;
	}

	//
	// Process information from the LABEL SET
	//
	if (!lsp->processLabelSetObj()) {
		G2RSVPTE_ERR("received LABEL SET obj with problems !!!");

		//send a PathErr upstream
		if (!lsp->usAckNack() || !lsp->usAckNack()->errorHandle(G2RSVPTE_ERROR_SPEC_FLAG_PathStateRemoved,
									G2RSVPTE_ERROR_SPEC_ROUTING,
									0,
									true)) {
			G2RSVPTE_ERR("could not send PathErr");
		}

		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_evRecvPathKo;
	}

	//
	// Process information from the UPSTREAM LABEL
	//
	if (!lsp->processUpstreamLabelObj()) {
		G2RSVPTE_ERR("received UPSTREAM LABEL obj with problems!!!");

		//send a PathErr upstream
		if (!lsp->usAckNack() || !lsp->usAckNack()->errorHandle(G2RSVPTE_ERROR_SPEC_FLAG_PathStateRemoved,
									G2RSVPTE_ERROR_SPEC_ROUTING,
									0,
									true)) {
			G2RSVPTE_ERR("could not send PathErr");
		}

		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_evRecvPathKo;
	}

	G2RSVPTE_INF("My PHOP will use tx label = 0x%x and rx label = 0x%x",
		     lsp->psb()->txLabel(),
		     lsp->psb()->rxLabel());

	//
	// Process exclude/include masks     [ OPTIONAL now ??? ]
	//
	RSVP::Session_Attr_Class * sessAttr;
	sessAttr = lsp->psb()->pathOut()->session_attribute_;
	if (sessAttr    &&
	    sessAttr->cType() == CTYPE_SESSION_ATTRIBUTE_LSP_TUNNEL_RA) {

		RSVP::LSPTunnelRASessionAttr_Object * raSessAttr;
		raSessAttr = static_cast
			<RSVP::LSPTunnelRASessionAttr_Object *>(sessAttr);

		if (raSessAttr->get().exclude_any &
		    raSessAttr->get().include_all) {
			G2RSVPTE_ERR("Exclude Any and Include All masks overlap");

			//send a PathErr upstream
			if (!lsp->usAckNack() || !lsp->usAckNack()->errorHandle(G2RSVPTE_ERROR_SPEC_FLAG_PathStateRemoved,
										G2RSVPTE_ERROR_SPEC_SYSTEM,
										0)) {
				G2RSVPTE_ERR("could not send PathErr");
			}

			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_evRecvPathKo;
		}
	}

	if (proto->adjType() != UNI) {
		//
		// ERO processing
		//
		if (!lsp->eroProcess()) {
			G2RSVPTE_ERR("failed processing ERO");

			//send a PathErr upstream
			if (!lsp->usAckNack() || !lsp->usAckNack()->errorHandle(G2RSVPTE_ERROR_SPEC_FLAG_PathStateRemoved,
										G2RSVPTE_ERROR_SPEC_ROUTING,
										G2RSVPTE_ERROR_SPEC_VAL_ROUTING_BAD_ERO,
										true)) {
				G2RSVPTE_ERR("could not send PathErr");
			}

			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_evRecvPathKo;
		}
		//TODO!!! eroProcess does not set the downstream DL!!!
	}
	// Check the exclude/include masks consistency with TE link colors
	// [ OPTIONAL now ??? ]
	//if (!checkTELinkColors()) {
	//      TODO!!!
	//	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_evRecvPathKo;
	//}

	assert(lsp->rsb());

	//
	// TIMES object processing
	//
	if (lsp->psb()->pathIn()->times_ && !lsp->processTimesObj()) {
		G2RSVPTE_ERR("received TIMES obj with problems !!!");

		//TODO send a PathErr
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_evRecvPathKo;
	}


	if (tailNode) { // TAIL NODE

		//
		// Notify upstairs (e.g. Recovery Manager) that a new LSP was born,
		// and provide all the info needed
		//

		//TODO LSP info:
		session->nodeId();   // dst node id
		session->tunnelId(); // tunnel id
		lsp->nid();          // src node id
		lsp->id();           // lsp id
		//lsp;                 // pointer for this LSP
		// etna_info

		// LSP features:
		// encType
		// switchCap
		// holdingPrior
		// ....

		// TODO:
		//if (!lsp->lspCtrl()) {
		//	G2RSVPTE_ERR("Invalid pointer for LSP Controller!");
		//	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_evRecvPathKo;
		//}
		//lsp->lspCtrl()->createLSP( params ...);

		//
		// Process information from the CALL ID
		//

		if (lsp->psb()->pathIn()->call_id_ && !lsp->processCallIdObj(true)) {
			G2RSVPTE_ERR("received CALL ID obj with problems !!!");

			//send a PathErr upstream
			if (!lsp->usAckNack() || !lsp->usAckNack()->errorHandle(G2RSVPTE_ERROR_SPEC_FLAG_PathStateRemoved,
										G2RSVPTE_ERROR_SPEC_CALL_MNGMT,
										G2RSVPTE_ERROR_SPEC_VAL_CALL_MNGMT_CALL_ID_CONTENTION)) {
				G2RSVPTE_ERR("could not send PathErr");
			}
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_evRecvPathKo;
		}

		//
		// Process information from the GENERALISED UNI
		//
		if (lsp->psb()->pathIn()->gen_uni_ && !lsp->processGenUniObj()) {
			G2RSVPTE_ERR("received GENERALISED UNI obj with problems !!!");

			//send a PathErr upstream
			if (!lsp->usAckNack() || !lsp->usAckNack()->errorHandle(G2RSVPTE_ERROR_SPEC_FLAG_PathStateRemoved,
										G2RSVPTE_ERROR_SPEC_ROUTING,
										0,
										true)) {
				G2RSVPTE_ERR("could not send PathErr");
			}

			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_evRecvPathKo;
		}

		// We assume that the DS RX label is equal to the DS TX one
		lsp->rsb()->rxLabel(lsp->rsb()->txLabel());

		//
		// Process information from the POLICY DATA
		//
		if (lsp->psb()->pathIn()->policy_data_ && !lsp->processPolicyData()) {
			G2RSVPTE_ERR("received POLICY DATA  obj with problems !!!");

			//send a PathErr upstream
			if (!lsp->usAckNack() || !lsp->usAckNack()->errorHandle(G2RSVPTE_ERROR_SPEC_FLAG_PathStateRemoved,
										G2RSVPTE_ERROR_SPEC_POLICY_CONTROL,
										G2RSVPTE_ERROR_SPEC_VAL_POLICY_CONTROL_POLICY_ELEMENT_SYNTAX)) {
				G2RSVPTE_ERR("could not send PathErr");
			}

			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_evRecvPathKo;
		}
		if (proto->adjType() == INNI) {
			// lsp->rsb()->interface() is NULL on Tail
			G2RSVPTE_INF("selected DL %s, tx label 0x%x, rx label 0x%x ...",
				     addr_ntoa(lsp->rsb()->localDL()),
				     lsp->rsb()->txLabel(),
				     lsp->rsb()->rxLabel());
			G2RSVPTE_INF("... under E-TNA %s",
				     //addr_ntoa(lsp->rsb()->interface()->localId()));
				     addr_ntoa(lsp->eRes().tna));

			// Request a resource setting at the TNRC
			if (!setResource(lsp)) {
				G2RSVPTE_ERR("Impossible to set resource");

				//send a PathErr upstream
				if (!lsp->usAckNack() || !lsp->usAckNack()->errorHandle(G2RSVPTE_ERROR_SPEC_FLAG_PathStateRemoved,
											G2RSVPTE_ERROR_SPEC_SYSTEM,
											0,
											true,
											true)) {
					G2RSVPTE_ERR("could not send PathErr");
				}

				return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_evRecvPathKo;
			}
		} else {
			lsp->xConnCompleted(true);
		}

	} else { // INTERMEDIATE NODE

		if (!lsp->rsb()->nodeId()) {
			G2RSVPTE_ERR("can't determine NHOP");

			// set error params
			lsp->errTELink(lsp->rsb()->interface()->localId());
			lsp->errLabel(lsp->rsb()->txLabel());
			//send a PathErr upstream
			if (!lsp->usAckNack() || !lsp->usAckNack()->errorHandle(G2RSVPTE_ERROR_SPEC_FLAG_PathStateRemoved,
										G2RSVPTE_ERROR_SPEC_ROUTING,
										G2RSVPTE_ERROR_SPEC_VAL_ROUTING_NO_ROUTE,
										true)) {
				G2RSVPTE_ERR("could not send PathErr");
			}

			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_evRecvPathKo;
		}

		if (lsp->psb()->pathIn()->rro_) {
			//FIX!!! Change RRO type
			//       (RecordedRoute_Class, not RecordedRoute_Object)
			if (lsp->psb()->pathIn()->rro_->cType() != CTYPE_RECORDED_ROUTE) {
				G2RSVPTE_ERR("temporarily, only ctype_rro is allowed");
				return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_InvalidEvent;
			}
			// Detect loops in ERO
			RSVP::RecordedRoute_Object * genRRO;
			genRRO = static_cast<RSVP::RecordedRoute_Object *>(lsp->psb()->pathIn()->rro_);

			if (genRRO && lsp->loopDetect()) {
				G2RSVPTE_ERR("problems with RRO obj");
				return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_evRecvPathKo;
			}
		}

		// prepare the PHOP in the outgoing Path
		if (!lsp->preparePHOPobj()) {
			G2RSVPTE_ERR("problems preparing outgoing Path PHOP obj");

			//send a PathErr upstream
			if (!lsp->usAckNack() || !lsp->usAckNack()->errorHandle(G2RSVPTE_ERROR_SPEC_FLAG_PathStateRemoved,
										G2RSVPTE_ERROR_SPEC_ROUTING,
										0,
										true)) {
				G2RSVPTE_ERR("could not send PathErr");
			}

			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_InvalidEvent;
		}

		if (proto->adjType() == INNI) {
			// Request a resource setting at the TNRC
			if (!setResource(lsp)) {
				G2RSVPTE_ERR("Impossible to set resource ... failded requesting cross-connection");

				//send a PathErr upstream
				if (!lsp->usAckNack() || !lsp->usAckNack()->errorHandle(G2RSVPTE_ERROR_SPEC_FLAG_PathStateRemoved,
											G2RSVPTE_ERROR_SPEC_SYSTEM,
											0,
											true,
											true)) {
					G2RSVPTE_ERR("could not send PathErr");
				}
				return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_InvalidEvent;
			}
		} else {
			lsp->xConnCompleted(true);
		}

		// prepare the LABEL_SET obj in the outgoing Path (using downstream tx label)
		if (!lsp->prepareLabelSetObj()) {
			G2RSVPTE_ERR("problems preparing outgoing Path Label Set obj");
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_InvalidEvent;
		}


		// prepare the UPSTREAM_LABEL obj in the outgoing Path (using downstream rx label)
		if (!lsp->prepareUpstreamLabelObj()) {
			G2RSVPTE_ERR("problems preparing outgoing Path Upstream Label obj");
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_InvalidEvent;
		}
	}

	// Add this node to the RRO
	if (lsp->psb()->pathOut()->rro_) {
		//FIX!!! Change RRO type
		//       (RecordedRoute_Class, not RecordedRoute_Object)
		if (lsp->psb()->pathOut()->rro_->cType() != CTYPE_RECORDED_ROUTE) {
			G2RSVPTE_ERR("temporarily, only ctype_rro is allowed");
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_InvalidEvent;
		}
		// Detect loops in ERO
		RSVP::RecordedRoute_Object * genRRO;
		genRRO = static_cast<RSVP::RecordedRoute_Object *>(lsp->psb()->pathOut()->rro_);

		if (!lsp->insertIntoRRO(genRRO, lsp->psb()->interface()->localId())) {
			G2RSVPTE_ERR("problems preparing outgoing Path RRO obj");
			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_InvalidEvent;
		}
	}

	// Set the refresh timer timeout
	if (!lsp->psb()->pathIn()->time_values_) {
		G2RSVPTE_ERR("problems setting refresh timer timeout ... empty TIME_VALUES obj");
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_InvalidEvent;
	}
	//FIX!!! Change time_values type
	//       (TimeValues_Class, not TimeValues_Object)
	if (lsp->psb()->pathIn()->time_values_->cType() != CTYPE_TIME_VALUES) {
		G2RSVPTE_ERR("problems time values CTYPE %d not supported", lsp->psb()->pathIn()->time_values_->cType());
		return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_InvalidEvent;
	}
	lsp->psb()->refreshTimeout(G2RSVPTE_DEFAULT_REFRESH_TIMEOUT *
				   (static_cast<RSVP::TimeValues_Object *>(lsp->psb()->pathIn()->time_values_))->get().refresh_period);

	G2RSVPTE_INF("set a Path refresh timeout = %u ms",
		     lsp->psb()->refreshTimeout());

	// Start ZEBRA timer with the refresh timer timeout     => No, so far!!!
	// TODO: FIXME!!! start timer => lsp->psb()->refreshTimeout()
	//                else => G2RSVPTE_WRN("could not start refresh-timeout timer; let's go on anyway");

	if (tailNode) {
		// If we want to use peer refresh period ->
		//   (for Resv msgs) use the refresh interval found in the incoming Path
		//lsp->psb()->refreshInterval(lsp->psb()->pathIn()->time_values_->get().refresh_period);

		if (proto->adjType() == INNI) {
			//Notify a new LSP to RC
			G2RSVPTE_INF("Posting a newLSP Event to RC");
			if (!postLspPathEvent(lsp, true)) {
				G2RSVPTE_ERR("Notify of new LSP not delivered to RC");

				//send a PathErr upstream
				if (!lsp->usAckNack() || !lsp->usAckNack()->errorHandle(G2RSVPTE_ERROR_SPEC_FLAG_PathStateRemoved,
											G2RSVPTE_ERROR_SPEC_SYSTEM,
											0)) {
					G2RSVPTE_ERR("could not send PathErr");
				}

				if ((lsp->getInstance()->adjType() == INNI) &&
				     lsp->tnrcCookie()) {
				        // Request a resource release at the TNRC
				        if (!releaseResource(lsp)) {
				                G2RSVPTE_ERR("Impossible to release resource");
				        }
					//set tnrcCookie to 0
					lsp->tnrcCookie(0);
				}
				return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_evRecvPathKo;
			}
		} else {
			bool failed = false;
			G2RSVPTE_INF("Posting a Recv Path Event to CC");
			//force a CallCreate in Mgmt interface of xCC
			if (!postCallCreate(lsp)) {
				failed = true;
			}
			//set TNA in Mgmt interface of xCC
			if (!failed && lsp->psb()->pathIn()->gen_uni_ && !postCallSetTna(lsp)) {
				failed = true;
			}
			//set eGNS-TNA in Mgmt interface of xCC
			if (!failed && lsp->psb()->pathIn()->gns_uni_eg_ && !postCallSetGnsTna(lsp, false)) {
				failed = true;
			}
			//set iGNS-TNA in Mgmt interface of xCC
			if (!failed && lsp->psb()->pathIn()->gns_uni_in_ && !postCallSetGnsTna(lsp, true)) {
				failed = true;
			}
			//set ERO in Mgmt interface of xCC (only ENNI)
			if ((proto->adjType() == ENNI) && !failed && !postCallAddEroPart(lsp)) {
				failed = true;
			}
			//Notify a recvPath LSP to xCC
			if (failed || !notifyLspRecvPathEvent(lsp)) {
				G2RSVPTE_ERR("Notify of recvPath not delivered to CC");

				//send a PathErr upstream
				if (!lsp->usAckNack() || !lsp->usAckNack()->errorHandle(G2RSVPTE_ERROR_SPEC_FLAG_PathStateRemoved,
											G2RSVPTE_ERROR_SPEC_SYSTEM,
											0)) {
					G2RSVPTE_ERR("could not send PathErr");
				}
				return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_evRecvPathKo;
			}
		}
	} else {
		// Set a random refresh timer
		// If we want to use peer refresh period
		//lsp->psb()->refreshInterval(lsp->psb()->pathIn()->time_values_->get().refresh_period);
		// else
		lsp->psb()->refreshInterval(lsp->refreshInterval());

		G2RSVPTE_INF("set a Path refresh interval = %u ms",
			     lsp->psb()->refreshInterval());

		// Start ZEBRA timer with the refresh timer   => No, so far!!!
		// TODO: FIXME!!! start timer => lsp->psb()->refreshInterval()
		//                else => G2RSVPTE_WRN("could not start refresh-interval timer; let's go on anyway");

		G2RSVPTE_INF("forwarding Path msg to NE %s through TE link %s",
			     numb_htoa(lsp->rsb()->nodeId()),
			     addr_ntoa(lsp->rsb()->interface()->localId()));

		if (!lsp->dsAckNack() || !lsp->dsAckNack()->sendMsg(lsp->psb()->pathOut())) {
			G2RSVPTE_ERR("could not deliver Path to the ACK FSM");

			//send a PathErr upstream
			if (!lsp->usAckNack() || !lsp->usAckNack()->errorHandle(G2RSVPTE_ERROR_SPEC_FLAG_PathStateRemoved,
										G2RSVPTE_ERROR_SPEC_SYSTEM,
										0)) {
				G2RSVPTE_ERR("could not send PathErr");
			}

			return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_InvalidEvent;
		}

		//TODO: FIXME!!! cancel message (if flags NOT_ERASE is not set)
		//delete lsp->psb()->pathOut();
		//lsp->psb()->pathOut(0);
	}

	return fsm::base_G2RSVPTE_LSP_FSM::G2RSVPTE_LSP_FSM_from_RecvPath_to_evRecvPathOk;
}

void
Down_i::after_evRecvPathKo_from_virt_Down(void * context)
{
	DUMP_STATE2(name(), "evRecvPathKo", "virt_Down");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

	if (!lsp->isHead()) {
		G2RSVPTE_DBG("Deletion of LSP after RecvPathKo in tail/core node!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
}

void
Down_i::after_evSendPathTear_from_virt_RebuildTearDown(void * context)
{
	DUMP_STATE2(name(), "evSendPathTear", "virt_RebuildTearDown");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

#ifdef PERSISTENCY_ON
	if (!lsp->getInstance()->persistency_.unstoreLsp(lsp)) {
		G2RSVPTE_ERR("Cannot unstore LSP data in Persistency Controller");
	}
#endif // PERSISTENCY_ON

	if (!lsp->isTail()) {
		G2RSVPTE_DBG("Deletion of LSP after SendPathTear in head/core node!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
};

void
Down_i::after_evRecvResvTearOk_from_virt_RebuildTearDown(void * context)
{
	DUMP_STATE2(name(), "evRecvResvTearOk", "virt_RebuildTearDown");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

#ifdef PERSISTENCY_ON
	if (!lsp->getInstance()->persistency_.unstoreLsp(lsp)) {
		G2RSVPTE_ERR("Cannot unstore LSP data in Persistency Controller");
	}
#endif // PERSISTENCY_ON

	if (!lsp->isTail()) {
		G2RSVPTE_DBG("Deletion of LSP after RecvResvTearOK in head/core node!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
};

void
Down_i::after_evSendResvTear_from_virt_RebuildTearDown(void * context)
{
	DUMP_STATE2(name(), "evSendResvTear", "virt_RebuildTearDown");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

#ifdef PERSISTENCY_ON
	if (!lsp->getInstance()->persistency_.unstoreLsp(lsp)) {
		G2RSVPTE_ERR("Cannot unstore LSP data in Persistency Controller");
	}
#endif // PERSISTENCY_ON

	if (!lsp->isHead()) {
		G2RSVPTE_DBG("Deletion of LSP after SendResvTear in tail/core node!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
};

void
Down_i::after_evRecvPathTearOk_from_virt_RebuildTearDown(void * context)
{
	DUMP_STATE2(name(), "evRecvPathTearOk", "virt_RebuildTearDown");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

#ifdef PERSISTENCY_ON
	if (!lsp->getInstance()->persistency_.unstoreLsp(lsp)) {
		G2RSVPTE_ERR("Cannot unstore LSP data in Persistency Controller");
	}
#endif // PERSISTENCY_ON

	if (!lsp->isHead()) {
		G2RSVPTE_DBG("Deletion of LSP after RecvPathTearOK in tail/core node!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
};

void
Down_i::after_evRecvPathErrOk_from_virt_RebuildTearDown(void * context)
{
	DUMP_STATE2(name(), "evRecvPathErrOk", "virt_RebuildTearDown");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

#ifdef PERSISTENCY_ON
	if (!lsp->getInstance()->persistency_.unstoreLsp(lsp)) {
		G2RSVPTE_ERR("Cannot unstore LSP data in Persistency Controller");
	}
#endif // PERSISTENCY_ON

	//if isHead (not INNI) LSP destroy must be called manually (VTY/upper layers)
	if (!lsp->isTail() && ((lsp->getInstance()->adjType() == INNI) || !lsp->isHead())) {
		G2RSVPTE_DBG("Deletion of LSP after RecvPathErr!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
};

void
Down_i::after_evRecvPathTearOk_from_virt_Rebuild(void * context)
{
	DUMP_STATE2(name(), "evRecvPathTearOk", "virt_Rebuild");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);
	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);

#ifdef PERSISTENCY_ON
	if (!lsp->getInstance()->persistency_.unstoreLsp(lsp)) {
		G2RSVPTE_ERR("Cannot unstore LSP data in Persistency Controller");
	}
#endif // PERSISTENCY_ON

	if (!lsp->isHead()) {
		G2RSVPTE_DBG("Deletion of LSP after RecvPathTearOK in tail/core node!");
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}
};

/*****************************************************************************
 *                    Class virt_WaitResvConf_i
 *****************************************************************************/

void
virt_WaitResvConf_i::after_RecvResv_from_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvResv", "WaitResvConf");
}

bool
virt_WaitResvConf_i::evRecvResvKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvKo");
	return true;
}

void
virt_WaitResvConf_i::after_RecvPathErr_from_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathErr", "WaitResvConf");
}

bool
virt_WaitResvConf_i::evRecvPathErrKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathErrKo");
	return true;
}

void
virt_WaitResvConf_i::after_RecvPath_from_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPath", "WaitResvConf");
}

bool
virt_WaitResvConf_i::evRecvPathOk(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathOk");
	return true;
}

void
virt_WaitResvConf_i::after_RecvResvTimer_from_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvResvTimer", "WaitResvConf");
}

bool
virt_WaitResvConf_i::evRecvResvTimer(void * context)
{
	DUMP_STATE(name(), "evRecvResvTimer");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

void
virt_WaitResvConf_i::after_XConnPreempt_from_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "XConnPreempt", "WaitResvConf");
}

bool
virt_WaitResvConf_i::evXConnPreempt(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evXConnPreempt");
	return true;
}

void
virt_WaitResvConf_i::after_SendPathTear_from_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendPathTear", "WaitResvConf");
}

bool
virt_WaitResvConf_i::evSendPathTear(void * context)
{
	DUMP_STATE(name(), "evSendPathTear");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

void
virt_WaitResvConf_i::after_SendConfirm_from_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendConfirm", "WaitResvConf");
}

bool
virt_WaitResvConf_i::evSendConfirm(void * context)
{
	DUMP_STATE(name(), "evSendConfirm");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

void
virt_WaitResvConf_i::after_RecvConfirm_from_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvConfirm", "WaitResvConf");
}

bool
virt_WaitResvConf_i::evRecvConfirmOk(void * context)
{
	DUMP_STATE(name(), "evRecvConfirmOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

void
virt_WaitResvConf_i::after_RecvPathDown_from_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathDown", "WaitResvConf");
}

bool
virt_WaitResvConf_i::evRecvPathDownOk(void * context)
{
	DUMP_STATE(name(), "evRecvPathDownOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

bool
virt_WaitResvConf_i::evRecvPathDownKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathDownKo");
	return true;
}

void
virt_WaitResvConf_i::after_RecvResvTear_from_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvResvTear", "WaitResvConf");
}

bool
virt_WaitResvConf_i::evRecvResvTearKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvTearKo");
	return true;
}

bool
virt_WaitResvConf_i::evRecvPathKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathKo");
	return true;
}

bool
virt_WaitResvConf_i::evRecvResvTearOk(void * context)
{
	DUMP_STATE(name(), "evRecvResvTearOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

bool
virt_WaitResvConf_i::evRecvConfirmKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvConfirmKo");
	return true;
}

bool
virt_WaitResvConf_i::evRecvPathAlarm(void * context)
{
	DUMP_STATE(name(), "evRecvPathAlarm");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

void
virt_WaitResvConf_i::after_SendResvTear_from_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendResvTear", "WaitResvConf");
}

bool
virt_WaitResvConf_i::evSendResvTear(void * context)
{
	DUMP_STATE(name(), "evSendResvTear");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

void
virt_WaitResvConf_i::after_RecvResvErr_from_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvResvErr", "WaitResvConf");
}

bool
virt_WaitResvConf_i::evRecvResvErrKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvErrKo");
	return true;
}

void
virt_WaitResvConf_i::after_RecvPathTear_from_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathTear", "WaitResvConf");
}

bool
virt_WaitResvConf_i::evRecvPathTearOk(void * context)
{
	DUMP_STATE(name(), "evRecvPathTearOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

bool
virt_WaitResvConf_i::evRecvPathTearKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathTearKo");
	return true;
}

bool
virt_WaitResvConf_i::evRecvResvErrOk(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvErrOk");
	return true;
}

void
virt_WaitResvConf_i::after_RecvPathTimer_from_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathTimer", "WaitResvConf");
}

bool
virt_WaitResvConf_i::evRecvPathTimer(void * context)
{
	DUMP_STATE(name(), "evRecvPathTimer");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

bool
virt_WaitResvConf_i::evRecvResvOk(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvOk");
	return true;
}

bool
virt_WaitResvConf_i::evRecvPathErrOk(void * context)
{
	DUMP_STATE(name(), "evRecvPathErrOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

void
virt_WaitResvConf_i::after_SendPathDown_from_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendPathDown", "WaitResvConf");
};

bool
virt_WaitResvConf_i::evSendPathDown(void* context)
{
	DUMP_STATE(name(), "evSendPathDown");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
};

void
virt_WaitResvConf_i::after_RecvResvDown_from_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvResvDown", "WaitResvConf");
};

bool
virt_WaitResvConf_i::evRecvResvDownKo(void* context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvDownKo");
	return true;
};

bool
virt_WaitResvConf_i::evRecvResvDownOk(void* context)
{
	DUMP_STATE(name(), "evRecvResvDownOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
};

void
virt_WaitResvConf_i::after_SendResvDown_from_WaitResvConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendResvDown", "WaitResvConf");
};

bool
virt_WaitResvConf_i::evSendResvDown(void* context)
{
	DUMP_STATE(name(), "evSendResvDown");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
};


/*****************************************************************************
 *                    Class virt_WaitResv_i
 *****************************************************************************/

void
virt_WaitResv_i::after_RecvPathTimeout_from_WaitResv(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathTimeout", "WaitResv");
}

bool
virt_WaitResv_i::evRecvPathTimeout(void * context)
{
	DUMP_STATE(name(), "evRecvPathTimeout");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

void
virt_WaitResv_i::after_RecvResv_from_WaitResv(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvResv", "WaitResv");
}

bool
virt_WaitResv_i::evRecvResvKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvKo");
	return true;
}

void
virt_WaitResv_i::after_RecvPathErr_from_WaitResv(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathErr", "WaitResv");
}

bool
virt_WaitResv_i::evRecvPathErrKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathErrKo");
	return true;
}

void
virt_WaitResv_i::after_RecvPath_from_WaitResv(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPath", "WaitResv");
}

bool
virt_WaitResv_i::evRecvPathOk(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathOk");
	return true;
}

void
virt_WaitResv_i::after_XConnPreempt_from_WaitResv(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "XConnPreempt", "WaitResv");
}

bool
virt_WaitResv_i::evXConnPreempt(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evXConnPreempt");
	return true;
}

void
virt_WaitResv_i::after_SendPathTear_from_WaitResv(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendPathTear", "WaitResv");
}

bool
virt_WaitResv_i::evSendPathTear(void * context)
{
	DUMP_STATE(name(), "evSendPathTear");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

bool
virt_WaitResv_i::evRecvResvVeryKo(void * context)
{
	DUMP_STATE(name(), "evRecvResvVeryKo");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

void
virt_WaitResv_i::after_RecvResvTear_from_WaitResv(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvResvTear", "WaitResv");
}

bool
virt_WaitResv_i::evRecvResvTearKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvTearKo");
	return true;
}

bool
virt_WaitResv_i::evRecvPathErrCrankback(void * context)
{
	DUMP_STATE(name(), "evRecvPathErrCrankback");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

bool
virt_WaitResv_i::evRecvPathKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathKo");
	return true;
}

bool
virt_WaitResv_i::evRecvResvTearOk(void * context)
{
	DUMP_STATE(name(), "evRecvResvTearOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

bool
virt_WaitResv_i::evRecvPathAlarm(void * context)
{
	DUMP_STATE(name(), "evRecvPathAlarm");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

void
virt_WaitResv_i::after_SendResvTear_from_WaitResv(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendResvTear", "WaitResv");
}

bool
virt_WaitResv_i::evSendResvTear(void * context)
{
	DUMP_STATE(name(), "evSendResvTear");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

void
virt_WaitResv_i::after_RecvPathTear_from_WaitResv(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathTear", "WaitResv");
}

bool
virt_WaitResv_i::evRecvPathTearOk(void * context)
{
	DUMP_STATE(name(), "evRecvPathTearOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

bool
virt_WaitResv_i::evRecvPathTearKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathTearKo");
	return true;
}

void
virt_WaitResv_i::after_RecvPathTimer_from_WaitResv(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathTimer", "WaitResv");
}

bool
virt_WaitResv_i::evRecvPathTimer(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathTimer");
	return true;
}

bool
virt_WaitResv_i::evRecvPathErrOk(void * context)
{
	DUMP_STATE(name(), "evRecvPathErrOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

bool
virt_WaitResv_i::evRecvResvOk(void * context)
{
	DUMP_STATE(name(), "evRecvResvOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

void
virt_WaitResv_i::after_SendResv_from_WaitResv(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendResv", "WaitResv");
}

bool
virt_WaitResv_i::evSendResv(void * context)
{
	DUMP_STATE(name(), "evSendResv");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

void
virt_WaitResv_i::after_SendPathDown_from_WaitResv(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendPathDown", "WaitResv");
};

bool
virt_WaitResv_i::evSendPathDown(void* context)
{
	DUMP_STATE(name(), "evSendPathDown");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
};

void
virt_WaitResv_i::after_RecvPathDown_from_WaitResv(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathDown", "WaitResv");
};

bool
virt_WaitResv_i::evRecvPathDownOk(void* context)
{
	DUMP_STATE(name(), "evRecvPathDownOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
};

bool
virt_WaitResv_i::evRecvPathDownKo(void* context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathDownKo");
	return true;
};

bool
virt_WaitResv_i::evRecvResvDownKo(void* context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvReavDownKo");
	return true;
};

void
virt_WaitResv_i::after_SendResvDown_from_WaitResv(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendResvDown", "WaitResv");
};

bool
virt_WaitResv_i::evSendResvDown(void* context)
{
	DUMP_STATE(name(), "evSendResvDown");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
};

void
virt_WaitResv_i::after_RecvResvDown_from_WaitResv(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvResvDown", "WaitResv");
};

bool
virt_WaitResv_i::evRecvResvDownOk(void* context)
{
	DUMP_STATE(name(), "evRecvResvDownOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
};


/*****************************************************************************
 *                    Class virt_WaitEqptConf_i
 *****************************************************************************/

void
virt_WaitEqptConf_i::after_RecvPathTimeout_from_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathTimeout", "WaitEqptConf");
}

bool
virt_WaitEqptConf_i::evRecvPathTimeout(void * context)
{
	DUMP_STATE(name(), "evRecvPathTimeout");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

void
virt_WaitEqptConf_i::after_XConnCompleted_from_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "XConnCompleted", "WaitEqptConf");
}

bool
virt_WaitEqptConf_i::evXConnCompleted(void * context)
{
	DUMP_STATE(name(), "evXConnCompleted");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

void
virt_WaitEqptConf_i::after_RecvResv_from_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvResv", "WaitEqptConf");
}

bool
virt_WaitEqptConf_i::evRecvResvKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvKo");
	return true;
}

void
virt_WaitEqptConf_i::after_RecvPathErr_from_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathErr", "WaitEqptConf");
}

bool
virt_WaitEqptConf_i::evRecvPathErrKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathErrKo");
	return true;
}

void
virt_WaitEqptConf_i::after_RecvPath_from_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPath", "WaitEqptConf");
}

bool
virt_WaitEqptConf_i::evRecvPathOk(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathOk");
	return true;
}

void
virt_WaitEqptConf_i::after_RecvResvTimer_from_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvResvTimer", "WaitEqptConf");
}

bool
virt_WaitEqptConf_i::evRecvResvTimer(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvTimer");
	return true;
}

void
virt_WaitEqptConf_i::after_SendPathTear_from_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendPathTear", "WaitEqptConf");
}

bool
virt_WaitEqptConf_i::evSendPathTear(void * context)
{
	DUMP_STATE(name(), "evSendPathTear");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

void
virt_WaitEqptConf_i::after_RecvResvTimeout_from_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvResvTimeout", "WaitEqptConf");
}

bool
virt_WaitEqptConf_i::evRecvResvTimeout(void * context)
{
	DUMP_STATE(name(), "evRecvResvTimeout");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

bool
virt_WaitEqptConf_i::evRecvResvVeryKo(void * context)
{
	DUMP_STATE(name(), "evRecvResvVeryKo");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

void
virt_WaitEqptConf_i::after_RecvResvTear_from_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvResvTear", "WaitEqptConf");
}

bool
virt_WaitEqptConf_i::evRecvResvTearKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvTearKo");
	return true;
}

bool
virt_WaitEqptConf_i::evRecvPathKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathKo");
	return true;
}

bool
virt_WaitEqptConf_i::evRecvResvTearOk(void * context)
{
	DUMP_STATE(name(), "evRecvResvTearOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

void
virt_WaitEqptConf_i::after_SendResvTear_from_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendResvTear", "WaitEqptConf");
}

bool
virt_WaitEqptConf_i::evSendResvTear(void * context)
{
	DUMP_STATE(name(), "evSendResvTear");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

void
virt_WaitEqptConf_i::after_RecvPathTear_from_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathTear", "WaitEqptConf");
}

bool
virt_WaitEqptConf_i::evRecvPathTearOk(void * context)
{
	DUMP_STATE(name(), "evRecvPathTearOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

void
virt_WaitEqptConf_i::after_RecvResvErr_from_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvResvErr", "WaitEqptConf");
}

bool
virt_WaitEqptConf_i::evRecvResvErrKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvErrKo");
	return true;
}

bool
virt_WaitEqptConf_i::evRecvPathTearKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathTearKo");
	return true;
}

bool
virt_WaitEqptConf_i::evRecvResvErrOk(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvErrOk");
	return true;
}

void
virt_WaitEqptConf_i::after_RecvPathTimer_from_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathTimer", "WaitEqptConf");
}

bool
virt_WaitEqptConf_i::evRecvPathTimer(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathTimer");
	return true;
}

void
virt_WaitEqptConf_i::after_XConnErr_from_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "XConnErr", "WaitEqptConf");
}

bool
virt_WaitEqptConf_i::evXConnErr(void * context)
{
	DUMP_STATE(name(), "evXConnErr");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

bool
virt_WaitEqptConf_i::evRecvPathErrOk(void * context)
{
	DUMP_STATE(name(), "evRecvPathErrOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

bool
virt_WaitEqptConf_i::evRecvResvOk(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvOk");
	return true;
}

void
virt_WaitEqptConf_i::after_SendPathDown_from_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendPathDown", "WaitEqptConf");
};

bool
virt_WaitEqptConf_i::evSendPathDown(void* context)
{
	DUMP_STATE(name(), "evSendPathDown");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
};

void
virt_WaitEqptConf_i::after_RecvResvDown_from_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvResvDown", "WaitEqptConf");
};

bool
virt_WaitEqptConf_i::evRecvResvDownKo(void* context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvDownKo");
	return true;
};

bool
virt_WaitEqptConf_i::evRecvResvDownOk(void* context)
{
	DUMP_STATE(name(), "evRecvResvDownOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
};

void
virt_WaitEqptConf_i::after_RecvPathDown_from_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathDown", "WaitEqptConf");
};

bool
virt_WaitEqptConf_i::evRecvPathDownOk(void* context)
{
	DUMP_STATE(name(), "evRecvPathDownOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
};

bool
virt_WaitEqptConf_i::evRecvPathDownKo(void* context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathDownKo");
	return true;
};

void
virt_WaitEqptConf_i::after_SendResvDown_from_WaitEqptConf(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendResvDown", "WaitEqptConf");
};

bool
virt_WaitEqptConf_i::evSendResvDown(void* context)
{
	DUMP_STATE(name(), "evSendResvDown");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
};


/*****************************************************************************
 *                    Class virt_Rebuild_i
 *****************************************************************************/

void
virt_Rebuild_i::after_RecvPathErr_from_Rebuild(void * context)
{
	DUMP_STATE2(name(), "RecvPathErr", "Rebuild");
};

bool
virt_Rebuild_i::evRecvPathErrOk(void* context)
{
	DUMP_STATE(name(), "evRecvPathErrOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
};

bool
virt_Rebuild_i::evRecvPathErrKo(void* context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathErrOk");
	return true;
};

void
virt_Rebuild_i::after_SendPathDown_from_Rebuild(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendPathDown", "Rebuild");
};

bool
virt_Rebuild_i::evSendPathDown(void* context)
{
	DUMP_STATE(name(), "evSendPathDown");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
};

void
virt_Rebuild_i::after_RecvPathDown_from_Rebuild(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathDown", "Rebuild");
};

bool
virt_Rebuild_i::evRecvPathDownOk(void* context)
{
	DUMP_STATE(name(), "evRecvPathDownOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
};

bool
virt_Rebuild_i::evRecvPathDownKo(void* context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathDownKo");
	return true;
};

void
virt_Rebuild_i::after_RecvResvDown_from_Rebuild(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvResvDown", "Rebuild");
};

bool
virt_Rebuild_i::evRecvResvDownOk(void* context)
{
	DUMP_STATE(name(), "evRecvResvDownOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
};

bool
virt_Rebuild_i::evRecvResvDownKo(void* context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvDownKo");
	return true;
};

void
virt_Rebuild_i::after_SendResvDown_from_Rebuild(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendResvDown", "Rebuild");
};

bool
virt_Rebuild_i::evSendResvDown(void* context)
{
	DUMP_STATE(name(), "evSendResvDown");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
};

void
virt_Rebuild_i::after_RecvPathTear_from_Rebuild(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathTear", "Rebuild");
};

bool
virt_Rebuild_i::evRecvPathTearOk(void* context)
{
	DUMP_STATE(name(), "evRecvPathTearOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
};

bool
virt_Rebuild_i::evRecvPathTearKo(void* context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathTearKo");
	return true;
};

void
virt_Rebuild_i::after_XConnCompleted_from_Rebuild(void * context)
{
        //
        // Add your code here, this it is just an example
        //
        DUMP_STATE2(name(), "XConnCompleted", "Rebuild");
};

bool
virt_Rebuild_i::evXConnCompleted(void* context)
{
        DUMP_STATE(name(), "evXConnCompleted");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

        return true;
};

void
virt_Rebuild_i::after_XConnErr_from_Rebuild(void * context)
{
        //
        // Add your code here, this it is just an example
        //
        DUMP_STATE2(name(), "XConnErr", "Rebuild");
};

bool
virt_Rebuild_i::evXConnErr(void* context)
{
        DUMP_STATE(name(), "evXConnErr");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

        return true;
};


/*****************************************************************************
 *                    Class virt_Installed_i
 *****************************************************************************/

void
virt_Installed_i::after_XConnCompleted_from_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "XConnCompleted", "Installed");
}

bool
virt_Installed_i::evXConnCompleted(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evXConnCompleted");
	return true;
}

void
virt_Installed_i::after_SendPathDown_from_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendPathDown", "Installed");
}

bool
virt_Installed_i::evSendPathDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evSendPathDown");
	return true;
}

void
virt_Installed_i::after_RecvConfirm_from_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvConfirm", "Installed");
}

bool
virt_Installed_i::evRecvConfirmOk(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvConfirmOk");
	return true;
}

void
virt_Installed_i::after_RecvPathDown_from_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathDown", "Installed");
}

bool
virt_Installed_i::evRecvPathDownKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathDownKo");
	return true;
}

void
virt_Installed_i::after_RecvPath_from_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPath", "Installed");
}

bool
virt_Installed_i::evRecvPathKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathKo");
	return true;
}

void
virt_Installed_i::after_RecvResvTear_from_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvResvTear", "Installed");
}

bool
virt_Installed_i::evRecvResvTearOk(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvTearOk");
	return true;
}

void
virt_Installed_i::after_RecvPathErr_from_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathErr", "Installed");
}

bool
virt_Installed_i::evRecvPathAlarm(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathAlarm");
	return true;
}

bool
virt_Installed_i::evRecvConfirmKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvConfirmKo");
	return true;
}

void
virt_Installed_i::after_RecvPathTear_from_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathTear", "Installed");
}

bool
virt_Installed_i::evRecvPathTearKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathTearKo");
	return true;
}

void
virt_Installed_i::after_RecvPathTimer_from_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathTimer", "Installed");
}

bool
virt_Installed_i::evRecvPathTimer(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathTimer");
	return true;
}

void
virt_Installed_i::after_XConnDown_from_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "XConnDown", "Installed");
}

bool
virt_Installed_i::evXConnDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evXConnDown");
	return true;
}

bool
virt_Installed_i::evRecvPathErrOk(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathErrOk");
	return true;
}

void
virt_Installed_i::after_RecvResv_from_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvResv", "Installed");
}

bool
virt_Installed_i::evRecvResvOk(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvOk");
	return true;
}

void
virt_Installed_i::after_RecvResvDown_from_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvResvDown", "Installed");
}

bool
virt_Installed_i::evRecvResvDownOk(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvDownOk");
	return true;
}

void
virt_Installed_i::after_RecvNotify_from_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvNotify", "Installed");
}

bool
virt_Installed_i::evRecvNotifyOk(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvNotifyOk");
	return true;
}

bool
virt_Installed_i::evRecvResvKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvKo");
	return true;
}

bool
virt_Installed_i::evRecvNotifyDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvNotifyDown");
	return true;
}

bool
virt_Installed_i::evRecvPathErrKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathErrKo");
	return true;
}

bool
virt_Installed_i::evRecvPathOk(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathOk");
	return true;
}

bool
virt_Installed_i::evRecvActivateErr(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvActivateErr");
	return true;
}

void
virt_Installed_i::after_RecvResvTimer_from_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvResvTimer", "Installed");
}

bool
virt_Installed_i::evRecvResvTimer(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvTimer");
	return true;
}

void
virt_Installed_i::after_XConnPreempt_from_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "XConnPreempt", "Installed");
}

bool
virt_Installed_i::evXConnPreempt(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evXConnPreempt");
	return true;
}

bool
virt_Installed_i::evRecvResvDownKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvDownKo");
	return true;
}

void
virt_Installed_i::after_RecvActivate_from_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvActivate", "Installed");
}

bool
virt_Installed_i::evRecvActivateOk(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvActivateOk");
	return true;
}

bool
virt_Installed_i::evRecvPathDownOk(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathDownOk");
	return true;
}

bool
virt_Installed_i::evRecvResvVeryKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvVeryKo");
	return true;
}

bool
virt_Installed_i::evRecvResvTearKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvTearKo");
	return true;
}

bool
virt_Installed_i::evRecvNotifyKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvNotifyKo");
	return true;
}

bool
virt_Installed_i::evRecvPathTearOk(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathTearOk");
	return true;
}

void
virt_Installed_i::after_RecvResvErr_from_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvResvErr", "Installed");
}

bool
virt_Installed_i::evRecvResvErrKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvErrKo");
	return true;
}

bool
virt_Installed_i::evRecvActivateKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvActivateKo");
	return true;
}

bool
virt_Installed_i::evRecvResvErrOk(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvErrOk");
	return true;
}

void
virt_Installed_i::after_XConnErr_from_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "XConnErr", "Installed");
}

bool
virt_Installed_i::evXConnErr(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evXConnErr");
	return true;
}

void
virt_Installed_i::after_SendResvDown_from_Installed(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendResvDown", "Installed");
}

bool
virt_Installed_i::evSendResvDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evSendResvDown");
	return true;
}


/*****************************************************************************
 *                    Class virt_RebuildTearDown_i
 *****************************************************************************/

void
virt_RebuildTearDown_i::after_SendPathDown_from_RebuildTearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendPathDown", "RebuildTearDown");
};

bool
virt_RebuildTearDown_i::evSendPathDown(void* context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evSendPathDown");
	return true;
};

void
virt_RebuildTearDown_i::after_XConnCompleted_from_RebuildTearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "XConnCompleted", "RebuildTearDown");
};

bool
virt_RebuildTearDown_i::evXConnCompleted(void* context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evXConnCompleted");
	return true;
};

void
virt_RebuildTearDown_i::after_RecvPathErr_from_RebuildTearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathErr", "RebuildTearDown");
};

bool
virt_RebuildTearDown_i::evRecvPathErrKo(void* context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathErrKo");
	return true;
};

void
virt_RebuildTearDown_i::after_SendPathTear_from_RebuildTearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendPathTear", "RebuildTearDown");
};

bool
virt_RebuildTearDown_i::evSendPathTear(void* context)
{
	DUMP_STATE(name(), "evSendPathTear");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
};

void
virt_RebuildTearDown_i::after_XConnErr_from_RebuildTearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "XConnErr", "RebuildTearDown");
};

bool
virt_RebuildTearDown_i::evXConnErr(void* context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evXConnErr");
	return true;
};

void
virt_RebuildTearDown_i::after_RecvResvDown_from_RebuildTearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvResvDown", "RebuildTearDown");
};

bool
virt_RebuildTearDown_i::evRecvResvDownKo(void* context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvDownKo");
	return true;
};

void
virt_RebuildTearDown_i::after_RecvPathDown_from_RebuildTearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathDown", "RebuildTearDown");
};

bool
virt_RebuildTearDown_i::evRecvPathDownOk(void* context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathDownOk");
	return true;
};

bool
virt_RebuildTearDown_i::evRecvPathDownKo(void* context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathDownKo");
	return true;
};

void
virt_RebuildTearDown_i::after_RecvResvTear_from_RebuildTearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvResvTear", "RebuildTearDown");
};

bool
virt_RebuildTearDown_i::evRecvResvTearKo(void* context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvTearKo");
	return true;
};

bool
virt_RebuildTearDown_i::evRecvResvTearOk(void* context)
{
	DUMP_STATE(name(), "evRecvResvTearOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
};

void
virt_RebuildTearDown_i::after_SendResvTear_from_RebuildTearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendResvTear", "RebuildTearDown");
};

bool
virt_RebuildTearDown_i::evSendResvTear(void* context)
{
	DUMP_STATE(name(), "evSendResvTear");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
};

void
virt_RebuildTearDown_i::after_RecvPathTear_from_RebuildTearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathTear", "RebuildTearDown");
};

bool
virt_RebuildTearDown_i::evRecvPathTearOk(void* context)
{
	DUMP_STATE(name(), "evRecvPathTearOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
};

void
virt_RebuildTearDown_i::after_RecvResvErr_from_RebuildTearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvResvErr", "RebuildTearDown");
};

bool
virt_RebuildTearDown_i::evRecvResvErrKo(void* context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvErrKo");
	return true;
};

bool
virt_RebuildTearDown_i::evRecvPathTearKo(void* context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathTearKo");
	return true;
};

bool
virt_RebuildTearDown_i::evRecvResvErrOk(void* context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvErrOk");
	return true;
};

bool
virt_RebuildTearDown_i::evRecvPathErrOk(void* context)
{
	DUMP_STATE(name(), "evRecvPathErrOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
};

bool
virt_RebuildTearDown_i::evRecvResvDownOk(void* context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvDownOk");
	return true;
};

void
virt_RebuildTearDown_i::after_SendResvDown_from_RebuildTearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendResvDown", "RebuildTearDown");
};

bool
virt_RebuildTearDown_i::evSendResvDown(void* context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evSendResvDown");
	return true;
};


/*****************************************************************************
 *                    Class virt_PathReceived_i
 *****************************************************************************/

void
virt_PathReceived_i::after_RecvPathTimeout_from_PathReceived(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathTimeout", "PathReceived");
}

bool
virt_PathReceived_i::evRecvPathTimeout(void * context)
{
	DUMP_STATE(name(), "evRecvPathTimeout");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

void
virt_PathReceived_i::after_XConnCompleted_from_PathReceived(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "XConnCompleted", "PathReceived");
}

bool
virt_PathReceived_i::evXConnCompleted(void * context)
{
	DUMP_STATE(name(), "evXConnCompleted");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

void
virt_PathReceived_i::after_RecvResv_from_PathReceived(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvResv", "PathReceived");
}

bool
virt_PathReceived_i::evRecvResvKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvKo");
	return true;
}

void
virt_PathReceived_i::after_RecvPathErr_from_PathReceived(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathErr", "PathReceived");
}

bool
virt_PathReceived_i::evRecvPathErrKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathErrKo");
	return true;
}

void
virt_PathReceived_i::after_RecvPath_from_PathReceived(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPath", "PathReceived");
}

bool
virt_PathReceived_i::evRecvPathOk(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathOk");
	return true;
}

void
virt_PathReceived_i::after_SendPathTear_from_PathReceived(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendPathTear", "PathReceived");
}

bool
virt_PathReceived_i::evSendPathTear(void * context)
{
	DUMP_STATE(name(), "evSendPathTear");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

bool
virt_PathReceived_i::evRecvPathErrCrankback(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathErrCrankback");
	return true;
}

bool
virt_PathReceived_i::evRecvPathKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathKo");
	return true;
}

void
virt_PathReceived_i::after_SendResvTear_from_PathReceived(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendResvTear", "PathReceived");
}

bool
virt_PathReceived_i::evSendResvTear(void * context)
{
	DUMP_STATE(name(), "evSendResvTear");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

void
virt_PathReceived_i::after_RecvPathTear_from_PathReceived(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathTear", "PathReceived");
}

bool
virt_PathReceived_i::evRecvPathTearOk(void * context)
{
	DUMP_STATE(name(), "evRecvPathTearOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

bool
virt_PathReceived_i::evRecvPathTearKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathTearKo");
	return true;
}

void
virt_PathReceived_i::after_RecvPathTimer_from_PathReceived(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathTimer", "PathReceived");
}

bool
virt_PathReceived_i::evRecvPathTimer(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathTimer");
	return true;
}

bool
virt_PathReceived_i::evRecvPathErrOk(void * context)
{
	DUMP_STATE(name(), "evRecvPathErrOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

bool
virt_PathReceived_i::evRecvResvOk(void * context)
{
	DUMP_STATE(name(), "evRecvResvOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

void
virt_PathReceived_i::after_XConnErr_from_PathReceived(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "XConnErr", "PathReceived");
}

bool
virt_PathReceived_i::evXConnErr(void * context)
{
	DUMP_STATE(name(), "evXConnErr");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

void
virt_PathReceived_i::after_SendResv_from_PathReceived(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendResv", "PathReceived");
}

bool
virt_PathReceived_i::evSendResv(void * context)
{
	DUMP_STATE(name(), "evSendResv");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

void
virt_PathReceived_i::after_SendPathDown_from_PathReceived(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendPathDown", "PathReceived");
};

bool
virt_PathReceived_i::evSendPathDown(void* context)
{
	DUMP_STATE(name(), "evSendPathDown");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
};

void
virt_PathReceived_i::after_RecvResvDown_from_PathReceived(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvResvDown", "PathReceived");
};

bool
virt_PathReceived_i::evRecvResvDownKo(void* context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvDownKo");
	return true;
};

bool
virt_PathReceived_i::evRecvResvDownOk(void* context)
{
	DUMP_STATE(name(), "evRecvResvDownOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
};

void
virt_PathReceived_i::after_RecvPathDown_from_PathReceived(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathDown", "PathReceived");
};

bool
virt_PathReceived_i::evRecvPathDownOk(void* context)
{
	DUMP_STATE(name(), "evRecvPathDownOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
};

bool
virt_PathReceived_i::evRecvPathDownKo(void* context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathDownKo");
	return true;
};

void
virt_PathReceived_i::after_SendResvDown_from_PathReceived(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendResvDown", "PathReceived");
};

bool
virt_PathReceived_i::evSendResvDown(void* context)
{
	DUMP_STATE(name(), "evSendResvDown");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
};


/*****************************************************************************
 *                    Class virt_TearDown_i
 *****************************************************************************/

void
virt_TearDown_i::after_RecvPathTimeout_from_TearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathTimeout", "TearDown");
}

bool
virt_TearDown_i::evRecvPathTimeout(void * context)
{
	DUMP_STATE(name(), "evRecvPathTimeout");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

void
virt_TearDown_i::after_SendPathDown_from_TearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendPathDown", "TearDown");
}

bool
virt_TearDown_i::evSendPathDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evSendPathDown");
	return true;
}

void
virt_TearDown_i::after_RecvPathErr_from_TearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathErr", "TearDown");
}

bool
virt_TearDown_i::evRecvPathErrKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathErrKo");
	return true;
}

void
virt_TearDown_i::after_SendPathTear_from_TearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendPathTear", "TearDown");
}

bool
virt_TearDown_i::evSendPathTear(void * context)
{
	DUMP_STATE(name(), "evSendPathTear");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

void
virt_TearDown_i::after_RecvResvDown_from_TearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvResvDown", "TearDown");
}

bool
virt_TearDown_i::evRecvResvDownKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvDownKo");
	return true;
}

void
virt_TearDown_i::after_RecvPathDown_from_TearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathDown", "TearDown");
}

bool
virt_TearDown_i::evRecvPathDownOk(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathDownOk");
	return true;
}

void
virt_TearDown_i::after_RecvResvTimeout_from_TearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvResvTimeout", "TearDown");
}

bool
virt_TearDown_i::evRecvResvTimeout(void * context)
{
	DUMP_STATE(name(), "evRecvResvTimeout");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

bool
virt_TearDown_i::evRecvPathDownKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathDownKo");
	return true;
}

void
virt_TearDown_i::after_RecvResvTear_from_TearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvResvTear", "TearDown");
}

bool
virt_TearDown_i::evRecvResvTearKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvTearKo");
	return true;
}

bool
virt_TearDown_i::evRecvResvTearOk(void * context)
{
	DUMP_STATE(name(), "evRecvResvTearOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

bool
virt_TearDown_i::evRecvPathAlarm(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathAlarm");
	return true;
}

void
virt_TearDown_i::after_SendResvTear_from_TearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendResvTear", "TearDown");
}

bool
virt_TearDown_i::evSendResvTear(void * context)
{
	DUMP_STATE(name(), "evSendResvTear");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

void
virt_TearDown_i::after_RecvPathTear_from_TearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPathTear", "TearDown");
}

bool
virt_TearDown_i::evRecvPathTearOk(void * context)
{
	DUMP_STATE(name(), "evRecvPathTearOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

void
virt_TearDown_i::after_RecvResvErr_from_TearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvResvErr", "TearDown");
}

bool
virt_TearDown_i::evRecvResvErrKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvErrKo");
	return true;
}

bool
virt_TearDown_i::evRecvPathTearKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathTearKo");
	return true;
}

bool
virt_TearDown_i::evRecvResvErrOk(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvErrOk");
	return true;
}

bool
virt_TearDown_i::evRecvPathErrOk(void * context)
{
	DUMP_STATE(name(), "evRecvPathErrOk");

	G2RSVPTE_LSP_FSM_context_t * ctxt;

	// Get context
	ctxt = (G2RSVPTE_LSP_FSM_context_t *) context;
	assert(ctxt);

	// Get LSP
	RSVP::LSP * lsp = ctxt->lsp;
	assert(lsp);
	assert(lsp->getInstance());

	// Delete timer for this FSM state
	FSMtimerDelete(lsp);

	return true;
}

bool
virt_TearDown_i::evRecvResvDownOk(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvResvDownOk");
	return true;
}

void
virt_TearDown_i::after_SendResvDown_from_TearDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendResvDown", "TearDown");
}

bool
virt_TearDown_i::evSendResvDown(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evSendResvDown");
	return true;
}


/*****************************************************************************
 *                    Class virt_Down_i
 *****************************************************************************/

void
virt_Down_i::after_LspRebuild_from_Down(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "LspRebuild", "Down");
};

bool
virt_Down_i::evLspRebuild(void* context)
{
	DUMP_STATE(name(), "evLspRebuild");

	return true;
};

void
virt_Down_i::after_SendPath_from_Down(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "SendPath", "Down");
}

bool
virt_Down_i::evSendPath(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evSendPath");
	return true;
}

void
virt_Down_i::after_RecvPath_from_Down(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE2(name(), "RecvPath", "Down");
}

bool
virt_Down_i::evRecvPathOk(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathOk");
	return true;
}

bool
virt_Down_i::evRecvPathKo(void * context)
{
	//
	// Add your code here, this it is just an example
	//
	DUMP_STATE(name(), "evRecvPathKo");
	return true;
}
